var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "d3", "common/appUrl"], function (require, exports, commandBase, d3, appUrl) {
    "use strict";
    var getIndexingBatchStatsCommand = (function (_super) {
        __extends(getIndexingBatchStatsCommand, _super);
        function getIndexingBatchStatsCommand(db, lastId) {
            if (lastId === void 0) { lastId = 0; }
            var _this = _super.call(this) || this;
            _this.db = db;
            _this.lastId = lastId;
            return _this;
        }
        getIndexingBatchStatsCommand.prototype.execute = function () {
            var url = "/debug/indexing-batch-stats"; //TODO: use endpoints
            var entryMapping = function (entry) {
                return {
                    indexName: entry.key,
                    stats: entry.value
                };
            };
            var args = { lastId: this.lastId };
            var parser = d3.time.format.iso;
            return this.query(url, args, this.db, function (result) {
                var mappedResult = result.map(function (item) {
                    return {
                        Id: item.Id,
                        BatchType: item.BatchType,
                        IndexesToWorkOn: item.IndexesToWorkOn,
                        TotalDocumentCount: item.TotalDocumentCount,
                        TotalDocumentSize: item.TotalDocumentSize,
                        StartedAt: item.StartedAt,
                        StartedAtDate: parser.parse(item.StartedAt),
                        TotalDurationMs: item.TotalDurationMs,
                        PerfStats: d3.map(item.PerformanceStats).entries().map(entryMapping)
                    };
                });
                mappedResult.forEach(function (r) {
                    r.PerfStats.forEach(function (p) {
                        p.stats.Operations.filter(function (x) { return !("Name" in x); }).forEach(function (o) {
                            o.BatchedOperations.forEach(function (b) {
                                b.Operations.forEach(function (x) {
                                    x.ParallelParent = b;
                                });
                                b.Parent = o;
                            });
                        });
                    });
                });
                return mappedResult;
            });
        };
        getIndexingBatchStatsCommand.prototype.getQueryUrl = function () {
            return appUrl.forResourceQuery(this.db) + this.getQueryUrlFragment();
        };
        getIndexingBatchStatsCommand.prototype.getQueryUrlFragment = function () {
            return "/debug/indexing-perf-stats-with-timings"; //TODO: use endpoints
        };
        return getIndexingBatchStatsCommand;
    }(commandBase));
    return getIndexingBatchStatsCommand;
});

//# sourceMappingURL=getIndexingBatchStatsCommand.js.map
