var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "common/pagedResultSet", "models/database/documents/document", "endpoints"], function (require, exports, commandBase, pagedResultSet, document, endpoints) {
    "use strict";
    /*
     * getAllDocumentsCommand is a specialized command that fetches all the documents in a specified database.
    */
    var getAllDocumentsCommand = (function (_super) {
        __extends(getAllDocumentsCommand, _super);
        function getAllDocumentsCommand(ownerDatabase, skip, take) {
            var _this = _super.call(this) || this;
            _this.ownerDatabase = ownerDatabase;
            _this.skip = skip;
            _this.take = take;
            return _this;
        }
        getAllDocumentsCommand.prototype.execute = function () {
            // Getting all documents requires a 2 step process:
            // 1. Fetch /collections/stats to get the total doc count.
            // 2. Fetch /docs to get the actual documents.
            var docsTask = this.fetchDocs();
            var totalResultsTask = this.fetchTotalResultCount();
            var doneTask = $.Deferred();
            var combinedTask = $.when(docsTask, totalResultsTask);
            combinedTask.done(function (_a, resultsCount) {
                var docsResult = _a[0];
                return doneTask.resolve(new pagedResultSet(docsResult, resultsCount));
            });
            combinedTask.fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        getAllDocumentsCommand.prototype.fetchDocs = function () {
            var args = {
                start: this.skip,
                pageSize: this.take
            };
            var docSelector = function (docs) { return docs.Results.map(function (d) { return new document(d); }); };
            var url = endpoints.databases.document.docs;
            return this.query(url, args, this.ownerDatabase, docSelector);
        };
        getAllDocumentsCommand.prototype.fetchTotalResultCount = function () {
            var url = endpoints.databases.collections.collectionsStats;
            var countSelector = function (dto) { return dto.NumberOfDocuments; };
            return this.query(url, null, this.ownerDatabase, countSelector);
        };
        return getAllDocumentsCommand;
    }(commandBase));
    return getAllDocumentsCommand;
});

//# sourceMappingURL=getAllDocumentsCommand.js.map
