var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "common/pagedResultSet", "models/database/query/facet", "models/database/documents/document"], function (require, exports, commandBase, pagedResultSet, facet, document) {
    "use strict";
    var queryFacetsCommand = (function (_super) {
        __extends(queryFacetsCommand, _super);
        function queryFacetsCommand(indexName, queryText, skip, take, facets, db, disableCache) {
            if (disableCache === void 0) { disableCache = false; }
            var _this = _super.call(this) || this;
            _this.indexName = indexName;
            _this.queryText = queryText;
            _this.skip = skip;
            _this.take = take;
            _this.facets = facets;
            _this.db = db;
            _this.disableCache = disableCache;
            _this.argsUrl = _this.urlEncodeArgs({
                query: _this.queryText ? _this.queryText : undefined,
                facetStart: _this.skip,
                facetPageSize: _this.take,
                disableCache: _this.disableCache ? Date.now() : undefined,
                facets: JSON.stringify(_this.facets)
            });
            return _this;
        }
        queryFacetsCommand.prototype.execute = function () {
            var _this = this;
            var url = "/facets/" + this.indexName + this.argsUrl; //TODO: use endpoints
            // Querying facets returns a facetResultSetDto. We need to massage that
            // data into something that can be displayed in the grid: the pagedResultSet.
            var finishedTask = $.Deferred();
            this.query(url, null, this.db)
                .fail(function (response) {
                _this.reportError("Unable to run query.", response.responseText, response.statusText);
                finishedTask.reject(response);
            })
                .done(function (results) {
                finishedTask.resolve(_this.parseResults(results));
            });
            return finishedTask;
        };
        queryFacetsCommand.prototype.parseResults = function (resultSet) {
            var items = [];
            var totalItemCount = 0;
            // The .Results property contains properties in the form of [facet name]-[facet aggregation field].
            // For example: Company-Total
            // Each of these properties will be of type facetResultDto.
            // In the UI, we display these as "[facet aggregation] of [facet aggregation field]", e.g. "Count of Total".
            var propIndex = 0;
            for (var prop in resultSet.Results) {
                var facetResult = resultSet.Results[prop];
                var propNameParts = prop.split('-');
                var aggregateField = propNameParts[1];
                var remainingTerms = facetResult.RemainingTermsCount || 0;
                totalItemCount = facetResult.Values.length + remainingTerms + this.skip;
                // Construct our result set, an array objects that take the following shape:
                // { 'Key': [facet range], 'Count of Foobar': 3, 'Min of Blah': 4, ... }
                for (var i = 0; i < facetResult.Values.length; i++) {
                    var facetValue = facetResult.Values[i];
                    var item = items[i];
                    if (!item) {
                        item = new document({});
                        items[i] = item;
                    }
                    item['Key'] = facetValue.Range;
                    for (var power = 0; power < 5; power++) {
                        var curFieldName = facet.getLabelForAggregation(Math.pow(2, power));
                        var curFieldVal = facetValue[curFieldName];
                        if (!!curFieldVal) {
                            if (typeof curFieldVal === "number") {
                                var fixedVal = curFieldVal.toFixed(2);
                                if (fixedVal != curFieldVal.toString()) {
                                    curFieldVal = fixedVal;
                                }
                            }
                            item[curFieldName + " of " + aggregateField] = curFieldVal;
                        }
                    }
                }
                propIndex++;
            }
            return new pagedResultSet(items, totalItemCount, resultSet.Duration);
        };
        return queryFacetsCommand;
    }(commandBase));
    return queryFacetsCommand;
});

//# sourceMappingURL=queryFacetsCommand.js.map
