var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "models/database/documents/document", "common/pagedResultSet"], function (require, exports, commandBase, document, pagedResultSet) {
    "use strict";
    var queryIndexCommand = (function (_super) {
        __extends(queryIndexCommand, _super);
        function queryIndexCommand(indexName, db, skip, take, queryText, sorts, transformerQuery, showFields, indexEntries, useAndOperator, disableCache) {
            var _this = _super.call(this) || this;
            _this.indexName = indexName;
            _this.db = db;
            _this.skip = skip;
            _this.take = take;
            _this.queryText = queryText;
            _this.sorts = sorts;
            _this.transformerQuery = transformerQuery;
            _this.showFields = showFields;
            _this.indexEntries = indexEntries;
            _this.useAndOperator = useAndOperator;
            _this.disableCache = disableCache;
            return _this;
        }
        queryIndexCommand.prototype.execute = function () {
            var _this = this;
            var selector = function (results) { return new pagedResultSet(results.Results.map(function (d) { return new document(d); }), results.TotalResults, results); };
            var queryTask = this.query(this.getUrl(), null, this.db, selector);
            queryTask.fail(function (response) { return _this.reportError("Error querying index", response.responseText, response.statusText); });
            return queryTask;
        };
        queryIndexCommand.prototype.getUrl = function () {
            var url = "/queries/" + this.indexName; //TODO: use endpoints
            //var resultsTransformerUrlFragment = this.transformer && this.transformer.name() ? "&resultsTransformer=" + this.transformer.name() : ""; // This should not be urlEncoded, as it breaks the query.
            var resultsTransformerUrlFragment = (this.transformerQuery ? this.transformerQuery.toUrl() : "");
            var urlArgs = this.urlEncodeArgs({
                query: this.queryText ? this.queryText : undefined,
                start: this.skip,
                pageSize: this.take,
                sort: this.sorts.map(function (s) { return s.toQuerySortString(); }),
                fetch: this.showFields ? "__all_fields" : undefined,
                debug: this.indexEntries ? "entries" : undefined,
                operator: this.useAndOperator ? "AND" : undefined,
                disableCache: this.disableCache ? Date.now() : undefined
            }) + resultsTransformerUrlFragment;
            return url + urlArgs;
        };
        queryIndexCommand.prototype.getCsvUrl = function () {
            var url = "/streams/query/" + this.indexName; //TODO: use endpoints
            var resultsTransformerUrlFragment = (this.transformerQuery ? this.transformerQuery.toUrl() : "");
            var urlArgs = this.urlEncodeArgs({
                query: this.queryText ? this.queryText : undefined,
                sort: this.sorts.map(function (s) { return s.toQuerySortString(); }),
                fetch: this.showFields ? "__all_fields" : undefined,
                debug: this.indexEntries ? "entries" : undefined,
                operator: this.useAndOperator ? "AND" : undefined,
                format: "excel",
                download: true
            }) + resultsTransformerUrlFragment;
            return url + urlArgs;
        };
        return queryIndexCommand;
    }(commandBase));
    return queryIndexCommand;
});

//# sourceMappingURL=queryIndexCommand.js.map
