var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "commands/filesystem/getConfigurationByKeyCommand"], function (require, exports, commandBase, getConfigurationByKeyCommand) {
    "use strict";
    var backupFilesystemCommand = (function (_super) {
        __extends(backupFilesystemCommand, _super);
        function backupFilesystemCommand(fs, backupLocation, updateBackupStatus, incremental) {
            var _this = _super.call(this) || this;
            _this.fs = fs;
            _this.backupLocation = backupLocation;
            _this.updateBackupStatus = updateBackupStatus;
            _this.incremental = incremental;
            return _this;
        }
        backupFilesystemCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            var args = {
                BackupLocation: this.backupLocation,
                DatabaseDocument: null
            };
            this.post('/admin/backup?incremental=' + this.incremental, JSON.stringify(args), this.fs, { dataType: 'text' })
                .fail(function (response) {
                _this.reportError("Failed to create backup!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function () { return _this.getBackupStatus(result); });
            return result;
        };
        backupFilesystemCommand.prototype.getBackupStatus = function (result) {
            var _this = this;
            new getConfigurationByKeyCommand(this.fs, "Raven/Backup/Status")
                .execute()
                .fail(function (response) {
                _this.reportError("Failed to fetch backup status!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (backupStatusAsString) {
                var backupStatus = JSON.parse(backupStatusAsString);
                _this.updateBackupStatus(backupStatus);
                if (backupStatus.IsRunning) {
                    setTimeout(function () { return _this.getBackupStatus(result); }, 1000);
                }
                else {
                    if (backupStatus.Success) {
                        _this.reportSuccess("Filesystem backup was successfully created!");
                    }
                    else {
                        var cause = _.findLast(backupStatus.Messages, function (x) { return x.Severity === 'Error'; });
                        _this.reportError("Failed to backup filesystem: " + cause.Message);
                    }
                    result.resolve();
                }
            });
        };
        return backupFilesystemCommand;
    }(commandBase));
    return backupFilesystemCommand;
});

//# sourceMappingURL=backupFilesystemCommand.js.map
