var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "models/filesystem/synchronizationDetail"], function (require, exports, commandBase, synchronizationDetail) {
    "use strict";
    var getSyncIncomingActivitiesCommand = (function (_super) {
        __extends(getSyncIncomingActivitiesCommand, _super);
        function getSyncIncomingActivitiesCommand(fs, activity, skip, take) {
            var _this = _super.call(this) || this;
            _this.fs = fs;
            _this.activity = activity;
            _this.skip = skip;
            _this.take = take;
            return _this;
        }
        getSyncIncomingActivitiesCommand.prototype.execute = function () {
            var _this = this;
            var doneTask = $.Deferred();
            this.getActivities()
                .done(function (list) { return doneTask.resolve(list); })
                .fail(function (xhr) {
                _this.reportError("Failed to get outgoing and " + _this.activity + " synchronization activities.");
                doneTask.reject(xhr);
            });
            return doneTask;
        };
        getSyncIncomingActivitiesCommand.prototype.getActivities = function () {
            var _this = this;
            var url;
            var resultsSelector;
            switch (this.activity) {
                case 1 /* Active */:
                    url = "/synchronization/incoming";
                    resultsSelector = function (x) {
                        return {
                            TotalCount: x.TotalCount,
                            Items: x.Items.map(function (item) { return new synchronizationDetail(item, 1 /* Incoming */, _this.activity); })
                        };
                    };
                    break;
                case 3 /* Finished */:
                    url = "/synchronization/finished";
                    resultsSelector = function (x) {
                        return {
                            TotalCount: x.TotalCount,
                            Items: x.Items.map(function (item) {
                                var detail = new synchronizationDetail({
                                    FileName: item.FileName,
                                    DestinationUrl: "",
                                    FileETag: item.FileETag,
                                    Type: item.Type
                                }, 1 /* Incoming */, _this.activity);
                                if (item.Exception != null) {
                                    detail.AdditionalInfo(item.Exception);
                                }
                                return detail;
                            })
                        };
                    };
                    break;
                default:
                    throw new TypeError("Not supported synchronization activity type when attempting to retrieve incoming activities. Given type: " + this.activity);
            }
            return this.query(url, { start: this.skip, pageSize: this.take }, this.fs, resultsSelector, 15000);
        };
        return getSyncIncomingActivitiesCommand;
    }(commandBase));
    return getSyncIncomingActivitiesCommand;
});

//# sourceMappingURL=getSyncIncomingActivitiesCommand.js.map
