var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "models/filesystem/uploadItem"], function (require, exports, commandBase, uploadItem) {
    "use strict";
    var uploadFileToFilesystemCommand = (function (_super) {
        __extends(uploadFileToFilesystemCommand, _super);
        function uploadFileToFilesystemCommand(source, directory, uploadId, fs, progressHandlingFunction, reportUploadProgress) {
            if (reportUploadProgress === void 0) { reportUploadProgress = true; }
            var _this = _super.call(this) || this;
            _this.source = source;
            _this.directory = directory;
            _this.uploadId = uploadId;
            _this.fs = fs;
            _this.reportUploadProgress = reportUploadProgress;
            return _this;
        }
        uploadFileToFilesystemCommand.prototype.execute = function () {
            var _this = this;
            if (this.reportUploadProgress) {
                this.reportInfo("File " + this.source.name + "queued for upload...");
            }
            var fileName = this.directory + "/" + this.source.name;
            var url = '/files?name=' + encodeURIComponent(fileName) + '&uploadId=' + this.uploadId;
            var customHeaders = {
                'RavenFS-Size': this.source.size
            };
            var deferred = $.Deferred();
            var jQueryOptions = {
                headers: customHeaders,
                processData: false,
                xhr: function () {
                    var myXhr = $.ajaxSettings.xhr();
                    if (myXhr.upload) {
                        myXhr.upload.addEventListener('progress', this.progressHandlingFunction, false); // For handling the progress of the upload
                    }
                    if (myXhr.upload) {
                        myXhr.upload.onprogress = this.progressHandlingFunction;
                    }
                    return myXhr;
                },
                cache: false,
                contentType: "application/json; charset=UTF-8",
                dataType: ''
            };
            var uploadTask = this.put(url, this.source, this.fs, jQueryOptions, 0);
            if (this.reportUploadProgress) {
                uploadTask.done(function () {
                    _this.reportSuccess("Uploaded " + fileName);
                    return deferred.resolve(new uploadItem(_this.uploadId, fileName, "Uploaded", _this.fs));
                });
                uploadTask.fail(function (response) {
                    _this.reportError("Failed to upload " + fileName, response.responseText, response.statusText);
                    return deferred.reject(new uploadItem(_this.uploadId, fileName, "Failed", _this.fs));
                });
            }
            return deferred;
        };
        return uploadFileToFilesystemCommand;
    }(commandBase));
    return uploadFileToFilesystemCommand;
});

//# sourceMappingURL=uploadFileToFilesystemCommand.js.map
