var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "commands/database/documents/getDocumentWithMetadataCommand"], function (require, exports, commandBase, getDocumentWithMetadataCommand) {
    "use strict";
    var backupDatabaseCommand = (function (_super) {
        __extends(backupDatabaseCommand, _super);
        function backupDatabaseCommand(db, backupLocation, updateBackupStatus, incremental) {
            var _this = _super.call(this) || this;
            _this.db = db;
            _this.backupLocation = backupLocation;
            _this.updateBackupStatus = updateBackupStatus;
            _this.incremental = incremental;
            return _this;
        }
        backupDatabaseCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            this.query('/admin/databases?name=' + this.db.name, null, null /* We should query the system URL here */) //TODO: use endpoints
                .fail(function (response) {
                _this.reportError("Failed to fetch database document!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (doc) {
                var args = {
                    BackupLocation: _this.backupLocation,
                    DatabaseDocument: doc
                };
                _this.post('/admin/backup?incremental=' + _this.incremental, JSON.stringify(args), _this.db, { dataType: 'text' }) //TODO: use endpoints
                    .fail(function (response) {
                    _this.reportError("Failed to create backup!", response.responseText, response.statusText);
                    result.reject();
                })
                    .done(function () { return _this.getBackupStatus(result); });
            });
            return result;
        };
        backupDatabaseCommand.prototype.getBackupStatus = function (result) {
            var _this = this;
            new getDocumentWithMetadataCommand("Raven/Backup/Status", this.db)
                .execute()
                .fail(function (response) {
                _this.reportError("Failed to fetch backup status!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (backupStatus) {
                _this.updateBackupStatus(backupStatus);
                if (backupStatus.IsRunning) {
                    setTimeout(function () { return _this.getBackupStatus(result); }, 1000);
                }
                else {
                    if (backupStatus.Success) {
                        _this.reportSuccess("Database backup was successfully created!");
                    }
                    else {
                        var cause = _.findLast(backupStatus.Messages, function (x) { return x.Severity === 'Error'; });
                        _this.reportError("Failed to backup database: " + cause.Message);
                    }
                    result.resolve();
                }
            });
        };
        return backupDatabaseCommand;
    }(commandBase));
    return backupDatabaseCommand;
});

//# sourceMappingURL=backupDatabaseCommand.js.map
