var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "models/database/documents/document", "commands/commandBase", "commands/resources/getDatabaseSettingsCommand", "commands/resources/saveDatabaseSettingsCommand", "commands/database/globalConfig/getConfigurationSettingsCommand"], function (require, exports, document, commandBase, getDatabaseSettingsCommand, saveDatabaseSettingsCommand, getConfigurationSettingsCommand) {
    "use strict";
    var createDefaultSettingsCommand = (function (_super) {
        __extends(createDefaultSettingsCommand, _super);
        function createDefaultSettingsCommand(db, bundles) {
            var _this = _super.call(this) || this;
            _this.db = db;
            _this.bundles = bundles;
            return _this;
        }
        createDefaultSettingsCommand.prototype.execute = function () {
            this.reportInfo("Creating default settings for '" + this.db.name + "'...");
            var tasksToWatch = [];
            if (_.includes(this.bundles, "Quotas")) {
                tasksToWatch.push(this.updateQuotasSettings());
            }
            /* TODO
            if (_.includes(this.bundles, "Versioning")) {
                tasksToWatch.push(this.saveVersioningConfiguration());
            }*/
            if (tasksToWatch.length > 0) {
                return $.when.apply(null, tasksToWatch);
            }
            else {
                return $.Deferred().resolve();
            }
        };
        createDefaultSettingsCommand.prototype.fillDefaultQuotasSettings = function (doc) {
            var result = new document(doc.toDto(true));
            result["Settings"]["Raven/Quotas/Size/HardLimitInKB"] = (50 * 1024).toString();
            result["Settings"]["Raven/Quotas/Size/SoftMarginInKB"] = (45 * 1024).toString();
            result["Settings"]["Raven/Quotas/Documents/HardLimit"] = (10000).toString();
            result["Settings"]["Raven/Quotas/Documents/SoftLimit"] = (8000).toString();
            return result;
        };
        createDefaultSettingsCommand.prototype.saveDatabaseSettings = function (databaseSettings) {
            return new saveDatabaseSettingsCommand(this.db, databaseSettings).execute();
        };
        createDefaultSettingsCommand.prototype.updateQuotasSettings = function () {
            var _this = this;
            var taskDone = $.Deferred();
            this.hasGlobalQuotaSettings().fail(function () { return taskDone.fail(); })
                .done(function (has) {
                if (has) {
                    // use global settings - nothing to do
                    taskDone.resolve();
                }
                else {
                    new getDatabaseSettingsCommand(_this.db, false)
                        .execute()
                        .fail(function () { return taskDone.fail(); })
                        .then(_this.fillDefaultQuotasSettings)
                        .then(function (doc) { return _this.saveDatabaseSettings(doc); })
                        .fail(function () { return taskDone.fail(); })
                        .then(function () { return taskDone.resolve(); });
                }
            });
            return taskDone;
        };
        /* TODO
        private createDefaultVersioningSettings(): Array<versioningEntry> {
            return [
                new versioningEntry({
                    Id: "DefaultConfiguration",
                    MaxRevisions: 5,
                    Exclude: false,
                    ExcludeUnlessExplicit: false,
                    PurgeOnDelete: false
                })
            ];
        }*/
        /*TODO
        private saveVersioningConfiguration(): JQueryPromise<any> {
    
            var saveTask = $.Deferred();
            this.hasGlobalVersioningSettings().fail(() => saveTask.fail())
                .done((has: boolean) => {
                    if (has) {
                        // use global settings - nothing to do
                        saveTask.resolve();
                    } else {
                        var entries: Array<versioningEntryDto> = this.createDefaultVersioningSettings()
                            .map((ve: versioningEntry) => ve.toDto(true));
                        new saveVersioningCommand(this.db, entries).execute()
                            .done(() => saveTask.resolve())
                            .fail(() => saveTask.reject());
                    }
                });
            return saveTask;
        }*/
        /* TODO
    
        private hasGlobalVersioningSettings(): JQueryPromise<boolean> {
            var hasGlobal = $.Deferred();
            new getEffectiveVersioningsCommand(this.db)
                .execute()
                .done((data: configurationDocument<versioningEntry>[]) => {
                    hasGlobal.resolve(!!data.first(config => config.globalExists()));
                })
                .fail(() => hasGlobal.reject());
    
            return hasGlobal.promise();
        }*/
        createDefaultSettingsCommand.prototype.hasGlobalQuotaSettings = function () {
            var hasGlobal = $.Deferred();
            new getConfigurationSettingsCommand(this.db, ["Raven/Quotas/Size/HardLimitInKB"])
                .execute()
                .done(function (result) {
                // note: we detect presence of global config based on single property!
                var hardLimit = result.results["Raven/Quotas/Size/HardLimitInKB"];
                hasGlobal.resolve(hardLimit.globalExists());
            })
                .fail(function () { return hasGlobal.reject(); });
            return hasGlobal.promise();
        };
        return createDefaultSettingsCommand;
    }(commandBase));
    return createDefaultSettingsCommand;
});

//# sourceMappingURL=createDefaultSettingsCommand.js.map
