var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/commandBase", "endpoints"], function (require, exports, commandBase, endpoints) {
    "use strict";
    var disableResourceToggleCommand = (function (_super) {
        __extends(disableResourceToggleCommand, _super);
        function disableResourceToggleCommand(resources, disable) {
            var _this = _super.call(this) || this;
            _this.resources = resources;
            _this.disable = disable;
            return _this;
        }
        Object.defineProperty(disableResourceToggleCommand.prototype, "action", {
            get: function () {
                return this.disable ? "disable" : "enable";
            },
            enumerable: true,
            configurable: true
        });
        disableResourceToggleCommand.prototype.execute = function () {
            var _this = this;
            var tasks = [];
            disableResourceToggleCommand
                .groupResourcesByQualifier(this.resources)
                .forEach(function (resources, group) {
                tasks.push(Promise.resolve(_this.toggleDisableForGroup(group, resources)));
            });
            var joinedResultTask = $.Deferred();
            Promise.all(tasks)
                .then(function (results) {
                var joinedResult = ([]).concat.apply([], results);
                joinedResultTask.resolve(joinedResult);
            })
                .catch(function (result) { return joinedResultTask.reject(result); });
            return joinedResultTask;
        };
        disableResourceToggleCommand.groupResourcesByQualifier = function (resources) {
            var result = new Map();
            resources.forEach(function (rs) {
                var qualifier = rs.qualifier;
                if (result.has(qualifier)) {
                    result.get(qualifier).push(rs);
                }
                else {
                    result.set(qualifier, [rs]);
                }
            });
            return result;
        };
        disableResourceToggleCommand.prototype.toggleDisableForGroup = function (qualifer, resources) {
            var _this = this;
            var args = {
                name: resources.map(function (x) { return x.name; })
            };
            var endPoint = this.disable ?
                endpoints.admin.adminResources.disable :
                endpoints.admin.adminResources.enable;
            var url = "/admin/" +
                resources[0].urlPrefix + endPoint + this.urlEncodeArgs(args);
            var task = $.Deferred();
            this.post(url, null)
                .done(function (result) { return task.resolve(_this.extractAndMapResult(qualifer, result)); })
                .fail(function (reason) { return task.reject(reason); });
            return task;
        };
        disableResourceToggleCommand.prototype.extractAndMapResult = function (qualifer, result) {
            return result.map(function (x) { return ({
                QualifiedName: qualifer + "/" + x.Name,
                Success: x.Success,
                Reason: x.Reason,
                Disabled: x.Disabled
            }); });
        };
        return disableResourceToggleCommand;
    }(commandBase));
    return disableResourceToggleCommand;
});

//# sourceMappingURL=disableResourceToggleCommand.js.map
