/// <reference path="../../typings/tsd.d.ts" />
define(["require", "exports", "common/appUrl", "common/changeSubscription", "common/changesCallback", "commands/commandBase", "common/idGenerator", "commands/database/debug/adminLogsConfigureCommand"], function (require, exports, appUrl, changeSubscription, changesCallback, commandBase, idGenerator, adminLogsConfigureCommand) {
    "use strict";
    //TODO: use abstractWebSocketClient
    var adminLogsClient = (function () {
        function adminLogsClient(token) {
            this.token = token;
            this.readyStateOpen = 1;
            this.isCleanClose = false;
            this.normalClosureCode = 1000;
            this.normalClosureMessage = "CLOSE_NORMAL";
            this.successfullyConnectedOnce = false;
            this.commandBase = new commandBase();
            this.adminLogsHandlers = ko.observableArray();
            this.eventsId = idGenerator.generateId();
            this.resourcePath = appUrl.baseUrl;
            this.connectionOpeningTask = $.Deferred();
            this.connectionClosingTask = $.Deferred();
        }
        adminLogsClient.prototype.connect = function () {
            var connectionString = 'singleUseAuthToken=' + this.token + '&id=' + this.eventsId;
            if ("WebSocket" in window) {
                this.connectWebSocket(connectionString);
            }
            else {
                // The browser doesn't support websocket
                this.connectionOpeningTask.reject();
            }
        };
        adminLogsClient.prototype.connectWebSocket = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            var wsProtocol = window.location.protocol === 'https:' ? 'wss://' : 'ws://';
            this.webSocket = new WebSocket(wsProtocol + window.location.host + this.resourcePath + '/admin/logs/events?' + connectionString);
            this.webSocket.onmessage = function (e) { return _this.onMessage(e); };
            this.webSocket.onerror = function (e) {
                if (connectionOpened == false) {
                    _this.connectionOpeningTask.reject();
                }
                else {
                    _this.connectionClosingTask.resolve({ Error: e });
                }
            };
            this.webSocket.onclose = function (e) {
                _this.connectionClosingTask.resolve();
            };
            this.webSocket.onopen = function () {
                console.log("Connected to WebSocket admin logs");
                _this.successfullyConnectedOnce = true;
                connectionOpened = true;
                _this.connectionOpeningTask.resolve();
            };
        };
        adminLogsClient.prototype.send = function (command, value, needToSaveSentMessages) {
            var _this = this;
            if (needToSaveSentMessages === void 0) { needToSaveSentMessages = true; }
            this.connectionOpeningTask.done(function () {
                var args = {
                    id: _this.eventsId,
                    command: command
                };
                if (value !== undefined) {
                    args["value"] = value;
                }
                //TODO: exception handling?
                _this.commandBase.query('/admin/logs/configure', args, null);
            });
        };
        adminLogsClient.prototype.fireEvents = function (events, param, filter) {
            for (var i = 0; i < events.length; i++) {
                if (filter(param)) {
                    events[i].fire(param);
                }
            }
        };
        adminLogsClient.prototype.onMessage = function (e) {
            var eventDto = JSON.parse(e.data);
            if (!!eventDto.Type && eventDto.Type == 'Heartbeat') {
                return;
            }
            this.fireEvents(this.adminLogsHandlers(), eventDto, function () { return true; });
        };
        adminLogsClient.prototype.watchAdminLogs = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            this.adminLogsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.adminLogsHandlers.remove(callback);
            });
        };
        adminLogsClient.prototype.configureCategories = function (categoriesConfig) {
            new adminLogsConfigureCommand(null, categoriesConfig, this.eventsId).execute();
        };
        adminLogsClient.prototype.dispose = function () {
            var _this = this;
            this.connectionOpeningTask.done(function () {
                if (_this.webSocket) {
                    if (_this.webSocket.readyState === _this.readyStateOpen) {
                        console.log("Disconnecting from WebSocket Logs API");
                        _this.webSocket.close(_this.normalClosureCode, _this.normalClosureMessage);
                    }
                    _this.send('disconnect', undefined, false);
                    _this.isCleanClose = true;
                }
            });
        };
        return adminLogsClient;
    }());
    adminLogsClient.messageWasShownOnce = false;
    return adminLogsClient;
});

//# sourceMappingURL=adminLogsClient.js.map
