/// <reference path="../../typings/tsd.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/changeSubscription", "common/changesCallback", "common/abstractWebSocketClient"], function (require, exports, changeSubscription, changesCallback, abstractWebSocketClient) {
    "use strict";
    /**
     * A.K.A. Global Changes API
     */
    var adminWatchClient = (function (_super) {
        __extends(adminWatchClient, _super);
        function adminWatchClient() {
            var _this = _super.call(this, null) || this;
            _this.allReconnectHandlers = ko.observableArray();
            _this.allAlertsHandlers = ko.observableArray();
            _this.allItemsHandlers = ko.observableArray();
            _this.watchedItems = new Map();
            _this.watchedItemPrefixes = new Map();
            return _this;
        }
        Object.defineProperty(adminWatchClient.prototype, "connectionDescription", {
            get: function () {
                return "Global Changes API";
            },
            enumerable: true,
            configurable: true
        });
        adminWatchClient.prototype.webSocketUrlFactory = function (token) {
            var connectionString = "singleUseAuthToken=" + token.Token;
            return "/admin/watch?" + connectionString;
        };
        adminWatchClient.prototype.onOpen = function () {
            _super.prototype.onOpen.call(this);
            this.connectToWebSocketTask.resolve();
            this.fireEvents(this.allReconnectHandlers(), undefined, function () { return true; });
        };
        adminWatchClient.prototype.onMessage = function (e) {
            var _this = this;
            if (!e.data.trim()) {
                // it is heartbeat only
                return;
            }
            var eventDto = JSON.parse(e.data);
            var eventOperation = eventDto.Operation;
            switch (eventOperation) {
                case "Write":
                case "Delete":
                case "Loaded":
                    this.fireEvents(this.allItemsHandlers(), eventDto, function () { return true; });
                    this.watchedItems.forEach(function (callbacks, key) {
                        _this.fireEvents(callbacks(), eventDto, function (event) { return event.Id != null && event.Id === key; });
                    });
                    this.watchedItemPrefixes.forEach(function (callbacks, key) {
                        _this.fireEvents(callbacks(), eventDto, function (event) { return event.Id != null && event.Id.startsWith(key); });
                    });
                    break;
                case "AlertRaised":
                case "AlertDeleted":
                    this.fireEvents(this.allAlertsHandlers(), eventDto, function () { return true; });
                    break;
                default:
                    console.error("Unhandled Changes API notification type: " + eventDto.Id);
            }
        };
        adminWatchClient.prototype.watchReconnect = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            this.allReconnectHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allReconnectHandlers.remove(callback);
            });
        };
        adminWatchClient.prototype.watchAlerts = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            this.allAlertsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allAlertsHandlers.remove(callback);
            });
        };
        adminWatchClient.prototype.watchAllItems = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            this.allItemsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allItemsHandlers.remove(callback);
            });
        };
        adminWatchClient.prototype.watchItem = function (itemId, onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (!this.watchedItems.has(itemId)) {
                this.watchedItems.set(itemId, ko.observableArray());
            }
            var callbacks = this.watchedItems.get(itemId);
            callbacks.push(callback);
            return new changeSubscription(function () {
                callbacks.remove(callback);
                if (callbacks().length === 0) {
                    _this.watchedItems.delete(itemId);
                }
            });
        };
        adminWatchClient.prototype.watchItemsStartingWith = function (itemIdPrefix, onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (!this.watchedItemPrefixes.has(itemIdPrefix)) {
                this.watchedItemPrefixes.set(itemIdPrefix, ko.observableArray());
            }
            var callbacks = this.watchedItemPrefixes.get(itemIdPrefix);
            callbacks.push(callback);
            return new changeSubscription(function () {
                callbacks.remove(callback);
                if (callbacks().length === 0) {
                    _this.watchedItemPrefixes.delete(itemIdPrefix);
                }
            });
        };
        return adminWatchClient;
    }(abstractWebSocketClient));
    return adminWatchClient;
});

//# sourceMappingURL=adminWatchClient.js.map
