/// <reference path="../../typings/tsd.d.ts"/>
define(["require", "exports", "models/resources/database", "models/filesystem/filesystem", "models/counter/counterStorage", "models/timeSeries/timeSeries", "common/shell/activeResourceTracker", "plugins/router", "common/messagePublisher"], function (require, exports, database, filesystem, counterStorage, timeSeries, activeResource, router, messagePublisher) {
    "use strict";
    var appUrl = (function () {
        function appUrl() {
        }
        appUrl.detectAppUrl = function () {
            var path = window.location.pathname.replace("\\", "/").replace("%5C", "/");
            var suffix = "studio/index.html";
            if (path.endsWith(suffix)) {
                return path.substring(0, path.length - suffix.length - 1);
            }
            return "";
        };
        appUrl.checkIsAreaActive = function (routeRoot) {
            var items = router.routes.filter(function (m) { return m.isActive() && m.route != null && m.route != ''; });
            var isThereAny = items.some(function (m) { return m.route.substring(0, routeRoot.length) === routeRoot; });
            return isThereAny;
        };
        appUrl.getEncodedCounterStoragePart = function (cs) {
            return cs ? "&counterstorage=" + encodeURIComponent(cs.name) : "";
        };
        appUrl.forCounterStorageCounters = function (gruopName, cs) {
            var groupPart = gruopName ? "group=" + encodeURIComponent(gruopName) : "";
            var counterStoragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#counterstorages/counters?" + groupPart + counterStoragePart;
        };
        appUrl.forCounterStorageReplication = function (cs) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#counterstorages/replication?" + counterStroragePart;
        };
        appUrl.forCounterStorageTasks = function (cs) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#counterstorages/tasks?" + counterStroragePart;
        };
        appUrl.forImportCounterStorage = function (cs) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#databases/tasks/importCounterStorage?" + counterStroragePart;
        };
        appUrl.forExportCounterStorage = function (cs) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#databases/tasks/exportCounterStorage?" + counterStroragePart;
        };
        appUrl.forCounterStorageStats = function (counterStorage) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(counterStorage);
            return "#counterstorages/stats?" + counterStroragePart;
        };
        appUrl.forCounterStorageConfiguration = function (counterStorage) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(counterStorage);
            return "#counterstorages/configuration?" + counterStroragePart;
        };
        appUrl.forTimeSeriesType = function (type, ts) {
            var url = "";
            if (type) {
                url = "type=" + encodeURIComponent(type);
            }
            var timeSeriesPart = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/types?" + url + timeSeriesPart;
        };
        appUrl.forTimeSeriesKey = function (type, key, ts) {
            var url = "";
            if (type && key) {
                url = "type=" + encodeURIComponent(type) + "&key=" + encodeURIComponent(key);
            }
            var timeSeriesPart = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/points?" + url + timeSeriesPart;
        };
        appUrl.forTimeSeriesStats = function (ts) {
            var part = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/stats?" + part;
        };
        appUrl.forTimeSeriesConfiguration = function (ts) {
            var part = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/configuration?" + part;
        };
        appUrl.forTimeSeriesConfigurationTypes = function (ts) {
            var part = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/configuration/types?" + part;
        };
        appUrl.getEncodedTimeSeriesPart = function (ts) {
            return ts ? "&timeseries=" + encodeURIComponent(ts.name) : "";
        };
        appUrl.forUpgrade = function (db) {
            return "#databases/upgrade?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forAdminSettings = function () {
            return "#admin/settings";
        };
        appUrl.forApiKeys = function () {
            return "#admin/settings/apiKeys";
        };
        appUrl.forCluster = function () {
            return "#admin/settings/cluster";
        };
        appUrl.forGlobalConfig = function () {
            return '#admin/settings/globalConfig';
        };
        appUrl.forServerSmugging = function () {
            return "#admin/settings/serverSmuggling";
        };
        appUrl.forGlobalConfigPeriodicExport = function () {
            return '#admin/settings/globalConfig';
        };
        appUrl.forGlobalConfigDatabaseSettings = function () {
            return '#admin/settings/globalConfigDatabaseSettings';
        };
        appUrl.forGlobalConfigReplication = function () {
            return '#admin/settings/globalConfigReplication';
        };
        appUrl.forGlobalConfigSqlReplication = function () {
            return "#admin/settings/globalConfigSqlReplication";
        };
        appUrl.forGlobalConfigQuotas = function () {
            return '#admin/settings/globalConfigQuotas';
        };
        appUrl.forGlobalConfigCustomFunctions = function () {
            return '#admin/settings/globalConfigCustomFunctions';
        };
        appUrl.forGlobalConfigVersioning = function () {
            return "#admin/settings/globalConfigVersioning";
        };
        appUrl.forBackup = function () {
            return "#admin/settings/backup";
        };
        appUrl.forHotSpare = function () {
            return "#admin/settings/hotSpare";
        };
        appUrl.forTempManageServer = function () {
            return "#admin/settings/manage";
        };
        appUrl.forCompact = function () {
            return "#admin/settings/compact";
        };
        appUrl.forRestore = function () {
            return "#admin/settings/restore";
        };
        appUrl.forAdminLogs = function () {
            return "#admin/settings/adminLogs";
        };
        appUrl.forServerTopology = function () {
            return "#admin/settings/topology";
        };
        appUrl.forTrafficWatch = function () {
            return "#admin/settings/trafficWatch";
        };
        appUrl.forLicenseInformation = function () {
            return "#admin/settings/licenseInformation";
        };
        appUrl.forDebugInfo = function () {
            return "#admin/settings/debugInfo";
        };
        appUrl.forIoTest = function () {
            return "#admin/settings/ioTest";
        };
        appUrl.forDiskIoViewer = function () {
            return "#admin/settings/diskIoViewer";
        };
        appUrl.forAdminJsConsole = function () {
            return "#admin/settings/console";
        };
        appUrl.forStudioConfig = function () {
            return "#admin/settings/studioConfig";
        };
        appUrl.forResources = function () {
            return "#resources";
        };
        appUrl.forAbout = function () {
            return "#about";
        };
        appUrl.forHasApiKey = function () {
            return "#has-api-key";
        };
        appUrl.forCounterStorages = function () {
            return "#counterstorages";
        };
        appUrl.forTimeSeries = function () {
            return "#timeseries";
        };
        appUrl.forEditDoc = function (id, db) {
            var databaseUrlPart = appUrl.getEncodedDbPart(db);
            var docIdUrlPart = id ? "&id=" + encodeURIComponent(id) : "";
            return "#databases/edit?" + docIdUrlPart + databaseUrlPart;
        };
        appUrl.forEditItem = function (itemId, rs, itemIndex, collectionName) {
            var urlPart = appUrl.getEncodedResourcePart(rs);
            var itemIdUrlPart = itemId ? "&id=" + encodeURIComponent(itemId) : "";
            var pagedListInfo = collectionName && itemIndex != null ? "&list=" + encodeURIComponent(collectionName) + "&item=" + itemIndex : "";
            var resourceTag = rs instanceof filesystem ? "#filesystems" : rs instanceof counterStorage ? "#counterstorages" : "#databases";
            return resourceTag + "/edit?" + itemIdUrlPart + urlPart + pagedListInfo;
        };
        appUrl.forEditCounter = function (rs, groupName, counterName) {
            var urlPart = appUrl.getEncodedResourcePart(rs);
            var itemIdUrlPart = "&groupName=" + encodeURIComponent(groupName) + "&counterName=" + encodeURIComponent(counterName);
            return "#counterstorages/edit?" + itemIdUrlPart + urlPart;
        };
        appUrl.forEditQueryItem = function (itemNumber, res, index, query, sort) {
            var databaseUrlPart = appUrl.getEncodedResourcePart(res);
            var indexUrlPart = "&index=" + index;
            var itemNumberUrlPart = "&item=" + itemNumber;
            var queryInfoUrlPart = query ? "&query=" + encodeURIComponent(query) : "";
            var sortInfoUrlPart = sort ? "&sorts=" + sort : "";
            var resourceTag = res instanceof filesystem ? "#filesystems" : "#databases";
            return resourceTag + "/edit?" + databaseUrlPart + indexUrlPart + itemNumberUrlPart + queryInfoUrlPart + sortInfoUrlPart;
        };
        appUrl.forNewDoc = function (db, collection) {
            if (collection === void 0) { collection = null; }
            var databaseUrlPart = appUrl.getEncodedDbPart(db);
            var url = "#databases/edit?" + databaseUrlPart;
            if (collection) {
                url += "&new=" + encodeURIComponent(collection);
            }
            return url;
        };
        /**
        * Gets the URL for status page.
        * @param database The database to use in the URL. If null, the current database will be used.
        */
        appUrl.forStatus = function (db) {
            return "#databases/status?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forReplicationPerfStats = function (db) {
            return "#databases/status/replicationPerfStats?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forSqlReplicationPerfStats = function (db) {
            return "#databases/status/sqlReplicationPerfStats?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIoStats = function (db) {
            return "#databases/status/ioStats?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forRequestsCount = function (db) {
            return "#databases/status/requests?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forRequestsTracing = function (db) {
            return "#databases/status/requests/tracing?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIndexPerformance = function (db, indexName) {
            return "#databases/indexes/performance?" + (appUrl.getEncodedDbPart(db)) + "&" + appUrl.getEncodedIndexNamePart(indexName);
        };
        appUrl.forIndexBatchSize = function (db) {
            return "#databases/status/indexing/batchSize?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIndexPrefetches = function (db) {
            return "#databases/status/indexing/prefetches?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebug = function (db) {
            return "#databases/status/debug?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugChanges = function (db) {
            return "#databases/status/debug?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugMetrics = function (db) {
            return "#databases/status/debug/metrics?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugConfig = function (db) {
            return "#databases/status/debug/config?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugDocrefs = function (db) {
            return "#databases/status/debug/docrefs?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugCurrentlyIndexing = function (db) {
            return "#databases/status/debug/currentlyIndexing?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugQueries = function (db) {
            return "#databases/status/debug/queries?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugTasks = function (db) {
            return "#databases/status/debug/tasks?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugRoutes = function (db) {
            return "#databases/status/debug/routes?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forRequestTracing = function (db) {
            return "#databases/status/requests/tracking?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugSqlReplication = function (db) {
            return "#databases/status/debug/sqlReplication?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugIndexFields = function (db) {
            return "#databases/status/debug/indexFields?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugIdentities = function (db) {
            return "#databases/status/debug/identities?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugWebSocket = function (db) {
            return "#databases/status/debug/webSocket?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugExplainReplication = function (db) {
            return "#databases/status/debug/explainReplication?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forInfoPackage = function (db) {
            return '#databases/status/infoPackage?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusStorageOnDisk = function (db) {
            return '#databases/status/storage?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusStorageStats = function (db) {
            return '#databases/status/storage/stats?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusStorageBreakdown = function (db) {
            return '#databases/status/storage/storageBreakdown?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusStorageCollections = function (db) {
            return '#databases/status/storage/collections?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusStorageReport = function (db) {
            return '#databases/status/storage/report?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forSettings = function (db) {
            return "#databases/settings/databaseSettings?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forLogs = function (db) {
            return "#databases/status/logs?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forRunningTasks = function (db) {
            return "#databases/status/runningTasks?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forAlerts = function (db) {
            return "#databases/status/alerts?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIndexErrors = function (db) {
            return "#databases/indexes/indexErrors?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forReplicationStats = function (db) {
            return "#databases/status/replicationStats?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forUserInfo = function (db) {
            return "#databases/status/userInfo?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forVisualizer = function (db, index) {
            if (index === void 0) { index = null; }
            var url = "#databases/indexes/visualizer?" + appUrl.getEncodedDbPart(db);
            if (index) {
                url += "&index=" + index;
            }
            return url;
        };
        appUrl.forDataSubscriptions = function (db) {
            return "#databases/status/debug/dataSubscriptions?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forDatabaseSettings = function (db) {
            return "#databases/settings/databaseSettings?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forQuotas = function (db) {
            return "#databases/settings/quotas?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forPeriodicExport = function (db) {
            return "#databases/settings/periodicExport?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forReplications = function (db) {
            return "#databases/settings/replication?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forEtl = function (db) {
            return "#databases/settings/etl?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forVersioning = function (db) {
            return "#databases/settings/versioning?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forSqlReplications = function (db) {
            return "#databases/settings/sqlReplication?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forEditSqlReplication = function (sqlReplicationName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/settings/editSqlReplication/" + encodeURIComponent(sqlReplicationName) + "?" + databasePart;
        };
        appUrl.forSqlReplicationConnections = function (db) {
            return "#databases/settings/sqlReplicationConnectionStringsManagement?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forScriptedIndexes = function (db) {
            return "#databases/settings/scriptedIndex?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forCustomFunctionsEditor = function (db) {
            return "#databases/settings/customFunctionsEditor?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forDatabaseStudioConfig = function (db) {
            return "#databases/settings/databaseStudioConfig?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forDocuments = function (collection, db) {
            var collectionPart = collection ? "collection=" + encodeURIComponent(collection) : "";
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/documents?" + collectionPart + databasePart;
        };
        appUrl.forDocumentsByDatabaseName = function (collection, dbName) {
            var collectionPart = collection ? "collection=" + encodeURIComponent(collection) : "";
            return "#/databases/documents?" + collectionPart + "&database=" + encodeURIComponent(dbName);
            ;
        };
        appUrl.forConflicts = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/conflicts?" + databasePart;
        };
        appUrl.forPatch = function (db, hashOfRecentPatch) {
            var databasePart = appUrl.getEncodedDbPart(db);
            if (hashOfRecentPatch) {
                var patchPath = "recentpatch-" + hashOfRecentPatch;
                return "#databases/patch/" + encodeURIComponent(patchPath) + "?" + databasePart;
            }
            else {
                return "#databases/patch?" + databasePart;
            }
        };
        appUrl.forIndexes = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes?" + databasePart;
        };
        appUrl.forNewIndex = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/edit?" + databasePart;
        };
        appUrl.forEditIndex = function (indexName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/edit/" + encodeURIComponent(indexName) + "?" + databasePart;
        };
        appUrl.forNewTransformer = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers/edit?" + databasePart;
        };
        appUrl.forEditTransformer = function (transformerName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers/edit/" + encodeURIComponent(transformerName) + "?" + databasePart;
        };
        appUrl.forTransformers = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers?" + databasePart;
        };
        appUrl.forQuery = function (db, indexNameOrHashToQuery) {
            var databasePart = appUrl.getEncodedDbPart(db);
            var indexToQueryComponent = indexNameOrHashToQuery;
            if (typeof indexNameOrHashToQuery === "number") {
                indexToQueryComponent = "recentquery-" + indexNameOrHashToQuery;
            }
            var indexPart = indexToQueryComponent ? "/" + encodeURIComponent(indexToQueryComponent) : "";
            return "#databases/query/index" + indexPart + "?" + databasePart;
        };
        appUrl.forReporting = function (db, indexName) {
            var databasePart = appUrl.getEncodedDbPart(db);
            var indexPart = indexName ? "/" + encodeURIComponent(indexName) : "";
            return "#databases/query/reporting" + indexPart + "?" + databasePart;
        };
        appUrl.forExploration = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/query/exploration?" + databasePart;
        };
        appUrl.forTasks = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks?" + databasePart;
        };
        appUrl.forResourceQuery = function (res) {
            if (res && res instanceof database) {
                return appUrl.baseUrl + "/databases/" + res.name;
            }
            else if (res && res instanceof filesystem) {
                return appUrl.baseUrl + "/fs/" + res.name;
            }
            else if (res && res instanceof counterStorage) {
                return appUrl.baseUrl + "/cs/" + res.name;
            }
            else if (res && res instanceof timeSeries) {
                return appUrl.baseUrl + "/ts/" + res.name;
            }
            return this.baseUrl;
        };
        appUrl.forTerms = function (indexName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/terms/" + encodeURIComponent(indexName) + "?" + databasePart;
        };
        appUrl.forMegeSuggestions = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/mergeSuggestions?" + databasePart;
        };
        appUrl.forImportDatabase = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/importDatabase?" + databasePart;
        };
        appUrl.forExportDatabase = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/exportDatabase?" + databasePart;
        };
        appUrl.forImportFilesystem = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/tasks/importFilesystem?" + filesystemPart;
        };
        appUrl.forExportFilesystem = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/tasks/exportFilesystem?" + filesystemPart;
        };
        appUrl.forExportCollectionCsv = function (collection, db, customColumns) {
            if (collection.isAllDocuments || collection.isSystemDocuments) {
                return null;
            }
            var args = {
                format: "excel",
                download: true,
                query: "Tag:" + collection.name,
                column: customColumns
            };
            //TODO: we don't have Raven/DocumentsByEntityName anymore
            return appUrl.forResourceQuery(db) + "/streams/query/Raven/DocumentsByEntityName" + appUrl.urlEncodeArgs(args);
        };
        appUrl.forSetAcknowledgedEtag = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/subscriptionsTask?" + databasePart;
        };
        appUrl.forSampleData = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/sampleData?" + databasePart;
        };
        appUrl.forCsvImport = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/csvImport?" + databasePart;
        };
        appUrl.forCounterStorage = function (cs) {
            var counterStoragePart = appUrl.getEncodedCounterPart(cs);
            return "#counterstorages?" + counterStoragePart;
        };
        appUrl.forIndexesRawData = function (db) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/indexes";
        };
        appUrl.forIndexQueryRawData = function (db, indexName) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/indexes/" + indexName;
        };
        appUrl.forReportingRawData = function (db, indexName) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/facets/" + indexName;
        };
        appUrl.forTransformersRawData = function (db) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/transformers";
        };
        appUrl.forDatabasesRawData = function () {
            return window.location.protocol + "//" + window.location.host + "/databases";
        };
        appUrl.forDocumentRawData = function (db, docId) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/docs?id=" + docId;
        };
        appUrl.forFilesystemFiles = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/files?" + filesystemPart;
        };
        appUrl.forFilesystemSearch = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/search?" + filesystemPart;
        };
        appUrl.forFilesystemSynchronization = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/synchronization?" + filesystemPart;
        };
        appUrl.forFilesystemSynchronizationDestinations = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/synchronization/destinations?" + filesystemPart;
        };
        appUrl.forFilesystemSynchronizationConfiguration = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/synchronization/configuration?" + filesystemPart;
        };
        appUrl.forFilesystemStatus = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/status?" + filesystemPart;
        };
        appUrl.forFilesystemTasks = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/tasks?" + filesystemPart;
        };
        appUrl.forFilesystemSettings = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/settings?" + filesystemPart;
        };
        appUrl.forFilesystemConfiguration = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/configuration?" + filesystemPart;
        };
        appUrl.forFilesystemVersioning = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/settings?" + filesystemPart;
        };
        appUrl.forFilesystemConfigurationWithKey = function (fs, key) {
            var filesystemPart = appUrl.getEncodedFsPart(fs) + "&key=" + encodeURIComponent(key);
            return "#filesystems/configuration?" + filesystemPart;
        };
        appUrl.forEditFile = function (id, fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            var fileIdPart = id ? "&id=" + encodeURIComponent(id) : "";
            return "#filesystems/edit?" + fileIdPart + filesystemPart;
        };
        appUrl.getResourceNameFromUrl = function (urlParamName) {
            var indicator = urlParamName + "=";
            var hash = window.location.hash;
            var index = hash.indexOf(indicator);
            if (index >= 0) {
                var segmentEnd = hash.indexOf("&", index);
                if (segmentEnd === -1) {
                    segmentEnd = hash.length;
                }
                var resourceName = hash.substring(index + indicator.length, segmentEnd);
                return decodeURIComponent(resourceName);
            }
            else {
                return null;
            }
        };
        appUrl.getDatabaseNameFromUrl = function () {
            return appUrl.getResourceNameFromUrl(database.type);
        };
        appUrl.getFileSystemNameFromUrl = function () {
            return appUrl.getResourceNameFromUrl(filesystem.type);
        };
        appUrl.getCounterStorageNameFromUrl = function () {
            return appUrl.getResourceNameFromUrl(counterStorage.type);
        };
        appUrl.getTimeSeriesNameFromUrl = function () {
            return appUrl.getResourceNameFromUrl(timeSeries.type);
        };
        /**
        * Gets the server URL.
        */
        appUrl.forServer = function () {
            return window.location.protocol + "//" + window.location.host + appUrl.baseUrl;
        };
        /**
        * Gets the address for the current page but for the specified resource.
        */
        appUrl.forCurrentPage = function (rs) {
            var routerInstruction = router.activeInstruction();
            if (routerInstruction) {
                var currentResourceName = null;
                var currentResourceType = null;
                var currentResourceQualifier = void 0;
                var dbInUrl = routerInstruction.queryParams[database.type];
                if (dbInUrl) {
                    currentResourceName = dbInUrl;
                    currentResourceType = database.type;
                    currentResourceQualifier = database.qualifier;
                }
                else {
                    var fsInUrl = routerInstruction.queryParams[filesystem.type];
                    if (fsInUrl) {
                        currentResourceName = fsInUrl;
                        currentResourceType = filesystem.type;
                        currentResourceQualifier = filesystem.qualifier;
                    }
                    else {
                        var csInUrl = routerInstruction.queryParams[counterStorage.type];
                        if (csInUrl) {
                            currentResourceName = csInUrl;
                            currentResourceType = counterStorage.type;
                            currentResourceQualifier = counterStorage.qualifier;
                        }
                        else {
                            var tsInUrl = routerInstruction.queryParams[timeSeries.type];
                            if (tsInUrl) {
                                currentResourceName = tsInUrl;
                                currentResourceType = timeSeries.type;
                                currentResourceQualifier = timeSeries.qualifier;
                            }
                        }
                    }
                }
                if (currentResourceType && currentResourceQualifier !== rs.qualifier) {
                    // user changed resource type - navigate to resources page and preselect resource
                    return appUrl.forResources() + "?" + rs.type + "=" + encodeURIComponent(rs.name);
                }
                var isDifferentResourceInAddress = !currentResourceName || currentResourceName !== rs.name.toLowerCase();
                if (isDifferentResourceInAddress) {
                    var existingAddress = window.location.hash;
                    var existingQueryString = currentResourceName ? currentResourceType + "=" + encodeURIComponent(currentResourceName) : null;
                    var newQueryString = currentResourceType + "=" + encodeURIComponent(rs.name);
                    return existingQueryString ?
                        existingAddress.replace(existingQueryString, newQueryString) :
                        existingAddress + (window.location.hash.indexOf("?") >= 0 ? "&" : "?") + rs.type + "=" + encodeURIComponent(rs.name);
                }
            }
        };
        appUrl.forCurrentDatabase = function () {
            return appUrl.currentDbComputeds;
        };
        appUrl.forCurrentFilesystem = function () {
            return appUrl.currentDbComputeds; //This is all mixed. maybe there should be separate structures for Db and Fs and Cs.
        };
        appUrl.forCurrentCounterStorage = function () {
            return appUrl.currentDbComputeds; //This is all mixed. maybe there should be separate structures for Db and Fs and Cs.
        };
        appUrl.forCurrentTimeSeries = function () {
            return appUrl.currentDbComputeds; //This is all mixed. maybe there should be separate structures for Db and Fs and Cs.
        };
        appUrl.getEncodedResourcePart = function (res) {
            if (!res)
                return "";
            if (res instanceof filesystem) {
                return appUrl.getEncodedFsPart(res);
            }
            if (res instanceof counterStorage) {
                return appUrl.getEncodedCounterStoragePart(res);
            }
            if (res instanceof timeSeries) {
                return appUrl.getEncodedTimeSeriesPart(res);
            }
            else {
                return appUrl.getEncodedDbPart(res);
            }
        };
        appUrl.getEncodedDbPart = function (db) {
            return db ? "&database=" + encodeURIComponent(db.name) : "";
        };
        appUrl.getEncodedFsPart = function (fs) {
            return fs ? "&filesystem=" + encodeURIComponent(fs.name) : "";
        };
        appUrl.getEncodedCounterPart = function (cs) {
            return cs ? "&counterstorage=" + encodeURIComponent(cs.name) : "";
        };
        appUrl.getEncodedIndexNamePart = function (indexName) {
            return indexName ? "indexName=" + encodeURIComponent(indexName) : "";
        };
        appUrl.mapUnknownRoutes = function (router) {
            router.mapUnknownRoutes(function (instruction) {
                var queryString = !!instruction.queryString ? ("?" + instruction.queryString) : "";
                if (instruction.fragment === "has-api-key" || instruction.fragment.startsWith("api-key")) {
                    // reload page to reinitialize shell and properly consume/provide OAuth token
                    location.reload();
                }
                else {
                    messagePublisher.reportError("Unknown route", "The route " + instruction.fragment + queryString + " doesn't exist, redirecting...");
                    var fragment = instruction.fragment;
                    var appUrls = appUrl.currentDbComputeds;
                    location.href = fragment.startsWith("admin/settings") ? appUrls.adminSettings() : appUrls.resourcesManagement();
                }
            });
        };
        appUrl.urlEncodeArgs = function (args) {
            var propNameAndValues = [];
            for (var _i = 0, _a = Object.keys(args); _i < _a.length; _i++) {
                var prop = _a[_i];
                var value = args[prop];
                if (value instanceof Array) {
                    for (var i = 0; i < value.length; i++) {
                        propNameAndValues.push(prop + "=" + encodeURIComponent(value[i]));
                    }
                }
                else if (value !== undefined) {
                    propNameAndValues.push(prop + "=" + encodeURIComponent(value));
                }
            }
            return "?" + propNameAndValues.join("&");
        };
        return appUrl;
    }());
    appUrl.baseUrl = appUrl.detectAppUrl();
    appUrl.currentDatabase = activeResource.default.database;
    appUrl.currentFilesystem = activeResource.default.fileSystem;
    appUrl.currentCounterStorage = activeResource.default.counterStorage;
    appUrl.currentTimeSeries = activeResource.default.timeSeries;
    // Stores some computed values that update whenever the current database updates.
    appUrl.currentDbComputeds = {
        adminSettings: ko.computed(function () { return appUrl.forAdminSettings(); }),
        adminSettingsCluster: ko.computed(function () { return appUrl.forCluster(); }),
        hasApiKey: ko.computed(function () { return appUrl.forHasApiKey(); }),
        resources: ko.computed(function () { return appUrl.forResources(); }),
        documents: ko.computed(function () { return appUrl.forDocuments(null, appUrl.currentDatabase()); }),
        conflicts: ko.computed(function () { return appUrl.forConflicts(appUrl.currentDatabase()); }),
        patch: ko.computed(function () { return appUrl.forPatch(appUrl.currentDatabase()); }),
        indexes: ko.computed(function () { return appUrl.forIndexes(appUrl.currentDatabase()); }),
        megeSuggestions: ko.computed(function () { return appUrl.forMegeSuggestions(appUrl.currentDatabase()); }),
        upgrade: ko.computed(function () { return appUrl.forUpgrade(appUrl.currentDatabase()); }),
        transformers: ko.computed(function () { return appUrl.forTransformers(appUrl.currentDatabase()); }),
        newIndex: ko.computed(function () { return appUrl.forNewIndex(appUrl.currentDatabase()); }),
        editIndex: function (indexName) { return ko.computed(function () { return appUrl.forEditIndex(indexName, appUrl.currentDatabase()); }); },
        newTransformer: ko.computed(function () { return appUrl.forNewTransformer(appUrl.currentDatabase()); }),
        editTransformer: function (transformerName) { return ko.computed(function () { return appUrl.forEditTransformer(transformerName, appUrl.currentDatabase()); }); },
        query: function (indexName) { return ko.computed(function () { return appUrl.forQuery(appUrl.currentDatabase(), indexName); }); },
        reporting: ko.computed(function () { return appUrl.forReporting(appUrl.currentDatabase()); }),
        exploration: ko.computed(function () { return appUrl.forExploration(appUrl.currentDatabase()); }),
        tasks: ko.computed(function () { return appUrl.forTasks(appUrl.currentDatabase()); }),
        status: ko.computed(function () { return appUrl.forStatus(appUrl.currentDatabase()); }),
        replicationPerfStats: ko.computed(function () { return appUrl.forReplicationPerfStats(appUrl.currentDatabase()); }),
        sqlReplicationPerfStats: ko.computed(function () { return appUrl.forSqlReplicationPerfStats(appUrl.currentDatabase()); }),
        ioStats: ko.computed(function () { return appUrl.forIoStats(appUrl.currentDatabase()); }),
        requestsCount: ko.computed(function () { return appUrl.forRequestsCount(appUrl.currentDatabase()); }),
        requestsTracing: ko.computed(function () { return appUrl.forRequestsTracing(appUrl.currentDatabase()); }),
        indexPerformance: ko.computed(function () { return appUrl.forIndexPerformance(appUrl.currentDatabase()); }),
        indexBatchSize: ko.computed(function () { return appUrl.forIndexBatchSize(appUrl.currentDatabase()); }),
        indexPrefetches: ko.computed(function () { return appUrl.forIndexPrefetches(appUrl.currentDatabase()); }),
        about: ko.computed(function () { return appUrl.forAbout(); }),
        settings: ko.computed(function () { return appUrl.forSettings(appUrl.currentDatabase()); }),
        logs: ko.computed(function () { return appUrl.forLogs(appUrl.currentDatabase()); }),
        runningTasks: ko.computed(function () { return appUrl.forRunningTasks(appUrl.currentDatabase()); }),
        alerts: ko.computed(function () { return appUrl.forAlerts(appUrl.currentDatabase()); }),
        indexErrors: ko.computed(function () { return appUrl.forIndexErrors(appUrl.currentDatabase()); }),
        replicationStats: ko.computed(function () { return appUrl.forReplicationStats(appUrl.currentDatabase()); }),
        userInfo: ko.computed(function () { return appUrl.forUserInfo(appUrl.currentDatabase()); }),
        visualizer: ko.computed(function () { return appUrl.forVisualizer(appUrl.currentDatabase()); }),
        databaseSettings: ko.computed(function () { return appUrl.forDatabaseSettings(appUrl.currentDatabase()); }),
        quotas: ko.computed(function () { return appUrl.forQuotas(appUrl.currentDatabase()); }),
        periodicExport: ko.computed(function () { return appUrl.forPeriodicExport(appUrl.currentDatabase()); }),
        replications: ko.computed(function () { return appUrl.forReplications(appUrl.currentDatabase()); }),
        etl: ko.computed(function () { return appUrl.forEtl(appUrl.currentDatabase()); }),
        hotSpare: ko.computed(function () { return appUrl.forHotSpare(); }),
        versioning: ko.computed(function () { return appUrl.forVersioning(appUrl.currentDatabase()); }),
        sqlReplications: ko.computed(function () { return appUrl.forSqlReplications(appUrl.currentDatabase()); }),
        editSqlReplication: ko.computed(function () { return appUrl.forEditSqlReplication(undefined, appUrl.currentDatabase()); }),
        sqlReplicationsConnections: ko.computed(function () { return appUrl.forSqlReplicationConnections(appUrl.currentDatabase()); }),
        scriptedIndexes: ko.computed(function () { return appUrl.forScriptedIndexes(appUrl.currentDatabase()); }),
        customFunctionsEditor: ko.computed(function () { return appUrl.forCustomFunctionsEditor(appUrl.currentDatabase()); }),
        databaseStudioConfig: ko.computed(function () { return appUrl.forDatabaseStudioConfig(appUrl.currentDatabase()); }),
        statusDebug: ko.computed(function () { return appUrl.forStatusDebug(appUrl.currentDatabase()); }),
        statusDebugChanges: ko.computed(function () { return appUrl.forStatusDebugChanges(appUrl.currentDatabase()); }),
        statusDebugMetrics: ko.computed(function () { return appUrl.forStatusDebugMetrics(appUrl.currentDatabase()); }),
        statusDebugConfig: ko.computed(function () { return appUrl.forStatusDebugConfig(appUrl.currentDatabase()); }),
        statusDebugDocrefs: ko.computed(function () { return appUrl.forStatusDebugDocrefs(appUrl.currentDatabase()); }),
        statusDebugCurrentlyIndexing: ko.computed(function () { return appUrl.forStatusDebugCurrentlyIndexing(appUrl.currentDatabase()); }),
        statusDebugQueries: ko.computed(function () { return appUrl.forStatusDebugQueries(appUrl.currentDatabase()); }),
        statusDebugTasks: ko.computed(function () { return appUrl.forStatusDebugTasks(appUrl.currentDatabase()); }),
        statusDebugRoutes: ko.computed(function () { return appUrl.forStatusDebugRoutes(appUrl.currentDatabase()); }),
        statusDebugSqlReplication: ko.computed(function () { return appUrl.forStatusDebugSqlReplication(appUrl.currentDatabase()); }),
        statusDebugIndexFields: ko.computed(function () { return appUrl.forStatusDebugIndexFields(appUrl.currentDatabase()); }),
        statusDebugIdentities: ko.computed(function () { return appUrl.forStatusDebugIdentities(appUrl.currentDatabase()); }),
        statusDebugWebSocket: ko.computed(function () { return appUrl.forStatusDebugWebSocket(appUrl.currentDatabase()); }),
        statusDebugExplainReplication: ko.computed(function () { return appUrl.forStatusDebugExplainReplication(appUrl.currentDatabase()); }),
        infoPackage: ko.computed(function () { return appUrl.forInfoPackage(appUrl.currentDatabase()); }),
        dataSubscriptions: ko.computed(function () { return appUrl.forDataSubscriptions(appUrl.currentDatabase()); }),
        statusStorageStats: ko.computed(function () { return appUrl.forStatusStorageStats(appUrl.currentDatabase()); }),
        statusStorageOnDisk: ko.computed(function () { return appUrl.forStatusStorageOnDisk(appUrl.currentDatabase()); }),
        statusStorageBreakdown: ko.computed(function () { return appUrl.forStatusStorageBreakdown(appUrl.currentDatabase()); }),
        statusStorageCollections: ko.computed(function () { return appUrl.forStatusStorageCollections(appUrl.currentDatabase()); }),
        statusStorageReport: ko.computed(function () { return appUrl.forStatusStorageReport(appUrl.currentDatabase()); }),
        isAreaActive: function (routeRoot) { return ko.pureComputed(function () { return appUrl.checkIsAreaActive(routeRoot); }); },
        isActive: function (routeTitle) { return ko.pureComputed(function () { return router.navigationModel().find(function (m) { return m.isActive() && m.title === routeTitle; }) != null; }); },
        resourcesManagement: ko.computed(function () { return appUrl.forResources(); }),
        filesystemFiles: ko.computed(function () { return appUrl.forFilesystemFiles(appUrl.currentFilesystem()); }),
        filesystemSearch: ko.computed(function () { return appUrl.forFilesystemSearch(appUrl.currentFilesystem()); }),
        filesystemSynchronization: ko.computed(function () { return appUrl.forFilesystemSynchronization(appUrl.currentFilesystem()); }),
        filesystemStatus: ko.computed(function () { return appUrl.forFilesystemStatus(appUrl.currentFilesystem()); }),
        filesystemTasks: ko.computed(function () { return appUrl.forFilesystemTasks(appUrl.currentFilesystem()); }),
        filesystemSettings: ko.computed(function () { return appUrl.forFilesystemSettings(appUrl.currentFilesystem()); }),
        filesystemSynchronizationDestinations: ko.computed(function () { return appUrl.forFilesystemSynchronizationDestinations(appUrl.currentFilesystem()); }),
        filesystemSynchronizationConfiguration: ko.computed(function () { return appUrl.forFilesystemSynchronizationConfiguration(appUrl.currentFilesystem()); }),
        filesystemConfiguration: ko.computed(function () { return appUrl.forFilesystemConfiguration(appUrl.currentFilesystem()); }),
        filesystemVersioning: ko.computed(function () { return appUrl.forFilesystemVersioning(appUrl.currentFilesystem()); }),
        counterStorages: ko.computed(function () { return appUrl.forCounterStorages(); }),
        counterStorageCounters: ko.computed(function () { return appUrl.forCounterStorageCounters(null, appUrl.currentCounterStorage()); }),
        counterStorageReplication: ko.computed(function () { return appUrl.forCounterStorageReplication(appUrl.currentCounterStorage()); }),
        counterStorageTasks: ko.computed(function () { return appUrl.forCounterStorageTasks(appUrl.currentCounterStorage()); }),
        counterStorageStats: ko.computed(function () { return appUrl.forCounterStorageStats(appUrl.currentCounterStorage()); }),
        counterStorageConfiguration: ko.computed(function () { return appUrl.forCounterStorageConfiguration(appUrl.currentCounterStorage()); }),
        timeSeriesType: ko.computed(function () { return appUrl.forTimeSeriesType(null, appUrl.currentTimeSeries()); }),
        timeSeriesPoints: ko.computed(function () { return appUrl.forTimeSeriesKey(null, null, appUrl.currentTimeSeries()); }),
        timeSeriesStats: ko.computed(function () { return appUrl.forTimeSeriesStats(appUrl.currentTimeSeries()); }),
        timeSeriesConfiguration: ko.computed(function () { return appUrl.forTimeSeriesConfiguration(appUrl.currentTimeSeries()); }),
        timeSeriesConfigurationTypes: ko.computed(function () { return appUrl.forTimeSeriesConfigurationTypes(appUrl.currentTimeSeries()); })
    };
    return appUrl;
});

//# sourceMappingURL=appUrl.js.map
