define(["require", "exports", "durandal/composition", "d3"], function (require, exports, composition, d3) {
    "use strict";
    var autoRefreshBindingHandler = (function () {
        function autoRefreshBindingHandler() {
        }
        autoRefreshBindingHandler.install = function () {
            if (!ko.bindingHandlers["autoRefresh"]) {
                ko.bindingHandlers["autoRefresh"] = new autoRefreshBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler('help');
            }
        };
        autoRefreshBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var config = ko.unwrap(valueAccessor());
            if (!config.onRefresh) {
                throw new Error("onRefresh attribute is required.");
            }
            if (!config.duration) {
                throw new Error("duration attribute is required.");
            }
            var context = autoRefreshBindingHandler.initRefresh(element, config);
            $(element).click(function () { return context.autorefreshEnabled(!context.autorefreshEnabled()); });
            var active = ko.unwrap(config.active);
            if (active) {
                autoRefreshBindingHandler.animatePath(context);
            }
            else {
                context.refreshing(true);
                context.autorefreshEnabled(false);
            }
            var childBindingContext = bindingContext.createChildContext(context, null, function (context) { return ko.utils.extend(context, autoRefreshBindingHandler); });
            ko.applyBindingsToDescendants(childBindingContext, element);
            ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                // break animation (if any)
                context.path
                    .transition()
                    .duration(0);
                context.disposed = true;
            });
            return { controlsDescendantBindings: true };
        };
        autoRefreshBindingHandler.initRefresh = function (element, config) {
            var _this = this;
            var container = d3.select(element);
            container.classed('auto-refresh', true);
            if (!container.attr('title')) {
                container.attr('title', 'Auto refresh');
            }
            container.append("i")
                .attr({
                'class': 'fa fa-fw',
                'data-bind': "css: { 'fa-pause': autorefreshEnabled(), 'fa-refresh': !autorefreshEnabled() }"
            });
            var divContainer = container.append("div")
                .attr({
                'class': 'circleRefresh',
                'data-bind': "css: { 'timer': autorefreshEnabled(), 'dashed': refreshing() }"
            });
            divContainer.append('svg')
                .attr({
                'class': 'inner-svg',
                'width': '80px',
                'height': '80px',
                'viewBox': '0 0 80 80'
            });
            var svg = container.select("svg");
            var group = svg
                .append('g')
                .attr('transform', 'translate(40,40)');
            group.append('circle')
                .attr({
                'class': 'circleRefresh-circlebg',
                cx: 0,
                cy: 0,
                r: 30
            });
            var path = group
                .append('path')
                .attr({
                'class': 'circleRefresh-circle'
            });
            var arc = d3.svg.arc()
                .innerRadius(30)
                .outerRadius(30)
                .startAngle(0)
                .endAngle(function (d) { return d * 2 * Math.PI; });
            var autorefreshEnabled = ko.isObservable(config.active) ? config.active : ko.observable(true);
            var context = {
                svg: svg,
                path: path,
                arc: arc,
                autorefreshEnabled: autorefreshEnabled,
                refreshing: ko.observable(false),
                duration: config.duration,
                onRefresh: config.onRefresh,
                disposed: false
            };
            autorefreshEnabled.subscribe(function () { return _this.toggleAutorefresh(context); });
            return context;
        };
        autoRefreshBindingHandler.animatePath = function (context) {
            var _this = this;
            context.refreshing(false);
            context.path.transition()
                .duration(context.duration)
                .ease('linear')
                .attrTween("d", function () { return function (t) { return context.arc(t); }; })
                .each('end', function () {
                context.refreshing(true);
                context.onRefresh()
                    .always(function () {
                    if (!context.disposed) {
                        _this.animatePath(context);
                    }
                });
            });
        };
        autoRefreshBindingHandler.toggleAutorefresh = function (context) {
            if (!context.autorefreshEnabled()) {
                // stop current animation if any
                if (!context.refreshing()) {
                    context.path
                        .transition()
                        .duration(0);
                }
            }
            else {
                // start with refresh and then animate refresh
                context.path
                    .attr('d', context.arc(1));
                context.refreshing(true);
                context.onRefresh()
                    .always(function () { return autoRefreshBindingHandler.animatePath(context); });
            }
        };
        autoRefreshBindingHandler.prototype.update = function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
        };
        return autoRefreshBindingHandler;
    }());
    return autoRefreshBindingHandler;
});

//# sourceMappingURL=autoRefreshBindingHandler.js.map
