/// <reference path="../../../typings/tsd.d.ts" />
define(["require", "exports", "durandal/composition", "moment"], function (require, exports, composition, moment) {
    "use strict";
    var datePickerBindingHandler = (function () {
        function datePickerBindingHandler() {
        }
        datePickerBindingHandler.install = function () {
            if (!ko.bindingHandlers["datePicker"]) {
                ko.bindingHandlers["datePicker"] = new datePickerBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler("datePicker");
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        datePickerBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var options = allBindings().datepickerOptions || {};
            var endDateElement = options.endDateElement;
            var startDateElement = options.startDateElement;
            delete options.endDateElement;
            delete options.startDateElement;
            var dpicker = $(element).datetimepicker(options);
            dpicker.on('dp.change', function (ev) {
                if (endDateElement) {
                    $("#" + endDateElement).data("DateTimePicker").minDate(ev.date);
                }
                if (startDateElement) {
                    $("#" + startDateElement).data("DateTimePicker").maxDate(ev.date);
                }
                var newDate = moment(ev.date);
                var value = valueAccessor();
                value(newDate);
            });
        };
        // Called by Knockout each time the dependent observable value changes.
        datePickerBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var date = ko.unwrap(valueAccessor());
            if (date) {
                $(element).data("DateTimePicker").date(date);
            }
        };
        return datePickerBindingHandler;
    }());
    return datePickerBindingHandler;
});

//# sourceMappingURL=datePickerBindingHandler.js.map
