define(["require", "exports", "durandal/composition", "durandal/system", "models/filesystem/uploadItem", "commands/filesystem/uploadFileToFilesystemCommand", "common/uploadQueueHelper", "common/shell/activeResourceTracker"], function (require, exports, composition, system, uploadItem, uploadFileToFilesystemCommand, uploadQueueHelper, activeResourceTracker) {
    "use strict";
    // Usage: <input type="file" data-bind="fileUpload: { files: files, uploads: uploadQueue, success: uploadSuccess.bind($data), fail: uploadFailed.bind($data) }" />
    // files: KnockoutObservable<File[]>
    // uploads: KnockoutObservableArray<uploadItem>
    // success: (i: uploadItem) => void;
    // fail: (i: uploadItem) => void;
    var fileUploadBindingHandler = (function () {
        function fileUploadBindingHandler() {
        }
        fileUploadBindingHandler.install = function () {
            if (!ko.bindingHandlers["fileUpload"]) {
                ko.bindingHandlers["fileUpload"] = new fileUploadBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler("fileUpload");
            }
        };
        fileUploadBindingHandler.prototype.init = function (element, valueAccessor) {
        };
        fileUploadBindingHandler.prototype.update = function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
            var _this = this;
            var options = ko.utils.unwrapObservable(valueAccessor());
            var filesystem = ko.utils.unwrapObservable(bindingContext.$data["activeFilesystem"]);
            if (options) {
                options.files();
                // Access the files observable now so that .update is called whenever it changes.
                if (element.files.length) {
                    if (options.before) {
                        options.before();
                    }
                    var files = element.files;
                    for (var i = 0; i < files.length; i++) {
                        var file = files[i];
                        var guid = system.guid();
                        var directory = options.directory() ? options.directory() : "";
                        var item = new uploadItem(guid, directory + "/" + file.name, uploadQueueHelper.queuedStatus, activeResourceTracker.default.fileSystem());
                        options.uploads.push(item);
                        new uploadFileToFilesystemCommand(file, directory, guid, filesystem, function (e) { return _this.uploadProgressReported(e); }, true)
                            .execute()
                            .done(function (x) { return options.success(x); })
                            .fail(function (x) { return options.fail(x); });
                        item.status(uploadQueueHelper.uploadingStatus);
                        options.uploads(uploadQueueHelper.sortUploadQueue(options.uploads()));
                        options.uploads.notifySubscribers(options.uploads());
                    }
                }
                options.files(null);
            }
        };
        fileUploadBindingHandler.prototype.uploadProgressReported = function (e) {
            if (e.lengthComputable) {
                var percentComplete = e.loaded / e.total;
            }
        };
        return fileUploadBindingHandler;
    }());
    return fileUploadBindingHandler;
});

//# sourceMappingURL=fileUploadBindingHandler.js.map
