/// <reference path="../../../typings/tsd.d.ts" />
define(["require", "exports", "durandal/composition", "common/appUrl", "commands/filesystem/getFoldersCommand"], function (require, exports, composition, appUrl, getFoldersCommand) {
    "use strict";
    var treeBindingHandler = (function () {
        function treeBindingHandler() {
        }
        treeBindingHandler.install = function () {
            if (!ko.bindingHandlers["tree"]) {
                ko.bindingHandlers["tree"] = new treeBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler("tree");
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        treeBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var options = ko.utils.unwrapObservable(valueAccessor());
            var tree = $(element).dynatree({
                children: [{ title: appUrl.getFileSystemNameFromUrl(), key: "/", isLazy: true, isFolder: true }],
                onLazyRead: function (node) {
                    treeBindingHandler.loadNodeChildren("#" + element.id, node, options);
                    node.activate();
                },
                onExpand: function (expanded, node) {
                    if (expanded) {
                        treeBindingHandler.loadNodeChildren("#" + element.id, node, options);
                        node.activate();
                    }
                },
                selectMode: 1,
                onSelect: function (flag, node) {
                    treeBindingHandler.onActivateAndSelect(node, valueAccessor());
                },
                onActivate: function (node) {
                    treeBindingHandler.onActivateAndSelect(node, valueAccessor());
                },
            });
            var firstNode = $(element).dynatree("getRoot", []).getChildren()[0];
            firstNode.activate();
            firstNode.expand(null);
        };
        treeBindingHandler.loadNodeChildren = function (tree, node, options) {
            var _this = this;
            var dir;
            if (node && node.data && node.data.key != "/") {
                dir = node.data.key;
            }
            //TODO: new filesystem(appUrl.getFileSystemNameFromUrl(), true, false)
            var command = new getFoldersCommand(null, 0, 1024, dir); //TODO: pass fs using injection or remove fs specific code from this class
            command.execute().done(function (results) {
                node.setLazyNodeStatus(0);
                var versioningEnabled = _this.includeRevisionsFunc();
                if (!dir && versioningEnabled) {
                    results.unshift({ key: "/$$revisions$$", title: "$revisions", isLazy: false, isFolder: true });
                }
                var newSet = {};
                var differenceSet = {};
                //this is being done creating a new dictionary to reduce complexity
                for (var i = 0; i < results.length; i++) {
                    newSet[results[i].key] = results[i];
                }
                if (node.hasChildren()) {
                    //calculate deleted and transient nodes
                    for (var j = 0; j < node.getChildren().length; j++) {
                        if (!newSet[node.getChildren()[j].data.key]) {
                            differenceSet[node.getChildren()[j].data.key] = node.getChildren()[j];
                        }
                    }
                    var nodesToRemove = [];
                    //mark deleted nodes filtering transient
                    for (var k = 0; k < node.getChildren().length; k++) {
                        var nodeK = node.getChildren()[k];
                        if (!newSet[nodeK.data.key] && differenceSet[nodeK.data.key] && differenceSet[nodeK.data.key].data.addClass !== treeBindingHandler.transientNodeStyle) {
                            nodesToRemove.push(k);
                        }
                        else {
                            newSet[nodeK.data.key] = null;
                        }
                    }
                    //remove deleted nodes
                    for (var m = 0; m < nodesToRemove.length; m++) {
                        node.getChildren()[nodesToRemove[m]].remove();
                    }
                }
                //add new nodes
                for (var key in newSet) {
                    if (newSet[key]) {
                        node.addChild(newSet[key]);
                    }
                }
                if (options && node.hasChildren()) {
                    var keys = node.getChildren().map(function (x) { return x.data.key; });
                    (_a = options.currentLevelNodes).push.apply(_a, keys);
                }
                var _a;
            });
        };
        treeBindingHandler.reloadNode = function (tree, nodeKey) {
            var dynaTree = $(tree).dynatree("getTree");
            var node = dynaTree.getNodeByKey(nodeKey);
            treeBindingHandler.loadNodeChildren(tree, node, null);
        };
        treeBindingHandler.onActivateAndSelect = function (node, valueAccessor) {
            var options = ko.utils.unwrapObservable(valueAccessor);
            var selectedNode = node.data && node.data.key != "/" ? node.data.key : null;
            options.selectedNode(selectedNode);
            if (node.data) {
                var siblings = [];
                if (node.hasChildren()) {
                    siblings = node.getChildren();
                }
                var mappedNodes = siblings.map(function (x) { return x.data.title; });
                options.currentLevelNodes(mappedNodes);
            }
        };
        // Called by Knockout each time the dependent observable value changes.
        treeBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var options = ko.utils.unwrapObservable(valueAccessor());
            if (options.addedNode()) {
                var newNode = options.addedNode();
                var activeNode = $(element).dynatree("getActiveNode", []);
                var parentOfNewNode = newNode.key.substring(0, newNode.key.lastIndexOf("/"));
                if (parentOfNewNode === "") {
                    parentOfNewNode = "/";
                }
                var parentNode = $(element).dynatree("getTree").getNodeByKey(parentOfNewNode);
                if (parentNode) {
                    parentNode.addChild(newNode);
                    if (parentNode == activeNode) {
                        options.currentLevelNodes(activeNode.getChildren().map(function (x) { return x.data.title; }));
                    }
                    options.addedNode(null);
                }
            }
        };
        treeBindingHandler.updateNodeHierarchyStyle = function (tree, key, styleClass) {
            var theTree = $(tree).dynatree("getTree");
            var slashPosition = key.length;
            while (slashPosition > 0) {
                key = key.substring(0, slashPosition);
                var temporaryNode = theTree.getNodeByKey(key);
                if (temporaryNode && temporaryNode.data.addClass != styleClass) {
                    temporaryNode.data.addClass = styleClass;
                    temporaryNode.reloadChildren();
                }
                slashPosition = key.lastIndexOf("/");
            }
        };
        treeBindingHandler.setNodeLoadStatus = function (tree, key, status) {
            // Set node load status
            // -1 = error
            // 0 = load OK
            // 1 = loading
            var object = $(tree);
            if (!object || object.length <= 0)
                return;
            var theTree = object.dynatree("getTree");
            if (!theTree) {
                return;
            }
            var slashPosition = key.length;
            while (slashPosition > 0) {
                key = key.substring(0, slashPosition);
                var temporaryNode = theTree.getNodeByKey(key);
                if (temporaryNode) {
                    temporaryNode.setLazyNodeStatus(status);
                }
                slashPosition = key.lastIndexOf("/");
            }
        };
        treeBindingHandler.nodeExists = function (tree, key) {
            var dynaTree = $(tree).dynatree("getTree");
            var node = dynaTree.getNodeByKey(key);
            if (node) {
                return true;
            }
            return false;
        };
        treeBindingHandler.isNodeExpanded = function (tree, key) {
            var dynaTree = $(tree).dynatree("getTree");
            if (!dynaTree)
                return false;
            var node = dynaTree.getNodeByKey(key);
            return node && node.isExpanded();
        };
        return treeBindingHandler;
    }());
    treeBindingHandler.transientNodeStyle = "temp-folder";
    return treeBindingHandler;
});

//# sourceMappingURL=treeBindingHandler.js.map
