/// <reference path="../../typings/tsd.d.ts" />
define(["require", "exports", "common/changesApi", "common/appUrl", "plugins/router", "common/adminWatchClient", "common/constants/events"], function (require, exports, changesApi, appUrl, router, adminWatchClient, EVENTS) {
    "use strict";
    var changesContext = (function () {
        function changesContext() {
            var _this = this;
            this.currentResourceChangesApi = ko.observable();
            this.globalChangesApi = ko.observable();
            this.afterConnection = $.Deferred();
            this.afterConnectionResolved = false;
            window.addEventListener("beforeunload", function () {
                _this.disconnectFromResourceChangesApi("ChangingResource");
                _this.globalChangesApi().dispose();
            });
            this.currentResourceChangesApi.subscribe(function (newValue) {
                if (!newValue) {
                    if (_this.afterConnectionResolved) {
                        _this.afterConnection = $.Deferred();
                        _this.afterConnectionResolved = false;
                    }
                }
                else {
                    _this.afterConnectionResolved = true;
                    _this.afterConnection.resolve(newValue);
                }
            });
        }
        changesContext.prototype.connectGlobalChangesApi = function () {
            var alreadyHasGlobalChangesApi = this.globalChangesApi();
            if (alreadyHasGlobalChangesApi) {
                return alreadyHasGlobalChangesApi.connectToWebSocketTask;
            }
            var globalChanges = new adminWatchClient();
            this.globalChangesApi(globalChanges);
            return globalChanges.connectToWebSocketTask;
        };
        changesContext.prototype.updateChangesApi = function (rs, subscriptions) {
            var _this = this;
            var currentChanges = this.currentResourceChangesApi();
            if (currentChanges && currentChanges.getResource().qualifiedName === rs.qualifiedName) {
                // nothing to do - already connected to requested changes api
                return;
            }
            if (currentChanges) {
                this.disconnectFromResourceChangesApi("ChangingResource");
            }
            if (rs.disabled()) {
                this.navigateToResourceSpecificPage(rs);
                return;
            }
            var newChanges = new changesApi(rs);
            newChanges.connectToWebSocketTask.done(function () {
                _this.currentResourceChangesApi(newChanges);
                _this.sentSubscriptions = subscriptions(newChanges);
                _this.navigateToResourceSpecificPage(rs);
            });
        };
        changesContext.prototype.navigateToResourceSpecificPage = function (rs) {
            var locationHash = window.location.hash;
            var isMainPage = locationHash === appUrl.forResources();
            if (!isMainPage) {
                var updatedUrl = appUrl.forCurrentPage(rs);
                if (updatedUrl) {
                    router.navigate(updatedUrl);
                }
            }
        };
        changesContext.prototype.disconnectFromResourceChangesApi = function (cause) {
            var currentChanges = this.currentResourceChangesApi();
            if (currentChanges) {
                this.sentSubscriptions.forEach(function (x) { return x.off(); });
                this.sentSubscriptions = [];
                currentChanges.dispose();
                this.currentResourceChangesApi(null);
                ko.postbox.publish(EVENTS.Resource.Disconnect, { resource: currentChanges.getResource(), cause: cause });
            }
        };
        changesContext.prototype.disconnectIfCurrent = function (rs, cause) {
            var currentChanges = this.currentResourceChangesApi();
            if (currentChanges && currentChanges.getResource().qualifiedName === rs.qualifiedName) {
                this.disconnectFromResourceChangesApi(cause);
            }
        };
        return changesContext;
    }());
    changesContext.default = new changesContext();
    return changesContext;
});

//# sourceMappingURL=changesContext.js.map
