define(["require", "exports"], function (require, exports) {
    "use strict";
    var eventsCollector = (function () {
        function eventsCollector() {
            // used for caching events fired before analytics initialization
            // if user don't agree on usage stats tracking we discard this data
            this.preInitializationQueue = [];
            this.enabled = false;
            this.initialized = false;
        }
        eventsCollector.prototype.initialize = function (version, build, env, fullVersion, enabled) {
            this.version = version;
            this.build = build;
            this.env = env;
            this.fullVersion = fullVersion;
            this.enabled = enabled && eventsCollector.gaDefined();
            this.createTracker();
            this.initialized = true;
            this.processQueue();
        };
        eventsCollector.gaDefined = function () {
            return typeof (ga) !== 'undefined';
        };
        eventsCollector.prototype.createTracker = function () {
            if (eventsCollector.gaDefined()) {
                ga('create', eventsCollector.UACode, 'auto');
                ga('set', 'dimension1', this.version);
                ga('set', 'dimension2', this.build);
                ga('set', 'dimension3', this.env);
                ga('set', 'dimension4', this.fullVersion);
            }
        };
        eventsCollector.prototype.processQueue = function () {
            if (this.enabled) {
                this.preInitializationQueue.forEach(function (action) {
                    action(ga);
                });
            }
            this.preInitializationQueue = [];
        };
        eventsCollector.prototype.reportViewModel = function (view) {
            this.internalLog(function (ga) {
                var viewName = view.__moduleId__;
                ga('set', 'location', "http://raven.studio/" + viewName + document.location.search);
                ga('send', 'pageview');
            });
        };
        eventsCollector.prototype.reportEvent = function (category, action, label) {
            if (label === void 0) { label = null; }
            this.internalLog(function (ga) {
                ga('send', 'event', category, action, label);
            });
        };
        eventsCollector.prototype.internalLog = function (action) {
            if (!this.initialized) {
                this.preInitializationQueue.push(action);
                return;
            }
            if (!this.enabled) {
                return;
            }
            action(ga);
        };
        return eventsCollector;
    }());
    eventsCollector.UACode = "UA-82335022-3"; //TODO @gregolsky liveTestDetector.isLiveTest() ? "UA-82335022-2" : "UA-82335022-1";
    eventsCollector.default = new eventsCollector();
    return eventsCollector;
});

//# sourceMappingURL=eventsCollector.js.map
