/// <reference path="../../../../typings/tsd.d.ts" />
define(["require", "exports", "models/database/documents/document"], function (require, exports, document) {
    "use strict";
    var documentHelpers = (function () {
        function documentHelpers() {
        }
        documentHelpers.findRelatedDocumentsCandidates = function (doc) {
            var results = [];
            var initialDocumentFields = doc.getDocumentPropertyNames();
            var documentNodesFlattenedList = [];
            // get initial nodes list to work with
            initialDocumentFields.forEach(function (curField) {
                documentNodesFlattenedList.push(doc[curField]);
            });
            for (var documentNodesCursor = 0; documentNodesCursor < documentNodesFlattenedList.length; documentNodesCursor++) {
                var curField = documentNodesFlattenedList[documentNodesCursor];
                if (typeof curField === "string" && /\w+\/\w+/ig.test(curField)) {
                    if (!results.find(function (x) { return x === curField.toString(); })) {
                        results.push(curField.toString());
                    }
                }
                else if (typeof curField == "object" && !!curField) {
                    for (var curInnerField in curField) {
                        documentNodesFlattenedList.push(curField[curInnerField]);
                    }
                }
            }
            return results;
        };
        documentHelpers.unescapeNewlinesAndTabsInTextFields = function (str) {
            var AceDocumentClass = ace.require("ace/document").Document;
            var AceEditSessionClass = ace.require("ace/edit_session").EditSession;
            var AceJSONMode = ace.require("ace/mode/json").Mode;
            var documentTextAceDocument = new AceDocumentClass(str);
            var jsonMode = new AceJSONMode();
            var documentTextAceEditSession = new AceEditSessionClass(documentTextAceDocument, jsonMode);
            var TokenIterator = ace.require("ace/token_iterator").TokenIterator;
            var iterator = new TokenIterator(documentTextAceEditSession, 0, 0);
            var curToken = iterator.getCurrentToken();
            // first, calculate newline indexes
            var rowsIndexes = str.split("").map(function (x, index) { return ({ char: x, index: index }); }).filter(function (x) { return x.char === "\n"; }).map(function (x) { return x.index; });
            // start iteration from the end of the document
            while (curToken) {
                curToken = iterator.stepForward();
            }
            curToken = iterator.stepBackward();
            var lastTextSectionPosEnd = null;
            while (curToken) {
                if (curToken.type === "string" || curToken.type == "constant.language.escape") {
                    if (lastTextSectionPosEnd == null) {
                        curToken = iterator.stepForward();
                        lastTextSectionPosEnd = { row: iterator.getCurrentTokenRow(), column: iterator.getCurrentTokenColumn() + 1 };
                        curToken = iterator.stepBackward();
                    }
                }
                else {
                    if (lastTextSectionPosEnd != null) {
                        curToken = iterator.stepForward();
                        var lastTextSectionPosStart = { row: iterator.getCurrentTokenRow(), column: iterator.getCurrentTokenColumn() + 1 };
                        var stringTokenStartIndexInSourceText = (lastTextSectionPosStart.row > 0 ? rowsIndexes[lastTextSectionPosStart.row - 1] : 0) + lastTextSectionPosStart.column;
                        var stringTokenEndIndexInSourceText = (lastTextSectionPosEnd.row > 0 ? rowsIndexes[lastTextSectionPosEnd.row - 1] : 0) + lastTextSectionPosEnd.column;
                        var newTextPrefix = str.substring(0, stringTokenStartIndexInSourceText);
                        var newTextSuffix = str.substring(stringTokenEndIndexInSourceText, str.length);
                        var newStringTokenValue = str.substring(stringTokenStartIndexInSourceText, stringTokenEndIndexInSourceText)
                            .replace(/(\\\\n|\\\\r\\\\n|\\n|\\r\\n|\\t|\\\\t)/g, function (x) {
                            if (x == "\\\\n" || x === "\\\\r\\\\n") {
                                return "\\r\\n";
                            }
                            else if (x === "\\n" || x === "\\r\\n") {
                                return "\r\n";
                            }
                            else if (x === "\\t") {
                                return "\t";
                            }
                            else if (x === "\\\\t") {
                                return "\\t";
                            }
                            else {
                                return "\r\n";
                            }
                        });
                        str = newTextPrefix + newStringTokenValue + newTextSuffix;
                        curToken = iterator.stepBackward();
                    }
                    lastTextSectionPosEnd = null;
                }
                curToken = iterator.stepBackward();
            }
            return str;
        };
        documentHelpers.escapeNewlinesAndTabsInTextFields = function (str) {
            var AceDocumentClass = ace.require("ace/document").Document;
            var AceEditSessionClass = ace.require("ace/edit_session").EditSession;
            var AceJSONMode = ace.require("ace/mode/json_newline_friendly").Mode;
            var documentTextAceDocument = new AceDocumentClass(str);
            var jsonMode = new AceJSONMode();
            var documentTextAceEditSession = new AceEditSessionClass(documentTextAceDocument, jsonMode);
            var previousLine = 0;
            var TokenIterator = ace.require("ace/token_iterator").TokenIterator;
            var iterator = new TokenIterator(documentTextAceEditSession, 0, 0);
            var curToken = iterator.getCurrentToken();
            var text = "";
            while (curToken) {
                if (iterator.$row - previousLine > 1) {
                    var rowsGap = iterator.$row - previousLine;
                    for (var i = 0; i < rowsGap - 1; i++) {
                        text += "\\r\\n";
                    }
                }
                if (curToken.type === "string" || curToken.type === "constant.language.escape") {
                    if (previousLine < iterator.$row) {
                        text += "\\r\\n";
                    }
                    var newTokenValue = curToken.value
                        .replace(/(\r\n)/g, '\\r\\n')
                        .replace(/(\n)/g, '\\n')
                        .replace(/(\t)/g, '\\t');
                    text += newTokenValue;
                }
                else {
                    text += curToken.value;
                }
                previousLine = iterator.$row;
                curToken = iterator.stepForward();
            }
            return text;
        };
        documentHelpers.findSchema = function (documents) {
            try {
                documents.forEach(function (doc) {
                    JSON.stringify(doc);
                });
            }
            catch (e) {
                throw new Error("Cannot find schema for not serializable objects");
            }
            var docDto = documentHelpers.findSchemaForObject(documents.map(function (x) { return x.toDto(false); }));
            var metadatas = documents.map(function (x) { return x.__metadata; });
            var ravenEntityName = documentHelpers.findCommonValue(metadatas, "ravenEntityName");
            var ravenClrType = documentHelpers.findCommonValue(metadatas, "ravenClrType");
            docDto["@metadata"] = {
                "Raven-Entity-Name": ravenEntityName,
                "Raven-Clr-Type": ravenClrType
            };
            return new document(docDto);
        };
        documentHelpers.findSchemaForObject = function (objects) {
            var result = {};
            var firstDocument = objects[0];
            for (var prop in firstDocument) {
                if (firstDocument.hasOwnProperty(prop)) {
                    var defaultValue = documentHelpers.findSchemaDefaultValue(objects, prop);
                    if (typeof (defaultValue) !== "undefined") {
                        result[prop] = defaultValue;
                    }
                }
            }
            return result;
        };
        documentHelpers.findCommonValue = function (elements, property) {
            var extractedValues = elements.map(function (doc) { return doc[property]; });
            var firstValue = extractedValues[0], restValues = extractedValues.slice(1);
            for (var i = 0; i < restValues.length; i++) {
                if (firstValue !== restValues[i]) {
                    return undefined;
                }
            }
            return firstValue;
        };
        documentHelpers.findSchemaDefaultValue = function (documents, property) {
            for (var i = 0; i < documents.length; i++) {
                if (!(property in documents[i])) {
                    return undefined;
                }
            }
            var extractedValues = documents.map(function (doc) { return doc[property]; });
            var extractedTypes = extractedValues.map(function (v) { return typeof (v); });
            var firstType = extractedTypes[0], restTypes = extractedTypes.slice(1);
            for (var i = 0; i < restTypes.length; i++) {
                if (firstType !== restTypes[i]) {
                    firstType = "undefined";
                    break;
                }
            }
            switch (firstType) {
                case "undefined":
                    return undefined;
                case "number":
                    return 0;
                case "boolean":
                    return false;
                case "string":
                    return "";
                case "function":
                    return undefined;
                case "object":
                    return documentHelpers.findSchemaForObject(extractedValues);
                default:
                    throw new Error("Unhandled type: " + firstType);
            }
        };
        return documentHelpers;
    }());
    return documentHelpers;
});

//# sourceMappingURL=documentHelpers.js.map
