/// <reference path="../../../../typings/tsd.d.ts" />
define(["require", "exports"], function (require, exports) {
    "use strict";
    var graphHelper = (function () {
        function graphHelper() {
        }
        graphHelper.truncText = function (input, measuredWidth, availableWidth, minWidth) {
            if (minWidth === void 0) { minWidth = 5; }
            if (availableWidth >= measuredWidth) {
                return input;
            }
            if (availableWidth < minWidth) {
                return null;
            }
            var approxCharactersToTake = Math.floor(availableWidth * input.length / measuredWidth);
            return input.substr(0, approxCharactersToTake);
        };
        graphHelper.drawArrow = function (ctx, x, y, rightArrow) {
            ctx.beginPath();
            if (rightArrow) {
                ctx.moveTo(x, y);
                ctx.lineTo(x + 7, y + 4);
                ctx.lineTo(x, y + 8);
            }
            else {
                ctx.moveTo(x, y + 1);
                ctx.lineTo(x + 4, y + 8);
                ctx.lineTo(x + 8, y + 1);
            }
            ctx.fill();
        };
        graphHelper.timeRangeFromSortedRanges = function (input) {
            if (input.length === 0) {
                return null;
            }
            var minDate = input[0][0];
            var maxDate = input[input.length - 1][1];
            return [minDate, maxDate];
        };
        /**
         * Divide elements
         * Ex. For Total width = 100, eleemntWidth = 20, elements = 2
         * We have:
         * | 20px padding | 20px element | 20px padding | 20px element | 20px padding |
         * So elements width stays the same and padding is divided equally,
         * We return start X (which in 20 px in this case)
         * and offset - as width between objects start (40px)
         */
        graphHelper.computeStartAndOffset = function (totalWidth, elements, elementWidth) {
            var offset = (totalWidth - elementWidth * elements) / (elements + 1) + elementWidth;
            var start = offset - elementWidth;
            return {
                offset: offset,
                start: start
            };
        };
        graphHelper.drawBezierDiagonal = function (ctx, source, target, withArrow) {
            if (withArrow === void 0) { withArrow = false; }
            ctx.beginPath();
            var m = (source[1] + target[1]) / 2;
            if (source[1] < target[1]) {
                ctx.moveTo(source[0], source[1]);
                ctx.lineTo(source[0], source[1] + graphHelper.arrowConfig.straightLine);
                ctx.bezierCurveTo(source[0], m, target[0], m, target[0], target[1] - graphHelper.arrowConfig.straightLine);
                ctx.lineTo(target[0], target[1]);
                ctx.stroke();
            }
            else {
                ctx.moveTo(source[0], source[1]);
                ctx.lineTo(source[0], source[1] - graphHelper.arrowConfig.straightLine);
                ctx.bezierCurveTo(source[0], m, target[0], m, target[0], target[1] + graphHelper.arrowConfig.straightLine);
                ctx.lineTo(target[0], target[1]);
                ctx.stroke();
            }
            if (withArrow) {
                ctx.beginPath();
                ctx.moveTo(target[0] - graphHelper.arrowConfig.halfWidth, target[1] + graphHelper.arrowConfig.height);
                ctx.lineTo(target[0], target[1]);
                ctx.lineTo(target[0] + graphHelper.arrowConfig.halfWidth, target[1] + graphHelper.arrowConfig.height);
                ctx.stroke();
            }
        };
        return graphHelper;
    }());
    graphHelper.arrowConfig = {
        halfWidth: 6,
        height: 8,
        straightLine: 7
    };
    return graphHelper;
});

//# sourceMappingURL=graphHelper.js.map
