/// <reference path="../../../../typings/tsd.d.ts" />
define(["require", "exports"], function (require, exports) {
    "use strict";
    var rangeAggregator = (function () {
        function rangeAggregator(inputItems) {
            this.items = [];
            this.indexesWork = [];
            this.maxConcurrentIndexes = 0;
            // Subtract 1 from endTime if we can
            // Because the endTime of one work batch can be reported by the server
            // as exactly the same time as the startTime of ANOTHER work batch for the SAME index
            this.items = inputItems.map(function (x) { return ((x[0].getTime() === x[1].getTime()) ?
                { startTime: x[0].getTime(), endTime: x[1].getTime() } :
                { startTime: x[0].getTime(), endTime: x[1].getTime() - 1 }); });
        }
        rangeAggregator.prototype.inputItems = function (inputItems) {
            this.items = inputItems;
        };
        rangeAggregator.prototype.aggregate = function () {
            // 1. Sort the times array by startTime & endTime value
            this.items.sort(function (a, b) { return a.startTime === b.startTime ? a.endTime - b.endTime : a.startTime - b.startTime; });
            // 2. Create the indexesWork Array
            if (this.items.length !== 0) {
                // 3. Push first item
                this.indexesWork.push({ pointInTime: this.items[0].startTime, numberOfIndexesWorking: 1 });
                this.indexesWork.push({ pointInTime: this.items[0].endTime + 1, numberOfIndexesWorking: 0 });
                this.maxConcurrentIndexes = 1;
                // 4. Push all other items 
                for (var i = 1; i < this.items.length; i++) {
                    this.pushRange(this.items[i].startTime, this.items[i].endTime);
                }
            }
            return this.indexesWork;
        };
        rangeAggregator.prototype.pushRange = function (startTime, endTime) {
            // 1. Find appropriate start position for the new element
            var i = 0;
            while ((i < this.indexesWork.length) && (startTime > this.indexesWork[i].pointInTime)) {
                i++;
            }
            this.ptrStart = i;
            var previousValue = (this.ptrStart === 0) ? this.indexesWork[0].numberOfIndexesWorking : this.indexesWork[this.ptrStart - 1].numberOfIndexesWorking;
            if (this.ptrStart === this.indexesWork.length) {
                // 1.1 Push new item in the end
                var newValue = previousValue + 1;
                this.indexesWork.push({ pointInTime: startTime, numberOfIndexesWorking: newValue });
                if (newValue > this.maxConcurrentIndexes) {
                    this.maxConcurrentIndexes++;
                }
                ;
            }
            else if (this.indexesWork[this.ptrStart].pointInTime === startTime) {
                // 1.2 Only increase counter
                var newValue = ++(this.indexesWork[this.ptrStart].numberOfIndexesWorking);
                if (newValue > this.maxConcurrentIndexes) {
                    this.maxConcurrentIndexes++;
                }
                ;
            }
            else if (this.indexesWork[this.ptrStart].pointInTime !== startTime) {
                // 1.3 Create new element in the start/middle 
                var newValue = previousValue + 1;
                this.indexesWork.splice(this.ptrStart, 0, { pointInTime: startTime, numberOfIndexesWorking: newValue });
                if (newValue > this.maxConcurrentIndexes) {
                    this.maxConcurrentIndexes++;
                }
                ;
            }
            // 2. Find appropriate end position for the new element AND update working indexes counter along the way...
            i = this.ptrStart;
            while ((i < this.indexesWork.length) && (endTime + 1 > this.indexesWork[i].pointInTime)) {
                if ((i + 1 < this.indexesWork.length) && (endTime + 1 > this.indexesWork[i + 1].pointInTime)) {
                    var newValue = ++(this.indexesWork[i + 1].numberOfIndexesWorking);
                    if (newValue > this.maxConcurrentIndexes) {
                        this.maxConcurrentIndexes++;
                    }
                    ;
                }
                i++;
            }
            this.ptrEnd = i;
            previousValue = (this.ptrEnd === 0) ? this.indexesWork[0].numberOfIndexesWorking : this.indexesWork[this.ptrEnd - 1].numberOfIndexesWorking - 1;
            if (this.ptrEnd === this.indexesWork.length) {
                // 2.1 Push new item in the end
                this.indexesWork.push({ pointInTime: endTime + 1, numberOfIndexesWorking: 0 });
            }
            else if (this.indexesWork[this.ptrEnd].pointInTime !== endTime + 1) {
                // 2.2 Create new element in the middle
                this.indexesWork.splice(this.ptrEnd, 0, { pointInTime: endTime + 1, numberOfIndexesWorking: previousValue });
            }
        };
        return rangeAggregator;
    }());
    return rangeAggregator;
});

//# sourceMappingURL=rangeAggregator.js.map
