define(["require", "exports", "common/alertArgs", "common/constants/events", "toastr"], function (require, exports, alertArgs, EVENTS, toastr) {
    "use strict";
    var messagePublisher = (function () {
        function messagePublisher() {
        }
        messagePublisher.initialize = function () {
            if (!messagePublisher.initialized) {
                messagePublisher.initialized = true;
                toastr.options.progressBar = true;
                toastr.options.positionClass = "toast-bottom-right";
            }
        };
        messagePublisher.reportInfo = function (title, details) {
            this.reportProgress("Info", title, details);
        };
        messagePublisher.reportError = function (title, details, httpStatusText, displayInRecentErrors) {
            if (displayInRecentErrors === void 0) { displayInRecentErrors = true; }
            this.reportProgress("Error", title, details, httpStatusText, displayInRecentErrors);
            console.error("Error during command execution", title, details, httpStatusText);
        };
        messagePublisher.reportSuccess = function (title, details) {
            this.reportProgress("Success", title, details);
        };
        messagePublisher.reportWarning = function (title, details, httpStatusText) {
            this.reportProgress("Warning", title, details, httpStatusText);
        };
        messagePublisher.reportProgress = function (type, title, details, httpStatusText, displayInRecentErrors) {
            if (displayInRecentErrors === void 0) { displayInRecentErrors = false; }
            var alert = new alertArgs(type, title, details, httpStatusText);
            var toastrMethod = messagePublisher.getDisplayMethod(alert.type);
            toastrMethod(alert.errorMessage, alert.title, {
                showDuration: messagePublisher.getDisplayDuration(alert.type),
                closeButton: true
            });
            if (displayInRecentErrors) {
                ko.postbox.publish(EVENTS.NotificationCenter.RecentError, alert);
            }
        };
        messagePublisher.getDisplayDuration = function (type) {
            return (type === "Error" || type === "Warning") ? 5000 : 2000;
        };
        messagePublisher.getDisplayMethod = function (type) {
            switch (type) {
                case "Success":
                    return toastr.success;
                case "Warning":
                    return toastr.warning;
                case "Error":
                    return toastr.error;
                case "Info":
                    return toastr.info;
                default:
                    throw new Error("Unhandled alert type = " + type);
            }
        };
        return messagePublisher;
    }());
    messagePublisher.initialized = false;
    messagePublisher.initialize();
    return messagePublisher;
});

//# sourceMappingURL=messagePublisher.js.map
