define(["require", "exports", "common/notifications/notificationCenterOperations", "common/notifications/notificationCenterRecentErrors", "common/notifications/notificationCenterAlerts"], function (require, exports, notificationCenterOperations, notificationCenterRecentErrors, notificationCenterAlerts) {
    "use strict";
    var notificationCenter = (function () {
        function notificationCenter() {
            var _this = this;
            this.showNotifications = ko.observable(false);
            this.operations = new notificationCenterOperations();
            this.recentErrors = new notificationCenterRecentErrors();
            this.alerts = new notificationCenterAlerts();
            this.alertCountAnimation = ko.observable();
            this.hideHandler = function (e) {
                if (_this.shouldConsumeHideEvent(e)) {
                    _this.showNotifications(false);
                }
            };
            this.initializeObservables();
        }
        notificationCenter.prototype.initializeObservables = function () {
            var _this = this;
            this.totalItemsCount = ko.pureComputed(function () {
                var ops = _this.operations.watchedOperations().length;
                var errors = _this.recentErrors.recordedErrors().length;
                var alerts = _this.alerts.alerts().length;
                return ops + errors + alerts;
            });
            this.totalItemsCount.subscribe(function (count) {
                if (count) {
                    _this.alertCountAnimation(false);
                    setTimeout(function () { return _this.alertCountAnimation(true); });
                }
                else {
                    _this.alertCountAnimation(false);
                }
            });
            this.noNewNotifications = ko.pureComputed(function () {
                return _this.totalItemsCount() === 0;
            });
            this.showNotifications.subscribe(function (show) {
                if (show) {
                    window.addEventListener("click", _this.hideHandler, true);
                }
                else {
                    window.removeEventListener("click", _this.hideHandler, true);
                }
            });
        };
        notificationCenter.prototype.monitorOperation = function (rs, operationId, onProgress) {
            if (onProgress === void 0) { onProgress = null; }
            return this.operations.monitorOperation(rs, operationId, onProgress);
        };
        notificationCenter.prototype.killOperation = function (operationId) {
            this.operations.killOperation(operationId);
        };
        notificationCenter.prototype.dismissOperation = function (operationId, saveOperations) {
            if (saveOperations === void 0) { saveOperations = true; }
            this.operations.dismissOperation(operationId, saveOperations);
        };
        notificationCenter.prototype.dismissRecentError = function (alert) {
            this.recentErrors.dismissRecentError(alert);
        };
        notificationCenter.prototype.showRecentErrorDialog = function (alert) {
            this.recentErrors.showRecentErrorDialog(alert);
        };
        notificationCenter.prototype.shouldConsumeHideEvent = function (e) {
            return $(e.target).closest(".notification-center-container").length === 0
                && $(e.target).closest("#notification-toggle").length === 0;
        };
        return notificationCenter;
    }());
    notificationCenter.instance = new notificationCenter();
    return notificationCenter;
});

//# sourceMappingURL=notificationCenter.js.map
