define(["require", "exports", "common/constants/events", "common/messagePublisher", "common/changesContext", "commands/alerts/getDatabaseAlertsCommand", "commands/alerts/getGlobalAlertsCommand"], function (require, exports, EVENTS, messagePublisher, changesContext, getDatabaseAlertsCommand, getGlobalAlertsCommand) {
    "use strict";
    var notificationCenterAlerts = (function () {
        function notificationCenterAlerts() {
            var _this = this;
            this.alerts = ko.observableArray();
            this.activeResourceChangesApi = changesContext.default.currentResourceChangesApi;
            this.onNewGlobalAlert();
            var globalAlertsSubscription = changesContext.default.globalChangesApi.subscribe(function (api) {
                api.watchAlerts(function (globalAlertEvent) {
                    _this.onNewGlobalAlert(globalAlertEvent);
                });
                globalAlertsSubscription.dispose();
            });
            ko.postbox.subscribe(EVENTS.ChangesApi.Reconnected, function () { return _this.onReconnectWatchAndShowActiveResourceAlerts(); });
            $(window).bind("storage", function (event) { return _this.onStorageEvent(event); });
        }
        notificationCenterAlerts.prototype.onStorageEvent = function (event) {
            var storageEvent = event.originalEvent;
            var rs = this.activeResourceChangesApi().getResource();
            //TODO: match key
            //TODO: handle action
        };
        notificationCenterAlerts.prototype.onReconnectWatchAndShowActiveResourceAlerts = function () {
            var _this = this;
            var changes = this.activeResourceChangesApi();
            var rs = changes.getResource();
            changes.watchAlerts(function (newAlertEvent) {
                _this.retrieveDatabaseAlerts(rs);
            });
        };
        notificationCenterAlerts.prototype.onNewGlobalAlert = function (msg) {
            var _this = this;
            new getGlobalAlertsCommand()
                .execute()
                .done(function (alerts) { return _this.consolidateAlerts(alerts); })
                .fail(function (response) {
                return messagePublisher.reportError("Failed to get alerts", response.responseText, response.statusText);
            });
        };
        notificationCenterAlerts.prototype.retrieveDatabaseAlerts = function (resource) {
            var _this = this;
            new getDatabaseAlertsCommand(resource)
                .execute()
                .done(function (alerts) { return _this.consolidateAlerts(alerts); })
                .fail(function (response) {
                return messagePublisher.reportError("Failed to get alerts", response.responseText, response.statusText);
            });
        };
        notificationCenterAlerts.prototype.consolidateAlerts = function (alerts) {
            var currentAlerts = this.alerts();
            var currentAlertKeys = new Set(currentAlerts.map(function (x) { return x.key; }));
            var alertsToUpdateDict = alerts.filter(function (a) { return currentAlertKeys.has(a.key); })
                .reduce(function (result, item) {
                result[item.key] = item;
                return result;
            }, {});
            currentAlerts.forEach(function (alert) {
                var update = alertsToUpdateDict[alert.key];
                if (update) {
                    alert.severity(update.severity());
                    alert.message(update.message());
                }
            });
            var newAlerts = alerts.filter(function (a) { return !currentAlertKeys.has(a.key); });
            (_a = this.alerts).push.apply(_a, newAlerts);
            var _a;
        };
        return notificationCenterAlerts;
    }());
    return notificationCenterAlerts;
});

//# sourceMappingURL=notificationCenterAlerts.js.map
