define(["require", "exports"], function (require, exports) {
    "use strict";
    var notificationCenterPersistanceStorage = (function () {
        function notificationCenterPersistanceStorage(serverStartTimeProvider) {
            this.serverStartTimeProvider = serverStartTimeProvider;
        }
        notificationCenterPersistanceStorage.prototype.loadOperations = function (rs) {
            var key = this.storageKey(rs);
            var operations = localStorage.getObject(key);
            if (!operations) {
                return [];
            }
            var startTime = this.serverStartTimeProvider();
            if (operations.ServerStartTime === startTime) {
                return operations.Operations;
            }
            else {
                this.removeSavedOperations(rs);
                return [];
            }
        };
        notificationCenterPersistanceStorage.prototype.saveOperations = function (rs, operations) {
            var startTime = this.serverStartTimeProvider();
            var dto = {
                Operations: operations,
                ServerStartTime: startTime
            };
            var key = this.storageKey(rs);
            if (operations.length === 0) {
                localStorage.removeItem(key);
            }
            else {
                localStorage.setObject(key, dto);
            }
        };
        //TODO: call when resource is deleted
        notificationCenterPersistanceStorage.prototype.removeSavedOperations = function (rs) {
            var key = this.storageKey(rs);
            localStorage.removeItem(key);
        };
        notificationCenterPersistanceStorage.prototype.isStorageKeyMatch = function (rs, storageEvent) {
            return this.storageKey(rs) === storageEvent.key;
        };
        notificationCenterPersistanceStorage.prototype.storageKey = function (rs) {
            return rs.fullTypeName + "_" + rs.name + "_operations";
        };
        return notificationCenterPersistanceStorage;
    }());
    return notificationCenterPersistanceStorage;
});

//# sourceMappingURL=notificationCenterPersistanceStorage.js.map
