define(["require", "exports"], function (require, exports) {
    "use strict";
    var requestExecution = (function () {
        function requestExecution(timeForSpinner, timeToAlert, sync) {
            if (timeToAlert === void 0) { timeToAlert = 0; }
            this.timeForSpinner = timeForSpinner;
            this.timeToAlert = timeToAlert;
            this.sync = sync;
            this.spinnerVisible = false;
            this.alertVisible = false;
            this.completed = false;
            this.setTimeouts();
        }
        requestExecution.prototype.markCompleted = function () {
            this.cleanState();
            this.sync();
            this.completed = true;
        };
        requestExecution.prototype.markProgress = function () {
            this.cleanState();
            this.setTimeouts();
            this.sync();
        };
        requestExecution.prototype.cleanState = function () {
            clearTimeout(this.spinnerTimeout);
            if (this.alertTimeout) {
                clearTimeout(this.alertTimeout);
            }
            this.spinnerVisible = false;
            this.alertVisible = false;
        };
        requestExecution.prototype.setTimeouts = function () {
            var _this = this;
            this.spinnerTimeout = setTimeout(function () {
                _this.spinnerVisible = true;
                _this.sync();
            }, this.timeForSpinner);
            if (this.timeToAlert > 0) {
                this.alertTimeout = setTimeout(function () {
                    _this.alertVisible = true;
                    _this.sync();
                }, this.timeToAlert);
            }
        };
        return requestExecution;
    }());
    var protractedCommandsDetector = (function () {
        function protractedCommandsDetector() {
            this.requestsInProgress = [];
            this.showSpinner = ko.observable(false);
            this.showServerNotResponding = ko.observable(false);
            this.showSpinner.subscribe(function (show) { return $("body").toggleClass("processing", show); });
        }
        protractedCommandsDetector.prototype.requestStarted = function (timeForSpinner, timeForAlert) {
            var _this = this;
            if (timeForAlert === void 0) { timeForAlert = 0; }
            var execution = new requestExecution(timeForSpinner, timeForAlert, function () { return _this.sync(); });
            this.requestsInProgress.push(execution);
            return execution;
        };
        protractedCommandsDetector.prototype.sync = function () {
            this.showSpinner(_.some(this.requestsInProgress, function (x) { return x.spinnerVisible; }));
            //TODO: handle block UI ?
            _.remove(this.requestsInProgress, function (x) { return x.completed; });
        };
        protractedCommandsDetector.prototype.showServerNotRespondingAlert = function () {
            this.showServerNotResponding(true);
            //TODO: update with new style
            $.blockUI({ message: '<div id="longTimeoutMessage"><span> This is taking longer than usual</span><br/><span>(Waiting for server to respond)</span></div>' });
        };
        return protractedCommandsDetector;
    }());
    protractedCommandsDetector.instance = new protractedCommandsDetector();
    return protractedCommandsDetector;
});

//# sourceMappingURL=protractedCommandsDetector.js.map
