define(["require", "exports"], function (require, exports) {
    "use strict";
    var watchedOperation = (function () {
        function watchedOperation(operationId, disposeHandle) {
            this.state = ko.observable();
            this.description = ko.observable();
            this.killable = ko.observable(false);
            this.operationId = operationId;
            this.disposeHandle = disposeHandle;
            this.initializeObservables();
        }
        watchedOperation.prototype.initializeObservables = function () {
            var _this = this;
            this.visible = ko.pureComputed(function () {
                var description = _this.description();
                var state = _this.state();
                return !!description && !!state;
            });
            this.completed = ko.pureComputed(function () {
                var state = _this.state();
                if (state) {
                    return state.Status !== "InProgress";
                }
                return false;
            });
            this.isSuccess = this.statusComparer("Completed");
            this.isCancelled = this.statusComparer("Canceled");
            this.isFailure = this.statusComparer("Faulted");
        };
        watchedOperation.prototype.statusComparer = function (desiredStatus) {
            var _this = this;
            return ko.pureComputed(function () {
                var state = _this.state();
                if (state) {
                    return state.Status === desiredStatus;
                }
                return false;
            });
        };
        watchedOperation.prototype.isPercentageProgress = function () {
            var state = this.state();
            if (state && state.Progress) {
                return state.Progress.hasOwnProperty("Processed") && state.Progress.hasOwnProperty("Total");
            }
            return false;
        };
        watchedOperation.prototype.percentageProgress = function () {
            var progress = this.state().Progress;
            return Math.round(progress.Processed * 100.0 / progress.Total);
        };
        watchedOperation.prototype.onInfoFetched = function (operation) {
            this.description(operation.Description);
            this.killable(operation.Killable);
            this.state(operation.State);
        };
        watchedOperation.prototype.onStatus = function (state) {
            this.state(state);
            if (state.Status !== "InProgress") {
                this.disposeHandle.off();
            }
        };
        return watchedOperation;
    }());
    return watchedOperation;
});

//# sourceMappingURL=watchedOperation.js.map
