define(["require", "exports", "common/constants/events", "models/resources/database", "models/filesystem/filesystem", "models/counter/counterStorage", "models/timeSeries/timeSeries", "plugins/router", "common/messagePublisher"], function (require, exports, EVENTS, database, filesystem, counterStorage, timeSeries, router, messagePublisher) {
    "use strict";
    var activeResourceTracker = (function () {
        function activeResourceTracker() {
            var _this = this;
            this.resource = ko.observable();
            this.database = ko.computed(function () {
                return _this.tryCastActiveResource(database.qualifier);
            });
            this.fileSystem = ko.computed(function () {
                return _this.tryCastActiveResource(filesystem.qualifier);
            });
            this.counterStorage = ko.computed(function () {
                return _this.tryCastActiveResource(counterStorage.qualifier);
            });
            this.timeSeries = ko.computed(function () {
                return _this.tryCastActiveResource(timeSeries.qualifier);
            });
            ko.postbox.subscribe(EVENTS.Resource.Activate, function (e) {
                // If the 'same' resource was selected from the top resources selector dropdown, 
                // then we want the knockout observable to be aware of it so that scrollling on page will occur
                if (e.resource === _this.resource()) {
                    _this.resource(null);
                }
                // Set the active resource
                _this.resource(e.resource);
            });
            ko.postbox.subscribe(EVENTS.Resource.Disconnect, function (e) {
                if (e.resource === _this.resource()) {
                    _this.resource(null);
                }
                // display warning to user if another user deleted or disabled active resource
                // but don't do this on resources page
                if (!_this.onResourcesPage()) {
                    if (e.cause === "ResourceDeleted") {
                        messagePublisher.reportWarning(e.resource.fullTypeName + " " + e.resource.name + " was deleted");
                        router.navigate("#resources");
                    }
                    else if (e.cause === "ResourceDisabled") {
                        messagePublisher.reportWarning(e.resource.fullTypeName + " " + e.resource.name + " was disabled");
                        router.navigate("#resources");
                    }
                }
            });
        }
        activeResourceTracker.prototype.onResourcesPage = function () {
            var instruction = router.activeInstruction();
            if (!instruction) {
                return false;
            }
            return instruction.fragment === "resources";
        };
        activeResourceTracker.prototype.tryCastActiveResource = function (expectedQualified) {
            var resource = this.resource();
            if (resource && resource.qualifier === expectedQualified) {
                return resource;
            }
            return null;
        };
        return activeResourceTracker;
    }());
    activeResourceTracker.default = new activeResourceTracker();
    return activeResourceTracker;
});

//# sourceMappingURL=activeResourceTracker.js.map
