define(["require", "exports", "common/appUrl", "common/shell/menu/separatorMenuItem", "common/shell/menu/leafMenuItem", "models/resources/database", "common/shell/menu/items/manageServer", "common/shell/menu/items/resources", "common/shell/menu/items/stats", "common/shell/menu/items/settings", "common/shell/menu/items/tasks", "common/shell/menu/items/transformers", "common/shell/menu/items/indexes", "common/shell/menu/items/documents"], function (require, exports, appUrl, separatorMenuItem, leafMenuItem, database, getManageServerMenuItem, getResourcesMenuItem, getStatsMenuItem, getSettingsMenuItem, getTasksMenuItem, getTransformersMenuItem, getIndexesMenuItem, getDocumentsMenuItem) {
    "use strict";
    function generateMenuItems(resource) {
        if (!resource) {
            return generateNoActiveResourceMenuItems();
        }
        else if (resource instanceof database) {
            return generateActiveDatabaseMenuItems();
        }
        else {
            throw new Error("Menu items for resource of type " + resource.fullTypeName + " are not implemented.");
        }
    }
    function aboutItem() {
        return new leafMenuItem({
            route: 'about',
            moduleId: 'viewmodels/shell/about',
            title: 'About',
            tooltip: "About",
            nav: true,
            css: 'icon-info',
            dynamicHash: appUrl.forAbout
        });
    }
    function generateNoActiveResourceMenuItems() {
        var appUrls = appUrl.forCurrentDatabase();
        return [
            new separatorMenuItem('Server'),
            getResourcesMenuItem(appUrls),
            getManageServerMenuItem(),
            aboutItem()
        ];
    }
    function generateActiveDatabaseMenuItems() {
        var appUrls = appUrl.forCurrentDatabase();
        return [
            getDocumentsMenuItem(appUrls),
            getIndexesMenuItem(appUrls),
            getTransformersMenuItem(appUrls),
            new separatorMenuItem('Manage'),
            getTasksMenuItem(appUrls),
            getSettingsMenuItem(appUrls),
            getStatsMenuItem(appUrls),
            new separatorMenuItem('Server'),
            getResourcesMenuItem(appUrls),
            getManageServerMenuItem(),
            aboutItem()
        ];
    }
    return generateMenuItems;
});

//# sourceMappingURL=generateMenuItems.js.map
