define(["require", "exports"], function (require, exports) {
    "use strict";
    var intermediateMenuItem = (function () {
        function intermediateMenuItem(title, children, css, hashes) {
            var _this = this;
            this.type = "intermediate";
            this.isOpen = ko.observable(false);
            this.parent = ko.observable(null);
            this.depth = ko.computed(function () {
                var next = _this.parent(), result = 0;
                while (next) {
                    result += 1;
                    next = next.parent();
                }
                return result;
            });
            this.title = title;
            this.children = children;
            this.css = css;
            if (hashes && hashes.dynamicHash) {
                this.dynamicHash = hashes.dynamicHash;
            }
            if (hashes && hashes.hash) {
                this.hash = hashes.hash;
            }
            if (this.children && this.children.length) {
                this.children.forEach(function (x) { return x.parent(_this); });
            }
            this.path = ko.pureComputed(function () {
                if (_this.hash) {
                    return _this.hash;
                }
                else if (_this.dynamicHash) {
                    return _this.dynamicHash();
                }
                return null;
            });
        }
        intermediateMenuItem.prototype.open = function () {
            this.isOpen(true);
        };
        intermediateMenuItem.prototype.close = function () {
            this.isOpen(false);
        };
        return intermediateMenuItem;
    }());
    return intermediateMenuItem;
});

//# sourceMappingURL=intermediateMenuItem.js.map
