define(["require", "exports", "common/shell/activeResourceTracker", "common/appUrl", "common/shell/menu/intermediateMenuItem", "common/shell/menu/leafMenuItem"], function (require, exports, activeResourceTracker, appUrl, intermediateMenuItem, leafMenuItem) {
    "use strict";
    function getTasksMenuItem(appUrls) {
        var activeDatabase = activeResourceTracker.default.database;
        var importDatabaseUrl = ko.computed(function () { return appUrl.forImportDatabase(activeDatabase()); });
        var exportDatabaseUrl = ko.computed(function () { return appUrl.forExportDatabase(activeDatabase()); });
        var setAcknowledgedEtagUrl = ko.computed(function () { return appUrl.forSetAcknowledgedEtag(activeDatabase()); });
        var sampleDataUrl = ko.computed(function () { return appUrl.forSampleData(activeDatabase()); });
        var csvImportUrl = ko.computed(function () { return appUrl.forCsvImport(activeDatabase()); });
        var submenu = [
            new leafMenuItem({
                route: [
                    'databases/tasks',
                    'databases/tasks/importDatabase'
                ],
                moduleId: 'viewmodels/database/tasks/importDatabase',
                title: 'Import Database',
                nav: true,
                css: 'icon-import-database',
                dynamicHash: importDatabaseUrl
            }),
            new leafMenuItem({
                route: 'databases/tasks/exportDatabase',
                moduleId: 'viewmodels/database/tasks/exportDatabase',
                title: 'Export Database',
                nav: true,
                css: 'icon-export-database',
                dynamicHash: exportDatabaseUrl
            }),
            /* TODO:
            new leafMenuItem({
                route: 'databases/tasks/subscriptionsTask',
                moduleId: 'viewmodels/database/tasks/subscriptionsTask',
                title: 'Subscriptions',
                nav: activeDatabase() && activeDatabase().isAdminCurrentTenant(),
                css: 'icon-plus',
                dynamicHash: setAcknowledgedEtagUrl
            }),*/
            new leafMenuItem({
                route: 'databases/tasks/sampleData',
                moduleId: 'viewmodels/database/tasks/createSampleData',
                title: 'Create Sample Data',
                nav: true,
                css: 'icon-create-sample-data',
                dynamicHash: sampleDataUrl
            }),
        ];
        return new intermediateMenuItem('Tasks', submenu, 'icon-tasks');
    }
    return getTasksMenuItem;
});

//# sourceMappingURL=tasks.js.map
