define(["require", "exports", "common/constants/events", "common/shell/resourcesManager", "common/appUrl"], function (require, exports, EVENTS, resourcesManager, appUrl) {
    "use strict";
    /*
        Events emitted through ko.postbox
            * ResourceSwitcher.Show - when searchbox is opened
            * ResourceSwitcher.Hide - when searchbox is hidden
            * ResourceSwitcher.ItemSelected - item selected from resource switcher pane
    */
    var resourceSwitcher = (function () {
        function resourceSwitcher() {
            var _this = this;
            this.resourcesManager = resourcesManager.default;
            this.hideHandler = function (e) {
                if (_this.shouldConsumeHideEvent(e)) {
                    _this.hide();
                }
            };
            this.filter = ko.observable();
            this.filteredResources = ko.computed(function () {
                var filter = _this.filter();
                var resources = _this.resourcesManager.resources();
                if (!filter)
                    return resources;
                if (!resources) {
                    return [];
                }
                return resources.filter(function (x) { return x.name.toLowerCase().includes(filter.toLowerCase()); });
            });
        }
        resourceSwitcher.prototype.initialize = function () {
            var _this = this;
            this.$selectDatabaseContainer = $('.resource-switcher-container');
            this.$selectDatabase = $('.form-control.btn-toggle.resource-switcher');
            this.$filter = $('.resource-switcher-container .database-filter');
            this.$selectDatabaseContainer.on('click', function (e) {
                e.stopPropagation();
                _this.show();
            });
            this.$selectDatabase.on('click', function (e) {
                if (_this.$selectDatabaseContainer.is('.active')) {
                    _this.hide();
                }
                else {
                    _this.show();
                }
                e.stopPropagation();
            });
            var self = this;
            $('.box-container a', this.$selectDatabaseContainer).on('click', function (e) {
                e.stopPropagation();
                self.hide();
                var a = this;
                ko.postbox.publish(EVENTS.ResourceSwitcher.ItemSelected, a.href);
            });
        };
        resourceSwitcher.prototype.selectResource = function (rs, $event) {
            if ($event.ctrlKey) {
                window.open(appUrl.forDocumentsByDatabaseName(null, rs.name));
            }
            else {
                rs.activate();
                this.hide();
            }
        };
        resourceSwitcher.prototype.show = function () {
            window.addEventListener("click", this.hideHandler, true);
            this.$selectDatabaseContainer.addClass('active');
            this.$filter.focus();
        };
        resourceSwitcher.prototype.hide = function () {
            window.removeEventListener("click", this.hideHandler, true);
            this.$selectDatabaseContainer.removeClass('active');
        };
        resourceSwitcher.prototype.shouldConsumeHideEvent = function (e) {
            return $(e.target).parents(".resource-switcher-container").length === 0
                && !$(e.target).hasClass(".resource-switcher");
        };
        return resourceSwitcher;
    }());
    return resourceSwitcher;
});

//# sourceMappingURL=resourceSwitcher.js.map
