define(["require", "exports", "common/appUrl", "common/shell/menu/leafMenuItem", "common/shell/menu/items/manageServer", "common/shell/menu/items/resources", "common/shell/menu/items/stats", "common/shell/menu/items/settings", "common/shell/menu/items/tasks", "common/shell/menu/items/indexes", "common/shell/menu/items/transformers", "common/shell/menu/items/documents"], function (require, exports, appUrl, leafMenuItem, getManageServerMenuItem, getResourcesMenuItem, getStatsMenuItem, getSettingsMenuItem, getTasksMenuItem, getIndexesMenuItem, getTransformersMenuItem, getDocumentsMenuItem) {
    "use strict";
    function getRouterConfiguration() {
        return generateAllMenuItems()
            .map(getMenuItemDurandalRoutes)
            .reduce(function (result, next) { return result.concat(next); }, [])
            .reduce(function (result, next) {
            var nextJson = JSON.stringify(next);
            if (!result.some(function (x) { return JSON.stringify(x) === nextJson; })) {
                result.push(next);
            }
            return result;
        }, []);
    }
    function convertToDurandalRoute(leaf) {
        return {
            route: leaf.route,
            title: leaf.title,
            moduleId: leaf.moduleId,
            nav: leaf.nav,
            dynamicHash: leaf.dynamicHash
        };
    }
    function getMenuItemDurandalRoutes(item) {
        if (item.type === 'intermediate') {
            var intermediateItem = item;
            return intermediateItem.children
                .map(function (child) { return getMenuItemDurandalRoutes(child); })
                .reduce(function (result, next) { return result.concat(next); }, []);
        }
        else if (item.type === 'leaf') {
            return [convertToDurandalRoute(item)];
        }
        return [];
    }
    function generateAllMenuItems() {
        var appUrls = appUrl.forCurrentDatabase();
        return [
            getDocumentsMenuItem(appUrls),
            getIndexesMenuItem(appUrls),
            getTransformersMenuItem(appUrls),
            getTasksMenuItem(appUrls),
            getSettingsMenuItem(appUrls),
            getStatsMenuItem(appUrls),
            getResourcesMenuItem(appUrls),
            getManageServerMenuItem(),
            new leafMenuItem({
                route: 'about',
                moduleId: 'viewmodels/shell/about',
                title: 'About',
                tooltip: "About",
                nav: true,
                css: 'icon-info',
                dynamicHash: appUrl.forAbout
            })
        ];
    }
    return getRouterConfiguration();
});

//# sourceMappingURL=routerConfiguration.js.map
