define(["require", "exports"], function (require, exports) {
    "use strict";
    /*
        Events emitted through ko.postbox
            * SearchBox.Show - when searchbox is opened
            * SearchBox.Hide - when searchbox is hidden
            * SearchBox.Input - on searchbox input, passes the searchbox value as arg
    */
    var searchBox = (function () {
        function searchBox() {
            var _this = this;
            this.hideHandler = function (e) {
                if (_this.shouldConsumeHideEvent(e)) {
                    _this.hide();
                }
            };
        }
        searchBox.prototype.initialize = function () {
            var _this = this;
            this.$searchInput = $('.search-container input[type=search]');
            this.$searchContainer = $('.search-container');
            this.$searchInput.on('input', function (e) {
                ko.postbox.publish('SearchBox.Input', _this.$searchInput.val());
            });
            this.$searchInput.click(function (e) {
                e.stopPropagation();
                _this.show();
            });
            $('.search-container .autocomplete-list.box-container')
                .click(function (e) { return e.stopPropagation(); });
            $('.search-container .autocomplete-list.box-container a').on('click', function (e) {
                e.stopPropagation();
                _this.hide();
            });
        };
        searchBox.prototype.show = function () {
            window.addEventListener("click", this.hideHandler, true);
            this.$searchContainer.addClass('active');
        };
        searchBox.prototype.hide = function () {
            window.removeEventListener("click", this.hideHandler, true);
            this.$searchContainer.removeClass('active');
        };
        searchBox.prototype.shouldConsumeHideEvent = function (e) {
            return $(e.target).parents(".search-container").length === 0;
        };
        return searchBox;
    }());
    return searchBox;
});

//# sourceMappingURL=searchBox.js.map
