define(["require", "exports", "common/generalUtils"], function (require, exports, genUtils) {
    "use strict";
    var sqlReplicationStats = (function () {
        function sqlReplicationStats(replicationStats) {
            var _this = this;
            this.statistics = ko.observable(); //TODO: use type here!
            this.metrics = ko.observable();
            this.name = ko.observable("");
            if (replicationStats.Statistics && replicationStats.Statistics.LastErrorTime && replicationStats.Statistics.LastErrorTime === "0001-01-01T00:00:00.0000000") {
                replicationStats.Statistics.LastErrorTime = "No Errors";
            }
            this.statistics(replicationStats.Statistics);
            this.name(replicationStats.Name);
            this.metrics(replicationStats.Metrics);
            this.rateMetrics = ko.computed(function () {
                var computedRateMetrics = [];
                var generalMetrics = _this.metrics().GeneralMetrics;
                var tablesMetrics = _this.metrics().TablesMetrics;
                if (!!generalMetrics) {
                    $.map(generalMetrics, function (value, key) {
                        if (value.Type == "Meter") {
                            value["Name"] = key;
                            value["Table"] = "";
                            $.map(value, function (propertyValue, propertyName) {
                                if (!isNaN(propertyValue)) {
                                    value[propertyName] = genUtils.formatAsCommaSeperatedString(propertyValue, 2);
                                }
                            });
                            computedRateMetrics.push(value);
                        }
                    });
                }
                if (!!tablesMetrics) {
                    $.map(tablesMetrics, function (tablesMetricsData, tableMetricsKey) {
                        $.map(tablesMetricsData, function (value, key) {
                            if (value.Type === "Meter") {
                                var newMetric = value;
                                newMetric["Name"] = key;
                                value["Table"] = tableMetricsKey;
                                $.map(value, function (propertyValue, propertyName) {
                                    if (!isNaN(propertyValue)) {
                                        value[propertyName] = genUtils.formatAsCommaSeperatedString(propertyValue, 2);
                                    }
                                });
                                computedRateMetrics.push(newMetric);
                            }
                        });
                    });
                }
                return computedRateMetrics;
            });
            this.histogramMetrics = ko.computed(function () {
                var computedHistogramMetrics = [];
                var generalMetrics = _this.metrics().GeneralMetrics;
                var tablesMetrics = _this.metrics().TablesMetrics;
                if (!!generalMetrics) {
                    $.map(generalMetrics, function (value, key) {
                        if (value.Type === "Histogram") {
                            value.Name = key;
                            value["Table"] = "";
                            value["Percentiles"] = $.map(value["Percentiles"], function (percentileValue, percentile) {
                                return {
                                    percentileValue: genUtils.formatAsCommaSeperatedString(percentileValue, 2),
                                    percentile: percentile
                                };
                            });
                            $.map(value, function (propertyValue, propertyName) {
                                if (!isNaN(propertyValue)) {
                                    value[propertyName] = genUtils.formatAsCommaSeperatedString(propertyValue, 2);
                                }
                            });
                            computedHistogramMetrics.push(value);
                        }
                    });
                }
                if (!!tablesMetrics) {
                    $.map(tablesMetrics, function (tablesMetricsData, tableMetricsKey) {
                        $.map(tablesMetricsData, function (value, key) {
                            if (value.Type == "Histogram") {
                                value["Name"] = key;
                                value["Table"] = tableMetricsKey;
                                value["Percentiles"] = $.map(value["Percentiles"], function (percentileValue, percentile) {
                                    return {
                                        percentileValue: genUtils.formatAsCommaSeperatedString(percentileValue, 2),
                                        percentile: percentile
                                    };
                                });
                                $.map(value, function (propertyValue, propertyName) {
                                    if (!isNaN(propertyValue)) {
                                        value[propertyName] = genUtils.formatAsCommaSeperatedString(propertyValue, 2);
                                    }
                                });
                                computedHistogramMetrics.push(value);
                            }
                        });
                    });
                }
                return computedHistogramMetrics;
            });
        }
        return sqlReplicationStats;
    }());
    sqlReplicationStats.ALL_TABLES = 'All Tables';
    return sqlReplicationStats;
});

//# sourceMappingURL=sqlReplicationStats.js.map
