define(["require", "exports", "d3"], function (require, exports, d3) {
    "use strict";
    var svgDownloader = (function () {
        function svgDownloader() {
        }
        svgDownloader.convertToData = function (svgElement, cssInliner) {
            var svgClone = svgElement.cloneNode(true);
            svgDownloader.fixAttributes(svgClone);
            var svgContainer = document.createElement('div');
            svgContainer.appendChild(svgClone);
            var targetStyles = cssInliner(svgClone);
            var s = document.createElement('style');
            s.setAttribute('type', 'text/css');
            s.innerHTML = "<![CDATA[\n" + targetStyles + "\n]]>";
            var defs = d3.select(svgClone).select('defs').node();
            if (!defs) {
                defs = document.createElement('defs');
                svgClone.insertBefore(defs, svgClone.firstChild);
            }
            defs.appendChild(s);
            return svgDownloader.svgHeader + "\n" + svgContainer.innerHTML;
        };
        svgDownloader.cleanup = function () {
            // clean previous image (in any)
            $("#downloadSvg").remove();
        };
        svgDownloader.b64toBlob = function (b64Data, contentType, sliceSize) {
            contentType = contentType || '';
            sliceSize = sliceSize || 512;
            var byteCharacters = atob(b64Data);
            var byteArrays = [];
            for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
                var slice = byteCharacters.slice(offset, offset + sliceSize);
                var byteNumbers = new Array(slice.length);
                for (var i = 0; i < slice.length; i++) {
                    byteNumbers[i] = slice.charCodeAt(i);
                }
                var byteArray = new Uint8Array(byteNumbers);
                byteArrays.push(byteArray);
            }
            var blob = new Blob(byteArrays, { type: contentType });
            return blob;
        };
        svgDownloader.createLinkAndStartDownload = function (blob, filename) {
            if (navigator && navigator.msSaveBlob) {
                navigator.msSaveBlob(blob, filename);
            }
            else {
                var blobUrl = URL.createObjectURL(blob);
                var a = document.createElement('a');
                a.id = "downloadSvg";
                a.download = filename;
                a.href = blobUrl;
                document.body.appendChild(a); // required by firefox
                a.click();
            }
        };
        svgDownloader.downloadSvg = function (svgElement, filename, cssInliner) {
            svgDownloader.cleanup();
            var textSvgData = svgDownloader.convertToData(svgElement, cssInliner);
            var encodedImage = window.btoa(textSvgData);
            var blob = svgDownloader.b64toBlob(encodedImage, 'image/svg+xml');
            svgDownloader.createLinkAndStartDownload(blob, filename);
        };
        svgDownloader.downloadPng = function (svgElement, filename, cssInliner) {
            svgDownloader.cleanup();
            var textSvgData = svgDownloader.convertToData(svgElement, cssInliner);
            var image = new Image();
            image.src = 'data:image/svg+xml;base64,' + window.btoa(textSvgData);
            image.onerror = function () {
                alert("Unable to download image!");
            };
            image.onload = function () {
                var canvas = document.createElement('canvas');
                canvas.width = image.width;
                canvas.height = image.height;
                var context = canvas.getContext('2d');
                context.drawImage(image, 0, 0);
                var dataUrlStripped = canvas.toDataURL('image/png').replace(/^data:image\/png;base64,/, "");
                var blob = svgDownloader.b64toBlob(dataUrlStripped, 'image/png');
                svgDownloader.createLinkAndStartDownload(blob, filename);
            };
        };
        // helper method to extract css from element
        svgDownloader.extractInlineCss = function (element) {
            var targetStyles = "";
            var sheets = document.styleSheets;
            for (var i = 0; i < sheets.length; i++) {
                var rules = sheets[i].cssRules;
                for (var j = 0; j < rules.length; j++) {
                    var rule = rules[j];
                    if (typeof (rule.style) != "undefined") {
                        var elems = element.querySelectorAll(rule.selectorText);
                        if (elems.length > 0) {
                            targetStyles += rule.selectorText + " { " + rule.style.cssText + " }\n";
                        }
                    }
                }
            }
            return targetStyles;
        };
        svgDownloader.fixAttributes = function (el) {
            var viewBox = el.getAttribute('viewBox');
            svgDownloader.removeAttributes(el);
            el.setAttribute("version", "1.1");
            el.setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns', 'http://www.w3.org/2000/svg');
            el.setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns:xlink', 'http://www.w3.org/1999/xlink');
            if (viewBox) {
                var splitedViewBox = viewBox.split(/\s+|,/);
                el.setAttribute("width", splitedViewBox[2]);
                el.setAttribute("height", splitedViewBox[3]);
            }
        };
        svgDownloader.removeAttributes = function (el) {
            var attributes = $.map(el.attributes, function (item) {
                return item.name;
            });
            var selection = $(el);
            $.each(attributes, function (i, item) {
                selection.removeAttr(item);
            });
        };
        return svgDownloader;
    }());
    svgDownloader.svgHeader = '<?xml version="1.0" standalone="no"?>\n' +
        '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">';
    return svgDownloader;
});

//# sourceMappingURL=svgDownloader.js.map
