define(["require", "exports", "common/changeSubscription", "common/changesCallback", "commands/commandBase", "common/idGenerator"], function (require, exports, changeSubscription, changesCallback, commandBase, idGenerator) {
    "use strict";
    //TODO: use abstractWebSocketClient
    var trafficWatchClient = (function () {
        function trafficWatchClient(resourcePath, token) {
            this.resourcePath = resourcePath;
            this.token = token;
            this.readyStateOpen = 1;
            this.isCleanClose = false;
            this.normalClosureCode = 1000;
            this.normalClosureMessage = "CLOSE_NORMAL";
            this.successfullyConnectedOnce = false;
            this.sentMessages = [];
            this.commandBase = new commandBase();
            this.adminLogsHandlers = ko.observableArray();
            this.connectionOpeningTask = $.Deferred();
            this.connectionClosingTask = $.Deferred();
            this.eventsId = idGenerator.generateId();
        }
        trafficWatchClient.prototype.connect = function () {
            var connectionString = 'singleUseAuthToken=' + this.token + '&id=' + this.eventsId;
            if ("WebSocket" in window) {
                this.connectWebSocket(connectionString);
            }
            else {
                //The browser doesn't support websocket
                this.connectionOpeningTask.reject();
            }
        };
        trafficWatchClient.prototype.connectWebSocket = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            var wsProtocol = window.location.protocol === 'https:' ? 'wss://' : 'ws://';
            this.webSocket = new WebSocket(wsProtocol + window.location.host + this.resourcePath + '/traffic-watch/websocket?' + connectionString);
            this.webSocket.onmessage = function (e) { return _this.onMessage(e); };
            this.webSocket.onerror = function (e) {
                if (connectionOpened == false) {
                    _this.connectionOpeningTask.reject();
                }
                else {
                    _this.connectionClosingTask.resolve({ Error: e });
                }
            };
            this.webSocket.onclose = function (e) {
                _this.connectionClosingTask.resolve();
            };
            this.webSocket.onopen = function () {
                console.log("Connected to WebSockets HTTP Trace for " + ((!!_this.resourcePath) ? (_this.resourcePath) : "admin"));
                _this.successfullyConnectedOnce = true;
                connectionOpened = true;
                _this.connectionOpeningTask.resolve();
            };
        };
        trafficWatchClient.prototype.fireEvents = function (events, param, filter) {
            for (var i = 0; i < events.length; i++) {
                if (filter(param)) {
                    events[i].fire(param);
                }
            }
        };
        trafficWatchClient.prototype.onMessage = function (e) {
            var eventDto = JSON.parse(e.data);
            var type = eventDto.Type;
            var value = eventDto.Value;
            if (type !== "Heartbeat") {
                if (type === "LogNotification") {
                    this.fireEvents(this.adminLogsHandlers(), value, function () { return true; });
                }
                else {
                    console.log("Unhandled Changes API notification type: " + type);
                }
            }
        };
        trafficWatchClient.prototype.watchTraffic = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            this.adminLogsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.adminLogsHandlers.remove(callback);
            });
        };
        trafficWatchClient.prototype.disconnect = function () {
            var _this = this;
            this.connectionOpeningTask.done(function () {
                if (_this.webSocket && _this.webSocket.readyState === _this.readyStateOpen) {
                    console.log("Disconnecting from WebSocket HTTP Trace for " + ((!!_this.resourcePath) ? (_this.resourcePath) : "admin"));
                    _this.isCleanClose = true;
                    _this.webSocket.close(_this.normalClosureCode, _this.normalClosureMessage);
                }
            });
        };
        return trafficWatchClient;
    }());
    trafficWatchClient.messageWasShownOnce = false;
    return trafficWatchClient;
});

//# sourceMappingURL=trafficWatchClient.js.map
