var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "models/resources/databaseAccess", "common/appUrl", "models/database/documents/documentMetadata", "models/database/documents/document"], function (require, exports, databaseAccess, appUrl, documentMetadata, document) {
    "use strict";
    var apiKey = (function (_super) {
        __extends(apiKey, _super);
        function apiKey(dto) {
            var _this = _super.call(this, dto) || this;
            _this.name = ko.observable();
            _this.secret = ko.observable();
            _this.enabled = ko.observable();
            _this.databases = ko.observableArray();
            _this.visible = ko.observable(true);
            _this.nameCustomValidity = ko.observable('');
            _this.name(dto.Name);
            _this.savedName = dto.Name;
            _this.secret(dto.Secret);
            _this.enabled(dto.Enabled);
            _this.databases(dto.Databases.map(function (d) { return new databaseAccess(d); }));
            _this.metadata = new documentMetadata(dto['@metadata']);
            _this.fullApiKey = ko.computed(function () {
                if (!_this.name() || !_this.secret()) {
                    return "Requires name and secret";
                }
                return _this.name() + "/" + _this.secret();
            });
            _this.connectionString = ko.computed(function () {
                if (!_this.name() || !_this.secret()) {
                    return "Requires name and secret";
                }
                return "Url = " + appUrl.forServer() + "; ApiKey = " + _this.fullApiKey() + "; Database = ";
            });
            _this.directLink = ko.computed(function () {
                if (!_this.name() || !_this.secret()) {
                    return "Requires name and secret";
                }
                return appUrl.forServer() + "/studio/index.html#api-key=" + _this.fullApiKey();
            });
            _this.needToShowSystemDatabaseWarning = ko.computed(function () {
                var resources = _this.databases();
                var hasAllDatabasesAccess = resources.filter(function (x) { return x.tenantId() === "*"; }).length > 0;
                var hasSystemDatabaseAccess = resources.filter(function (x) { return x.tenantId() === "<system>"; }).length > 0;
                return hasAllDatabasesAccess && !hasSystemDatabaseAccess;
            });
            return _this;
        }
        apiKey.empty = function () {
            return new apiKey({
                Databases: [databaseAccess.empty().toDto()],
                Enabled: false,
                Name: "",
                Secret: ""
            });
        };
        apiKey.prototype.toDto = function () {
            var meta = this.__metadata.toDto();
            meta['@id'] = "Raven/ApiKeys/" + this.name();
            return {
                '@metadata': meta,
                Databases: this.databases().map(function (db) { return db.toDto(); }),
                Enabled: this.enabled(),
                Name: this.name(),
                Secret: this.secret()
            };
        };
        apiKey.prototype.enable = function () {
            this.enabled(true);
        };
        apiKey.prototype.disable = function () {
            this.enabled(false);
        };
        apiKey.prototype.generateSecret = function () {
            // The old Silverlight Studio would create a new GUID, strip out the 
            // dashes, and convert to base62.
            //
            // For the time being (is there a better way?), we're just creating a 
            // random string of alpha numeric characters.
            var minimumLength = 10;
            var maxLength = 32;
            var randomLength = Math.max(minimumLength, Math.random() * maxLength);
            var randomSecret = apiKey.randomString(randomLength, '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ');
            this.secret(randomSecret);
        };
        apiKey.prototype.addEmptyDatabase = function () {
            var newItem = { TenantId: '', Admin: false, ReadOnly: false };
            this.databases.push(new databaseAccess(newItem));
        };
        apiKey.prototype.removeDatabase = function (database) {
            this.databases.remove(database);
        };
        apiKey.prototype.setIdFromName = function () {
            this.__metadata.id = "Raven/ApiKeys/" + this.name();
            if (this.savedName !== this.name()) {
                this.__metadata.etag(null);
                this.savedName = this.name();
            }
        };
        apiKey.prototype.isValid = function (index) {
            var isApiKeyNameValid = this.name().indexOf("\\") === -1;
            var requiredValues = [this.name(), this.secret()];
            return requiredValues.every(function (v) { return v != null && v.length > 0; }) && isApiKeyNameValid;
        };
        apiKey.randomString = function (length, chars) {
            var result = '';
            for (var i = length; i > 0; --i)
                result += chars[Math.round(Math.random() * (chars.length - 1))];
            return result;
        };
        apiKey.prototype.getKey = function () {
            return "Raven/ApiKeys/" + this.name();
        };
        return apiKey;
    }(document));
    return apiKey;
});

//# sourceMappingURL=apiKey.js.map
