/// <reference path="../../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var counterChange = (function () {
        function counterChange(dto, isNew) {
            if (isNew === void 0) { isNew = false; }
            var _this = this;
            this.value = ko.observable(0);
            this.counterName = ko.observable("");
            this.group = ko.observable("");
            this.delta = ko.observable();
            this.isNew = ko.observable(false);
            this.maxNameLength = 200;
            this.value(dto.CurrentValue);
            this.group(dto.Group);
            this.counterName(dto.CounterName);
            this.delta("0");
            this.isNew(isNew);
            this.counterNameCustomValidityError = ko.computed(function () {
                var counterName = _this.counterName();
                return _this.checkName(counterName, "counter name");
            });
            this.groupCustomValidityError = ko.computed(function () {
                var group = _this.group();
                return _this.checkName(group, "group name");
            });
            this.deltaCustomValidityError = ko.computed(function () {
                var delta = _this.delta();
                if (_this.isNew() === false && delta === "0")
                    return "The change must be different than 0!";
                if (_this.isNumber(delta) === false)
                    return "Please enter a valid natural number!";
                return "";
            });
        }
        counterChange.empty = function () {
            return new counterChange({
                CurrentValue: 0,
                Group: "",
                CounterName: "",
                Delta: 0
            }, true);
        };
        counterChange.prototype.checkName = function (name, fieldName) {
            var message = "";
            if (!$.trim(name)) {
                message = "An empty " + fieldName + " is forbidden for use!";
            }
            else if (name.length > this.maxNameLength) {
                message = "The  " + fieldName + " length can't exceed " + this.maxNameLength + " characters!";
            }
            return message;
        };
        counterChange.prototype.isNumber = function (num) {
            if (num < 0)
                return true;
            var n1 = Math.abs(num);
            var n2 = parseInt(num, 10);
            return !isNaN(n1) && n2 === n1 && n1.toString() === num;
        };
        counterChange.prototype.getValue = function () {
            return this.value().toLocaleString();
        };
        return counterChange;
    }());
    return counterChange;
});

//# sourceMappingURL=counterChange.js.map
