define(["require", "exports", "common/pagedList", "commands/counter/getCountersCommand", "common/cssGenerator"], function (require, exports, pagedList, getCountersCommand, cssGenerator) {
    "use strict";
    var counterGroup = (function () {
        function counterGroup(name, ownerCounterStorage, count) {
            if (count === void 0) { count = 0; }
            var _this = this;
            this.name = name;
            this.ownerCounterStorage = ownerCounterStorage;
            this.colorClass = "";
            this.countersCount = ko.observable(0);
            this.countersCountWithThousandsSeparator = ko.computed(function () { return _this.countersCount().toLocaleString(); });
            this.isAllGroupsGroup = false;
            this.countersCount(count);
            this.isAllGroupsGroup = name === counterGroup.allGroupsGroupName;
            this.colorClass = counterGroup.getGroupCssClass(name, ownerCounterStorage);
        }
        counterGroup.prototype.activate = function () {
            ko.postbox.publish("ActivateGroup", this);
        };
        counterGroup.prototype.getCounters = function () {
            if (!this.countersList) {
                this.countersList = this.createPagedList();
            }
            return this.countersList;
        };
        counterGroup.prototype.invalidateCache = function () {
            var countersList = this.getCounters();
            countersList.invalidateCache();
        };
        counterGroup.createAllGroupsCollection = function (ownerCounterStorage) {
            return new counterGroup(counterGroup.allGroupsGroupName, ownerCounterStorage);
        };
        counterGroup.fromDto = function (dto, storage) {
            return new counterGroup(dto.Name, storage, dto.Count);
        };
        counterGroup.getGroupCssClass = function (entityName, cs) {
            if (entityName === counterGroup.allGroupsGroupName) {
                return "all-documents-collection";
            }
            return cssGenerator.getCssClass(entityName, counterGroup.groupColorMaps, cs);
        };
        counterGroup.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchCounters(skip, take); };
            var list = new pagedList(fetcher);
            list.collectionName = this.name;
            return list;
        };
        counterGroup.prototype.fetchCounters = function (skip, take) {
            var group = this.isAllGroupsGroup ? null : this.name;
            return new getCountersCommand(this.ownerCounterStorage, skip, take, group).execute();
        };
        return counterGroup;
    }());
    counterGroup.allGroupsGroupName = "All Groups";
    counterGroup.groupColorMaps = [];
    return counterGroup;
});

//# sourceMappingURL=counterGroup.js.map
