/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var nodeConnectionInfo = (function () {
        function nodeConnectionInfo(dto) {
            var _this = this;
            this.uri = ko.observable();
            this.name = ko.observable();
            this.username = ko.observable();
            this.password = ko.observable();
            this.domain = ko.observable();
            this.apiKey = ko.observable();
            this.state = ko.observable(); // used to store owning collection name (voting, non-voting, promotable)
            this.status = ko.observable("Loading");
            this.isVoting = ko.observable();
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
            this.uri(dto.Uri);
            this.name(dto.Name);
            this.username(dto.Username);
            this.password(dto.Password);
            this.domain(dto.Domain);
            this.apiKey(dto.ApiKey);
            this.isVoting(!dto.IsNoneVoter);
            if (this.username()) {
                this.isUserCredentials(true);
            }
            else if (this.apiKey()) {
                this.isApiKeyCredentials(true);
            }
        }
        nodeConnectionInfo.prototype.toDto = function () {
            return {
                Uri: this.uri(),
                Name: this.name(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey(),
                IsNoneVoter: !this.isVoting()
            };
        };
        nodeConnectionInfo.empty = function () {
            return new nodeConnectionInfo({
                Uri: null,
                Name: null,
                IsNoneVoter: false
            });
        };
        nodeConnectionInfo.prototype.useUserCredentials = function () {
            this.isUserCredentials(true);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
        };
        nodeConnectionInfo.prototype.useApiKeyCredentials = function () {
            this.isApiKeyCredentials(true);
            this.isUserCredentials(false);
            this.clearUserCredentials();
        };
        nodeConnectionInfo.prototype.useNoCredentials = function () {
            this.isUserCredentials(false);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
            this.clearUserCredentials();
        };
        nodeConnectionInfo.prototype.clearApiKeyCredentials = function () {
            this.apiKey(null);
        };
        nodeConnectionInfo.prototype.clearUserCredentials = function () {
            this.username(null);
            this.password(null);
            this.domain(null);
        };
        return nodeConnectionInfo;
    }());
    return nodeConnectionInfo;
});

//# sourceMappingURL=nodeConnectionInfo.js.map
