/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var serverConnectionInfo = (function () {
        function serverConnectionInfo() {
            var _this = this;
            this.url = ko.observable();
            this.username = ko.observable();
            this.password = ko.observable();
            this.domain = ko.observable();
            this.apiKey = ko.observable();
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
        }
        serverConnectionInfo.prototype.guessCredentialsType = function () {
            if (this.apiKey()) {
                this.useApiKeyCredentials();
            }
            else if (this.username()) {
                this.useUserCredentials();
            }
        };
        serverConnectionInfo.prototype.useUserCredentials = function () {
            this.isUserCredentials(true);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
        };
        serverConnectionInfo.prototype.useApiKeyCredentials = function () {
            this.isApiKeyCredentials(true);
            this.isUserCredentials(false);
            this.clearUserCredentials();
        };
        serverConnectionInfo.prototype.useNoCredentials = function () {
            this.isUserCredentials(false);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
            this.clearUserCredentials();
        };
        serverConnectionInfo.prototype.clearApiKeyCredentials = function () {
            this.apiKey(undefined);
        };
        serverConnectionInfo.prototype.clearUserCredentials = function () {
            this.username(undefined);
            this.password(undefined);
            this.domain(undefined);
        };
        serverConnectionInfo.prototype.toDto = function () {
            return {
                Url: this.url(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey()
            };
        };
        return serverConnectionInfo;
    }());
    return serverConnectionInfo;
});

//# sourceMappingURL=serverConnectionInfo.js.map
