define(["require", "exports"], function (require, exports) {
    "use strict";
    var logEntry = (function () {
        function logEntry(dto, nowProvider) {
            var _this = this;
            this.nowProvider = nowProvider;
            this.level = ko.observable();
            this.timeStamp = ko.observable();
            this.message = ko.observable();
            this.exception = ko.observable();
            this.loggerName = ko.observable();
            this.database = ko.observable();
            this.stackTrace = ko.observable();
            this.dateMoment = ko.observable();
            this.dateRaw = ko.observable();
            this.level(dto.Level);
            this.timeStamp(dto.TimeStamp);
            this.message(dto.Message);
            this.exception(dto.Exception);
            this.loggerName(dto.LoggerName);
            this.database(dto.Database);
            this.stackTrace(dto.StackTrace);
            // since we know date is in ISO format parse it to date and then pass to moment - it will skip format detection
            this.dateMoment(moment(new Date(this.timeStamp())));
            this.dateRaw(this.dateMoment().toDate().getTime());
            this.humanizedTimestamp = ko.pureComputed(function () {
                var dateMoment = _this.dateMoment();
                var agoInMs = dateMoment.diff(_this.nowProvider());
                var humanized = moment.duration(agoInMs).humanize(true);
                return humanized;
            });
            this.timeStampText = ko.pureComputed(function () {
                var dateMoment = _this.dateMoment();
                return dateMoment.format(" (MM/DD/YY, h:mma)");
            });
        }
        logEntry.prototype.copyFrom = function (newEntry) {
            this.level(newEntry.level());
            this.timeStamp(newEntry.timeStamp());
            this.message(newEntry.message());
            this.exception(newEntry.exception());
            this.loggerName(newEntry.loggerName());
            this.database(newEntry.database());
            this.stackTrace(newEntry.stackTrace());
            this.dateMoment(moment(new Date(newEntry.timeStamp())));
            this.dateRaw(this.dateMoment().toDate().getTime());
        };
        return logEntry;
    }());
    return logEntry;
});

//# sourceMappingURL=logEntry.js.map
