/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports", "d3"], function (require, exports, d3) {
    "use strict";
    var performanceTestResultWrapped = (function () {
        function performanceTestResultWrapped(dto) {
            var _this = this;
            this.formatter = d3.format(",.2f");
            this.totalRead = ko.computed(function () { return _this.formatter(dto.Result.TotalRead / 1024 / 1024) + " MB"; });
            this.totalWrite = ko.computed(function () { return _this.formatter(dto.Result.TotalWrite / 1024 / 1024) + " MB"; });
            this.hasReads = ko.computed(function () { return dto.Request.TestType == "generic" && (dto.Request.OperationType == "Read" || dto.Request.OperationType == "Mix"); });
            this.hasWrites = ko.computed(function () { return dto.Request.TestType == "batch" || dto.Request.OperationType == "Write" || dto.Request.OperationType == "Mix"; });
            this.testTime = ko.computed(function () { return _this.formatter(dto.Result.TotalTimeMs / 1000) + "s"; });
            this.avgRead = ko.computed(function () { return _this.formatter(dto.Result.TotalRead / 1024 / 1024 / (dto.Result.TotalTimeMs / 1000)) + "MB/s"; });
            this.avgWrite = ko.computed(function () { return _this.formatter(dto.Result.TotalWrite / 1024 / 1024 / (dto.Result.TotalTimeMs / 1000)) + "MB/s"; });
            this.readLatencyMin = ko.computed(function () { return _this.formatter(dto.Result.ReadLatency.Min) + "ms"; });
            this.readLatencyMax = ko.computed(function () { return _this.formatter(dto.Result.ReadLatency.Max) + "ms"; });
            this.readLatencyMean = ko.computed(function () { return _this.formatter(dto.Result.ReadLatency.Mean) + "ms"; });
            this.readLatencyStdev = ko.computed(function () { return _this.formatter(dto.Result.ReadLatency.Stdev) + "ms"; });
            this.writeLatencyMin = ko.computed(function () { return _this.formatter(dto.Result.WriteLatency.Min) + "ms"; });
            this.writeLatencyMax = ko.computed(function () { return _this.formatter(dto.Result.WriteLatency.Max) + "ms"; });
            this.writeLatencyMean = ko.computed(function () { return _this.formatter(dto.Result.WriteLatency.Mean) + "ms"; });
            this.writeLatencyStdev = ko.computed(function () { return _this.formatter(dto.Result.WriteLatency.Stdev) + "ms"; });
            this.readPercentiles = ko.computed(function () { return $.map(dto.Result.ReadLatency.Percentiles, function (value, key) {
                return {
                    key: key,
                    value: _this.formatter(value)
                };
            }); });
            this.writePercentiles = ko.computed(function () { return $.map(dto.Result.WriteLatency.Percentiles, function (value, key) {
                return {
                    key: key,
                    value: _this.formatter(value)
                };
            }); });
        }
        return performanceTestResultWrapped;
    }());
    return performanceTestResultWrapped;
});

//# sourceMappingURL=performanceTestResultWrapped.js.map
