define(["require", "exports"], function (require, exports) {
    "use strict";
    var runningTask = (function () {
        function runningTask(dto) {
            this.id = dto.Id;
            this.status = dto.Status;
            this.taskType = dto.TaskType;
            this.description = dto.Description;
            this.completed = dto.Completed;
            this.faulted = dto.Faulted;
            this.canceled = dto.Canceled;
            this.fullStatus = runningTask.createFullStatus(dto);
            this.startTime = dto.StartTime; // used for sorting
            this.timeStampText = this.createHumanReadableTime(dto.StartTime);
            this.exceptionText = dto.Exception ? JSON.stringify(dto.Exception, null, 2) : null;
            this.killable = dto.Killable;
        }
        runningTask.createFullStatus = function (dto) {
            var taskState = "Running";
            if (dto.Canceled) {
                taskState = "Canceled";
            }
            else if (dto.Faulted) {
                taskState = "Faulted";
            }
            else if (dto.Completed) {
                taskState = "Completed";
            }
            var status = null;
            if (dto.Status) {
                status = dto.Faulted || dto.Canceled ? dto.Status.Error : dto.Status.Progress;
            }
            return status ? taskState + ": " + status : taskState;
        };
        runningTask.prototype.createHumanReadableTime = function (time) {
            var now = moment();
            if (time) {
                return ko.computed(function () {
                    var dateMoment = moment(time);
                    var agoInMs = dateMoment.diff(now);
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                });
            }
            return ko.computed(function () { return time; });
        };
        return runningTask;
    }());
    return runningTask;
});

//# sourceMappingURL=runningTask.js.map
