define(["require", "exports", "common/pagedList", "commands/database/documents/getDocumentsFromCollectionCommand", "commands/database/documents/getAllDocumentsCommand", "common/cssGenerator"], function (require, exports, pagedList, getDocumentsFromCollectionCommand, getAllDocumentsCommand, cssGenerator) {
    "use strict";
    var collection = (function () {
        function collection(name, ownerDatabase, docCount) {
            if (docCount === void 0) { docCount = 0; }
            var _this = this;
            this.name = name;
            this.ownerDatabase = ownerDatabase;
            this.colorClass = "";
            this.documentCount = ko.observable(0);
            this.documentsCountWithThousandsSeparator = ko.computed(function () { return _this.documentCount().toLocaleString(); });
            this.isAllDocuments = false;
            this.bindings = ko.observable();
            this.collectionName = name;
            this.isAllDocuments = name === collection.allDocsCollectionName;
            this.colorClass = collection.getCollectionCssClass(name, ownerDatabase);
            this.documentCount(docCount);
        }
        Object.defineProperty(collection.prototype, "isSystemDocuments", {
            get: function () {
                return this.collectionName === collection.systemDocusCollectionName;
            },
            enumerable: true,
            configurable: true
        });
        // Notifies consumers that this collection should be the selected one.
        // Called from the UI when a user clicks a collection the documents page.
        collection.prototype.activate = function () {
            ko.postbox.publish("ActivateCollection", this);
        };
        collection.prototype.prettyLabel = function (text) {
            return text.replace(/__/g, '/');
        };
        collection.prototype.getDocuments = function () {
            if (!this.documentsList) {
                this.documentsList = this.createPagedList();
            }
            return this.documentsList;
        };
        collection.prototype.invalidateCache = function () {
            var documentsList = this.getDocuments();
            documentsList.invalidateCache();
        };
        collection.prototype.clearCollection = function () {
            if (this.isAllDocuments && !!this.documentsList) {
                this.documentsList.clear();
            }
        };
        collection.prototype.fetchDocuments = function (skip, take) {
            if (this.isAllDocuments) {
                return new getAllDocumentsCommand(this.ownerDatabase, skip, take).execute();
            }
            else {
                return new getDocumentsFromCollectionCommand(this, skip, take).execute();
            }
        };
        collection.createAllDocsCollection = function (ownerDatabase) {
            return new collection(collection.allDocsCollectionName, ownerDatabase);
        };
        collection.getCollectionCssClass = function (entityName, db) {
            if (entityName === collection.allDocsCollectionName) {
                return "all-documents-collection";
            }
            //TODO: any special color for system documents?
            return cssGenerator.getCssClass(entityName, collection.collectionColorMaps, db);
        };
        collection.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchDocuments(skip, take); };
            var list = new pagedList(fetcher);
            list.collectionName = this.name;
            return list;
        };
        return collection;
    }());
    collection.allDocsCollectionName = "All Documents";
    collection.systemDocusCollectionName = "@system";
    collection.collectionColorMaps = [];
    return collection;
});

//# sourceMappingURL=collection.js.map
