define(["require", "exports", "models/database/documents/customColumnParams"], function (require, exports, customColumnParams) {
    "use strict";
    var customColumns = (function () {
        function customColumns(dto) {
            this.columns = ko.observableArray();
            this.customMode = ko.observable(false);
            this.enabled = ko.observable(true);
            this.columns($.map(dto.Columns, function (c) { return new customColumnParams(c); }));
        }
        customColumns.empty = function () {
            return new customColumns({ Columns: [] });
        };
        customColumns.prototype.copyFrom = function (src) {
            this.columns(src.columns());
            this.customMode(src.customMode());
        };
        customColumns.prototype.clone = function () {
            var copy = new customColumns(this.toDto());
            copy.customMode(this.customMode());
            return copy;
        };
        customColumns.prototype.getBindings = function () {
            if (!this.hasOverrides()) {
                return undefined;
            }
            return this.columns().map(function (x) { return x.binding(); });
        };
        customColumns.prototype.hasOverrides = function () {
            return this.enabled() && this.customMode() && this.columns().length > 0;
        };
        customColumns.prototype.toDto = function () {
            return {
                'Columns': $.map(this.columns(), function (c) { return c.toDto(); })
            };
        };
        customColumns.prototype.findConfigFor = function (binding) {
            var colParams = this.columns();
            for (var i = 0; i < colParams.length; i++) {
                var colParam = colParams[i];
                if (colParam.binding() === binding) {
                    return colParam;
                }
            }
            return null;
        };
        return customColumns;
    }());
    return customColumns;
});

//# sourceMappingURL=customColumns.js.map
