define(["require", "exports", "models/database/documents/documentMetadata"], function (require, exports, documentMetadata) {
    "use strict";
    var document = (function () {
        function document(dto) {
            this.__metadata = new documentMetadata(dto["@metadata"]);
            for (var property in dto) {
                if (property !== "@metadata") {
                    this[property] = dto[property];
                }
            }
        }
        document.prototype.getEntityName = function () {
            return this.__metadata.ravenEntityName;
        };
        document.prototype.getId = function () {
            return this.__metadata.id;
        };
        document.prototype.getUrl = function () {
            return this.getId();
        };
        document.prototype.getDocumentPropertyNames = function () {
            var propertyNames = [];
            for (var property in this) {
                var isMeta = property === "__metadata" || property === "__moduleId__";
                var isFunction = typeof this[property] === "function";
                if (!isMeta && !isFunction) {
                    propertyNames.push(property);
                }
            }
            return propertyNames;
        };
        document.prototype.toDto = function (includeMeta) {
            if (includeMeta === void 0) { includeMeta = false; }
            var dto = {};
            var properties = this.getDocumentPropertyNames();
            for (var i = 0; i < properties.length; i++) {
                var property = properties[i];
                dto[property] = this[property];
            }
            dto["@metadata"] = includeMeta && this.__metadata ? this.__metadata.toDto() : undefined;
            return dto;
        };
        document.prototype.toBulkDoc = function (method) {
            var dto = this.toDto(true);
            var bulkDoc = {
                Document: dto,
                Key: this.getId(),
                Method: method,
                AdditionalData: null
            };
            var meta = dto["@metadata"];
            if (meta) {
                bulkDoc.Metadata = meta;
                if (meta["@etag"]) {
                    bulkDoc.Etag = meta["@etag"];
                }
            }
            return bulkDoc;
        };
        document.empty = function () {
            var emptyDto = {
                '@metadata': {}
            };
            return new document(emptyDto);
        };
        document.getEntityNameFromId = function (id) {
            if (!id) {
                return null;
            }
            var slashIndex = id.lastIndexOf("/");
            if (slashIndex >= 1) {
                return id.substring(0, 1).toUpperCase() + id.substring(1, slashIndex);
            }
            return id;
        };
        return document;
    }());
    return document;
});

//# sourceMappingURL=document.js.map
