/// <reference path="../../../../typings/tsd.d.ts" />
define(["require", "exports"], function (require, exports) {
    "use strict";
    var documentMetadata = (function () {
        function documentMetadata(dto) {
            var _this = this;
            this.etag = ko.observable(null);
            this.ravenLastModified = ko.observable(null);
            this.now = ko.observable(new Date());
            if (dto) {
                this.ravenEntityName = dto['Raven-Entity-Name'];
                this.ravenClrType = dto['Raven-Clr-Type'];
                this.nonAuthoritativeInfo = dto['Non-Authoritative-Information'];
                this.id = dto['@id'];
                this.tempIndexScore = dto['Temp-Index-Score'];
                this.lastModified = dto['Last-Modified'];
                setInterval(function () { return _this.now(new Date()); }, 60 * 1000);
                this.ravenLastModified(dto['Raven-Last-Modified']);
                this.lastModifiedAsAgo = ko.computed(function () {
                    if (!!_this.ravenLastModified()) {
                        var lastModifiedMoment = moment(_this.ravenLastModified());
                        return lastModifiedMoment.from(_this.now());
                    }
                    return "";
                });
                this.lastModifiedFullDate = ko.computed(function () {
                    if (!!_this.ravenLastModified()) {
                        var lastModifiedMoment = moment(_this.ravenLastModified());
                        var fullTimeSinceUtc = lastModifiedMoment.utc().format("DD/MM/YYYY HH:mm (UTC)");
                        return fullTimeSinceUtc;
                    }
                    return "";
                });
                this.etag(dto['@etag']);
                for (var property in dto) {
                    if (property.toUpperCase() !== 'Raven-Entity-Name'.toUpperCase() &&
                        property.toUpperCase() !== 'Raven-Clr-Type'.toUpperCase() &&
                        property.toUpperCase() !== 'Non-Authoritative-Information'.toUpperCase() &&
                        property.toUpperCase() !== '@id'.toUpperCase() &&
                        property.toUpperCase() !== 'Temp-Index-Score'.toUpperCase() &&
                        property.toUpperCase() !== 'Last-Modified'.toUpperCase() &&
                        property.toUpperCase() !== 'Raven-Last-Modified'.toUpperCase() &&
                        property.toUpperCase() !== '@etag'.toUpperCase() &&
                        property.toUpperCase() !== 'toDto'.toUpperCase()) {
                        this.nonStandardProps = this.nonStandardProps || [];
                        this[property] = dto[property];
                        this.nonStandardProps.push(property);
                    }
                }
            }
        }
        documentMetadata.prototype.toDto = function () {
            var _this = this;
            var dto = {
                'Raven-Entity-Name': this.ravenEntityName,
                'Raven-Clr-Type': this.ravenClrType,
                'Non-Authoritative-Information': this.nonAuthoritativeInfo,
                '@id': this.id,
                'Temp-Index-Score': this.tempIndexScore,
                'Last-Modified': this.lastModified,
                'Raven-Last-Modified': this.ravenLastModified(),
                '@etag': this.etag()
            };
            if (this.nonStandardProps) {
                this.nonStandardProps.forEach(function (p) { return dto[p] = _this[p]; });
            }
            return dto;
        };
        documentMetadata.filterMetadata = function (metaDto, removedProps) {
            if (removedProps === void 0) { removedProps = null; }
            // We don't want to show certain reserved properties in the metadata text area.
            // Remove them from the DTO, restore them on save.
            var metaPropsToRemove = ["@id", "@etag", "Raven-Last-Modified"];
            for (var property in metaDto) {
                if (metaDto.hasOwnProperty(property) && _.includes(metaPropsToRemove, property)) {
                    if (metaDto[property] && removedProps) {
                        removedProps.push({ name: property, value: metaDto[property] });
                    }
                    delete metaDto[property];
                }
            }
            return metaDto;
        };
        return documentMetadata;
    }());
    return documentMetadata;
});

//# sourceMappingURL=documentMetadata.js.map
