define(["require", "exports", "common/appUrl"], function (require, exports, appUrl) {
    "use strict";
    var recentDocuments = (function () {
        function recentDocuments() {
        }
        recentDocuments.prototype.getTopRecentDocuments = function (activeDatabase, documentId) {
            var currentDbName = activeDatabase ? activeDatabase.name : null;
            var recentDocumentsForCurDb = recentDocuments.recentDocumentsInDatabases()
                .find(function (x) { return x.databaseName === currentDbName; });
            if (recentDocumentsForCurDb) {
                var value = recentDocumentsForCurDb
                    .recentDocuments()
                    .filter(function (x) { return x !== documentId; })
                    .slice(0, recentDocuments.maxRecentItems)
                    .map(function (docId) { return ({
                    id: docId,
                    href: appUrl.forEditDoc(docId, activeDatabase)
                }); });
                return value;
            }
            else {
                return [];
            }
        };
        recentDocuments.prototype.documentRemoved = function (db, docId) {
            var recentDocsForDatabase = recentDocuments.recentDocumentsInDatabases()
                .find(function (x) { return x.databaseName === db.name; });
            if (recentDocsForDatabase) {
                recentDocsForDatabase.recentDocuments.remove(docId);
            }
        };
        recentDocuments.prototype.appendRecentDocument = function (db, docId) {
            var existingRecentDocumentsStore = recentDocuments.recentDocumentsInDatabases().find(function (x) { return x.databaseName === db.name; });
            if (existingRecentDocumentsStore) {
                var recentDocs = existingRecentDocumentsStore.recentDocuments();
                _.pull(recentDocs, docId);
                recentDocs.unshift(docId);
                while (recentDocs.length > recentDocuments.maxRecentItems) {
                    recentDocs.pop();
                }
                existingRecentDocumentsStore.recentDocuments(recentDocs);
            }
            else {
                recentDocuments.recentDocumentsInDatabases.push({ databaseName: db.name, recentDocuments: ko.observableArray([docId]) });
            }
        };
        recentDocuments.prototype.getPreviousDocument = function (db) {
            var recentDocsForDatabase = recentDocuments.recentDocumentsInDatabases()
                .find(function (x) { return x.databaseName === db.name; });
            if (recentDocsForDatabase && recentDocsForDatabase.recentDocuments().length) {
                return recentDocsForDatabase.recentDocuments()[0];
            }
            return null;
        };
        return recentDocuments;
    }());
    //TODO: consider using es6 map
    recentDocuments.recentDocumentsInDatabases = ko.observableArray();
    recentDocuments.maxRecentItems = 20;
    return recentDocuments;
});

//# sourceMappingURL=recentDocuments.js.map
