define(["require", "exports", "common/appUrl"], function (require, exports, appUrl) {
    "use strict";
    var index = (function () {
        function index(dto) {
            this.isStale = ko.observable(false);
            this.lockMode = ko.observable();
            this.priority = ko.observable();
            this.state = ko.observable();
            this.filteredOut = ko.observable(false); //UI only property
            this.pausedUntilRestart = ko.observable();
            this.collections = dto.Collections;
            this.collectionNames = index.extractCollectionNames(dto.Collections);
            this.createdTimestamp = dto.CreatedTimestamp;
            this.entriesCount = dto.EntriesCount;
            this.errorsCount = dto.ErrorsCount;
            this.id = dto.Id;
            this.isStale(dto.IsStale);
            this.isInvalidIndex = dto.IsInvalidIndex;
            this.isTestIndex = dto.IsTestIndex;
            this.lastIndexingTime = dto.LastIndexingTime;
            this.lastQueryingTime = dto.LastQueryingTime;
            this.lockMode(dto.LockMode);
            this.mapAttempts = dto.MapAttempts;
            this.mapErrors = dto.MapErrors;
            this.mapSuccesses = dto.MapSuccesses;
            this.memory = dto.Memory;
            this.name = dto.Name;
            this.priority(dto.Priority);
            this.reduceAttempts = dto.ReduceAttempts;
            this.reduceErrors = dto.ReduceErrors;
            this.reduceSuccesses = dto.ReduceSuccesses;
            this.type = dto.Type;
            this.state(dto.State);
            this.initializeObservables();
        }
        index.prototype.getTypeForUI = function () {
            switch (this.type) {
                case "Map":
                    return "Map";
                case "MapReduce":
                    return "Map-Reduce";
                case "AutoMap":
                    return "Auto Map";
                case "AutoMapReduce":
                    return "Auto Map-Reduce";
                default:
                    return this.type;
            }
        };
        index.prototype.initializeObservables = function () {
            var _this = this;
            var urls = appUrl.forCurrentDatabase();
            this.queryUrl = urls.query(this.name);
            this.editUrl = urls.editIndex(this.name);
            this.isNormalPriority = ko.pureComputed(function () { return _this.priority() === "Normal"; });
            this.isLowPriority = ko.pureComputed(function () { return _this.priority() === "Low"; });
            this.isHighPriority = ko.pureComputed(function () { return _this.priority() === "High"; });
            this.isIdleState = ko.pureComputed(function () { return _this.state() === "Idle"; });
            this.isDisabledState = ko.pureComputed(function () { return _this.state() === "Disabled"; });
            this.isErrorState = ko.pureComputed(function () { return _this.state() === "Error"; });
            this.isNormalState = ko.pureComputed(function () { return _this.state() === "Normal"; });
            this.canBePaused = ko.pureComputed(function () {
                var disabled = _this.isDisabledState();
                var paused = _this.pausedUntilRestart();
                return !disabled && !paused;
            });
            this.canBeResumed = ko.pureComputed(function () {
                var disabled = _this.isDisabledState();
                var paused = _this.pausedUntilRestart();
                return !disabled && paused;
            });
            this.canBeDisabled = ko.pureComputed(function () {
                return !_this.isDisabledState();
            });
            this.canBeEnabled = ko.pureComputed(function () {
                return _this.isDisabledState();
            });
            this.isFaulty = ko.pureComputed(function () {
                var faultyType = "Faulty";
                return _this.type === faultyType;
            });
            this.badgeClass = ko.pureComputed(function () {
                if (_this.isFaulty()) {
                    return "state-danger";
                }
                if (_this.pausedUntilRestart()) {
                    return "state-warning";
                }
                if (_this.isDisabledState()) {
                    return "state-warning";
                }
                if (_this.isIdleState()) {
                    return "state-warning";
                }
                if (_this.isErrorState()) {
                    return "state-danger";
                }
                return "state-success";
            });
            this.badgeText = ko.pureComputed(function () {
                if (_this.isFaulty()) {
                    return "Faulty";
                }
                if (_this.pausedUntilRestart()) {
                    return "Paused";
                }
                if (_this.isDisabledState()) {
                    return "Disabled";
                }
                if (_this.isIdleState()) {
                    return "Idle";
                }
                if (_this.isErrorState()) {
                    return "Error";
                }
                return "Normal";
            });
        };
        index.extractCollectionNames = function (collections) {
            return collections ? Object.keys(collections) : [];
        };
        index.prototype.getGroupName = function () {
            var collections = this.collectionNames;
            if (collections && collections.length) {
                return collections.slice(0).sort(function (l, r) { return l.toLowerCase() > r.toLowerCase() ? 1 : -1; }).join(", ");
            }
            else {
                return index.DefaultIndexGroupName;
            }
        };
        return index;
    }());
    index.SideBySideIndexPrefix = "ReplacementOf/";
    index.TestIndexPrefix = "Test/";
    index.DefaultIndexGroupName = "Other";
    return index;
});

//# sourceMappingURL=index.js.map
