define(["require", "exports", "models/database/documents/collection", "models/database/documents/document", "commands/database/documents/getCollectionsStatsCommand"], function (require, exports, collection, document, getCollectionsStatsCommand) {
    "use strict";
    var indexAceAutoCompleteProvider = (function () {
        function indexAceAutoCompleteProvider(activeDatabase, editedIndex) {
            this.activeDatabase = activeDatabase;
            this.editedIndex = editedIndex;
        }
        indexAceAutoCompleteProvider.prototype.getCollectionAliasesInsideIndexText = function (session) {
            var aliases = [];
            var curAliasKey = null;
            var curAliasValuePrefix = null;
            var curAliasValueSuffix = null;
            for (var curRow = 0; curRow < session.getLength(); curRow++) {
                var curRowTokens = session.getTokens(curRow);
                for (var curTokenInRow = 0; curTokenInRow < curRowTokens.length; curTokenInRow++) {
                    if (curRowTokens[curTokenInRow].type == "from.alias") {
                        curAliasKey = curRowTokens[curTokenInRow].value.trim();
                    }
                    else if (!!curAliasKey) {
                        if (curRowTokens[curTokenInRow].type == "docs" || curRowTokens[curTokenInRow].type == "collections") {
                            curAliasValuePrefix = curRowTokens[curTokenInRow].value;
                        }
                        else if (curRowTokens[curTokenInRow].type == "collectionName") {
                            curAliasValueSuffix = curRowTokens[curTokenInRow].value;
                            aliases.push({ aliasKey: curAliasKey, aliasValuePrefix: curAliasValuePrefix.replace('.', '').trim(), aliasValueSuffix: curAliasValueSuffix.replace('.', '').trim() });
                            curAliasKey = null;
                            curAliasValuePrefix = null;
                            curAliasValueSuffix = null;
                        }
                    }
                }
            }
            return aliases;
        };
        indexAceAutoCompleteProvider.prototype.getIndexMapCollectionFieldsForAutocomplete = function (session, currentToken) {
            var deferred = $.Deferred();
            var collectionAliases = this.getCollectionAliasesInsideIndexText(session);
            // find the matching alias and get list of fields
            if (collectionAliases.length > 0) {
                var matchingAliasKeyValue = collectionAliases.find(function (x) { return x.aliasKey.replace('.', '').trim() === currentToken.value.replace('.', '').trim(); });
                if (!!matchingAliasKeyValue) {
                    // get list of fields according to it's collection's first row
                    if (matchingAliasKeyValue.aliasValuePrefix.toLowerCase() === "docs") {
                        new collection(matchingAliasKeyValue.aliasValuePrefix, this.activeDatabase)
                            .fetchDocuments(0, 1)
                            .done(function (result) {
                            if (!!result && result.totalResultCount > 0) {
                                var documentPattern = new document(result.items[0]);
                                deferred.resolve(documentPattern.getDocumentPropertyNames());
                            }
                            else {
                                deferred.reject();
                            }
                        }).fail(function () { return deferred.reject(); });
                    }
                    else {
                        deferred.reject();
                    }
                }
            }
            else {
                deferred.reject();
            }
            return deferred;
        };
        indexAceAutoCompleteProvider.prototype.getIndexMapCompleterValues = function (editor, session, pos) {
            var currentToken = session.getTokenAt(pos.row, pos.column);
            var completedToken;
            var TokenIterator = ace.require("ace/token_iterator").TokenIterator;
            var curPosIterator = new TokenIterator(editor.getSession(), pos.row, pos.column);
            var prevToken = curPosIterator.stepBackward();
            var returnedDeferred = $.Deferred();
            var suggestionsArray = [];
            // validation: if is null or it's type is represented by a string
            if (!currentToken || typeof currentToken.type == "string") {
                // if in beginning of text or in free text token
                if (!currentToken || currentToken.type === "text") {
                    suggestionsArray.push("from");
                    suggestionsArray.push("docs");
                }
                else if (!!currentToken.value && (currentToken.type === "docs" || (!!prevToken && prevToken.type === "docs"))) {
                    new getCollectionsStatsCommand(this.activeDatabase)
                        .execute()
                        .done(function (collectionsStats) {
                        var collections = collectionsStats.collections;
                        collections.forEach(function (curCollection) {
                            return suggestionsArray.push(curCollection.name);
                        });
                        returnedDeferred.resolve(suggestionsArray);
                    })
                        .fail(function () { return returnedDeferred.reject(); });
                }
                else if (currentToken.type === "collections" || currentToken.type === "collectionName") {
                    if (currentToken.type === "collections") {
                        completedToken = currentToken;
                    }
                    else {
                        completedToken = prevToken;
                    }
                    this.getIndexMapCollectionFieldsForAutocomplete(session, completedToken)
                        .done(function (x) { return returnedDeferred.resolve(x); })
                        .fail(function () { return returnedDeferred.reject(); });
                }
                else if (currentToken.type === "data.prefix" || currentToken.type === "data.suffix") {
                    if (currentToken.type === "data.prefix") {
                        completedToken = currentToken;
                    }
                    else {
                        completedToken = prevToken;
                    }
                    var firstToken = session.getTokenAt(0, 0);
                    // treat a "from [foo] in [bar] type of index syntax
                    if (firstToken.value === "from") {
                        var aliases = this.getCollectionAliasesInsideIndexText(session);
                        this.getIndexMapCollectionFieldsForAutocomplete(session, completedToken)
                            .done(function (x) { return returnedDeferred.resolve(x); })
                            .fail(function () { return returnedDeferred.reject(); });
                    }
                    else {
                        returnedDeferred.resolve(["Methodical Syntax Not Supported"]);
                    }
                }
                else {
                    returnedDeferred.reject();
                }
            }
            else {
                returnedDeferred.reject();
            }
            return returnedDeferred;
        };
        indexAceAutoCompleteProvider.prototype.indexMapCompleter = function (editor, session, pos, prefix, callback) {
            this.getIndexMapCompleterValues(editor, session, pos)
                .done(function (x) {
                callback(null, x.map(function (val) {
                    return { name: val, value: val, score: 100, meta: "suggestion" };
                }));
            })
                .fail(function () {
                callback([{ error: "notext" }], null);
            });
        };
        indexAceAutoCompleteProvider.prototype.getIndexReduceCompleterValues = function () {
            var firstMapSrting = this.editedIndex().maps()[0]();
            var dotPrefixes = firstMapSrting.match(/[.]\w*/g);
            var equalPrefixes = firstMapSrting.match(/\w*\s*=\s*/g);
            var autoCompletes = [];
            if (!!dotPrefixes) {
                dotPrefixes.forEach(function (curPrefix) {
                    autoCompletes.push(curPrefix.replace(".", "").trim());
                });
            }
            if (!!equalPrefixes) {
                equalPrefixes.forEach(function (curPrefix) {
                    autoCompletes.push(curPrefix.replace("=", "").trim());
                });
            }
            return autoCompletes;
        };
        indexAceAutoCompleteProvider.prototype.indexReduceCompleter = function (editor, session, pos, prefix, callback) {
            var autoCompletes = this.getIndexReduceCompleterValues();
            callback(null, autoCompletes.map(function (curField) {
                return { name: curField, value: curField, score: 100, meta: "suggestion" };
            }));
        };
        return indexAceAutoCompleteProvider;
    }());
    return indexAceAutoCompleteProvider;
});

//# sourceMappingURL=indexAceAutoCompleteProvider.js.map
