/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var spatialIndexField = (function () {
        function spatialIndexField(fieldName, dto) {
            var _this = this;
            this.name = ko.observable().extend({ required: true });
            this.type = ko.observable().extend({ required: true });
            this.strategy = ko.observable().extend({ required: true });
            this.minX = ko.observable().extend({ required: true });
            this.maxX = ko.observable().extend({ required: true });
            this.circleRadiusUnits = ko.observable().extend({ required: true });
            this.maxTreeLevel = ko.observable().extend({ required: true });
            this.minY = ko.observable().extend({ required: true });
            this.maxY = ko.observable().extend({ required: true });
            this.units = ko.observable();
            this.availableStrategies = ko.observableArray();
            this.name(fieldName);
            this.type(dto.Type);
            this.strategy(dto.Strategy);
            this.maxTreeLevel(dto.MaxTreeLevel);
            this.minX(dto.MinX);
            this.maxX(dto.MaxX);
            this.minY(dto.MinY);
            this.maxY(dto.MaxY);
            this.units(dto.Units);
            this.availableStrategies(this.getAvailableStrategies());
            this.canSpecifyUnits = ko.computed(function () { return _this.type() === spatialIndexField.typeGeo; });
            this.canSpecifyTreeLevel = ko.computed(function () { return _this.strategy() !== spatialIndexField.strategyBounding; });
            this.precision = ko.computed(function () { return _this.getPrecisionString(); });
            this.canSpecifyCoordinates = ko.computed(function () { return _this.type() === spatialIndexField.typeCart; });
            this.type.subscribe(function (newType) { return _this.resetCoordinates(); });
            this.type.subscribe(function () { return _this.availableStrategies(_this.getAvailableStrategies()); });
            this.strategy.subscribe(function (newStrategy) { return _this.updateMaxTreeLevelFromStrategy(newStrategy); });
        }
        spatialIndexField.prototype.toDto = function () {
            return {
                Type: this.type(),
                Strategy: this.strategy(),
                MaxTreeLevel: this.maxTreeLevel(),
                MinX: this.minX(),
                MaxX: this.maxX(),
                MinY: this.minY(),
                MaxY: this.maxY(),
                Units: this.units()
            };
        };
        spatialIndexField.empty = function () {
            var dto = {
                Type: spatialIndexField.typeGeo,
                MaxTreeLevel: 9,
                MinX: -180,
                MaxX: 180,
                MinY: -90,
                MaxY: 90,
                Strategy: spatialIndexField.strategyGeo,
                Units: "Kilometers"
            };
            return new spatialIndexField("", dto);
        };
        spatialIndexField.prototype.resetCoordinates = function () {
            this.minX(-180);
            this.maxX(180);
            this.minY(-90);
            this.maxY(90);
        };
        spatialIndexField.prototype.getPrecisionString = function () {
            var strategy = this.strategy();
            var minX = this.minX();
            var maxX = this.maxX();
            var minY = this.minY();
            var maxY = this.maxY();
            var maxTreeLevel = this.maxTreeLevel();
            var type = this.type();
            var units = this.units();
            if (strategy === spatialIndexField.strategyBounding) {
                return "";
            }
            var x = maxX - minX;
            var y = maxY - minY;
            for (var i = 0; i < maxTreeLevel; i++) {
                if (strategy === spatialIndexField.strategyGeo) {
                    if (i % 2 == 0) {
                        x /= 8;
                        y /= 4;
                    }
                    else {
                        x /= 4;
                        y /= 8;
                    }
                }
                else if (strategy === spatialIndexField.strategyQuad) {
                    x /= 2;
                    y /= 2;
                }
            }
            if (type === spatialIndexField.typeGeo) {
                var earthMeanRadiusKm = 6371.0087714;
                var milesToKm = 1.60934;
                var factor = (earthMeanRadiusKm * Math.PI * 2) / 360;
                x = x * factor;
                y = y * factor;
                if (units === "Miles") {
                    x /= milesToKm;
                    y /= milesToKm;
                }
                return "Precision at equator; X: " + x.toFixed(6) + ", Y: " + y.toFixed(6) + " " + units.toLowerCase();
            }
            else {
                return "Precision; X: " + x.toFixed(6) + ", Y: " + y.toFixed(6);
            }
        };
        spatialIndexField.prototype.getAvailableStrategies = function () {
            if (this.type() === spatialIndexField.typeGeo) {
                return [spatialIndexField.strategyGeo, spatialIndexField.strategyQuad, spatialIndexField.strategyBounding];
            }
            else {
                return [spatialIndexField.strategyQuad, spatialIndexField.strategyBounding];
            }
        };
        spatialIndexField.prototype.updateMaxTreeLevelFromStrategy = function (strategy) {
            if (strategy === spatialIndexField.strategyGeo) {
                this.maxTreeLevel(9);
            }
            else if (strategy === spatialIndexField.strategyQuad) {
                this.maxTreeLevel(23);
            }
        };
        return spatialIndexField;
    }());
    spatialIndexField.strategyGeo = "GeohashPrefixTree";
    spatialIndexField.strategyBounding = "BoundingBox";
    spatialIndexField.strategyQuad = "QuadPrefixTree";
    spatialIndexField.typeGeo = "Geography";
    spatialIndexField.typeCart = "Cartesian";
    return spatialIndexField;
});

//# sourceMappingURL=spatialIndexField.js.map
