/// <reference path="../../../../typings/tsd.d.ts" />
define(["require", "exports", "common/appUrl"], function (require, exports, appUrl) {
    "use strict";
    var transformer = (function () {
        function transformer(dto) {
            var _this = this;
            this.name = ko.observable();
            this.transformResults = ko.observable();
            this.lockMode = ko.observable();
            this.originalName = ko.observable();
            this.temporary = ko.observable();
            this.nameChanged = ko.pureComputed(function () { return _this.name() !== _this.originalName(); });
            this.transformerId = ko.observable();
            this.isLocked = ko.pureComputed(function () { return _this.lockMode() === "LockedIgnore"; });
            this.filteredOut = ko.observable(false); //UI only property
            this.originalName(dto.Name);
            this.name(dto.Name);
            this.lockMode(dto.LockMode);
            this.temporary(dto.Temporary);
            this.transformerId(dto.TransfomerId);
            this.transformResults(dto.TransformResults);
            this.initializeObservables();
        }
        transformer.prototype.initializeObservables = function () {
            var urls = appUrl.forCurrentDatabase();
            this.editUrl = urls.editTransformer(this.name());
        };
        transformer.prototype.toDto = function () {
            return {
                LockMode: this.lockMode(),
                Name: this.name(),
                Temporary: this.temporary(),
                TransfomerId: this.transformerId(),
                TransformResults: this.transformResults()
            };
        };
        transformer.prototype.extractInputs = function () {
            if (this.transformResults()) {
                var matcher = /(Query|Parameter)\(["'].*?["']\)/g;
                var defaultMatcher = /(Query|Parameter)OrDefault\(["'].*?["'],\s+["'].*?["']\)/g;
                var parameters = this.transformResults().match(matcher);
                var parametersWithDefault = this.transformResults().match(defaultMatcher);
                var results_1 = [];
                if (parameters) {
                    parameters.forEach(function (value) { return results_1.push({
                        name: value.substring(value.indexOf("(") + 2, value.length - 2),
                        hasDefault: false
                    }); });
                }
                if (parametersWithDefault) {
                    parametersWithDefault.forEach(function (value) { return results_1.push({
                        name: value.substring(value.indexOf("(") + 2, value.indexOf(",") - 1),
                        hasDefault: true
                    }); });
                }
                return results_1;
            }
            return [];
        };
        transformer.empty = function () {
            return new transformer({
                LockMode: "Unlock",
                Name: "",
                TransformResults: ""
            });
        };
        return transformer;
    }());
    return transformer;
});

//# sourceMappingURL=transformer.js.map
