/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var querySort = (function () {
        function querySort() {
            var _this = this;
            this.fieldName = ko.observable();
            this.isRange = ko.observable(false);
            this.isAscending = ko.observable(true);
            this.fieldNameOrDefault = ko.computed(function () { return _this.fieldName() ? _this.fieldName() : "Select a field"; });
        }
        querySort.prototype.toggleAscending = function () {
            this.isAscending.toggle();
        };
        querySort.prototype.toggleRange = function () {
            this.isRange.toggle();
        };
        querySort.prototype.toQuerySortString = function () {
            var querySortString;
            if (this.isRange()) {
                if (this.isAscending()) {
                    querySortString = this.fieldName() + querySort.rangeIndicator; //ascending range
                }
                else {
                    querySortString = "-" + this.fieldName() + querySort.rangeIndicator; //descending range
                }
            }
            else {
                if (this.isAscending()) {
                    querySortString = this.fieldName(); // ascending
                }
                else {
                    querySortString = "-" + this.fieldName(); // descending
                }
            }
            return querySortString;
        };
        querySort.prototype.toHumanizedString = function () {
            var str;
            if (this.isRange()) {
                if (this.isAscending()) {
                    str = this.fieldName() + " range"; //ascending range
                }
                else {
                    str = this.fieldName() + " range descending"; //descending range
                }
            }
            else {
                if (this.isAscending()) {
                    str = this.fieldName(); // ascending
                }
                else {
                    str = this.fieldName() + " descending"; // descending
                }
            }
            return str;
        };
        querySort.fromQuerySortString = function (querySortText) {
            var isDescending = querySortText.slice(0, 1) === "-";
            var isRange = querySortText.length > querySort.rangeIndicator.length
                && querySortText.substr(querySortText.length - querySort.rangeIndicator.length) === querySort.rangeIndicator;
            var sortField = querySort.getSortField(querySortText, isDescending, isRange);
            var q = new querySort();
            q.isAscending(!isDescending);
            q.isRange(isRange);
            q.fieldName(sortField);
            return q;
        };
        querySort.getSortField = function (querySortText, isDescending, isRange) {
            var sortField;
            if (isRange && isDescending) {
                sortField = querySortText.substr(1, querySortText.length - 1 - querySort.rangeIndicator.length);
            }
            else if (isRange && !isDescending) {
                sortField = querySortText.substr(0, querySortText.length - 1 - querySort.rangeIndicator.length);
            }
            else if (!isRange && isDescending) {
                sortField = querySortText.substr(1, querySortText.length - 1);
            }
            else {
                sortField = querySortText;
            }
            return sortField;
        };
        return querySort;
    }());
    querySort.rangeIndicator = "_Range";
    return querySort;
});

//# sourceMappingURL=querySort.js.map
