define(["require", "exports", "models/database/replication/replicationDestination"], function (require, exports, replicationDestination) {
    "use strict";
    var replicationsSetup = (function () {
        function replicationsSetup(dto) {
            var _this = this;
            this.source = ko.observable();
            this.destinations = ko.observableArray();
            this.clientFailoverBehaviour = ko.observable(null);
            this.requestTimeSlaThreshold = ko.observable(null);
            this.readFromAllAllowWriteToSecondaries = ko.computed(function () {
                var behaviour = _this.clientFailoverBehaviour();
                if (behaviour == null) {
                    return false;
                }
                var tokens = behaviour.split(",").map(function (x) { return x.trim(); });
                return _.includes(tokens, "ReadFromAllServers") && _.includes(tokens, "AllowReadsFromSecondariesAndWritesToSecondaries");
            });
            this.readFromAllButSwitchWhenRequestTimeSlaThresholdIsReached = ko.computed(function () {
                var behaviour = _this.clientFailoverBehaviour();
                if (behaviour == null) {
                    return false;
                }
                var tokens = behaviour.split(",").map(function (x) { return x.trim(); });
                return _.includes(tokens, "ReadFromAllServers") && _.includes(tokens, "AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached");
            });
            this.source(dto.Source);
            this.destinations(dto.Destinations.map(function (d) { return new replicationDestination(d); }));
            var clientConfiguration = dto.ClientConfiguration;
            if (clientConfiguration) {
                if (clientConfiguration.FailoverBehavior) {
                    this.clientFailoverBehaviour(clientConfiguration.FailoverBehavior);
                }
            }
            this.showRequestTimeSlaThreshold = ko.computed(function () {
                return _this.clientFailoverBehaviour() && _this.clientFailoverBehaviour().includes("AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached");
            });
            this.clientFailoverBehaviour.subscribe(function (newValue) {
                if (!newValue.includes('AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached')) {
                    _this.requestTimeSlaThreshold(undefined);
                }
                else if (!_this.requestTimeSlaThreshold()) {
                    _this.requestTimeSlaThreshold(100);
                }
            });
            this.hasAnyReplicationDestination = ko.computed(function () { return _this.destinations().filter(function (x) { return !x.enableReplicateOnlyFromCollections(); }).length > 0; });
        }
        replicationsSetup.prototype.toDto = function (filterLocal) {
            if (filterLocal === void 0) { filterLocal = true; }
            var dto = {
                Destinations: this.destinations().map(function (dest) { return dest.toDto(); }),
                Source: this.source()
            };
            dto.ClientConfiguration = {
                RequestTimeSlaThresholdInMilliseconds: undefined,
                FailoverBehavior: undefined
            };
            if (this.clientFailoverBehaviour()) {
                dto.ClientConfiguration.FailoverBehavior = this.clientFailoverBehaviour();
            }
            if (this.showRequestTimeSlaThreshold()) {
                dto.ClientConfiguration.RequestTimeSlaThresholdInMilliseconds = this.requestTimeSlaThreshold();
            }
            return dto;
        };
        replicationsSetup.prototype.clear = function () {
            this.destinations.removeAll();
            this.clientFailoverBehaviour(null);
            this.source(null);
        };
        return replicationsSetup;
    }());
    return replicationsSetup;
});

//# sourceMappingURL=replicationsSetup.js.map
