define(["require", "exports", "models/database/cluster/nodeConnectionInfo"], function (require, exports, nodeConnectionInfo) {
    "use strict";
    var topology = (function () {
        function topology(dto) {
            var _this = this;
            this.currentLeader = ko.observable();
            this.currentTerm = ko.observable();
            this.state = ko.observable();
            this.commitIndex = ko.observable();
            this.allNodes = ko.observableArray();
            this.allVotingNodes = ko.observableArray();
            this.promotableNodes = ko.observableArray();
            this.nonVotingNodes = ko.observableArray();
            this.topologyId = ko.observable();
            this.clusterMode = ko.computed(function () {
                var voting = _this.allVotingNodes().length;
                var promatable = _this.promotableNodes().length;
                var nonVoting = _this.nonVotingNodes().length;
                return voting + promatable + nonVoting > 0;
            });
            this.currentLeader(dto.CurrentLeader);
            this.currentTerm(dto.CurrentTerm);
            this.state(dto.State);
            this.commitIndex(dto.CommitIndex);
            dto.AllVotingNodes.map(function (n) {
                var nci = new nodeConnectionInfo(n);
                nci.state("voting");
                _this.allVotingNodes.push(nci);
                _this.allNodes.push(nci);
            });
            dto.PromotableNodes.map(function (n) {
                var nci = new nodeConnectionInfo(n);
                nci.state("promotable");
                _this.promotableNodes.push(nci);
                _this.allNodes.push(nci);
            });
            dto.NonVotingNodes.map(function (n) {
                var nci = new nodeConnectionInfo(n);
                nci.state("non-voting");
                _this.nonVotingNodes.push(nci);
                _this.allNodes.push(nci);
            });
            var leader = this.currentLeader();
            if (leader) {
                var leaderNode = this.allNodes().find(function (n) { return n.name() === leader; });
                if (leaderNode) {
                    leaderNode.state("leader");
                }
            }
            this.topologyId(dto.TopologyId);
        }
        return topology;
    }());
    return topology;
});

//# sourceMappingURL=topology.js.map
