define(["require", "exports", "durandal/app", "commands/database/sqlReplication/testSqlConnectionCommand"], function (require, exports, app, testSqlConnectionCommand) {
    "use strict";
    var predefinedSqlConnection = (function () {
        function predefinedSqlConnection(name, dto) {
            this.name = ko.observable();
            this.factoryName = ko.observable();
            this.connectionString = ko.observable();
            this.connectionTestState = ko.observable(this.CONNECTION_STATE_STAND_BY);
            this.CONNECTION_STATE_STAND_BY = "stand-by";
            this.CONNECTION_STATE_CONNECTING = "connecting";
            this.CONNECTION_STATE_CONNECTED = "connected";
            this.name(name);
            this.factoryName(dto.FactoryName);
            this.connectionString(dto.ConnectionString);
            this.connectionTestState = ko.observable(this.CONNECTION_STATE_STAND_BY);
        }
        predefinedSqlConnection.empty = function () {
            return new predefinedSqlConnection("", {
                FactoryName: "",
                ConnectionString: ""
            });
        };
        predefinedSqlConnection.prototype.toDto = function () {
            return {
                FactoryName: this.factoryName(),
                ConnectionString: this.connectionString()
            };
        };
        predefinedSqlConnection.prototype.testConnection = function (dbObservable) {
            var _this = this;
            this.connectionTestState(this.CONNECTION_STATE_CONNECTING);
            new testSqlConnectionCommand(dbObservable(), this.factoryName(), this.connectionString())
                .execute()
                .done(function () {
                _this.connectionTestState(_this.CONNECTION_STATE_CONNECTED);
                app.showBootstrapMessage("Connection " + _this.name() + " is valid", "SQL Connection test");
            })
                .fail(function (request, status, error) {
                var errorText = !!request.responseJSON ? !!request.responseJSON.Exception ? request.responseJSON.Exception.Message : error : error;
                app.showBootstrapMessage("Connection " + _this.name() + " is not valid, error: " + errorText, "SQL Connection test");
            }).
                always(function () { return setTimeout(_this.connectionTestState, 500, _this.CONNECTION_STATE_STAND_BY); });
        };
        return predefinedSqlConnection;
    }());
    return predefinedSqlConnection;
});

//# sourceMappingURL=predefinedSqlConnection.js.map
