var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "models/database/sqlReplication/sqlReplicationTable", "models/database/documents/document", "models/database/documents/documentMetadata", "common/bindingHelpers/aceEditorBindingHandler"], function (require, exports, sqlReplicationTable, document, documentMetadata, aceEditorBindingHandler) {
    "use strict";
    var sqlReplication = (function (_super) {
        __extends(sqlReplication, _super);
        function sqlReplication(dto) {
            var _this = _super.call(this, dto) || this;
            _this.CONNECTION_STRING = "Connection String";
            _this.PREDEFINED_CONNECTION_STRING_NAME = "Predefined Connection String Name";
            _this.CONNECTION_STRING_NAME = "Connection String Name";
            _this.CONNECTION_STRING_SETTING_NAME = "Connection String Setting Name";
            _this.availableConnectionStringTypes = [
                _this.PREDEFINED_CONNECTION_STRING_NAME,
                _this.CONNECTION_STRING,
                _this.CONNECTION_STRING_NAME,
                _this.CONNECTION_STRING_SETTING_NAME
            ];
            _this.name = ko.observable().extend({ required: true });
            _this.disabled = ko.observable().extend({ required: true });
            _this.connectionStringType = ko.observable().extend({ required: true });
            _this.connectionStringValue = ko.observable(null).extend({ required: true });
            _this.collection = ko.observable("");
            _this.parameterizeDeletesDisabled = ko.observable(false).extend({ required: true });
            _this.forceSqlServerQueryRecompile = ko.observable(false);
            _this.quoteTables = ko.observable(true);
            _this.sqlReplicationTables = ko.observableArray().extend({ required: true });
            _this.script = ko.observable("").extend({ required: true });
            _this.connectionString = ko.observable(null);
            _this.connectionStringName = ko.observable(null);
            _this.connectionStringSettingName = ko.observable(null);
            _this.collections = ko.observableArray();
            _this.isVisible = ko.observable(true);
            _this.showReplicationConfiguration = ko.observable(false);
            _this.hasAnyInsertOnlyOption = ko.computed(function () {
                var hasAny = false;
                _this.sqlReplicationTables().forEach(function (s) {
                    // don't use return here to register all deps in knockout
                    if (s.insertOnly()) {
                        hasAny = true;
                    }
                });
                return hasAny;
            });
            _this.name(dto.Name);
            _this.disabled(dto.Disabled);
            _this.collection(dto.Collection);
            _this.parameterizeDeletesDisabled(dto.ParameterizeDeletesDisabled);
            _this.sqlReplicationTables(dto.SqlReplicationTables.map(function (tab) { return new sqlReplicationTable(tab); }));
            _this.script(dto.Script);
            _this.forceSqlServerQueryRecompile(!!dto.ForceSqlServerQueryRecompile ? dto.ForceSqlServerQueryRecompile : false);
            _this.quoteTables(("PerformTableQuatation" in dto) ? dto.QuoteTables : ("QuoteTables" in dto ? dto.QuoteTables : true));
            //TODO: this.setupConnectionString(dto);
            _this.connectionStringType(_this.PREDEFINED_CONNECTION_STRING_NAME); //TODO: delete 
            _this.connectionStringValue(dto.ConnectionStringName); //TODO: delete
            _this.metadata = new documentMetadata(dto["@metadata"]);
            _this.connectionStringSourceFieldName = ko.computed(function () {
                if (_this.connectionStringType() == _this.CONNECTION_STRING) {
                    return "Connection String Text";
                }
                else if (_this.connectionStringType() == _this.PREDEFINED_CONNECTION_STRING_NAME) {
                    return "Predefined connection string name";
                }
                else if (_this.connectionStringType() == _this.CONNECTION_STRING_NAME) {
                    return "Setting name in local machine configuration";
                }
                else {
                    return "Setting name in memory/remote configuration";
                }
            });
            _this.searchResults = ko.computed(function () {
                var collection = _this.collection();
                return _this.collections().filter(function (name) { return name.toLowerCase().indexOf(collection.toLowerCase()) > -1; });
            });
            _this.script.subscribe(function (newValue) {
                var message = "";
                var currentEditor = aceEditorBindingHandler.currentEditor;
                var textarea = $(currentEditor.container).find("textarea")[0];
                if (newValue === "") {
                    message = "Please fill out this field.";
                }
                textarea.setCustomValidity(message);
                setTimeout(function () {
                    var annotations = currentEditor.getSession().getAnnotations();
                    var isErrorExists = false;
                    for (var i = 0; i < annotations.length; i++) {
                        var annotationType = annotations[i].type;
                        if (annotationType === "error" || annotationType === "warning") {
                            isErrorExists = true;
                            break;
                        }
                    }
                    if (isErrorExists) {
                        message = "The script isn't a javascript legal expression!";
                        textarea.setCustomValidity(message);
                    }
                }, 700);
            });
            return _this;
        }
        /*TODO private setupConnectionString(dto: sqlReplicationDto) {
            
            if (dto.ConnectionStringName) {
                this.connectionStringType(this.CONNECTION_STRING_NAME);
                this.connectionStringValue(dto.ConnectionStringName);
            } else if (dto.ConnectionStringSettingName) {
                this.connectionStringType(this.CONNECTION_STRING_SETTING_NAME);
                this.connectionStringValue(dto.ConnectionStringSettingName);
            } else if (dto.ConnectionString){
                this.connectionStringType(this.CONNECTION_STRING);
                this.connectionStringValue(dto.ConnectionString);
            }
            else {
                this.connectionStringType(this.PREDEFINED_CONNECTION_STRING_NAME);
                this.connectionStringValue(dto.PredefinedConnectionStringSettingName);
            }
        }*/
        sqlReplication.prototype.setConnectionStringType = function (strType) {
            this.connectionStringType(strType);
        };
        sqlReplication.empty = function () {
            return new sqlReplication({
                Name: "",
                Disabled: true,
                ParameterizeDeletesDisabled: false,
                Collection: "",
                Script: "",
                Id: null,
                FactoryName: null,
                ConnectionString: null,
                PredefinedConnectionStringSettingName: null,
                ConnectionStringName: null,
                ConnectionStringSettingName: null,
                SqlReplicationTables: [sqlReplicationTable.empty().toDto()],
                ForceSqlServerQueryRecompile: false,
                QuoteTables: true
            });
        };
        sqlReplication.prototype.toDto = function () {
            //TODO:var meta = this.__metadata.toDto();
            //TODO: meta["@id"] = "Raven/SqlReplication/Configuration/" + this.name();
            return {
                //TODO: '@metadata': meta,
                Id: null,
                Name: this.name(),
                Disabled: this.disabled(),
                ParameterizeDeletesDisabled: this.parameterizeDeletesDisabled(),
                Collection: this.collection(),
                Script: this.script(),
                //TODO: FactoryName: this.factoryName(),
                //TODO: ConnectionString: this.prepareConnectionString(this.CONNECTION_STRING),
                //TODO: PredefinedConnectionStringSettingName: this.prepareConnectionString(this.PREDEFINED_CONNECTION_STRING_NAME),
                ConnectionStringName: this.prepareConnectionString(this.PREDEFINED_CONNECTION_STRING_NAME),
                //TODO: ConnectionStringSettingName: this.prepareConnectionString(this.CONNECTION_STRING_SETTING_NAME),
                ForceSqlServerQueryRecompile: this.forceSqlServerQueryRecompile(),
                QuoteTables: this.quoteTables(),
                SqlReplicationTables: this.sqlReplicationTables().map(function (tab) { return tab.toDto(); })
            };
        };
        sqlReplication.prototype.prepareConnectionString = function (expectedType) {
            return ((this.connectionStringType() === expectedType) ? this.connectionStringValue() : null);
        };
        sqlReplication.prototype.enable = function () {
            this.disabled(false);
        };
        sqlReplication.prototype.disable = function () {
            this.disabled(true);
        };
        sqlReplication.prototype.enableParameterizeDeletes = function () {
            this.parameterizeDeletesDisabled(false);
        };
        sqlReplication.prototype.disableParameterizeDeletes = function () {
            this.parameterizeDeletesDisabled(true);
        };
        sqlReplication.prototype.addNewTable = function () {
            this.sqlReplicationTables.push(sqlReplicationTable.empty());
        };
        sqlReplication.prototype.removeTable = function (table) {
            this.sqlReplicationTables.remove(table);
        };
        /* TODO
        setIdFromName() {
            this.__metadata.id = "Raven/SqlReplication/Configuration/" + this.name();
        }*/
        sqlReplication.prototype.saveNewCollectionName = function (newRavenEntityName) {
            this.collection(newRavenEntityName);
        };
        sqlReplication.prototype.isSqlServerKindOfFactory = function (factoryName) {
            if (factoryName == "System.Data.SqlClient" || factoryName == "System.Data.SqlServerCe.4.0" || factoryName == "System.Data.SqlServerCe.3.5") {
                return true;
            }
            return false;
        };
        return sqlReplication;
    }(document));
    return sqlReplication;
});

//# sourceMappingURL=sqlReplication.js.map
