define(["require", "exports", "models/database/stats/indexStatistics"], function (require, exports, indexStatistics) {
    "use strict";
    var statistics = (function () {
        function statistics(dbStats, indexStats) {
            // The observable indexes array, ordered by type
            this.indexesByType = ko.observableArray();
            this.dataBaseId = dbStats.DatabaseId;
            this.lastDocEtag = dbStats.LastDocEtag;
            this.countOfIndexes = dbStats.CountOfIndexes.toLocaleString();
            this.countOfDocuments = dbStats.CountOfDocuments.toLocaleString();
            this.countOfTransformers = dbStats.CountOfTransformers.toLocaleString();
            this.is64Bit = dbStats.Is64Bit;
            // 1. Create the array with all indexes that we got from the endpoint
            var allIndexes = indexStats.map(function (x) { return new indexStatistics(x); });
            // 2. Create an array where indexes are ordered by type
            var indexesByTypeTemp = Array();
            allIndexes.forEach(function (index) {
                var existingEntry = indexesByTypeTemp.find(function (x) { return x.indexType === index.indexType; });
                if (!existingEntry) {
                    // A new type encountered
                    var newType = new indexesWithType(index.indexType);
                    newType.add(index);
                    indexesByTypeTemp.push(newType);
                }
                else {
                    // Type already exists, only add the index
                    existingEntry.add(index);
                }
            });
            // 3. Sort by index name & type
            indexesByTypeTemp.forEach(function (x) { x.indexes.sort(function (a, b) { return a.indexName > b.indexName ? 1 : -1; }); });
            indexesByTypeTemp.sort(function (a, b) { return a.indexType > b.indexType ? 1 : -1; });
            // 4. Update the observable array 
            this.indexesByType(indexesByTypeTemp);
        }
        return statistics;
    }());
    var indexesWithType = (function () {
        function indexesWithType(type) {
            this.indexType = type;
            this.indexes = [];
        }
        indexesWithType.prototype.add = function (index) {
            this.indexes.push(index);
        };
        return indexesWithType;
    }());
    return statistics;
});

//# sourceMappingURL=statistics.js.map
