/// <reference path="../../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var synchronizationConfig = (function () {
        function synchronizationConfig(dto) {
            this.fileConflictResolution = ko.observable();
            this.maxNumberOfSynchronizationsPerDestination = ko.observable();
            this.synchronizationLockTimeout = ko.observable();
            this.synchronizationLockTimeoutUnit = ko.observable();
            this.availableIntervalUnits = [synchronizationConfig.TU_MINUTES, synchronizationConfig.TU_HOURS, synchronizationConfig.TU_DAYS];
            this.fileConflictResolution(dto.FileConflictResolution);
            this.maxNumberOfSynchronizationsPerDestination(dto.MaxNumberOfSynchronizationsPerDestination);
            var timeout = this.prepareInterval(dto.SynchronizationLockTimeoutMiliseconds);
            this.synchronizationLockTimeout(timeout[0]);
            this.synchronizationLockTimeoutUnit(timeout[1]);
        }
        synchronizationConfig.empty = function () {
            return new synchronizationConfig({
                FileConflictResolution: "None",
                MaxNumberOfSynchronizationsPerDestination: 5,
                SynchronizationLockTimeoutMiliseconds: 10 * 60 * 1000
            });
        };
        synchronizationConfig.prototype.toDto = function () {
            return {
                FileConflictResolution: this.fileConflictResolution(),
                MaxNumberOfSynchronizationsPerDestination: this.maxNumberOfSynchronizationsPerDestination(),
                SynchronizationLockTimeoutMiliseconds: this.convertToMilliseconds(this.synchronizationLockTimeout(), this.synchronizationLockTimeoutUnit())
            };
        };
        synchronizationConfig.prototype.convertToMilliseconds = function (value, unit) {
            if (value && unit) {
                switch (unit) {
                    case synchronizationConfig.TU_MINUTES:
                        return value * 1000 * 60;
                    case synchronizationConfig.TU_HOURS:
                        return value * 1000 * 60 * 60;
                    case synchronizationConfig.TU_DAYS:
                        return value * 1000 * 60 * 60 * 24;
                }
            }
            return null;
        };
        synchronizationConfig.prototype.prepareInterval = function (milliseconds) {
            if (milliseconds) {
                var seconds = milliseconds / 1000;
                var minutes = seconds / 60;
                var hours = minutes / 60;
                if (this.isValidTimeValue(hours)) {
                    var days = hours / 24;
                    if (this.isValidTimeValue(days)) {
                        return [days, synchronizationConfig.TU_DAYS];
                    }
                    return [hours, synchronizationConfig.TU_HOURS];
                }
                return [minutes, synchronizationConfig.TU_MINUTES];
            }
            return [0, synchronizationConfig.TU_MINUTES];
        };
        synchronizationConfig.prototype.isValidTimeValue = function (value) {
            return value >= 1 && value % 1 === 0;
        };
        return synchronizationConfig;
    }());
    synchronizationConfig.TU_MINUTES = "minutes";
    synchronizationConfig.TU_HOURS = "hours";
    synchronizationConfig.TU_DAYS = "days";
    return synchronizationConfig;
});

//# sourceMappingURL=synchronizationConfig.js.map
