/// <reference path="../../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var synchronizationDestination = (function () {
        function synchronizationDestination(dto) {
            var _this = this;
            this.url = ko.observable();
            this.username = ko.observable();
            this.password = ko.observable();
            this.domain = ko.observable();
            this.apiKey = ko.observable();
            this.filesystem = ko.observable();
            this.disabled = ko.observable();
            this.name = ko.computed(function () {
                if (_this.url() && _this.filesystem()) {
                    return _this.filesystem() + " on " + _this.url();
                }
                else if (_this.url()) {
                    return _this.url();
                }
                else if (_this.filesystem()) {
                    return _this.filesystem();
                }
                return "[empty]";
            });
            this.isValid = ko.computed(function () { return _this.url() != null && _this.url().length > 0; });
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
            this.url(dto.ServerUrl);
            this.username(dto.Username);
            this.password(dto.Password);
            this.domain(dto.Domain);
            this.apiKey(dto.ApiKey);
            this.disabled(!dto.Enabled);
            this.filesystem(dto.FileSystem);
            if (this.username()) {
                this.isUserCredentials(true);
            }
            else if (this.apiKey()) {
                this.isApiKeyCredentials(true);
            }
        }
        synchronizationDestination.prototype.clearApiKeyCredentials = function () {
            this.apiKey(null);
        };
        synchronizationDestination.prototype.clearUserCredentials = function () {
            this.username(null);
            this.password(null);
            this.domain(null);
        };
        synchronizationDestination.prototype.useUserCredentials = function () {
            this.isUserCredentials(true);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
        };
        synchronizationDestination.prototype.useApiKeyCredentials = function () {
            this.isApiKeyCredentials(true);
            this.isUserCredentials(false);
            this.clearUserCredentials();
        };
        synchronizationDestination.prototype.useNoCredentials = function () {
            this.isUserCredentials(false);
            this.isApiKeyCredentials(false);
            this.clearUserCredentials();
            this.clearApiKeyCredentials();
        };
        synchronizationDestination.empty = function (fileSystemName) {
            return new synchronizationDestination({
                ServerUrl: location.protocol + "//" + location.host,
                Username: null,
                Password: null,
                Domain: null,
                ApiKey: null,
                FileSystem: fileSystemName,
                Enabled: false
            });
        };
        synchronizationDestination.prototype.enable = function () {
            this.disabled(false);
        };
        synchronizationDestination.prototype.disable = function () {
            this.disabled(true);
        };
        synchronizationDestination.prototype.toDto = function () {
            return {
                ServerUrl: this.prepareUrl(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey(),
                FileSystem: this.filesystem(),
                Enabled: !this.disabled(),
            };
        };
        synchronizationDestination.prototype.prepareUrl = function () {
            var url = this.url();
            if (url && url.charAt(url.length - 1) === "/") {
                url = url.substring(0, url.length - 1);
            }
            return url;
        };
        return synchronizationDestination;
    }());
    return synchronizationDestination;
});

//# sourceMappingURL=synchronizationDestination.js.map
