/// <reference path="../../../../typings/tsd.d.ts"/>
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "models/resources/creation/resourceCreationModel", "configuration"], function (require, exports, resourceCreationModel, configuration) {
    "use strict";
    var databaseCreationModel = (function (_super) {
        __extends(databaseCreationModel, _super);
        function databaseCreationModel() {
            var _this = _super.apply(this, arguments) || this;
            _this.indexesPath = ko.observable();
            _this.incrementalBackup = ko.observable();
            _this.incrementalBackupSettings = {
                alertTimeout: ko.observable(24),
                alertRecurringTimeout: ko.observable(7)
            };
            _this.alertTimeoutOptions = [4, 8, 12, 24, 48, 72];
            _this.alertRecurringTimeoutOptions = [1, 2, 4, 7, 14];
            _this.advancedValidationGroup = ko.validatedObservable({
                dataPath: _this.dataPath,
                journalsPath: _this.journalsPath,
                tempPath: _this.tempPath,
                indexesPath: _this.indexesPath
            });
            return _this;
        }
        Object.defineProperty(databaseCreationModel.prototype, "resourceType", {
            get: function () {
                return "database";
            },
            enumerable: true,
            configurable: true
        });
        databaseCreationModel.prototype.setupValidation = function (resourceDoesntExist) {
            _super.prototype.setupValidation.call(this, resourceDoesntExist);
            this.setupPathValidation(this.indexesPath, "Indexes");
        };
        databaseCreationModel.prototype.toDto = function () {
            var settings = {};
            var securedSettings = {};
            if (this.incrementalBackup()) {
                settings[configuration.storage.allowIncrementalBackups] = "true";
            }
            if (this.tempPath() && this.tempPath().trim()) {
                settings[configuration.storage.tempPath] = this.tempPath();
            }
            settings[configuration.core.dataDirectory] = (this.dataPath() && this.dataPath().trim) ? this.dataPath() : "~/" + this.name();
            if (this.indexesPath() && this.indexesPath().trim()) {
                settings[configuration.indexing.indexStoragePath] = this.indexesPath();
            }
            if (this.journalsPath() && this.journalsPath().trim()) {
                settings[configuration.storage.journalsStoragePath] = this.journalsPath();
            }
            /* TODO
                    if (!clusterWide) settings["Raven-Non-Cluster-Database"] = "true";
    
            //TODO: return alertTimeout as null if left with default values (24/7)
                    if (alertTimeout !== "") {
                        settings["Raven/IncrementalBackup/AlertTimeoutHours"] = alertTimeout;
                    }
                    if (alertRecurringTimeout !== "") {
                        settings["Raven/IncrementalBackup/RecurringAlertTimeoutDays"] = alertRecurringTimeout;
                    }
            */
            this.fillEncryptionSettingsIfNeeded(securedSettings);
            return {
                Id: this.name(),
                Settings: settings,
                SecuredSettings: securedSettings,
                Disabled: false
            };
        };
        databaseCreationModel.prototype.setAlertTimeout = function (value) {
            this.incrementalBackupSettings.alertTimeout(value);
        };
        databaseCreationModel.prototype.setRecurringAlertTimeout = function (value) {
            this.incrementalBackupSettings.alertRecurringTimeout(value);
        };
        return databaseCreationModel;
    }(resourceCreationModel));
    return databaseCreationModel;
});

//# sourceMappingURL=databaseCreationModel.js.map
