define(["require", "exports"], function (require, exports) {
    "use strict";
    var databaseAccess = (function () {
        function databaseAccess(dto) {
            var _this = this;
            this.admin = ko.observable();
            this.tenantId = ko.observable();
            this.readOnly = ko.observable();
            this.currentAccessType = ko.computed({
                read: function () {
                    if (_this.admin()) {
                        return databaseAccess.adminAccessType;
                    }
                    else if (_this.readOnly()) {
                        return databaseAccess.readOnlyAccessType;
                    }
                    return databaseAccess.readWriteAccessType;
                },
                write: function (value) {
                    switch (value) {
                        case databaseAccess.adminAccessType:
                            _this.admin(true);
                            _this.readOnly(false);
                            break;
                        case databaseAccess.readOnlyAccessType:
                            _this.admin(false);
                            _this.readOnly(true);
                            break;
                        case databaseAccess.readWriteAccessType:
                            _this.admin(false);
                            _this.readOnly(false);
                            break;
                        default:
                    }
                }
            });
            this.admin(dto.Admin);
            this.readOnly(dto.ReadOnly);
            this.tenantId(dto.TenantId != null ? dto.TenantId : '');
            /* TODO:
            this.searchResults = ko.computed(() => {
                var readOnly = accessHelper.isGlobalAdmin() === false &&
                    accessHelper.canReadWriteSettings() === false &&
                    accessHelper.canReadSettings();
                if (readOnly)
                    return [];
    
                var newResourceName: string = this.tenantId();
                var resourceNames = shell.resources().map((rs: resource) => rs.name)
                                .concat("*")
                                .filter((x, i, c) => c.indexOf(x) == i);
                return resourceNames.filter((name) => name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1);
            });*/
            /* TODO:
            this.tenantCustomValidityError = ko.computed(() => {
                var errorMessage: string = "";
                var newTenantId = this.tenantId();
                var resourceNames = shell.resources().map((rs: resource) => rs.name)
                                .concat("*")
                                .filter((x, i, c) => c.indexOf(x) == i);
                var foundResource = resourceNames.first(name => newTenantId === name);
    
                if (!foundResource && newTenantId.length > 0) {
                    errorMessage = "There is no database nor file system with such a name!";
                }
    
                return errorMessage;
            });*/
        }
        databaseAccess.prototype.toDto = function () {
            return {
                Admin: this.admin(),
                TenantId: this.tenantId(),
                ReadOnly: this.readOnly()
            };
        };
        databaseAccess.empty = function () {
            return new databaseAccess({
                Admin: false,
                TenantId: "",
                ReadOnly: false
            });
        };
        databaseAccess.prototype.getTypes = function () {
            return databaseAccess.databaseAccessTypes();
        };
        return databaseAccess;
    }());
    databaseAccess.adminAccessType = 'Admin';
    databaseAccess.readWriteAccessType = 'Read, Write';
    databaseAccess.readOnlyAccessType = 'Read Only';
    databaseAccess.databaseAccessTypes = ko.observableArray([databaseAccess.adminAccessType, databaseAccess.readWriteAccessType, databaseAccess.readOnlyAccessType]);
    return databaseAccess;
});

//# sourceMappingURL=databaseAccess.js.map
