/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports", "models/resources/info/databaseInfo", "models/resources/info/filesystemInfo"], function (require, exports, databaseInfo, filesystemInfo) {
    "use strict";
    var resourcesInfo = (function () {
        function resourcesInfo(dto) {
            this.sortedResources = ko.observableArray();
            var databases = dto.Databases.map(function (db) { return new databaseInfo(db); });
            //TODO: fs, cs, ts
            var resources = databases.slice();
            resources.sort(function (a, b) { return a.name.toLowerCase().localeCompare(b.name.toLowerCase()); });
            this.sortedResources(resources);
            this.initObservables();
        }
        resourcesInfo.prototype.getByQualifiedName = function (qualifiedName) {
            return this.sortedResources().find(function (x) { return x.qualifiedName.toLowerCase() === qualifiedName.toLowerCase(); });
        };
        resourcesInfo.prototype.updateResource = function (newResourceInfo, resourceType) {
            var resourceToUpdate = this.getByQualifiedName(resourceType + "/" + newResourceInfo.Name);
            if (resourceToUpdate) {
                resourceToUpdate.update(newResourceInfo);
            }
            else {
                var resourceToAdd = void 0;
                switch (resourceType) {
                    case "db":
                        var dto = newResourceInfo;
                        resourceToAdd = new databaseInfo(dto);
                        break;
                    default:
                        throw new Error("Unsupported resource type = " + resourceType);
                }
                var locationToInsert = _.sortedIndexBy(this.sortedResources(), resourceToAdd, function (item) { return item.name.toLowerCase(); });
                this.sortedResources.splice(locationToInsert, 0, resourceToAdd);
            }
        };
        resourcesInfo.prototype.initObservables = function () {
            var _this = this;
            this.databasesCount = ko.pureComputed(function () { return _this
                .sortedResources()
                .filter(function (r) { return r instanceof databaseInfo; })
                .length; });
            this.filesystemCount = ko.pureComputed(function () { return _this
                .sortedResources()
                .filter(function (r) { return r instanceof filesystemInfo; })
                .length; });
        };
        return resourcesInfo;
    }());
    return resourcesInfo;
});

//# sourceMappingURL=resourcesInfo.js.map
