define(["require", "exports", "common/pagedList", "common/cssGenerator", "commands/timeSeries/getKeysCommand"], function (require, exports, pagedList, cssGenerator, getKeysCommand) {
    "use strict";
    var timeSeriesType = (function () {
        function timeSeriesType(name, fields, keysCount, ownerTimeSeries) {
            var _this = this;
            this.name = name;
            this.fields = fields;
            this.ownerTimeSeries = ownerTimeSeries;
            this.colorClass = "";
            this.keysCount = ko.observable(0);
            this.timeSeriesCountWithThousandsSeparator = ko.computed(function () { return _this.keysCount().toLocaleString(); });
            this.keysCount(keysCount);
            this.colorClass = timeSeriesType.getTypeCssClass(this.name, ownerTimeSeries);
            /* TODO:
            this["Name"] = name;
            this["Fields"] = fields.join(', ');
            this["Keys"] = keysCount; */
        }
        timeSeriesType.prototype.activate = function () {
            ko.postbox.publish("ActivateType", this);
        };
        timeSeriesType.prototype.getKeys = function () {
            if (!this.timeSeriesList) {
                this.timeSeriesList = this.createPagedList();
            }
            return this.timeSeriesList;
        };
        timeSeriesType.fromDto = function (dto, ts) {
            return new timeSeriesType(dto.Type, dto.Fields, dto.KeysCount, ts);
        };
        timeSeriesType.getTypeCssClass = function (type, ts) {
            return cssGenerator.getCssClass(type, timeSeriesType.typeColorMaps, ts);
        };
        timeSeriesType.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchKeys(skip, take); };
            var list = new pagedList(fetcher);
            list.collectionName = this.name;
            return list;
        };
        timeSeriesType.prototype.fetchKeys = function (skip, take) {
            return new getKeysCommand(this.ownerTimeSeries, skip, take, this.name, this.keysCount()).execute();
        };
        timeSeriesType.prototype.getEntityName = function () {
            return this.getId();
        };
        timeSeriesType.prototype.getDocumentPropertyNames = function () {
            return ["Name", "Fields", "Keys"];
        };
        timeSeriesType.prototype.getId = function () {
            return this.name;
        };
        timeSeriesType.prototype.getUrl = function () {
            return this.getId();
        };
        return timeSeriesType;
    }());
    timeSeriesType.typeColorMaps = [];
    return timeSeriesType;
});

//# sourceMappingURL=timeSeriesType.js.map
