var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "models/database/documents/document", "models/filesystem/file", "models/counter/counterSummary", "models/timeSeries/timeSeriesPoint", "models/timeSeries/timeSeriesKey", "models/timeSeries/timeSeriesType", "plugins/dialog", "commands/database/documents/deleteDocumentsCommand", "commands/filesystem/deleteFilesCommand", "commands/counter/deleteCountersCommand", "commands/timeSeries/deletePointsCommand", "commands/timeSeries/deleteTypesCommand", "viewmodels/dialogViewModelBase"], function (require, exports, document, file, counterSummary, timeSeriesPoint, timeSeriesKey, timeSeriesType, dialog, deleteDocumentsCommand, deleteFilesCommand, deleteCountersCommand, deletePointsCommand, deleteTypesCommand, dialogViewModelBase) {
    "use strict";
    var deleteItems = (function (_super) {
        __extends(deleteItems, _super);
        function deleteItems(items, rs, elementToFocusOnDismissal) {
            var _this = _super.call(this, elementToFocusOnDismissal) || this;
            _this.rs = rs;
            _this.items = ko.observableArray();
            _this.deletionStarted = false;
            _this.deletionTask = $.Deferred(); // Gives consumers a way to know when the async delete operation completes.
            if (items.length === 0) {
                throw new Error("Must have at least one document to delete.");
            }
            _this.items(items);
            return _this;
        }
        deleteItems.prototype.deleteItems = function () {
            var _this = this;
            var deleteItemsIds = this.items().map(function (i) { return i.getUrl(); });
            var deleteCommand;
            var firstItem = this.items()[0];
            if (firstItem instanceof document) {
                deleteCommand = new deleteDocumentsCommand(deleteItemsIds, this.rs);
            }
            else if (firstItem instanceof file) {
                deleteCommand = new deleteFilesCommand(deleteItemsIds, this.rs);
            }
            else if (firstItem instanceof counterSummary) {
                var counters = this.items();
                var groupAndNames = counters.map(function (x) {
                    return {
                        groupName: x.getEntityName(),
                        counterName: x.getId()
                    };
                });
                deleteCommand = new deleteCountersCommand(groupAndNames, this.rs);
            }
            else if (firstItem instanceof timeSeriesPoint) {
                var points = this.items().map(function (x) {
                    return {
                        Type: x.type,
                        Key: x.key,
                        At: x.At
                    };
                });
                deleteCommand = new deletePointsCommand(points, this.rs);
            }
            else if (firstItem instanceof timeSeriesKey) {
                debugger;
            }
            else if (firstItem instanceof timeSeriesType) {
                var types = this.items().map(function (type) {
                    return type.name;
                });
                deleteCommand = new deleteTypesCommand(types, this.rs);
            }
            else {
                debugger;
            }
            var deleteCommandTask = deleteCommand.execute();
            deleteCommandTask.done(function () { return _this.deletionTask.resolve(_this.items()); });
            deleteCommandTask.fail(function (response) { return _this.deletionTask.reject(response); });
            this.deletionStarted = true;
            dialog.close(this);
        };
        deleteItems.prototype.getDeletedItemName = function () {
            var item = this.items()[0];
            if (item instanceof counterSummary) {
                var summary = item;
                return " counter name: " + summary.getId() + ", group name: " + summary.getEntityName();
            }
            return item.getId();
        };
        deleteItems.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteItems.prototype.deactivate = function (args) {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never carried it out.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteItems;
    }(dialogViewModelBase));
    return deleteItems;
});

//# sourceMappingURL=deleteItems.js.map
