var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "models/database/documents/document", "plugins/dialog", "viewmodels/dialogViewModelBase", "models/database/documents/customColumnParams", "commands/database/documents/saveDocumentCommand", "commands/database/documents/deleteDocumentCommand", "common/autoCompleterSupport", "common/messagePublisher", "common/eventsCollector"], function (require, exports, document, dialog, dialogViewModelBase, customColumnParams, saveDocumentCommand, deleteDocumentCommand, autoCompleterSupport, messagePublisher, eventsCollector) {
    "use strict";
    var selectColumns = (function (_super) {
        __extends(selectColumns, _super);
        function selectColumns(customColumns, customFunctions, context, database, names) {
            var _this = _super.call(this) || this;
            _this.customColumns = customColumns;
            _this.customFunctions = customFunctions;
            _this.context = context;
            _this.database = database;
            _this.nextTask = $.Deferred();
            _this.nextTaskStarted = false;
            _this.autoCompleteBase = ko.observableArray([]);
            _this.autoCompleteResults = ko.observableArray([]);
            _this.completionSearchSubscriptions = [];
            _this.columnsNames = ko.observableArray([]);
            _this.columnsNames(names);
            _this.generateCompletionBase();
            _this.regenerateBindingSubscriptions();
            _this.monitorForNewRows();
            _this.autoCompleterSupport = new autoCompleterSupport(_this.autoCompleteBase, _this.autoCompleteResults, true);
            return _this;
        }
        selectColumns.prototype.generateCompletionBase = function () {
            this.autoCompleteBase([]);
            this.updateColumnsNames();
            var moduleSource = "var exports = {}; " + this.customFunctions.functions + "; return exports;";
            var exports = new Function(moduleSource)();
            for (var funcName in exports) {
                this.autoCompleteBase().push(ko.observable(funcName + "()"));
            }
        };
        selectColumns.prototype.regenerateBindingSubscriptions = function () {
            var _this = this;
            this.completionSearchSubscriptions.forEach(function (subscription) { return subscription.dispose(); });
            this.completionSearchSubscriptions = [];
            this.customColumns.columns().forEach(function (column, index) {
                return _this.completionSearchSubscriptions.push(column.binding.subscribe(_this.searchForCompletions.bind(_this)));
            });
        };
        selectColumns.prototype.monitorForNewRows = function () {
            var _this = this;
            this.customColumns.columns().forEach(function (column) { return column.binding.subscribe(function () {
                _this.updateColumnsNames();
            }); });
            this.customColumns.columns.subscribe(function (changes) {
                _this.updateColumnsNames();
                var somethingRemoved = false;
                changes.forEach(function (change) {
                    if (change.status === "added") {
                        _this.completionSearchSubscriptions.push(change.value.binding.subscribe(_this.searchForCompletions.bind(_this)));
                    }
                    else if (change.status === "deleted") {
                        _this.completionSearchSubscriptions.push(change.value.binding.subscribe(_this.searchForCompletions.bind(_this)));
                        somethingRemoved = true;
                    }
                    if (somethingRemoved) {
                        _this.regenerateBindingSubscriptions();
                    }
                });
            }, null, "arrayChange");
        };
        selectColumns.prototype.updateColumnsNames = function () {
            var customColumnsArray = this.customColumns.columns();
            var customColumnsNamesArray = customColumnsArray.map(function (column) { return column.binding(); });
            var allColumns = this.columnsNames();
            var arrayWithoutCustomNames = allColumns.filter(function (name) {
                var filteredNames = customColumnsNamesArray.filter(function (customName) { return customName.localeCompare(name) === 0; });
                return filteredNames.length === 0;
            });
            var array = arrayWithoutCustomNames.map(function (name) { return ko.observable(name); });
            this.autoCompleteBase(array);
        };
        selectColumns.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.form = $("#selectColumnsForm");
        };
        selectColumns.prototype.cancel = function () {
            dialog.close(this);
        };
        selectColumns.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.nextTaskStarted) {
                this.nextTask.reject();
            }
        };
        selectColumns.prototype.onExit = function () {
            return this.nextTask.promise();
        };
        selectColumns.prototype.changeCurrentColumns = function () {
            this.nextTaskStarted = true;
            this.nextTask.resolve(this.customColumns);
            dialog.close(this);
        };
        selectColumns.prototype.insertNewRow = function () {
            this.customColumns.columns.push(customColumnParams.empty());
        };
        selectColumns.prototype.deleteRow = function (row) {
            this.customColumns.columns.remove(row);
        };
        selectColumns.prototype.moveUp = function (row) {
            var i = this.customColumns.columns.indexOf(row);
            if (i >= 1) {
                var array = this.customColumns.columns();
                this.customColumns.columns.splice(i - 1, 2, array[i], array[i - 1]);
            }
        };
        selectColumns.prototype.moveDown = function (row) {
            var i = this.customColumns.columns.indexOf(row);
            if (i >= 0 && i < this.customColumns.columns().length - 1) {
                var array = this.customColumns.columns();
                this.customColumns.columns.splice(i, 2, array[i + 1], array[i]);
            }
        };
        selectColumns.prototype.customScheme = function (val) {
            if (this.customColumns.customMode() != val) {
                this.customColumns.customMode(val);
            }
        };
        selectColumns.prototype.saveAsDefault = function () {
            var _this = this;
            if (this.form[0].checkValidity() === true) {
                if (this.customColumns.customMode()) {
                    eventsCollector.default.reportEvent("custom-columns", "save-as-default", "custom");
                    var configurationDocument = new document(this.customColumns.toDto());
                    new saveDocumentCommand(this.context, configurationDocument, this.database, false).execute()
                        .done(function () { return _this.onConfigSaved(); })
                        .fail(function () { return messagePublisher.reportError("Unable to save configuration!"); });
                }
                else {
                    eventsCollector.default.reportEvent("custom-columns", "save-as-default", "auto");
                    new deleteDocumentCommand(this.context, this.database).execute().done(function () { return _this.onConfigSaved(); })
                        .fail(function () { return messagePublisher.reportError("Unable to save configuration!"); });
                }
            }
            else {
                messagePublisher.reportWarning('Configuration contains errors. Not saving it.');
            }
        };
        selectColumns.prototype.onConfigSaved = function () {
            messagePublisher.reportSuccess('Configuration saved!');
        };
        selectColumns.prototype.generateBindingInputId = function (index) {
            return 'binding-' + index;
        };
        selectColumns.prototype.enterKeyPressed = function () {
            var focusedBindingInput = $("[id ^= 'binding-']:focus");
            if (focusedBindingInput.length) {
                // insert first completion
                if (this.autoCompleteResults().length > 0) {
                    this.completeTheWord(this.autoCompleteResults()[0]());
                }
                // prevent submitting the form and closing dialog when accepting completion
                return false;
            }
            return _super.prototype.enterKeyPressed.call(this);
        };
        selectColumns.prototype.consumeUpDownArrowKeys = function (columnParams, event) {
            if (event.keyCode === 38 || event.keyCode === 40) {
                event.preventDefault();
                return false;
            }
            return true;
        };
        selectColumns.prototype.consumeClick = function (columnParams, event) {
            if (columnParams.binding().length === 0) {
                columnParams.binding.valueHasMutated();
                event.preventDefault();
                return false;
            }
            return true;
        };
        selectColumns.prototype.searchForCompletions = function () {
            this.activeInput = $("[id ^= 'binding-']:focus");
            if (this.activeInput.length > 0) {
                this.autoCompleterSupport.searchForCompletions(this.activeInput);
                this.lastActiveInput = this.activeInput;
            }
            else if (!!this.lastActiveInput) {
                this.autoCompleterSupport.searchForCompletions(this.lastActiveInput);
            }
        };
        selectColumns.prototype.completeTheWord = function (selectedCompletion) {
            var _this = this;
            if (this.activeInput.length > 0) {
                this.autoCompleterSupport.completeTheWord(this.activeInput, selectedCompletion, function (newValue) {
                    var columnParams = ko.dataFor(_this.activeInput[0]);
                    columnParams.binding(newValue);
                });
            }
        };
        return selectColumns;
    }(dialogViewModelBase));
    return selectColumns;
});

//# sourceMappingURL=selectColumns.js.map
