var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "common/autoCompleterSupport"], function (require, exports, dialog, dialogViewModelBase, autoCompleterSupport) {
    "use strict";
    var csvColumn = (function () {
        function csvColumn(value) {
            if (value === void 0) { value = ""; }
            this.name = ko.observable(value);
        }
        return csvColumn;
    }());
    var selectColumns = (function (_super) {
        __extends(selectColumns, _super);
        function selectColumns(columnNames) {
            var _this = _super.call(this) || this;
            _this.columnNames = columnNames;
            _this.nextTask = $.Deferred();
            _this.nextTaskStarted = false;
            _this.csvColumns = ko.observableArray([]);
            _this.autoCompleteBase = ko.observableArray([]);
            _this.autoCompleteResults = ko.observableArray([]);
            _this.completionSearchSubscriptions = [];
            _this.csvColumns(columnNames.map(function (x) { return new csvColumn(x); }));
            _this.generateCompletionBase();
            _this.regenerateBindingSubscriptions();
            _this.monitorForNewRows();
            _this.autoCompleterSupport = new autoCompleterSupport(_this.autoCompleteBase, _this.autoCompleteResults, true);
            return _this;
        }
        selectColumns.prototype.generateCompletionBase = function () {
            this.autoCompleteBase([]);
            this.prepareAutocompleteOptions();
        };
        selectColumns.prototype.regenerateBindingSubscriptions = function () {
            var _this = this;
            this.completionSearchSubscriptions.forEach(function (subscription) { return subscription.dispose(); });
            this.completionSearchSubscriptions = [];
            this.csvColumns().forEach(function (column) {
                return _this.completionSearchSubscriptions.push(column.name.subscribe(_this.searchForCompletions.bind(_this)));
            });
        };
        selectColumns.prototype.monitorForNewRows = function () {
            var _this = this;
            this.csvColumns().forEach(function (column) { return column.name.subscribe(function () {
                _this.prepareAutocompleteOptions();
            }); });
            this.csvColumns.subscribe(function (changes) {
                _this.prepareAutocompleteOptions();
                var somethingRemoved = false;
                changes.forEach(function (change) {
                    if (change.status === "added" || change.status === "deleted") {
                        _this.completionSearchSubscriptions.push(change.value.name.subscribe(_this.searchForCompletions.bind(_this)));
                        change.value.name;
                    }
                    if (change.status === "deleted") {
                        somethingRemoved = true;
                    }
                });
                if (somethingRemoved) {
                    _this.regenerateBindingSubscriptions();
                }
            }, null, "arrayChange");
        };
        /**
         * Prepare auto complate options by filtering out already used columns
         */
        selectColumns.prototype.prepareAutocompleteOptions = function () {
            var alreadyUsedColumns = this.csvColumns().map(function (x) { return x.name(); }).filter(function (x) { return !!x; });
            var allAvailableColumns = this.columnNames;
            var notAlreadyUsedColumns = allAvailableColumns.filter(function (name) {
                var nameLower = name.toLocaleLowerCase();
                return !alreadyUsedColumns.find(function (x) { return x.toLocaleLowerCase() === nameLower; });
            });
            this.autoCompleteBase(notAlreadyUsedColumns.map(function (x) { return ko.observable(x); }));
        };
        selectColumns.prototype.cancel = function () {
            dialog.close(this);
        };
        selectColumns.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.nextTaskStarted) {
                this.nextTask.reject();
            }
        };
        selectColumns.prototype.onExit = function () {
            return this.nextTask.promise();
        };
        selectColumns.prototype.exportCsv = function () {
            this.nextTaskStarted = true;
            this.nextTask.resolve(this.csvColumns().map(function (x) { return x.name(); }));
            dialog.close(this);
        };
        selectColumns.prototype.insertNewRow = function () {
            this.csvColumns.push(new csvColumn());
        };
        selectColumns.prototype.deleteRow = function (row) {
            this.csvColumns.remove(row);
        };
        selectColumns.prototype.moveUp = function (row) {
            var i = this.csvColumns.indexOf(row);
            if (i >= 1) {
                var array = this.csvColumns();
                this.csvColumns.splice(i - 1, 2, array[i], array[i - 1]);
            }
        };
        selectColumns.prototype.moveDown = function (row) {
            var i = this.csvColumns.indexOf(row);
            if (i >= 0 && i < this.csvColumns().length - 1) {
                var array = this.csvColumns();
                this.csvColumns.splice(i, 2, array[i + 1], array[i]);
            }
        };
        selectColumns.prototype.generateBindingInputId = function (index) {
            return 'binding-' + index;
        };
        selectColumns.prototype.enterKeyPressed = function () {
            var focusedBindingInput = $("[id ^= 'binding-']:focus");
            if (focusedBindingInput.length) {
                // insert first completion
                if (this.autoCompleteResults().length > 0) {
                    this.completeTheWord(this.autoCompleteResults()[0]());
                }
                // prevent submitting the form and closing dialog when accepting completion
                return false;
            }
            return _super.prototype.enterKeyPressed.call(this);
        };
        selectColumns.prototype.consumeUpDownArrowKeys = function (columnParams, event) {
            if (event.keyCode === 38 || event.keyCode === 40) {
                event.preventDefault();
                return false;
            }
            return true;
        };
        selectColumns.prototype.consumeClick = function (columnParams, event) {
            if (columnParams.name().length === 0) {
                columnParams.name.valueHasMutated();
                event.preventDefault();
                return false;
            }
            return true;
        };
        selectColumns.prototype.searchForCompletions = function () {
            this.activeInput = $("[id ^= 'binding-']:focus");
            if (this.activeInput.length > 0) {
                this.autoCompleterSupport.searchForCompletions(this.activeInput);
                this.lastActiveInput = this.activeInput;
            }
            else if (!!this.lastActiveInput) {
                this.autoCompleterSupport.searchForCompletions(this.lastActiveInput);
            }
        };
        selectColumns.prototype.completeTheWord = function (selectedCompletion) {
            var _this = this;
            if (this.activeInput.length > 0) {
                this.autoCompleterSupport.completeTheWord(this.activeInput, selectedCompletion, function (newValue) {
                    var columnParams = ko.dataFor(_this.activeInput[0]);
                    columnParams.name(newValue);
                });
            }
        };
        return selectColumns;
    }(dialogViewModelBase));
    return selectColumns;
});

//# sourceMappingURL=selectCsvColumns.js.map
