var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/appUrl", "viewmodels/viewModelBase", "models/counter/counterStorageReplicationSetup", "models/counter/counterStorageReplicationDestination", "commands/counter/getCounterStorageReplicationCommand", "commands/counter/saveCounterStorageReplicationCommand"], function (require, exports, appUrl, viewModelBase, counterStorageReplicationSetup, counterStorageReplicationDestination, getCounterStorageReplicationCommand, saveCounterStorageReplicationCommand) {
    "use strict";
    var counterStorageReplication = (function (_super) {
        __extends(counterStorageReplication, _super);
        function counterStorageReplication() {
            var _this = _super.call(this) || this;
            _this.replicationsSetup = ko.observable().extend({ required: true });
            _this.appUrls = appUrl.forCurrentCounterStorage();
            return _this;
        }
        counterStorageReplication.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            var counterStorage = this.activeCounterStorage();
            if (counterStorage) {
                this.fetchCountersDestinations(counterStorage)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forCounterStorage(counterStorage) }); });
            }
            return deferred;
        };
        counterStorageReplication.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.replicationsSetup]);
            this.isSaveEnabled = ko.computed(function () {
                return _this.dirtyFlag().isDirty();
            });
        };
        counterStorageReplication.prototype.fetchCountersDestinations = function (counterStorage, reportFetchProgress) {
            var _this = this;
            if (reportFetchProgress === void 0) { reportFetchProgress = false; }
            var deferred = $.Deferred();
            if (counterStorage) {
                new getCounterStorageReplicationCommand(counterStorage, reportFetchProgress)
                    .execute()
                    .done(function (data) { return _this.replicationsSetup(new counterStorageReplicationSetup({ Destinations: null /* TODO: data.Destinations - looks like it isn't available on server side for now */ })); })
                    .fail(function () { return _this.replicationsSetup(new counterStorageReplicationSetup({ Destinations: [] })); })
                    .always(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        counterStorageReplication.prototype.saveChanges = function () {
            var _this = this;
            var counter = this.activeCounterStorage();
            if (counter) {
                new saveCounterStorageReplicationCommand(this.replicationsSetup().toDto(), counter)
                    .execute()
                    .done(function () { return _this.dirtyFlag().reset(); });
            }
        };
        counterStorageReplication.prototype.createNewDestination = function () {
            var cs = this.activeCounterStorage();
            this.replicationsSetup().destinations.unshift(counterStorageReplicationDestination.empty(cs.name));
        };
        counterStorageReplication.prototype.removeDestination = function (resplicationDestination) {
            this.replicationsSetup().destinations.remove(resplicationDestination);
        };
        counterStorageReplication.prototype.refreshFromServer = function () {
            var _this = this;
            var canContinue = this.canContinueIfNotDirty('Unsaved Data', 'You have unsaved data. Are you sure you want to refresh the data from the server?');
            canContinue.done(function () {
                _this.fetchCountersDestinations(_this.activeCounterStorage(), true)
                    .done(function () { return _this.dirtyFlag().reset(); });
            });
        };
        return counterStorageReplication;
    }(viewModelBase));
    return counterStorageReplication;
});

//# sourceMappingURL=counterStorageReplication.js.map
