var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "models/counter/counterChange", "models/counter/counterGroup", "commands/counter/getCounterGroupsCommand", "commands/counter/updateCounterCommand", "commands/counter/resetCounterCommand", "viewmodels/viewModelBase", "viewmodels/counter/editCounterDialog", "viewmodels/counter/deleteGroup", "common/constants/events"], function (require, exports, app, router, appUrl, counterChange, counterGroup, getCounterGroupsCommand, updateCounterCommand, resetCounterCommand, viewModelBase, editCounterDialog, deleteGroup, EVENTS) {
    "use strict";
    var counters = (function (_super) {
        __extends(counters, _super);
        function counters() {
            var _this = _super.call(this) || this;
            _this.viewType = 1 /* Counters */;
            _this.groups = ko.observableArray();
            _this.selectedGroup = ko.observable().subscribeTo("ActivateGroup").distinctUntilChanged();
            _this.currentGroup = ko.observable();
            _this.currentGroupPagedItems = ko.observable();
            _this.selectedCounterIndices = ko.observableArray();
            _this.isAnyCountersAutoSelected = ko.observable(false);
            _this.isAllCountersAutoSelected = ko.observable(false);
            _this.showLoadingIndicator = ko.observable(false);
            _this.showLoadingIndicatorThrottled = _this.showLoadingIndicator.throttle(250);
            _this.isInitialized = counters.isInitialized;
            _this.selectedGroup.subscribe(function (c) { return _this.selectedGroupChanged(c); });
            _this.isAllGroupsGroup = ko.computed(function () { return !!_this.selectedGroup() ? _this.selectedGroup().isAllGroupsGroup : true; });
            _this.hasCounters = ko.computed(function () {
                var selectedGroup = _this.selectedGroup();
                if (!!selectedGroup) {
                    if (selectedGroup.name === counterGroup.allGroupsGroupName) {
                        var cs = _this.activeCounterStorage();
                        return true; //TODO: return !!cs.statistics() ? cs.statistics().countersCount() > 0 : false;
                    }
                    return _this.selectedGroup().countersCount() > 0;
                }
                return false;
            });
            _this.hasAnyCountersSelected = ko.computed(function () { return _this.selectedCounterIndices().length > 0; });
            _this.hasAllCountersSelected = ko.computed(function () {
                var numOfSelectedCounters = _this.selectedCounterIndices().length;
                if (!!_this.selectedGroup() && numOfSelectedCounters !== 0) {
                    return numOfSelectedCounters === _this.selectedGroup().countersCount();
                }
                return false;
            });
            _this.selectedCountersText = ko.computed(function () {
                if (!!_this.selectedCounterIndices()) {
                    var documentsText = "counter";
                    if (_this.selectedCounterIndices().length !== 1) {
                        documentsText += "s";
                    }
                    return documentsText;
                }
                return "";
            });
            _this.countersSelection = ko.computed(function () {
                if (_this.hasAllCountersSelected())
                    return 2 /* Checked */;
                if (_this.hasAnyCountersSelected())
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
            return _this;
        }
        counters.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            //TODO: update this in documentation
            //this.updateHelpLink('G8CDCP');
            // We can optionally pass in a group name to view's URL, e.g. #counterstorages/counters?group=Foo&counterstorage=test
            this.groupToSelectName = args ? args.group : null;
            var cs = this.activeCounterStorage();
            //initial groups fetch
            this.fetchGroups(cs, 0, Number.MAX_VALUE).done(function (results) {
                _this.groupsLoaded(results, cs);
                counters.isInitialized(true);
            });
        };
        counters.prototype.attached = function () {
            _super.prototype.attached.call(this);
            /*super.createKeyboardShortcut("F2", () => this.editSelectedCounter(), counters.gridSelector);*/
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            /*var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", () => this.getDocumentsGrid().deleteSelectedItems(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", () => this.copySelectedDocs(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", () => this.copySelectedDocIds(), docsPageSelector);*/
        };
        counters.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            counters.isInitialized(false);
        };
        counters.prototype.createNotifications = function () {
            return [];
        };
        counters.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                ko.postbox.subscribe("ChangeCounterValue", function () { return _this.change(); }),
                ko.postbox.subscribe("ResetCounter", function () { return _this.reset(); }),
                ko.postbox.subscribe(EVENTS.ChangesApi.Reconnected, function (cs) { return _this.reloadCountersData(cs); }),
                ko.postbox.subscribe("SortGroups", function () { return _this.sortGroups(); }),
                ko.postbox.subscribe("SelectGroup", function (groupName) {
                    var groupToSelect = _this.groups().find(function (g) { return g.name === groupName; });
                    if (!!groupToSelect) {
                        _this.selectGroupInternal(groupToSelect);
                    }
                })
            ];
        };
        counters.prototype.fetchGroups = function (cs, skip, pageSize) {
            var deferred = $.Deferred();
            var getGroupsCommand = new getCounterGroupsCommand(cs, skip, pageSize);
            getGroupsCommand.execute().done(function (results) { return deferred.resolve(results); });
            return deferred;
        };
        counters.prototype.groupsLoaded = function (groups, cs) {
            var _this = this;
            // Create the "All Groups" pseudo collection.
            this.allGroupsGroup = counterGroup.createAllGroupsCollection(cs);
            this.allGroupsGroup.countersCount = ko.observable(0); //TODO:  ko.computed(() => !!cs.statistics() ? cs.statistics().countersCount() : 0);
            // All systems a-go. Load them into the UI and select the first one.
            var allGroups = [this.allGroupsGroup].concat(groups);
            this.groups(allGroups);
            var groupToSelect = this.groups().find(function (g) { return g.name === _this.groupToSelectName; }) || this.allGroupsGroup;
            groupToSelect.activate();
        };
        counters.prototype.newCounter = function () {
            var _this = this;
            var counterChangeVm;
            var currentGroupName = this.selectedGroup().name;
            if (currentGroupName === counterGroup.allGroupsGroupName) {
                counterChangeVm = new editCounterDialog();
            }
            else {
                var dto = {
                    CurrentValue: 0,
                    Group: currentGroupName,
                    CounterName: "",
                    Delta: 0
                };
                var change = new counterChange(dto, true);
                counterChangeVm = new editCounterDialog(change);
            }
            counterChangeVm.updateTask.done(function (change) {
                var counterCommand = new updateCounterCommand(_this.activeCounterStorage(), change.group(), change.counterName(), change.delta(), change.isNew());
                var execute = counterCommand.execute();
                execute.done(function () { return _this.refreshGridAndGroup(change.group()); });
            });
            app.showBootstrapDialog(counterChangeVm);
        };
        counters.prototype.refresh = function () {
            var selectedGroupName = this.selectedGroup().name;
            this.refreshGridAndGroup(selectedGroupName);
        };
        counters.prototype.edit = function () {
            var grid = this.getCountersGrid();
            if (grid) {
                grid.editLastSelectedItem();
            }
        };
        counters.prototype.change = function () {
            var _this = this;
            var grid = this.getCountersGrid();
            if (grid) {
                var counterData = grid.getSelectedItems(1)[0];
                var dto = {
                    CurrentValue: counterData.Total,
                    Group: counterData["Group Name"],
                    CounterName: counterData["Counter Name"],
                    Delta: 0
                };
                var change = new counterChange(dto);
                var counterChangeVm = new editCounterDialog(change);
                counterChangeVm.updateTask.done(function (change, isNew) {
                    var counterCommand = new updateCounterCommand(_this.activeCounterStorage(), change.group(), change.counterName(), change.delta(), isNew);
                    var execute = counterCommand.execute();
                    execute.done(function () { return _this.refreshGridAndGroup(counterData.GroupName); });
                });
                app.showBootstrapDialog(counterChangeVm);
            }
        };
        counters.prototype.reset = function () {
            var _this = this;
            var grid = this.getCountersGrid();
            if (grid) {
                var counterData = grid.getSelectedItems(1)[0];
                var confirmation = this.confirmationMessage("Reset Counter", "Are you sure that you want to reset the counter?");
                confirmation.done(function () {
                    var resetCommand = new resetCounterCommand(_this.activeCounterStorage(), counterData["Group Name"], counterData["Counter Name"]);
                    var execute = resetCommand.execute();
                    execute.done(function () { return _this.refreshGridAndGroup(counterData.GroupName); });
                });
            }
        };
        counters.prototype.refreshGridAndGroup = function (changedGroupName) {
            var group = this.selectedGroup();
            if (group.name === changedGroupName || group.name === counterGroup.allGroupsGroupName) {
                this.getCountersGrid().refreshCollectionData();
            }
            group.invalidateCache();
            this.selectNone();
        };
        counters.prototype.selectedGroupChanged = function (selected) {
            if (!!selected) {
                var pagedList = selected.getCounters();
                this.currentGroupPagedItems(pagedList);
                this.currentGroup(selected);
            }
        };
        counters.prototype.toggleSelectAll = function () {
            var countersGrid = this.getCountersGrid();
            if (!!countersGrid) {
                if (this.hasAnyCountersSelected()) {
                    countersGrid.selectNone();
                }
                else {
                    countersGrid.selectSome();
                    this.isAnyCountersAutoSelected(this.hasAllCountersSelected() === false);
                }
            }
        };
        counters.prototype.selectAll = function () {
            var countersGrid = this.getCountersGrid();
            var group = this.selectedGroup();
            if (!!countersGrid && !!group) {
                countersGrid.selectAll(group.countersCount());
            }
        };
        counters.prototype.selectNone = function () {
            var countersGrid = this.getCountersGrid();
            if (!!countersGrid) {
                countersGrid.selectNone();
            }
        };
        counters.prototype.deleteSelectedCounters = function () {
            if (this.hasAllCountersSelected()) {
                this.deleteGroupInternal(this.selectedGroup());
            }
            else {
                var grid = this.getCountersGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        counters.prototype.deleteGroupInternal = function (group) {
            var _this = this;
            var deleteGroupVm = new deleteGroup(group, this.activeCounterStorage());
            deleteGroupVm.deletionTask.done(function () {
                if (!group.isAllGroupsGroup) {
                    _this.groups.remove(group);
                    var selectedCollection = _this.selectedGroup();
                    if (group.name === selectedCollection.name) {
                        _this.selectedGroup(_this.allGroupsGroup);
                    }
                }
                else {
                    _this.refreshGridAndGroup(group.name);
                }
            });
            app.showBootstrapDialog(deleteGroupVm);
        };
        counters.prototype.updateGroups = function (receivedGroups) {
            var _this = this;
            var deletedGroups = [];
            this.groups().forEach(function (group) {
                if (!receivedGroups.find(function (receivedGroup) { return group.name === receivedGroup.name; }) && group.name !== "All Groups") {
                    deletedGroups.push(group);
                }
            });
            this.groups.removeAll(deletedGroups);
            receivedGroups.forEach(function (receivedGroup) {
                var foundGroup = _this.groups().find(function (group) { return group.name === receivedGroup.name; });
                if (!foundGroup) {
                    _this.groups.push(receivedGroup);
                }
                else {
                    foundGroup.countersCount(receivedGroup.countersCount());
                }
            });
            //if the group is deleted, go to the all groups group
            var currentGroup = this.groups().find(function (g) { return g.name === _this.selectedGroup().name; });
            if (!currentGroup || currentGroup.countersCount() === 0) {
                this.selectedGroup(this.allGroupsGroup);
            }
        };
        counters.prototype.refreshGroupsData = function () {
            this.groups().forEach(function (group) {
                var pagedList = group.getCounters();
                pagedList.invalidateCache();
            });
        };
        counters.prototype.refreshGroups = function () {
            var _this = this;
            var deferred = $.Deferred();
            var cs = this.activeCounterStorage();
            //TODO: add proper paging support here too
            this.fetchGroups(cs, 0, Number.MAX_VALUE).done(function (results) {
                _this.updateGroups(results);
                _this.refreshGroupsData();
                //TODO: add a button to refresh the counters and than use this.refreshCollectionsData();
                deferred.resolve();
            });
            return deferred;
        };
        counters.prototype.reloadCountersData = function (cs) {
            var _this = this;
            if (cs.name === this.activeCounterStorage().name) {
                this.refreshGroups().done(function () { return _this.refreshGroupsData(); });
            }
        };
        counters.prototype.selectGroup = function (group, event) {
            if (!event || event.which !== 3) {
                this.selectGroupInternal(group);
            }
        };
        counters.prototype.selectGroupInternal = function (group) {
            group.activate();
            var countersWithGroupUrl = appUrl.forCounterStorageCounters(group.name, this.activeCounterStorage());
            router.navigate(countersWithGroupUrl, false);
        };
        counters.prototype.getCountersGrid = function () {
            var gridContents = $(counters.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        counters.prototype.sortGroups = function () {
            this.groups.sort(function (c1, c2) {
                if (c1.isAllGroupsGroup)
                    return -1;
                if (c2.isAllGroupsGroup)
                    return 1;
                return c1.name.toLowerCase() > c2.name.toLowerCase() ? 1 : -1;
            });
        };
        // Animation callbacks for the groups list
        counters.prototype.showGroupElement = function (element) {
            if (element.nodeType === 1 && counters.isInitialized()) {
                $(element).hide().slideDown(500, function () {
                    ko.postbox.publish("SortGroups");
                    $(element).highlight();
                });
            }
        };
        counters.prototype.hideGroupElement = function (element) {
            if (element.nodeType === 1) {
                $(element).slideUp(1000, function () { $(element).remove(); });
            }
        };
        return counters;
    }(viewModelBase));
    counters.gridSelector = "#countersGrid";
    counters.isInitialized = ko.observable(false);
    return counters;
});

//# sourceMappingURL=counters.js.map
