var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "common/messagePublisher", "models/counter/counterChange", "models/counter/counterSummary", "viewmodels/counter/editCounterDialog", "commands/counter/getCounterCommand", "commands/counter/updateCounterCommand", "commands/counter/resetCounterCommand", "viewmodels/viewModelBase", "viewmodels/common/deleteItems"], function (require, exports, app, router, appUrl, messagePublisher, counterChange, counterSummary, editCounterDialog, getCounterCommand, updateCounterCommand, resetCounterCommand, viewModelBase, deleteItems) {
    "use strict";
    var editCounter = (function (_super) {
        __extends(editCounter, _super);
        function editCounter() {
            var _this = _super.call(this) || this;
            _this.groupName = ko.observable();
            _this.counterName = ko.observable();
            _this.groupLink = ko.observable();
            _this.counter = ko.observable();
            _this.isLoading = ko.observable();
            _this.topRecentCounters = ko.computed(function () { return _this.getTopRecentCounters(); });
            _this.isBusy = ko.observable(false);
            return _this;
        }
        editCounter.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deffered = $.Deferred();
            if (!args.groupName || !args.counterName) {
                messagePublisher.reportError("Can't find group name or counter name in query string!");
                deffered.resolve({ redirect: appUrl.forCounterStorageCounters(null, this.activeCounterStorage()) });
            }
            var cs = this.activeCounterStorage();
            this.load(args.groupName, args.counterName)
                .done(function () {
                _this.groupName(args.groupName);
                _this.counterName(args.counterName);
                _this.groupLink(appUrl.forCounterStorageCounters(args.groupName, cs));
                _this.appendRecentCounter(args.groupName, args.counterName);
                deffered.resolve({ can: true });
            })
                .fail(function () {
                messagePublisher.reportError("Can't find counter!");
                _this.removeFromTopRecentCounters(args.groupName, args.counterName);
                deffered.resolve({ redirect: appUrl.forCounterStorageCounters(null, cs) });
            });
            return deffered;
        };
        editCounter.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.setupKeyboardShortcuts();
        };
        editCounter.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteCounter(); }, editCounter.container);
        };
        editCounter.prototype.load = function (groupName, counterName) {
            var _this = this;
            this.isLoading(true);
            return new getCounterCommand(this.activeCounterStorage(), groupName, counterName)
                .execute()
                .done(function (result) {
                _this.counter(result);
                _this.isLoading(false);
            });
        };
        editCounter.prototype.refresh = function () {
            this.load(this.groupName(), this.counterName());
        };
        editCounter.prototype.change = function () {
            var _this = this;
            var dto = {
                CurrentValue: this.counter().total(),
                Group: this.groupName(),
                CounterName: this.counterName(),
                Delta: 0
            };
            var change = new counterChange(dto);
            var counterChangeVm = new editCounterDialog(change);
            counterChangeVm.updateTask.done(function (change, isNew) {
                var counterCommand = new updateCounterCommand(_this.activeCounterStorage(), change.group(), change.counterName(), change.delta(), isNew);
                var execute = counterCommand.execute();
                execute.done(function () { return _this.refresh(); });
            });
            app.showBootstrapDialog(counterChangeVm);
        };
        editCounter.prototype.reset = function () {
            var _this = this;
            var confirmation = this.confirmationMessage("Reset Counter", "Are you sure that you want to reset the counter?");
            confirmation.done(function () {
                var resetCommand = new resetCounterCommand(_this.activeCounterStorage(), _this.groupName(), _this.counterName());
                var execute = resetCommand.execute();
                execute.done(function () { return _this.refresh(); });
            });
        };
        editCounter.prototype.deleteCounter = function () {
            var _this = this;
            var summary = new counterSummary({
                GroupName: this.groupName(),
                CounterName: this.counterName(),
                Total: this.counter().total()
            });
            var viewModel = new deleteItems([summary], this.activeCounterStorage());
            viewModel.deletionTask.done(function () {
                var countersUrl = appUrl.forCounterStorageCounters(null, _this.activeCounterStorage());
                router.navigate(countersUrl);
            });
            app.showBootstrapDialog(viewModel, editCounter.container);
        };
        editCounter.prototype.removeFromTopRecentCounters = function (groupName, counterName) {
            var currentFilesystemName = this.activeFilesystem().name;
            var recentFilesForCurFilesystem = editCounter.recentCountersInCounterStorage().find(function (x) { return x.counterStorageName === currentFilesystemName; });
            if (recentFilesForCurFilesystem) {
                var counter = {
                    groupName: groupName,
                    counterName: counterName
                };
                recentFilesForCurFilesystem.recentCounters.remove(counter);
            }
        };
        editCounter.prototype.getTopRecentCounters = function () {
            var _this = this;
            var cs = this.activeCounterStorage();
            var recentFilesForCurFilesystem = editCounter.recentCountersInCounterStorage().find(function (x) { return x.counterStorageName === cs.name; });
            if (recentFilesForCurFilesystem) {
                var value = recentFilesForCurFilesystem
                    .recentCounters()
                    .filter(function (x) {
                    return x.groupName !== _this.groupName() && x.counterName !== _this.counterName();
                })
                    .slice(0, 5)
                    .map(function (groupAndCounterName) {
                    var groupName = groupAndCounterName.groupName;
                    var counterName = groupAndCounterName.counterName;
                    return {
                        groupName: groupName,
                        counterName: counterName,
                        counterUrl: appUrl.forEditCounter(cs, groupName, counterName)
                    };
                });
                return value;
            }
            else {
                return [];
            }
        };
        editCounter.prototype.appendRecentCounter = function (groupName, counterName) {
            var csName = this.activeCounterStorage().name;
            var existingRecentCounters = editCounter.recentCountersInCounterStorage().find(function (x) { return x.counterStorageName === csName; });
            if (existingRecentCounters) {
                var existingCounter = existingRecentCounters.recentCounters().find(function (x) { return x.groupName === groupName && x.counterName === counterName; });
                if (!existingCounter) {
                    if (existingRecentCounters.recentCounters().length === 5) {
                        existingRecentCounters.recentCounters.pop();
                    }
                    existingRecentCounters.recentCounters.unshift({ groupName: groupName, counterName: counterName });
                }
            }
            else {
                editCounter.recentCountersInCounterStorage.push({ counterStorageName: csName, recentCounters: ko.observableArray([{ groupName: groupName, counterName: counterName }]) });
            }
        };
        return editCounter;
    }(viewModelBase));
    editCounter.container = "#editCounterContainer";
    editCounter.recentCountersInCounterStorage = ko.observableArray();
    return editCounter;
});

//# sourceMappingURL=editCounter.js.map
