var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/router", "common/appUrl", "common/pagedList", "models/database/documents/customColumns", "models/database/documents/customColumnParams", "commands/database/replication/getConflictsCommand", "commands/database/replication/getReplicationSourcesCommand", "commands/database/index/getIndexDefinitionCommand", "commands/database/transformers/getSingleTransformerCommand", "commands/database/replication/conflictsResolveCommand", "commands/database/globalConfig/getEffectiveConflictResolutionCommand", "common/eventsCollector", "viewmodels/viewModelBase"], function (require, exports, router, appUrl, pagedList, customColumns, customColumnParams, getConflictsCommand, getReplicationSourcesCommand, getIndexDefinitionCommand, getSingleTransformerCommand, conflictsResolveCommand, getEffectiveConflictResolutionCommand, eventsCollector, viewModelBase) {
    "use strict";
    var conflicts = (function (_super) {
        __extends(conflicts, _super);
        function conflicts() {
            var _this = _super.apply(this, arguments) || this;
            _this.displayName = "conflicts";
            _this.sourcesLookup = {};
            _this.refreshConflictsObservable = ko.observable();
            _this.currentColumns = ko.observable(customColumns.empty());
            _this.currentConflictsPagedItems = ko.observable();
            _this.selectedDocumentIndices = ko.observableArray();
            _this.serverConflictResolution = ko.observable();
            return _this;
        }
        conflicts.prototype.createNotifications = function () {
            return [];
        };
        conflicts.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.conflictsSubscription = this.refreshConflictsObservable.throttle(3000).subscribe(function (e) { return _this.fetchConflicts(_this.activeDatabase()); });
        };
        conflicts.prototype.detached = function () {
            _super.prototype.detached.call(this);
            if (this.conflictsSubscription != null) {
                this.conflictsSubscription.dispose();
            }
        };
        conflicts.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function (db) { return _this.databaseChanged(db); });
            this.updateHelpLink('5FRRCA');
            this.hasAnyConflict = ko.computed(function () {
                var pagedItems = _this.currentConflictsPagedItems();
                if (pagedItems) {
                    return pagedItems.totalResultCount() > 0;
                }
                return false;
            });
            this.currentColumns().columns([
                new customColumnParams({ Header: "Detected At (UTC)", Binding: "conflictDetectedAt", DefaultWidth: 300 }),
                new customColumnParams({ Header: "Versions", Binding: "versions", DefaultWidth: 400, Template: 'versions-template' })
            ]);
            this.currentColumns().customMode(true);
            return this.performIndexCheck(this.activeDatabase()).then(function () {
                return _this.loadReplicationSources(_this.activeDatabase());
            }).done(function () {
                _this.fetchAutomaticConflictResolution(_this.activeDatabase());
                _this.fetchConflicts(_this.activeDatabase());
            });
        };
        conflicts.prototype.fetchAutomaticConflictResolution = function (db) {
            var _this = this;
            return new getEffectiveConflictResolutionCommand(db)
                .execute()
                .done(function (repConfig) {
                _this.serverConflictResolution(repConfig.MergedDocument.DocumentConflictResolution);
            });
        };
        conflicts.prototype.fetchConflicts = function (database) {
            this.currentConflictsPagedItems(this.createPagedList(database));
        };
        conflicts.prototype.loadReplicationSources = function (db) {
            var _this = this;
            return new getReplicationSourcesCommand(db)
                .execute()
                .done(function (results) { return _this.replicationSourcesLoaded(results, db); });
        };
        conflicts.prototype.performIndexCheck = function (db) {
            // first look in cache
            if (_.includes(conflicts.performedIndexChecks, db.name)) {
                return $.Deferred().resolve();
            }
            var performCheckTask = $.Deferred();
            // perform index check against DB
            $.when(new getIndexDefinitionCommand(conflicts.conflictsIndexName, db).execute(), new getSingleTransformerCommand(conflicts.conflictsTransformerName, db).execute())
                .done(function () {
                conflicts.performedIndexChecks.push(db.name);
                performCheckTask.resolve();
            })
                .fail(function () {
                conflicts.performedIndexChecks.push(db.name);
            });
            return performCheckTask;
        };
        conflicts.prototype.replicationSourcesLoaded = function (sources, db) {
            this.sourcesLookup = sources;
        };
        conflicts.prototype.databaseChanged = function (db) {
            var _this = this;
            var conflictsUrl = appUrl.forConflicts(db);
            router.navigate(conflictsUrl, false);
            this.performIndexCheck(db).then(function () {
                return _this.loadReplicationSources(db);
            }).done(function () {
                _this.fetchConflicts(db);
            });
        };
        conflicts.prototype.createPagedList = function (database) {
            var fetcher = function (skip, take) { return new getConflictsCommand(database, skip, take).execute(); };
            return new pagedList(fetcher);
        };
        conflicts.prototype.getUrlForConflict = function (conflictVersion) {
            return appUrl.forEditDoc(conflictVersion.id, this.activeDatabase());
        };
        conflicts.prototype.getTextForVersion = function (conflictVersion) {
            var replicationSource = this.sourcesLookup[conflictVersion.sourceId];
            var text = "";
            if (replicationSource) {
                text = " (" + replicationSource + ")";
            }
            return text;
        };
        conflicts.prototype.getServerUrlForVersion = function (conflictVersion) {
            return this.sourcesLookup[conflictVersion.sourceId] || "";
        };
        conflicts.prototype.resolveToLocal = function () {
            var _this = this;
            eventsCollector.default.reportEvent("conflicts", "resolve-to-local");
            this.confirmationMessage("Sure?", "You're resolving all conflicts to local.", ["No", "Yes"])
                .done(function () {
                _this.performResolve("ResolveToLocal");
            });
        };
        conflicts.prototype.resolveToNewestRemote = function () {
            var _this = this;
            eventsCollector.default.reportEvent("conflicts", "resolve-to-newest-remote");
            this.confirmationMessage("Sure?", "You're resolving all conflicts to newest remote.", ["No", "Yes"])
                .done(function () {
                _this.performResolve("ResolveToRemote");
            });
        };
        conflicts.prototype.resolveToLatest = function () {
            var _this = this;
            eventsCollector.default.reportEvent("conflicts", "resolve-to-latest");
            this.confirmationMessage("Sure?", "You're resolving all conflicts to latest.", ["No", "Yes"])
                .done(function () {
                _this.performResolve("ResolveToLatest");
            });
        };
        conflicts.prototype.performResolve = function (resolution) {
            var _this = this;
            new conflictsResolveCommand(this.activeDatabase(), resolution)
                .execute()
                .done(function () {
                _this.fetchConflicts(_this.activeDatabase());
            });
        };
        return conflicts;
    }(viewModelBase));
    conflicts.performedIndexChecks = [];
    conflicts.conflictsIndexName = "Raven/ConflictDocuments";
    conflicts.conflictsTransformerName = "Raven/ConflictDocumentsTransformer";
    conflicts.gridSelector = "#conflictsGrid";
    return conflicts;
});

//# sourceMappingURL=conflicts.js.map
