var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "common/constants/events", "viewmodels/viewModelBase", "viewmodels/database/documents/deleteCollection", "viewmodels/common/selectColumns", "viewmodels/common/selectCsvColumns", "viewmodels/common/showDataDialog", "models/database/documents/collection", "models/database/documents/customFunctions", "models/database/documents/customColumns", "models/database/documents/customColumnParams", "commands/database/documents/getCollectionsStatsCommand", "commands/database/documents/getCustomColumnsCommand", "commands/database/globalConfig/getEffectiveCustomFunctionsCommand", "commands/operations/getOperationStatusCommand", "commands/database/documents/generateClassCommand", "common/eventsCollector"], function (require, exports, app, router, appUrl, EVENTS, viewModelBase, deleteCollection, selectColumns, selectCsvColumnsDialog, showDataDialog, collection, customFunctions, customColumns, customColumnParams, getCollectionsStatsCommand, getCustomColumnsCommand, getEffectiveCustomFunctionsCommand, getOperationStatusCommand, generateClassCommand, eventsCollector) {
    "use strict";
    var documents = (function (_super) {
        __extends(documents, _super);
        function documents() {
            var _this = _super.call(this) || this;
            _this.displayName = "documents";
            _this.collections = ko.observableArray();
            _this.selectedCollection = ko.observable().subscribeTo("ActivateCollection").distinctUntilChanged();
            _this.allDocumentsCollection = ko.observable();
            _this.currentCollectionPagedItems = ko.observable();
            _this.currentColumnsParams = ko.observable(customColumns.empty());
            _this.currentCustomFunctions = ko.observable(customFunctions.empty());
            _this.selectedDocumentIndices = ko.observableArray();
            _this.contextName = ko.observable('');
            _this.currentCollection = ko.observable();
            _this.showLoadingIndicator = ko.observable(false);
            _this.showLoadingIndicatorThrottled = _this.showLoadingIndicator.throttle(250);
            _this.isAnyDocumentsAutoSelected = ko.observable(false);
            _this.isAllDocumentsAutoSelected = ko.observable(false);
            _this.lastCollectionCountUpdate = ko.observable();
            _this.isInitialized = documents.isInitialized;
            _this.showCollectionChanged = ko.observable(false);
            _this.documentsCount = ko.observable(0);
            _this.selectedCollection.subscribe(function (c) { return _this.selectedCollectionChanged(c); });
            _this.hasDocuments = ko.computed(function () {
                var selectedCollection = _this.selectedCollection();
                if (!!selectedCollection) {
                    if (selectedCollection.name === collection.allDocsCollectionName) {
                        return _this.documentsCount() > 0;
                    }
                    return _this.selectedCollection().documentCount() > 0;
                }
                return false;
            });
            _this.hasAnyDocumentsSelected = ko.computed(function () { return _this.selectedDocumentIndices().length > 0; });
            _this.hasAllDocumentsSelected = ko.computed(function () {
                var numOfSelectedDocuments = _this.selectedDocumentIndices().length;
                if (!!_this.selectedCollection() && numOfSelectedDocuments !== 0) {
                    return numOfSelectedDocuments === _this.selectedCollection().documentCount();
                }
                return false;
            });
            _this.documentsSelection = ko.computed(function () {
                var selected = _this.selectedDocumentIndices();
                if (_this.hasAllDocumentsSelected()) {
                    return 2 /* Checked */;
                }
                if (selected.length > 0) {
                    return 1 /* SomeChecked */;
                }
                return 0 /* UnChecked */;
            });
            _this.canCopyAllSelected = ko.computed(function () {
                _this.showLoadingIndicator(); //triggers computing the new cached selected items
                var numOfSelectedDocuments = _this.selectedDocumentIndices().length;
                var docsGrid = _this.getDocumentsGrid();
                if (!!docsGrid) {
                    var cachedItems = docsGrid.getNumberOfCachedItems();
                    return cachedItems >= numOfSelectedDocuments;
                }
                return false;
            });
            _this.isSystemDocumentsCollection = ko.computed(function () {
                var collection = _this.selectedCollection();
                return !!collection && collection.isSystemDocuments;
            });
            _this.isRegularCollection = ko.computed(function () {
                var collection = _this.selectedCollection();
                return !!collection && !collection.isAllDocuments && !collection.isSystemDocuments;
            });
            _this.selectedDocumentsText = ko.computed(function () {
                if (!!_this.selectedDocumentIndices()) {
                    var documentsText = "document";
                    if (_this.selectedDocumentIndices().length !== 1) {
                        documentsText += "s";
                    }
                    return documentsText;
                }
                return "";
            });
            _this.collectionsExceptAllDocs = ko.computed(function () {
                var allDocs = _this.allDocumentsCollection();
                if (!allDocs) {
                    return [];
                }
                var collections = _this.collections();
                return collections.filter(function (x) { return x !== allDocs; });
            });
            return _this;
        }
        documents.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            if (args.withStop) {
                viewModelBase.hasContinueTestOption(true);
            }
            this.fetchCustomFunctions();
            this.updateHelpLink("G8CDCP");
            // We can optionally pass in a collection name to view's URL, e.g. #/documents?collection=Foo&database="blahDb"
            this.collectionToSelectName = args ? args.collection : null;
            var db = this.activeDatabase();
            this.fetchCollectionsStats(db).done(function (results) {
                _this.collectionsLoaded(results, db);
                documents.isInitialized(true);
            });
        };
        documents.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            _super.prototype.createKeyboardShortcut.call(this, "F2", function () { return _this.editSelectedDoc(); }, "#documentsGrid");
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", function () { return _this.getDocumentsGrid().deleteSelectedItems(); }, docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", function () { return _this.copySelectedDocs(); }, docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", function () { return _this.copySelectedDocIds(); }, docsPageSelector);
            //this.registerCollectionsResize();
        };
        documents.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            documents.isInitialized(false);
            this.unregisterCollectionsResizing();
        };
        documents.prototype.registerCollectionsResize = function () {
            var resizingColumn = false;
            var startX = 0;
            var startingLeftWidth = 0;
            var startingRightWidth = 0;
            var totalStartingWidth = 0;
            $(document).on("mousedown.collectionsResize", ".collection-resize", function (e) {
                startX = e.pageX;
                resizingColumn = true;
                startingLeftWidth = $("#documents-page-container").innerWidth();
                startingRightWidth = $("#documents-page-right-container").innerWidth();
                totalStartingWidth = startingLeftWidth + startingRightWidth;
            });
            $(document).on("mouseup.collectionsResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.collectionsResize", "", function (e) {
                if (resizingColumn) {
                    // compute new percentage values
                    var diff = e.pageX - startX;
                    $("#documents-page-container").innerWidth(((startingLeftWidth + diff) * 100.0 / totalStartingWidth) + "%");
                    $("#documents-page-right-container").innerWidth(((startingRightWidth - diff) * 100.0 / totalStartingWidth) + "%");
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        documents.prototype.unregisterCollectionsResizing = function () {
            $(document).off("mousedown.collectionsResize");
            $(document).off("mouseup.collectionsResize");
            $(document).off("mousemove.collectionsResize");
        };
        documents.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                ko.postbox.subscribe("ViewItem", function () { return _this.editSelectedDoc(); }),
                ko.postbox.subscribe(EVENTS.ChangesApi.Reconnected, function (db) { return _this.reloadDocumentsData(db); }),
                ko.postbox.subscribe("SortCollections", function () { return _this.sortCollections(); })
            ];
        };
        documents.prototype.createNotifications = function () {
            var _this = this;
            return [
                //TODO: this.changesContext.currentResourceChangesApi().watchAllIndexes(() => this.refreshCollections()),
                this.changesContext.currentResourceChangesApi().watchAllDocs(function () { return _this.refreshCollections(); }),
            ];
        };
        documents.prototype.exportCsv = function () {
            eventsCollector.default.reportEvent("documents", "export-csv");
            this.exportCsvInternal();
        };
        documents.prototype.exportCsvInternal = function (customColumns) {
            if (this.isRegularCollection()) {
                var collection = this.selectedCollection();
                var db = this.activeDatabase();
                var url = appUrl.forExportCollectionCsv(collection, collection.ownerDatabase, customColumns);
                this.downloader.download(db, url);
            }
        };
        documents.prototype.fetchCollectionsStats = function (db) {
            return new getCollectionsStatsCommand(db).execute();
        };
        documents.prototype.refreshCollections = function () {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            this.fetchCollectionsStats(db)
                .done(function (results) {
                _this.documentsCount(results.numberOfDocuments());
                _this.updateCollections(results.collections);
                _this.refreshCollectionsData();
                deferred.resolve();
            });
            return deferred;
        };
        documents.prototype.collectionsLoaded = function (collectionsStats, db) {
            var _this = this;
            var collections = collectionsStats.collections;
            // Create the "All Documents" pseudo collection.
            this.allDocumentsCollection(collection.createAllDocsCollection(db));
            this.allDocumentsCollection().documentCount(collectionsStats.numberOfDocuments());
            // All systems a-go. Load them into the UI and select the first one.
            var allCollections = [this.allDocumentsCollection()].concat(collections);
            this.collections(allCollections);
            var collectionToSelect = allCollections.find(function (c) { return c.name === _this.collectionToSelectName; }) || this.allDocumentsCollection();
            collectionToSelect.activate();
        };
        documents.prototype.fetchCustomFunctions = function () {
            var _this = this;
            var customFunctionsCommand = new getEffectiveCustomFunctionsCommand(this.activeDatabase()).execute();
            customFunctionsCommand.done(function (cf) {
                _this.currentCustomFunctions(new customFunctions(cf.MergedDocument));
            });
        };
        //TODO: this binding has notification leak!
        documents.prototype.selectedCollectionChanged = function (selected) {
            var _this = this;
            if (!!selected) {
                var customColumnsCommand = selected.isAllDocuments ?
                    getCustomColumnsCommand.forAllDocuments(this.activeDatabase()) : getCustomColumnsCommand.forCollection(selected.name, this.activeDatabase());
                this.contextName(customColumnsCommand.docName);
                customColumnsCommand.execute().done(function (dto) {
                    if (dto) {
                        _this.currentColumnsParams().columns($.map(dto.Columns, function (c) { return new customColumnParams(c); }));
                        _this.currentColumnsParams().customMode(true);
                        selected.bindings(_this.currentColumnsParams().getBindings());
                    }
                    else {
                        // use default values!
                        selected.bindings(undefined);
                        _this.currentColumnsParams().columns.removeAll();
                        _this.currentColumnsParams().customMode(false);
                    }
                    var pagedList = selected.getDocuments();
                    _this.currentCollectionPagedItems(pagedList);
                    _this.currentCollection(selected);
                });
            }
        };
        documents.prototype.deleteCollection = function (collection) {
            var _this = this;
            if (collection) {
                var viewModel = new deleteCollection(collection, this.activeDatabase());
                viewModel.deletionTask.done(function (result) {
                    if (!collection.isAllDocuments) {
                        _this.collections.remove(collection);
                        var selectedCollection = _this.selectedCollection();
                        if (collection.name === selectedCollection.name) {
                            _this.selectCollection(_this.allDocumentsCollection());
                        }
                    }
                    else {
                        _this.selectNone();
                    }
                    _this.updateGridAfterOperationComplete(collection, result.OperationId);
                });
                app.showBootstrapDialog(viewModel);
            }
        };
        documents.prototype.updateGridAfterOperationComplete = function (collection, operationId) {
            var getOperationStatusTask = new getOperationStatusCommand(collection.ownerDatabase, operationId);
            getOperationStatusTask.execute();
            /* TODO .done((result: bulkOperationStatusDto) => {
                if (result.Completed) {
                    var selectedCollection: collection = this.selectedCollection();

                    if (selectedCollection.isAllDocuments) {
                        var docsGrid = this.getDocumentsGrid();
                        docsGrid.refreshCollectionData();
                    } else {
                        var allDocumentsPagedList = this.allDocumentsCollection().getDocuments();
                        allDocumentsPagedList.invalidateCache();
                    }
                } else {
                    setTimeout(() => this.updateGridAfterOperationComplete(collection, operationId), 500);
                }
            });*/
        };
        documents.prototype.updateCollections = function (receivedCollections) {
            var _this = this;
            var deletedCollections = [];
            this.collections().forEach(function (col) {
                if (!receivedCollections.find(function (receivedCol) { return col.name === receivedCol.name; }) && col.name !== "System Documents" && col.name !== "All Documents") {
                    deletedCollections.push(col);
                }
            });
            this.collections.removeAll(deletedCollections);
            //update collections, including collection count
            receivedCollections.forEach(function (receivedCol) {
                var foundCollection = _this.collections().find(function (col) { return col.name === receivedCol.name; });
                if (!foundCollection) {
                    _this.collections.push(receivedCol);
                }
                else {
                    var oldCount = foundCollection.documentCount();
                    var newCount = receivedCol.documentCount();
                    var selectedCollection = _this.selectedCollection();
                    if (oldCount !== newCount) {
                        if (selectedCollection.name === receivedCol.name || selectedCollection.isAllDocuments) {
                            _this.showCollectionChanged(true);
                        }
                    }
                    foundCollection.documentCount(receivedCol.documentCount());
                }
            });
            //if the collection is deleted, go to the all documents collection
            var currentCollection = this.collections().find(function (c) { return c.name === _this.selectedCollection().name; });
            if (!currentCollection || currentCollection.documentCount() === 0) {
                this.selectCollection(this.allDocumentsCollection());
            }
        };
        documents.prototype.refreshCollectionsData = function () {
            this.collections().forEach(function (collection) {
                var pagedList = collection.getDocuments();
                pagedList.invalidateCache();
            });
        };
        documents.prototype.reloadDocumentsData = function (db) {
            var _this = this;
            if (db.name === this.activeDatabase().name) {
                this.refreshCollections().done(function () {
                    _this.refreshCollectionsData();
                });
            }
        };
        documents.prototype.selectCollection = function (collection, event) {
            if (!event || event.which !== 3) {
                var documentsWithCollectionUrl = appUrl.forDocuments(collection.name, this.activeDatabase());
                if (event.ctrlKey) {
                    window.open(documentsWithCollectionUrl);
                    $(document.activeElement).blur();
                }
                else {
                    router.navigate(documentsWithCollectionUrl, false);
                    this.showCollectionChanged(false);
                    collection.activate();
                }
            }
        };
        documents.prototype.selectCsvColumns = function () {
            var _this = this;
            eventsCollector.default.reportEvent("documents", "export-csv-custom-columns");
            var dialog = new selectCsvColumnsDialog(this.getDocumentsGrid().getColumnsNames());
            app.showBootstrapDialog(dialog);
            dialog.onExit().done(function (cols) {
                _this.exportCsvInternal(cols);
            });
        };
        documents.prototype.selectColumns = function () {
            var _this = this;
            eventsCollector.default.reportEvent("documents", "select-columns");
            // Fetch column widths from virtual table
            var virtualTable = this.getDocumentsGrid();
            var columnsNames = virtualTable.getColumnsNames();
            var vtColumns = virtualTable.columns();
            this.currentColumnsParams().columns().forEach(function (column) {
                for (var i = 0; i < vtColumns.length; i++) {
                    if (column.binding() === vtColumns[i].binding) {
                        column.width(vtColumns[i].width() | 0);
                        break;
                    }
                }
            });
            var selectColumnsViewModel = new selectColumns(this.currentColumnsParams().clone(), this.currentCustomFunctions(), this.contextName(), this.activeDatabase(), columnsNames);
            app.showBootstrapDialog(selectColumnsViewModel);
            selectColumnsViewModel.onExit().done(function (cols) {
                _this.currentColumnsParams(cols);
                _this.currentCollection().bindings(_this.currentColumnsParams().getBindings());
                var pagedList = _this.currentCollection().getDocuments();
                pagedList.invalidateCache();
                _this.currentCollectionPagedItems(pagedList);
            });
        };
        documents.prototype.newDocument = function (docs, $event) {
            eventsCollector.default.reportEvent("document", "new");
            var url = appUrl.forNewDoc(this.activeDatabase());
            if ($event.ctrlKey) {
                window.open(url);
            }
            else {
                router.navigate(url);
            }
        };
        documents.prototype.newDocumentInCollection = function (docs, $event) {
            var url = appUrl.forNewDoc(this.activeDatabase(), this.currentCollection().name);
            if ($event.ctrlKey) {
                window.open(url);
            }
            else {
                router.navigate(url);
            }
        };
        documents.prototype.refresh = function () {
            eventsCollector.default.reportEvent("documents", "refresh");
            this.getDocumentsGrid().refreshCollectionData();
            var selectedCollection = this.selectedCollection();
            selectedCollection.invalidateCache();
            this.selectNone();
            this.showCollectionChanged(false);
        };
        documents.prototype.toggleSelectAll = function () {
            var docsGrid = this.getDocumentsGrid();
            if (!!docsGrid) {
                if (this.hasAnyDocumentsSelected()) {
                    docsGrid.selectNone();
                }
                else {
                    docsGrid.selectSome();
                    this.isAnyDocumentsAutoSelected(this.hasAllDocumentsSelected() == false);
                }
            }
        };
        documents.prototype.selectAll = function () {
            var docsGrid = this.getDocumentsGrid();
            var c = this.selectedCollection();
            if (!!docsGrid && !!c) {
                docsGrid.selectAll(c.documentCount());
            }
        };
        documents.prototype.selectNone = function () {
            var docsGrid = this.getDocumentsGrid();
            if (!!docsGrid) {
                docsGrid.selectNone();
            }
        };
        documents.prototype.editSelectedDoc = function () {
            eventsCollector.default.reportEvent("document", "edit");
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.editLastSelectedItem();
            }
        };
        documents.prototype.deleteSelectedDocs = function () {
            if (this.selectedCollection().isSystemDocuments === false && this.hasAllDocumentsSelected() && !this.selectedCollection().isAllDocuments) {
                eventsCollector.default.reportEvent("collection", "delete");
                this.deleteCollection(this.selectedCollection());
            }
            else {
                eventsCollector.default.reportEvent("documents", "delete");
                var grid = this.getDocumentsGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        documents.prototype.copySelectedDocs = function () {
            eventsCollector.default.reportEvent("documents", "copy");
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.copySelectedDocs();
            }
        };
        documents.prototype.copySelectedDocIds = function () {
            eventsCollector.default.reportEvent("documents", "copy-ids");
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.copySelectedDocIds();
            }
        };
        documents.prototype.generateDocCode = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                var selectedItem = grid.getSelectedItems(1)[0];
                var metadata = selectedItem["__metadata"];
                var id = metadata["id"];
                var generate = new generateClassCommand(this.activeDatabase(), id, "csharp");
                var deffered = generate.execute();
                deffered.done(function (code) {
                    app.showBootstrapDialog(new showDataDialog("Generated Class", code["Code"]));
                });
            }
        };
        documents.prototype.getDocumentsGrid = function () {
            var gridContents = $(documents.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        documents.prototype.sortCollections = function () {
            this.collections.sort(function (c1, c2) {
                if (c1.isAllDocuments)
                    return -1;
                if (c2.isAllDocuments)
                    return 1;
                return c1.name.toLowerCase() > c2.name.toLowerCase() ? 1 : -1;
            });
        };
        // Animation callbacks for the groups list
        documents.prototype.showCollectionElement = function (element) {
            if (element.nodeType === 1 && documents.isInitialized()) {
                $(element).hide().slideDown(500, function () {
                    ko.postbox.publish("SortCollections");
                    $(element).highlight();
                });
            }
        };
        documents.prototype.hideCollectionElement = function (element) {
            if (element.nodeType === 1) {
                $(element).slideUp(1000, function () { $(element).remove(); });
            }
        };
        return documents;
    }(viewModelBase));
    documents.gridSelector = "#documentsGrid";
    documents.isInitialized = ko.observable(false);
    return documents;
});

//# sourceMappingURL=documents.js.map
