var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "models/database/documents/document", "models/database/documents/documentMetadata", "models/database/documents/collection", "commands/database/documents/saveDocumentCommand", "commands/database/documents/getDocumentWithMetadataCommand", "commands/database/studio/resolveMergeCommand", "commands/database/documents/getDocumentsFromCollectionCommand", "commands/database/documents/generateClassCommand", "common/appUrl", "common/jsonUtil", "common/messagePublisher", "common/bindingHelpers/aceEditorBindingHandler", "common/generalUtils", "common/helpers/database/documentHelpers", "common/copyToClipboard", "viewmodels/common/deleteDocuments", "viewmodels/viewModelBase", "viewmodels/common/showDataDialog", "viewmodels/database/documents/editDocumentConnectedDocuments", "common/eventsCollector"], function (require, exports, app, router, document, documentMetadata, collection, saveDocumentCommand, getDocumentWithMetadataCommand, resolveMergeCommand, getDocumentsFromCollectionCommand, generateClassCommand, appUrl, jsonUtil, messagePublisher, aceEditorBindingHandler, genUtils, documentHelpers, copyToClipboard, deleteDocuments, viewModelBase, showDataDialog, connectedDocuments, eventsCollector) {
    "use strict";
    var editDocument = (function (_super) {
        __extends(editDocument, _super);
        function editDocument() {
            var _this = _super.call(this) || this;
            _this.document = ko.observable();
            _this.documentText = ko.observable("");
            _this.isCreatingNewDocument = ko.observable(false);
            _this.collectionForNewDocument = ko.observable();
            _this.provideCustomNameForNewDocument = ko.observable(false);
            _this.userIdHasFocus = ko.observable(false);
            _this.userSpecifiedId = ko.observable("");
            _this.entityName = ko.observable("");
            _this.isConflictDocument = ko.observable(false);
            _this.isNewLineFriendlyMode = ko.observable(false);
            _this.autoCollapseMode = ko.observable(false);
            _this.isSaving = ko.observable(false);
            _this.displayDocumentChangeNotification = ko.observable(false);
            _this.metaPropsToRestoreOnSave = [];
            _this.connectedDocuments = new connectedDocuments(_this.document, _this.activeDatabase, function (docId) { return _this.loadDocument(docId); });
            aceEditorBindingHandler.install();
            _this.initializeObservables();
            return _this;
        }
        editDocument.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            if (args && args.id) {
                return this.activateById(args.id);
            }
            else {
                return $.Deferred().resolve({ can: true });
            }
        };
        editDocument.prototype.activate = function (navigationArgs) {
            var _this = this;
            _super.prototype.activate.call(this, navigationArgs);
            this.updateHelpLink('M72H1R');
            this.dirtyFlag = new ko.DirtyFlag([this.documentText, this.userSpecifiedId], false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.pureComputed(function () {
                var dirty = _this.dirtyFlag().isDirty();
                var isSaving = _this.isSaving();
                return dirty && !isSaving;
            });
            if (navigationArgs && navigationArgs.id) {
                ko.postbox.publish("SetRawJSONUrl", appUrl.forDocumentRawData(this.activeDatabase(), navigationArgs.id));
            }
            else {
                return this.editNewDocument(navigationArgs ? navigationArgs.new : null);
            }
        };
        // Called when the view is attached to the DOM.
        editDocument.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.setupKeyboardShortcuts();
            this.$documentName = $(editDocument.documentNameSelector);
            this.$docEditor = $(editDocument.docEditorSelector);
            this.isNewLineFriendlyMode.subscribe(function (val) {
                _this.updateNewlineLayoutInDocument(val);
            });
        };
        editDocument.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.docEditor = aceEditorBindingHandler.getEditorBySelection(this.$docEditor);
            // preload json newline friendly mode to avoid issues with document save
            ace.config.loadModule("ace/mode/json_newline_friendly");
            this.$docEditor.on('DynamicHeightSet', function () { return _this.docEditor.resize(); });
            this.focusOnEditor();
        };
        editDocument.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.$docEditor.off('DynamicHeightSet');
        };
        editDocument.prototype.activateById = function (id) {
            var _this = this;
            var canActivateResult = $.Deferred();
            this.loadDocument(id)
                .done(function () {
                canActivateResult.resolve({ can: true });
            })
                .fail(function () {
                messagePublisher.reportError("Could not find " + id + " document");
                canActivateResult.resolve({ redirect: appUrl.forDocuments(collection.allDocsCollectionName, _this.activeDatabase()) });
            });
            return canActivateResult;
        };
        editDocument.prototype.createNotifications = function () {
            this.syncChangeNotification();
            return [];
        };
        editDocument.prototype.syncChangeNotification = function () {
            if (this.changeNotification) {
                this.removeNotification(this.changeNotification);
                this.changeNotification.off();
            }
            if (!this.document() || this.isCreatingNewDocument())
                return;
            this.changeNotification = this.createDocumentChangeNotification(this.document().getId());
            this.addNotification(this.changeNotification);
        };
        editDocument.prototype.initializeObservables = function () {
            var _this = this;
            this.userSpecifiedIdCustomValidityError = ko.computed(function () {
                var documentId = _this.userSpecifiedId();
                return documentId.includes("\\") ? "Document name cannot contain '\\'" : "";
            });
            this.document.subscribe(function (doc) {
                if (doc) {
                    if (_this.isConflictDocument()) {
                        _this.resolveConflicts();
                    }
                    else {
                        var docDto = doc.toDto(true);
                        var metaDto = docDto["@metadata"];
                        if (metaDto) {
                            _this.metaPropsToRestoreOnSave.length = 0;
                            documentMetadata.filterMetadata(metaDto, _this.metaPropsToRestoreOnSave);
                        }
                        var docText = _this.stringify(docDto);
                        _this.documentText(docText);
                    }
                }
            });
            this.metadata = ko.pureComputed(function () { return _this.document() ? _this.document().__metadata : null; });
            this.isConflictDocument = ko.computed(function () {
                var metadata = _this.metadata();
                return metadata && ("Raven-Replication-Conflict" in metadata) && !metadata.id.includes("/conflicts/");
            });
            this.documentSize = ko.pureComputed(function () {
                try {
                    var size = genUtils.getSizeInBytesAsUTF8(_this.documentText()) / 1024;
                    return genUtils.formatAsCommaSeperatedString(size, 2);
                }
                catch (e) {
                    return "cannot compute";
                }
            });
            this.metadata.subscribe(function (meta) {
                if (meta && meta.id) {
                    _this.userSpecifiedId(meta.id);
                    _this.entityName(document.getEntityNameFromId(meta.id));
                }
            });
            this.editedDocId = ko.pureComputed(function () { return _this.metadata() ? _this.metadata().id : ""; });
            this.editedDocId.subscribe(function (docId) {
                return ko.postbox.publish("SetRawJSONUrl", docId ? appUrl.forDocumentRawData(_this.activeDatabase(), docId) : "");
            });
            this.displayLastModifiedDate = ko.pureComputed(function () {
                var hasMetadata = !!_this.metadata();
                var inEditMode = !_this.isCreatingNewDocument();
                var displayChangedNotification = _this.displayDocumentChangeNotification();
                return hasMetadata && inEditMode && !displayChangedNotification;
            });
        };
        editDocument.prototype.enableCustomNameProvider = function () {
            this.provideCustomNameForNewDocument(true);
            this.userIdHasFocus(true);
        };
        editDocument.prototype.createDocumentChangeNotification = function (docId) {
            var _this = this;
            return this.changesContext.currentResourceChangesApi().watchDocument(docId, function (n) { return _this.documentChangeNotification(n); });
        };
        editDocument.prototype.documentChangeNotification = function (n) {
            if (this.isSaving() || n.Etag === this.metadata().etag()) {
                return;
            }
            this.displayDocumentChangeNotification(true);
        };
        editDocument.prototype.updateNewlineLayoutInDocument = function (unescapeNewline) {
            var dirtyFlagValue = this.dirtyFlag().isDirty();
            if (unescapeNewline) {
                this.documentText(documentHelpers.unescapeNewlinesAndTabsInTextFields(this.documentText()));
                this.docEditor.getSession().setMode('ace/mode/json_newline_friendly');
            }
            else {
                this.documentText(documentHelpers.escapeNewlinesAndTabsInTextFields(this.documentText()));
                this.docEditor.getSession().setMode('ace/mode/json');
                this.formatDocument();
            }
            if (!dirtyFlagValue) {
                this.dirtyFlag().reset();
            }
        };
        editDocument.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+r", function () { return _this.refreshDocument(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+c", function () { return _this.focusOnEditor(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteDocument(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+s", function () { return _this.saveDocument(); }, editDocument.editDocSelector); // Q. Why do we have to setup ALT+S, when we could just use HTML's accesskey attribute? A. Because the accesskey attribute causes the save button to take focus, thus stealing the focus from the user's editing spot in the doc editor, disrupting his workflow.
        };
        editDocument.prototype.focusOnEditor = function () {
            this.docEditor.focus();
        };
        editDocument.prototype.editNewDocument = function (collectionForNewDocument) {
            var _this = this;
            this.isCreatingNewDocument(true);
            this.collectionForNewDocument(collectionForNewDocument);
            var documentTask = $.Deferred();
            if (collectionForNewDocument) {
                this.userSpecifiedId(collectionForNewDocument + "/");
                new getDocumentsFromCollectionCommand(new collection(collectionForNewDocument, this.activeDatabase()), 0, 3)
                    .execute()
                    .done(function (documents) {
                    var schemaDoc = documentHelpers.findSchema(documents.items);
                    _this.document(schemaDoc);
                    documentTask.resolve(schemaDoc);
                })
                    .fail(function () { return documentTask.reject(); });
            }
            else {
                var doc = document.empty();
                this.document(doc);
                documentTask.resolve(doc);
            }
            return documentTask;
        };
        editDocument.prototype.toClipboard = function () {
            copyToClipboard.copy(this.documentText(), "Document has been copied to clipboard");
        };
        editDocument.prototype.toggleNewlineMode = function () {
            var _this = this;
            eventsCollector.default.reportEvent("document", "toggle-newline-mode");
            if (this.isNewLineFriendlyMode() === false && parseInt(this.documentSize().replace(",", "")) > 1024) {
                app.showBootstrapMessage("This operation might take long time with big documents, are you sure you want to continue?", "Toggle newline mode", ["Cancel", "Continue"])
                    .then(function (dialogResult) {
                    if (dialogResult === "Continue") {
                        _this.isNewLineFriendlyMode.toggle();
                    }
                });
            }
            else {
                this.isNewLineFriendlyMode.toggle();
            }
        };
        editDocument.prototype.toggleAutoCollapse = function () {
            eventsCollector.default.reportEvent("document", "toggle-auto-collapse");
            this.autoCollapseMode.toggle();
            if (this.autoCollapseMode()) {
                this.foldAll();
            }
            else {
                this.docEditor.getSession().unfold(null, true);
            }
        };
        editDocument.prototype.foldAll = function () {
            var _this = this;
            var AceRange = ace.require("ace/range").Range;
            this.docEditor.getSession().foldAll();
            var folds = this.docEditor.getSession().getFoldsInRange(new AceRange(0, 0, this.docEditor.getSession().getLength(), 0));
            folds.map(function (f) { return _this.docEditor.getSession().expandFold(f); });
        };
        editDocument.prototype.createClone = function () {
            // Show current document as a new document..
            this.isCreatingNewDocument(true);
            this.syncChangeNotification();
            // Clear data..
            this.userSpecifiedId("");
            this.metadata().etag(null);
            this.metadata().ravenLastModified(null);
        };
        editDocument.prototype.saveDocument = function () {
            eventsCollector.default.reportEvent("document", "save");
            this.saveInternal(this.userSpecifiedId(), false);
        };
        editDocument.prototype.saveInternal = function (documentId, eraseEtag) {
            var _this = this;
            var message = "";
            var updatedDto;
            try {
                if (this.isNewLineFriendlyMode()) {
                    updatedDto = JSON.parse(documentHelpers.escapeNewlinesAndTabsInTextFields(this.documentText()));
                }
                else {
                    updatedDto = JSON.parse(this.documentText());
                }
            }
            catch (e) {
                if (updatedDto == undefined) {
                    message = "The document data isn't a legal JSON expression!";
                }
                this.focusOnEditor();
            }
            if (message) {
                messagePublisher.reportError(message, undefined, undefined, false);
                return;
            }
            if (!updatedDto['@metadata']) {
                updatedDto["@metadata"] = {};
            }
            var meta = updatedDto['@metadata'];
            // Fix up the metadata: if we're a new doc, attach the expected reserved properties like ID, ETag, and RavenEntityName.
            // AFAICT, Raven requires these reserved meta properties in order for the doc to be seen as a member of a collection.
            if (this.isCreatingNewDocument()) {
                this.attachReservedMetaProperties(documentId, meta);
            }
            else {
                // If we're editing a document, we hide some reserved properties from the user.
                // Restore these before we save.
                this.metaPropsToRestoreOnSave.forEach(function (p) {
                    if (p.name !== "Origin") {
                        meta[p.name] = p.value;
                    }
                });
                // force document id to support save as new
                meta['@id'] = documentId;
                if (eraseEtag) {
                    meta['@etag'] = 0;
                }
            }
            // skip some not necessary meta in headers
            var metaToSkipInHeaders = ['Raven-Replication-History'];
            for (var i in metaToSkipInHeaders) {
                var skippedHeader = metaToSkipInHeaders[i];
                delete meta[skippedHeader];
            }
            var newDoc = new document(updatedDto);
            var saveCommand = new saveDocumentCommand(documentId, newDoc, this.activeDatabase());
            this.isSaving(true);
            saveCommand
                .execute()
                .done(function (saveResult) { return _this.onDocumentSaved(saveResult); });
        };
        editDocument.prototype.onDocumentSaved = function (saveResult) {
            var _this = this;
            var savedDocumentDto = saveResult.Results[0];
            var currentSelection = this.docEditor.getSelectionRange();
            this.loadDocument(savedDocumentDto.Key)
                .always(function () {
                _this.updateNewlineLayoutInDocument(_this.isNewLineFriendlyMode());
                // Try to restore the selection.
                _this.docEditor.selection.setRange(currentSelection, false);
                _this.isSaving(false);
                _this.syncChangeNotification();
            });
            this.updateUrl(savedDocumentDto.Key);
            this.dirtyFlag().reset(); //Resync Changes
            this.isCreatingNewDocument(false);
            this.collectionForNewDocument(null);
        };
        editDocument.prototype.attachReservedMetaProperties = function (id, target) {
            target['@etag'] = 0;
            target['Raven-Entity-Name'] = target['Raven-Entity-Name'] || document.getEntityNameFromId(id);
            target['@id'] = id;
        };
        editDocument.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };
        editDocument.prototype.loadDocument = function (id) {
            var _this = this;
            this.isBusy(true);
            return new getDocumentWithMetadataCommand(id, this.activeDatabase())
                .execute()
                .done(function (doc) {
                _this.document(doc);
                _this.dirtyFlag().reset();
                if (_this.autoCollapseMode()) {
                    _this.foldAll();
                }
            })
                .fail(function () { return messagePublisher.reportError("Could not find " + id + " document"); })
                .always(function () { return _this.isBusy(false); });
        };
        editDocument.prototype.refreshDocument = function () {
            var _this = this;
            eventsCollector.default.reportEvent("document", "refresh");
            this.canContinueIfNotDirty("Refresh", "You have unsaved data. Are you sure you want to continue?")
                .done(function () {
                var docId = _this.editedDocId();
                _this.document(null);
                _this.documentText(null);
                _this.userSpecifiedId("");
                _this.loadDocument(docId);
                _this.displayDocumentChangeNotification(false);
            });
        };
        editDocument.prototype.deleteDocument = function () {
            var _this = this;
            eventsCollector.default.reportEvent("document", "delete");
            var doc = this.document();
            if (doc) {
                var viewModel = new deleteDocuments([doc], this.activeDatabase());
                viewModel.deletionTask.done(function () { return _this.connectedDocuments.onDocumentDeleted(); });
                app.showBootstrapDialog(viewModel, editDocument.editDocSelector);
            }
        };
        editDocument.prototype.formatDocument = function () {
            eventsCollector.default.reportEvent("document", "format");
            try {
                var docEditorText = this.docEditor.getSession().getValue();
                var tempDoc = JSON.parse(docEditorText);
                var formatted = this.stringify(tempDoc);
                this.documentText(formatted);
            }
            catch (e) {
                messagePublisher.reportError("Could not format json", undefined, undefined, false);
            }
        };
        editDocument.prototype.navigateToCollection = function (collectionName) {
            var collectionUrl = appUrl.forDocuments(collectionName, this.activeDatabase());
            router.navigate(collectionUrl);
        };
        editDocument.prototype.updateUrl = function (docId) {
            var editDocUrl = appUrl.forEditDoc(docId, this.activeDatabase());
            router.navigate(editDocUrl, false);
        };
        editDocument.prototype.resolveConflicts = function () {
            var _this = this;
            eventsCollector.default.reportEvent("document", "resolve-conflicts");
            new resolveMergeCommand(this.activeDatabase(), this.editedDocId())
                .execute()
                .done(function (response) {
                _this.documentText(response.Document);
                //TODO: handle metadata
            });
        };
        editDocument.prototype.generateClass = function () {
            eventsCollector.default.reportEvent("document", "generate-csharp-class");
            var doc = this.document();
            var generate = new generateClassCommand(this.activeDatabase(), doc.getId(), "csharp");
            var deffered = generate.execute();
            deffered.done(function (code) { return app.showBootstrapDialog(new showDataDialog("Generated Class", code, null)); });
        };
        return editDocument;
    }(viewModelBase));
    editDocument.editDocSelector = "#editDocumentContainer";
    editDocument.documentNameSelector = "#documentName";
    editDocument.docEditorSelector = "#docEditor";
    return editDocument;
});

//# sourceMappingURL=editDocument.js.map
