var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/database/documents/getCollectionsStatsCommand", "common/bindingHelpers/aceEditorBindingHandler", "common/appUrl", "models/database/query/dataExplorationRequest", "commands/database/query/dataExplorationCommand", "common/pagedResultSet", "common/pagedList", "models/database/documents/document", "common/messagePublisher", "common/eventsCollector"], function (require, exports, viewModelBase, getCollectionsStatsCommand, aceEditorBindingHandler, appUrl, dataExplorationRequest, dataExplorationCommand, pagedResultSet, pagedList, document, messagePublisher, eventsCollector) {
    "use strict";
    var exploration = (function (_super) {
        __extends(exploration, _super);
        function exploration() {
            var _this = _super.call(this) || this;
            _this.collections = ko.observableArray([]);
            _this.isBusy = ko.observable(false);
            _this.explorationRequest = dataExplorationRequest.empty();
            _this.queryResults = ko.observable();
            _this.isLoading = ko.observable(false);
            _this.dataLoadingXhr = ko.observable();
            _this.selectedCollectionLabel = ko.computed(function () { return _this.explorationRequest.collection() || "Select a collection"; });
            _this.runEnabled = ko.computed(function () { return !!_this.explorationRequest.collection(); });
            _this.appUrls = appUrl.forCurrentDatabase();
            aceEditorBindingHandler.install();
            return _this;
        }
        exploration.prototype.canActivate = function (args) {
            var _this = this;
            var deffered = $.Deferred();
            new getCollectionsStatsCommand(this.activeDatabase())
                .execute()
                .done(function (collectionStats) {
                _this.collections(collectionStats.collections.map(function (x) { return x.name; }));
            })
                .always(function () { return deffered.resolve({ can: true }); });
            return deffered;
        };
        exploration.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("FP59PJ");
        };
        exploration.prototype.exportCsv = function () {
            eventsCollector.default.reportEvent("exploration", "export-csv");
            var db = this.activeDatabase();
            var url = new dataExplorationCommand(this.explorationRequest.toDto(), db).getCsvUrl();
            this.downloader.download(db, url);
        };
        exploration.prototype.runExploration = function () {
            var _this = this;
            eventsCollector.default.reportEvent("exploration", "run");
            this.isBusy(true);
            var requestDto = this.explorationRequest.toDto();
            var command = new dataExplorationCommand(requestDto, this.activeDatabase());
            command.execute()
                .done(function (results) {
                if (results.Error) {
                    messagePublisher.reportError("Unable to execute query", results.Error);
                }
                else {
                    var mainSelector = new pagedResultSet(results.Results.map(function (d) { return new document(d); }), results.Results.length, results);
                    var resultsFetcher = function (skip, take) {
                        var slicedResult = new pagedResultSet(mainSelector.items.slice(skip, Math.min(skip + take, mainSelector.totalResultCount)), mainSelector.totalResultCount);
                        return $.Deferred().resolve(slicedResult).promise();
                    };
                    var resultsList = new pagedList(resultsFetcher);
                    _this.queryResults(resultsList);
                }
            })
                .always(function () {
                _this.isBusy(false);
            });
            this.dataLoadingXhr(command.xhr);
        };
        exploration.prototype.killTask = function () {
            eventsCollector.default.reportEvent("exploration", "kill");
            var xhr = this.dataLoadingXhr();
            if (xhr) {
                xhr.abort();
            }
            this.isBusy(false);
            this.queryResults(null);
        };
        return exploration;
    }(viewModelBase));
    return exploration;
});

//# sourceMappingURL=exploration.js.map
