var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "common/mergedIndexesStorage", "commands/resources/getDatabaseStatsCommand", "moment", "plugins/dialog", "common/optional", "viewmodels/database/indexes/deleteIndexesConfirm", "common/eventsCollector", "common/generalUtils"], function (require, exports, viewModelBase, appUrl, mergedIndexesStorage, getDatabaseStatsCommand, moment, dialog, optional, deleteIndexesConfirm, eventsCollector, genUtils) {
    "use strict";
    var indexMergeSuggestions = (function (_super) {
        __extends(indexMergeSuggestions, _super);
        function indexMergeSuggestions() {
            var _this = _super.call(this) || this;
            _this.suggestions = ko.observableArray();
            _this.unmergables = ko.observableArray();
            _this.idleOrAbandonedIndexes = ko.observableArray();
            _this.notUsedForLastWeek = ko.observableArray();
            _this.appUrls = appUrl.forCurrentDatabase();
            return _this;
        }
        indexMergeSuggestions.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            this.reload()
                .done(function () { return deferred.resolve({ can: true }); })
                .fail(function () { return deferred.resolve({ redirect: appUrl.forIndexes(_this.activeDatabase()) }); });
            return deferred;
        };
        indexMergeSuggestions.prototype.reload = function () {
            var fetchIndexMergeSuggestionsTask = this.fetchIndexMergeSuggestions();
            var fetchStatsTask = this.fetchStats();
            return $.when(fetchIndexMergeSuggestionsTask, fetchStatsTask);
        };
        indexMergeSuggestions.prototype.createNotifications = function () {
            return [];
        };
        indexMergeSuggestions.prototype.fetchStats = function () {
            var db = this.activeDatabase();
            if (db) {
                return new getDatabaseStatsCommand(db)
                    .execute();
            }
            return null;
        };
        indexMergeSuggestions.prototype.processStatsResults = function (stats) {
            var _this = this;
            this.idleOrAbandonedIndexes([]);
            this.notUsedForLastWeek([]);
            var now = moment();
            var miliSecondsInWeek = 1000 * 3600 * 24 * 7;
            stats.Indexes.forEach(function (indexDto) {
                // we are using contains not equals as priority may contains 
                if (indexDto.Priority.includes("Idle") || indexDto.Priority.includes("Abandoned")) {
                    _this.idleOrAbandonedIndexes.push(indexDto);
                }
                if (indexDto.LastQueryTimestamp) {
                    var lastQueryDate = moment(indexDto.LastQueryTimestamp);
                    if (lastQueryDate.isValid()) {
                        var agoInMs = now.diff(lastQueryDate);
                        if (agoInMs > miliSecondsInWeek) {
                            indexDto["LastQueryTimestampText"] = optional.val(indexDto.LastQueryTimestamp).bind(function (v) { return genUtils.toHumanizedDate(v); });
                            _this.notUsedForLastWeek.push(indexDto);
                        }
                    }
                }
            });
        };
        indexMergeSuggestions.prototype.fetchIndexMergeSuggestions = function () {
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            /* TODO
            new getIndexMergeSuggestionsCommand(db)
                .execute()
                .done((results: indexMergeSuggestionsDto) => {
                    var suggestions = results.Suggestions.map((suggestion: suggestionDto) => new indexMergeSuggestion(suggestion));
                    this.suggestions(suggestions);
    
                    var unmergables = Object.keys(results.Unmergables).map((value, index) => {
                        return { indexName: value, reason: results.Unmergables[value] }
                    });
                    this.unmergables(unmergables);
                    deferred.resolve();
                })
                .fail(() => deferred.reject());
            */
            return deferred;
        };
        indexMergeSuggestions.prototype.mergeSuggestionIndex = function (index) {
            return parseInt(index) + 1;
        };
        indexMergeSuggestions.prototype.mergedIndexUrl = function (id) {
            var db = this.activeDatabase();
            var mergedIndexName = mergedIndexesStorage.getMergedIndexName(db, id);
            return this.appUrls.editIndex(mergedIndexName);
        };
        indexMergeSuggestions.prototype.saveMergedIndex = function (id, suggestion) {
            eventsCollector.default.reportEvent("index-merge-suggestions", "save-merged");
            var db = this.activeDatabase();
            mergedIndexesStorage.saveMergedIndex(db, id, suggestion);
            return true;
        };
        indexMergeSuggestions.prototype.deleteIndexes = function (index) {
            var _this = this;
            eventsCollector.default.reportEvent("index-merge-suggestions", "delete-indexes");
            var mergeSuggestion = this.suggestions()[index];
            var indexesToDelete = mergeSuggestion.canDelete;
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm(indexesToDelete, db);
            deleteViewModel.deleteTask.always(function () { return _this.reload(); });
            dialog.show(deleteViewModel);
        };
        indexMergeSuggestions.prototype.deleteIndex = function (name) {
            var _this = this;
            eventsCollector.default.reportEvent("index-merge-suggestions", "delete-index");
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm([name], db);
            deleteViewModel.deleteTask.always(function () { return _this.reload(); });
            dialog.show(deleteViewModel);
        };
        indexMergeSuggestions.prototype.deleteAllIdleOrAbandoned = function () {
            var _this = this;
            eventsCollector.default.reportEvent("index-merge-suggestions", "delete-all-idle-or-abandoned");
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm(this.idleOrAbandonedIndexes().map(function (index) { return index.Name; }), db, "Delete all idle or abandoned indexes?");
            deleteViewModel.deleteTask.always(function () { return _this.reload(); });
            dialog.show(deleteViewModel);
        };
        indexMergeSuggestions.prototype.deleteAllNotUsedForWeek = function () {
            var _this = this;
            eventsCollector.default.reportEvent("index-merge-suggestions", "delete-all-not-used-for-week");
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm(this.notUsedForLastWeek().map(function (index) { return index.Name; }), db, "Delete all indexes not used within last week?");
            deleteViewModel.deleteTask.always(function () { return _this.reload(); });
            dialog.show(deleteViewModel);
        };
        return indexMergeSuggestions;
    }(viewModelBase));
    return indexMergeSuggestions;
});

//# sourceMappingURL=indexMergeSuggestions.js.map
