var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "durandal/app", "viewmodels/database/indexes/indexPerformanceDetails", "commands/database/debug/getIndexesPerformance", "common/fileDownloader", "common/helpers/graph/graphHelper", "d3", "rbush", "common/helpers/graph/gapFinder", "common/generalUtils", "common/helpers/graph/rangeAggregator"], function (require, exports, viewModelBase, app, indexPerformanceDetails, getIndexesPerformance, fileDownloader, graphHelper, d3, rbush, gapFinder, generalUtils, rangeAggregator) {
    "use strict";
    var hitTest = (function () {
        function hitTest() {
            this.cursor = ko.observable("auto");
            this.rTree = rbush();
        }
        hitTest.prototype.reset = function () {
            this.rTree.clear();
        };
        hitTest.prototype.init = function (container, onToggleIndex, handleTooltip, onTrackClicked) {
            this.container = container;
            this.onToggleIndex = onToggleIndex;
            this.handleTooltip = handleTooltip;
            this.onTrackClicked = onTrackClicked;
        };
        hitTest.prototype.registerTrackItem = function (x, y, width, height, element) {
            var data = {
                minX: x,
                minY: y,
                maxX: x + width,
                maxY: y + height,
                actionType: "trackItem",
                arg: element
            };
            this.rTree.insert(data);
        };
        hitTest.prototype.registerIndexToggle = function (x, y, width, height, indexName) {
            var data = {
                minX: x,
                minY: y,
                maxX: x + width,
                maxY: y + height,
                actionType: "toggleIndex",
                arg: indexName
            };
            this.rTree.insert(data);
        };
        hitTest.prototype.onClick = function () {
            var clickLocation = d3.mouse(this.container.node());
            if (d3.event.defaultPrevented) {
                return;
            }
            var items = this.findItems(clickLocation[0], clickLocation[1]);
            for (var i = 0; i < items.length; i++) {
                var item = items[i];
                if (item.actionType === "toggleIndex") {
                    this.onToggleIndex(item.arg);
                }
                else if (item.actionType === "trackItem") {
                    this.onTrackClicked(item.arg);
                }
            }
        };
        hitTest.prototype.onMouseMove = function () {
            var clickLocation = d3.mouse(this.container.node());
            var items = this.findItems(clickLocation[0], clickLocation[1]);
            this.cursor(items.length ? "pointer" : "auto");
            var currentItem = items.filter(function (x) { return x.actionType === "trackItem"; }).map(function (x) { return x.arg; })[0];
            this.handleTooltip(currentItem, clickLocation[0], clickLocation[1]);
        };
        hitTest.prototype.findItems = function (x, y) {
            return this.rTree.search({
                minX: x,
                maxX: x,
                minY: y - metrics.brushSectionHeight,
                maxY: y - metrics.brushSectionHeight
            });
        };
        return hitTest;
    }());
    var metrics = (function (_super) {
        __extends(metrics, _super);
        function metrics() {
            var _this = _super.call(this) || this;
            _this.data = [];
            _this.searchText = ko.observable();
            _this.indexNames = ko.observableArray();
            _this.filteredIndexNames = ko.observableArray();
            _this.expandedTracks = ko.observableArray();
            _this.isoParser = d3.time.format.iso;
            _this.xTickFormat = d3.time.format("%H:%M:%S");
            _this.currentYOffset = 0;
            _this.maxYOffset = 0;
            _this.hitTest = new hitTest();
            _this.dialogVisible = false;
            _this.canExpandAll = ko.pureComputed(function () {
                var indexNames = _this.indexNames();
                var expandedTracks = _this.expandedTracks();
                return indexNames.length && indexNames.length !== expandedTracks.length;
            });
            _this.searchText.throttle(200).subscribe(function () { return _this.filterIndexes(); });
            return _this;
        }
        metrics.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            if (args.indexName) {
                this.expandedTracks.push(args.indexName);
            }
            return new getIndexesPerformance(this.activeDatabase())
                .execute()
                .done(function (result) { return _this.data = result; });
        };
        metrics.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.tooltip = d3.select(".tooltip");
            _a = this.getPageHostDimenensions(), this.totalWidth = _a[0], this.totalHeight = _a[1];
            this.totalHeight -= 50; // substract toolbar height
            this.initCanvas();
            this.hitTest.init(this.svg, function (indexName) { return _this.onToggleIndex(indexName); }, function (item, x, y) { return _this.handleTooltip(item, x, y); }, function (item) { return _this.showDialog(item); });
            this.draw();
            var _a;
        };
        metrics.prototype.initCanvas = function () {
            var _this = this;
            var metricsContainer = d3.select("#metricsContainer");
            this.canvas = metricsContainer
                .append("canvas")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight);
            this.svg = metricsContainer
                .append("svg")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight);
            this.xBrushNumericScale = d3.scale.linear()
                .range([0, this.totalWidth - 1]) // substract 1px to avoid issue with missing right stroke
                .domain([0, this.totalWidth]);
            this.xNumericScale = d3.scale.linear()
                .range([0, this.totalWidth])
                .domain([0, this.totalWidth]);
            this.brush = d3.svg.brush()
                .x(this.xBrushNumericScale)
                .on("brush", function () { return _this.onBrush(); });
            this.zoom = d3.behavior.zoom()
                .x(this.xNumericScale)
                .on("zoom", function () { return _this.onZoom(); });
            this.svg
                .append("svg:rect")
                .attr("class", "pane")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight - metrics.brushSectionHeight)
                .attr("transform", "translate(" + 0 + "," + metrics.brushSectionHeight + ")")
                .call(this.zoom)
                .call(function (d) { return _this.setupEvents(d); });
        };
        metrics.prototype.setupEvents = function (selection) {
            var _this = this;
            var mousePressed = false;
            var onMove = function () {
                _this.hitTest.onMouseMove();
            };
            this.hitTest.cursor.subscribe(function (cursor) {
                selection.style("cursor", cursor);
            });
            selection.on("mousemove.tip", onMove);
            selection.on("click", function () { return _this.hitTest.onClick(); });
            selection
                .on("mousedown.tip", function () { return selection.on("mousemove.tip", null); })
                .on("mouseup.tip", function () { return selection.on("mousemove.tip", onMove); });
            selection
                .on("mousedown.yShift", function () {
                var node = selection.node();
                var initialClickLocation = d3.mouse(node);
                var initialOffset = _this.currentYOffset;
                selection.on("mousemove.yShift", function () {
                    var currentMouseLocation = d3.mouse(node);
                    var yDiff = currentMouseLocation[1] - initialClickLocation[1];
                    var newYOffset = initialOffset - yDiff;
                    _this.currentYOffset = newYOffset;
                    _this.fixCurrentOffset();
                });
                selection.on("mouseup.yShift", function () { return selection.on("mousemove.yShift", null); });
            });
            selection.on("dblclick.zoom", null);
        };
        metrics.prototype.filterIndexes = function () {
            var criteria = this.searchText().toLowerCase();
            this.filteredIndexNames(this.indexNames().filter(function (x) { return x.toLowerCase().includes(criteria); }));
            this.drawMainSection();
        };
        metrics.prototype.draw = function () {
            if (this.data.length === 0) {
                //TODO: show no data section
                return;
            }
            this.prepareBrushSection();
            this.prepareMainSection();
            var canvas = this.canvas.node();
            var context = canvas.getContext("2d");
            context.drawImage(this.brushSection, 0, 0);
            this.drawMainSection();
        };
        metrics.prototype.prepareBrushSection = function () {
            var timeRanges = this.extractTimeRanges();
            var aggregatedRanges = new rangeAggregator(timeRanges);
            var workData = aggregatedRanges.aggregate();
            var maxConcurrentIndexes = aggregatedRanges.maxConcurrentIndexes;
            this.brushSection = document.createElement("canvas");
            this.brushSection.width = this.totalWidth;
            this.brushSection.height = metrics.brushSectionHeight;
            this.gapFinder = new gapFinder(timeRanges, metrics.minGapSize);
            this.xBrushTimeScale = this.gapFinder.createScale(this.totalWidth, 0);
            this.yBrushValueScale = d3.scale.linear()
                .domain([0, maxConcurrentIndexes])
                .range([0, metrics.brushSectionIndexesWorkHeight]);
            var context = this.brushSection.getContext("2d");
            this.drawXaxis(context, this.xBrushTimeScale, metrics.brushSectionHeight);
            context.strokeStyle = metrics.colors.axis;
            context.strokeRect(0.5, 0.5, this.totalWidth - 1, metrics.brushSectionHeight - 1);
            context.fillStyle = metrics.colors.brushChartColor;
            context.strokeStyle = metrics.colors.brushChartStrokeColor;
            context.lineWidth = metrics.brushSectionLineWidth;
            // Draw area chart showing indexes work
            var x1, x2, y0 = 0, y1;
            for (var i = 0; i < workData.length - 1; i++) {
                context.beginPath();
                x1 = this.xBrushTimeScale(new Date(workData[i].pointInTime));
                y1 = Math.round(this.yBrushValueScale(workData[i].numberOfIndexesWorking)) + 0.5;
                x2 = this.xBrushTimeScale(new Date(workData[i + 1].pointInTime));
                context.moveTo(x1, metrics.brushSectionHeight - y0);
                context.lineTo(x1, metrics.brushSectionHeight - y1);
                // Don't want to draw line -or- rect at level 0
                if (y1 !== 0) {
                    context.lineTo(x2, metrics.brushSectionHeight - y1);
                    context.fillRect(x1, metrics.brushSectionHeight - y1, x2 - x1, y1);
                }
                context.stroke();
                y0 = y1;
            }
            // Draw last line:
            context.beginPath();
            context.moveTo(x2, metrics.brushSectionHeight - y1);
            context.lineTo(x2, metrics.brushSectionHeight);
            context.stroke();
            this.drawBrushGaps(context);
            this.prepareBrush();
        };
        metrics.prototype.drawBrushGaps = function (context) {
            for (var i = 0; i < this.gapFinder.gapsPositions.length; i++) {
                var gap = this.gapFinder.gapsPositions[i];
                context.strokeStyle = metrics.colors.gaps;
                var gapX = this.xBrushTimeScale(gap.start);
                context.moveTo(gapX, 1);
                context.lineTo(gapX, metrics.brushSectionHeight - 2);
                context.stroke();
            }
        };
        metrics.prototype.prepareBrush = function () {
            var hasBrush = !!this.svg.select("g.brush").node();
            if (!hasBrush) {
                this.brushContainer = this.svg
                    .append("g")
                    .attr("class", "x brush");
                this.brushContainer
                    .call(this.brush)
                    .selectAll("rect")
                    .attr("y", 0)
                    .attr("height", metrics.brushSectionHeight - 1);
            }
        };
        metrics.prototype.prepareMainSection = function () {
            this.indexNames(this.findIndexNames());
            this.filteredIndexNames(this.indexNames());
        };
        metrics.prototype.fixCurrentOffset = function () {
            this.currentYOffset = Math.min(Math.max(0, this.currentYOffset), this.maxYOffset);
        };
        metrics.prototype.constructYScale = function () {
            var currentOffset = metrics.axisHeight - this.currentYOffset;
            var domain = [];
            var range = [];
            var indexesInfo = this.filteredIndexNames();
            for (var i = 0; i < indexesInfo.length; i++) {
                var indexName = indexesInfo[i];
                domain.push(indexName);
                range.push(currentOffset);
                var isOpened = _.includes(this.expandedTracks(), indexName);
                var itemHeight = isOpened ? metrics.openedTrackHeight : metrics.closedTrackHeight;
                currentOffset += itemHeight + metrics.trackMargin;
            }
            this.yScale = d3.scale.ordinal()
                .domain(domain)
                .range(range);
        };
        metrics.prototype.calcMaxYOffset = function () {
            var expandedTracksCount = this.expandedTracks().length;
            var closedTracksCount = this.filteredIndexNames().length - expandedTracksCount;
            var offset = metrics.axisHeight
                + this.filteredIndexNames().length * metrics.trackMargin
                + expandedTracksCount * metrics.openedTrackHeight
                + closedTracksCount * metrics.closedTrackHeight;
            var availableHeightForTracks = this.totalHeight - metrics.brushSectionHeight;
            var extraBottomMargin = 100;
            this.maxYOffset = Math.max(offset + extraBottomMargin - availableHeightForTracks, 0);
        };
        metrics.prototype.findIndexNames = function () {
            var result = new Set();
            this.data.forEach(function (perfItem) {
                result.add(perfItem.IndexName);
            });
            return Array.from(result);
        };
        metrics.prototype.drawXaxis = function (context, scale, height) {
            var _this = this;
            context.save();
            var step = 200;
            var initialOffset = 100;
            var ticks = d3.range(initialOffset, this.totalWidth - step, step)
                .map(function (y) { return scale.invert(y); });
            context.strokeStyle = metrics.colors.axis;
            context.fillStyle = metrics.colors.axis;
            context.beginPath();
            context.setLineDash([4, 2]);
            ticks.forEach(function (x, i) {
                context.moveTo(initialOffset + (i * step) + 0.5, 0);
                context.lineTo(initialOffset + (i * step) + 0.5, height);
            });
            context.stroke();
            context.beginPath();
            context.textAlign = "left";
            context.textBaseline = "top";
            context.font = "10px Lato";
            ticks.forEach(function (x, i) {
                // draw text with 5px left padding
                context.fillText(_this.xTickFormat(x), initialOffset + (i * step) + 5, 5);
            });
            context.restore();
        };
        metrics.prototype.onZoom = function () {
            this.brush.extent(this.xNumericScale.domain());
            this.brushContainer
                .call(this.brush);
            this.drawMainSection();
        };
        metrics.prototype.onBrush = function () {
            this.xNumericScale.domain((this.brush.empty() ? this.xBrushNumericScale.domain() : this.brush.extent()));
            this.zoom.x(this.xNumericScale);
            this.drawMainSection();
        };
        metrics.prototype.extractTimeRanges = function () {
            var _this = this;
            var result = [];
            this.data.forEach(function (indexStats) {
                indexStats.Performance.forEach(function (perfStat) {
                    var start = _this.isoParser.parse(perfStat.Started);
                    var end;
                    if (perfStat.Completed) {
                        end = _this.isoParser.parse(perfStat.Completed);
                    }
                    else {
                        end = new Date(start.getTime() + perfStat.DurationInMilliseconds);
                    }
                    result.push([start, end]);
                });
            });
            return result;
        };
        metrics.prototype.drawMainSection = function () {
            var _this = this;
            this.hitTest.reset();
            this.calcMaxYOffset();
            this.fixCurrentOffset();
            this.constructYScale();
            var visibleTimeFrame = this.xNumericScale.domain().map(function (x) { return _this.xBrushTimeScale.invert(x); });
            var xScale = this.gapFinder.trimmedScale(visibleTimeFrame, this.totalWidth, 0);
            var canvas = this.canvas.node();
            var context = canvas.getContext("2d");
            context.save();
            try {
                context.translate(0, metrics.brushSectionHeight);
                context.clearRect(0, 0, this.totalWidth, this.totalHeight - metrics.brushSectionHeight);
                this.drawTracksBackground(context, xScale);
                if (xScale.domain().length) {
                    this.drawXaxis(context, xScale, this.totalHeight);
                }
                context.save();
                context.beginPath();
                context.rect(0, metrics.axisHeight, this.totalWidth, this.totalHeight - metrics.brushSectionHeight);
                context.clip();
                this.drawTracks(context, xScale);
                this.drawIndexNames(context);
                this.drawGaps(context, xScale);
                context.restore();
            }
            finally {
                context.restore();
            }
        };
        metrics.prototype.drawTracksBackground = function (context, xScale) {
            var _this = this;
            context.save();
            context.beginPath();
            context.rect(0, metrics.axisHeight, this.totalWidth, this.totalHeight - metrics.brushSectionHeight);
            context.clip();
            this.data.forEach(function (perfStat) {
                var yStart = _this.yScale(perfStat.IndexName);
                perfStat.Performance.forEach(function (perf) {
                    var isOpened = _.includes(_this.expandedTracks(), perfStat.IndexName);
                    context.fillStyle = metrics.colors.trackBackground;
                    context.fillRect(0, yStart, _this.totalWidth, isOpened ? metrics.openedTrackHeight : metrics.closedTrackHeight);
                });
            });
            context.restore();
        };
        metrics.prototype.drawTracks = function (context, xScale) {
            var _this = this;
            if (xScale.domain().length === 0) {
                return;
            }
            var extentFunc = gapFinder.extentGeneratorForScaleWithGaps(xScale);
            this.data.forEach(function (perfStat) {
                var isOpened = _.includes(_this.expandedTracks(), perfStat.IndexName);
                var yStart = _this.yScale(perfStat.IndexName);
                yStart += isOpened ? metrics.openedTrackPadding : metrics.closedTrackPadding;
                perfStat.Performance.forEach(function (perf) {
                    var startDate = _this.isoParser.parse(perf.Started);
                    var x1 = xScale(startDate);
                    var yOffset = isOpened ? metrics.trackHeight + metrics.stackPadding : 0;
                    _this.drawStripes(context, [perf.Details], x1, yStart + (isOpened ? yOffset : 0), yOffset, extentFunc);
                });
            });
        };
        metrics.prototype.getColorForOperation = function (operationName) {
            if (operationName.startsWith("Collection_")) {
                return metrics.colors.tracks.Collection;
            }
            if (operationName in metrics.colors.tracks) {
                return metrics.colors.tracks[operationName];
            }
            throw new Error("Unable to find color for: " + operationName);
        };
        metrics.prototype.drawStripes = function (context, operations, xStart, yStart, yOffset, extentFunc) {
            var currentX = xStart;
            for (var i = 0; i < operations.length; i++) {
                var op = operations[i];
                context.fillStyle = this.getColorForOperation(op.Name);
                var dx = extentFunc(op.DurationInMilliseconds);
                context.fillRect(currentX, yStart, dx, metrics.trackHeight);
                if (yOffset !== 0) {
                    this.hitTest.registerTrackItem(currentX, yStart, dx, metrics.trackHeight, op);
                    if (op.Name.startsWith("Collection_")) {
                        context.fillStyle = metrics.colors.collectionNameTextColor;
                        var text = op.Name.substr("Collection_".length);
                        var textWidth = context.measureText(text).width;
                        var truncatedText = graphHelper.truncText(text, textWidth, dx - 4);
                        if (truncatedText) {
                            context.font = "12px Lato";
                            context.fillText(truncatedText, currentX + 2, yStart + 13, dx - 4);
                        }
                    }
                }
                if (op.Operations.length > 0) {
                    this.drawStripes(context, op.Operations, currentX, yStart + yOffset, yOffset, extentFunc);
                }
                currentX += dx;
            }
        };
        metrics.prototype.drawIndexNames = function (context) {
            var _this = this;
            var yScale = this.yScale;
            var textShift = 14.5;
            var textStart = 3 + 8 + 4;
            this.filteredIndexNames().forEach(function (indexName) {
                context.font = "12px Lato";
                var rectWidth = context.measureText(indexName).width + 2 * 3 /* left right padding */ + 8 /* arrow space */ + 4; /* padding between arrow and text */
                context.fillStyle = metrics.colors.trackNameBg;
                context.fillRect(2, yScale(indexName) + metrics.closedTrackPadding, rectWidth, metrics.trackHeight);
                _this.hitTest.registerIndexToggle(2, yScale(indexName), rectWidth, metrics.
                    trackHeight, indexName);
                context.fillStyle = metrics.colors.trackNameFg;
                context.fillText(indexName, textStart + 0.5, yScale(indexName) + textShift);
                var isOpened = _.includes(_this.expandedTracks(), indexName);
                context.fillStyle = isOpened ? metrics.colors.openedTrackArrow : metrics.colors.closedTrackArrow;
                graphHelper.drawArrow(context, 5, yScale(indexName) + 6, !isOpened);
            });
        };
        metrics.prototype.drawGaps = function (context, xScale) {
            var range = xScale.range();
            context.strokeStyle = metrics.colors.gaps;
            for (var i = 1; i < range.length; i += 2) {
                var gapX = Math.floor(range[i]) + 0.5;
                context.beginPath();
                context.moveTo(gapX, metrics.axisHeight);
                context.lineTo(gapX, this.totalHeight);
                context.stroke();
            }
        };
        metrics.prototype.onToggleIndex = function (indexName) {
            if (_.includes(this.expandedTracks(), indexName)) {
                this.expandedTracks.remove(indexName);
            }
            else {
                this.expandedTracks.push(indexName);
            }
            this.drawMainSection();
        };
        metrics.prototype.expandAll = function () {
            this.expandedTracks(this.indexNames().slice());
            this.drawMainSection();
        };
        metrics.prototype.collapseAll = function () {
            this.expandedTracks([]);
            this.drawMainSection();
        };
        /*
         * Called by hitTest class on mouse move
         * show tooltip when element !== null
         * hide if it is null
         */
        metrics.prototype.handleTooltip = function (element, x, y) {
            if (element && !this.dialogVisible) {
                var currentDatum = this.tooltip.datum();
                if (currentDatum !== element) {
                    this.tooltip.transition()
                        .duration(200)
                        .style("opacity", 1);
                    var html = element.Name;
                    html += "<br />Duration: " + generalUtils.formatMillis(element.DurationInMilliseconds);
                    this.tooltip.html(html);
                    this.tooltip.datum(element);
                }
                var tooltipWidth = $("#indexingPerformance .tooltip").width() + 30;
                x = Math.min(x, Math.max(this.totalWidth - tooltipWidth, 0));
                this.tooltip
                    .style("left", (x + 10) + "px")
                    .style("top", (y + 10) + "px");
            }
            else {
                this.hideTooltip();
            }
        };
        metrics.prototype.hideTooltip = function () {
            this.tooltip.datum(null);
            this.tooltip.transition()
                .duration(200)
                .style("opacity", 0);
        };
        metrics.prototype.showDialog = function (element) {
            var _this = this;
            var dialog = new indexPerformanceDetails(element);
            this.dialogVisible = true;
            app.showBootstrapDialog(dialog)
                .always(function () { return _this.dialogVisible = false; });
            this.hideTooltip();
        };
        metrics.prototype.fileSelected = function () {
            var fileInput = document.querySelector("#importFilePicker");
            var self = this;
            if (fileInput.files.length === 0) {
                return;
            }
            var file = fileInput.files[0];
            var reader = new FileReader();
            reader.onload = function () {
                // ReSharper disable once SuspiciousThisUsage
                self.dataImported(this.result);
            };
            reader.onerror = function (error) {
                alert(error);
            };
            reader.readAsText(file);
        };
        metrics.prototype.dataImported = function (result) {
            this.data = JSON.parse(result);
            this.expandedTracks([]);
            this.draw();
        };
        metrics.prototype.exportAsJson = function () {
            fileDownloader.downloadAsJson(this.data, "perf.json", "perf");
        };
        return metrics;
    }(viewModelBase));
    metrics.colors = {
        axis: "#546175",
        gaps: "#ca1c59",
        brushChartColor: "#37404b",
        brushChartStrokeColor: "#008cc9",
        trackBackground: "#2c343a",
        trackNameBg: "rgba(57, 67, 79, 0.8)",
        trackNameFg: "#98a7b7",
        openedTrackArrow: "#ca1c59",
        closedTrackArrow: "#98a7b7",
        collectionNameTextColor: "#2c343a",
        tracks: {
            "Collection": "#046293",
            "Indexing": "#607d8b",
            "Cleanup": "#1a858e",
            "References": "#ac2258",
            "Map": "#0b4971",
            "Storage/DocumentRead": "#0077b5",
            "Linq": "#008cc9",
            "LoadDocument": "#008cc9",
            "Bloom": "#34b3e4",
            "Lucene/Delete": "#66418c",
            "Lucene/AddDocument": "#8d6cab",
            "Lucene/Convert": "#7b539d",
            "CreateBlittableJson": "#313fa0",
            "Aggregation/BlittableJson": "#ec407a",
            "GetMapEntriesTree": "#689f39",
            "GetMapEntries": "#8cc34b",
            "Storage/RemoveMapResult": "#ff7000",
            "Storage/PutMapResult": "#fe8f01",
            "Reduce": "#98041b",
            "Tree": "#af1923",
            "Aggregation/Leafs": "#890e4f",
            "Aggregation/Branches": "#ad1457",
            "Storage/ReduceResults": "#e65100",
            "NestedValues": "#795549",
            "Storage/Read": "#faa926",
            "Aggregation/NestedValues": "#d81a60",
            "Lucene/FlushToDisk": "#a487ba",
            "Storage/Commit": "#5b912d",
            "Lucene/RecreateSearcher": "#b79ec7"
        }
    };
    metrics.brushSectionHeight = 40;
    metrics.brushSectionIndexesWorkHeight = 22;
    metrics.brushSectionLineWidth = 1;
    metrics.trackHeight = 18; // height used for callstack item
    metrics.stackPadding = 1; // space between call stacks
    metrics.trackMargin = 4;
    metrics.closedTrackPadding = 2;
    metrics.openedTrackPadding = 4;
    metrics.axisHeight = 35;
    metrics.maxRecursion = 5;
    metrics.minGapSize = 10 * 1000; // 10 seconds
    metrics.openedTrackHeight = metrics.openedTrackPadding
        + (metrics.maxRecursion + 1) * metrics.trackHeight
        + metrics.maxRecursion * metrics.stackPadding
        + metrics.openedTrackPadding;
    metrics.closedTrackHeight = metrics.closedTrackPadding
        + metrics.trackHeight
        + metrics.closedTrackPadding;
    return metrics;
});

//# sourceMappingURL=indexPerformance.js.map
