var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/database/index/getIndexTermsCommand", "commands/database/index/getIndexEntriesFieldsCommand"], function (require, exports, viewModelBase, getIndexTermsCommand, getIndexEntriesFieldsCommand) {
    "use strict";
    var indexTerms = (function (_super) {
        __extends(indexTerms, _super);
        function indexTerms() {
            var _this = _super.apply(this, arguments) || this;
            _this.fields = ko.observableArray();
            return _this;
        }
        indexTerms.prototype.activate = function (indexName) {
            _super.prototype.activate.call(this, indexName);
            this.indexName = indexName;
            this.indexPageUrl = this.appUrls.editIndex(this.indexName);
            return this.fetchIndexEntriesFields(indexName);
        };
        indexTerms.prototype.fetchIndexEntriesFields = function (indexName) {
            var _this = this;
            return new getIndexEntriesFieldsCommand(indexName, this.activeDatabase())
                .execute()
                .done(function (fields) { return _this.processFields(fields); });
        };
        indexTerms.createTermsForField = function (fieldName) {
            return {
                fromValue: null,
                name: fieldName,
                hasMoreTerms: ko.observable(true),
                terms: ko.observableArray(),
                loadInProgress: ko.observable(false)
            };
        };
        indexTerms.prototype.processFields = function (fields) {
            var _this = this;
            this.fields(fields.map(function (fieldName) { return indexTerms.createTermsForField(fieldName); }));
            this.fields()
                .forEach(function (field) { return _this.loadTerms(_this.indexName, field); });
        };
        indexTerms.prototype.loadTerms = function (indexName, termsForField) {
            return new getIndexTermsCommand(indexName, termsForField.name, this.activeDatabase(), indexTerms.termsPageLimit + 1, termsForField.fromValue)
                .execute()
                .done(function (loadedTermsResponse) {
                var loadedTerms = loadedTermsResponse.Terms;
                if (loadedTerms.length > indexTerms.termsPageLimit) {
                    termsForField.hasMoreTerms(true);
                    loadedTerms = loadedTerms.slice(0, indexTerms.termsPageLimit);
                }
                else {
                    termsForField.hasMoreTerms(false);
                }
                (_a = termsForField.terms).push.apply(_a, loadedTerms);
                if (loadedTerms.length > 0) {
                    termsForField.fromValue = loadedTerms[loadedTerms.length - 1];
                }
                var _a;
            });
        };
        indexTerms.prototype.loadMore = function (fieldName) {
            var field = this.fields().find(function (x) { return x.name === fieldName; });
            if (!field || !field.hasMoreTerms()) {
                return;
            }
            field.loadInProgress(true);
            this.loadTerms(this.indexName, field)
                .always(function () { return field.loadInProgress(false); });
        };
        return indexTerms;
    }(viewModelBase));
    indexTerms.termsPageLimit = 100; //TODO: consider higher value?
    return indexTerms;
});

//# sourceMappingURL=indexTerms.js.map
