var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/database/index/index", "common/appUrl", "commands/database/index/saveIndexLockModeCommand", "durandal/app", "models/database/index/indexReplaceDocument", "commands/database/index/getPendingIndexReplacementsCommand", "viewmodels/database/indexes/cancelSideBySizeConfirm", "viewmodels/database/indexes/deleteIndexesConfirm", "commands/database/index/forceIndexReplace", "commands/database/index/saveIndexPriorityCommand", "commands/database/index/getIndexesStatsCommand", "commands/database/index/getIndexesStatusCommand", "commands/database/index/resetIndexCommand", "commands/database/index/toggleIndexingCommand", "common/eventsCollector", "commands/database/index/enableIndexCommand", "commands/database/index/disableIndexCommand"], function (require, exports, viewModelBase, index, appUrl, saveIndexLockModeCommand, app, indexReplaceDocument, getPendingIndexReplacementsCommand, cancelSideBySizeConfirm, deleteIndexesConfirm, forceIndexReplace, saveIndexPriorityCommand, getIndexesStatsCommand, getIndexesStatusCommand, resetIndexCommand, toggleIndexingCommand, eventsCollector, enableIndexCommand, disableIndexCommand) {
    "use strict";
    var indexes = (function (_super) {
        __extends(indexes, _super);
        function indexes() {
            var _this = _super.call(this) || this;
            _this.indexGroups = ko.observableArray();
            _this.newIndexUrl = appUrl.forCurrentDatabase().newIndex;
            _this.searchText = ko.observable();
            _this.selectedIndexesName = ko.observableArray();
            _this.spinners = {
                globalStartStop: ko.observable(false),
                globalLockChanges: ko.observable(false),
                localPriority: ko.observableArray([]),
                localLockChanges: ko.observableArray([]),
                localState: ko.observableArray([])
            };
            _this.indexingEnabled = ko.observable(true);
            _this.resetsInProgress = new Set();
            _this.initObservables();
            _this.bindToCurrentInstance("lowPriority", "highPriority", "normalPriority", "resetIndex", "deleteIndex", "unlockIndex", "lockIndex", "lockErrorIndex", "lockSideBySide", "enableIndex", "disableIndex", "unlockSelectedIndexes", "lockSelectedIndexes", "lockSideBySideSelectedIndexes", "lockErrorSelectedIndexes", "deleteSelectedIndexes", "startIndexing", "stopIndexing", "resumeIndexing", "pauseUntilRestart", "toggleSelectAll");
            return _this;
        }
        indexes.prototype.getAllIndexes = function () {
            var all = [];
            this.indexGroups().forEach(function (g) { return all.push.apply(all, g.indexes()); });
            return _.uniq(all);
        };
        indexes.prototype.getSelectedIndexes = function () {
            var selectedIndexes = this.selectedIndexesName();
            return this.getAllIndexes().filter(function (x) { return _.includes(selectedIndexes, x.name); });
        };
        indexes.prototype.initObservables = function () {
            var _this = this;
            this.searchText.throttle(200).subscribe(function () { return _this.filterIndexes(); });
            this.sortedGroups = ko.computed(function () {
                var groups = _this.indexGroups().slice(0).sort(function (l, r) { return l.entityName.toLowerCase() > r.entityName.toLowerCase() ? 1 : -1; });
                groups.forEach(function (group) {
                    group.indexes(group.indexes().slice(0).sort(function (l, r) { return l.name.toLowerCase() > r.name.toLowerCase() ? 1 : -1; }));
                });
                return groups;
            });
            this.lockModeCommon = ko.computed(function () {
                var selectedIndexes = _this.getSelectedIndexes();
                if (selectedIndexes.length === 0)
                    return "None";
                var firstLockMode = selectedIndexes[0].lockMode();
                for (var i = 1; i < selectedIndexes.length; i++) {
                    if (selectedIndexes[i].lockMode() !== firstLockMode) {
                        return "Mixed";
                    }
                }
                return firstLockMode;
            });
            this.indexesSelectionState = ko.pureComputed(function () {
                var selectedCount = _this.selectedIndexesName().length;
                var indexesCount = _this.getAllIndexes().length;
                if (indexesCount && selectedCount === indexesCount)
                    return 2 /* Checked */;
                if (selectedCount > 0)
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
        };
        indexes.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('AIHAR1');
            return this.fetchIndexes();
        };
        indexes.prototype.attached = function () {
            _super.prototype.attached.call(this);
            ko.postbox.publish("SetRawJSONUrl", appUrl.forIndexesRawData(this.activeDatabase())); //TODO: do we need it?
        };
        indexes.prototype.fetchIndexes = function () {
            var _this = this;
            var statsTask = new getIndexesStatsCommand(this.activeDatabase())
                .execute();
            var statusTask = new getIndexesStatusCommand(this.activeDatabase())
                .execute();
            var replacementTask = new getPendingIndexReplacementsCommand(this.activeDatabase()).execute(); //TODO: this is not working yet!
            return $.when(statsTask, replacementTask, statusTask)
                .done(function (_a, _b, _c) {
                var stats = _a[0];
                var replacements = _b[0];
                var statuses = _c[0];
                return _this.processData(stats, replacements, statuses);
            });
        };
        indexes.prototype.processData = function (stats, replacements, statuses) {
            //TODO: handle replacements
            var _this = this;
            this.indexingEnabled(statuses.Status !== "Paused");
            stats
                .map(function (i) { return new index(i); })
                .forEach(function (i) {
                var paused = !!statuses.Indexes.find(function (x) { return x.Name === i.name && x.Status === "Paused"; });
                i.pausedUntilRestart(paused);
                _this.putIndexIntoGroups(i);
            });
        };
        indexes.prototype.putIndexIntoGroups = function (i) {
            this.putIndexIntoGroupNamed(i, i.getGroupName());
        };
        indexes.prototype.putIndexIntoGroupNamed = function (i, groupName) {
            var group = this.indexGroups().find(function (g) { return g.entityName === groupName; });
            if (group) {
                var oldIndex = group.indexes().find(function (cur) { return cur.name === i.name; });
                if (oldIndex) {
                    group.indexes.replace(oldIndex, i);
                }
                else {
                    group.indexes.push(i);
                }
            }
            else {
                this.indexGroups.push({
                    entityName: groupName,
                    indexes: ko.observableArray([i]),
                    groupHidden: ko.observable(false)
                });
            }
        };
        indexes.prototype.filterIndexes = function () {
            var filterLower = this.searchText().toLowerCase();
            this.selectedIndexesName([]);
            this.indexGroups().forEach(function (indexGroup) {
                var hasAnyInGroup = false;
                indexGroup.indexes().forEach(function (index) {
                    var match = index.name.toLowerCase().indexOf(filterLower) >= 0;
                    index.filteredOut(!match);
                    if (match) {
                        hasAnyInGroup = true;
                    }
                });
                indexGroup.groupHidden(!hasAnyInGroup);
            });
        };
        indexes.prototype.resetIndex = function (indexToReset) {
            var _this = this;
            this.confirmationMessage("Reset index?", "You're resetting " + indexToReset.name)
                .done(function (result) {
                if (result.can) {
                    eventsCollector.default.reportEvent("indexes", "reset");
                    // reset index is implemented as delete and insert, so we receive notification about deleted index via changes API
                    // let's issue marker to ignore index delete information for next few seconds because it might be caused by reset.
                    // Unfortunettely we can't use resetIndexVm.resetTask.done, because we receive event via changes api before resetTask promise 
                    // is resolved. 
                    _this.resetsInProgress.add(indexToReset.name);
                    new resetIndexCommand(indexToReset.name, _this.activeDatabase())
                        .execute();
                    setTimeout(function () {
                        _this.resetsInProgress.delete(indexToReset.name);
                    }, 30000);
                }
            });
        };
        indexes.prototype.deleteIndex = function (i) {
            eventsCollector.default.reportEvent("indexes", "delete");
            this.promptDeleteIndexes([i]);
            this.resetsInProgress.delete(i.name);
        };
        indexes.prototype.processIndexEvent = function (e) {
            var _this = this;
            var indexRemovedEvent = "IndexRemoved";
            if (e.Type === indexRemovedEvent) {
                if (!this.resetsInProgress.has(e.Name)) {
                    this.removeIndexesFromAllGroups(this.findIndexesByName(e.Name));
                }
            }
            else {
                setTimeout(function () {
                    _this.fetchIndexes();
                }, 5000);
            }
        };
        indexes.prototype.removeIndexesFromAllGroups = function (indexes) {
            this.indexGroups().forEach(function (g) {
                g.indexes.removeAll(indexes);
            });
            // Remove any empty groups.
            this.indexGroups.remove(function (item) { return item.indexes().length === 0; });
        };
        indexes.prototype.findIndexesByName = function (indexName) {
            var result = new Array();
            this.indexGroups().forEach(function (g) {
                g.indexes().forEach(function (i) {
                    if (i.name === indexName) {
                        result.push(i);
                    }
                });
            });
            return result;
        };
        indexes.prototype.promptDeleteIndexes = function (indexes) {
            var _this = this;
            if (indexes.length > 0) {
                var deleteIndexesVm = new deleteIndexesConfirm(indexes.map(function (i) { return i.name; }), this.activeDatabase());
                app.showBootstrapDialog(deleteIndexesVm);
                deleteIndexesVm.deleteTask
                    .done(function (deleted) {
                    if (deleted) {
                        _this.removeIndexesFromAllGroups(indexes);
                    }
                });
            }
        };
        indexes.prototype.unlockIndex = function (i) {
            eventsCollector.default.reportEvent("indexes", "set-lock-mode", "Unlock");
            this.updateIndexLockMode(i, "Unlock", "Unlocked");
        };
        indexes.prototype.lockIndex = function (i) {
            eventsCollector.default.reportEvent("indexes", "set-lock-mode", "LockedIgnore");
            this.updateIndexLockMode(i, "LockedIgnore", "Locked");
        };
        indexes.prototype.lockErrorIndex = function (i) {
            eventsCollector.default.reportEvent("indexes", "set-lock-mode", "LockedError");
            this.updateIndexLockMode(i, "LockedError", "Locked (Error)");
        };
        indexes.prototype.lockSideBySide = function (i) {
            eventsCollector.default.reportEvent("indexes", "set-lock-mode", "SideBySide");
            this.updateIndexLockMode(i, "SideBySide", "Locked (Side by Side)");
        };
        indexes.prototype.updateIndexLockMode = function (i, newLockMode, lockModeStrForTitle) {
            var _this = this;
            if (i.lockMode() !== newLockMode) {
                this.spinners.localLockChanges.push(i.name);
                new saveIndexLockModeCommand([i], newLockMode, this.activeDatabase(), lockModeStrForTitle)
                    .execute()
                    .done(function () { return i.lockMode(newLockMode); })
                    .always(function () { return _this.spinners.localLockChanges.remove(i.name); });
            }
        };
        indexes.prototype.enableIndex = function (idx) {
            var _this = this;
            eventsCollector.default.reportEvent("indexes", "set-state", "enabled");
            if (idx.canBeEnabled()) {
                this.spinners.localState.push(idx.name);
                new enableIndexCommand(idx.name, this.activeDatabase())
                    .execute()
                    .done(function () { return idx.state("Normal"); })
                    .always(function () { return _this.spinners.localState.remove(idx.name); });
            }
        };
        indexes.prototype.disableIndex = function (idx) {
            var _this = this;
            eventsCollector.default.reportEvent("indexes", "set-state", "disabled");
            if (idx.canBeDisabled()) {
                this.spinners.localState.push(idx.name);
                new disableIndexCommand(idx.name, this.activeDatabase())
                    .execute()
                    .done(function () {
                    idx.state("Disabled");
                    idx.pausedUntilRestart(false);
                })
                    .always(function () { return _this.spinners.localState.remove(idx.name); });
            }
        };
        indexes.prototype.normalPriority = function (idx) {
            eventsCollector.default.reportEvent("index", "priority", "normal");
            this.setIndexPriority(idx, "Normal");
        };
        indexes.prototype.lowPriority = function (idx) {
            eventsCollector.default.reportEvent("index", "priority", "low");
            this.setIndexPriority(idx, "Low");
        };
        indexes.prototype.highPriority = function (idx) {
            eventsCollector.default.reportEvent("index", "priority", "high");
            this.setIndexPriority(idx, "High");
        };
        indexes.prototype.setIndexPriority = function (idx, newPriority) {
            var _this = this;
            var originalPriority = idx.priority();
            if (originalPriority !== newPriority) {
                this.spinners.localPriority.push(idx.name);
                var optionalResumeTask = idx.pausedUntilRestart()
                    ? this.resumeIndexingInternal(idx)
                    : $.Deferred().resolve();
                optionalResumeTask.done(function () {
                    new saveIndexPriorityCommand(idx.name, newPriority, _this.activeDatabase())
                        .execute()
                        .done(function () { return idx.priority(newPriority); })
                        .always(function () { return _this.spinners.localPriority.remove(idx.name); });
                });
            }
        };
        indexes.prototype.createNotifications = function () {
            var _this = this;
            return [
                this.changesContext.currentResourceChangesApi().watchAllIndexes(function (e) { return _this.processIndexEvent(e); }),
                this.changesContext.currentResourceChangesApi().watchDocsStartingWith(indexReplaceDocument.replaceDocumentPrefix, function () { return _this.processReplaceEvent(); })
            ];
        };
        indexes.prototype.processReplaceEvent = function () {
            var _this = this;
            setTimeout(function () { return _this.fetchIndexes(); }, 10);
        };
        indexes.prototype.cancelSideBySideIndex = function (i) {
            var _this = this;
            eventsCollector.default.reportEvent("index", "cancel-side-by-side");
            var cancelSideBySideIndexViewModel = new cancelSideBySizeConfirm([i.name], this.activeDatabase());
            app.showBootstrapDialog(cancelSideBySideIndexViewModel);
            cancelSideBySideIndexViewModel.cancelTask
                .done(function (closedWithoutDeletion) {
                if (!closedWithoutDeletion) {
                    _this.removeIndexesFromAllGroups([i]);
                }
            })
                .fail(function () {
                _this.removeIndexesFromAllGroups([i]);
                _this.fetchIndexes();
            });
        };
        indexes.prototype.forceSideBySide = function (idx) {
            eventsCollector.default.reportEvent("index", "force-side-by-side");
            new forceIndexReplace(idx.name, this.activeDatabase()).execute();
        };
        indexes.prototype.unlockSelectedIndexes = function () {
            this.setLockModeSelectedIndexes("Unlock", "Unlock");
        };
        indexes.prototype.lockSelectedIndexes = function () {
            this.setLockModeSelectedIndexes("LockedIgnore", "Lock");
        };
        indexes.prototype.lockSideBySideSelectedIndexes = function () {
            this.setLockModeSelectedIndexes("SideBySide", "Lock (Side By Side)");
        };
        indexes.prototype.lockErrorSelectedIndexes = function () {
            this.setLockModeSelectedIndexes("LockedError", "Lock (Error)");
        };
        indexes.prototype.setLockModeSelectedIndexes = function (lockModeString, lockModeStrForTitle) {
            var _this = this;
            if (this.lockModeCommon() === lockModeString)
                return;
            this.confirmationMessage("Are you sure?", "Do you want to " + lockModeStrForTitle + " selected indexes?")
                .done(function (can) {
                if (can) {
                    eventsCollector.default.reportEvent("index", "set-lock-mode-selected", lockModeString);
                    _this.spinners.globalLockChanges(true);
                    var indexes_1 = _this.getSelectedIndexes();
                    new saveIndexLockModeCommand(indexes_1, lockModeString, _this.activeDatabase(), lockModeStrForTitle)
                        .execute()
                        .done(function () { return indexes_1.forEach(function (i) { return i.lockMode(lockModeString); }); })
                        .always(function () { return _this.spinners.globalLockChanges(false); });
                }
            });
        };
        indexes.prototype.deleteSelectedIndexes = function () {
            eventsCollector.default.reportEvent("indexes", "delete-selected");
            this.promptDeleteIndexes(this.getSelectedIndexes());
        };
        indexes.prototype.startIndexing = function () {
            var _this = this;
            this.confirmationMessage("Are you sure?", "Do you want to resume indexing?")
                .done(function (result) {
                if (result.can) {
                    eventsCollector.default.reportEvent("indexes", "resume-all");
                    _this.spinners.globalStartStop(true);
                    new toggleIndexingCommand(true, _this.activeDatabase())
                        .execute()
                        .done(function () { return _this.indexingEnabled(true); })
                        .always(function () {
                        _this.spinners.globalStartStop(false);
                        _this.fetchIndexes();
                    });
                }
            });
        };
        indexes.prototype.stopIndexing = function () {
            var _this = this;
            this.confirmationMessage("Are you sure?", "Do you want to pause indexing until server restart?")
                .done(function (result) {
                if (result.can) {
                    eventsCollector.default.reportEvent("indexes", "pause-all");
                    _this.spinners.globalStartStop(true);
                    new toggleIndexingCommand(false, _this.activeDatabase())
                        .execute()
                        .done(function () { return _this.indexingEnabled(false); })
                        .always(function () {
                        _this.spinners.globalStartStop(false);
                        _this.fetchIndexes();
                    });
                }
            });
        };
        indexes.prototype.resumeIndexing = function (idx) {
            var _this = this;
            eventsCollector.default.reportEvent("indexes", "resume");
            this.spinners.localState.push(idx.name);
            return this.resumeIndexingInternal(idx)
                .always(function () { return _this.spinners.localState.remove(idx.name); });
        };
        indexes.prototype.resumeIndexingInternal = function (idx) {
            return new toggleIndexingCommand(true, this.activeDatabase(), { name: [idx.name] })
                .execute()
                .done(function () { return idx.pausedUntilRestart(false); });
        };
        indexes.prototype.pauseUntilRestart = function (idx) {
            var _this = this;
            eventsCollector.default.reportEvent("indexes", "pause");
            this.spinners.localState.push(idx.name);
            new toggleIndexingCommand(false, this.activeDatabase(), { name: [idx.name] })
                .execute()
                .done(function () { return idx.pausedUntilRestart(true); })
                .always(function () { return _this.spinners.localState.remove(idx.name); });
        };
        indexes.prototype.toggleSelectAll = function () {
            eventsCollector.default.reportEvent("indexes", "toggle-select-all");
            var selectedIndexesCount = this.selectedIndexesName().length;
            if (selectedIndexesCount > 0) {
                this.selectedIndexesName([]);
            }
            else {
                var namesToSelect_1 = [];
                this.indexGroups().forEach(function (indexGroup) {
                    if (!indexGroup.groupHidden()) {
                        indexGroup.indexes().forEach(function (index) {
                            if (!index.filteredOut() && !_.includes(namesToSelect_1, index.name)) {
                                namesToSelect_1.push(index.name);
                            }
                        });
                    }
                });
                this.selectedIndexesName(namesToSelect_1);
            }
        };
        return indexes;
    }(viewModelBase));
    return indexes;
});

//# sourceMappingURL=indexes.js.map
