var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/resources/getDatabaseStatsCommand", "common/bindingHelpers/datePickerBindingHandler", "moment", "models/database/index/indexReplaceDocument"], function (require, exports, dialog, dialogViewModelBase, getDatabaseStatsCommand, datePickerBindingHandler, moment, indexReplaceDocument) {
    "use strict";
    var replaceIndexDialog = (function (_super) {
        __extends(replaceIndexDialog, _super);
        function replaceIndexDialog(indexName, db, elementToFocusOnDismissal) {
            var _this = _super.call(this, elementToFocusOnDismissal) || this;
            _this.indexName = indexName;
            _this.db = db;
            _this.replaceSettingsTask = $.Deferred();
            _this.etagMode = ko.observable(false);
            _this.dateMode = ko.observable(false);
            _this.lastIndexedEtag = ko.observable();
            _this.replaceDate = ko.observable(moment(new Date()));
            _this.replaceDateText = ko.computed(function () {
                return _this.replaceDate() != null ? _this.replaceDate().format("YYYY/MM/DD H:mm:ss") : "";
            });
            datePickerBindingHandler.install();
            return _this;
        }
        replaceIndexDialog.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            this.fetchIndexes()
                .done(function () { return deferred.resolve({ can: true }); })
                .fail(function () { return deferred.resolve({ can: false }); });
            return deferred.promise();
        };
        replaceIndexDialog.prototype.fetchIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.db)
                .execute()
                .done(function (stats) { return _this.processDbStats(stats); });
        };
        replaceIndexDialog.prototype.processDbStats = function (stats) {
            var _this = this;
            var oldIndex = stats.Indexes.find(function (i) { return i.Name === _this.indexName; });
            //TODO: this.lastIndexedEtag(oldIndex.LastIndexedEtag);       
        };
        replaceIndexDialog.prototype.saveReplace = function () {
            var replaceDocument = new indexReplaceDocument({ IndexToReplace: this.indexName });
            if (this.etagMode()) {
                replaceDocument.minimumEtagBeforeReplace = this.lastIndexedEtag();
            }
            if (this.dateMode()) {
                replaceDocument.replaceTimeUtc = this.replaceDate().toISOString();
            }
            this.replaceSettingsTask.resolve(replaceDocument);
        };
        replaceIndexDialog.prototype.close = function () {
            this.replaceSettingsTask.reject();
            dialog.close(this);
        };
        replaceIndexDialog.prototype.toggleEtagMode = function () {
            this.etagMode(!this.etagMode());
        };
        replaceIndexDialog.prototype.toggleDateMode = function () {
            this.dateMode(!this.dateMode());
        };
        return replaceIndexDialog;
    }(dialogViewModelBase));
    return replaceIndexDialog;
});

//# sourceMappingURL=replaceIndexDialog.js.map
