var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "common/bindingHelpers/aceEditorBindingHandler", "models/database/index/scriptedIndex", "commands/database/index/getScriptedIndexesCommand", "commands/database/documents/saveScriptedIndexesCommand", "common/appUrl"], function (require, exports, viewModelBase, getDatabaseStatsCommand, aceEditorBindingHandler, scriptedIndex, getScriptedIndexesCommand, saveScriptedIndexesCommand, appUrl) {
    "use strict";
    var scriptedIndexes = (function (_super) {
        __extends(scriptedIndexes, _super);
        function scriptedIndexes() {
            var _this = _super.call(this) || this;
            _this.allScriptedIndexes = ko.observableArray();
            _this.activeScriptedIndexes = ko.observableArray().extend({ required: true });
            _this.indexNames = ko.observableArray();
            _this.isFirstLoad = ko.observable(true);
            aceEditorBindingHandler.install();
            _this.inactiveIndexNames = ko.computed(function () {
                var activeIndexNames = _this.allScriptedIndexes()
                    .filter(function (index) { return !index.isMarkedToDelete(); })
                    .map(function (index) { return index.indexName(); });
                return _this.indexNames().filter(function (indexName) { return activeIndexNames.indexOf(indexName) < 0; });
            }, _this);
            _this.firstIndex = ko.computed(function () {
                return !_this.isFirstLoad() ? 0 : -1;
            }, _this);
            return _this;
        }
        scriptedIndexes.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                this.fetchAllIndexes(db)
                    .done(function () {
                    _this.fetchAllScriptedIndexes(db)
                        .done(function () { return deferred.resolve({ can: true }); })
                        .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
                });
            }
            return deferred;
        };
        scriptedIndexes.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.activeScriptedIndexes]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        scriptedIndexes.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.addScriptsLabelPopover();
            this.initializeCollapsedInvalidElements();
            $('pre').each(function (index, currentPreElement) {
                if (currentPreElement) {
                    var editor = ko.utils.domData.get(currentPreElement, "aceEditor");
                    var editorValue = editor.getSession().getValue();
                    _this.initializeAceValidity(currentPreElement, editorValue);
                }
            });
        };
        scriptedIndexes.prototype.createScriptedIndex = function (indexName) {
            var _this = this;
            if (this.isFirstLoad()) {
                this.isFirstLoad(false);
            }
            var results = this.allScriptedIndexes().filter(function (index) { return index.indexName() == indexName; });
            var activeScriptedIndex = results[0];
            if (activeScriptedIndex) {
                activeScriptedIndex.cancelDeletion();
            }
            else {
                activeScriptedIndex = scriptedIndex.emptyForIndex(indexName);
                this.allScriptedIndexes.unshift(activeScriptedIndex);
            }
            this.activeScriptedIndexes.unshift(activeScriptedIndex);
            var twoCreatedPreElements = $('.in pre');
            var editor = ko.utils.domData.get(twoCreatedPreElements[0], "aceEditor");
            editor.focus();
            twoCreatedPreElements.each(function (index, preElement) {
                _this.initializeAceValidity(preElement, "");
            });
            this.initializeCollapsedInvalidElements();
        };
        scriptedIndexes.prototype.removeScriptedIndex = function (scriptedIndexToDelete) {
            this.activeScriptedIndexes.remove(scriptedIndexToDelete);
            scriptedIndexToDelete.markToDelete();
        };
        scriptedIndexes.prototype.saveChanges = function () {
            var _this = this;
            var db = this.activeDatabase();
            new saveScriptedIndexesCommand(this.allScriptedIndexes(), db)
                .execute()
                .done(function (result) {
                _this.updateIndexes(result);
                _this.dirtyFlag().reset(); //Resync Changes
            });
        };
        scriptedIndexes.prototype.fetchAllIndexes = function (db) {
            return new getDatabaseStatsCommand(db)
                .execute();
            //TODO: .done((results: databaseStatisticsDto) => this.performAllIndexesResult(results));
        };
        scriptedIndexes.prototype.performAllIndexesResult = function (results) {
            this.indexNames(results.Indexes.map(function (i) { return i.Name; }));
        };
        scriptedIndexes.prototype.fetchAllScriptedIndexes = function (db) {
            var _this = this;
            return new getScriptedIndexesCommand(db)
                .execute()
                .done(function (indexes) {
                (_a = _this.allScriptedIndexes).push.apply(_a, indexes);
                (_b = _this.activeScriptedIndexes).push.apply(_b, indexes);
                var _a, _b;
            });
        };
        scriptedIndexes.prototype.addScriptsLabelPopover = function () {
            var indexScriptpopOverSettings = {
                html: true,
                trigger: 'hover',
                content: 'Index Scripts are written in JavaScript.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> company = LoadDocument(<span class="code-keyword">this</span>.Company);<br/><span class="code-keyword">if</span>(company == null) <span class="code-keyword">return</span>;<br/>company.Orders = { Count: <span class="code-keyword">this</span>.Count, Total: <span class="code-keyword">this</span>.Total };<br/>PutDocument(<span class="code-keyword">this</span>.Company, company);</pre>',
                selector: '.index-script-label'
            };
            $('#accordion').popover(indexScriptpopOverSettings);
            var deleteScriptPopOverSettings = {
                html: true,
                trigger: 'hover',
                content: 'Index Scripts are written in JavaScript.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> company = LoadDocument(<span class="code-keyword">this</span>.Company);<br/><span class="code-keyword">if</span> (company == null) <span class="code-keyword">return</span>;<br/><span class="code-keyword">delete</span> company.Orders;<br/>PutDocument(<span class="code-keyword">this</span>.Company, company);</pre>',
                selector: '.delete-script-label'
            };
            $('#scriptedIndexesForm').popover(deleteScriptPopOverSettings);
        };
        //when pressing the save button, show all elements which are collapsed and at least one of its' fields isn't valid.
        scriptedIndexes.prototype.initializeCollapsedInvalidElements = function () {
            $('textarea').bind('invalid', function (e) {
                var element = e.target;
                if (!element.validity.valid) {
                    var parentElement = $(this).parents('.panel-default');
                    parentElement.children('.collapse').collapse('show');
                }
            });
        };
        scriptedIndexes.prototype.initializeAceValidity = function (element, editorValue) {
            if (editorValue === "") {
                var textarea = $(element).find('textarea')[0];
                textarea.setCustomValidity("Please fill out this field.");
            }
        };
        scriptedIndexes.prototype.updateIndexes = function (serverIndexes) {
            for (var i = 0; i < this.allScriptedIndexes().length; i++) {
                var index = this.allScriptedIndexes()[i];
                var serverIndex = serverIndexes.find(function (k) { return k.Key === index.getId(); });
                if (serverIndex && !serverIndex.Deleted) {
                    index.__metadata.etag(serverIndex.Etag);
                    index.__metadata.lastModified = serverIndex.Metadata['Last-Modified']; //TODO last -modified is not longer used? use Raven-Last-Modified
                }
                else if (serverIndex && serverIndex.Deleted) {
                    this.allScriptedIndexes().splice(i--, 1);
                }
            }
        };
        return scriptedIndexes;
    }(viewModelBase));
    return scriptedIndexes;
});

//# sourceMappingURL=scriptedIndexes.js.map
