var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "viewmodels/database/indexes/visualizer/visualizerGraphGlobal", "viewmodels/database/indexes/visualizer/visualizerGraphDetails", "commands/database/index/getIndexesStatsCommand", "commands/database/index/getIndexMapReduceTreeCommand", "commands/database/index/getIndexDebugSourceDocumentsCommand"], function (require, exports, viewModelBase, visualizerGraphGlobal, visualizerGraphDetails, getIndexesStatsCommand, getIndexMapReduceTreeCommand, getIndexDebugSourceDocumentsCommand) {
    "use strict";
    var visualizer = (function (_super) {
        __extends(visualizer, _super);
        function visualizer() {
            var _this = _super.call(this) || this;
            _this.indexes = ko.observableArray();
            _this.indexName = ko.observable();
            _this.currentIndex = ko.observable();
            _this.documents = {
                docKey: ko.observable(""),
                hasFocusDocKey: ko.observable(false),
                loadingDocKeySearchResults: ko.observable(false),
                docKeys: ko.observableArray(),
                docKeysSearchResults: ko.observableArray()
            };
            _this.trees = [];
            _this.globalGraph = new visualizerGraphGlobal();
            _this.detailsGraph = new visualizerGraphDetails();
            _this.bindToCurrentInstance("setSelectedIndex", "selectDocKey", "addCurrentDocumentKey");
            _this.initObservables();
            return _this;
        }
        visualizer.prototype.initObservables = function () {
            var _this = this;
            this.currentIndexUi = ko.pureComputed(function () {
                var currentIndex = _this.currentIndex();
                return currentIndex || visualizer.noIndexSelected;
            });
            this.hasIndexSelected = ko.pureComputed(function () { return !!_this.currentIndex(); });
            this.documents.hasFocusDocKey.subscribe(function (value) {
                if (!value) {
                    return;
                }
                _this.fetchDocKeySearchResults("");
            });
            this.documents.docKey.throttle(100).subscribe(function (query) { return _this.fetchDocKeySearchResults(query); });
        };
        visualizer.prototype.activate = function (args) {
            var _this = this;
            return new getIndexesStatsCommand(this.activeDatabase())
                .execute()
                .done(function (result) { return _this.onIndexesLoaded(result); });
        };
        visualizer.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.globalGraph.init(function (treeName) { return _this.detailsGraph.openFor(treeName); });
            this.detailsGraph.init(function () { return _this.globalGraph.restoreView(); }, this.trees);
        };
        visualizer.prototype.onIndexesLoaded = function (indexes) {
            this.indexes(indexes.filter(function (x) { return x.Type === "AutoMapReduce" || x.Type === "MapReduce"; }).map(function (x) { return x.Name; }));
        };
        visualizer.prototype.setSelectedIndex = function (indexName) {
            this.currentIndex(indexName);
            this.resetGraph();
        };
        visualizer.prototype.resetGraph = function () {
            this.documents.docKeys([]);
            this.documents.docKey("");
            this.documents.docKeysSearchResults([]);
            this.globalGraph.reset();
            this.detailsGraph.reset();
        };
        visualizer.prototype.addCurrentDocumentKey = function () {
            this.addDocKey(this.documents.docKey());
        };
        visualizer.prototype.addDocKey = function (key) {
            var _this = this;
            if (!key) {
                return;
            }
            if (_.includes(this.documents.docKeys(), key)) {
                this.globalGraph.zoomToDocument(key);
            }
            else {
                //TODO: spinner
                new getIndexMapReduceTreeCommand(this.activeDatabase(), this.currentIndex(), key)
                    .execute()
                    .done(function (mapReduceTrees) {
                    if (!_.includes(_this.documents.docKeys(), key)) {
                        _this.documents.docKeys.push(key);
                        _this.addDocument(key);
                        _this.addTrees(mapReduceTrees);
                        _this.globalGraph.zoomToDocument(key);
                    }
                });
            }
        };
        visualizer.prototype.addDocument = function (docName) {
            this.globalGraph.addDocument(docName);
            this.detailsGraph.addDocument(docName);
        };
        visualizer.prototype.addTrees = function (result) {
            var treesToAdd = [];
            var _loop_1 = function (i) {
                var incomingTree = result[i];
                var existingTree = this_1.trees.find(function (x) { return x.Name === incomingTree.Name; });
                if (existingTree) {
                    this_1.mergeTrees(incomingTree, existingTree);
                    treesToAdd.push(existingTree);
                }
                else {
                    treesToAdd.push(incomingTree);
                    this_1.trees.push(incomingTree);
                }
            };
            var this_1 = this;
            for (var i = 0; i < result.length; i++) {
                _loop_1(i);
            }
            this.globalGraph.addTrees(treesToAdd);
            this.detailsGraph.setDocumentsColors(this.globalGraph.getDocumentsColors());
        };
        visualizer.prototype.mergeTrees = function (incoming, mergeOnto) {
            if (incoming.PageCount !== mergeOnto.PageCount || incoming.NumberOfEntries !== mergeOnto.NumberOfEntries) {
                throw new Error("Looks like tree data was changed. Can't render graph");
            }
            var existingLeafs = visualizer.extractLeafs(mergeOnto.Root);
            var newLeafs = visualizer.extractLeafs(incoming.Root);
            existingLeafs.forEach(function (page, pageNumber) {
                var newPage = newLeafs.get(pageNumber);
                for (var i = 0; i < newPage.Entries.length; i++) {
                    if (newPage.Entries[i].Source) {
                        page.Entries[i].Source = newPage.Entries[i].Source;
                    }
                }
            });
        };
        visualizer.extractLeafs = function (root) {
            var result = new Map();
            var visitor = function (node) {
                if (node.Entries && node.Entries.length) {
                    result.set(node.PageNumber, node);
                }
                if (node.Children) {
                    for (var i = 0; i < node.Children.length; i++) {
                        visitor(node.Children[i]);
                    }
                }
            };
            visitor(root);
            return result;
        };
        visualizer.prototype.selectDocKey = function (value) {
            this.addDocKey(value);
            this.documents.docKey("");
            this.documents.docKeysSearchResults.removeAll();
        };
        visualizer.prototype.fetchDocKeySearchResults = function (query) {
            var _this = this;
            this.documents.loadingDocKeySearchResults(true);
            new getIndexDebugSourceDocumentsCommand(this.activeDatabase(), this.currentIndex(), query, 0, 10)
                .execute()
                .done(function (result) {
                if (_this.documents.docKey() === query) {
                    _this.documents.docKeysSearchResults(result.Results);
                }
            })
                .always(function () { return _this.documents.loadingDocKeySearchResults(false); });
        };
        return visualizer;
    }(viewModelBase));
    visualizer.noIndexSelected = "Select an index";
    return visualizer;
});

//# sourceMappingURL=visualizer.js.map
