var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/helpers/graph/graphHelper", "d3"], function (require, exports, graphHelper, d3) {
    "use strict";
    var reduceValuesFormatter = (function () {
        function reduceValuesFormatter() {
        }
        reduceValuesFormatter.formatData = function (data) {
            var output = "";
            var valuesMap = reduceValuesFormatter.extractValues(data);
            var first = true;
            valuesMap.forEach(function (value, key) {
                if (!first)
                    output += ", ";
                first = false;
                output += key + ": " + reduceValuesFormatter.formatValue(data[key]);
            });
            return output;
        };
        reduceValuesFormatter.formatValue = function (value) {
            if (value === null) {
                return "null";
            }
            if (typeof (value) === "number") {
                return value.toLocaleString();
            }
            if (typeof (value) === "string") {
                return value;
            }
            return JSON.stringify(value);
        };
        reduceValuesFormatter.extractValues = function (object) {
            var result = new Map();
            var keys = Object.keys(object);
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                var value = reduceValuesFormatter.formatValue(object[key]);
                result.set(key, value);
            }
            return result;
        };
        return reduceValuesFormatter;
    }());
    var layoutableItem = (function () {
        function layoutableItem() {
        }
        return layoutableItem;
    }());
    var documentItem = (function (_super) {
        __extends(documentItem, _super);
        function documentItem(name) {
            var _this = _super.call(this) || this;
            _this.connectedEntries = [];
            _this.name = name;
            return _this;
        }
        Object.defineProperty(documentItem.prototype, "visible", {
            get: function () {
                return this.connectedEntries.length;
            },
            enumerable: true,
            configurable: true
        });
        documentItem.prototype.getSourceConnectionPoint = function () {
            return [this.x + this.width / 2, this.y];
        };
        documentItem.prototype.reset = function () {
            this.connectedEntries = [];
        };
        return documentItem;
    }(layoutableItem));
    var entryItem = (function (_super) {
        __extends(entryItem, _super);
        function entryItem(source, data) {
            var _this = _super.call(this) || this;
            _this.data = data;
            _this.source = source;
            _this.dataAsString = reduceValuesFormatter.formatData(data);
            return _this;
        }
        entryItem.estimateTextWidth = function (text) {
            return text.length * 6;
        };
        entryItem.prototype.getGlobalTargetConnectionPoint = function () {
            return [this.x + this.parent.x, this.y + this.parent.y + this.height / 2];
        };
        return entryItem;
    }(layoutableItem));
    var entryPaddingItem = (function (_super) {
        __extends(entryPaddingItem, _super);
        function entryPaddingItem() {
            return _super.apply(this, arguments) || this;
        }
        return entryPaddingItem;
    }(layoutableItem));
    entryPaddingItem.margins = {
        minWidth: 130
    };
    var pageItem = (function (_super) {
        __extends(pageItem, _super);
        function pageItem(parentPage, pageNumber, aggregationResult) {
            var _this = _super.call(this) || this;
            _this.pageNumber = pageNumber;
            _this.parentPage = parentPage;
            _this.aggregationResult = aggregationResult;
            _this.aggregationResultAsMap = reduceValuesFormatter.extractValues(aggregationResult);
            return _this;
        }
        pageItem.prototype.layout = function () {
            this.height = pageItem.margins.pageNumberTopMargin + pageItem.margins.entriesAndAggregationTextHeight
                + pageItem.margins.aggregationItemHeight * this.aggregationResultAsMap.size
                + pageItem.margins.bottomPadding;
            var longestTextWidth = pageItem.estimateAggregationTextWidth(this.findLongestAggregationTextLength());
            this.width = pageItem.margins.aggragationTextHorizontalPadding + longestTextWidth + pageItem.margins.aggragationTextHorizontalPadding;
        };
        pageItem.prototype.findLongestAggregationTextLength = function () {
            var max = 0;
            this.aggregationResultAsMap.forEach(function (v, k) {
                var len = k.length + 2 + v.length;
                if (len > max) {
                    max = len;
                }
            });
            return max;
        };
        pageItem.estimateAggregationTextWidth = function (textLength) {
            return textLength * 6;
        };
        pageItem.prototype.getSourceConnectionPoint = function () {
            return [this.x + this.width / 2, this.y];
        };
        pageItem.prototype.getTargetConnectionPoint = function () {
            return [this.x + this.width / 2, this.y + this.height];
        };
        return pageItem;
    }(layoutableItem));
    pageItem.margins = {
        horizonalPadding: 5,
        pageNumberTopMargin: 12,
        entryHeight: 22,
        betweenEntryPadding: 1,
        bottomPadding: 5,
        entryTextPadding: 5,
        aggregationItemHeight: 16,
        aggragationTextHorizontalPadding: 10,
        betweenPagesMinWidth: 40,
        entriesAndAggregationTextHeight: 20,
        nestedSection: {
            width: 58,
            height: 31,
            rightMargin: 7,
            topMargin: 42
        }
    };
    var leafPageItem = (function (_super) {
        __extends(leafPageItem, _super);
        function leafPageItem(parentPage, pageNumber, aggregationResult, nestedSection, entries) {
            var _this = _super.call(this, parentPage, pageNumber, aggregationResult) || this;
            _this.entries = [];
            _this.nestedSection = nestedSection;
            _this.entries = entries;
            return _this;
        }
        leafPageItem.prototype.layout = function () {
            _super.prototype.layout.call(this);
            var entriesData = this.entries
                .filter(function (x) { return x instanceof entryItem; })
                .map(function (x) { return x.dataAsString; })
                .sort(function (a, b) { return a.length > b.length ? -1 : 1; });
            var longestText = entriesData[0];
            var longestTextWidth = longestText ? entryItem.estimateTextWidth(longestText) : entryPaddingItem.margins.minWidth;
            var entriesWidth = pageItem.margins.horizonalPadding + pageItem.margins.entryTextPadding + longestTextWidth + pageItem.margins.entryTextPadding + pageItem.margins.horizonalPadding;
            this.width = Math.max(entriesWidth, this.width);
            this.height -= pageItem.margins.bottomPadding;
            this.entriesTextY = this.height;
            var yStart = this.height + pageItem.margins.entriesAndAggregationTextHeight;
            var yOffset = pageItem.margins.betweenEntryPadding + pageItem.margins.entryHeight;
            for (var i = 0; i < this.entries.length; i++) {
                var entry = this.entries[i];
                entry.x = pageItem.margins.horizonalPadding;
                entry.y = yStart;
                entry.width = longestTextWidth + 2 * pageItem.margins.entryTextPadding;
                entry.height = pageItem.margins.entryHeight;
                yStart += yOffset;
            }
            yStart += -pageItem.margins.betweenEntryPadding + pageItem.margins.bottomPadding;
            this.height = yStart;
        };
        leafPageItem.findEntries = function (documents, entries) {
            var documentNames = documents.map(function (x) { return x.name; });
            var hasAnySource = false;
            // get elements with source and one before and ahead
            var entiresToTake = new Array(entries.length).fill(0);
            for (var i = 0; i < entries.length; i++) {
                var entry = entries[i];
                if (entry.Source) {
                    hasAnySource = true;
                    if (i > 0)
                        entiresToTake[i - 1] = 1;
                    entiresToTake[i] = 1;
                    if (i < entries.length - 1)
                        entiresToTake[i + 1] = 1;
                }
            }
            if (!hasAnySource) {
                // display few first items then
                for (var i = 0; i < Math.min(entries.length, 3); i++) {
                    entiresToTake[i] = 1;
                }
            }
            var result = [];
            var _loop_1 = function (i) {
                if (entiresToTake[i]) {
                    var entry_1 = new entryItem(entries[i].Source, entries[i].Data);
                    if (entry_1.source) {
                        var matchedDocument = documents.find(function (x) { return x.name === entry_1.source; });
                        if (matchedDocument) {
                            matchedDocument.connectedEntries.push(entry_1);
                        }
                    }
                    result.push(entry_1);
                }
                else {
                    if (result.length === 0 || _.last(result) instanceof entryItem) {
                        result.push(new entryPaddingItem());
                    }
                }
            };
            for (var i = 0; i < entries.length; i++) {
                _loop_1(i);
            }
            return result;
        };
        return leafPageItem;
    }(pageItem));
    var branchPageItem = (function (_super) {
        __extends(branchPageItem, _super);
        function branchPageItem(parentPage, pageNumber, aggregationResult) {
            return _super.call(this, parentPage, pageNumber, aggregationResult) || this;
        }
        return branchPageItem;
    }(pageItem));
    var reduceTreeItem = (function () {
        function reduceTreeItem(tree) {
            this.itemsAtDepth = new Map(); // items after filtering depth -> list of items
            this.tree = tree;
            this.displayName = tree.DisplayName;
            this.depth = tree.Depth;
            this.countItemsPerDepth();
        }
        reduceTreeItem.prototype.countItemsPerDepth = function () {
            var _this = this;
            this.itemsCountAtDepth = new Array(this.depth);
            var countEntries = function (depth, node) {
                _this.itemsCountAtDepth[depth] = (_this.itemsCountAtDepth[depth] || 0) + 1;
                if (node.Children && node.Children.length) {
                    for (var i = 0; i < node.Children.length; i++) {
                        countEntries(depth + 1, node.Children[i]);
                    }
                }
            };
            countEntries(0, this.tree.Root);
        };
        reduceTreeItem.prototype.filterAndLayoutVisibleItems = function (documents) {
            this.cleanCache(documents);
            this.filterVisibleItems(documents);
            return this.layout();
        };
        reduceTreeItem.prototype.cleanCache = function (documents) {
            this.itemsAtDepth.clear();
        };
        reduceTreeItem.prototype.filterVisibleItems = function (documents) {
            var _this = this;
            var filterAtDepth = function (depth, node, parentPage) {
                if (!_this.itemsAtDepth.has(depth)) {
                    _this.itemsAtDepth.set(depth, []);
                }
                var items = _this.itemsAtDepth.get(depth);
                //TODO: can we have both in simple pages?
                if (node.Children) {
                    var item = new branchPageItem(parentPage, node.PageNumber, node.AggregationResult);
                    items.push(item);
                    for (var i = 0; i < node.Children.length; i++) {
                        filterAtDepth(depth + 1, node.Children[i], item);
                    }
                }
                if (node.Entries && node.Entries.length) {
                    var entries = leafPageItem.findEntries(documents, node.Entries);
                    var isNestedSection = depth === 0; // if depth is zero and node has entries
                    var item_1 = new leafPageItem(parentPage, node.PageNumber, node.AggregationResult, isNestedSection, entries);
                    entries.filter(function (x) { return x instanceof entryItem; }).forEach(function (entry) { return entry.parent = item_1; });
                    items.push(item_1);
                }
            };
            filterAtDepth(0, this.tree.Root, null);
        };
        reduceTreeItem.prototype.layout = function () {
            this.itemsAtDepth.forEach(function (pages, depth) {
                pages.forEach(function (page) { return page.layout(); });
            });
            var lastLevelItems = this.itemsAtDepth.get(this.depth - 1);
            // make all items at last level at the same size
            var maxWidth = d3.max(lastLevelItems, function (x) { return x.width; });
            for (var i = 0; i < lastLevelItems.length; i++) {
                lastLevelItems[i].width = maxWidth;
            }
            this.totalWidth = lastLevelItems.reduce(function (p, c) { return p + c.width; }, 0)
                + (lastLevelItems.length + 1) * pageItem.margins.betweenPagesMinWidth;
            var yStart = visualizerGraphDetails.margins.top;
            var avgElementWidthPerDepth = Array.from(this.itemsAtDepth.values()).map(function (pages) {
                var totalWidth = pages.reduce(function (p, c) { return p + c.width; }, 0);
                return totalWidth / pages.length;
            });
            var maxHeightPerLevel = Array.from(this.itemsAtDepth.values()).map(function (pages) { return d3.max(pages, function (x) { return x.height; }); });
            for (var depth = 0; depth < this.depth; depth++) {
                var items = this.itemsAtDepth.get(depth);
                var startAndOffset = graphHelper.computeStartAndOffset(this.totalWidth, items.length, avgElementWidthPerDepth[depth]);
                var xOffset = startAndOffset.offset;
                var xStart = startAndOffset.start;
                for (var i = 0; i < items.length; i++) {
                    var item = items[i];
                    item.x = xStart;
                    item.y = yStart;
                    xStart += xOffset;
                }
                yStart += maxHeightPerLevel[depth] + visualizerGraphDetails.margins.verticalMarginBetweenLevels;
            }
            return yStart;
        };
        return reduceTreeItem;
    }());
    var visualizerGraphDetails = (function () {
        function visualizerGraphDetails() {
            this.totalWidth = 1500; //TODO: use dynamic value
            this.totalHeight = 700; //TODO: use dynamic value
            this.documents = [];
            this.viewActive = ko.observable(false);
            this.currentTreeIndex = 0;
            this.currentLineOffset = 0;
            this.connectionsBaseY = 0;
            _.bindAll(this, "goToMasterView");
        }
        visualizerGraphDetails.prototype.init = function (goToMasterViewCallback, trees) {
            var _this = this;
            this.gotoMasterViewCallback = goToMasterViewCallback;
            this.trees = trees;
            var container = d3.select("#visualizerContainer");
            this.canvas = container
                .append("canvas")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight);
            this.svg = container
                .append("svg")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight);
            this.toggleUiElements(false);
            this.xScale = d3.scale.linear()
                .domain([0, this.totalWidth])
                .range([0, this.totalWidth]);
            this.yScale = d3.scale.linear()
                .domain([0, this.totalHeight])
                .range([0, this.totalHeight]);
            this.zoom = d3.behavior.zoom()
                .x(this.xScale)
                .y(this.yScale)
                .on("zoom", function () { return _this.onZoom(); });
            this.svg
                .append("svg:rect")
                .attr("class", "pane")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight)
                .call(this.zoom)
                .call(function (d) { return _this.setupEvents(d); });
        };
        visualizerGraphDetails.prototype.addDocument = function (documentName) {
            var document = new documentItem(documentName);
            this.currentLineOffset += visualizerGraphDetails.margins.betweenLinesOffset;
            this.documents.push(document);
        };
        visualizerGraphDetails.prototype.reset = function () {
            this.restoreView();
            this.documents = [];
        };
        visualizerGraphDetails.prototype.setDocumentsColors = function (documentsColorsSetup) {
            var _loop_2 = function (i) {
                var documentItem_1 = this_1.documents.find(function (d) { return d.name === documentsColorsSetup[i].docName; });
                documentItem_1.color = documentsColorsSetup[i].docColor;
            };
            var this_1 = this;
            for (var i = 0; i < documentsColorsSetup.length; i++) {
                _loop_2(i);
            }
        };
        visualizerGraphDetails.prototype.restoreView = function () {
            this.zoom.translate([0, 0]).scale(1).event(this.canvas);
            this.documents.forEach(function (doc) { return doc.reset(); });
        };
        visualizerGraphDetails.prototype.onZoom = function () {
            this.draw();
        };
        visualizerGraphDetails.prototype.setupEvents = function (selection) {
            selection.on("dblclick.zoom", null);
            //TODO: allow to click selection.on("click", () => this.onClick());
        };
        visualizerGraphDetails.prototype.goToMasterView = function () {
            this.viewActive(false);
            this.toggleUiElements(false);
            //TODO: exit animation before calling callback
            this.gotoMasterViewCallback();
        };
        visualizerGraphDetails.prototype.openFor = function (treeName) {
            this.restoreView();
            this.viewActive(true); //TODO: consider setting this after initial animation if any
            this.toggleUiElements(true);
            var treeIdx = this.trees.findIndex(function (x) { return x.Name === treeName; });
            this.currentTreeIndex = treeIdx;
            this.currentTree = new reduceTreeItem(this.trees[this.currentTreeIndex]); //TODO: consider moving this to layout?
            this.layout();
            var initialTranslation = [this.totalWidth / 2 - this.currentTree.totalWidth / 2, 0];
            this.zoom.translate(initialTranslation).scale(1).event(this.canvas);
            this.draw();
        };
        visualizerGraphDetails.prototype.layout = function () {
            var yStart = this.currentTree.filterAndLayoutVisibleItems(this.documents);
            var visibleDocuments = this.getVisibleDocuments();
            this.connectionsBaseY = yStart
                + visualizerGraphDetails.margins.betweenTreesAndDocumentsPadding / 2
                - (visualizerGraphDetails.margins.betweenLinesOffset * visibleDocuments.length) / 2;
            yStart += visualizerGraphDetails.margins.betweenTreesAndDocumentsPadding;
            this.layoutDocuments(yStart);
        };
        // Return the visible items for the detailed view, ordered by their connectedEntries y coordinate
        visualizerGraphDetails.prototype.getVisibleDocuments = function () {
            return this.documents.filter(function (x) { return x.visible; }).sort(function (a, b) { return a.connectedEntries[0].y - b.connectedEntries[0].y; });
        };
        // set x & y of document & the draw offsets
        visualizerGraphDetails.prototype.layoutDocuments = function (yStart) {
            var totalWidth = 0;
            var visibleDocuments = this.getVisibleDocuments();
            for (var i = 0; i < visibleDocuments.length; i++) {
                var doc = visibleDocuments[i];
                var documentNameWidthEstimation = function (text) { return text.length * 9; };
                doc.width = visualizerGraphDetails.margins.badgePadding * 2 + documentNameWidthEstimation(doc.name);
                doc.height = 35;
                doc.y = yStart;
                totalWidth += doc.width;
            }
            totalWidth += visibleDocuments.length * (visualizerGraphDetails.margins.minMarginBetweenDocumentNames + 1);
            var extraItemPadding = 0;
            if (totalWidth > this.currentTree.totalWidth) {
            }
            else {
                extraItemPadding = (this.currentTree.totalWidth - totalWidth) / (visibleDocuments.length + 1);
            }
            // if there are many documents to draw than we better start with a lower y coordinate
            var yOffsetOfHorizontalLine = visibleDocuments.length > 10 ? 20 : 0;
            var currentX = visualizerGraphDetails.margins.minMarginBetweenDocumentNames + extraItemPadding;
            for (var i = 0; i < visibleDocuments.length; i++) {
                var doc = visibleDocuments[i];
                doc.x = currentX;
                currentX += doc.width + visualizerGraphDetails.margins.minMarginBetweenDocumentNames + extraItemPadding;
                doc.drawOffset = yOffsetOfHorizontalLine;
                yOffsetOfHorizontalLine -= visualizerGraphDetails.margins.betweenLinesOffset;
            }
        };
        visualizerGraphDetails.prototype.draw = function () {
            var canvas = this.canvas.node();
            var ctx = canvas.getContext("2d");
            ctx.fillStyle = "#2c3333";
            ctx.fillRect(0, 0, this.totalWidth, this.totalHeight);
            ctx.save();
            try {
                var translation = this.zoom.translate();
                ctx.translate(translation[0], translation[1]);
                ctx.scale(this.zoom.scale(), this.zoom.scale());
                if (this.currentTree) {
                    this.drawTree(ctx, this.currentTree);
                }
                var visibleDocuments = this.getVisibleDocuments();
                for (var i = 0; i < visibleDocuments.length; i++) {
                    var doc = visibleDocuments[i];
                    this.drawDocument(ctx, doc);
                }
                this.drawDocumentConnections(ctx);
            }
            finally {
                ctx.restore();
            }
        };
        visualizerGraphDetails.prototype.drawTree = function (ctx, tree) {
            var _this = this;
            tree.itemsAtDepth.forEach(function (pages) {
                for (var i = 0; i < pages.length; i++) {
                    var page = pages[i];
                    _this.drawPage(ctx, page);
                    if (page.parentPage) {
                        ctx.strokeStyle = "#686f6f";
                        ctx.lineWidth = 2;
                        var sourcePoint = page.getSourceConnectionPoint();
                        var targetPoint = page.parentPage.getTargetConnectionPoint();
                        var middleY = (sourcePoint[1] + targetPoint[1]) / 2;
                        ctx.beginPath();
                        ctx.moveTo(sourcePoint[0], sourcePoint[1]);
                        ctx.lineTo(sourcePoint[0], middleY);
                        ctx.lineTo(targetPoint[0], middleY);
                        ctx.lineTo(targetPoint[0], targetPoint[1]);
                        ctx.stroke();
                        ctx.beginPath();
                        ctx.moveTo(targetPoint[0] - visualizerGraphDetails.margins.arrowWidth, targetPoint[1] + visualizerGraphDetails.margins.arrowHalfHeight);
                        ctx.lineTo(targetPoint[0], targetPoint[1]);
                        ctx.lineTo(targetPoint[0] + visualizerGraphDetails.margins.arrowWidth, targetPoint[1] + visualizerGraphDetails.margins.arrowHalfHeight);
                        ctx.stroke();
                    }
                }
            });
        };
        visualizerGraphDetails.prototype.drawPage = function (ctx, page) {
            ctx.fillStyle = "#3a4242";
            ctx.fillRect(page.x, page.y, page.width, page.height);
            ctx.save();
            ctx.translate(page.x, page.y);
            try {
                // page number
                ctx.fillStyle = "#008cc9";
                ctx.font = "bold 24px Lato";
                ctx.textAlign = "right";
                ctx.textBaseline = "top";
                ctx.fillText("#" + page.pageNumber, page.width - pageItem.margins.aggragationTextHorizontalPadding, pageItem.margins.pageNumberTopMargin);
                if (page instanceof leafPageItem && page.nestedSection) {
                    var nestedSectionMargins = pageItem.margins.nestedSection;
                    ctx.fillStyle = "#008cc9";
                    ctx.fillRect(page.width - nestedSectionMargins.rightMargin - nestedSectionMargins.width, nestedSectionMargins.topMargin, nestedSectionMargins.width, nestedSectionMargins.height);
                    ctx.textAlign = "center";
                    ctx.fillStyle = "white";
                    ctx.font = "11px Lato";
                    ctx.fillText("NESTED", page.width - nestedSectionMargins.rightMargin - nestedSectionMargins.width / 2, nestedSectionMargins.topMargin + 3);
                    ctx.fillText("SECTION", page.width - nestedSectionMargins.rightMargin - nestedSectionMargins.width / 2, nestedSectionMargins.topMargin + 15);
                }
                this.drawAggregation(ctx, page);
                if (page instanceof leafPageItem) {
                    this.drawEntries(ctx, page);
                }
            }
            finally {
                ctx.restore();
            }
        };
        visualizerGraphDetails.prototype.drawEntries = function (ctx, leaf) {
            var entries = leaf.entries;
            ctx.font = "11px Lato";
            ctx.fillStyle = "#626969";
            ctx.textAlign = "left";
            ctx.textBaseline = "top";
            ctx.fillText("Entries:", pageItem.margins.aggragationTextHorizontalPadding, leaf.entriesTextY);
            for (var i = 0; i < entries.length; i++) {
                var entry = entries[i];
                ctx.fillStyle = "#2c3333";
                ctx.fillRect(entry.x, entry.y, entry.width, entry.height);
                ctx.fillStyle = "#a9adad";
                ctx.font = "12px Lato";
                if (entry instanceof entryPaddingItem) {
                    ctx.textAlign = "center";
                    ctx.fillText("...", entry.x + entry.width / 2, entry.y);
                }
                else {
                    var castedEntry = entry;
                    ctx.textAlign = "left";
                    ctx.fillText(castedEntry.dataAsString, castedEntry.x + pageItem.margins.entryTextPadding, castedEntry.y + 3);
                }
            }
        };
        visualizerGraphDetails.prototype.drawAggregation = function (ctx, branch) {
            ctx.font = "11px Lato";
            ctx.fillStyle = "#626969";
            ctx.textAlign = "left";
            ctx.textBaseline = "top";
            ctx.fillText("Aggregation:", pageItem.margins.aggragationTextHorizontalPadding, pageItem.margins.pageNumberTopMargin);
            ctx.fillStyle = "#a9adad";
            ctx.font = "12px Lato";
            ctx.textAlign = "left";
            var currentY = pageItem.margins.pageNumberTopMargin + pageItem.margins.entriesAndAggregationTextHeight - 5;
            var yOffset = pageItem.margins.aggregationItemHeight;
            branch.aggregationResultAsMap.forEach(function (value, key) {
                ctx.fillText(key + ": " + value, pageItem.margins.aggragationTextHorizontalPadding, currentY);
                currentY += yOffset;
            });
        };
        visualizerGraphDetails.prototype.drawDocument = function (ctx, docItem) {
            if (docItem.visible) {
                //TODO: it is the same as in global - consider merging?
                ctx.fillStyle = docItem.color;
                ctx.fillRect(docItem.x, docItem.y, docItem.width, docItem.height);
                ctx.textAlign = "center";
                ctx.textBaseline = "middle";
                ctx.font = "18px Lato";
                ctx.fillStyle = "black";
                ctx.fillText(docItem.name, docItem.x + docItem.width / 2, docItem.y + docItem.height / 2);
            }
        };
        visualizerGraphDetails.prototype.drawDocumentConnections = function (ctx) {
            ctx.lineWidth = 2;
            var visibleDocuments = this.getVisibleDocuments();
            for (var i = 0; i < visibleDocuments.length; i++) {
                var doc = visibleDocuments[i];
                ctx.strokeStyle = doc.color;
                for (var j = 0; j < doc.connectedEntries.length; j++) {
                    var entry = doc.connectedEntries[j];
                    var source = doc.getSourceConnectionPoint();
                    var target = entry.getGlobalTargetConnectionPoint();
                    ctx.beginPath();
                    ctx.moveTo(source[0], source[1]);
                    ctx.lineTo(source[0], this.connectionsBaseY + doc.drawOffset);
                    ctx.lineTo(target[0] - visualizerGraphDetails.margins.straightLine - doc.drawOffset - visualizerGraphDetails.margins.betweenLinesOffset * visibleDocuments.length, this.connectionsBaseY + doc.drawOffset);
                    ctx.lineTo(target[0] - visualizerGraphDetails.margins.straightLine - doc.drawOffset - visualizerGraphDetails.margins.betweenLinesOffset * visibleDocuments.length, target[1]);
                    ctx.lineTo(target[0], target[1]);
                    ctx.stroke();
                    ctx.beginPath();
                    ctx.moveTo(target[0] - visualizerGraphDetails.margins.arrowWidth, target[1] - visualizerGraphDetails.margins.arrowHalfHeight);
                    ctx.lineTo(target[0], target[1]);
                    ctx.lineTo(target[0] - visualizerGraphDetails.margins.arrowWidth, target[1] + visualizerGraphDetails.margins.arrowHalfHeight);
                    ctx.stroke();
                }
            }
        };
        visualizerGraphDetails.prototype.toggleUiElements = function (show) {
            this.svg.style("display", show ? "block" : "none");
            this.canvas.style("display", show ? "block" : "none");
        };
        return visualizerGraphDetails;
    }());
    visualizerGraphDetails.margins = {
        top: 40,
        verticalMarginBetweenLevels: 60,
        betweenTreesAndDocumentsPadding: 80,
        badgePadding: 30,
        minMarginBetweenDocumentNames: 30,
        straightLine: 12,
        arrowHalfHeight: 6,
        arrowWidth: 8,
        betweenLinesOffset: 5
    };
    return visualizerGraphDetails;
});

//# sourceMappingURL=visualizerGraphDetails.js.map
