var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "viewmodels/viewModelBase", "models/database/patch/patchDocument", "common/bindingHelpers/aceEditorBindingHandler", "commands/resources/getDatabaseStatsCommand", "commands/database/documents/getCollectionsStatsCommand", "models/database/documents/collection", "models/database/documents/document", "common/pagedList", "common/jsonUtil", "common/appUrl", "commands/database/query/queryIndexCommand", "commands/database/documents/getDocumentWithMetadataCommand", "viewmodels/database/patch/savePatch", "viewmodels/database/patch/executePatchConfirm", "commands/database/patch/savePatchCommand", "commands/database/patch/executePatchCommand", "commands/database/patch/evalByQueryCommand", "commands/database/patch/evalByCollectionCommand", "models/database/documents/documentMetadata", "commands/database/index/getIndexDefinitionCommand", "common/queryUtil", "common/recentPatchesStorage", "commands/database/patch/getPatchesCommand", "commands/operations/killOperationCommand", "common/eventsCollector", "common/notifications/notificationCenter", "common/generalUtils"], function (require, exports, app, viewModelBase, patchDocument, aceEditorBindingHandler, getDatabaseStatsCommand, getCollectionsStatsCommand, collection, document, pagedList, jsonUtil, appUrl, queryIndexCommand, getDocumentWithMetadataCommand, savePatch, executePatchConfirm, savePatchCommand, executePatchCommand, evalByQueryCommand, evalByCollectionCommand, documentMetadata, getIndexDefinitionCommand, queryUtil, recentPatchesStorage, getPatchesCommand, killOperationComamnd, eventsCollector, notificationCenter, genUtils) {
    "use strict";
    var patch = (function (_super) {
        __extends(patch, _super);
        function patch() {
            var _this = _super.call(this) || this;
            _this.displayName = "patch";
            _this.indices = ko.observableArray([]);
            _this.collections = ko.observableArray([]);
            _this.recentPatches = ko.observableArray();
            _this.savedPatches = ko.observableArray();
            _this.currentCollectionPagedItems = ko.observable();
            _this.selectedDocumentIndices = ko.observableArray();
            _this.patchDocument = ko.observable();
            _this.beforePatchDoc = ko.observable();
            _this.beforePatchMeta = ko.observable();
            _this.beforePatchDocMode = ko.observable(true);
            _this.afterPatch = ko.observable();
            _this.afterPatchDoc = ko.observable();
            _this.afterPatchMeta = ko.observable();
            _this.afterPatchDocMode = ko.observable(true);
            _this.loadedDocuments = ko.observableArray();
            _this.putDocuments = ko.observableArray();
            _this.outputLog = ko.observableArray();
            _this.runningPatchesCount = ko.observable();
            _this.runningTasksUrl = ko.computed(function () { return appUrl.forRunningTasks(_this.activeDatabase()); });
            _this.runningPatchesText = ko.computed(function () {
                var count = _this.runningPatchesCount();
                if (count > 1) {
                    return count + ' patches in progress';
                }
                return count + ' patch in progress';
            });
            _this.documentKey = ko.observable();
            _this.isPatchingInProgress = ko.observable(false);
            _this.showPatchingProgress = ko.observable(false);
            _this.patchOperationId = ko.observable();
            _this.patchingProgress = ko.observable(0);
            _this.patchingProgressText = ko.observable();
            _this.patchSuccess = ko.observable(false);
            _this.patchFailure = ko.observable(false);
            _this.patchKillInProgress = ko.observable(false);
            _this.indexFields = ko.observableArray();
            _this.selectedIndex = ko.observable();
            _this.dynamicPrefix = "dynamic/";
            _this.isTestIndex = ko.observable(false);
            _this.queryText = ko.observable("");
            aceEditorBindingHandler.install();
            // When we programmatically change the document text or meta text, push it into the editor.
            _this.beforePatchDocMode.subscribe(function () {
                if (_this.beforePatchEditor) {
                    var text = _this.beforePatchDocMode() ? _this.beforePatchDoc() : _this.beforePatchMeta();
                    _this.beforePatchEditor.getSession().setValue(text);
                }
            });
            _this.beforePatch = ko.computed({
                read: function () {
                    return _this.beforePatchDocMode() ? _this.beforePatchDoc() : _this.beforePatchMeta();
                },
                write: function (text) {
                    var currentObservable = _this.beforePatchDocMode() ? _this.beforePatchDoc : _this.beforePatchMeta;
                    currentObservable(text);
                },
                owner: _this
            });
            _this.afterPatchDocMode.subscribe(function () {
                if (_this.afterPatchEditor) {
                    var text = _this.afterPatchDocMode() ? _this.afterPatchDoc() : _this.afterPatchMeta();
                    _this.afterPatchEditor.getSession().setValue(text);
                }
            });
            _this.afterPatch = ko.computed({
                read: function () {
                    return _this.afterPatchDocMode() ? _this.afterPatchDoc() : _this.afterPatchMeta();
                },
                write: function (text) {
                    var currentObservable = _this.afterPatchDocMode() ? _this.afterPatchDoc : _this.afterPatchMeta;
                    currentObservable(text);
                },
                owner: _this
            });
            // Refetch the index fields whenever the selected index name changes.
            _this.selectedIndex
                .subscribe(function (indexName) {
                if (indexName) {
                    _this.fetchIndexFields(indexName);
                }
            });
            _this.indicesToSelect = ko.computed(function () {
                var indicies = _this.indices();
                var patchDocument = _this.patchDocument();
                if (indicies.length === 0 || !patchDocument)
                    return [];
                return indicies.filter(function (x) { return x.name !== patchDocument.selectedItem(); });
            });
            _this.collectionToSelect = ko.computed(function () {
                var collections = _this.collections();
                var patchDocument = _this.patchDocument();
                if (collections.length === 0 || !patchDocument)
                    return [];
                return collections.filter(function (x) { return x.name !== patchDocument.selectedItem(); });
            });
            _this.showDocumentsPreview = ko.computed(function () {
                if (!_this.patchDocument()) {
                    return false;
                }
                var indexPath = _this.patchDocument().isIndexPatch();
                var collectionPath = _this.patchDocument().isCollectionPatch();
                return indexPath || collectionPath;
            });
            return _this;
        }
        patch.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            var beforePatchEditorElement = $("#beforePatchEditor");
            if (beforePatchEditorElement.length > 0) {
                this.beforePatchEditor = ko.utils.domData.get(beforePatchEditorElement[0], "aceEditor");
            }
            var afterPatchEditorElement = $("#afterPatchEditor");
            if (afterPatchEditorElement.length > 0) {
                this.afterPatchEditor = ko.utils.domData.get(afterPatchEditorElement[0], "aceEditor");
            }
        };
        patch.prototype.activate = function (recentPatchHash) {
            var _this = this;
            _super.prototype.activate.call(this, recentPatchHash);
            this.updateHelpLink("QGGJR5");
            this.patchDocument(patchDocument.empty());
            this.queryText.throttle(1000).subscribe(function () {
                _this.runQuery();
            });
            this.isExecuteAllowed = ko.computed(function () { return !!_this.patchDocument().script() && !!_this.beforePatchDoc(); });
            this.isMapReduceIndexSelected = ko.computed(function () {
                if (_this.patchDocument().patchOnOption() !== "Index") {
                    return false;
                }
                var indexName = _this.selectedIndex();
                var usedIndex = _this.indices().find(function (x) { return x.name === indexName; });
                if (usedIndex) {
                    return usedIndex.isMapReduce;
                }
                return false;
            });
            this.keyOfTestedDocument = ko.computed(function () {
                switch (_this.patchDocument().patchOnOption()) {
                    case "Collection":
                    case "Index":
                        return _this.documentKey();
                    case "Document":
                        return _this.patchDocument().selectedItem();
                }
            });
            this.selectedDocumentIndices.subscribe(function (list) {
                if (list.length === 1) {
                    var firstCheckedOnList = list[0];
                    _this.currentCollectionPagedItems().getNthItem(firstCheckedOnList)
                        .done(function (document) {
                        // load document directly from server as documents on list are loaded using doc-preview endpoint, which doesn't display entire document
                        _this.loadDocumentToTest(document.__metadata.id);
                        _this.documentKey(document.__metadata.id);
                    });
                }
                else {
                    _this.clearDocumentPreview();
                }
            });
            this.patchingProgressPercentage = ko.computed(function () { return _this.patchingProgress() + "%"; });
            var db = this.activeDatabase();
            if (!!db) {
                this.fetchRecentPatches();
                this.fetchAllPatches();
                this.fetchRunningPatches();
            }
            if (recentPatchHash) {
                this.selectInitialPatch(recentPatchHash);
            }
        };
        patch.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#indexQueryLabel").popover({
                html: true,
                trigger: "hover",
                container: '.form-horizontal',
                content: '<p>Queries use Lucene syntax. Examples:</p><pre><span class="code-keyword">Name</span>: Hi?berna*<br/><span class="code-keyword">Count</span>: [0 TO 10]<br/><span class="code-keyword">Title</span>: "RavenDb Queries 1010" AND <span class="code-keyword">Price</span>: [10.99 TO *]</pre>'
            });
            $("#patchScriptsLabel").popover({
                html: true,
                trigger: "hover",
                container: ".form-horizontal",
                content: '<p>Patch Scripts are written in JavaScript. Examples:</p><pre><span class="code-keyword">this</span>.NewProperty = <span class="code-keyword">this</span>.OldProperty + myParameter;<br/><span class="code-keyword">delete this</span>.UnwantedProperty;<br/><span class="code-keyword">this</span>.Comments.RemoveWhere(<span class="code-keyword">function</span>(comment){<br/>  <span class="code-keyword">return</span> comment.Spam;<br/>});</pre>'
            });
            var rowCreatedEvent = app.on(patch.gridSelector + 'RowsCreated').then(function () {
                rowCreatedEvent.off();
            });
            var self = this;
            $(window).bind('storage', function () {
                self.fetchRecentPatches();
            });
        };
        patch.prototype.fetchAllPatches = function () {
            var _this = this;
            new getPatchesCommand(this.activeDatabase())
                .execute()
                .done(function (patches) { return _this.savedPatches(patches); });
        };
        patch.prototype.fetchRunningPatches = function () {
            if (this.runningPatchesPollingHandle)
                return;
            /* TODO:
            new getOperationsCommand(this.activeDatabase())
                .execute()
                .done((tasks: runningTaskDto[]) => {
                    var count = tasks.filter(x => x.TaskType === "IndexBulkOperation" && !x.Completed).length;
                    this.runningPatchesCount(count);
    
                    // we enable polling only if at least one patch is in progress
                    if (count > 0) {
                        this.runningPatchesPollingHandle = setTimeout(() => {
                            this.runningPatchesPollingHandle = null;
                            this.fetchRunningPatches();
                        }, 5000);
                    } else {
                        this.runningPatchesPollingHandle = null;
                    }
                });*/
        };
        patch.prototype.fetchRecentPatches = function () {
            this.recentPatches(recentPatchesStorage.getRecentPatches(this.activeDatabase()));
        };
        patch.prototype.detached = function () {
            _super.prototype.detached.call(this);
            aceEditorBindingHandler.detached();
        };
        patch.prototype.selectInitialPatch = function (recentPatchHash) {
            if (recentPatchHash.indexOf("recentpatch-") === 0) {
                var hash = parseInt(recentPatchHash.substr("recentpatch-".length), 10);
                var matchingPatch = this.recentPatches().find(function (q) { return q.Hash === hash; });
                if (matchingPatch) {
                    this.useRecentPatch(matchingPatch);
                }
                else {
                    this.navigate(appUrl.forPatch(this.activeDatabase()));
                }
            }
        };
        patch.prototype.loadDocumentToTest = function (selectedItem) {
            var _this = this;
            if (selectedItem) {
                var loadDocTask = new getDocumentWithMetadataCommand(selectedItem, this.activeDatabase()).execute();
                loadDocTask.done(function (document) {
                    _this.beforePatchDoc(JSON.stringify(document.toDto(), null, 4));
                    _this.beforePatchMeta(JSON.stringify(documentMetadata.filterMetadata(document.__metadata.toDto()), null, 4));
                }).fail(this.clearDocumentPreview);
            }
            else {
                this.clearDocumentPreview();
            }
        };
        patch.prototype.clearDocumentPreview = function () {
            this.beforePatchDoc("");
            this.beforePatchMeta("");
            this.afterPatchDoc("");
            this.afterPatchMeta("");
            this.putDocuments([]);
            this.loadedDocuments([]);
            this.outputLog([]);
        };
        patch.prototype.setSelectedPatchOnOption = function (patchOnOption) {
            var _this = this;
            this.resetProgressBar();
            this.patchDocument().patchOnOption(patchOnOption);
            this.patchDocument().selectedItem('');
            this.clearDocumentPreview();
            switch (patchOnOption) {
                case "Collection":
                    this.fetchAllCollections();
                    break;
                case "Index":
                    this.fetchAllIndexes()
                        .done(function () { return _this.runQuery(); });
                    $("#matchingDocumentsGrid").resize();
                    break;
                default:
                    this.currentCollectionPagedItems(null);
                    break;
            }
        };
        patch.prototype.fetchAllCollections = function () {
            var _this = this;
            return new getCollectionsStatsCommand(this.activeDatabase())
                .execute()
                .done(function (stats) {
                var colls = stats.collections;
                var currentlySelectedCollection = null;
                if (_this.patchDocument().selectedItem()) {
                    var selected = _this.patchDocument().selectedItem();
                    currentlySelectedCollection = _this.collections().find(function (c) { return c.name === selected; });
                }
                _this.collections(colls);
                if (_this.collections().length > 0) {
                    _this.setSelectedCollection(currentlySelectedCollection || _this.collections()[0]);
                }
            });
        };
        patch.prototype.setSelectedCollection = function (coll) {
            this.resetProgressBar();
            this.patchDocument().selectedItem(coll.name);
            var list = coll.getDocuments();
            this.currentCollectionPagedItems(list);
            list.fetch(0, 20).always(function () { return $("#matchingDocumentsGrid").resize(); });
        };
        patch.prototype.fetchAllIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase())
                .execute()
                .done(function (results) {
                _this.indices(results.Indexes.map(function (i) {
                    return {
                        name: i.Name,
                        isMapReduce: false,
                    };
                }));
                if (_this.indices().length > 0) {
                    _this.setSelectedIndex(_this.indices()[0].name);
                }
            });
        };
        patch.prototype.setSelectedIndex = function (indexName) {
            this.resetProgressBar();
            this.selectedIndex(indexName);
            this.patchDocument().selectedItem(indexName);
        };
        patch.prototype.useIndex = function (indexName) {
            this.setSelectedIndex(indexName);
            this.runQuery();
        };
        patch.prototype.runQuery = function () {
            var selectedIndex = this.patchDocument().selectedItem();
            if (selectedIndex) {
                var queryText = this.queryText();
                this.patchDocument().query(queryText);
                var database = this.activeDatabase();
                var resultsFetcher = function (skip, take) {
                    var command = new queryIndexCommand(selectedIndex, database, skip, take, queryText, []);
                    return command.execute()
                        .fail(function () { return recentPatchesStorage.removeIndexFromRecentPatches(database, selectedIndex); });
                };
                var resultsList = new pagedList(resultsFetcher);
                this.currentCollectionPagedItems(resultsList);
                return resultsList;
            }
            return null;
        };
        patch.prototype.savePatch = function () {
            var _this = this;
            eventsCollector.default.reportEvent("patch", "save");
            var savePatchViewModel = new savePatch();
            app.showBootstrapDialog(savePatchViewModel);
            savePatchViewModel.onExit().done(function (patchName) {
                new savePatchCommand(patchName, _this.patchDocument(), _this.activeDatabase())
                    .execute()
                    .done(function () { return _this.fetchAllPatches(); });
            });
        };
        patch.prototype.usePatch = function (patch) {
            var _this = this;
            this.clearDocumentPreview();
            var selectedItem = patch.selectedItem();
            patch = patch.clone();
            patch.resetMetadata();
            this.patchDocument(patch);
            switch (this.patchDocument().patchOnOption()) {
                case "Collection":
                    this.fetchAllCollections().then(function () {
                        _this.setSelectedCollection(_this.collections().filter(function (coll) { return (coll.name === selectedItem); })[0]);
                    });
                    break;
                case "Index":
                    this.fetchAllIndexes().then(function () {
                        _this.setSelectedIndex(selectedItem);
                        _this.queryText(patch.query());
                        _this.runQuery();
                    });
                    break;
                case "Document":
                    this.loadDocumentToTest(patch.selectedItem());
                    break;
            }
        };
        patch.prototype.testPatch = function () {
            var _this = this;
            eventsCollector.default.reportEvent("patch", "test");
            var values = {};
            this.patchDocument().parameters().map(function (param) {
                var dto = param.toDto();
                values[dto.Key] = dto.Value;
            });
            var bulkDocs = [];
            bulkDocs.push({
                Key: this.keyOfTestedDocument(),
                Method: 'PATCH',
                DebugMode: true,
                Patch: {
                    Script: this.patchDocument().script(),
                    Values: values
                }
            });
            new executePatchCommand(bulkDocs, this.activeDatabase(), true)
                .execute()
                .done(function (result) {
                var testResult = new document(result.Results[0].AdditionalData['Document']);
                _this.afterPatchDoc(JSON.stringify(testResult.toDto(), null, 4));
                _this.afterPatchMeta(JSON.stringify(documentMetadata.filterMetadata(testResult.__metadata.toDto()), null, 4));
                _this.updateActions(result.Results[0].AdditionalData['Actions']);
                _this.outputLog(result.Results[0].AdditionalData["Debug"]);
            })
                .fail(function (result) { return console.log(result.responseText); });
            this.recordPatchRun();
        };
        patch.prototype.updatePageUrl = function (hash) {
            // Put the patch into the URL, so that if the user refreshes the page, he's still got this patch loaded.
            var queryUrl = appUrl.forPatch(this.activeDatabase(), hash);
            this.updateUrl(queryUrl);
        };
        patch.prototype.recordPatchRun = function () {
            var patchDocument = this.patchDocument();
            var newPatch = patchDocument.toDto();
            delete newPatch["@metadata"];
            newPatch.Hash = 0;
            var stringForHash = newPatch.PatchOnOption + newPatch.SelectedItem + newPatch.Script + newPatch.Values;
            if (patchDocument.patchOnOption() === "Index") {
                newPatch.Query = this.queryText();
                stringForHash += newPatch.Query;
            }
            newPatch.Hash = genUtils.hashCode(stringForHash);
            this.updatePageUrl(newPatch.Hash);
            // Add this query to our recent patches list in the UI, or move it to the top of the list if it's already there.
            var existing = this.recentPatches().find(function (q) { return q.Hash === newPatch.Hash; });
            if (existing) {
                this.recentPatches.remove(existing);
                this.recentPatches.unshift(existing);
            }
            else {
                this.recentPatches.unshift(newPatch);
            }
            // Limit us to 15 recent patchs
            if (this.recentPatches().length > 15) {
                this.recentPatches.remove(this.recentPatches()[15]);
            }
            //save the recent queries to local storage
            recentPatchesStorage.saveRecentPatches(this.activeDatabase(), this.recentPatches());
        };
        patch.prototype.useRecentPatch = function (patchToUse) {
            eventsCollector.default.reportEvent("patch", "use-recent");
            var patchDoc = new patchDocument(patchToUse);
            this.usePatch(patchDoc);
        };
        patch.prototype.updateActions = function (actions) {
            this.loadedDocuments(actions.LoadDocument || []);
            this.putDocuments((actions.PutDocument || []).map(function (doc) { return jsonUtil.syntaxHighlight(doc); }));
        };
        patch.prototype.executePatchOnSingle = function () {
            eventsCollector.default.reportEvent("patch", "run", "single");
            var keys = [];
            keys.push(this.patchDocument().selectedItem());
            this.confirmAndExecutePatch(keys);
        };
        patch.prototype.executePatchOnSelected = function () {
            eventsCollector.default.reportEvent("patch", "run", "selected");
            this.confirmAndExecutePatch(this.getDocumentsGrid().getSelectedItems().map(function (doc) { return doc.__metadata.id; }));
        };
        patch.prototype.executePatchOnAll = function () {
            var _this = this;
            eventsCollector.default.reportEvent("patch", "run", "all");
            var confirmExec = new executePatchConfirm();
            confirmExec.viewTask.done(function () {
                _this.executePatchAllCommand()
                    .done(function (result) { return _this.onPatchAllScheduled(result); });
                _this.recordPatchRun();
            });
            app.showBootstrapDialog(confirmExec);
        };
        patch.prototype.executePatchAllCommand = function () {
            this.patchSuccess(false);
            this.patchFailure(false);
            var values = {};
            this.patchDocument().parameters().map(function (param) {
                var dto = param.toDto();
                values[dto.Key] = dto.Value;
            });
            var patch = {
                Script: this.patchDocument().script(),
                Values: values
            };
            var patchOption = this.patchDocument().patchOnOption();
            if (patchOption === "Collection") {
                var collectionToPatch = this.patchDocument().selectedItem();
                return new evalByCollectionCommand(collectionToPatch, patch, this.activeDatabase()).execute();
            }
            else if (patchOption === "Index") {
                var index = this.patchDocument().selectedItem();
                var query = this.patchDocument().query();
                return new evalByQueryCommand(index, query, patch, this.activeDatabase()).execute();
            }
            else {
                throw new Error("Unhandled patch option: " + patchOption);
            }
        };
        patch.prototype.onPatchAllScheduled = function (result) {
            var _this = this;
            this.resetProgressBar();
            this.isPatchingInProgress(true);
            this.showPatchingProgress(true);
            // TODO: this.fetchRunningPatches();
            notificationCenter.instance.monitorOperation(this.activeDatabase(), result.OperationId)
                .done(function () {
                _this.patchSuccess(true);
                _this.patchKillInProgress(false);
                _this.isPatchingInProgress(false);
            });
        };
        patch.prototype.resetProgressBar = function () {
            this.showPatchingProgress(false);
            this.patchingProgress(0);
            this.patchingProgressText("");
        };
        patch.prototype.updateProgress = function (status) {
            if (status.OperationProgress != null) {
                var progressValue = Math.round(100 * (status.OperationProgress.ProcessedEntries / status.OperationProgress.TotalEntries));
                this.patchingProgress(progressValue);
                var progressPrefix = "";
                if (status.Completed) {
                    if (status.Canceled) {
                        progressPrefix = "Patch canceled: ";
                        this.patchFailure(true);
                    }
                    else if (status.Faulted) {
                        progressPrefix = "Patch failed: ";
                        this.patchFailure(true);
                    }
                    else {
                        progressPrefix = "Patch completed: ";
                        this.patchSuccess(true);
                    }
                }
                if (status.OperationProgress.TotalEntries) {
                    this.patchingProgressText(progressPrefix + status.OperationProgress.ProcessedEntries.toLocaleString() + " / " + status.OperationProgress.TotalEntries.toLocaleString() + " (" + progressValue + "%)");
                }
            }
            if (status.Completed) {
                this.patchKillInProgress(false);
                this.isPatchingInProgress(false);
            }
        };
        patch.prototype.confirmAndExecutePatch = function (keys) {
            var _this = this;
            var confirmExec = new executePatchConfirm();
            confirmExec.viewTask.done(function () { return _this.executePatch(keys); });
            app.showBootstrapDialog(confirmExec);
        };
        patch.prototype.executePatch = function (keys) {
            var _this = this;
            var values = {};
            this.patchDocument().parameters().map(function (param) {
                var dto = param.toDto();
                values[dto.Key] = dto.Value;
            });
            var bulkDocs = [];
            keys.forEach(function (key) { return bulkDocs.push({
                Key: key,
                Method: 'PATCH',
                DebugMode: false,
                Patch: {
                    Script: _this.patchDocument().script(),
                    Values: values
                }
            }); });
            new executePatchCommand(bulkDocs, this.activeDatabase(), false)
                .execute()
                .done(function (result) {
                _this.afterPatchDoc("");
                _this.afterPatchMeta("");
                if (_this.patchDocument().patchOnOption() === 'Document') {
                    _this.loadDocumentToTest(_this.patchDocument().selectedItem());
                }
                _this.updateDocumentsList();
            })
                .fail(function (result) { return console.log(result.responseText); });
            this.recordPatchRun();
        };
        patch.prototype.updateDocumentsList = function () {
            var _this = this;
            switch (this.patchDocument().patchOnOption()) {
                case "Collection":
                    this.fetchAllCollections().then(function () {
                        _this.setSelectedCollection(_this.collections().filter(function (coll) { return (coll.name === _this.patchDocument().selectedItem()); })[0]);
                    });
                    break;
                case "Index":
                    this.useIndex(this.patchDocument().selectedItem());
                    break;
            }
        };
        patch.prototype.getDocumentsGrid = function () {
            var gridContents = $(patch.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        patch.prototype.activateBeforeDoc = function () {
            this.beforePatchDocMode(true);
        };
        patch.prototype.activateBeforeMeta = function () {
            this.beforePatchDocMode(false);
        };
        patch.prototype.activateAfterDoc = function () {
            this.afterPatchDocMode(true);
        };
        patch.prototype.activateAfterMeta = function () {
            this.afterPatchDocMode(false);
        };
        patch.prototype.queryCompleter = function (editor, session, pos, prefix, callback) {
            queryUtil.queryCompleter(this.indexFields, this.selectedIndex, this.dynamicPrefix, this.activeDatabase, editor, session, pos, prefix, callback);
        };
        patch.prototype.killPatch = function () {
            var _this = this;
            var operationToKill = this.patchOperationId();
            if (operationToKill) {
                this.confirmationMessage("Are you sure?", "You are stopping patch execution.")
                    .done(function () {
                    if (_this.patchOperationId()) {
                        new killOperationComamnd(_this.activeDatabase(), operationToKill)
                            .execute()
                            .done(function () {
                            if (_this.patchOperationId()) {
                                _this.patchKillInProgress(true);
                            }
                        });
                        eventsCollector.default.reportEvent("patch", "kill");
                    }
                });
            }
        };
        patch.prototype.fetchIndexFields = function (indexName) {
            var _this = this;
            // Fetch the index definition so that we get an updated list of fields to be used as sort by options.
            // Fields don't show for All Documents.
            var self = this;
            var isAllDocumentsDynamicQuery = indexName === "All Documents";
            if (!isAllDocumentsDynamicQuery) {
                //if index is dynamic, get columns using index definition, else get it using first index result
                if (indexName.indexOf(this.dynamicPrefix) === 0) {
                    var collectionName = indexName.substring(8);
                    new collection(collectionName, this.activeDatabase())
                        .fetchDocuments(0, 1)
                        .done(function (result) {
                        if (!!result && result.totalResultCount > 0 && result.items.length > 0) {
                            var dynamicIndexPattern = new document(result.items[0]);
                            if (!!dynamicIndexPattern) {
                                _this.indexFields(dynamicIndexPattern.getDocumentPropertyNames());
                            }
                        }
                    });
                }
                else {
                    new getIndexDefinitionCommand(indexName, this.activeDatabase())
                        .execute()
                        .done(function (result) {
                        self.isTestIndex(result.IsTestIndex);
                        self.indexFields(Object.keys(result.Fields));
                    })
                        .fail(function () {
                        recentPatchesStorage.removeIndexFromRecentPatches(_this.activeDatabase(), indexName);
                    });
                }
            }
        };
        return patch;
    }(viewModelBase));
    patch.gridSelector = "#matchingDocumentsGrid";
    return patch;
});

//# sourceMappingURL=patch.js.map
