var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "commands/resources/getDatabaseStatsCommand", "commands/database/index/getIndexDefinitionCommand", "models/database/query/facet", "commands/database/query/queryFacetsCommand", "common/bindingHelpers/aceEditorBindingHandler", "common/pagedList", "common/eventsCollector"], function (require, exports, viewModelBase, appUrl, getDatabaseStatsCommand, getIndexDefinitionCommand, facet, queryFacetsCommand, aceEditorBindingHandler, pagedList, eventsCollector) {
    "use strict";
    var reporting = (function (_super) {
        __extends(reporting, _super);
        function reporting() {
            var _this = _super.call(this) || this;
            _this.selectedIndexName = ko.observable();
            _this.selectedIndexLabel = ko.computed(function () { return _this.selectedIndexName() ? _this.selectedIndexName() : "[Select an index]"; });
            _this.indexNames = ko.observableArray();
            _this.hasSelectedIndex = ko.computed(function () { return _this.selectedIndexName() && _this.selectedIndexName().length > 0; });
            _this.editSelectedIndexUrl = ko.computed(function () { return _this.hasSelectedIndex() ? appUrl.forEditIndex(_this.selectedIndexName(), _this.activeDatabase()) : null; });
            _this.availableFields = ko.observableArray();
            _this.sortOptions = ko.observableArray();
            _this.selectedField = ko.observable();
            _this.selectedFieldLabel = ko.computed(function () { return _this.selectedField() ? _this.selectedField() : "Select a field"; });
            _this.addedValues = ko.observableArray();
            _this.filter = ko.observable();
            _this.hasFilter = ko.observable(false);
            _this.reportResults = ko.observable();
            _this.totalQueryResults = ko.computed(function () { return _this.reportResults() ? _this.reportResults().totalResultCount() : null; });
            _this.queryDuration = ko.observable();
            _this.isCacheDisable = ko.observable(false);
            _this.isExportEnabled = ko.computed(function () { return _this.reportResults() ? _this.reportResults().totalResultCount() > 0 : false; });
            _this.appUrls = appUrl.forCurrentDatabase();
            return _this;
        }
        reporting.prototype.exportCsv = function () {
            eventsCollector.default.reportEvent("reporting", "export-csv");
            if (this.isExportEnabled() === false)
                return false;
            var objArray = JSON.stringify(this.reportResults().getAllCachedItems());
            var array = typeof objArray != 'object' ? JSON.parse(objArray) : objArray;
            if (array[0] === undefined)
                return false;
            var str = '';
            var line = '';
            for (var header in array[0]) {
                if (header === "__metadata")
                    continue;
                if (line)
                    line += ',';
                line += header;
            }
            str += line + '\r\n';
            for (var i = 0; i < array.length; i++) {
                line = '';
                for (var index in array[i]) {
                    if (index === "__metadata")
                        continue;
                    if (line)
                        line += ',';
                    line += array[i][index];
                }
                str += line + '\r\n';
            }
            var uriContent = encodeURIComponent(str);
            var link = document.createElement('a');
            link["download"] = this.selectedIndexName() ? "Reporting_" + this.selectedIndexName() + ".csv" : "reporting.csv";
            link.href = 'data:,' + uriContent;
            link.click();
            return true;
        };
        reporting.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#filterQueryLabel").popover({
                html: true,
                trigger: "hover",
                container: ".form-horizontal",
                content: '<p>Queries use Lucene syntax. Examples:</p><pre><span class="code-keyword">Name</span>: Hi?berna*<br/><span class="code-keyword">Count</span>: [0 TO 10]<br/><span class="code-keyword">Title</span>: "RavenDb Queries 1010" AND <span class="code-keyword">Price</span>: [10.99 TO *]</pre>',
            });
        };
        reporting.prototype.activate = function (indexToActivateOrNull) {
            var _this = this;
            _super.prototype.activate.call(this, indexToActivateOrNull);
            this.updateHelpLink('O3EA1R');
            this.fetchIndexes().done(function () { return _this.selectInitialIndex(indexToActivateOrNull); });
            this.selectedIndexName.subscribe(function () { return _this.resetSelections(); });
            aceEditorBindingHandler.install();
        };
        reporting.prototype.fetchIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase())
                .execute()
                .done(function (results) { return _this.indexNames(results.Indexes.map(function (i) { return i.Name; })); });
        };
        reporting.prototype.fetchIndexDefinition = function (indexName) {
            new getIndexDefinitionCommand(indexName, this.activeDatabase())
                .execute()
                .done(function (dto) {
                /* TODO this.sortOptions(dto.Index.SortOptions);
                this.availableFields(dto.Index.Fields);*/
            });
        };
        reporting.prototype.selectInitialIndex = function (indexToActivateOrNull) {
            if (indexToActivateOrNull && _.includes(this.indexNames(), indexToActivateOrNull)) {
                this.setSelectedIndex(indexToActivateOrNull);
            }
            else if (this.indexNames().length > 0) {
                this.setSelectedIndex(_.first(this.indexNames()));
            }
        };
        reporting.prototype.setSelectedIndex = function (indexName) {
            this.selectedIndexName(indexName);
            this.updateUrl(appUrl.forReporting(this.activeDatabase(), indexName));
            this.fetchIndexDefinition(indexName);
        };
        reporting.prototype.setSelectedField = function (fieldName) {
            this.selectedField(fieldName);
            // Update all facets to use that too.
            this.addedValues().forEach(function (v) { return v.name = fieldName; });
        };
        reporting.prototype.resetSelections = function () {
            this.selectedField(null);
            this.addedValues([]);
            this.availableFields([]);
            if (!!this.reportResults()) {
                this.reportResults(null);
            }
        };
        reporting.prototype.mapSortToType = function (sort) {
            switch (sort) {
                case 'Int':
                    return "System.Int32";
                case 'Float':
                    return "System.Single";
                case 'Long':
                    return 'System.Int64';
                case 'Double':
                    return "System.Double";
                case 'Short':
                    return 'System.Int16';
                case 'Byte':
                    return 'System.Byte';
                default:
                    return 'System.String';
            }
        };
        reporting.prototype.addValue = function (fieldName) {
            eventsCollector.default.reportEvent("reporting", "add-value");
            var sortOps = this.sortOptions();
            var sortOption = (fieldName in sortOps) ? sortOps[fieldName] : "String";
            var val = facet.fromNameAndAggregation(this.selectedField(), fieldName, this.mapSortToType(sortOption));
            this.addedValues.push(val);
        };
        reporting.prototype.removeValue = function (val) {
            eventsCollector.default.reportEvent("reporting", "remove-value");
            this.addedValues.remove(val);
        };
        reporting.prototype.runReport = function () {
            var _this = this;
            eventsCollector.default.reportEvent("reporting", "run");
            var selectedIndex = this.selectedIndexName();
            var filterQuery = this.hasFilter() ? this.filter() : null;
            var facets = this.addedValues().map(function (v) { return v.toDto(); });
            var groupedFacets = [];
            facets.forEach(function (curFacet) {
                var foundFacet = groupedFacets.find(function (x) { return x.AggregationField == curFacet.AggregationField; });
                if (foundFacet) {
                    foundFacet.Aggregation += curFacet.Aggregation;
                }
                else {
                    groupedFacets.push(curFacet);
                }
            });
            var db = this.activeDatabase();
            var resultsFetcher = function (skip, take) {
                var command = new queryFacetsCommand(selectedIndex, filterQuery, skip, take, groupedFacets, db, _this.isCacheDisable());
                ko.postbox.publish("SetRawJSONUrl", appUrl.forReportingRawData(_this.activeDatabase(), _this.selectedIndexName()) + command.argsUrl);
                return command.execute()
                    .done(function (resultSet) { return _this.queryDuration(resultSet.additionalResultInfo); });
            };
            this.reportResults(new pagedList(resultsFetcher));
        };
        reporting.prototype.toggleCacheEnable = function () {
            eventsCollector.default.reportEvent("reporting", "toggle-cache");
            this.isCacheDisable(!this.isCacheDisable());
        };
        return reporting;
    }(viewModelBase));
    return reporting;
});

//# sourceMappingURL=reporting.js.map
