var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/bindingHelpers/aceEditorBindingHandler", "viewmodels/viewModelBase", "commands/database/documents/getCustomFunctionsCommand", "commands/database/documents/saveCustomFunctionsCommand", "commands/database/globalConfig/getEffectiveCustomFunctionsCommand", "models/database/documents/customFunctions", "common/jsonUtil", "common/messagePublisher", "common/eventsCollector"], function (require, exports, aceEditorBindingHandler, viewModelBase, getCustomFunctionsCommand, saveCustomFunctionsCommand, getEffectiveCustomFunctionsCommand, customFunctions, jsonUtil, messagePublisher, eventsCollector) {
    "use strict";
    var customFunctionsEditor = (function (_super) {
        __extends(customFunctionsEditor, _super);
        function customFunctionsEditor() {
            var _this = _super.call(this) || this;
            _this.hasGlobal = ko.observable(false);
            aceEditorBindingHandler.install();
            _this.documentText = ko.observable("");
            _this.globalDocumentText = ko.observable("");
            _this.dirtyFlag = new ko.DirtyFlag([_this.documentText], false, jsonUtil.newLineNormalizingHashFunction);
            _this.isSaveEnabled = ko.computed(function () {
                return _this.dirtyFlag().isDirty();
            });
            return _this;
        }
        customFunctionsEditor.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('XLDBRW');
        };
        customFunctionsEditor.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#customFunctionsExample").popover({
                html: true,
                trigger: "hover",
                content: "<p>Examples:</p><pre>exports.greet = <span class=\"code-keyword\">function</span>(name) {<br/>    <span class=\"code-keyword\">return</span> <span class=\"code-string\">\"Hello \" + name + \"!\"</span>;<br/>}</pre>"
            });
        };
        customFunctionsEditor.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#customFunctionsEditor.editor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#customFunctionsEditor").on("DynamicHeightSet", function () { return _this.docEditor.resize(); });
            this.fetchCustomFunctions();
        };
        customFunctionsEditor.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#customFunctionsEditor").off("DynamicHeightSet");
        };
        customFunctionsEditor.prototype.fetchCustomFunctions = function () {
            var _this = this;
            var fetchTask = new getCustomFunctionsCommand(this.activeDatabase()).execute();
            fetchTask.done(function (cf) {
                _this.documentText(cf.functions);
                _this.dirtyFlag().reset();
            });
            var globalFetchTask = new getEffectiveCustomFunctionsCommand(this.activeDatabase()).execute();
            globalFetchTask.done(function (result) {
                _this.hasGlobal(result.GlobalExists);
                if (result.GlobalExists) {
                    _this.globalDocumentText(result.GlobalDocument.Functions);
                }
            });
        };
        customFunctionsEditor.prototype.saveChanges = function () {
            var _this = this;
            eventsCollector.default.reportEvent("custom-functions", "save");
            var annotations = this.docEditor.getSession().getAnnotations();
            var hasErrors = false;
            for (var i = 0; i < annotations.length; i++) {
                if (annotations[i].type === "error") {
                    hasErrors = true;
                    break;
                }
            }
            if (!hasErrors) {
                var cf = new customFunctions({
                    Functions: this.documentText()
                });
                var saveTask = new saveCustomFunctionsCommand(this.activeDatabase(), cf).execute();
                saveTask.done(function () { return _this.dirtyFlag().reset(); });
            }
            else {
                messagePublisher.reportError("Errors in the functions file", "Please correct the errors in the file in order to save it.");
            }
        };
        return customFunctionsEditor;
    }(viewModelBase));
    return customFunctionsEditor;
});

//# sourceMappingURL=customFunctionsEditor.js.map
