var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "common/appUrl", "common/bindingHelpers/aceEditorBindingHandler", "commands/resources/getDatabaseSettingsCommand", "commands/resources/saveDatabaseSettingsCommand", "models/database/documents/document", "common/jsonUtil", "viewmodels/viewModelBase", "viewmodels/common/viewSystemDatabaseConfirm", "common/messagePublisher", "viewmodels/shell/accessHelper", "common/eventsCollector"], function (require, exports, app, appUrl, aceEditorBindingHandler, getDatabaseSettingsCommand, saveDatabaseSettingsCommand, document, jsonUtil, viewModelBase, viewSystemDatabaseConfirm, messagePublisher, accessHelper, eventsCollector) {
    "use strict";
    var databaseSettings = (function (_super) {
        __extends(databaseSettings, _super);
        function databaseSettings() {
            var _this = _super.call(this) || this;
            _this.document = ko.observable();
            _this.documentText = ko.observable().extend({ required: true });
            _this.metadataText = ko.observable("{}").extend({ required: true });
            _this.isEditingEnabled = ko.observable(false);
            _this.isEditingMetadata = ko.observable(false);
            _this.isForbidden = ko.observable(false);
            aceEditorBindingHandler.install();
            _this.document.subscribe(function (doc) {
                if (doc) {
                    var docDto = doc.toDto();
                    _this.securedSettings = ko.toJSON(docDto.SecuredSettings);
                    var docText = _this.stringify(doc.toDto());
                    _this.documentText(docText);
                }
            });
            _this.isEditingMetadata.subscribe(function () {
                if (_this.docEditor) {
                    var text = _this.isEditingMetadata() ? _this.metadataText() : _this.documentText();
                    _this.docEditor.getSession().setValue(text);
                }
            });
            _this.text = ko.computed({
                read: function () {
                    return _this.isEditingMetadata() ? _this.metadataText() : _this.documentText();
                },
                write: function (text) {
                    var currentObservable = _this.isEditingMetadata() ? _this.metadataText : _this.documentText;
                    currentObservable(text);
                },
                owner: _this
            });
            return _this;
        }
        databaseSettings.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            this.isForbidden(!accessHelper.isGlobalAdmin());
            if (this.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                var db = this.activeDatabase();
                this.fetchDatabaseSettings(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function (response) {
                    messagePublisher.reportError("Error fetching database document!", response.responseText, response.statusText);
                    deferred.resolve({ redirect: appUrl.forStatus(db) });
                });
            }
            return deferred;
        };
        databaseSettings.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('3QMLGH');
            this.dirtyFlag = new ko.DirtyFlag([this.documentText, this.metadataText], false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        databaseSettings.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#dbDocEditor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#dbDocEditor").on('DynamicHeightSet', function () { return _this.docEditor.resize(); });
        };
        databaseSettings.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#dbDocEditor").off('DynamicHeightSet');
        };
        databaseSettings.prototype.editDatabaseSettings = function () {
            var _this = this;
            eventsCollector.default.reportEvent("database-settings", "edit");
            var editDbConfirm = new viewSystemDatabaseConfirm("Meddling with the database settings document could cause irreversible damage!");
            editDbConfirm
                .viewTask
                .done(function () {
                _this.isEditingEnabled(true);
            });
            app.showBootstrapDialog(editDbConfirm);
        };
        databaseSettings.prototype.refreshFromServer = function () {
            var _this = this;
            eventsCollector.default.reportEvent("database-settings", "refresh");
            var canContinue = this.canContinueIfNotDirty('Unsaved Data', 'You have unsaved data. Are you sure you want to refresh the data from the server?');
            canContinue.done(function () {
                _this.fetchDatabaseSettings(_this.activeDatabase(), true)
                    .done(function () {
                    _this.metadataText("{}");
                    _this.isEditingEnabled(false);
                    _this.activateDoc();
                    _this.dirtyFlag().reset(); //Resync Changes
                });
            });
        };
        databaseSettings.prototype.formatDocument = function () {
            eventsCollector.default.reportEvent("database-settings", "format");
            var text = this.isEditingMetadata() ? this.metadataText() : this.documentText();
            var observableToUpdate = this.isEditingMetadata() ? this.metadataText : this.documentText;
            try {
                var tempDoc = JSON.parse(text);
                var formatted = this.stringify(tempDoc);
                observableToUpdate(formatted);
            }
            catch (e) { }
        };
        databaseSettings.prototype.activateMeta = function () {
            this.isEditingMetadata(true);
        };
        databaseSettings.prototype.activateDoc = function () {
            this.isEditingMetadata(false);
        };
        databaseSettings.prototype.saveChanges = function () {
            var _this = this;
            var updatedDto;
            var meta;
            eventsCollector.default.reportEvent("database-settings", "save");
            var editDbConfirm = new viewSystemDatabaseConfirm("Meddling with the database settings document could cause irreversible damage!");
            editDbConfirm
                .viewTask
                .done(function () {
                try {
                    updatedDto = JSON.parse(_this.documentText());
                    meta = JSON.parse(_this.metadataText());
                    var newDoc = new document(updatedDto);
                    newDoc.__metadata.etag = _this.document().__metadata.etag;
                    var saveCommand = new saveDatabaseSettingsCommand(_this.activeDatabase(), newDoc);
                    var saveTask = saveCommand.execute();
                    saveTask.done(function (saveResult) {
                        _this.document().__metadata.etag = saveResult.ETag;
                        _this.metadataText("{}");
                        _this.isEditingEnabled(false);
                        _this.formatDocument();
                        _this.dirtyFlag().reset(); //Resync Changes
                    });
                }
                catch (e) {
                    var message = "";
                    if (updatedDto == undefined) {
                        message = "The data isn't a legal JSON expression!";
                        _this.isEditingMetadata(false);
                    }
                    else if (meta == undefined) {
                        message = "The metadata isn't a legal JSON expression!";
                        _this.isEditingMetadata(true);
                    }
                    _this.docEditor.focus();
                    messagePublisher.reportError(message);
                }
            });
            app.showBootstrapDialog(editDbConfirm);
        };
        databaseSettings.prototype.fetchDatabaseSettings = function (db, reportFetchProgress) {
            var _this = this;
            if (reportFetchProgress === void 0) { reportFetchProgress = false; }
            return new getDatabaseSettingsCommand(db, reportFetchProgress)
                .execute()
                .done(function (document) { return _this.document(document); });
        };
        databaseSettings.prototype.getDatabaseSettingsDocumentId = function (db) {
            return "Raven/Databases/" + db.name;
        };
        databaseSettings.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };
        return databaseSettings;
    }(viewModelBase));
    databaseSettings.containerId = "#databaseSettingsContainer";
    return databaseSettings;
});

//# sourceMappingURL=databaseSettings.js.map
