var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/router", "viewmodels/viewModelBase", "common/appUrl", "common/jsonUtil", "common/bindingHelpers/aceEditorBindingHandler", "common/messagePublisher", "durandal/app", "models/database/sqlReplication/sqlReplication", "commands/database/documents/getCollectionsStatsCommand", "viewmodels/database/status/sqlReplicationStatsDialog", "models/database/documents/document", "commands/database/documents/saveDocumentCommand", "viewmodels/common/deleteDocuments", "commands/database/documents/getDocumentWithMetadataCommand", "commands/database/documents/getDocumentsMetadataByIDPrefixCommand", "models/database/documents/documentMetadata", "commands/database/sqlReplication/resetSqlReplicationCommand", "viewmodels/database/status/sqlReplicationSimulationDialog", "models/database/sqlReplication/sqlReplicationConnections", "commands/database/sqlReplication/getSqlReplicationConnectionsCommand", "common/eventsCollector"], function (require, exports, router, viewModelBase, appUrl, jsonUtil, aceEditorBindingHandler, messagePublisher, app, sqlReplication, getCollectionsStatsCommand, sqlReplicationStatsDialog, document, saveDocumentCommand, deleteDocuments, getDocumentWithMetadataCommand, getDocumentsMetadataByIDPrefixCommand, documentMetadata, resetSqlReplicationCommand, sqlReplicationSimulationDialog, sqlReplicationConnections, getSqlReplicationConnectionsCommand, eventsCollector) {
    "use strict";
    var editSqlReplication = (function (_super) {
        __extends(editSqlReplication, _super);
        function editSqlReplication() {
            var _this = _super.call(this) || this;
            _this.editedReplication = ko.observable();
            _this.collections = ko.observableArray();
            _this.loadedSqlReplications = [];
            _this.isEditingNewReplication = ko.observable(false);
            _this.isBasicView = ko.observable(true);
            _this.availableConnectionStrings = ko.observableArray();
            _this.sqlReplicationStatsAndMetricsHref = appUrl.forCurrentDatabase().statusDebugSqlReplication;
            _this.script = ko.computed({
                read: function () {
                    var r = _this.editedReplication();
                    return r ? r.script() : "";
                },
                write: function (v) { return _this.editedReplication().script(v); }
            });
            _this.simulationDocumentId = ko.observable();
            _this.isBusy = ko.observable(false);
            _this.initialReplicationId = '';
            aceEditorBindingHandler.install();
            _this.appUrls = appUrl.forCurrentDatabase();
            _this.sqlReplicationName = ko.computed(function () { return (!!_this.editedReplication() && !_this.isEditingNewReplication()) ? _this.editedReplication().name() : null; });
            return _this;
        }
        editSqlReplication.prototype.toggleBasicMode = function () {
            this.isBasicView(!this.isBasicView());
        };
        editSqlReplication.prototype.addScriptLabelPopover = function () {
            var popOverSettings = {
                html: true,
                trigger: 'hover',
                content: 'Replication scripts use JavaScript.<br/><br/>The script will be called once for each document in the source document collection, with <span class="code-keyword">this</span> representing the document, and the document id available as <i>documentId</i>.<br/><br/>Call <i>replicateToTableName</i> for each row you want to write to the database.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> orderData = {<br/>   Id: documentId,<br/>   OrderLinesCount: <span class="code-keyword">this</span>.Lines.length,<br/>   TotalCost: 0<br/>};<br/><br/>for (<span class="code-keyword">var</span> i = 0; i &lt; <span class="code-keyword">this</span>.Lines.length; i++) {<br/>   <span class="code-keyword">var</span> line = <span class="code-keyword">this</span>.Lines[i];<br/>   <span class="code-keyword">var</span> lineCost = ((line.Quantity * line.PricePerUnit) <br />                     * (1 - line.Discount));<br/>   orderData.TotalCost += lineCost;<br/><br/>   replicateToOrderLines({<br/>      OrderId: documentId,<br/>      Qty: line.Quantity,<br/>      Product: line.Product,<br/>      Cost: lineCost<br/>   });<br/>}<br/><br/>replicateToOrders(orderData);</pre>',
                selector: '.script-label',
                placement: "right"
            };
            $('body').popover(popOverSettings);
            $('form :input[name="ravenEntityName"]').on("keypress", function (e) { return e.which != 13; });
        };
        editSqlReplication.prototype.loadSqlReplicationConnections = function () {
            var _this = this;
            return new getSqlReplicationConnectionsCommand(this.activeDatabase())
                .execute()
                .done(function (dto) {
                var connections = new sqlReplicationConnections(dto);
                if (connections.predefinedConnections().length > 0) {
                    connections.predefinedConnections().forEach(function (x) { return _this.availableConnectionStrings.push(x); });
                }
            });
        };
        editSqlReplication.prototype.canActivate = function (replicationToEditName) {
            var _this = this;
            var canActivateResult = $.Deferred();
            this.loadSqlReplicationConnections().always(function () {
                if (replicationToEditName) {
                    _this.loadSqlReplication(replicationToEditName)
                        .done(function () { return canActivateResult.resolve({ can: true }); })
                        .fail(function () {
                        messagePublisher.reportError("Could not find " + decodeURIComponent(replicationToEditName) + " replication");
                        canActivateResult.resolve({ redirect: appUrl.forSqlReplications(_this.activeDatabase()) });
                    });
                }
                else {
                    _this.isEditingNewReplication(true);
                    _this.editedReplication(_this.createSqlReplication());
                    _this.fetchCollections(_this.activeDatabase()).always(function () { return canActivateResult.resolve({ can: true }); });
                }
            });
            return canActivateResult;
        };
        editSqlReplication.prototype.activate = function (replicationToEditName) {
            var _this = this;
            _super.prototype.activate.call(this, replicationToEditName);
            this.dirtyFlag = new ko.DirtyFlag([this.editedReplication], false, jsonUtil.newLineNormalizingHashFunctionWithIgnoredFields(["__metadata", "metadata"]));
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        editSqlReplication.prototype.providerChanged = function (obj, event) {
            if (event.originalEvent && obj.connectionStringType() == obj.CONNECTION_STRING) {
                var curConnectionString = !!this.editedReplication().connectionStringValue() ? this.editedReplication().connectionStringValue().trim() : "";
                if (curConnectionString === "" ||
                    sqlReplicationConnections.sqlProvidersConnectionStrings.find(function (x) { return x.ConnectionString == curConnectionString; })) {
                    var matchingConnectionStringPair = sqlReplicationConnections.sqlProvidersConnectionStrings.find(function (x) { return x.ProviderName == event.originalEvent.srcElement.selectedOptions[0].value; });
                    if (!!matchingConnectionStringPair) {
                        var matchingConnectionStringValue = matchingConnectionStringPair.ConnectionString;
                        this.editedReplication().connectionStringValue(matchingConnectionStringValue);
                    }
                }
            }
        };
        editSqlReplication.prototype.loadSqlReplication = function (replicationToLoadName) {
            var _this = this;
            var loadDeferred = $.Deferred();
            $.when(this.fetchSqlReplicationToEdit(replicationToLoadName), this.fetchCollections(this.activeDatabase()))
                .done(function () {
                _this.editedReplication().collections = _this.collections;
                new getDocumentsMetadataByIDPrefixCommand(editSqlReplication.sqlReplicationDocumentPrefix, 256, _this.activeDatabase())
                    .execute()
                    .done(function (results) {
                    _this.loadedSqlReplications = results.Results;
                    loadDeferred.resolve();
                }).
                    fail(function () { return loadDeferred.reject(); });
            })
                .fail(function () {
                loadDeferred.reject();
            });
            return loadDeferred;
        };
        editSqlReplication.prototype.fetchSqlReplicationToEdit = function (sqlReplicationName) {
            var _this = this;
            var loadDocTask = new getDocumentWithMetadataCommand(editSqlReplication.sqlReplicationDocumentPrefix + sqlReplicationName, this.activeDatabase()).execute();
            loadDocTask.done(function (document) {
                var sqlReplicationDto = document.toDto(true);
                _this.editedReplication(new sqlReplication(sqlReplicationDto));
                _this.initialReplicationId = _this.editedReplication().name();
                _this.dirtyFlag().reset(); //Resync Changes
            });
            loadDocTask.always(function () { return _this.isBusy(false); });
            this.isBusy(true);
            return loadDocTask;
        };
        editSqlReplication.prototype.fetchCollections = function (db) {
            var _this = this;
            return new getCollectionsStatsCommand(db)
                .execute()
                .done(function (collectionsStats) {
                _this.collections(collectionsStats.collections.map(function (x) { return x.name; }));
            });
        };
        editSqlReplication.prototype.showStats = function () {
            eventsCollector.default.reportEvent("edit-sql-replication", "stats");
            var viewModel = new sqlReplicationStatsDialog(this.activeDatabase(), this.editedReplication().name());
            app.showBootstrapDialog(viewModel);
        };
        editSqlReplication.prototype.refreshSqlReplication = function () {
            eventsCollector.default.reportEvent("edit-sql-replicaton", "refresh");
            if (this.isEditingNewReplication() === false) {
                var docId = this.initialReplicationId;
                this.loadSqlReplication(docId);
            }
            else {
                this.editedReplication(this.createSqlReplication());
            }
        };
        editSqlReplication.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.addScriptLabelPopover();
            $('pre').each(function (index, currentPreElement) {
                _this.initializeAceValidity(currentPreElement);
            });
            var editorElement = $("#sqlReplicationEditor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#sqlReplicationEditor").on('DynamicHeightSet', function () { return _this.docEditor.resize(); });
        };
        editSqlReplication.prototype.createSqlReplication = function () {
            eventsCollector.default.reportEvent("edit-sql-replication", "create");
            var newSqlReplication = sqlReplication.empty();
            newSqlReplication.collections(this.collections());
            this.collections.subscribe(function (value) { return newSqlReplication.collections(value); });
            this.subscribeToSqlReplicationName(newSqlReplication);
            return newSqlReplication;
        };
        editSqlReplication.prototype.subscribeToSqlReplicationName = function (sqlReplicationElement) {
            var _this = this;
            sqlReplicationElement.name.subscribe(function (previousName) {
                //Get the previous value of 'name' here before it's set to newValue
                var nameInputArray = $('input[name="name"]').filter(function () { return this.value === previousName; });
                if (nameInputArray.length === 1) {
                    var inputField = nameInputArray[0];
                    inputField.setCustomValidity("");
                }
            }, this, "beforeChange");
            sqlReplicationElement.name.subscribe(function (newName) {
                var message = "";
                if (newName === "") {
                    message = "Please fill out this field.";
                }
                else if (_this.isSqlReplicationNameExists(newName)) {
                    message = "SQL Replication name already exists.";
                }
                $('input[name="name"]')
                    .filter(function () { return this.value === newName; })
                    .each(function (index, element) {
                    element.setCustomValidity(message);
                });
            });
        };
        editSqlReplication.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#sqlReplicationEditor").off('DynamicHeightSet');
        };
        editSqlReplication.prototype.isSqlReplicationNameExists = function (name) {
            return !!this.loadedSqlReplications.find(function (x) { return x === name; });
        };
        editSqlReplication.prototype.initializeAceValidity = function (element) {
            var editorElement = $("#aceEditor");
            if (editorElement.length > 0) {
                var editor = ko.utils.domData.get(editorElement[0], "aceEditor");
                var editorValue = editor.getSession().getValue();
                if (editorValue === "") {
                    var textarea = $(element).find('textarea')[0];
                    textarea.setCustomValidity("Please fill out this field.");
                }
            }
        };
        editSqlReplication.prototype.save = function () {
            var _this = this;
            eventsCollector.default.reportEvent("edit-sql-replication", "save");
            var currentDocumentId = this.editedReplication().name();
            this.editedReplication().script(this.script());
            if (this.initialReplicationId !== currentDocumentId) {
                delete this.editedReplication().__metadata.etag();
                delete this.editedReplication().__metadata.lastModified; //TODO last -modified is not longer used? use Raven-Last-Modified
            }
            var newDoc = new document(this.editedReplication().toDto());
            newDoc.__metadata = new documentMetadata();
            this.attachReservedMetaProperties(editSqlReplication.sqlReplicationDocumentPrefix + currentDocumentId, newDoc.__metadata);
            var saveCommand = new saveDocumentCommand(editSqlReplication.sqlReplicationDocumentPrefix + currentDocumentId, newDoc, this.activeDatabase());
            var saveTask = saveCommand.execute();
            saveTask.done(function (saveResult) {
                var savedDocumentDto = saveResult.Results[0];
                var sqlReplicationKey = savedDocumentDto.Key.substring(editSqlReplication.sqlReplicationDocumentPrefix.length);
                _this.loadSqlReplication(sqlReplicationKey)
                    .done(function () { return _this.dirtyFlag().reset(); });
                _this.updateUrl(sqlReplicationKey);
                _this.isEditingNewReplication(false);
                _this.initialReplicationId = currentDocumentId;
            });
        };
        editSqlReplication.prototype.updateUrl = function (docId) {
            var url = appUrl.forEditSqlReplication(docId, this.activeDatabase());
            router.navigate(url, false);
        };
        editSqlReplication.prototype.attachReservedMetaProperties = function (id, target) {
            //TODO: target.etag = '';
            target.ravenEntityName = !target.ravenEntityName ? document.getEntityNameFromId(id) : target.ravenEntityName;
            target.id = id;
        };
        editSqlReplication.prototype.deleteSqlReplication = function () {
            var _this = this;
            eventsCollector.default.reportEvent("edit-sql-replication", "delete");
            var newDoc = new document(this.editedReplication().toDto());
            if (newDoc) {
                var viewModel = new deleteDocuments([newDoc], this.activeDatabase());
                viewModel.deletionTask.done(function () {
                    _this.dirtyFlag().reset(); //Resync Changes
                    router.navigate(appUrl.forCurrentDatabase().sqlReplications());
                });
                app.showBootstrapDialog(viewModel, editSqlReplication.editSqlReplicationSelector);
            }
        };
        editSqlReplication.prototype.resetSqlReplication = function () {
            var _this = this;
            eventsCollector.default.reportEvent("edit-sql-replication", "reset");
            app.showBootstrapMessage("You are about to reset this SQL Replication, forcing replication of all collection items", "SQL Replication Reset", ["Cancel", "Reset"])
                .then(function (dialogResult) {
                if (dialogResult === "Reset") {
                    var replicationId = _this.initialReplicationId;
                    new resetSqlReplicationCommand(_this.activeDatabase(), replicationId).execute()
                        .done(function () { return messagePublisher.reportSuccess("SQL replication " + replicationId + " was reset successfully!"); })
                        .fail(function () { return messagePublisher.reportError("SQL replication " + replicationId + " failed to reset!"); });
                }
            });
        };
        editSqlReplication.prototype.simulateSqlReplication = function () {
            eventsCollector.default.reportEvent("edit-sql-replication", "simulate");
            this.editedReplication().script(this.script());
            var viewModel = new sqlReplicationSimulationDialog(this.activeDatabase(), this.editedReplication(), this.simulationDocumentId);
            app.showBootstrapDialog(viewModel);
        };
        editSqlReplication.prototype.getSqlReplicationConnectionStringsUrl = function (sqlReplicationName) {
            return appUrl.forSqlReplicationConnections(this.activeDatabase());
        };
        return editSqlReplication;
    }(viewModelBase));
    editSqlReplication.editSqlReplicationSelector = "#editSQLReplication";
    editSqlReplication.sqlReplicationDocumentPrefix = "Raven/SqlReplication/Configuration/";
    return editSqlReplication;
});

//# sourceMappingURL=editSqlReplication.js.map
