var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseSettingsCommand", "models/database/documents/document", "models/database/documents/periodicExportSetup", "common/appUrl", "commands/database/documents/getPeriodicExportSetupCommand", "commands/database/documents/saveDocumentCommand", "common/eventsCollector"], function (require, exports, viewModelBase, getDatabaseSettingsCommand, document, periodicExportSetup, appUrl, getPeriodicExportSetupCommand, saveDocumentCommand, eventsCollector) {
    "use strict";
    var periodicExport = (function (_super) {
        __extends(periodicExport, _super);
        function periodicExport() {
            var _this = _super.call(this) || this;
            _this.backupSetup = ko.observable().extend({ required: true });
            _this.exportDisabled = ko.observable(false);
            _this.isForbidden = ko.observable(false);
            _this.activeDatabase.subscribe(function (db) { return _this.isForbidden(!db.isAdminCurrentTenant()); });
            _this.showOnDiskExportRow = ko.computed(function () { return _this.backupSetup() && _this.backupSetup().onDiskExportEnabled(); });
            return _this;
        }
        periodicExport.prototype.attached = function () {
            _super.prototype.attached.call(this);
            var content = "Could not decrypt the access settings, if you are running on IIS, make sure that load user profile is set to true. " +
                "Alternatively this can happen when the server was started using different account than when the settings were created.<br />" +
                "Reenter your settings and click save.";
            $("#awsDecryptFailureSpan").popover({
                html: true,
                trigger: "hover",
                container: "body",
                content: content
            });
            $("#azureDecryptFailureSpan").popover({
                html: true,
                trigger: "hover",
                container: "body",
                content: content
            });
        };
        periodicExport.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            this.backupSetup(new periodicExportSetup);
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            this.isForbidden(!db.isAdminCurrentTenant());
            if (db.isAdminCurrentTenant()) {
                $.when(this.fetchPeriodicExportSetup(db), this.fetchPeriodicExportAccountsSettings(db))
                    .done(function () {
                    _this.updateExportDisabledFlag();
                    deferred.resolve({ can: true });
                })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        periodicExport.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("OU78CB");
            this.dirtyFlag = new ko.DirtyFlag([this.backupSetup]);
            var self = this;
            this.isSaveEnabled = ko.computed(function () {
                var onDisk = self.backupSetup().onDiskExportEnabled();
                var remote = self.backupSetup().remoteUploadEnabled();
                var hasAnyOption = onDisk || remote;
                var isDirty = _this.dirtyFlag().isDirty();
                return hasAnyOption && isDirty;
            });
        };
        periodicExport.prototype.fetchPeriodicExportSetup = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getPeriodicExportSetupCommand(db)
                .execute()
                .done(function (result) {
                _this.backupSetup().fromDto(result);
            })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        periodicExport.prototype.fetchPeriodicExportAccountsSettings = function (db) {
            var _this = this;
            var task = $.Deferred();
            var dbSettingsTask = new getDatabaseSettingsCommand(db)
                .execute()
                .done(function (document) {
                _this.backupSetup().fromDatabaseSettingsDto(document.toDto(false));
            });
            dbSettingsTask.then(function () {
                task.resolve(); //TODO: delete me!
                /* TODO add support for remote export
                new getConfigurationSettingsCommand(db,
                    ["Raven/AWSAccessKey", "Raven/AWSSecretKey", "Raven/AzureStorageAccount", "Raven/AzureStorageKey"])
                    .execute()
                    .done((result: configurationSettings) => {
                        var awsAccess = result.results["Raven/AWSAccessKey"];
                        var awsSecret = result.results["Raven/AWSSecretKey"];
                        var azureAccess = result.results["Raven/AzureStorageAccount"];
                        var azureSecret = result.results["Raven/AzureStorageKey"];
                        this.backupSetup().awsAccessKey(awsAccess.effectiveValue());
                        this.backupSetup().awsSecretKey(awsSecret.effectiveValue());
                        this.backupSetup().azureStorageAccount(azureAccess.effectiveValue());
                        this.backupSetup().azureStorageKey(azureSecret.effectiveValue());
                        task.resolve();
                });*/
            });
            return task;
        };
        periodicExport.prototype.saveChanges = function () {
            var _this = this;
            eventsCollector.default.reportEvent("periodic-export", "save");
            var db = this.activeDatabase();
            if (db) {
                var periodicExportConfig = this.backupSetup().toDto();
                new saveDocumentCommand("Raven/PeriodicExport/Configuration", new document(periodicExportConfig), this.activeDatabase())
                    .execute()
                    .done(function () {
                    _this.backupSetup().resetDecryptionFailures();
                    _this.dirtyFlag().reset();
                    _this.updateExportDisabledFlag();
                });
            }
        };
        periodicExport.prototype.updateExportDisabledFlag = function () {
            this.exportDisabled(!this.backupSetup().active());
        };
        return periodicExport;
    }(viewModelBase));
    return periodicExport;
});

//# sourceMappingURL=periodicExport.js.map
