var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/resources/saveDatabaseSettingsCommand", "commands/database/globalConfig/getConfigurationSettingsCommand", "models/database/documents/document", "common/appUrl", "commands/resources/getDatabaseSettingsCommand", "viewmodels/shell/accessHelper", "common/eventsCollector"], function (require, exports, viewModelBase, saveDatabaseSettingsCommand, getConfigurationSettingsCommand, document, appUrl, getDatabaseSettingsCommand, accessHelper, eventsCollector) {
    "use strict";
    var quotas = (function (_super) {
        __extends(quotas, _super);
        function quotas() {
            var _this = _super.call(this) || this;
            _this.settingsDocument = ko.observable();
            _this.usingGlobal = ko.observable(false);
            _this.hasGlobalValues = ko.observable(false);
            _this.isForbidden = ko.observable(false);
            _this.activeDatabase.subscribe(function (db) { return _this.isForbidden(!db.isAdminCurrentTenant()); });
            return _this;
        }
        quotas.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            this.isForbidden(accessHelper.isGlobalAdmin() == false);
            if (this.isForbidden() == false) {
                var db = this.activeDatabase();
                // fetch current quotas from the database
                this.fetchQuotas(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        quotas.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('594W7T');
            this.initializeDirtyFlag();
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty() === true; });
        };
        quotas.prototype.fetchQuotas = function (db, reportFetchProgress) {
            var _this = this;
            if (reportFetchProgress === void 0) { reportFetchProgress = false; }
            var dbSettingsTask = new getDatabaseSettingsCommand(db, reportFetchProgress)
                .execute()
                .done(function (doc) { return _this.settingsDocument(doc); });
            var configTask = new getConfigurationSettingsCommand(db, ["Raven/Quotas/Size/HardLimitInKB", "Raven/Quotas/Size/SoftMarginInKB", "Raven/Quotas/Documents/HardLimit", "Raven/Quotas/Documents/SoftLimit"])
                .execute()
                .done(function (result) {
                _this.maximumSize = result.results["Raven/Quotas/Size/HardLimitInKB"];
                _this.warningLimitThreshold = result.results["Raven/Quotas/Size/SoftMarginInKB"];
                _this.maxNumberOfDocs = result.results["Raven/Quotas/Documents/HardLimit"];
                _this.warningThresholdForDocs = result.results["Raven/Quotas/Documents/SoftLimit"];
                _this.usingGlobal(_this.maximumSize.isUsingGlobal());
                _this.hasGlobalValues(_this.maximumSize.globalExists());
                var divideBy1024 = function (x) {
                    if (x()) {
                        x(x() / 1024);
                    }
                };
                divideBy1024(_this.maximumSize.effectiveValue);
                divideBy1024(_this.maximumSize.globalValue);
                divideBy1024(_this.warningLimitThreshold.effectiveValue);
                divideBy1024(_this.warningLimitThreshold.globalValue);
            });
            return $.when(dbSettingsTask, configTask);
        };
        quotas.prototype.initializeDirtyFlag = function () {
            this.dirtyFlag = new ko.DirtyFlag([
                this.maximumSize.effectiveValue, this.maximumSize.localExists,
                this.warningLimitThreshold.effectiveValue, this.warningLimitThreshold.localExists,
                this.maxNumberOfDocs.effectiveValue, this.maxNumberOfDocs.localExists,
                this.warningThresholdForDocs.effectiveValue, this.warningThresholdForDocs.localExists,
                this.usingGlobal
            ]);
        };
        quotas.prototype.saveChanges = function () {
            var _this = this;
            eventsCollector.default.reportEvent("quotas", "save");
            var db = this.activeDatabase();
            if (db) {
                var settingsDocument = this.settingsDocument();
                settingsDocument["@metadata"] = this.settingsDocument().__metadata;
                settingsDocument["@metadata"]["@etag"] = this.settingsDocument().__metadata["@etag"];
                var doc = new document(settingsDocument.toDto(true));
                if (this.usingGlobal()) {
                    delete doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"];
                    delete doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"];
                    delete doc["Settings"]["Raven/Quotas/Documents/HardLimit"];
                    delete doc["Settings"]["Raven/Quotas/Documents/SoftLimit"];
                }
                else {
                    doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"] = this.maximumSize.effectiveValue() * 1024;
                    doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"] = this.warningLimitThreshold.effectiveValue() * 1024;
                    doc["Settings"]["Raven/Quotas/Documents/HardLimit"] = this.maxNumberOfDocs.effectiveValue();
                    doc["Settings"]["Raven/Quotas/Documents/SoftLimit"] = this.warningThresholdForDocs.effectiveValue();
                }
                var saveTask = new saveDatabaseSettingsCommand(db, doc).execute();
                saveTask.done(function (saveResult) {
                    _this.settingsDocument().__metadata["@etag"] = saveResult.ETag;
                    _this.dirtyFlag().reset(); //Resync Changes
                });
            }
        };
        quotas.prototype.useLocal = function () {
            eventsCollector.default.reportEvent("quotas", "use-local");
            this.usingGlobal(false);
        };
        quotas.prototype.useGlobal = function () {
            eventsCollector.default.reportEvent("quotas", "use-global");
            this.usingGlobal(true);
            this.maximumSize.copyFromGlobal();
            this.warningLimitThreshold.copyFromGlobal();
            this.maxNumberOfDocs.copyFromGlobal();
            this.warningThresholdForDocs.copyFromGlobal();
        };
        return quotas;
    }(viewModelBase));
    return quotas;
});

//# sourceMappingURL=quotas.js.map
