var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/database/replication/replicationsSetup", "models/database/replication/replicationConfig", "models/database/replication/replicationDestination", "commands/resources/getDatabaseStatsCommand", "commands/database/replication/getReplicationsCommand", "commands/database/documents/updateServerPrefixHiLoCommand", "commands/database/replication/saveReplicationDocumentCommand", "commands/database/replication/saveAutomaticConflictResolutionDocumentCommand", "commands/database/documents/getServerPrefixForHiLoCommand", "commands/database/replication/replicateAllIndexesCommand", "common/bindingHelpers/aceEditorBindingHandler", "commands/database/replication/replicateAllTransformersCommand", "commands/database/replication/replicateIndexesCommand", "commands/database/replication/replicateTransformersCommand", "commands/database/globalConfig/getEffectiveConflictResolutionCommand", "common/appUrl", "commands/database/replication/enableReplicationCommand", "commands/database/replication/resolveAllConflictsCommand", "common/eventsCollector"], function (require, exports, viewModelBase, replicationsSetup, replicationConfig, replicationDestination, getDatabaseStatsCommand, getReplicationsCommand, updateServerPrefixHiLoCommand, saveReplicationDocumentCommand, saveAutomaticConflictResolutionDocumentCommand, getServerPrefixForHiLoCommand, replicateAllIndexesCommand, aceEditorBindingHandler, replicateAllTransformersCommand, replicateIndexesCommand, replicateTransformersCommand, getEffectiveConflictResolutionCommand, appUrl, enableReplicationCommand, resolveAllConflictsCommand, eventsCollector) {
    "use strict";
    var replications = (function (_super) {
        __extends(replications, _super);
        function replications() {
            var _this = _super.call(this) || this;
            _this.replicationEnabled = ko.observable(true);
            _this.prefixForHilo = ko.observable("");
            _this.replicationConfig = ko.observable(new replicationConfig({ DocumentConflictResolution: "None" }));
            _this.replicationsSetup = ko.observable(new replicationsSetup({ Source: null, Destinations: [], ClientConfiguration: null, DocumentConflictResolution: null, Id: null }));
            _this.serverPrefixForHiLoDirtyFlag = new ko.DirtyFlag([]);
            _this.replicationConfigDirtyFlag = new ko.DirtyFlag([]);
            _this.replicationsSetupDirtyFlag = new ko.DirtyFlag([]);
            _this.skipIndexReplicationForAllDestinationsStatus = ko.observable();
            _this.skipIndexReplicationForAll = ko.observable();
            aceEditorBindingHandler.install();
            _this.showRequestTimeoutRow = ko.computed(function () { return _this.replicationsSetup().showRequestTimeSlaThreshold(); });
            return _this;
        }
        replications.prototype.refereshSkipIndexReplicationForAllDestinations = function () {
            var _this = this;
            if (this.skipIndexReplicationForAllSubscription != null)
                this.skipIndexReplicationForAllSubscription.dispose();
            var newStatus = this.getIndexReplicationStatusForAllDestinations();
            this.skipIndexReplicationForAll(newStatus === 'all');
            this.skipIndexReplicationForAllSubscription = this.skipIndexReplicationForAll.subscribe(function (newValue) { return _this.toggleIndexReplication(newValue); });
        };
        replications.prototype.getIndexReplicationStatusForAllDestinations = function () {
            var nonEtlDestinations = this.replicationsSetup().destinations().filter(function (x) { return !x.enableReplicateOnlyFromCollections(); });
            var nonEtlWithSkipedIndexReplicationCount = nonEtlDestinations.filter(function (x) { return x.skipIndexReplication(); }).length;
            if (nonEtlWithSkipedIndexReplicationCount === 0)
                return 'none';
            if (nonEtlWithSkipedIndexReplicationCount === nonEtlDestinations.length)
                return 'all';
            return 'mixed';
        };
        replications.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                //TODO: we don't have active bundles in v4.0, let assume it is enabled
                this.replicationEnabled(true);
                $.when(this.fetchServerPrefixForHiLoCommand(db), this.fetchAutomaticConflictResolution(db), this.fetchReplications(db))
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forSettings(db) }); });
            }
            return deferred;
        };
        replications.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("7K1KES");
            this.serverPrefixForHiLoDirtyFlag = new ko.DirtyFlag([this.prefixForHilo]);
            this.isServerPrefixForHiLoSaveEnabled = ko.computed(function () { return _this.serverPrefixForHiLoDirtyFlag().isDirty(); });
            this.replicationConfigDirtyFlag = new ko.DirtyFlag([this.replicationConfig]);
            this.isConfigSaveEnabled = ko.computed(function () { return _this.replicationConfigDirtyFlag().isDirty(); });
            var replicationSetupDirtyFlagItems = [this.replicationsSetup, this.replicationsSetup().destinations(), this.skipIndexReplicationForAll, this.replicationConfig, this.replicationsSetup().clientFailoverBehaviour];
            this.replicationsSetupDirtyFlag = new ko.DirtyFlag(replicationSetupDirtyFlagItems);
            this.isSetupSaveEnabled = ko.computed(function () { return _this.replicationsSetupDirtyFlag().isDirty(); });
            var combinedFlag = ko.computed(function () {
                var rc = _this.replicationConfigDirtyFlag().isDirty();
                var rs = _this.replicationsSetupDirtyFlag().isDirty();
                var sp = _this.serverPrefixForHiLoDirtyFlag().isDirty();
                return rc || rs || sp;
            });
            this.dirtyFlag = new ko.DirtyFlag([combinedFlag]);
        };
        replications.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $.each(this.replicationsSetup().destinations(), this.addScriptHelpPopover);
        };
        replications.prototype.fetchServerPrefixForHiLoCommand = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getServerPrefixForHiLoCommand(db)
                .execute()
                .done(function (result) { return _this.prefixForHilo(result); })
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        replications.prototype.fetchAutomaticConflictResolution = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getEffectiveConflictResolutionCommand(db)
                .execute()
                .done(function (repConfig) {
                _this.replicationConfig(new replicationConfig(repConfig.MergedDocument));
            })
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        replications.prototype.fetchReplications = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            ko.postbox.subscribe('skip-index-replication', function () { return _this.refereshSkipIndexReplicationForAllDestinations(); });
            new getReplicationsCommand(db)
                .execute()
                .done(function (repSetup) {
                if (repSetup) {
                    _this.replicationsSetup(new replicationsSetup(repSetup));
                }
                var status = _this.getIndexReplicationStatusForAllDestinations();
                if (status === 'all')
                    _this.skipIndexReplicationForAll(true);
                else
                    _this.skipIndexReplicationForAll(false);
                _this.skipIndexReplicationForAllSubscription = _this.skipIndexReplicationForAll.subscribe(function (newValue) { return _this.toggleIndexReplication(newValue); });
            })
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        replications.prototype.addScriptHelpPopover = function () {
            $(".scriptPopover").popover({
                html: true,
                trigger: 'hover',
                container: ".form-horizontal",
                content: '<p>Return <code>null</code> in transform script to skip document from replication. </p>' +
                    '<p>Example: </p>' +
                    '<pre><span class="code-keyword">if</span> (<span class="code-keyword">this</span>.Region !== <span class="code-string">"Europe"</span>) { <br />   <span class="code-keyword">return null</span>; <br />}<br/><span class="code-keyword">this</span>.Currency = <span class="code-string">"EUR"</span>; </pre>'
            });
        };
        replications.prototype.onTransformCollectionClick = function (destination, collectionName) {
            var collections = destination.specifiedCollections();
            var item = collections.find(function (c) { return c.collection() === collectionName; });
            if (typeof (item.script()) === "undefined") {
                item.script("");
            }
            else {
                item.script(undefined);
            }
            destination.specifiedCollections.notifySubscribers();
        };
        replications.prototype.toggleSkipIndexReplicationForAll = function () {
            eventsCollector.default.reportEvent("replications", "toggle-skip-for-all");
            this.skipIndexReplicationForAll.toggle();
        };
        replications.prototype.createNewDestination = function () {
            eventsCollector.default.reportEvent("replications", "create");
            var db = this.activeDatabase();
            this.replicationsSetup().destinations.unshift(replicationDestination.empty(db.name));
            this.refereshSkipIndexReplicationForAllDestinations();
            this.addScriptHelpPopover();
        };
        replications.prototype.removeDestination = function (repl) {
            eventsCollector.default.reportEvent("replications", "toggle-skip-for-all");
            this.replicationsSetup().destinations.remove(repl);
        };
        replications.prototype.saveChanges = function () {
            var _this = this;
            eventsCollector.default.reportEvent("replications", "save");
            if (this.isConfigSaveEnabled())
                this.saveAutomaticConflictResolutionSettings();
            if (this.isSetupSaveEnabled()) {
                if (this.replicationsSetup().source()) {
                    this.saveReplicationSetup();
                }
                else {
                    var db = this.activeDatabase();
                    if (db) {
                        new getDatabaseStatsCommand(db)
                            .execute()
                            .done(function (result) {
                            _this.prepareAndSaveReplicationSetup(result.DatabaseId);
                        });
                    }
                }
            }
        };
        replications.prototype.prepareAndSaveReplicationSetup = function (source) {
            this.replicationsSetup().source(source);
            this.saveReplicationSetup();
        };
        replications.prototype.saveReplicationSetup = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                new saveReplicationDocumentCommand(this.replicationsSetup().toDto(), db)
                    .execute()
                    .done(function () {
                    _this.replicationsSetupDirtyFlag().reset();
                    _this.dirtyFlag().reset();
                });
            }
        };
        replications.prototype.toggleIndexReplication = function (skipReplicationValue) {
            this.replicationsSetup().destinations().forEach(function (dest) {
                // since we are on replications page filter toggle to non-etl destinations only
                if (!dest.enableReplicateOnlyFromCollections()) {
                    dest.skipIndexReplication(skipReplicationValue);
                }
            });
        };
        replications.prototype.sendReplicateCommand = function (destination, parentClass) {
            eventsCollector.default.reportEvent("replications", "send-replicate");
            var db = parentClass.activeDatabase();
            if (db) {
                new replicateIndexesCommand(db, destination).execute();
                new replicateTransformersCommand(db, destination).execute();
            }
            else {
                alert("No database selected! This error should not be seen."); //precaution to ease debugging - in case something bad happens
            }
        };
        replications.prototype.sendReplicateAllCommand = function () {
            eventsCollector.default.reportEvent("replications", "send-replicate-all");
            var db = this.activeDatabase();
            if (db) {
                new replicateAllIndexesCommand(db).execute();
                new replicateAllTransformersCommand(db).execute();
            }
            else {
                alert("No database selected! This error should not be seen."); //precaution to ease debugging - in case something bad happens
            }
        };
        replications.prototype.sendResolveAllConflictsCommand = function () {
            eventsCollector.default.reportEvent("replications", "resolve-all");
            var db = this.activeDatabase();
            if (db) {
                new resolveAllConflictsCommand(db).execute();
            }
            else {
                alert("No database selected! This error should not be seen."); //precaution to ease debugging - in case something bad happens
            }
        };
        replications.prototype.saveServerPrefixForHiLo = function () {
            var _this = this;
            eventsCollector.default.reportEvent("replications", "save-hilo-prefix");
            var db = this.activeDatabase();
            if (db) {
                new updateServerPrefixHiLoCommand(this.prefixForHilo(), db)
                    .execute()
                    .done(function () {
                    _this.serverPrefixForHiLoDirtyFlag().reset();
                    _this.dirtyFlag().reset();
                });
            }
        };
        replications.prototype.saveAutomaticConflictResolutionSettings = function () {
            var _this = this;
            eventsCollector.default.reportEvent("replications", "save-auto-conflict-resolution");
            var db = this.activeDatabase();
            if (db) {
                new saveAutomaticConflictResolutionDocumentCommand(this.replicationConfig().toDto(), db)
                    .execute()
                    .done(function () {
                    _this.replicationConfigDirtyFlag().reset();
                    _this.dirtyFlag().reset();
                });
            }
        };
        replications.prototype.enableReplication = function () {
            var _this = this;
            eventsCollector.default.reportEvent("replications", "enable-replication");
            new enableReplicationCommand(this.activeDatabase())
                .execute()
                .done(function (bundles) {
                var db = _this.activeDatabase();
                db.activeBundles(bundles);
                _this.replicationEnabled(true);
                _this.fetchServerPrefixForHiLoCommand(db);
                _this.fetchAutomaticConflictResolution(db);
                _this.fetchReplications(db);
            });
        };
        return replications;
    }(viewModelBase));
    return replications;
});

//# sourceMappingURL=replications.js.map
