var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/database/sqlReplication/sqlReplicationConnections", "models/database/sqlReplication/predefinedSqlConnection", "models/database/documents/document", "commands/database/documents/saveDocumentCommand", "common/appUrl", "commands/database/sqlReplication/getSqlReplicationConnectionsCommand", "common/eventsCollector"], function (require, exports, viewModelBase, sqlReplicationConnections, predefinedSqlConnection, document, saveDocumentCommand, appUrl, getSqlReplicationConnectionsCommand, eventsCollector) {
    "use strict";
    var sqlReplicationConnectionStringsManagement = (function (_super) {
        __extends(sqlReplicationConnectionStringsManagement, _super);
        function sqlReplicationConnectionStringsManagement() {
            var _this = _super.call(this) || this;
            _this.htmlSelector = "#sqlReplicationConnectionsManagement";
            _this.connections = ko.observable();
            return _this;
        }
        sqlReplicationConnectionStringsManagement.prototype.loadConnections = function () {
            var _this = this;
            return new getSqlReplicationConnectionsCommand(this.activeDatabase())
                .execute()
                .done(function (repSetup) {
                _this.connections(new sqlReplicationConnections(repSetup));
                if (_this.connections().predefinedConnections().length > 0) {
                    _this.connections().predefinedConnections().forEach(function (x) { return _this.subscribeToSqlReplicationConnectionName(x); });
                }
            })
                .fail(function () {
                _this.connections(sqlReplicationConnections.empty());
            });
        };
        sqlReplicationConnectionStringsManagement.prototype.canActivate = function () {
            var def = $.Deferred();
            this.loadConnections()
                .always(function () { return def.resolve({ can: true }); });
            return def;
        };
        sqlReplicationConnectionStringsManagement.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.connections]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        sqlReplicationConnectionStringsManagement.prototype.save = function () {
            var _this = this;
            eventsCollector.default.reportEvent("sql-replication-connections", "save");
            var newDoc = new document(this.connections().toDto());
            this.attachReservedMetaProperties("Raven/SqlReplication/Connections", newDoc.__metadata);
            var saveCommand = new saveDocumentCommand("Raven/SqlReplication/Connections", newDoc, this.activeDatabase());
            var saveTask = saveCommand.execute();
            saveTask.done(function () { return _this.dirtyFlag().reset(); });
        };
        sqlReplicationConnectionStringsManagement.prototype.attachReservedMetaProperties = function (id, target) {
            target.etag(0);
            target.ravenEntityName = !target.ravenEntityName ? document.getEntityNameFromId(id) : target.ravenEntityName;
            target.id = id;
        };
        sqlReplicationConnectionStringsManagement.prototype.getSqlReplicationConnectionsUrl = function () {
            return appUrl.forSqlReplicationConnections(this.activeDatabase());
        };
        sqlReplicationConnectionStringsManagement.prototype.addSqlReplicationConnection = function () {
            eventsCollector.default.reportEvent("sql-replication-connections", "create");
            var newPredefinedConnection = predefinedSqlConnection.empty();
            this.connections().predefinedConnections.unshift(newPredefinedConnection);
            this.subscribeToSqlReplicationConnectionName(newPredefinedConnection);
            newPredefinedConnection.name("New");
        };
        sqlReplicationConnectionStringsManagement.prototype.removeSqlReplicationConnection = function (connection) {
            eventsCollector.default.reportEvent("sql-replication-connections", "remove");
            this.connections().predefinedConnections.remove(connection);
        };
        sqlReplicationConnectionStringsManagement.prototype.subscribeToSqlReplicationConnectionName = function (con) {
            var _this = this;
            con.name.subscribe(function (previousName) {
                //Get the previous value of 'name' here before it's set to newValue
                var nameInputArray = $('input[name="name"]')
                    .each(function (index, inputField) {
                    inputField.setCustomValidity("");
                });
            }, this, "beforeChange");
            con.name.subscribe(function (newName) {
                var message = "";
                if (newName === "") {
                    message = "Please fill out this field.";
                }
                else if (_this.isSqlPredefinedConnectionNameExists(newName)) {
                    message = "SQL Replication Connection name already exists.";
                }
                $('input[name="name"]')
                    .filter(function () { return this.value === newName; })
                    .each(function (index, element) {
                    element.setCustomValidity(message);
                });
            });
        };
        sqlReplicationConnectionStringsManagement.prototype.isSqlPredefinedConnectionNameExists = function (connectionName) {
            if (this.connections().predefinedConnections().some(function (x) { return x.name() === connectionName; })) {
                return true;
            }
            return false;
        };
        sqlReplicationConnectionStringsManagement.prototype.providerChanged = function (obj, event) {
            if (event.originalEvent) {
                var curConnectionString = !!obj.connectionString() ? obj.connectionString().trim() : "";
                if (curConnectionString === "" ||
                    sqlReplicationConnections.sqlProvidersConnectionStrings.find(function (x) { return x.ConnectionString == curConnectionString; })) {
                    var matchingConnectionStringPair = sqlReplicationConnections.sqlProvidersConnectionStrings.find(function (x) { return x.ProviderName === event.originalEvent.srcElement.selectedOptions[0].value; });
                    if (!!matchingConnectionStringPair) {
                        var matchingConnectionStringValue = matchingConnectionStringPair.ConnectionString;
                        obj.connectionString(matchingConnectionStringValue);
                    }
                }
            }
        };
        return sqlReplicationConnectionStringsManagement;
    }(viewModelBase));
    return sqlReplicationConnectionStringsManagement;
});

//# sourceMappingURL=sqlReplicationConnectionStringsManagement.js.map
