var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/database/sqlReplication/getSqlReplicationsCommand", "common/appUrl", "viewmodels/database/status/sqlReplicationStatsDialog", "durandal/app", "models/database/documents/document", "viewmodels/common/deleteDocuments", "common/messagePublisher", "commands/database/sqlReplication/resetSqlReplicationCommand", "commands/database/sqlReplication/toggleDisable", "common/eventsCollector"], function (require, exports, viewModelBase, getSqlReplicationsCommand, appUrl, sqlReplicationStatsDialog, app, document, deleteDocuments, messagePublisher, resetSqlReplicationCommand, toggleDisable, eventsCollector) {
    "use strict";
    var sqlReplications = (function (_super) {
        __extends(sqlReplications, _super);
        function sqlReplications() {
            var _this = _super.call(this) || this;
            _this.replications = ko.observableArray();
            _this.searchText = ko.observable("");
            _this.itemNumber = function (index) {
                return index + 1;
            };
            _this.areAllSqlReplicationsDisabled = ko.computed(function () {
                var replications = _this.replications();
                for (var i = 0; i < replications.length; i++) {
                    if (replications[i].disabled() === false)
                        return false;
                }
                return true;
            });
            _this.areAllSqlReplicationsEnabled = ko.computed(function () {
                var replications = _this.replications();
                for (var i = 0; i < replications.length; i++) {
                    if (replications[i].disabled())
                        return false;
                }
                return true;
            });
            _this.searchText.extend({ throttle: 200 }).subscribe(function () { return _this.filterSqlReplications(); });
            _this.summary = ko.computed(function () {
                var summary = "";
                if (_this.replications().length === 0) {
                    return summary;
                }
                var visibleSqlReplications = _this.replications().filter(function (x) { return x.isVisible(); });
                summary += visibleSqlReplications.length + " SQL Replication";
                if (visibleSqlReplications.length > 1) {
                    summary += "s";
                }
                var disabled = visibleSqlReplications.filter(function (x) { return x.disabled(); }).length;
                if (disabled > 0) {
                    summary += " (" + disabled + " disabled)";
                }
                return summary;
            });
            return _this;
        }
        sqlReplications.prototype.filterSqlReplications = function () {
            var filter = this.searchText();
            var filterLower = filter.toLowerCase();
            this.replications().forEach(function (sql) {
                var isMatch = (!filter || (sql.name().toLowerCase().indexOf(filterLower) >= 0));
                sql.isVisible(isMatch);
            });
        };
        sqlReplications.prototype.toggleDisable = function (disable) {
            var _this = this;
            var self = this;
            var action = disable ? "disable" : "enable";
            var actionCapitalized = _.upperFirst(action);
            app.showBootstrapMessage("Are you sure that you want to " + action + " all SQL Replications?", actionCapitalized + " SQL Replications", ["Cancel", actionCapitalized])
                .then(function (dialogResult) {
                if (dialogResult === actionCapitalized) {
                    new toggleDisable(self.activeDatabase(), disable).execute()
                        .done(function () {
                        _this.replications().forEach(function (x) { return x.disabled(disable); });
                        messagePublisher.reportSuccess("Successfully " + action + "d all SQL replications");
                    });
                }
            });
        };
        sqlReplications.prototype.showStats = function (replicationName) {
            eventsCollector.default.reportEvent("sql-replications", "stats");
            var viewModel = new sqlReplicationStatsDialog(this.activeDatabase(), replicationName);
            app.showBootstrapDialog(viewModel);
        };
        sqlReplications.prototype.getSqlReplicationUrl = function (sqlReplicationName) {
            return appUrl.forEditSqlReplication(sqlReplicationName, this.activeDatabase());
        };
        sqlReplications.prototype.getSqlReplicationConnectionStringsUrl = function (sqlReplicationName) {
            return appUrl.forSqlReplicationConnections(this.activeDatabase());
        };
        sqlReplications.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                this.fetchSqlReplications(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forSettings(_this.activeDatabase()) }); });
            }
            return deferred;
        };
        sqlReplications.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('J9VD59');
        };
        sqlReplications.prototype.removeSqlReplication = function (sr) {
            var _this = this;
            eventsCollector.default.reportEvent("sql-replications", "remove");
            var newDoc = new document(sr);
            if (newDoc) {
                var viewModel = new deleteDocuments([newDoc], this.activeDatabase());
                viewModel.deletionTask.done(function () {
                    _this.replications.remove(sr);
                    //this.fetchSqlReplications(this.activeDatabase());
                });
                app.showBootstrapDialog(viewModel, sqlReplications.sqlReplicationsSelector);
            }
        };
        sqlReplications.prototype.resetSqlReplication = function (replicationId) {
            var _this = this;
            eventsCollector.default.reportEvent("sql-replications", "reset");
            app.showBootstrapMessage("You are about to reset this SQL Replication, forcing replication of all collection items", "SQL Replication Reset", ["Cancel", "Reset"])
                .then(function (dialogResult) {
                if (dialogResult === "Reset") {
                    new resetSqlReplicationCommand(_this.activeDatabase(), replicationId).execute()
                        .done(function () { return messagePublisher.reportSuccess("SQL replication " + replicationId + " was reset successfully!"); });
                }
            });
        };
        sqlReplications.prototype.fetchSqlReplications = function (db) {
            var _this = this;
            return new getSqlReplicationsCommand(db)
                .execute()
                .done(function (results) {
                _this.replications(results);
            });
        };
        return sqlReplications;
    }(viewModelBase));
    sqlReplications.sqlReplicationsSelector = "#sqlReplications";
    return sqlReplications;
});

//# sourceMappingURL=sqlReplications.js.map
