var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/database/documents/versioningEntry", "common/appUrl", "commands/database/documents/getVersioningsCommand", "commands/database/documents/saveDocumentCommand", "models/database/documents/document", "common/eventsCollector"], function (require, exports, viewModelBase, versioningEntry, appUrl, getVersioningsCommand, saveDocumentCommand, document, eventsCollector) {
    "use strict";
    var versioning = (function (_super) {
        __extends(versioning, _super);
        function versioning() {
            var _this = _super.apply(this, arguments) || this;
            //TODO: introduce model!
            _this.versionings = ko.observableArray();
            return _this;
        }
        versioning.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                this.fetchVersioningEntries(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
            }
            return deferred;
        };
        versioning.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("1UZ5WL");
            this.dirtyFlag = new ko.DirtyFlag([this.versionings]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        versioning.prototype.fetchVersioningEntries = function (db) {
            var _this = this;
            return new getVersioningsCommand(db).execute()
                .done(function (versionings) { return _this.versioningsLoaded(versionings); });
        };
        versioning.prototype.toDto = function () {
            var defaultConfiguration = this.versionings().find(function (x) { return x.collection() === versioningEntry.DefaultConfiguration; });
            var nonDefaultConfiguration = this.versionings().filter(function (x) { return x !== defaultConfiguration; });
            var collectionsDto = {};
            nonDefaultConfiguration.forEach(function (config) {
                collectionsDto[config.collection()] = config.toDto();
            });
            return {
                Default: defaultConfiguration.toDto(),
                Collections: collectionsDto
            };
        };
        versioning.prototype.saveChanges = function () {
            var _this = this;
            //TODO: check etag
            eventsCollector.default.reportEvent("versioning", "save");
            var dto = this.toDto();
            var versioningDocument = new document(dto);
            new saveDocumentCommand("Raven/Versioning/Configuration", versioningDocument, this.activeDatabase())
                .execute()
                .done(function (saveResult) { return _this.versioningsSaved(saveResult); });
        };
        versioning.prototype.createNewVersioning = function () {
            eventsCollector.default.reportEvent("versioning", "create");
            var emptyVersioning = versioningEntry.empty();
            if (this.versionings().length === 0) {
                emptyVersioning.collection(versioningEntry.DefaultConfiguration);
            }
            this.versionings.push(emptyVersioning);
        };
        versioning.prototype.removeVersioning = function (entry) {
            eventsCollector.default.reportEvent("versioning", "remove");
            this.versionings.remove(entry);
        };
        versioning.prototype.versioningsLoaded = function (data) {
            if (data) {
                var versionings = [];
                versionings.push(new versioningEntry(versioningEntry.DefaultConfiguration, data.Default));
                for (var collection in data.Collections) {
                    var configuration = data.Collections[collection];
                    versionings.push(new versioningEntry(collection, configuration));
                }
                this.versionings(versionings);
                this.dirtyFlag().reset();
            }
        };
        versioning.prototype.versioningsSaved = function (saveResult) {
            //TODO: test if we have to update etag in metadata to allow subsequent saves
            this.dirtyFlag().reset();
        };
        return versioning;
    }(viewModelBase));
    return versioning;
});

//# sourceMappingURL=versioning.js.map
