var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/router", "viewmodels/viewModelBase", "moment", "common/appUrl", "common/bindingHelpers/autoRefreshBindingHandler", "common/tableNavigationTrait", "common/eventsCollector"], function (require, exports, router, viewModelBase, moment, appUrl, autoRefreshBindingHandler, tableNavigationTrait, eventsCollector) {
    "use strict";
    var alerts = (function (_super) {
        __extends(alerts, _super);
        function alerts() {
            var _this = _super.call(this) || this;
            //TODO: alertDoc = ko.observable<alertContainerDto>();
            _this.allAlerts = ko.observableArray();
            _this.filterLevel = ko.observable("All");
            _this.selectedAlert = ko.observable();
            _this.selectedAlertIndex = ko.observable();
            _this.now = ko.observable();
            _this.updateNowTimeoutHandle = 0;
            _this.autoRefreshEnabled = ko.observable(true);
            autoRefreshBindingHandler.install();
            //TODO: this.unreadAlertCount = ko.computed(() => this.allAlerts().count(a => !a.observed()));
            //TODO: this.readAlertCount = ko.computed(() => this.allAlerts().count(a => a.observed()));
            _this.updateCurrentNowTime();
            //TODO: this.activeDatabase.subscribe(() => this.fetchAlerts());
            _this.tableNavigation = new tableNavigationTrait("#alertTableContainer", _this.selectedAlert, _this.allAlerts, function (i) { return "#alertsContainer table tbody tr:nth-child(" + (i + 1) + ")"; });
            return _this;
        }
        alerts.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            //TODO: this.fetchAlerts();
            var item = !!args.item && !isNaN(args.item) ? args.item : 0;
            this.updateHelpLink('HL46QE');
            this.selectedAlertIndex(item);
            this.dirtyFlag = new ko.DirtyFlag([this.allAlerts]);
            this.isSaveEnabled = ko.computed(function () {
                var refresh = _this.autoRefreshEnabled();
                var isDirty = _this.dirtyFlag().isDirty();
                return !refresh && isDirty;
            });
        };
        alerts.prototype.deactivate = function () {
            clearTimeout(this.updateNowTimeoutHandle);
        };
        /* TODO
        fetchAlerts(): JQueryPromise<alertContainerDto> {
            var db = this.activeDatabase();
            if (db) {
                return new getAlertsCommand(db)
                    .execute()
                    .done((result: alertContainerDto) => this.processAlertsResults(result));
            }
    
            return null;
        }
    
        processAlertsResults(result: alertContainerDto) {
            var alerts = result.Alerts.map(a => new alert(a));
            alerts.forEach(r => {
                r.createdAtHumanized = this.createHumanReadableTime(r.createdAt),
                r.isVisible = ko.computed(() => this.matchesFilter(r));
            });
            this.alertDoc(result);
            this.allAlerts(alerts);
            if (alerts.length > 0) {
                this.selectAlert(alerts[this.selectedAlertIndex()]);
            }
            this.dirtyFlag().reset();
        }
    
        matchesFilter(a: alert): boolean {
            if (this.filterLevel() === "All") {
                return true;
            }
    
            var unreadFilterWithUnreadAlert = this.filterLevel() === "Unread" && !a.observed();
            var readFilterWithReadAlert = this.filterLevel() === "Read" && a.observed();
            return unreadFilterWithUnreadAlert || readFilterWithReadAlert;
        }*/
        alerts.prototype.createHumanReadableTime = function (time) {
            var _this = this;
            if (time) {
                // Return a computed that returns a humanized string based off the current time, e.g. "7 minutes ago".
                // It's a computed so that it updates whenever we update this.now (scheduled to occur every minute.)
                return ko.computed(function () {
                    var dateMoment = moment(time);
                    var agoInMs = dateMoment.diff(_this.now());
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                });
            }
            return ko.computed(function () { return time; });
        };
        alerts.prototype.selectAlert = function (selection) {
            var index = this.allAlerts.indexOf(selection);
            this.selectedAlertIndex(index);
            this.selectedAlert(selection);
            var alertUrl = appUrl.forAlerts(this.activeDatabase()) + "&item=" + this.selectedAlertIndex();
            router.navigate(alertUrl, false);
        };
        alerts.prototype.setFilterAll = function () {
            this.filterLevel("All");
        };
        alerts.prototype.setFilterUnread = function () {
            this.filterLevel("Unread");
        };
        alerts.prototype.setFilterRead = function () {
            this.filterLevel("Read");
        };
        alerts.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 60000);
        };
        alerts.prototype.toggleSelectedReadState = function () {
            /* TODO
            this.disableAutoRefresh();
            var alert = this.selectedAlert();
            if (alert) {
                if (!alert.observed()) {
                    alert.lastDismissedAt = this.now().toISOString();
                }
                alert.observed(!alert.observed());
            }*/
        };
        alerts.prototype.disableAutoRefresh = function () {
            this.autoRefreshEnabled(false);
        };
        alerts.prototype.deleteSelectedAlert = function () {
            eventsCollector.default.reportEvent("alerts", "delete", "selected");
            this.disableAutoRefresh();
            var alert = this.selectedAlert();
            if (alert) {
                this.allAlerts.remove(alert);
            }
        };
        alerts.prototype.deleteReadAlerts = function () {
            eventsCollector.default.reportEvent("alerts", "delete", "read");
            this.disableAutoRefresh();
            //TODO: this.allAlerts.remove(a => a.observed());
        };
        alerts.prototype.deleteAllAlerts = function () {
            eventsCollector.default.reportEvent("alerts", "delete", "all");
            this.disableAutoRefresh();
            this.allAlerts.removeAll();
        };
        alerts.prototype.saveAlerts = function () {
            eventsCollector.default.reportEvent("alerts", "save");
            /* TODO
            var alertDoc = this.alertDoc();
            var db = this.activeDatabase();
            if (alertDoc && db) {
                alertDoc.Alerts = this.allAlerts().map(a => a.toDto());
                new saveAlertsCommand(alertDoc, db)
                    .execute()
                    .done(() => this.fetchAlerts());
    
            }*/
        };
        return alerts;
    }(viewModelBase));
    return alerts;
});

//# sourceMappingURL=alerts.js.map
