var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "common/extensions", "commands/database/replication/getReplicationsCommand", "commands/database/documents/getDocumentsMetadataByIDPrefixCommand", "commands/database/replication/explainReplicationCommand", "common/eventsCollector"], function (require, exports, viewModelBase, extensions, getReplicationsCommand, getDocumentsMetadataByIDPrefixCommand, explainReplicationCommand, eventsCollector) {
    "use strict";
    var statusDebugExplainReplication = (function (_super) {
        __extends(statusDebugExplainReplication, _super);
        function statusDebugExplainReplication() {
            var _this = _super.call(this) || this;
            _this.destinations = ko.observable([]);
            _this.selectedDestination = ko.observable();
            _this.documentId = ko.observable();
            _this.documentIdSearchResults = ko.observableArray();
            _this.explanation = ko.observable();
            _this.buttonEnabled = ko.computed(function () {
                var destionationSelected = _this.selectedDestination() != null;
                var documentSelected = _this.documentId();
                return destionationSelected && (documentSelected != null && documentSelected.length > 0);
            });
            _this.selectedDestionationText = ko.computed(function () {
                var dest = _this.selectedDestination();
                if (dest) {
                    return dest.Database + ' on ' + dest.Url;
                }
                else {
                    return 'select destination';
                }
            });
            extensions.install();
            _this.documentId.throttle(250).subscribe(function (search) { return _this.fetchDocSearchResults(search); });
            return _this;
        }
        statusDebugExplainReplication.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            $.when(this.fetchReplicationDestinations())
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        statusDebugExplainReplication.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
        };
        statusDebugExplainReplication.prototype.fetchReplicationDestinations = function () {
            var _this = this;
            return new getReplicationsCommand(this.activeDatabase())
                .execute()
                .done(function (destinations) {
                _this.destinations(destinations.MergedDocument.Destinations);
            });
        };
        statusDebugExplainReplication.prototype.setSelectedDestination = function (destination) {
            this.selectedDestination(destination);
        };
        statusDebugExplainReplication.prototype.selectDocument = function (data) {
            this.documentId(data['@metadata']['@id']);
        };
        statusDebugExplainReplication.prototype.fetchDocSearchResults = function (query) {
            var _this = this;
            if (query.length >= 2) {
                new getDocumentsMetadataByIDPrefixCommand(query, 10, this.activeDatabase())
                    .execute()
                    .done(function (results) {
                    if (_this.documentId() === query) {
                        _this.documentIdSearchResults(results.Results);
                    }
                });
            }
            else if (query.length == 0) {
                this.documentIdSearchResults.removeAll();
            }
        };
        statusDebugExplainReplication.prototype.explain = function () {
            var _this = this;
            eventsCollector.default.reportEvent("replicaton", "explain");
            new explainReplicationCommand(this.activeDatabase(), this.documentId(), this.selectedDestination().Url, this.selectedDestination().Database)
                .execute()
                .done(function (result) { return _this.explanation(result); });
        };
        return statusDebugExplainReplication;
    }(viewModelBase));
    return statusDebugExplainReplication;
});

//# sourceMappingURL=statusDebugExplainReplication.js.map
