var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/database/debug/getDebugMetricsCommand", "viewmodels/viewModelBase"], function (require, exports, getDebugMetricsCommand, viewModelBase) {
    "use strict";
    var statusDebugMetrics = (function (_super) {
        __extends(statusDebugMetrics, _super);
        function statusDebugMetrics() {
            var _this = _super.apply(this, arguments) || this;
            _this.data = ko.observable();
            _this.requestPercentiles = ko.computed(function () {
                if (_this.data()) {
                    return _this.extractPercentiles(_this.data().RequestsDuration.Percentiles);
                }
                return null;
            });
            _this.staleIndexMapsPercentiles = ko.computed(function () {
                if (_this.data()) {
                    return _this.extractPercentiles(_this.data().StaleIndexMaps.Percentiles);
                }
                return null;
            });
            _this.staleIndexReducesPercentiles = ko.computed(function () {
                if (_this.data()) {
                    return _this.extractPercentiles(_this.data().StaleIndexReduces.Percentiles);
                }
                return null;
            });
            _this.gauges = ko.computed(function () {
                if (_this.data()) {
                    return $.map(_this.data().Gauges, function (v, k) {
                        return {
                            key: k,
                            values: $.map(v, function (innerValue, innerKey) {
                                return {
                                    key: innerKey,
                                    value: innerValue
                                };
                            })
                        };
                    });
                }
                return null;
            });
            _this.replicationDestinations = ko.computed(function () {
                if (_this.data()) {
                    // sample destinations using ReplicationDurationHistogram
                    return $.map(_this.data().ReplicationBatchSizeHistogram, function (v, key) { return key; });
                }
                return null;
            });
            return _this;
        }
        statusDebugMetrics.prototype.extractPercentiles = function (input) {
            var result = [];
            for (var prop in input) {
                var v = input[prop];
                result.push({ "key": prop, "value": v });
            }
            return result;
        };
        statusDebugMetrics.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchStatusDebugMetrics(); });
            return this.fetchStatusDebugMetrics();
        };
        statusDebugMetrics.prototype.fetchStatusDebugMetrics = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getDebugMetricsCommand(db)
                    .execute()
                    .done(function (results) { return _this.data(results); });
            }
            return null;
        };
        return statusDebugMetrics;
    }(viewModelBase));
    return statusDebugMetrics;
});

//# sourceMappingURL=statusDebugMetrics.js.map
