var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/database/debug/getStatusDebugQueriesCommand", "commands/database/query/getKillQueryCommand", "viewmodels/viewModelBase", "common/bindingHelpers/aceEditorBindingHandler", "models/database/debug/statusDebugQueriesGroup", "models/database/debug/statusDebugQueriesQuery", "common/bindingHelpers/autoRefreshBindingHandler", "common/eventsCollector"], function (require, exports, getStatusDebugQueriesCommand, getKillQueryCommand, viewModelBase, aceEditorBindingHandler, statusDebugQueriesGroup, statusDebugQueriesQuery, autoRefreshBindingHandler, eventsCollector) {
    "use strict";
    var statusDebugQueries = (function (_super) {
        __extends(statusDebugQueries, _super);
        function statusDebugQueries() {
            var _this = _super.call(this) || this;
            _this.data = ko.observableArray();
            autoRefreshBindingHandler.install();
            aceEditorBindingHandler.install();
            return _this;
        }
        statusDebugQueries.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchCurrentQueries(); });
            return this.fetchCurrentQueries();
        };
        statusDebugQueries.prototype.fetchCurrentQueries = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugQueriesCommand(db)
                    .execute()
                    .done(function (results) { return _this.onResultsLoaded(results); });
            }
            return null;
        };
        statusDebugQueries.prototype.onResultsLoaded = function (results) {
            var _this = this;
            var currentGroups = $.map(this.data(), function (group) { return group.indexName; });
            $.map(results, function (dtoGroup) {
                if (dtoGroup.Queries.length > 0) {
                    var foundGroup = _this.data().find(function (item) { return item.indexName === dtoGroup.IndexName; });
                    if (foundGroup) {
                        _.pull(currentGroups, dtoGroup.IndexName);
                    }
                    else {
                        foundGroup = new statusDebugQueriesGroup(dtoGroup);
                        _this.data.push(foundGroup);
                    }
                    _this.updateGroup(foundGroup, dtoGroup);
                }
            });
            // remove empty and unused groups
            currentGroups.forEach(function (group) {
                var foundGroup = _this.data().find(function (item) { return item.indexName === group; });
                if (foundGroup) {
                    _this.data.remove(foundGroup);
                }
            });
        };
        statusDebugQueries.prototype.updateGroup = function (group, dtoGroup) {
            var currentQueryIds = $.map(group.queries(), function (query) { return query.queryId; });
            $.map(dtoGroup.Queries, function (dtoQuery) {
                var foundQuery = group.queries().find(function (item) { return item.queryId === dtoQuery.QueryId; });
                if (foundQuery) {
                    _.pull(currentQueryIds, foundQuery.queryId);
                    foundQuery.duration(dtoQuery.Duration);
                }
                else {
                    group.queries.push(new statusDebugQueriesQuery(dtoQuery));
                }
            });
            // remove unused queries
            currentQueryIds.forEach(function (query) {
                var foundQuery = group.queries().find(function (item) { return item.queryId === query; });
                if (foundQuery) {
                    group.queries.remove(foundQuery);
                }
            });
        };
        statusDebugQueries.prototype.killQuery = function (queryId) {
            var _this = this;
            eventsCollector.default.reportEvent("query", "kill");
            new getKillQueryCommand(this.activeDatabase(), queryId)
                .execute()
                .done(function () {
                // find and delete query from model
                _this.data().forEach(function (group) {
                    var foundQuery = group.queries().find(function (q) { return q.queryId === queryId; });
                    if (foundQuery) {
                        group.queries.remove(foundQuery);
                    }
                });
            });
        };
        return statusDebugQueries;
    }(viewModelBase));
    return statusDebugQueries;
});

//# sourceMappingURL=statusDebugQueries.js.map
