var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/database/debug/getStatusDebugSqlReplicationCommand", "viewmodels/viewModelBase", "common/sqlReplicationStats"], function (require, exports, getStatusDebugSqlReplicationCommand, viewModelBase, sqlReplicationStats) {
    "use strict";
    var statusDebugSqlReplication = (function (_super) {
        __extends(statusDebugSqlReplication, _super);
        function statusDebugSqlReplication() {
            var _this = _super.call(this) || this;
            _this.data = ko.observable();
            return _this;
        }
        statusDebugSqlReplication.prototype.activate = function (args) {
            var _this = this;
            var widthUnit = 8;
            this.columnWidths = [
                ko.observable(widthUnit),
                ko.observable(12),
                ko.observable(widthUnit),
                ko.observable(widthUnit),
                ko.observable(widthUnit),
                ko.observable(widthUnit),
                ko.observable(10),
                ko.observable(6),
                ko.observable(18),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(4 * 6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6)
            ];
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchSqlReplicationStats(); });
            this.registerColumnResizing();
            return this.fetchSqlReplicationStats();
        };
        statusDebugSqlReplication.prototype.deactivate = function () {
            this.unregisterColumnResizing();
        };
        statusDebugSqlReplication.prototype.fetchSqlReplicationStats = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugSqlReplicationCommand(db)
                    .execute()
                    .done(function (results) {
                    _this.data(results.map(function (x) { return new sqlReplicationStats(x); }));
                });
            }
            return null;
        };
        statusDebugSqlReplication.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(document).on("mousedown.logTableColumnResize", ".column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = _this.columnWidths[columnIndex]();
                startX = e.pageX;
                resizingColumn = true;
            });
            $(document).on("mouseup.logTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.logTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var elemFontSize = parseInt(window.getComputedStyle(e.toElement, null).fontSize, 10);
                    var targetColumnSize = startingWidth + (e.pageX - startX) / elemFontSize;
                    _this.columnWidths[columnIndex](targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        statusDebugSqlReplication.prototype.unregisterColumnResizing = function () {
            $(document).off("mousedown.logTableColumnResize");
            $(document).off("mouseup.logTableColumnResize");
            $(document).off("mousemove.logTableColumnResize");
        };
        return statusDebugSqlReplication;
    }(viewModelBase));
    return statusDebugSqlReplication;
});

//# sourceMappingURL=statusDebugSqlReplication.js.map
