var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "durandal/app", "viewmodels/database/status/indexing/tempStatDialog", "commands/database/debug/getIOMetricsCommand", "common/fileDownloader", "d3"], function (require, exports, viewModelBase, app, tempStatDialog, getIOMetricsCommand, fileDownloader, d3) {
    "use strict";
    var ioStats = (function (_super) {
        __extends(ioStats, _super);
        function ioStats() {
            var _this = _super.apply(this, arguments) || this;
            _this.isoParser = d3.time.format.iso;
            _this.xTickFormat = d3.time.format("%H:%M:%S");
            return _this;
        }
        ioStats.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            return new getIOMetricsCommand(this.activeDatabase())
                .execute()
                .done(function (result) { return _this.data = result; });
        };
        ioStats.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.svg = d3.select("#ioStatsGraph");
        };
        ioStats.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            this.draw();
        };
        ioStats.prototype.draw = function () {
            if (!this.data || this.data.Environments.length === 0) {
                return;
            }
            this.ensurePathsEndsWithSlash();
            this.commonPathsPrefix = this.findCommonPathPrefix();
            var _a = this.findTimeRanges(), minTime = _a[0], maxTime = _a[1];
            var files = this.findFileNames();
            this.drawContainer(minTime, maxTime, files);
            this.fillGraph();
        };
        ioStats.prototype.ensurePathsEndsWithSlash = function () {
            this.data.Environments.forEach(function (env) {
                if (!env.Path.endsWith("\\")) {
                    env.Path += "\\";
                }
            });
        };
        ioStats.prototype.findFileNames = function () {
            var _this = this;
            var result = [];
            this.data.Environments.forEach(function (env) {
                env.Files.forEach(function (file) {
                    result.push(_this.normalizeFileName(env.Path, file.File));
                });
            });
            return result;
        };
        ioStats.prototype.normalizeFileName = function (path, fileName) {
            if (fileName.endsWith(".journal")) {
                fileName = "*.journal";
            }
            return path.substring(this.commonPathsPrefix.length) + fileName;
        };
        ioStats.prototype.findCommonPathPrefix = function () {
            var paths = this.data.Environments.map(function (env) { return env.Path; });
            return this.findPrefix(paths);
        };
        ioStats.prototype.findPrefix = function (strings) {
            if (!strings.length) {
                return "";
            }
            var sorted = strings.slice(0).sort();
            var string1 = sorted[0];
            var string2 = sorted[sorted.length - 1];
            var i = 0;
            var l = Math.min(string1.length, string2.length);
            while (i < l && string1[i] === string2[i]) {
                i++;
            }
            return string1.slice(0, i);
        };
        ioStats.prototype.findTimeRanges = function () {
            var _this = this;
            var minDate = null;
            var maxDateAsNumber = null;
            this.data.Environments.forEach(function (env) {
                env.Files.forEach(function (file) {
                    file.History.forEach(function (historyItem) {
                        if (!minDate || minDate > historyItem.Start) {
                            minDate = historyItem.Start;
                        }
                        var itemEndDate = _this.isoParser.parse(historyItem.End).getTime();
                        if (!maxDateAsNumber || maxDateAsNumber < itemEndDate) {
                            maxDateAsNumber = itemEndDate;
                        }
                    });
                    file.Recent.forEach(function (recentItem) {
                        if (!minDate || minDate > recentItem.Start) {
                            minDate = recentItem.Start;
                        }
                        var itemEndDate = _this.isoParser.parse(recentItem.Start).getTime() + recentItem.Duration;
                        if (!maxDateAsNumber || maxDateAsNumber < itemEndDate) {
                            maxDateAsNumber = itemEndDate;
                        }
                    });
                });
            });
            return [minDate ? this.isoParser.parse(minDate) : null, new Date(maxDateAsNumber)];
        };
        ioStats.extent = function (timeExtent) {
            return timeExtent / 1000.0 * ioStats.pixelsPerSecond;
        };
        ioStats.prototype.drawContainer = function (minTime, maxTime, files) {
            var _this = this;
            var timeExtent = maxTime.getTime() - minTime.getTime();
            var totalWidth = ioStats.extent(timeExtent);
            this.xScale = d3.time.scale()
                .range([0, totalWidth])
                .domain([minTime, maxTime]);
            this.svg.append("g")
                .attr("class", "x axis");
            var ticks = d3.scale.linear()
                .domain([0, timeExtent])
                .ticks(Math.ceil(timeExtent / 10000)).map(function (y) { return _this.xScale.invert(y); });
            this.xAxis = d3.svg.axis()
                .scale(this.xScale)
                .orient("top")
                .tickValues(ticks)
                .tickSize(10)
                .tickFormat(this.xTickFormat);
            var totalHeight = files.length * (ioStats.barHeight + ioStats.betweenGroupPadding);
            this.svg.select(".x.axis")
                .attr("transform", "translate(" + (ioStats.leftPadding + ioStats.legendWidth) + "," + ioStats.topPadding + ")")
                .call(this.xAxis);
            this.yScale = d3.scale.ordinal()
                .domain(files)
                .rangeBands([0, totalHeight]);
            this.yAxis = d3.svg.axis()
                .scale(this.yScale)
                .orient("left");
            this.svg.append('g')
                .attr('class', 'y axis map')
                .attr("transform", "translate(" + (ioStats.leftPadding + ioStats.legendWidth) + "," + ioStats.topPadding + ")");
            this.svg.select(".y.axis.map")
                .call(this.yAxis);
            $("#ioStatsGraph")
                .width(ioStats.leftPadding + ioStats.legendWidth + totalWidth + 20)
                .height(totalHeight + ioStats.topPadding + 20);
            this.graphData = this.svg.append("g")
                .attr('class', 'graph_data')
                .attr('transform', "translate(" + (ioStats.leftPadding + ioStats.legendWidth) + "," + (ioStats.topPadding + ioStats.betweenGroupPadding / 2) + ")");
        };
        ioStats.prototype.fillGraph = function () {
            var _this = this;
            this.data.Environments.forEach(function (env) {
                var path = env.Path;
                env.Files.forEach(function (fileStats) {
                    var file = fileStats.File;
                    var normalizedFileName = _this.normalizeFileName(path, file);
                    var group = _this.graphData.append('g')
                        .attr('class', 'per_file_group')
                        .attr('data-filename', normalizedFileName)
                        .attr('transform', "translate(0," + _this.yScale(normalizedFileName) + ")");
                    fileStats.Recent.forEach(function (recentItem) {
                        recentItem.file = file; // add information about file, since we join journals into single line
                        group.append('rect')
                            .attr('class', 'type_' + recentItem.Type)
                            .attr('x', _this.xScale(_this.isoParser.parse(recentItem.Start)))
                            .attr('y', ioStats.calcOffset(recentItem.Type))
                            .attr('width', Math.max(1, ioStats.extent(recentItem.Duration)))
                            .attr('height', ioStats.calcBarHeight(recentItem.Type))
                            .datum(recentItem)
                            .on('click', function (data) { return _this.showDetails(data); });
                    });
                    fileStats.History.forEach(function (history) {
                        history.file = file; // add information about file, since we join journals into single line
                        group.append('rect')
                            .attr('class', 'type_bg_' + history.Type)
                            .attr('x', _this.xScale(_this.isoParser.parse(history.Start)))
                            .attr('y', ioStats.calcOffset(history.Type))
                            .attr('width', Math.max(1, ioStats.extent(history.Duration)))
                            .attr('height', ioStats.calcBarHeight(history.Type));
                        group.append('rect')
                            .attr('class', 'type_fg_' + history.Type)
                            .attr('x', _this.xScale(_this.isoParser.parse(history.Start)))
                            .attr('y', ioStats.calcOffset(history.Type))
                            .attr('width', Math.max(1, ioStats.extent(history.ActiveDuration)))
                            .attr('height', ioStats.calcBarHeight(history.Type))
                            .datum(history)
                            .on('click', function (data) { return _this.showDetails(data); });
                    });
                });
            });
        };
        ioStats.prototype.showDetails = function (op) {
            var dialog = new tempStatDialog(op);
            app.showBootstrapDialog(dialog);
        };
        ioStats.calcBarHeight = function (type) {
            return type === "JournalWrite" ? ioStats.barHeight : ioStats.barHeight / 2;
        };
        ioStats.calcOffset = function (type) {
            return type === "DataSync" ? ioStats.barHeight / 2 : 0;
        };
        ioStats.prototype.exportAsJson = function () {
            fileDownloader.downloadAsJson({
                data: this.data
            }, "ioStats.json", "perf");
        };
        ioStats.prototype.fileSelected = function () {
            var fileInput = document.querySelector("#importFilePicker");
            var self = this;
            if (fileInput.files.length === 0) {
                return;
            }
            var file = fileInput.files[0];
            var reader = new FileReader();
            reader.onload = function () {
                // ReSharper disable once SuspiciousThisUsage
                self.dataImported(this.result);
            };
            reader.onerror = function (error) {
                alert(error);
            };
            reader.readAsText(file);
        };
        ioStats.prototype.dataImported = function (result) {
            var json = JSON.parse(result);
            this.data = json.data;
            $("#ioStatsGraph").empty();
            this.draw();
        };
        return ioStats;
    }(viewModelBase));
    ioStats.barHeight = 16;
    ioStats.pixelsPerSecond = 8;
    ioStats.betweenGroupPadding = 10;
    ioStats.leftPadding = 20;
    ioStats.topPadding = 30;
    ioStats.legendWidth = 500;
    return ioStats;
});

//# sourceMappingURL=ioStats.js.map
