var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/database/debug/getLogsCommand", "viewmodels/viewModelBase", "moment", "models/database/documents/document", "common/bindingHelpers/autoRefreshBindingHandler", "models/database/debug/logEntry", "common/tableNavigationTrait", "common/eventsCollector"], function (require, exports, getLogsCommand, viewModelBase, moment, document, autoRefreshBindingHandler, logEntry, tableNavigationTrait, eventsCollector) {
    "use strict";
    var logs = (function (_super) {
        __extends(logs, _super);
        function logs() {
            var _this = _super.call(this) || this;
            _this.allLogs = ko.observableArray();
            _this.filterLevel = ko.observable("All");
            _this.selectedLog = ko.observable();
            _this.searchText = ko.observable("");
            _this.now = ko.observable();
            _this.updateNowTimeoutHandle = 0;
            _this.filteredLoggers = ko.observableArray();
            _this.sortColumn = ko.observable("timeStamp");
            _this.sortAsc = ko.observable(true);
            _this.showLogDetails = ko.observable(false);
            autoRefreshBindingHandler.install();
            _this.debugLogCount = ko.pureComputed(function () { return _this.allLogs().filter(function (l) { return l.level() === "Debug"; }).length; });
            _this.infoLogCount = ko.pureComputed(function () { return _this.allLogs().filter(function (l) { return l.level() === "Info"; }).length; });
            _this.warningLogCount = ko.pureComputed(function () { return _this.allLogs().filter(function (l) { return l.level() === "Warn"; }).length; });
            _this.errorLogCount = ko.pureComputed(function () { return _this.allLogs().filter(function (l) { return l.level() === "Error"; }).length; });
            _this.fatalLogCount = ko.pureComputed(function () { return _this.allLogs().filter(function (l) { return l.level() === "Fatal"; }).length; });
            _this.searchTextThrottled = _this.searchText.throttle(400);
            _this.activeDatabase.subscribe(function () { return _this.fetchLogs(); });
            _this.updateCurrentNowTime();
            _this.sortColumn.subscribe(function () { return _this.sortInPlace(); });
            _this.sortAsc.subscribe(function () { return _this.sortInPlace(); });
            _this.tableNavigation = new tableNavigationTrait("#logRecords", _this.selectedLog, _this.allLogs, function (i) { return "#logRecords > div:nth-child(" + (i + 1) + ")"; });
            return _this;
        }
        logs.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.columnWidths = [
                ko.observable(100),
                ko.observable(265),
                ko.observable(300),
                ko.observable(200),
                ko.observable(360)
            ];
            this.registerColumnResizing();
            this.updateHelpLink('3Z9IJS');
            return this.fetchLogs();
        };
        logs.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.showLogDetails.subscribe(function () {
                $(".logRecords").toggleClass("logRecords-small");
            });
            $("#logRecordsContainer").width();
            var widthUnit = 0.08;
            this.columnWidths[0](100 * widthUnit);
            this.columnWidths[1](100 * widthUnit);
            this.columnWidths[2](100 * widthUnit * 6);
            this.columnWidths[3](100 * widthUnit * 2);
            this.columnWidths[4](100 * widthUnit * 2);
        };
        logs.prototype.deactivate = function () {
            clearTimeout(this.updateNowTimeoutHandle);
            this.unregisterColumnResizing();
        };
        logs.prototype.fetchLogs = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var deferred = $.Deferred();
                new getLogsCommand(db)
                    .execute()
                    .done(function (results) {
                    _this.processLogResults(results);
                    deferred.resolve(results);
                });
                return deferred;
            }
            return null;
        };
        logs.prototype.processLogResults = function (results) {
            var _this = this;
            var mappedResults = results.map(function (r) {
                var mapped = new logEntry(r, _this.now);
                mapped.isVisible = ko.pureComputed(function () {
                    var matchesSearch = _this.matchesFilterAndSearch(mapped);
                    var matchesFilters = _.includes(_this.filteredLoggers(), mapped.loggerName());
                    return matchesSearch && !matchesFilters;
                });
                return mapped;
            });
            var sortedResults = this.sortResults(mappedResults.reverse());
            var allLogsRaw = this.allLogs();
            if (sortedResults.length === allLogsRaw.length) {
                for (var i = 0; i < sortedResults.length; i++) {
                    allLogsRaw[i].copyFrom(sortedResults[i]);
                }
            }
            else {
                this.allLogs(sortedResults);
            }
        };
        logs.prototype.sortInPlace = function () {
            var dataToSort = this.sortResults(this.allLogs());
            this.allLogs(dataToSort);
        };
        logs.prototype.sortResults = function (dataToSort) {
            var column = this.sortColumn();
            var asc = this.sortAsc();
            var test = asc ? (function (l, r) { return l < r; }) : (function (l, r) { return l > r; });
            var sortFunc = function (left, right) {
                if (left[column]() === right[column]()) {
                    return 0;
                }
                return test(left[column](), right[column]()) ? 1 : -1;
            };
            return dataToSort.sort(sortFunc);
        };
        logs.prototype.matchesFilterAndSearch = function (log) {
            var searchTextThrottled = this.searchTextThrottled().toLowerCase();
            var filterLevel = this.filterLevel();
            var matchesLogLevel = filterLevel === "All" || log.level() === filterLevel;
            var matchesSearchText = !searchTextThrottled ||
                (log.message() && log.message().toLowerCase().indexOf(searchTextThrottled) >= 0) ||
                (log.exception() && log.exception().toLowerCase().indexOf(searchTextThrottled) >= 0);
            return matchesLogLevel && matchesSearchText;
        };
        logs.prototype.selectLog = function (log) {
            this.selectedLog(log);
            this.showLogDetails(true);
            $(".logRecords").addClass("logRecords-small");
        };
        logs.prototype.unSelectLog = function (log) {
            this.selectedLog(null);
            this.showLogDetails(false);
        };
        logs.prototype.setFilterAll = function () {
            this.filterLevel("All");
        };
        logs.prototype.setFilterDebug = function () {
            this.filterLevel("Debug");
        };
        logs.prototype.setFilterInfo = function () {
            this.filterLevel("Info");
        };
        logs.prototype.setFilterWarning = function () {
            this.filterLevel("Warn");
        };
        logs.prototype.setFilterError = function () {
            this.filterLevel("Error");
        };
        logs.prototype.setFilterFatal = function () {
            this.filterLevel("Fatal");
        };
        logs.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 60000);
        };
        logs.prototype.hideLogType = function (log) {
            eventsCollector.default.reportEvent("logs", "toggle-type", "hide");
            if (!_.includes(this.filteredLoggers(), log.loggerName())) {
                this.filteredLoggers.push(log.loggerName());
            }
        };
        logs.prototype.unHidelogType = function (loggerName) {
            eventsCollector.default.reportEvent("logs", "toggle-type", "show");
            this.filteredLoggers.remove(loggerName);
        };
        logs.prototype.sortBy = function (columnName) {
            if (this.sortColumn() === columnName) {
                this.sortAsc(!this.sortAsc());
            }
            else {
                this.sortColumn(columnName);
                this.sortAsc(true);
            }
        };
        logs.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(document).on("mousedown.logTableColumnResize", ".column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = _this.columnWidths[columnIndex]();
                startX = e.pageX;
                resizingColumn = true;
            });
            $(document).on("mouseup.logTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.logTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var logsRecordsContainerWidth = $("#logRecordsContainer").width();
                    var targetColumnSize = startingWidth + 100 * (e.pageX - startX) / logsRecordsContainerWidth;
                    _this.columnWidths[columnIndex](targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        logs.prototype.unregisterColumnResizing = function () {
            $(document).off("mousedown.logTableColumnResize");
            $(document).off("mouseup.logTableColumnResize");
            $(document).off("mousemove.logTableColumnResize");
        };
        return logs;
    }(viewModelBase));
    return logs;
});

//# sourceMappingURL=logs.js.map
