var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/database/debug/getRequestTracingCommand", "common/bindingHelpers/autoRefreshBindingHandler", "common/tableNavigationTrait"], function (require, exports, viewModelBase, getRequestTracingCommand, autoRefreshBindingHandler, tableNavigationTrait) {
    "use strict";
    var requestTracing = (function (_super) {
        __extends(requestTracing, _super);
        function requestTracing() {
            var _this = _super.call(this) || this;
            _this.allEntries = ko.observableArray();
            _this.statusFilter = ko.observable("All");
            _this.selectedEntry = ko.observable();
            autoRefreshBindingHandler.install();
            _this.failedCount = ko.computed(function () { return _this.allEntries().filter(function (l) { return l.StatusCode >= 400; }).length; });
            _this.successCount = ko.computed(function () { return _this.allEntries().filter(function (l) { return l.StatusCode < 400; }).length; });
            _this.activeDatabase.subscribe(function () { return _this.fetchRequestTracing(); });
            _this.tableNavigation = new tableNavigationTrait("#requestTracingTableContainer", _this.selectedEntry, _this.allEntries, function (i) { return "#requestTracingTableContainer table tbody tr:nth-child(" + (i + 1) + ")"; });
            return _this;
        }
        requestTracing.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            return this.fetchRequestTracing();
        };
        requestTracing.prototype.fetchRequestTracing = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getRequestTracingCommand(db)
                    .execute()
                    .done(function (results) { return _this.processResults(results); });
            }
            return null;
        };
        requestTracing.prototype.processResults = function (results) {
            var _this = this;
            results.forEach(function (r) {
                r['IsVisible'] = ko.computed(function () { return _this.matchesFilter(r); });
            });
            this.allEntries(results.reverse());
        };
        requestTracing.prototype.matchesFilter = function (entry) {
            var statusFilter = this.statusFilter();
            var matchesStatusFilter = true;
            if (statusFilter === "Success" && entry.StatusCode >= 400) {
                matchesStatusFilter = false;
            }
            else if (statusFilter === "Failed" && entry.StatusCode < 400) {
                matchesStatusFilter = false;
            }
            return matchesStatusFilter;
        };
        requestTracing.prototype.selectEntry = function (entry) {
            this.selectedEntry(entry);
        };
        requestTracing.prototype.showContextMenu = function () {
            //alert("this");
        };
        requestTracing.prototype.setStatusAll = function () {
            this.statusFilter("All");
        };
        requestTracing.prototype.setStatusSuccess = function () {
            this.statusFilter("Success");
        };
        requestTracing.prototype.setStatusFailed = function () {
            this.statusFilter("Failed");
        };
        return requestTracing;
    }(viewModelBase));
    return requestTracing;
});

//# sourceMappingURL=requestTracing.js.map
