var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/operations/killOperationCommand", "moment", "models/database/documents/document", "models/database/debug/runningTask", "common/bindingHelpers/autoRefreshBindingHandler", "common/messagePublisher", "common/tableNavigationTrait", "common/eventsCollector"], function (require, exports, viewModelBase, killOperationComamnd, moment, document, runningTask, autoRefreshBindingHandler, messagePublisher, tableNavigationTrait, eventsCollector) {
    "use strict";
    var runningTasks = (function (_super) {
        __extends(runningTasks, _super);
        function runningTasks() {
            var _this = _super.call(this) || this;
            _this.allTasks = ko.observableArray();
            _this.filterType = ko.observable(null);
            _this.selectedTask = ko.observable();
            _this.noExceptionText = ko.computed(function () { return !!_this.selectedTask() && !!_this.selectedTask().exceptionText; });
            _this.taskTypes = ko.observableArray([]);
            _this.searchText = ko.observable("");
            _this.now = ko.observable();
            _this.updateNowTimeoutHandle = 0;
            _this.sortColumn = ko.observable("logged");
            _this.sortAsc = ko.observable(true);
            autoRefreshBindingHandler.install();
            _this.searchTextThrottled = _this.searchText.throttle(200);
            _this.activeDatabase.subscribe(function () { return _this.fetchTasks(); });
            _this.updateCurrentNowTime();
            _this.filteredAndSortedTasks = ko.computed(function () {
                var tasks = _this.allTasks().filter(function (t) { return _this.matchesFilterAndSearch(t); });
                var column = _this.sortColumn();
                var asc = _this.sortAsc();
                var sortFunc = function (left, right) {
                    if (left[column] === right[column]) {
                        return 0;
                    }
                    var test = asc ? (function (l, r) { return l < r; }) : (function (l, r) { return l > r; });
                    return test(left[column], right[column]) ? 1 : -1;
                };
                return tasks.sort(sortFunc);
            });
            _this.tableNavigation = new tableNavigationTrait("#runningTasksTableContainer", _this.selectedTask, _this.filteredAndSortedTasks, function (i) { return "#runningTasksItemsContainer > div:nth-child(" + (i + 1) + ")"; });
            return _this;
        }
        runningTasks.prototype.recalculateTaskTypes = function () {
            var types = [];
            var allTasks = this.allTasks();
            allTasks.forEach(function (task) {
                var type = task.taskType;
                var existingItem = types.find(function (t) { return t.name === type; });
                if (existingItem) {
                    existingItem.count++;
                }
                else {
                    types.push({ name: type, count: 1 });
                }
            });
            this.taskTypes(types);
        };
        runningTasks.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.columnWidths = [
                ko.observable(100),
                ko.observable(300),
                ko.observable(460),
                ko.observable(265),
                ko.observable(100)
            ];
            this.registerColumnResizing();
            this.updateHelpLink('2KH22A');
            return this.fetchTasks();
        };
        runningTasks.prototype.deactivate = function () {
            clearTimeout(this.updateNowTimeoutHandle);
            this.unregisterColumnResizing();
        };
        runningTasks.prototype.fetchTasks = function () {
            var db = this.activeDatabase();
            if (db) {
            }
            return null;
        };
        runningTasks.prototype.processRunningTasksResults = function (results) {
            this.allTasks(results.reverse().map(function (r) { return new runningTask(r); }));
            this.recalculateTaskTypes();
        };
        runningTasks.prototype.matchesFilterAndSearch = function (task) {
            var searchTextThrottled = this.searchTextThrottled().toLowerCase();
            var filterType = this.filterType();
            var matchesLogLevel = filterType === null || task.taskType === filterType;
            var matchesSearchText = !searchTextThrottled ||
                (task.description && task.description.toLowerCase().indexOf(searchTextThrottled) >= 0) ||
                (task.exceptionText && task.exceptionText.toLowerCase().indexOf(searchTextThrottled) >= 0);
            return matchesLogLevel && matchesSearchText;
        };
        runningTasks.prototype.selectTask = function (task) {
            this.selectedTask(task);
        };
        runningTasks.prototype.taskKill = function (task) {
            var _this = this;
            eventsCollector.default.reportEvent("task", "kill");
            new killOperationComamnd(this.activeDatabase(), task.id).execute()
                .done(function () {
                messagePublisher.reportSuccess("Send kill task request");
            })
                .always(function () { return setTimeout(function () {
                _this.selectedTask(null);
                _this.fetchTasks();
            }, 1000); });
        };
        runningTasks.prototype.setFilterTypeAll = function () {
            this.filterType(null);
        };
        runningTasks.prototype.setFilterType = function (value) {
            this.filterType(value);
        };
        runningTasks.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 60000);
        };
        runningTasks.prototype.sortBy = function (columnName, logs, event) {
            if (this.sortColumn() === columnName) {
                this.sortAsc(!this.sortAsc());
            }
            else {
                this.sortColumn(columnName);
                this.sortAsc(true);
            }
        };
        runningTasks.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(document).on("mousedown.tasksTableColumnResize", ".column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = _this.columnWidths[columnIndex]();
                startX = e.pageX;
                resizingColumn = true;
            });
            $(document).on("mouseup.tasksTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.tasksTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var targetColumnSize = startingWidth + e.pageX - startX;
                    _this.columnWidths[columnIndex](targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        runningTasks.prototype.unregisterColumnResizing = function () {
            $(document).off("mousedown.tasksTableColumnResize");
            $(document).off("mouseup.tasksTableColumnResize");
            $(document).off("mousemove.tasksTableColumnResize");
        };
        return runningTasks;
    }(viewModelBase));
    return runningTasks;
});

//# sourceMappingURL=runningTasks.js.map
