var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/dialogViewModelBase", "commands/database/sqlReplication/simulateSqlReplicationCommand", "commands/database/documents/getDocumentsMetadataByIDPrefixCommand", "plugins/dialog", "models/database/documents/collection", "models/database/sqlReplication/sqlReplicationSimulatedCommand"], function (require, exports, dialogViewModelBase, simulateSqlReplicationCommand, getDocumentsMetadataByIDPrefixCommand, dialog, collection, sqlReplicationSimulatedCommand) {
    "use strict";
    var sqlReplicationSimulationDialog = (function (_super) {
        __extends(sqlReplicationSimulationDialog, _super);
        function sqlReplicationSimulationDialog(db, simulatedSqlReplication, documentId) {
            var _this = _super.call(this) || this;
            _this.db = db;
            _this.simulatedSqlReplication = simulatedSqlReplication;
            _this.documentId = documentId;
            _this.simulationResults = ko.observableArray([]);
            _this.rolledBackTransactionPassed = ko.observable(false);
            _this.documentAutocompletes = ko.observableArray();
            _this.lastSearchedDocumentID = ko.observable("");
            _this.rolledbackTransactionPerformed = ko.observable(false);
            _this.lastAlert = ko.observable("");
            _this.waitingToAnswer = ko.observable(false);
            _this.documentId.throttle(250).subscribe(function (search) { return _this.fetchDocumentIdAutocompletes(search); });
            _this.isAutoCompleteVisible = ko.computed(function () {
                return _this.lastSearchedDocumentID() !== _this.documentId() &&
                    (_this.documentAutocompletes().length > 1 || _this.documentAutocompletes().length == 1 && _this.documentId() !== _this.documentAutocompletes()[0]);
            });
            return _this;
        }
        sqlReplicationSimulationDialog.prototype.toggleCommandParamView = function (command) {
            command.showParamsValues.toggle();
        };
        sqlReplicationSimulationDialog.prototype.getResults = function (performRolledbackTransaction) {
            var _this = this;
            this.lastSearchedDocumentID(this.documentId());
            this.waitingToAnswer(true);
            new simulateSqlReplicationCommand(this.db, this.simulatedSqlReplication, this.documentId(), performRolledbackTransaction)
                .execute()
                .done(function (result) {
                if (!!result.Results) {
                    _this.simulationResults(result.Results.map(function (x) { return x.Commands; }).reduce(function (x, y) { return x.concat(y); }).map(function (x) { return new sqlReplicationSimulatedCommand(!performRolledbackTransaction, x); }));
                }
                else {
                    _this.simulationResults([]);
                }
                /* TODO
                this.rolledBackTransactionPassed(!result.LastAlert);

                if (!!result.LastAlert) {
                    this.lastAlert(result.LastAlert.Exception);
                } else {
                    this.lastAlert("");
                }*/
                _this.rolledbackTransactionPerformed(performRolledbackTransaction);
            })
                .fail(function () {
                _this.simulationResults([]);
                _this.rolledBackTransactionPassed(false);
            })
                .always(function () { return _this.waitingToAnswer(false); });
        };
        // overrid dialogViewModelBase shortcuts behavior
        sqlReplicationSimulationDialog.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#docIdInput").focus();
        };
        sqlReplicationSimulationDialog.prototype.fetchDocumentIdAutocompletes = function (query) {
            var _this = this;
            if (query.length >= 2) {
                new getDocumentsMetadataByIDPrefixCommand(query, 10, this.db)
                    .execute()
                    .done(function (results) {
                    if (_this.documentId() === query) {
                        if (results.Results.length == 1 && _this.documentId() == results.Results[0]) {
                            _this.documentAutocompletes.removeAll();
                            return;
                        }
                        _this.documentAutocompletes(results.Results);
                    }
                });
            }
            else if (query.length == 0) {
                this.documentAutocompletes.removeAll();
            }
        };
        sqlReplicationSimulationDialog.prototype.documentIdSubmitted = function (submittedDocumentId) {
            this.documentId(submittedDocumentId);
            $('#docIdInput').focus();
            this.getResults(false);
        };
        sqlReplicationSimulationDialog.prototype.getDocCssClass = function (doc) {
            return collection.getCollectionCssClass(doc['@metadata']['Raven-Entity-Name'], this.db);
        };
        sqlReplicationSimulationDialog.prototype.keyPressedOnDocumentAutocomplete = function (doc, event) {
            if (event.keyCode == 13 && !!doc) {
                var docId = !!doc['@metadata'] ? doc['@metadata']['@id'] : null;
                if (!!docId) {
                    this.documentId(docId);
                }
            }
        };
        sqlReplicationSimulationDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        return sqlReplicationSimulationDialog;
    }(dialogViewModelBase));
    return sqlReplicationSimulationDialog;
});

//# sourceMappingURL=sqlReplicationSimulationDialog.js.map
