var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "commands/database/index/getIndexesStatsCommand", "common/appUrl", "models/database/stats/statistics"], function (require, exports, viewModelBase, getDatabaseStatsCommand, getIndexesStatsCommand, appUrl, statsModel) {
    "use strict";
    var statistics = (function (_super) {
        __extends(statistics, _super);
        function statistics() {
            var _this = _super.apply(this, arguments) || this;
            _this.stats = ko.observable();
            _this.refreshStatsObservable = ko.observable();
            return _this;
        }
        statistics.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.statsSubscription = this.refreshStatsObservable.throttle(3000).subscribe(function (e) { return _this.fetchStats(); });
            this.fetchStats();
            this.updateHelpLink('H6GYYL');
        };
        statistics.prototype.detached = function () {
            _super.prototype.detached.call(this);
            if (this.statsSubscription != null) {
                this.statsSubscription.dispose();
            }
        };
        statistics.prototype.fetchStats = function () {
            var _this = this;
            var db = this.activeDatabase();
            var dbStatsTask = new getDatabaseStatsCommand(db)
                .execute();
            var indexesStatsTask = new getIndexesStatsCommand(db)
                .execute();
            return $.when(dbStatsTask, indexesStatsTask)
                .done(function (_a, _b) {
                var dbStats = _a[0];
                var indexesStats = _b[0];
                _this.processStatsResults(dbStats, indexesStats);
            });
        };
        statistics.prototype.createNotifications = function () {
            var _this = this;
            //TODO: don't bother about this now
            return [
                this.changesContext.currentResourceChangesApi().watchAllDocs(function (e) { return _this.refreshStatsObservable(new Date().getTime()); })
            ];
        };
        statistics.prototype.processStatsResults = function (dbStats, indexesStats) {
            this.stats(new statsModel(dbStats, indexesStats));
        };
        statistics.prototype.urlForIndexPerformance = function (indexName) {
            return appUrl.forIndexPerformance(this.activeDatabase(), indexName);
        };
        return statistics;
    }(viewModelBase));
    return statistics;
});

//# sourceMappingURL=statistics.js.map
