var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/router", "viewmodels/viewModelBase", "common/appUrl", "viewmodels/shell/accessHelper"], function (require, exports, durandalRouter, viewModelBase, appUrl, accessHelper) {
    "use strict";
    var status = (function (_super) {
        __extends(status, _super);
        function status() {
            var _this = _super.call(this) || this;
            _this.bundleMap = { replication: "Replication Stats", sqlreplication: "SQL Replication Stats" };
            _this.userDatabasePages = ko.observableArray(status.initialVisibleViews.slice());
            _this.appUrls = appUrl.forCurrentDatabase();
            _this.router = durandalRouter.createChildRouter()
                .map([
                { route: 'databases/status/indexing*details', moduleId: 'viewmodels/database/status/indexing/indexing', title: 'Indexing', nav: true, hash: appUrl.forCurrentDatabase().indexPerformance },
                { route: 'databases/status/requests*details', moduleId: 'viewmodels/database/status/requests/requests', title: 'Request Tracking', nav: true, hash: appUrl.forCurrentDatabase().requestsCount },
                { route: 'databases/status', moduleId: 'viewmodels/database/status/statistics', title: 'Stats', nav: true, hash: appUrl.forCurrentDatabase().status },
                { route: 'databases/status/logs', moduleId: 'viewmodels/database/status/logs', title: 'Logs', nav: true, hash: appUrl.forCurrentDatabase().logs },
                { route: 'databases/status/runningTasks', moduleId: 'viewmodels/database/status/runningTasks', title: 'Running Tasks', nav: true, hash: appUrl.forCurrentDatabase().runningTasks },
                { route: 'databases/status/alerts', moduleId: 'viewmodels/database/status/alerts', title: 'Alerts', nav: true, hash: appUrl.forCurrentDatabase().alerts },
                { route: 'databases/status/replicationStats', moduleId: 'viewmodels/database/status/replicationStats', title: 'Replication Stats', nav: true, hash: appUrl.forCurrentDatabase().replicationStats },
                { route: 'databases/status/sqlReplicationPerfStats', moduleId: 'viewmodels/database/status/sqlReplicationPerfStats', title: 'SQL Replication Stats', nav: true, hash: appUrl.forCurrentDatabase().sqlReplicationPerfStats },
                { route: 'databases/status/userInfo', moduleId: 'viewmodels/database/status/userInfo', title: 'User Info', nav: true, hash: appUrl.forCurrentDatabase().userInfo },
                { route: 'databases/status/debug*details', moduleId: 'viewmodels/database/status/debug/statusDebug', title: 'Debug', nav: true, hash: appUrl.forCurrentDatabase().statusDebug },
                { route: 'databases/status/storage*details', moduleId: 'viewmodels/database/status/storage/statusStorage', title: 'Storage', nav: true, hash: appUrl.forCurrentDatabase().statusStorageOnDisk },
                { route: 'databases/status/infoPackage', moduleId: 'viewmodels/manage/infoPackage', title: 'Gather Debug Info', nav: accessHelper.canExposeConfigOverTheWire(), hash: appUrl.forCurrentDatabase().infoPackage }
            ])
                .buildNavigationModel();
            status.statusRouter = _this.router;
            _this.router.guardRoute = function (instance, instruction) { return _this.getValidRoute(instance, instruction); };
            appUrl.mapUnknownRoutes(_this.router);
            _this.currentRouteTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().find(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
            return _this;
        }
        /**
        * Checks whether the route can be navigated to. Returns true if it can be navigated to, or a redirect URI if it can't be navigated to.
        * This is used for preventing a navigating to system-only pages when the current databagse is non-system, and vice-versa.
        */
        status.prototype.getValidRoute = function (instance, instruction) {
            var db = this.activeDatabase();
            var pathArr = instruction.fragment.split('/');
            var bundelName = pathArr[pathArr.length - 1].toLowerCase();
            var isLegalBundelName = (this.bundleMap[bundelName] != undefined);
            var isBundleExists = this.userDatabasePages.indexOf(this.bundleMap[bundelName]) > -1;
            if (isLegalBundelName && isBundleExists == false) {
                return appUrl.forCurrentDatabase().databaseSettings();
            }
            return true;
        };
        status.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.userDatabasePages(status.initialVisibleViews.slice());
            var db = this.activeDatabase();
            var bundles = db.activeBundles();
            bundles.forEach(function (bundle) {
                var bundleName = _this.bundleMap[bundle.toLowerCase()];
                if (bundleName != undefined) {
                    _this.userDatabasePages.push(bundleName);
                }
            });
        };
        status.prototype.routeIsVisible = function (route) {
            var bundleTitle = route.title;
            if (this.userDatabasePages.indexOf(bundleTitle) !== -1) {
                // Replication stats and Sql Replication stats are visible only when we're on a user database.
                return true;
            }
            return false;
        };
        return status;
    }(viewModelBase));
    status.initialVisibleViews = ["Stats", "Indexing", 'Request Tracking', 'Logs', 'Running Tasks', 'Alerts', 'Index Errors', 'User Info', 'Map/Reduce Visualizer', 'Debug', 'Storage', 'Gather Debug Info'];
    return status;
});

//# sourceMappingURL=status.js.map
