var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/database/debug/getStorageReportCommand", "commands/database/debug/getEnvironmentStorageReportCommand", "common/notifications/protractedCommandsDetector", "common/generalUtils", "models/database/status/storageReportItem", "d3"], function (require, exports, viewModelBase, getStorageReportCommand, getEnvironmentStorageReportCommand, protractedCommandsDetector, generalUtils, storageReportItem, d3) {
    "use strict";
    var storageReport = (function (_super) {
        __extends(storageReport, _super);
        function storageReport() {
            var _this = _super.call(this) || this;
            _this.rawData = [];
            _this.node = ko.observable();
            _this.transitioning = false;
            _this.showLoader = ko.observable(false);
            _this.bindToCurrentInstance("onClick");
            return _this;
        }
        storageReport.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.initObservables();
            return new getStorageReportCommand(this.activeDatabase())
                .execute()
                .done(function (result) {
                _this.rawData = result;
            });
        };
        storageReport.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            this.processData();
            this.initGraph();
            this.draw(undefined, undefined);
        };
        storageReport.prototype.initObservables = function () {
            var _this = this;
            this.currentPath = ko.pureComputed(function () {
                var node = _this.node();
                var items = [];
                var currentItem = node;
                while (currentItem) {
                    items.unshift(currentItem);
                    currentItem = currentItem.parent;
                }
                return items;
            });
            this.showEntriesColumn = ko.pureComputed(function () {
                var node = _this.node();
                return !!node.internalChildren.find(function (x) { return x.type === "table" || x.type === "tree"; });
            });
            this.showPagesColumn = ko.pureComputed(function () {
                var node = _this.node();
                return !!node.internalChildren.find(function (x) { return x.type === "tree"; });
            });
        };
        storageReport.prototype.processData = function () {
            var _this = this;
            var data = this.rawData;
            var mappedData = data.map(function (x) { return _this.mapReport(x); });
            var totalSize = mappedData.reduce(function (p, c) { return p + c.size; }, 0);
            var item = new storageReportItem("/", "Database", false, totalSize, mappedData);
            this.root = item;
            this.sortBySize(this.root);
            this.node(this.root);
        };
        storageReport.prototype.sortBySize = function (node) {
            var _this = this;
            if (node.internalChildren && node.internalChildren.length) {
                node.internalChildren.forEach(function (x) { return _this.sortBySize(x); });
                node.internalChildren.sort(function (a, b) { return d3.descending(a.size, b.size); });
            }
        };
        storageReport.prototype.mapReport = function (reportItem) {
            var dataFile = this.mapDataFile(reportItem.Report);
            var journals = this.mapJournals(reportItem.Report);
            return new storageReportItem(reportItem.Name, reportItem.Type.toLowerCase(), storageReport.showDisplayReportType(reportItem.Type), dataFile.size + journals.size, [dataFile, journals]);
        };
        storageReport.showDisplayReportType = function (reportType) {
            return reportType !== "Configuration" && reportType !== "Subscriptions";
        };
        storageReport.prototype.mapDataFile = function (report) {
            var dataFile = report.DataFile;
            var storageItem = new storageReportItem("Datafile", "data", false, dataFile.AllocatedSpaceInBytes);
            storageItem.lazyLoadChildren = true;
            return storageItem;
        };
        storageReport.prototype.mapDetailedReport = function (report, d) {
            d.lazyLoadChildren = false;
            var tables = this.mapTables(report.Tables);
            var trees = this.mapTrees(report.Trees, "Trees");
            var freeSpace = new storageReportItem("Free", "free", false, report.DataFile.FreeSpaceInBytes, []);
            d.internalChildren = [tables, trees, freeSpace];
        };
        storageReport.prototype.mapTables = function (tables) {
            var _this = this;
            var mappedTables = tables.map(function (x) { return _this.mapTable(x); });
            return new storageReportItem("Tables", "tables", false, mappedTables.reduce(function (p, c) { return p + c.size; }, 0), mappedTables);
        };
        storageReport.prototype.mapTable = function (table) {
            var structure = this.mapTrees(table.Structure, "Structure");
            var data = new storageReportItem("Table Data", "table_data", false, table.DataSizeInBytes, []);
            var indexes = this.mapTrees(table.Indexes, "Indexes");
            var totalSize = structure.size + table.DataSizeInBytes + indexes.size;
            var tableItem = new storageReportItem(table.Name, "table", true, totalSize, [
                structure,
                data,
                indexes
            ]);
            tableItem.numberOfEntries = table.NumberOfEntries;
            return tableItem;
        };
        storageReport.prototype.mapTrees = function (trees, name) {
            var _this = this;
            return new storageReportItem(name, name.toLowerCase(), false, trees.reduce(function (p, c) { return p + c.AllocatedSpaceInBytes; }, 0), trees.map(function (x) { return _this.mapTree(x); }));
        };
        storageReport.prototype.mapTree = function (tree) {
            var item = new storageReportItem(tree.Name, "tree", true, tree.AllocatedSpaceInBytes, []);
            item.pageCount = tree.PageCount;
            item.numberOfEntries = tree.NumberOfEntries;
            return item;
        };
        storageReport.prototype.mapJournals = function (report) {
            var journals = report.Journals;
            var mappedJournals = journals.map(function (journal) {
                return new storageReportItem("Journal #" + journal.Number, "journal", false, journal.AllocatedSpaceInBytes, []);
            });
            return new storageReportItem("Journals", "journals", false, mappedJournals.reduce(function (p, c) { return p + c.size; }, 0), mappedJournals);
        };
        storageReport.prototype.initGraph = function () {
            this.detectContainerSize();
            this.x = d3.scale.linear().range([0, this.w]);
            this.y = d3.scale.linear().range([0, this.h]);
            this.svg = d3.select("#storage-report-container .chart")
                .append("svg:svg")
                .attr("width", this.w)
                .attr("height", this.h)
                .attr("transform", "translate(.5,.5)");
            this.addHashing();
        };
        storageReport.prototype.detectContainerSize = function () {
            var $chartNode = $("#storage-report-container .chart");
            this.w = $chartNode.width();
            this.h = $chartNode.height();
        };
        storageReport.prototype.addHashing = function () {
            var defs = this.svg.append('defs');
            var g = defs.append("pattern")
                .attr('id', 'hash')
                .attr('patternUnits', 'userSpaceOnUse')
                .attr('width', '10')
                .attr('height', '10')
                .append("g").style("fill", "none")
                .style("stroke", "grey")
                .style("stroke-width", 1);
            g.append("path").attr("d", "M0,0 l10,10");
            g.append("path").attr("d", "M10,0 l-10,10");
        };
        storageReport.prototype.getChildren = function (node, depth) {
            return depth === 0 ? node.internalChildren : [];
        };
        storageReport.prototype.draw = function (goingIn, previousNode) {
            var _this = this;
            var levelDown = goingIn === true;
            var levelUp = goingIn === false;
            this.treemap = d3.layout.treemap()
                .children(function (n, depth) { return _this.getChildren(n, depth); })
                .value(function (d) { return d.size; })
                .size([this.w, this.h]);
            this.tooltip = d3.select(".chart-tooltip");
            var oldLocation = {
                dx: this.node().dx,
                dy: this.node().dy,
                x: this.node().x,
                y: this.node().y
            };
            var nodes = this.treemap.nodes(this.node())
                .filter(function (n) { return !n.children; });
            if (levelDown) {
                this.animateZoomIn(nodes, oldLocation);
            }
            else if (levelUp) {
                this.animateZoomOut(nodes);
            }
            else {
                // initial state
                this.svg.select(".treemap")
                    .remove();
                var container = this.svg.append("g")
                    .classed("treemap", true);
                this.drawNewTreeMap(nodes, container);
            }
        };
        storageReport.prototype.animateZoomIn = function (nodes, oldLocation) {
            var _this = this;
            this.transitioning = true;
            var oldContainer = this.svg.select(".treemap");
            var oldCells = oldContainer.selectAll("g.cell");
            var newGroup = this.svg.append("g")
                .classed("treemap", true);
            var scaleX = this.w / oldLocation.dx;
            var scaleY = this.h / oldLocation.dy;
            var transX = -oldLocation.x * scaleX;
            var transY = -oldLocation.y * scaleY;
            oldContainer
                .selectAll("text")
                .transition()
                .duration(storageReport.animationLegth / 4)
                .style('opacity', 0);
            oldContainer
                .transition()
                .duration(storageReport.animationLegth)
                .attr("transform", "translate(" + transX + "," + transY + ")scale(" + scaleX + "," + scaleY + ")")
                .each("end", function () {
                var newCells = _this.drawNewTreeMap(nodes, newGroup);
                newCells
                    .style('opacity', 0)
                    .transition()
                    .duration(storageReport.animationLegth)
                    .style('opacity', 1)
                    .each("end", function () {
                    oldContainer.remove();
                    _this.transitioning = false;
                });
            });
        };
        storageReport.prototype.animateZoomOut = function (nodes) {
            var _this = this;
            this.transitioning = true;
            var oldContainer = this.svg.select(".treemap");
            var oldCells = oldContainer.selectAll("g.cell");
            var newGroup = this.svg.append("g")
                .classed("treemap", true);
            var newCells = this.drawNewTreeMap(nodes, newGroup);
            newCells
                .style('opacity', 0)
                .transition()
                .duration(storageReport.animationLegth)
                .style('opacity', 1)
                .each("end", function () {
                oldContainer.remove();
                _this.transitioning = false;
            });
        };
        storageReport.prototype.drawNewTreeMap = function (nodes, container) {
            var _this = this;
            var self = this;
            var showTypeOffset = 7;
            var showTypePredicate = function (d) { return d.showType && d.dy > 22 && d.dx > 20; };
            var cell = container.selectAll("g.cell-no-such") // we always select non-existing nodes to draw from scratch - we don't update elements here
                .data(nodes)
                .enter().append("svg:g")
                .attr("class", function (d) { return "cell " + d.type; })
                .attr("transform", function (d) { return "translate(" + d.x + "," + d.y + ")"; })
                .on("click", function (d) { return _this.onClick(d, true); })
                .on("mouseover", function (d) { return _this.onMouseOver(d); })
                .on("mouseout", function (d) { return _this.onMouseOut(d); })
                .on("mousemove", function (d) { return _this.onMouseMove(d); });
            var rectangles = cell.append("svg:rect")
                .attr("width", function (d) { return d.dx - 1; })
                .attr("height", function (d) { return d.dy - 1; });
            rectangles
                .filter(function (x) { return x.hasChildren() || x.lazyLoadChildren; })
                .style('cursor', 'pointer');
            cell.append("svg:text")
                .filter(function (d) { return d.dx > 20 && d.dy > 8; })
                .attr("x", function (d) { return d.dx / 2; })
                .attr("y", function (d) { return showTypePredicate(d) ? d.dy / 2 - showTypeOffset : d.dy / 2; })
                .attr("dy", ".35em")
                .attr("text-anchor", "middle")
                .text(function (d) { return d.name; })
                .each(function (d) {
                self.wrap(this, d.dx);
            });
            cell.filter(function (d) { return showTypePredicate(d); })
                .append("svg:text")
                .attr("x", function (d) { return d.dx / 2; })
                .attr("y", function (d) { return showTypePredicate(d) ? d.dy / 2 + showTypeOffset : d.dy / 2; })
                .attr("dy", ".35em")
                .attr("text-anchor", "middle")
                .text(function (d) { return _.upperFirst(d.type); })
                .each(function (d) {
                self.wrap(this, d.dx);
            });
            return cell;
        };
        storageReport.prototype.wrap = function ($self, width) {
            var self = d3.select($self), textLength = self.node().getComputedTextLength(), text = self.text();
            while (textLength > (width - 6) && text.length > 0) {
                text = text.slice(0, -1);
                self.text(text + '...');
                textLength = self.node().getComputedTextLength();
            }
        };
        storageReport.prototype.loadDetailedReport = function (d) {
            var _this = this;
            if (!d.lazyLoadChildren) {
                return;
            }
            var env = d.parent;
            var showLoaderTimer = setTimeout(function () {
                _this.showLoader(true);
            }, 100);
            return new getEnvironmentStorageReportCommand(this.activeDatabase(), env.name, _.capitalize(env.type))
                .execute()
                .done(function (envReport) {
                _this.mapDetailedReport(envReport.Report, d);
            })
                .always(function () {
                if (_this.showLoader()) {
                    _this.showLoader(false);
                }
                else {
                    clearTimeout(showLoaderTimer);
                }
            });
        };
        storageReport.prototype.onClick = function (d, goingIn) {
            var _this = this;
            if (this.transitioning || this.node() === d) {
                return;
            }
            if (d.lazyLoadChildren) {
                var requestExecution_1 = protractedCommandsDetector.instance.requestStarted(500);
                this.loadDetailedReport(d)
                    .done(function () {
                    var prev = _this.node();
                    _this.node(d);
                    _this.draw(true, prev);
                })
                    .always(function () { return requestExecution_1.markCompleted(); });
                if (d3.event) {
                    d3.event.stopPropagation();
                }
                return;
            }
            if (!d.internalChildren || !d.internalChildren.length) {
                return;
            }
            var prev = this.node();
            this.node(d);
            this.draw(goingIn, prev);
            if (d3.event) {
                d3.event.stopPropagation();
            }
        };
        storageReport.prototype.onMouseMove = function (d) {
            var _a = d3.mouse(this.svg.node()), x = _a[0], y = _a[1];
            var tooltipWidth = $(".chart-tooltip").width() + 20;
            x = Math.min(x, Math.max(this.w - tooltipWidth, 0));
            this.tooltip
                .style("left", (x + 10) + "px")
                .style("top", (y + 10) + "px");
        };
        storageReport.prototype.onMouseOver = function (d) {
            this.tooltip.transition()
                .duration(200)
                .style("opacity", 1);
            var html = "Name: " + d.name;
            if (d.showType) {
                html += "<br />Type: <strong>" + _.upperFirst(d.type) + "</strong>";
            }
            if (this.shouldDisplayNumberOfEntires(d)) {
                html += "<br />Entries: <strong>" + d.numberOfEntries.toLocaleString() + "</strong>";
            }
            html += " <br /> <span class='size'>Size: <strong>" + generalUtils.formatBytesToSize(d.size) + "</strong></span>";
            this.tooltip.html(html);
            this.onMouseMove(d);
        };
        storageReport.prototype.shouldDisplayNumberOfEntires = function (d) {
            return d.type === "tree" || d.type === "table";
        };
        storageReport.prototype.onMouseOut = function (d) {
            this.tooltip.transition()
                .duration(500)
                .style("opacity", 0);
        };
        return storageReport;
    }(viewModelBase));
    storageReport.animationLegth = 200;
    return storageReport;
});

//# sourceMappingURL=storageReport.js.map
