var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/copyToClipboard", "viewmodels/viewModelBase", "commands/database/studio/createSampleDataCommand", "commands/database/studio/createSampleDataClassCommand", "common/bindingHelpers/aceEditorBindingHandler", "common/eventsCollector", "commands/database/documents/getCollectionsStatsCommand", "common/appUrl"], function (require, exports, copyToClipboard, viewModelBase, createSampleDataCommand, createSampleDataClassCommand, aceEditorBindingHandler, eventsCollector, getCollectionsStatsCommand, appUrl) {
    "use strict";
    var createSampleData = (function (_super) {
        __extends(createSampleData, _super);
        function createSampleData() {
            var _this = _super.call(this) || this;
            _this.classData = ko.observable();
            _this.canCreateSampleData = ko.observable(false);
            _this.justCreatedSampleData = ko.observable(false);
            _this.classesVisible = ko.observable(false);
            aceEditorBindingHandler.install();
            return _this;
        }
        createSampleData.prototype.generateSampleData = function () {
            var _this = this;
            eventsCollector.default.reportEvent("sample-data", "create");
            this.isBusy(true);
            new createSampleDataCommand(this.activeDatabase())
                .execute()
                .done(function () {
                _this.canCreateSampleData(false);
                _this.justCreatedSampleData(true);
            })
                .always(function () { return _this.isBusy(false); });
        };
        createSampleData.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('OGRN53');
            return $.when(this.fetchSampleDataClasses(), this.fetchCollectionsStats());
        };
        createSampleData.prototype.showCode = function () {
            this.classesVisible(true);
            var $pageHostRoot = $("#page-host-root");
            var $sampleDataMain = $(".sample-data-main");
            $pageHostRoot.animate({
                scrollTop: $sampleDataMain.height()
            }, 'fast');
        };
        createSampleData.prototype.copyClasses = function () {
            eventsCollector.default.reportEvent("sample-data", "copy-classes");
            copyToClipboard.copy(this.classData(), "Copied C# classes to clipboard.");
        };
        createSampleData.prototype.fetchCollectionsStats = function () {
            var _this = this;
            new getCollectionsStatsCommand(this.activeDatabase())
                .execute()
                .done(function (stats) { return _this.onCollectionsFetched(stats); });
        };
        createSampleData.prototype.onCollectionsFetched = function (stats) {
            var nonEmptyNonSystemCollectionsCount = stats
                .collections
                .filter(function (x) { return !x.isSystemDocuments && x.documentCount() > 0; })
                .length;
            this.canCreateSampleData(nonEmptyNonSystemCollectionsCount === 0);
        };
        createSampleData.prototype.fetchSampleDataClasses = function () {
            var _this = this;
            return new createSampleDataClassCommand(this.activeDatabase())
                .execute()
                .done(function (results) {
                _this.classData(results);
            });
        };
        createSampleData.prototype.urlForDatabaseDocuments = function () {
            return appUrl.forDocuments("", this.activeDatabase());
        };
        return createSampleData;
    }(viewModelBase));
    return createSampleData;
});

//# sourceMappingURL=createSampleData.js.map
