var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "endpoints", "moment", "common/copyToClipboard", "common/appUrl", "common/messagePublisher", "common/bindingHelpers/aceEditorBindingHandler", "common/notifications/notificationCenter", "models/database/tasks/exportDatabaseModel", "commands/database/studio/validateExportDatabaseOptionsCommand", "commands/database/documents/getCollectionsStatsCommand", "commands/auth/getSingleAuthTokenCommand", "commands/database/studio/getNextOperationId", "common/eventsCollector"], function (require, exports, viewModelBase, endpoints, moment, copyToClipboard, appUrl, messagePublisher, aceEditorBindingHandler, notificationCenter, exportDatabaseModel, validateExportDatabaseOptionsCommand, getCollectionsStatsCommand, getSingleAuthTokenCommand, getNextOperationId, eventsCollector) {
    "use strict";
    var exportDatabase = (function (_super) {
        __extends(exportDatabase, _super);
        function exportDatabase() {
            var _this = _super.call(this) || this;
            _this.model = new exportDatabaseModel();
            _this.isExporting = exportDatabase.isExporting;
            _this.showAdvancedOptions = ko.observable(false);
            _this.showTransformScript = ko.observable(false);
            _this.collections = ko.observableArray();
            _this.filter = ko.observable("");
            aceEditorBindingHandler.install();
            _this.setupDefaultExportFilename();
            return _this;
        }
        exportDatabase.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("YD9M1R");
            this.initializeObservables();
            this.fetchCollections()
                .done(function (collections) {
                _this.collections(collections);
            });
        };
        exportDatabase.prototype.fetchCollections = function () {
            var collectionsTask = $.Deferred();
            new getCollectionsStatsCommand(this.activeDatabase())
                .execute()
                .done(function (stats) {
                collectionsTask.resolve(stats.collections.map(function (x) { return x.name; }));
            })
                .fail(function () { return collectionsTask.reject(); });
            return collectionsTask;
        };
        exportDatabase.prototype.setupDefaultExportFilename = function () {
            var dbName = this.activeDatabase().name;
            var date = moment().format("YYYY-MM-DD HH-mm");
            this.model.exportFileName("Dump of " + dbName + " " + date);
        };
        exportDatabase.prototype.initializeObservables = function () {
            var _this = this;
            this.filteredCollections = ko.pureComputed(function () {
                var filter = _this.filter();
                var collections = _this.collections();
                if (!filter) {
                    return collections;
                }
                var filterLowerCase = filter.toLowerCase();
                return collections.filter(function (x) { return x.toLowerCase().includes(filterLowerCase); });
            });
            this.exportCommand = ko.pureComputed(function () {
                //TODO: review this!
                var targetServer = appUrl.forServer();
                var model = _this.model;
                var outputFilename = exportDatabase.escapeForShell(model.exportFileName());
                var commandTokens = ["Raven.Smuggler", "out", targetServer, outputFilename];
                var types = [];
                if (model.includeDocuments()) {
                    types.push("Documents");
                }
                if (model.includeIndexes()) {
                    types.push("Indexes");
                }
                if (model.includeTransformers()) {
                    types.push("Transformers");
                }
                if (types.length > 0) {
                    commandTokens.push("--operate-on-types=" + types.join(","));
                }
                //TODO: identities, remove analyzers, RevisionDocuments
                var databaseName = _this.activeDatabase().name;
                commandTokens.push("--database=" + exportDatabase.escapeForShell(databaseName));
                if (model.includeExpiredDocuments()) {
                    commandTokens.push("--includeexpired");
                }
                if (!model.includeAllCollections()) {
                    var collections = model.includedCollections();
                    commandTokens.push("--metadata-filter=Raven-Entity-Name=" + exportDatabase.escapeForShell(collections.toString()));
                }
                if (model.transformScript()) {
                    commandTokens.push("--transform=" + exportDatabase.escapeForShell(model.transformScript()));
                }
                return commandTokens.join(" ");
            });
        };
        exportDatabase.prototype.copyCommandToClipboard = function () {
            copyToClipboard.copy(this.exportCommand(), "Command was copied to clipboard.");
        };
        exportDatabase.escapeForShell = function (input) {
            return '"' + input.replace(/[\r\n]/g, "").replace(/(["\\])/g, '\\$1') + '"';
        };
        exportDatabase.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#transformScriptPopover").popover({
                html: true,
                trigger: "hover",
                content: "Transform scripts are written in JavaScript. <br /><br/>Example:<pre><span class=\"code-keyword\">function</span>(doc) {<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">var</span> id = doc['@metadata']['@id'];<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">if</span> (id === 'orders/999')<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">return null</span>;<br /><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">return</span> doc;<br />}</pre>"
            });
        };
        exportDatabase.prototype.startExport = function () {
            var _this = this;
            eventsCollector.default.reportEvent("database", "export");
            exportDatabase.isExporting(true);
            var exportArg = this.model.toDto();
            new validateExportDatabaseOptionsCommand(exportArg, this.activeDatabase())
                .execute()
                .done(function () { return _this.startDownload(exportArg); })
                .fail(function (response) {
                messagePublisher.reportError("Invalid export options", response.responseText, response.statusText);
                exportDatabase.isExporting(false);
            });
        };
        exportDatabase.prototype.getNextOperationId = function (db) {
            return new getNextOperationId(db).execute()
                .fail(function (qXHR, textStatus, errorThrown) {
                messagePublisher.reportError("Could not get next task id.", errorThrown);
                exportDatabase.isExporting(false);
            });
        };
        exportDatabase.prototype.getAuthToken = function (db) {
            return new getSingleAuthTokenCommand(db).execute()
                .fail(function (qXHR, textStatus, errorThrown) {
                messagePublisher.reportError("Could not get single auth token for download.", errorThrown);
                exportDatabase.isExporting(false);
            });
        };
        exportDatabase.prototype.startDownload = function (args) {
            var $form = $("#exportDownloadForm");
            var db = this.activeDatabase();
            var $downloadOptions = $("[name=DownloadOptions]", $form);
            $.when(this.getNextOperationId(db), this.getAuthToken(db))
                .then(function (_a, _b) {
                var operationId = _a[0];
                var token = _b[0];
                var url = endpoints.databases.smuggler.smugglerExport;
                var authToken = (url.indexOf("?") === -1 ? "?" : "&") + "singleUseAuthToken=" + token.Token;
                var operationPart = "&operationId=" + operationId;
                $form.attr("action", appUrl.forResourceQuery(db) + url + authToken + operationPart);
                $downloadOptions.val(JSON.stringify(args));
                $form.submit();
                notificationCenter.instance.monitorOperation(db, operationId)
                    .fail(function (exception) {
                    messagePublisher.reportError("Could not export database: " + exception.Message, exception.StackTrace, null, false);
                }).always(function () { return exportDatabase.isExporting(false); });
            });
        };
        return exportDatabase;
    }(viewModelBase));
    exportDatabase.isExporting = ko.observable(false);
    return exportDatabase;
});

//# sourceMappingURL=exportDatabase.js.map
