var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/bindingHelpers/aceEditorBindingHandler", "viewmodels/viewModelBase", "common/messagePublisher", "commands/database/studio/importDatabaseCommand", "models/database/tasks/importDatabaseModel", "common/notifications/notificationCenter", "common/eventsCollector", "common/copyToClipboard", "commands/database/studio/getNextOperationId", "commands/auth/getSingleAuthTokenCommand", "common/constants/events"], function (require, exports, aceEditorBindingHandler, viewModelBase, messagePublisher, importDatabaseCommand, importDatabaseModel, notificationCenter, eventsCollector, copyToClipboard, getNextOperationId, getSingleAuthTokenCommand, EVENTS) {
    "use strict";
    var importDatabase = (function (_super) {
        __extends(importDatabase, _super);
        function importDatabase() {
            var _this = _super.call(this) || this;
            _this.model = new importDatabaseModel();
            _this.isImporting = importDatabase.isImporting;
            _this.showAdvancedOptions = ko.observable(false);
            _this.showTransformScript = ko.observable(false);
            _this.hasFileSelected = ko.observable(false);
            _this.importedFileName = ko.observable();
            _this.isUploading = ko.observable(false);
            _this.uploadStatus = ko.observable();
            _this.validationGroup = ko.validatedObservable({
                importedFileName: _this.importedFileName
            });
            _this.bindToCurrentInstance("copyCommandToClipboard", "fileSelected");
            aceEditorBindingHandler.install();
            _this.isUploading.subscribe(function (v) {
                if (!v) {
                    _this.uploadStatus(0);
                }
            });
            //TODO: implement this command
            _this.importCommand = ko.pureComputed(function () { return 'Raven.Smuggler out http://live-test.ravendb.net raven.dump --operate-on-types=Documents,Indexes,Transformers --database="Media" --batch-size=1024 --excludeexpired'; });
            _this.setupValidation();
            return _this;
        }
        importDatabase.prototype.setupValidation = function () {
            this.importedFileName.extend({
                required: true,
                validation: [{
                        validator: function (name) { return name && name.endsWith(".ravendbdump"); },
                        message: "Invalid file extension."
                    }]
            });
        };
        importDatabase.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#transformScriptPopover").popover({
                html: true,
                trigger: "hover",
                content: "Transform scripts are written in JavaScript. <br /><br/>Example:<pre><span class=\"code-keyword\">function</span>(doc) {<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">var</span> id = doc['@metadata']['@id'];<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">if</span> (id === 'orders/999')<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">return null</span>;<br /><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">return</span> doc;<br />}</pre>"
            });
            this.updateHelpLink("YD9M1R");
        };
        importDatabase.prototype.canDeactivate = function (isClose) {
            _super.prototype.canDeactivate.call(this, isClose);
            if (this.isUploading()) {
                this.confirmationMessage("Upload is in progress", "Please wait until uploading is complete.", ["OK"]);
                return false;
            }
            return true;
        };
        importDatabase.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                ko.postbox.subscribe("UploadProgress", function (percentComplete) {
                    var db = _this.activeDatabase();
                    if (!db) {
                        return;
                    }
                    if (!_this.isUploading()) {
                        return;
                    }
                    if (percentComplete === 100) {
                        setTimeout(function () { return _this.isUploading(false); }, 700);
                    }
                    _this.uploadStatus(percentComplete);
                }),
                ko.postbox.subscribe(EVENTS.ChangesApi.Reconnected, function (db) {
                    _this.isUploading(false);
                })
            ];
        };
        importDatabase.prototype.fileSelected = function (fileName) {
            var isFileSelected = fileName ? !!fileName.trim() : false;
            this.hasFileSelected(isFileSelected);
            this.importedFileName(isFileSelected ? fileName.split(/(\\|\/)/g).pop() : null);
        };
        importDatabase.prototype.importDb = function () {
            var _this = this;
            if (!this.isValid(this.validationGroup)) {
                return;
            }
            eventsCollector.default.reportEvent("database", "import");
            this.isUploading(true);
            var fileInput = document.querySelector(importDatabase.filePickerTag);
            var db = this.activeDatabase();
            $.when(this.getNextOperationId(db), this.getAuthToken(db))
                .then(function (_a, _b) {
                var operationId = _a[0];
                var token = _b[0];
                notificationCenter.instance.monitorOperation(db, operationId);
                new importDatabaseCommand(db, operationId, token, fileInput.files[0], _this.model)
                    .execute()
                    .always(function () { return _this.isUploading(false); });
            });
        };
        importDatabase.prototype.copyCommandToClipboard = function () {
            copyToClipboard.copy(this.importCommand(), "Command was copied to clipboard.");
        };
        importDatabase.prototype.getNextOperationId = function (db) {
            var _this = this;
            return new getNextOperationId(db).execute()
                .fail(function (qXHR, textStatus, errorThrown) {
                messagePublisher.reportError("Could not get next task id.", errorThrown);
                _this.isUploading(false);
            });
        };
        importDatabase.prototype.getAuthToken = function (db) {
            var _this = this;
            return new getSingleAuthTokenCommand(db).execute()
                .fail(function (qXHR, textStatus, errorThrown) {
                messagePublisher.reportError("Could not get single auth token.", errorThrown);
                _this.isUploading(false);
            });
        };
        return importDatabase;
    }(viewModelBase));
    importDatabase.filePickerTag = "#importDatabaseFilePicker";
    importDatabase.isImporting = ko.observable(false);
    return importDatabase;
});

//# sourceMappingURL=importDatabase.js.map
