var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/database/index/transformer", "commands/database/transformers/saveTransformerCommand", "commands/database/transformers/getSingleTransformerCommand", "common/bindingHelpers/aceEditorBindingHandler", "viewmodels/database/transformers/deleteTransformerConfirm", "viewmodels/database/transformers/saveTransformerWithNewNameConfirm", "plugins/dialog", "common/appUrl", "common/jsonUtil", "plugins/router", "common/messagePublisher", "commands/database/index/formatIndexCommand", "common/eventsCollector"], function (require, exports, viewModelBase, transformer, saveTransformerCommand, getSingleTransformerCommand, aceEditorBindingHandler, deleteTransformerConfirm, saveTransformerWithNewNameConfirm, dialog, appUrl, jsonUtil, router, messagePublisher, formatIndexCommand, eventsCollector) {
    "use strict";
    var editTransformer = (function (_super) {
        __extends(editTransformer, _super);
        function editTransformer() {
            var _this = _super.call(this) || this;
            _this.editedTransformer = ko.observable();
            _this.isEditingExistingTransformer = ko.observable(false);
            _this.popoverOptions = ko.observable();
            _this.editorCollection = ko.observableArray();
            _this.loadedTransformerName = ko.observable();
            aceEditorBindingHandler.install();
            _this.appUrls = appUrl.forCurrentDatabase();
            _this.transformerName = ko.computed(function () { return (!!_this.editedTransformer() && _this.isEditingExistingTransformer()) ? _this.editedTransformer().name() : null; });
            return _this;
        }
        editTransformer.prototype.canActivate = function (transformerToEditName) {
            var _this = this;
            if (transformerToEditName) {
                var canActivateResult = $.Deferred();
                this.editExistingTransformer(transformerToEditName)
                    .done(function () { return canActivateResult.resolve({ can: true }); })
                    .fail(function () {
                    messagePublisher.reportError("Could not find " + transformerToEditName + " transformer");
                    canActivateResult.resolve({ redirect: appUrl.forTransformers(_this.activeDatabase()) });
                });
                return canActivateResult;
            }
            else {
                return $.Deferred().resolve({ can: true });
            }
        };
        editTransformer.prototype.activate = function (transformerToEditName) {
            var _this = this;
            _super.prototype.activate.call(this, transformerToEditName);
            this.updateHelpLink('S467UO');
            if (transformerToEditName) {
                this.isEditingExistingTransformer(true);
            }
            else {
                this.editedTransformer(transformer.empty());
            }
            this.dirtyFlag = new ko.DirtyFlag([this.editedTransformer().name, this.editedTransformer().transformResults], false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.computed(function () { return !!_this.editedTransformer().name() && _this.dirtyFlag().isDirty(); });
        };
        editTransformer.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.addTransformerHelpPopover();
            this.createKeyboardShortcut("alt+c", this.focusOnEditor, editTransformer.containerSelector);
            this.createKeyboardShortcut("alt+shift+del", this.deleteTransformer, editTransformer.containerSelector);
        };
        editTransformer.prototype.addTransformerHelpPopover = function () {
            $("#transformerResultsLabel").popover({
                html: true,
                trigger: "hover",
                content: 'The Transform function allows you to change the shape of individual result documents before the server returns them. It uses C# LINQ query syntax <br/> <br/> Example: <pre> <br/> <span class="code-keyword">from</span> result <span class="code-keyword">in</span> results <br/> <span class="code-keyword">let</span> category = LoadDocument(result.Category) <br/> <span class="code-keyword">select new</span> { <br/>    result.Name, <br/>    result.PricePerUnit, <br/>    Category = category.Name, <br/>    CategoryDescription = category.Description <br/>}</pre>',
            });
        };
        editTransformer.prototype.focusOnEditor = function () {
            var editorElement = $("#transformerAceEditor").length == 1 ? $("#transformerAceEditor")[0] : null;
            if (editorElement) {
                var docEditor = ko.utils.domData.get($("#transformerAceEditor")[0], "aceEditor");
                if (docEditor) {
                    docEditor.focus();
                }
            }
        };
        editTransformer.prototype.editExistingTransformer = function (unescapedTransformerName) {
            var _this = this;
            var transformerName = decodeURIComponent(unescapedTransformerName);
            this.loadedTransformerName(transformerName);
            return this.fetchTransformerToEdit(transformerName)
                .done(function (trans) { return _this.editedTransformer(new transformer(trans)); });
        };
        editTransformer.prototype.fetchTransformerToEdit = function (transformerName) {
            return new getSingleTransformerCommand(transformerName, this.activeDatabase()).execute();
        };
        editTransformer.prototype.saveTransformer = function () {
            var _this = this;
            eventsCollector.default.reportEvent("transformer", "save");
            if (this.isEditingExistingTransformer() && this.editedTransformer().nameChanged()) {
                var db = this.activeDatabase();
                var saveTransformerWithNewNameViewModel = new saveTransformerWithNewNameConfirm(this.editedTransformer(), db);
                saveTransformerWithNewNameViewModel.saveTask.done(function () {
                    _this.dirtyFlag().reset(); // Resync Changes
                    _this.updateUrl(_this.editedTransformer().name());
                });
                dialog.show(saveTransformerWithNewNameViewModel);
            }
            else {
                this.editedTransformer().name(this.editedTransformer().name().trim());
                new saveTransformerCommand(this.editedTransformer(), this.activeDatabase())
                    .execute()
                    .done(function () {
                    _this.dirtyFlag().reset();
                    if (!_this.isEditingExistingTransformer()) {
                        _this.isEditingExistingTransformer(true);
                        _this.updateUrl(_this.editedTransformer().name());
                    }
                });
            }
        };
        editTransformer.prototype.updateUrl = function (transformerName) {
            router.navigate(appUrl.forEditTransformer(transformerName, this.activeDatabase()));
        };
        editTransformer.prototype.refreshTransformer = function () {
            var _this = this;
            eventsCollector.default.reportEvent("transformer", "refresh");
            var canContinue = this.canContinueIfNotDirty("Unsaved Data", "You have unsaved data. Are you sure you want to refresh the transformer from the server?");
            canContinue
                .done(function () {
                var transformerName = _this.loadedTransformerName();
                _this.fetchTransformerToEdit(transformerName)
                    .always(function () { return _this.dirtyFlag().reset(); })
                    .done(function (trans) { return _this.editedTransformer(new transformer(trans)); })
                    .fail(function () {
                    messagePublisher.reportError("Could not find " + transformerName + " transformer");
                    _this.navigate(appUrl.forTransformers(_this.activeDatabase()));
                });
            });
        };
        editTransformer.prototype.formatTransformer = function () {
            eventsCollector.default.reportEvent("transformer", "format");
            var editedTransformer = this.editedTransformer();
            new formatIndexCommand(this.activeDatabase(), [editedTransformer.transformResults()])
                .execute()
                .done(function (result) {
                var formatedTransformer = result[0];
                if (formatedTransformer.indexOf("Could not format:") == -1) {
                    editedTransformer.transformResults(formatedTransformer);
                }
                else {
                    messagePublisher.reportError("Failed to format transformer!", formatedTransformer);
                }
            });
        };
        editTransformer.prototype.deleteTransformer = function () {
            eventsCollector.default.reportEvent("transformer", "delete");
            var transformer = this.editedTransformer();
            if (transformer) {
                var db_1 = this.activeDatabase();
                var deleteViewmodel = new deleteTransformerConfirm([transformer.name()], db_1);
                deleteViewmodel.deleteTask.done(function () {
                    router.navigate(appUrl.forTransformers(db_1));
                });
                dialog.show(deleteViewmodel);
            }
        };
        return editTransformer;
    }(viewModelBase));
    editTransformer.containerSelector = "#editTransformerContainer";
    return editTransformer;
});

//# sourceMappingURL=editTransformer.js.map
