var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/database/index/transformer", "commands/database/transformers/getTransformersCommand", "commands/database/transformers/saveTransformerLockModeCommand", "common/appUrl", "viewmodels/database/transformers/deleteTransformerConfirm", "durandal/app", "common/eventsCollector"], function (require, exports, viewModelBase, transformer, getTransformersCommand, saveTransformerLockModeCommand, appUrl, deleteTransformerConfirm, app, eventsCollector) {
    "use strict";
    var transformers = (function (_super) {
        __extends(transformers, _super);
        function transformers() {
            var _this = _super.call(this) || this;
            _this.newTransformerUrl = appUrl.forCurrentDatabase().newTransformer;
            _this.transformersGroups = ko.observableArray();
            _this.selectedTransformersName = ko.observableArray();
            _this.searchText = ko.observable();
            _this.globalLockChangesInProgress = ko.observable(false);
            _this.localLockChangesInProgress = ko.observableArray([]);
            _this.bindToCurrentInstance("deleteTransformer", "deleteSelectedTransformers", "unlockTransformer", "lockTransformer", "unlockSelectedTransformers", "lockSelectedTransformers");
            _this.initObservables();
            return _this;
        }
        transformers.prototype.initObservables = function () {
            var _this = this;
            this.searchText.throttle(200).subscribe(function () { return _this.filterTransformers(); });
            this.lockModeCommon = ko.computed(function () {
                var selectedTransformers = _this.getSelectedTransformers();
                if (selectedTransformers.length === 0)
                    return "None";
                var firstLockMode = selectedTransformers[0].lockMode();
                for (var i = 1; i < selectedTransformers.length; i++) {
                    if (selectedTransformers[i].lockMode() !== firstLockMode) {
                        return "Mixed";
                    }
                }
                return firstLockMode;
            });
            this.selectionState = ko.pureComputed(function () {
                var selectedCount = _this.selectedTransformersName().length;
                if (selectedCount === _this.getAllTransformers().length)
                    return 2 /* Checked */;
                if (selectedCount > 0)
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
        };
        transformers.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            return this.fetchTransformers(this.activeDatabase());
        };
        transformers.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.updateHelpLink("OWRJLV");
            ko.postbox.publish("SetRawJSONUrl", appUrl.forTransformersRawData(this.activeDatabase()));
        };
        transformers.prototype.filterTransformers = function () {
            var filterLower = this.searchText().toLowerCase();
            this.selectedTransformersName([]);
            this.transformersGroups()
                .forEach(function (transformerGroup) {
                var hasAnyInGroup = false;
                transformerGroup.transformers().forEach(function (trans) {
                    var match = trans.name().toLowerCase().indexOf(filterLower) >= 0;
                    trans.filteredOut(!match);
                    if (match) {
                        hasAnyInGroup = true;
                    }
                });
                transformerGroup.groupHidden(!hasAnyInGroup);
            });
        };
        transformers.prototype.fetchTransformers = function (db) {
            var _this = this;
            return new getTransformersCommand(db)
                .execute()
                .done(function (transformers) {
                transformers
                    .map(function (t) { return new transformer(t); })
                    .forEach(function (i) { return _this.putTransformerIntoGroup(i); });
            });
        };
        transformers.prototype.createNotifications = function () {
            var _this = this;
            return [
                this.changesContext.currentResourceChangesApi().watchAllTransformers(function (e) { return _this.processTransformerEvent(e); })
            ];
        };
        transformers.prototype.processTransformerEvent = function (e) {
            if (e.Type === "TransformerRemoved") {
                var existingTransformer = this.findTransformerByName(e.Name);
                if (existingTransformer) {
                    this.removeTransformersFromAllGroups([existingTransformer]);
                }
            }
            else {
                this.fetchTransformers(this.activeDatabase());
            }
        };
        transformers.prototype.findTransformerByName = function (transformerName) {
            var transformsGroups = this.transformersGroups();
            for (var i = 0; i < transformsGroups.length; i++) {
                var group = transformsGroups[i];
                var transformers_1 = group.transformers();
                for (var j = 0; j < transformers_1.length; j++) {
                    if (transformers_1[j].name() === transformerName) {
                        return transformers_1[j];
                    }
                }
            }
            return null;
        };
        transformers.prototype.putTransformerIntoGroup = function (trans) {
            eventsCollector.default.reportEvent("transformers", "put-into-group");
            var groupName = trans.name().split("/")[0];
            var group = this.transformersGroups().find(function (g) { return g.entityName === groupName; });
            if (group) {
                var existingTrans = group.transformers().find(function (cur) { return cur.name() === trans.name(); });
                if (!existingTrans) {
                    group.transformers.push(trans);
                }
            }
            else {
                this.transformersGroups.push({
                    entityName: groupName,
                    transformers: ko.observableArray([trans]),
                    groupHidden: ko.observable(false)
                });
            }
        };
        transformers.prototype.deleteSelectedTransformers = function () {
            eventsCollector.default.reportEvent("transformers", "delete-selected");
            this.promptDeleteTransformers(this.getSelectedTransformers());
        };
        transformers.prototype.getAllTransformers = function () {
            var all = [];
            this.transformersGroups().forEach(function (g) { return all.push.apply(all, g.transformers()); });
            return _.uniq(all);
        };
        transformers.prototype.getSelectedTransformers = function () {
            var selectedTransformers = this.selectedTransformersName();
            return this.getAllTransformers().filter(function (x) { return _.includes(selectedTransformers, x.name()); });
        };
        transformers.prototype.deleteTransformer = function (transformerToDelete) {
            eventsCollector.default.reportEvent("transformers", "delete");
            this.promptDeleteTransformers([transformerToDelete]);
        };
        transformers.prototype.promptDeleteTransformers = function (transformers) {
            var _this = this;
            var db = this.activeDatabase();
            var deleteViewmodel = new deleteTransformerConfirm(transformers.map(function (i) { return i.name(); }), db);
            deleteViewmodel.deleteTask.done(function () { return _this.removeTransformersFromAllGroups(transformers); });
            app.showBootstrapDialog(deleteViewmodel);
        };
        transformers.prototype.removeTransformersFromAllGroups = function (transformers) {
            this.transformersGroups().forEach(function (transGroup) { return transGroup.transformers.removeAll(transformers); });
            this.transformersGroups.remove(function (item) { return item.transformers().length === 0; });
        };
        transformers.prototype.unlockSelectedTransformers = function () {
            this.setLockModeSelectedTransformers("Unlock", "Unlock");
        };
        transformers.prototype.lockSelectedTransformers = function () {
            this.setLockModeSelectedTransformers("LockedIgnore", "Lock");
        };
        transformers.prototype.setLockModeSelectedTransformers = function (lockModeString, localModeString) {
            var _this = this;
            if (this.lockModeCommon() === lockModeString)
                return;
            this.confirmationMessage("Are you sure?", "Do you want to " + localModeString + " selected transformers?")
                .done(function (can) {
                if (can) {
                    eventsCollector.default.reportEvent("transformers", "lock-selected");
                    _this.globalLockChangesInProgress(true);
                    var transformers_2 = _this.getSelectedTransformers();
                    new saveTransformerLockModeCommand(transformers_2, lockModeString, _this.activeDatabase())
                        .execute()
                        .done(function () { return transformers_2.forEach(function (t) { return t.lockMode(lockModeString); }); })
                        .always(function () { return _this.globalLockChangesInProgress(false); });
                }
            });
        };
        transformers.prototype.lockTransformer = function (t) {
            eventsCollector.default.reportEvent("transformers", "lock");
            this.updateTransformerLockMode(t, "LockedIgnore");
        };
        transformers.prototype.unlockTransformer = function (t) {
            eventsCollector.default.reportEvent("transformers", "unlock");
            this.updateTransformerLockMode(t, "Unlock");
        };
        transformers.prototype.updateTransformerLockMode = function (t, lockMode) {
            var _this = this;
            if (t.lockMode() !== lockMode) {
                this.localLockChangesInProgress.push(t.name());
                new saveTransformerLockModeCommand([t], lockMode, this.activeDatabase())
                    .execute()
                    .done(function () { return t.lockMode(lockMode); })
                    .always(function () { return _this.localLockChangesInProgress.remove(t.name()); });
            }
        };
        transformers.prototype.toggleSelectAll = function () {
            eventsCollector.default.reportEvent("transformers", "toggle-select-all");
            var selectedCount = this.selectedTransformersName().length;
            if (selectedCount > 0) {
                this.selectedTransformersName([]);
            }
            else {
                var allTransformerNames = this.getAllTransformers().map(function (idx) { return idx.name(); });
                this.selectedTransformersName(allTransformerNames);
            }
        };
        return transformers;
    }(viewModelBase));
    return transformers;
});

//# sourceMappingURL=transformers.js.map
