/// <reference path="../../typings/tsd.d.ts"/>
define(["require", "exports", "common/helpers/text/pluralizeHelpers"], function (require, exports, pluralizeHelpers) {
    "use strict";
    var dialogViewModelBase = (function () {
        function dialogViewModelBase(options) {
            this.pluralize = pluralizeHelpers.pluralize;
            if (options) {
                this.elementToFocusOnDismissal = options.elementToFocusOnDismissal;
                this.dialogSelector = options.dialogSelectorName || dialogViewModelBase.dialogSelector;
            }
            else {
                this.dialogSelector = dialogViewModelBase.dialogSelector;
            }
        }
        dialogViewModelBase.prototype.bindToCurrentInstance = function () {
            var methods = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                methods[_i] = arguments[_i];
            }
            _.bindAll.apply(_, [this].concat(methods));
        };
        dialogViewModelBase.prototype.attached = function () {
            var _this = this;
            this.onEnterBinding = jwerty.key("enter", function () { return _this.enterKeyPressed(); });
        };
        dialogViewModelBase.prototype.deactivate = function (args) {
            if (this.onEnterBinding) {
                this.onEnterBinding.unbind();
            }
        };
        dialogViewModelBase.prototype.detached = function () {
            if (this.elementToFocusOnDismissal) {
                $(this.elementToFocusOnDismissal).focus();
            }
        };
        dialogViewModelBase.prototype.compositionComplete = function (view, parent) {
            var _this = this;
            setTimeout(function () { return _this.setInitialFocus(); }, 100); // We have to time-delay this, else it never receives focus.
        };
        dialogViewModelBase.prototype.setInitialFocus = function () {
            var autoFocusElement = $(this.dialogSelector + " [autofocus]");
            if (autoFocusElement.length) {
                autoFocusElement.focus();
                autoFocusElement.select();
            }
            else {
                $(this.dialogSelector).focus();
            }
        };
        dialogViewModelBase.prototype.enterKeyPressed = function () {
            var confirmButton = this.getCofirmButton();
            if (confirmButton && confirmButton.click) {
                confirmButton.click();
            }
            return true;
        };
        dialogViewModelBase.prototype.getCofirmButton = function () {
            return $(".modal-footer:visible .btn-primary")[0];
        };
        dialogViewModelBase.prototype.isValid = function (context, showErrors) {
            if (showErrors === void 0) { showErrors = true; }
            if (context.isValid()) {
                return true;
            }
            else {
                if (showErrors) {
                    context.errors.showAllMessages();
                }
                return false;
            }
        };
        return dialogViewModelBase;
    }());
    dialogViewModelBase.dialogSelector = ".modal-dialog";
    return dialogViewModelBase;
});

//# sourceMappingURL=dialogViewModelBase.js.map
