var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "ace/ace", "viewmodels/viewModelBase", "commands/filesystem/getConfigurationCommand", "models/filesystem/configurationKey", "common/bindingHelpers/aceEditorBindingHandler", "common/messagePublisher", "commands/filesystem/saveConfigurationCommand", "viewmodels/filesystem/configurations/deleteConfigurationKeys", "viewmodels/filesystem/configurations/createConfigurationKey", "common/eventsCollector"], function (require, exports, app, router, appUrl, ace, viewModelBase, getConfigurationCommand, configurationKey, aceEditorBindingHandler, messagePublisher, saveConfigurationCommand, deleteConfigurationKeys, createConfigurationKey, eventsCollector) {
    "use strict";
    var configuration = (function (_super) {
        __extends(configuration, _super);
        function configuration() {
            var _this = _super.call(this) || this;
            _this.router = router;
            _this.keys = ko.observableArray();
            _this.selectedKeyValue = ko.observable();
            _this.selectedKey = ko.observable().subscribeTo("ActivateConfigurationKey").distinctUntilChanged();
            _this.currentKey = ko.observable();
            _this.configurationKeyText = ko.observable('').extend({ required: true });
            _this.isBusy = ko.observable(false);
            _this.enabled = true;
            aceEditorBindingHandler.install();
            _this.selectedKey.subscribe(function (k) { return _this.selectedKeyChanged(k); });
            // When we programmatically change a configuration doc, push it into the editor.
            //this.subscription = this.configurationKeyText.subscribe(() => this.updateConfigurationText());
            _this.text = ko.computed({
                read: function () {
                    return _this.configurationKeyText();
                },
                write: function (text) {
                    _this.configurationKeyText(text);
                },
                owner: _this
            });
            return _this;
        }
        configuration.prototype.activate = function (navigationArgs) {
            var _this = this;
            _super.prototype.activate.call(this, navigationArgs);
            this.updateHelpLink("J3KIEN");
            this.appUrls = appUrl.forCurrentFilesystem();
            this.dirtyFlag = new ko.DirtyFlag([this.configurationKeyText]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
            return this.loadKeys(this.activeFilesystem());
        };
        configuration.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.activeFilesystem.subscribe(function (x) {
                _this.loadKeys(x);
            });
            $('.keys-collection').contextmenu({
                target: '#keys-context-menu'
            });
            this.initializeDocEditor();
            this.configurationEditor.focus();
            this.setupKeyboardShortcuts();
        };
        configuration.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#configurationEditor");
            if (editorElement.length > 0) {
                this.editor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#configurationEditor").on('DynamicHeightSet', function () { return _this.editor.resize(); });
            this.focusOnEditor();
        };
        configuration.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.selectedKey.unsubscribeFrom("ActivateConfigurationKey");
            $("#configurationEditor").off('DynamicHeightSet');
        };
        configuration.prototype.focusOnEditor = function () {
            if (!!this.editor) {
                this.editor.focus();
            }
        };
        configuration.prototype.createNotifications = function () {
            return [];
        };
        configuration.prototype.processFsConfigNotification = function (e) {
            switch (e.Action) {
                case 0 /* Set */:
                    this.addKey(e.Name);
                    break;
                case 1 /* Delete */:
                    this.removeKey(e.Name);
                    break;
                default:
                    console.error("Unknown notification action.");
            }
        };
        configuration.prototype.selectKeyValue = function (selection) {
            this.selectedKeyValue(selection);
        };
        configuration.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteConfiguration(); }, configuration.configSelector);
        };
        configuration.prototype.initializeDocEditor = function () {
            // Startup the Ace editor with JSON syntax highlighting.
            // TODO: Just use the simple binding handler instead.
            this.configurationEditor = ace.edit("configurationEditor");
            this.configurationEditor.setTheme("ace/theme/xcode");
            this.configurationEditor.setFontSize("16px");
            this.configurationEditor.getSession().setMode("ace/mode/json");
        };
        configuration.prototype.updateConfigurationText = function () {
            if (this.configurationEditor) {
                this.configurationEditor.getSession().setValue(this.configurationKeyText());
            }
        };
        configuration.prototype.loadKeys = function (fs) {
            var _this = this;
            if (this.enabled) {
                return new getConfigurationCommand(fs)
                    .execute()
                    .done(function (x) {
                    _this.keys(x);
                    if (x.length > 0) {
                        _this.selectKey(x[0]);
                    }
                    else {
                        _this.enableEditor(false);
                    }
                });
            }
            return $.Deferred().resolve();
        };
        configuration.prototype.selectKey = function (key) {
            key.activate();
        };
        configuration.prototype.enableEditor = function (enable) {
            if (this.configurationEditor) {
                this.configurationEditor.setReadOnly(!enable);
                this.configurationEditor.getSession().setUseWorker(enable);
                if (!enable) {
                    this.configurationKeyText("");
                    this.dirtyFlag().reset();
                }
            }
            this.enabled = enable;
        };
        configuration.prototype.selectedKeyChanged = function (selected) {
            var _this = this;
            if (selected) {
                this.isBusy(true);
                selected.getValues().done(function (data) {
                    _this.configurationKeyText(data);
                }).always(function () {
                    _this.dirtyFlag().reset();
                    _this.isBusy(false);
                });
                this.currentKey(selected);
                this.focusOnEditor();
            }
        };
        configuration.prototype.save = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-config", "save");
            var message = "";
            try {
                var jsonConfigDoc = JSON.parse(this.configurationKeyText());
            }
            catch (e) {
                if (jsonConfigDoc == undefined) {
                    message = "The configuration key data isn't a legal JSON expression!";
                }
                this.focusOnEditor();
            }
            if (message) {
                messagePublisher.reportError(message, undefined, undefined, false);
                return;
            }
            var saveCommand = new saveConfigurationCommand(this.activeFilesystem(), this.currentKey(), jsonConfigDoc);
            var saveTask = saveCommand.execute();
            saveTask.done(function () { return _this.dirtyFlag().reset(); });
        };
        configuration.prototype.refreshConfig = function () {
            eventsCollector.default.reportEvent("fs-config", "refresh");
            this.selectedKeyChanged(this.currentKey());
        };
        configuration.prototype.deleteConfiguration = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-config", "delete");
            var deleteConfigurationKeyViewModel = new deleteConfigurationKeys(this.activeFilesystem(), [this.currentKey()]);
            deleteConfigurationKeyViewModel
                .deletionTask
                .done(function () {
                _this.removeKey(_this.currentKey().key);
            });
            app.showBootstrapDialog(deleteConfigurationKeyViewModel);
        };
        configuration.prototype.removeKey = function (key) {
            eventsCollector.default.reportEvent("fs-config", "create");
            var foundKey = this.keys().filter(function (configKey) { return configKey.key === key; });
            if (foundKey.length > 0) {
                var currentIndex = this.keys.indexOf(this.currentKey());
                var foundIndex = this.keys.indexOf(foundKey[0]);
                var newIndex = currentIndex;
                if (currentIndex + 1 === this.keys().length) {
                    newIndex = currentIndex - 1;
                }
                this.keys.remove(foundKey[0]);
                if (this.keys()[newIndex] && currentIndex === foundIndex) {
                    this.selectKey(this.keys()[newIndex]);
                }
                else {
                    this.enableEditor(false);
                }
            }
        };
        configuration.prototype.addKey = function (key) {
            var foundKey = this.keys().find(function (configKey) { return configKey.key === key; });
            if (!foundKey) {
                var newKey = new configurationKey(this.activeFilesystem(), key);
                this.keys.push(newKey);
                if (this.keys().length > 0 && !this.enabled) {
                    this.enableEditor(true);
                }
                return newKey;
            }
            return foundKey;
        };
        configuration.prototype.newConfigurationKey = function () {
            var _this = this;
            var createConfigurationKeyViewModel = new createConfigurationKey(this.keys());
            createConfigurationKeyViewModel
                .creationTask
                .done(function (key) {
                new saveConfigurationCommand(_this.activeFilesystem(), new configurationKey(_this.activeFilesystem(), key), JSON.parse("{}")).execute()
                    .done(function () {
                    var newKey = _this.addKey(key);
                    _this.selectKey(newKey);
                })
                    .fail(function (qXHR, textStatus, errorThrown) { return messagePublisher.reportError("Could not create Configuration Key!", errorThrown); });
            });
            app.showBootstrapDialog(createConfigurationKeyViewModel);
        };
        return configuration;
    }(viewModelBase));
    configuration.configSelector = "#settingsContainer";
    return configuration;
});

//# sourceMappingURL=configuration.js.map
