var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/dialog", "commands/filesystem/deleteConfigurationKeyCommand", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, deleteConfigurationKeyCommand, dialogViewModelBase) {
    "use strict";
    var deleteConfigurationKeys = (function (_super) {
        __extends(deleteConfigurationKeys, _super);
        function deleteConfigurationKeys(fs, keys, elementToFocusOnDismissal) {
            var _this = _super.call(this, elementToFocusOnDismissal) || this;
            _this.fs = fs;
            _this.keys = ko.observableArray();
            _this.deletionStarted = false;
            _this.deletionTask = $.Deferred(); // Gives consumers a way to know when the async delete operation completes.
            if (keys.length === 0) {
                throw new Error("Must have at least one key to delete.");
            }
            _this.keys(keys);
            return _this;
        }
        deleteConfigurationKeys.prototype.deleteKeys = function () {
            var _this = this;
            this.deletionStarted = true;
            var deleteItemsIds = this.keys().map(function (i) { return i.key; });
            var deletionTasks = [];
            for (var i = 0; i < deleteItemsIds.length; i++) {
                deletionTasks.push(new deleteConfigurationKeyCommand(this.fs, deleteItemsIds[i]).execute());
            }
            var combinedTask = $.when.apply($, deletionTasks);
            combinedTask
                .done(function () { return _this.deletionTask.resolve(_this.keys()); })
                .fail(function (response) { return _this.deletionTask.reject(response); });
            dialog.close(this);
        };
        deleteConfigurationKeys.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteConfigurationKeys.prototype.deactivate = function (args) {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never carried it out.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteConfigurationKeys;
    }(dialogViewModelBase));
    return deleteConfigurationKeys;
});

//# sourceMappingURL=deleteConfigurationKeys.js.map
