var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "commands/filesystem/getFileCommand", "commands/filesystem/updateFileMetadataCommand", "viewmodels/viewModelBase", "viewmodels/common/deleteItems", "viewmodels/filesystem/files/fileRenameDialog", "common/bindingHelpers/aceEditorBindingHandler", "common/eventsCollector"], function (require, exports, app, router, appUrl, getFileCommand, updateFileMetadataCommand, viewModelBase, deleteItems, fileRenameDialog, aceEditorBindingHandler, eventsCollector) {
    "use strict";
    var filesystemEditFile = (function (_super) {
        __extends(filesystemEditFile, _super);
        function filesystemEditFile() {
            var _this = _super.call(this) || this;
            _this.fileName = ko.observable();
            _this.file = ko.observable();
            _this.topRecentFiles = ko.computed(function () { return _this.getTopRecentFiles(); });
            _this.fileMetadataText = ko.observable();
            _this.isBusy = ko.observable(false);
            _this.metaPropsToRestoreOnSave = [];
            aceEditorBindingHandler.install();
            _this.fileName.subscribe(function (x) { return _this.loadFile(x); });
            _this.isSaveEnabled = ko.pureComputed(function () { return _this.dirtyFlag().isDirty(); });
            return _this;
        }
        filesystemEditFile.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("RJBNGR");
            this.metadata = ko.computed(function () { return _this.file() ? _this.file().__metadata : null; });
            this.filesystemForEditedFile = this.activeFilesystem();
            if (args.id != null) {
                this.appendRecentFile(args.id);
                this.fileName(args.id);
            }
            this.dirtyFlag = new ko.DirtyFlag([this.fileMetadataText]);
            this.metadata.subscribe(function (meta) { return _this.metadataChanged(meta); });
        };
        filesystemEditFile.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.setupKeyboardShortcuts();
        };
        filesystemEditFile.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#fileMetadataEditor");
            if (editorElement.length > 0) {
                this.metadataEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            this.focusOnEditor();
        };
        filesystemEditFile.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteFile(); }, filesystemEditFile.editFileSelector);
        };
        filesystemEditFile.prototype.focusOnEditor = function () {
            this.metadataEditor.focus();
        };
        filesystemEditFile.prototype.loadFile = function (fileName) {
            var _this = this;
            new getFileCommand(this.activeFilesystem(), fileName)
                .execute()
                .done(function (result) {
                _this.file(result);
                _this.dirtyFlag().reset();
            });
        };
        filesystemEditFile.prototype.navigateToFiles = function () {
            var filesUrl = appUrl.forFilesystemFiles(this.activeFilesystem());
            router.navigate(filesUrl);
        };
        filesystemEditFile.prototype.saveFileMetadata = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-file", "save");
            //the name of the document was changed and we have to save it as a new one
            var meta = JSON.parse(this.fileMetadataText());
            this.metaPropsToRestoreOnSave.forEach(function (p) { return meta[p.name] = p.value; });
            var saveCommand = new updateFileMetadataCommand(this.fileName(), meta, this.activeFilesystem(), true);
            var saveTask = saveCommand.execute();
            saveTask.done(function () {
                _this.loadFile(_this.fileName());
            });
        };
        filesystemEditFile.prototype.downloadFile = function () {
            eventsCollector.default.reportEvent("fs-file", "download");
            var fs = this.activeFilesystem();
            var fileName = this.fileName();
            var url = appUrl.forResourceQuery(fs) + "/files/" + encodeURIComponent(fileName);
            this.downloader.download(fs, url);
        };
        filesystemEditFile.prototype.refreshFile = function () {
            eventsCollector.default.reportEvent("fs-file", "refresh");
            this.loadFile(this.fileName());
        };
        filesystemEditFile.prototype.deleteFile = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-file", "delete");
            var file = this.file();
            if (file) {
                var viewModel = new deleteItems([file], this.activeFilesystem());
                viewModel.deletionTask.done(function () {
                    var filesUrl = appUrl.forFilesystemFiles(_this.activeFilesystem());
                    router.navigate(filesUrl);
                });
                app.showBootstrapDialog(viewModel, filesystemEditFile.editFileSelector);
            }
            this.dirtyFlag().reset(); // Resync Changes
        };
        filesystemEditFile.prototype.renameFile = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-file", "rename");
            var currentFileName = this.fileName();
            var dialog = new fileRenameDialog(currentFileName, this.activeFilesystem());
            dialog.onExit().done(function (newName) {
                _this.removeFromTopRecentFiles(currentFileName);
                router.navigate(appUrl.forEditFile(newName, _this.activeFilesystem()));
            });
            app.showBootstrapDialog(dialog);
        };
        filesystemEditFile.prototype.removeFromTopRecentFiles = function (fileName) {
            var currentFilesystemName = this.activeFilesystem().name;
            var recentFilesForCurFilesystem = filesystemEditFile.recentDocumentsInFilesystem().find(function (x) { return x.filesystemName === currentFilesystemName; });
            if (recentFilesForCurFilesystem) {
                recentFilesForCurFilesystem.recentFiles.remove(fileName);
            }
        };
        filesystemEditFile.prototype.getTopRecentFiles = function () {
            var _this = this;
            var currentFilesystemName = this.activeFilesystem().name;
            var recentFilesForCurFilesystem = filesystemEditFile.recentDocumentsInFilesystem().find(function (x) { return x.filesystemName === currentFilesystemName; });
            if (recentFilesForCurFilesystem) {
                var value = recentFilesForCurFilesystem
                    .recentFiles()
                    .filter(function (x) {
                    return x !== _this.fileName();
                })
                    .slice(0, 5)
                    .map(function (fileId) {
                    return {
                        fileId: fileId,
                        fileUrl: appUrl.forEditFile(fileId, _this.activeFilesystem())
                    };
                });
                return value;
            }
            else {
                return [];
            }
        };
        filesystemEditFile.prototype.metadataChanged = function (meta) {
            if (meta) {
                //this.metaPropsToRestoreOnSave.length = 0;
                var metaDto = this.metadata().toDto();
                // We don't want to show certain reserved properties in the metadata text area.
                // Remove them from the DTO, restore them on save.
                var metaPropsToRemove = ["Raven-Last-Modified", "Raven-Creation-Date", "Last-Modified", "Creation-Date", "ETag", "RavenFS-Size"];
                for (var property in metaDto) {
                    if (metaDto.hasOwnProperty(property) && _.includes(metaPropsToRemove, property)) {
                        var value = metaDto[property];
                        if (typeof (value) != "string" && typeof (value) != "number") {
                            this.metaPropsToRestoreOnSave.push({ name: property, value: JSON.stringify(value) });
                        }
                        else {
                            this.metaPropsToRestoreOnSave.push({ name: property, value: metaDto[property].toString() });
                        }
                        delete metaDto[property];
                    }
                }
                var metaString = this.stringify(metaDto);
                this.fileMetadataText(metaString);
            }
        };
        filesystemEditFile.prototype.appendRecentFile = function (fileId) {
            var _this = this;
            var existingRecentFilesStore = filesystemEditFile.recentDocumentsInFilesystem().find(function (x) { return x.filesystemName === _this.filesystemForEditedFile.name; });
            if (existingRecentFilesStore) {
                var existingDocumentInStore = existingRecentFilesStore.recentFiles().find(function (x) { return x === fileId; });
                if (!existingDocumentInStore) {
                    if (existingRecentFilesStore.recentFiles().length === 5) {
                        existingRecentFilesStore.recentFiles.pop();
                    }
                    existingRecentFilesStore.recentFiles.unshift(fileId);
                }
            }
            else {
                filesystemEditFile.recentDocumentsInFilesystem.push({ filesystemName: this.filesystemForEditedFile.name, recentFiles: ko.observableArray([fileId]) });
            }
        };
        filesystemEditFile.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };
        return filesystemEditFile;
    }(viewModelBase));
    filesystemEditFile.editFileSelector = "#editFileContainer";
    filesystemEditFile.recentDocumentsInFilesystem = ko.observableArray();
    return filesystemEditFile;
});

//# sourceMappingURL=filesystemEditFile.js.map
