var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/router", "common/appUrl", "durandal/app", "common/pagedList", "commands/filesystem/getFilesCommand", "commands/filesystem/getFilesystemRevisionsCommand", "viewmodels/filesystem/files/createFolderInFilesystem", "common/bindingHelpers/treeBindingHandler", "viewmodels/viewModelBase", "models/filesystem/folder", "common/bindingHelpers/fileUploadBindingHandler", "common/uploadQueueHelper", "viewmodels/filesystem/deleteFilesMatchingQueryConfirm", "commands/filesystem/searchByQueryCommand", "commands/filesystem/getFileSystemStatsCommand", "viewmodels/filesystem/files/filesystemEditFile", "viewmodels/filesystem/files/fileRenameDialog", "common/queryUtil", "common/eventsCollector"], function (require, exports, router, appUrl, app, pagedList, getFilesystemFilesCommand, getFilesystemRevisionsCommand, createFolderInFilesystem, treeBindingHandler, viewModelBase, folder, fileUploadBindingHandler, uploadQueueHelper, deleteFilesMatchingQueryConfirm, searchByQueryCommand, getFileSystemStatsCommand, filesystemEditFile, fileRenameDialog, queryUtil, eventsCollector) {
    "use strict";
    var filesystemFiles = (function (_super) {
        __extends(filesystemFiles, _super);
        function filesystemFiles() {
            var _this = _super.call(this) || this;
            _this.allFilesPagedItems = ko.observable();
            _this.selectedFilesIndices = ko.observableArray();
            _this.selectedFolder = ko.observable();
            _this.addedFolder = ko.observable();
            _this.currentLevelSubdirectories = ko.observableArray();
            _this.uploadFiles = ko.observable();
            _this.uploadQueue = ko.observableArray();
            _this.folderNotificationSubscriptions = {};
            _this.isAnyFilesAutoSelected = ko.observable(false);
            _this.isAllFilesAutoSelected = ko.observable(false);
            _this.inRevisionsFolder = ko.observable(false);
            _this.showLoadingIndicator = ko.observable(false);
            _this.showLoadingIndicatorThrottled = _this.showLoadingIndicator.throttle(250);
            _this.uploadQueue.subscribe(function (x) { return _this.newUpload(x); });
            fileUploadBindingHandler.install();
            treeBindingHandler.install();
            treeBindingHandler.includeRevisionsFunc = function () { return _.includes(_this.activeFilesystem().activeBundles(), "Versioning"); };
            _this.selectedFilesText = ko.computed(function () {
                if (!!_this.selectedFilesIndices()) {
                    var documentsText = "file";
                    if (_this.selectedFilesIndices().length !== 1) {
                        documentsText += "s";
                    }
                    return documentsText;
                }
                return "";
            });
            _this.filesCount = ko.computed(function () {
                if (!!_this.allFilesPagedItems()) {
                    var p = _this.allFilesPagedItems();
                    return p.totalResultCount();
                }
                return 0;
            });
            _this.selectedFolderName = ko.computed(function () {
                if (!_this.selectedFolder())
                    return "Root Folder";
                var splittedName = _this.selectedFolder().split("/");
                return _.last(splittedName);
            });
            _this.hasFiles = ko.computed(function () {
                if (!!_this.allFilesPagedItems()) {
                    var p = _this.allFilesPagedItems();
                    return p.totalResultCount() > 0;
                }
                return false;
            });
            _this.hasAllFilesSelected = ko.computed(function () {
                var filesCount = _this.filesCount();
                return filesCount > 0 && filesCount === _this.selectedFilesIndices().length;
            });
            _this.filesSelection = ko.computed(function () {
                var selected = _this.selectedFilesIndices();
                if (_this.hasAllFilesSelected()) {
                    return 2 /* Checked */;
                }
                if (selected.length > 0) {
                    return 1 /* SomeChecked */;
                }
                return 0 /* UnChecked */;
            });
            _this.anyUploadInProgess = ko.pureComputed(function () {
                var queue = _this.uploadQueue();
                for (var i = 0; i < queue.length; i++) {
                    if (queue[i].status() === uploadQueueHelper.uploadingStatus) {
                        return true;
                    }
                }
                return false;
            });
            _this.uploadsStatus = ko.pureComputed(function () {
                var queue = _this.uploadQueue();
                if (queue.length === 0) {
                    return 'panel-info';
                }
                var allSuccess = true;
                for (var i = 0; i < queue.length; i++) {
                    if (queue[i].status() === uploadQueueHelper.failedStatus) {
                        return 'panel-danger';
                    }
                    if (queue[i].status() !== uploadQueueHelper.uploadedStatus) {
                        allSuccess = false;
                    }
                }
                return allSuccess ? 'panel-success' : 'panel-info';
            });
            return _this;
        }
        filesystemFiles.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("Y1TNKH");
            this.appUrls = appUrl.forCurrentFilesystem();
            this.hasAnyFilesSelected = ko.computed(function () { return _this.selectedFilesIndices().length > 0; });
            this.loadFiles();
            this.selectedFolder.subscribe(function (newValue) { return _this.folderChanged(newValue); });
            var storageKeyForFs = uploadQueueHelper.localStorageUploadQueueKey + this.activeFilesystem().name;
            if (window.localStorage.getItem(storageKeyForFs)) {
                this.uploadQueue(uploadQueueHelper.parseUploadQueue(window.localStorage.getItem(storageKeyForFs), this.activeFilesystem()));
            }
        };
        filesystemFiles.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.collapseUploadQueuePanel();
        };
        filesystemFiles.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
        };
        filesystemFiles.prototype.loadFiles = function () {
            this.allFilesPagedItems(this.createPagedList(this.selectedFolder()));
        };
        filesystemFiles.prototype.folderChanged = function (newFolder) {
            this.loadFiles();
            this.inRevisionsFolder(newFolder === filesystemFiles.revisionsFolderId);
            // treat notifications events
            if (!newFolder) {
                newFolder = "/";
            }
            /* TODO
            if (!this.folderNotificationSubscriptions[newFolder] && changesContext.currentResourceChangesApi() != null) {
                this.folderNotificationSubscriptions[newFolder] = changesContext.currentResourceChangesApi()
                    .watchFsFolders(newFolder, (e: fileChangeNotification) => {
                        var callbackFolder = new folder(newFolder);
    
                        switch (e.Action) {
    
                        case "Add":
                        {
                            var eventFolder = folder.getFolderFromFilePath(e.File);
    
                            if (!eventFolder || !treeBindingHandler.isNodeExpanded(filesystemFiles.treeSelector, callbackFolder.path)) {
                                return;
                            }
    
                            //check if the file is new at the folder level to add it
                            if (callbackFolder.isFileAtFolderLevel(e.File)) {
                                this.loadFiles();
                            } else {
                                //check if a new folder at this level was added so we add it to the tree
                                var subPaths = eventFolder.getSubpathsFrom(callbackFolder.path);
                                if (subPaths.length > 1 && !treeBindingHandler.nodeExists(filesystemFiles.treeSelector, subPaths[1].path)) {
                                    var newNode = {
                                        key: subPaths[1].path,
                                        title: subPaths[1].name,
                                        isLazy: true,
                                        isFolder: true,
                                    };
                                    this.addedFolder(newNode);
                                }
                            }
    
                            break;
                        }
                        case "Delete":
                        {
                            var eventFolder = folder.getFolderFromFilePath(e.File);
    
                            //check if the file is new at the folder level to remove it from the table
                            if (callbackFolder.isFileAtFolderLevel(e.File)) {
                                this.loadFiles();
                            } else {
                                //reload node and its children
                                treeBindingHandler.reloadNode(filesystemFiles.treeSelector, callbackFolder.path);
                            }
                            break;
                        }
                        case "Renaming":
                        {
                            //nothing to do here
                        }
                        case "Renamed":
                        {
                            //reload files to load the new names
                            if (callbackFolder.isFileAtFolderLevel(e.File)) {
                                this.loadFiles();
                            }
                            break;
                        }
                        case "Update":
                        {
                            //check if the file is new at the folder level to add it
                            if (callbackFolder.isFileAtFolderLevel(e.File)) {
                                this.loadFiles();
                            }
                            break;
                        }
                        default:
                            console.error("unknown notification action");
                        }
                    });
            }*/
        };
        filesystemFiles.prototype.createPagedList = function (directory) {
            var _this = this;
            var fetcher;
            if (directory === filesystemFiles.revisionsFolderId) {
                fetcher = function (skip, take) { return _this.fetchRevisions(skip, take); };
            }
            else {
                fetcher = function (skip, take) { return _this.fetchFiles(directory, skip, take); };
            }
            var list = new pagedList(fetcher);
            return list;
        };
        filesystemFiles.prototype.fetchFiles = function (directory, skip, take) {
            var task = new getFilesystemFilesCommand(this.activeFilesystem(), directory, skip, take).execute();
            return task;
        };
        filesystemFiles.prototype.fetchRevisions = function (skip, take) {
            var task = new getFilesystemRevisionsCommand(this.activeFilesystem(), skip, take).execute();
            return task;
        };
        filesystemFiles.prototype.fileSystemChanged = function (fs) {
            if (!!fs) {
                this.loadFiles();
            }
        };
        filesystemFiles.prototype.editSelectedFile = function () {
            var grid = this.getFilesGrid();
            if (grid) {
                grid.editLastSelectedItem();
            }
        };
        filesystemFiles.prototype.toggleSelectAll = function () {
            var filesGrid = this.getFilesGrid();
            if (!!filesGrid) {
                if (this.hasAnyFilesSelected()) {
                    filesGrid.selectNone();
                }
                else {
                    filesGrid.selectSome();
                    this.isAnyFilesAutoSelected(this.hasAllFilesSelected() === false);
                }
            }
        };
        filesystemFiles.prototype.selectAll = function () {
            var filesGrid = this.getFilesGrid();
            if (!!filesGrid && !!this.allFilesPagedItems()) {
                var p = this.allFilesPagedItems();
                filesGrid.selectAll(p.totalResultCount());
            }
        };
        filesystemFiles.prototype.selectNone = function () {
            var filesGrid = this.getFilesGrid();
            if (!!filesGrid) {
                filesGrid.selectNone();
            }
        };
        filesystemFiles.prototype.getFilesGrid = function () {
            var gridContents = $(filesystemFiles.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        filesystemFiles.prototype.createFolder = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-folder", "create");
            var createFolderVm = new createFolderInFilesystem(this.currentLevelSubdirectories());
            createFolderVm.creationTask.done(function (folderName) {
                var parentDirectory = _this.selectedFolder() ? _this.selectedFolder() : "";
                var newNode = {
                    key: parentDirectory + "/" + folderName,
                    title: folderName,
                    isLazy: true,
                    isFolder: true,
                    addClass: treeBindingHandler.transientNodeStyle
                };
                _this.addedFolder(newNode);
            });
            app.showBootstrapDialog(createFolderVm);
        };
        filesystemFiles.prototype.refresh = function () {
            eventsCollector.default.reportEvent("fs-files", "refresh");
            var grid = this.getFilesGrid();
            if (grid) {
                grid.refreshCollectionData();
            }
            this.selectNone();
        };
        filesystemFiles.prototype.deleteSelectedFiles = function () {
            eventsCollector.default.reportEvent("fs-files", "delete");
            if (this.hasAllFilesSelected()) {
                this.deleteFolder(false);
            }
            else {
                var grid = this.getFilesGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        filesystemFiles.prototype.downloadSelectedFiles = function () {
            eventsCollector.default.reportEvent("fs-file", "download");
            var grid = this.getFilesGrid();
            if (grid) {
                var selectedItem = grid.getSelectedItems(1)[0];
                var fs = this.activeFilesystem();
                var url = appUrl.forResourceQuery(fs) + "/files/" + encodeURIComponent(selectedItem.getUrl());
                this.downloader.download(fs, url);
            }
        };
        filesystemFiles.prototype.renameSelectedFile = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-file", "rename");
            var grid = this.getFilesGrid();
            if (grid) {
                var selectedItem = grid.getSelectedItems(1)[0];
                var currentFileName = selectedItem.getId();
                var dialog = new fileRenameDialog(currentFileName, this.activeFilesystem());
                dialog.onExit().done(function (newName) {
                    var currentFilesystemName = _this.activeFilesystem().name;
                    var recentFilesForCurFilesystem = filesystemEditFile.recentDocumentsInFilesystem().find(function (x) { return x.filesystemName === currentFilesystemName; });
                    if (recentFilesForCurFilesystem) {
                        recentFilesForCurFilesystem.recentFiles.remove(currentFileName);
                    }
                    selectedItem.setId(newName);
                    grid.refreshCollectionData();
                });
                app.showBootstrapDialog(dialog);
            }
        };
        filesystemFiles.prototype.clearUploadQueue = function () {
            eventsCollector.default.reportEvent("fs-files", "clear-queue");
            window.localStorage.removeItem(uploadQueueHelper.localStorageUploadQueueKey + this.activeFilesystem().name);
            this.uploadQueue.removeAll();
        };
        filesystemFiles.prototype.navigateToFiles = function () {
            router.navigate(appUrl.forFilesystemFiles(this.activeFilesystem()));
        };
        filesystemFiles.prototype.uploadSuccess = function (x) {
            ko.postbox.publish("UploadFileStatusChanged", x);
            uploadQueueHelper.updateQueueStatus(x.id(), uploadQueueHelper.uploadedStatus, this.uploadQueue());
            this.uploadQueue(uploadQueueHelper.sortUploadQueue(this.uploadQueue()));
            var persistedFolder = folder.getFolderFromFilePath(x.fileName());
            if (persistedFolder) {
                treeBindingHandler.updateNodeHierarchyStyle(filesystemFiles.treeSelector, persistedFolder.path, "");
                treeBindingHandler.setNodeLoadStatus(filesystemFiles.treeSelector, persistedFolder.path, 0);
            }
            this.loadFiles();
        };
        filesystemFiles.prototype.uploadFailed = function (x) {
            ko.postbox.publish("UploadFileStatusChanged", x);
            uploadQueueHelper.updateQueueStatus(x.id(), uploadQueueHelper.failedStatus, this.uploadQueue());
            this.uploadQueue(uploadQueueHelper.sortUploadQueue(this.uploadQueue()));
        };
        filesystemFiles.prototype.newUpload = function (x) {
            if (x && x.length > 0) {
                uploadQueueHelper.updateLocalStorage(x, this.activeFilesystem());
                var waitingItems = x.filter(function (x) { return x.status() === uploadQueueHelper.queuedStatus || x.status() === uploadQueueHelper.uploadingStatus; });
                for (var i = 0; i < waitingItems.length; i++) {
                    var persistedFolder = folder.getFolderFromFilePath(waitingItems[i].fileName());
                    treeBindingHandler.setNodeLoadStatus(filesystemFiles.treeSelector, persistedFolder.path, 1);
                }
            }
        };
        filesystemFiles.prototype.toggleCollapseUploadQueue = function () {
            if ($(filesystemFiles.uploadQueuePanelToggleSelector).hasClass('opened')) {
                this.collapseUploadQueuePanel();
            }
            else {
                this.expandUploadQueuePanel();
            }
        };
        filesystemFiles.prototype.expandUploadQueuePanel = function () {
            $(filesystemFiles.uploadQueuePanelToggleSelector).addClass("opened").find('i').removeClass('fa-angle-double-up').addClass('fa-angle-double-down');
            $(filesystemFiles.uploadQueueSelector).removeClass("hidden");
            $(filesystemFiles.uploadQueuePanelCollapsedSelector).addClass("hidden");
            $(".upload-queue").removeClass("upload-queue-min");
        };
        filesystemFiles.prototype.collapseUploadQueuePanel = function () {
            $(filesystemFiles.uploadQueuePanelToggleSelector).removeClass('opened').find('i').removeClass('fa-angle-double-down').addClass('fa-angle-double-up');
            $(filesystemFiles.uploadQueueSelector).addClass("hidden");
            $(filesystemFiles.uploadQueuePanelCollapsedSelector).removeClass("hidden");
            $(".upload-queue").addClass("upload-queue-min");
        };
        filesystemFiles.prototype.deleteFolder = function (recursive) {
            var _this = this;
            if (recursive === void 0) { recursive = true; }
            eventsCollector.default.reportEvent("fs-folder", "delete");
            if (!this.selectedFolder() && recursive) {
                // delete all files from filesystem
                new getFileSystemStatsCommand(this.activeFilesystem())
                    .execute()
                    .done(function (fs) {
                    _this.promptDeleteFilesMatchingQuery(fs.FileCount, "");
                });
            }
            else {
                // Run the query so that we have an idea of what we'll be deleting.
                var query;
                if (recursive) {
                    query = "__directoryName:" + queryUtil.escapeTerm(this.selectedFolder());
                }
                else {
                    var folder = !this.selectedFolder() ? "/" : this.selectedFolder();
                    query = "__directory:" + queryUtil.escapeTerm(folder);
                }
                new searchByQueryCommand(this.activeFilesystem(), query, 0, 1)
                    .execute()
                    .done(function (results) {
                    if (results.totalResultCount === 0) {
                        app.showBootstrapMessage("There are no files matching your query.", "Nothing to do");
                    }
                    else {
                        _this.promptDeleteFilesMatchingQuery(results.totalResultCount, query);
                    }
                });
            }
        };
        filesystemFiles.prototype.promptDeleteFilesMatchingQuery = function (resultCount, query) {
            var _this = this;
            var viewModel = new deleteFilesMatchingQueryConfirm(query, resultCount, this.activeFilesystem());
            app.showBootstrapDialog(viewModel);
            viewModel.deletionTask.done(function () { return _this.loadFiles(); });
        };
        return filesystemFiles;
    }(viewModelBase));
    filesystemFiles.revisionsFolderId = "/$$revisions$$";
    filesystemFiles.treeSelector = "#filesTree";
    filesystemFiles.gridSelector = "#filesGrid";
    filesystemFiles.uploadQueuePanelToggleSelector = "#uploadQueuePanelToggle";
    filesystemFiles.uploadQueueSelector = "#uploadQueue";
    filesystemFiles.uploadQueuePanelCollapsedSelector = "#uploadQueuePanelCollapsed";
    return filesystemFiles;
});

//# sourceMappingURL=filesystemFiles.js.map
