var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/dialogViewModelBase", "plugins/dialog"], function (require, exports, dialogViewModelBase, dialog) {
    "use strict";
    /*
     * Base view model for search dialogs.
     * Extends from dialogViewModelBase
     * Provides some extra functionality:
     *    - Listens to Enter key and submits the form only if all the fields have been completed
     */
    var searchDialogViewModel = (function (_super) {
        __extends(searchDialogViewModel, _super);
        function searchDialogViewModel(inputs) {
            var _this = _super.call(this) || this;
            _this.inputs = inputs;
            return _this;
        }
        searchDialogViewModel.prototype.close = function () {
            dialog.close(this);
        };
        searchDialogViewModel.prototype.enterKeyPressed = function () {
            if (this.inputs == null)
                return false;
            var submit = this.enabled();
            if (submit) {
                _super.prototype.enterKeyPressed.call(this);
            }
            return submit;
        };
        searchDialogViewModel.prototype.enabled = function () {
            return this.checkRequired(true);
        };
        // check inputs 
        searchDialogViewModel.prototype.checkRequired = function (allRequired) {
            if (allRequired) {
                var result = true;
                this.inputs.forEach(function (input) {
                    if (input() == null || !input().trim()) {
                        result = false;
                    }
                });
                return result;
            }
            else {
                var result = false;
                this.inputs.forEach(function (input) {
                    if (input() != null && input().trim()) {
                        result = true;
                    }
                });
                return result;
            }
        };
        return searchDialogViewModel;
    }(dialogViewModelBase));
    return searchDialogViewModel;
});

//# sourceMappingURL=searchDialogViewModel.js.map
