var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "commands/filesystem/searchByQueryCommand", "common/pagedList", "viewmodels/filesystem/files/searchInFolderClause", "viewmodels/filesystem/files/searchSingleInputClause", "viewmodels/filesystem/files/searchFileSizeRangeClause", "viewmodels/filesystem/files/searchHasMetadataClause", "viewmodels/filesystem/files/searchLastModifiedBetweenClause", "viewmodels/filesystem/deleteFilesMatchingQueryConfirm", "viewmodels/filesystem/search/resetIndexConfirm", "common/queryUtil", "common/eventsCollector"], function (require, exports, app, router, appUrl, viewModelBase, searchByQueryCommand, pagedList, searchInFolderClause, searchSingleInputClause, searchFileSizeRangeClause, searchHasMetadataClause, searchLastModifiedBetweenClause, deleteFilesMatchingQueryConfirm, resetIndexConfirm, queryUtil, eventsCollector) {
    "use strict";
    var search = (function (_super) {
        __extends(search, _super);
        function search() {
            var _this = _super.call(this) || this;
            _this.router = router;
            _this.searchUrl = appUrl.forCurrentDatabase().filesystemSearch;
            _this.searchText = ko.observable("");
            _this.allFilesPagedItems = ko.observable();
            _this.selectedFilesIndices = ko.observableArray();
            _this.searchText.extend({ throttle: 200 }).subscribe(function (s) { return _this.searchFiles(s); });
            return _this;
        }
        search.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("SRTQ8C");
            this.appUrls = appUrl.forCurrentFilesystem();
            this.loadFiles();
        };
        search.prototype.clear = function () {
            this.searchText("");
        };
        search.prototype.search = function () {
            this.searchFiles(this.searchText());
        };
        search.prototype.searchFiles = function (query) {
            this.allFilesPagedItems(this.createPagedList(query));
        };
        search.prototype.loadFiles = function () {
            this.allFilesPagedItems(this.createPagedList(""));
        };
        search.prototype.createPagedList = function (query) {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchFiles(query, skip, take); };
            return new pagedList(fetcher);
        };
        search.prototype.fetchFiles = function (query, skip, take) {
            var task = new searchByQueryCommand(this.activeFilesystem(), query, skip, take).execute();
            return task;
        };
        search.prototype.fileNameStartsWith = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-files", "search", "starts-with");
            var searchSingleInputClauseViewModel = new searchSingleInputClause("Filename starts with: ");
            searchSingleInputClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput("__fileName:" + queryUtil.escapeTerm(input) + "*"); });
            app.showBootstrapDialog(searchSingleInputClauseViewModel);
        };
        search.prototype.fileNameEndsWith = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-files", "search", "ends-with");
            var searchSingleInputClauseViewModel = new searchSingleInputClause("Filename ends with: ");
            searchSingleInputClauseViewModel
                .applyFilterTask
                .done(function (input) {
                var stringReversed = input.split("").reverse().join("");
                _this.addToSearchInput("__rfileName:" + queryUtil.escapeTerm(stringReversed) + "*");
            });
            app.showBootstrapDialog(searchSingleInputClauseViewModel);
        };
        search.prototype.fileSizeBetween = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-files", "search", "size-between");
            var searchFileSizeRangeClauseViewModel = new searchFileSizeRangeClause();
            searchFileSizeRangeClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput(input); });
            app.showBootstrapDialog(searchFileSizeRangeClauseViewModel);
        };
        search.prototype.hasMetadata = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-files", "search", "has-metadata");
            var searchHasMetadataClauseViewModel = new searchHasMetadataClause(this.activeFilesystem());
            searchHasMetadataClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput(queryUtil.escapeTerm(input)); });
            app.showBootstrapDialog(searchHasMetadataClauseViewModel);
        };
        search.prototype.inFolder = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-files", "search", "in-folder");
            var inFolderViewModel = new searchInFolderClause(this.activeFilesystem());
            inFolderViewModel
                .applyFilterTask
                .done(function (input) {
                if (!input.startsWith("/")) {
                    input = "/" + input;
                }
                var escaped = queryUtil.escapeTerm(input);
                _this.addToSearchInput("__directoryName:" + escaped);
            });
            app.showBootstrapDialog(inFolderViewModel);
        };
        search.prototype.lastModifiedBetween = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-files", "search", "modified-between");
            var searchLastModifiedBetweenClauseViewModel = new searchLastModifiedBetweenClause();
            searchLastModifiedBetweenClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput(input); });
            app.showBootstrapDialog(searchLastModifiedBetweenClauseViewModel);
        };
        search.prototype.addToSearchInput = function (input) {
            var currentSearchText = this.searchText();
            if (currentSearchText != null && currentSearchText.trim().length > 0)
                currentSearchText += " AND ";
            this.searchText(currentSearchText + input);
        };
        search.prototype.deleteFilesMatchingQuery = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-files", "delete");
            // Run the query so that we have an idea of what we'll be deleting.
            this.search();
            this.allFilesPagedItems()
                .fetch(0, 1)
                .done(function (results) {
                if (results.totalResultCount === 0) {
                    app.showBootstrapMessage("There are no files matching your query.", "Nothing to do");
                }
                else {
                    _this.promptDeleteFilesMatchingQuery(results.totalResultCount);
                }
            });
        };
        search.prototype.promptDeleteFilesMatchingQuery = function (resultCount) {
            var _this = this;
            var viewModel = new deleteFilesMatchingQueryConfirm(this.searchText(), resultCount, this.activeFilesystem());
            app
                .showBootstrapDialog(viewModel)
                .done(function () { return _this.search(); });
        };
        search.prototype.resetIndex = function () {
            eventsCollector.default.reportEvent("fs-index", "reset");
            var resetIndexVm = new resetIndexConfirm(this.activeFilesystem());
            app.showBootstrapDialog(resetIndexVm);
        };
        return search;
    }(viewModelBase));
    search.gridSelector = "#filesGrid";
    return search;
});

//# sourceMappingURL=search.js.map
