var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase"], function (require, exports, durandalRouter, appUrl, viewModelBase) {
    "use strict";
    var settings = (function (_super) {
        __extends(settings, _super);
        function settings() {
            var _this = _super.call(this) || this;
            _this.router = null;
            _this.bundleMap = { versioning: "Versioning" };
            _this.userDatabasePages = ko.observableArray([]);
            _this.appUrls = appUrl.forCurrentDatabase();
            var versioningRoute = { route: 'filesystems/settings', moduleId: 'viewmodels/filesystem/settings/versioning', title: 'Versioning', nav: true, hash: appUrl.forCurrentFilesystem().filesystemVersioning };
            _this.router = durandalRouter.createChildRouter()
                .map([
                versioningRoute
            ])
                .buildNavigationModel();
            _this.router.guardRoute = function (instance, instruction) { return _this.getValidRoute(instance, instruction); };
            appUrl.mapUnknownRoutes(_this.router);
            _this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().find(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
            return _this;
        }
        /**
        * Checks whether the route can be navigated to. Returns true if it can be navigated to, or a redirect URI if it can't be navigated to.
        * This is used for preventing a navigating to system-only pages when the current databagse is non-system, and vice-versa.
        */
        settings.prototype.getValidRoute = function (instance, instruction) {
            var fs = this.activeFilesystem();
            var pathArr = instruction.fragment.split('/');
            var bundelName = pathArr[pathArr.length - 1].toLowerCase();
            var isLegalBundelName = (this.bundleMap[bundelName] != undefined);
            var isBundleExists = this.userDatabasePages.indexOf(this.bundleMap[bundelName]) > -1;
            if (isLegalBundelName && isBundleExists == false) {
                return appUrl.forCurrentDatabase().filesystemSettings();
            }
            return true;
        };
        settings.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.userDatabasePages([]);
            var fs = this.activeFilesystem();
            var bundles = fs.activeBundles();
            bundles.forEach(function (bundle) {
                var bundleName = _this.bundleMap[bundle.toLowerCase()];
                if (bundleName != undefined) {
                    _this.userDatabasePages.push(bundleName);
                }
            });
        };
        settings.prototype.routeIsVisible = function (route) {
            var bundleTitle = route.title;
            if (this.userDatabasePages.indexOf(bundleTitle) !== -1) {
                return true;
            }
            return false;
        };
        return settings;
    }(viewModelBase));
    return settings;
});

//# sourceMappingURL=settings.js.map
