define(["require", "exports", "commands/filesystem/getSyncOutgoingActivitiesCommand", "commands/filesystem/getSyncIncomingActivitiesCommand"], function (require, exports, getSyncOutgoingActivitiesCommand, getSyncIncomingActivitiesCommand) {
    "use strict";
    var activityItems = (function () {
        function activityItems(fs, activity, direction) {
            var _this = this;
            this.fs = fs;
            this.activity = activity;
            this.direction = direction;
            this.pageSize = 50;
            this.totalCount = ko.observable();
            this.pageIndex = ko.observable(0);
            this.items = ko.observableArray();
            this.maxPageIndex = ko.computed(function () {
                return Math.ceil(_this.totalCount() / _this.pageSize) - 1;
            });
            this.allPages = ko.computed(function () {
                var pages = [];
                for (var i = 0; i <= _this.maxPageIndex(); i++) {
                    pages.push({ pageNumber: (i + 1) });
                }
                return pages;
            });
            this.retrieveItems();
        }
        activityItems.prototype.moveToPage = function (index) {
            this.pageIndex(index);
            this.retrieveItems();
        };
        ;
        activityItems.prototype.refresh = function () {
            this.retrieveItems();
        };
        activityItems.prototype.retrieveItems = function () {
            var _this = this;
            var start = this.pageIndex() * this.pageSize;
            var command;
            switch (this.direction) {
                case 0 /* Outgoing */:
                    command = getSyncOutgoingActivitiesCommand;
                    break;
                case 1 /* Incoming */:
                    command = getSyncIncomingActivitiesCommand;
                    break;
                default:
                    throw new TypeError("Not supported synchronization direction type when attempting. Given type: " + this.direction);
            }
            new command(this.fs, this.activity, start, this.pageSize).execute()
                .done(function (x) {
                _this.totalCount(x.TotalCount);
                _this.items.removeAll();
                for (var i = 0; i < x.Items.length; i++) {
                    _this.items.push(x.Items[i]);
                }
            });
        };
        return activityItems;
    }());
    return activityItems;
});

//# sourceMappingURL=activityItems.js.map
