var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/appUrl", "viewmodels/viewModelBase", "commands/filesystem/synchronizeNowCommand", "viewmodels/filesystem/status/activityItems", "commands/filesystem/getConfigurationsByPrefixCommand", "commands/filesystem/getConfigurationByKeyCommand", "viewmodels/filesystem/status/sourceSynchronizationInformation", "common/eventsCollector"], function (require, exports, appUrl, viewModelBase, synchronizeNowCommand, activityItems, getConfigurationNamesByPrefixCommand, getConfigurationByKeyCommand, sourceSynchronizationInformation, eventsCollector) {
    "use strict";
    var status = (function (_super) {
        __extends(status, _super);
        function status() {
            var _this = _super.apply(this, arguments) || this;
            _this.sourceSynchronizationInformations = ko.observableArray();
            _this.isFsSyncUpToDate = true;
            return _this;
        }
        status.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("T6FI4K");
            this.appUrls = appUrl.forCurrentFilesystem();
            this.pendingOutgoing = new activityItems(this.activeFilesystem(), 2 /* Pending */, 0 /* Outgoing */);
            this.activeOutgoing = new activityItems(this.activeFilesystem(), 1 /* Active */, 0 /* Outgoing */);
            this.finishedIncoming = new activityItems(this.activeFilesystem(), 3 /* Finished */, 1 /* Incoming */);
            this.activeIncoming = new activityItems(this.activeFilesystem(), 1 /* Active */, 1 /* Incoming */);
            this.retrieveLastSynchronizedEtags();
        };
        status.prototype.createNotifications = function () {
            return [];
        };
        status.prototype.showException = function (exception) {
            var prettifySpacing = 4;
            var configText = JSON.stringify(exception, null, prettifySpacing);
            this.confirmationMessage("Exception details", configText, ["Close"]);
        };
        status.prototype.retrieveLastSynchronizedEtags = function () {
            var _this = this;
            new getConfigurationNamesByPrefixCommand(this.activeFilesystem(), "Raven/Synchronization/Sources").execute().done(function (searchResult) {
                searchResult.ConfigNames.forEach(function (name) {
                    new getConfigurationByKeyCommand(_this.activeFilesystem(), name).execute().
                        done(function (sourceSyncInfoAsString) {
                        var syncInfo = JSON.parse(sourceSyncInfoAsString);
                        var match = ko.utils.arrayFirst(_this.sourceSynchronizationInformations(), function (item) { return (syncInfo.SourceServerUrl === item.SourceServerUrl()); });
                        if (!match) {
                            _this.sourceSynchronizationInformations.push(new sourceSynchronizationInformation(syncInfo.LastSourceFileEtag, syncInfo.SourceServerUrl));
                        }
                        else {
                            match.LastSourceFileEtag(syncInfo.LastSourceFileEtag);
                        }
                    });
                });
            });
        };
        status.prototype.processFsSync = function (e) {
            // treat notifications events
            this.isFsSyncUpToDate = false;
            switch (e.Direction) {
                case "Outgoing":
                    if (e.Action === "Enqueue") {
                        this.pendingOutgoing.refresh();
                    }
                    else if (e.Action === "Start" || e.Action === "Finish") {
                        this.activeOutgoing.refresh();
                        this.pendingOutgoing.refresh();
                    }
                    break;
                case "Incoming":
                    if (e.Action === "Start") {
                        this.activeIncoming.refresh();
                    }
                    else if (e.Action === "Finish") {
                        this.activeIncoming.refresh();
                        this.finishedIncoming.refresh();
                        this.retrieveLastSynchronizedEtags();
                    }
                    break;
                default:
                    break;
            }
        };
        status.prototype.synchronizeNow = function () {
            eventsCollector.default.reportEvent("fs-status", "sync");
            var fs = this.activeFilesystem();
            if (fs) {
                new synchronizeNowCommand(fs).execute();
            }
        };
        status.prototype.getActionDescription = function (action) {
            switch (action) {
                case 0 /* Enqueue */:
                    return 2 /* Pending */;
                case 1 /* Start */:
                    return 1 /* Active */;
                case 2 /* Finish */:
                    return 3 /* Finished */;
                default:
                    return 0 /* Unknown */;
            }
        };
        return status;
    }(viewModelBase));
    return status;
});

//# sourceMappingURL=status.js.map
