var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "viewmodels/viewModelBase", "viewmodels/filesystem/synchronization/resolveConflict", "models/database/documents/customColumns", "models/database/documents/customColumnParams", "common/pagedList", "commands/filesystem/getFilesConflictsCommand", "commands/filesystem/resolveConflictCommand", "commands/filesystem/resolveConflictsCommand", "common/eventsCollector"], function (require, exports, app, viewModelBase, resolveConflict, customColumns, customColumnParams, pagedList, getFilesConflictsCommand, resolveConflictCommand, resolveConflictsCommand, eventsCollector) {
    "use strict";
    var synchronizationConflicts = (function (_super) {
        __extends(synchronizationConflicts, _super);
        function synchronizationConflicts() {
            var _this = _super.call(this) || this;
            _this.conflictStatus = {
                detected: "Detected",
                resolved: "Resolved"
            };
            _this.currentConflictsPagedItems = ko.observable();
            _this.selectedDocumentIndices = ko.observableArray();
            _this.currentColumns = ko.observable(customColumns.empty());
            _this.isAnyConflictsAutoSelected = ko.observable(false);
            _this.isAllConflictsAutoSelected = ko.observable(false);
            _this.conflictsCount = ko.computed(function () {
                if (!!_this.currentConflictsPagedItems()) {
                    var p = _this.currentConflictsPagedItems();
                    return p.totalResultCount();
                }
                return 0;
            });
            _this.selectedConflictsText = ko.computed(function () {
                if (!!_this.selectedDocumentIndices()) {
                    var documentsText = "conflict";
                    if (_this.selectedDocumentIndices().length !== 1) {
                        documentsText += "s";
                    }
                    return documentsText;
                }
                return "";
            });
            _this.hasAnyConflictsSelected = ko.computed(function () { return _this.selectedDocumentIndices().length > 0; });
            _this.hasAllConflictsSelected = ko.computed(function () {
                var filesCount = _this.conflictsCount();
                return filesCount > 0 && filesCount === _this.selectedDocumentIndices().length;
            });
            _this.conflictsSelection = ko.computed(function () {
                var selected = _this.selectedDocumentIndices();
                if (_this.hasAllConflictsSelected()) {
                    return 2 /* Checked */;
                }
                if (selected.length > 0) {
                    return 1 /* SomeChecked */;
                }
                return 0 /* UnChecked */;
            });
            return _this;
        }
        synchronizationConflicts.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("NMDELS");
            this.currentColumns().columns([
                new customColumnParams({ Header: "File Name", Binding: "fileName", DefaultWidth: 400 }),
                new customColumnParams({ Header: "Remote Server Url", Binding: "remoteServerUrl", DefaultWidth: 400 }),
                new customColumnParams({ Header: "Status", Binding: "status", DefaultWidth: 320 })
            ]);
            this.currentColumns().customMode(true);
            this.fetchConflicts(this.activeFilesystem());
        };
        synchronizationConflicts.prototype.createNotifications = function () {
            return [];
        };
        synchronizationConflicts.prototype.processFsConflicts = function (e) {
            switch (e.Status) {
                case this.conflictStatus.detected:
                case this.conflictStatus.resolved:
                    this.fetchConflicts(this.activeFilesystem());
                    break;
            }
        };
        synchronizationConflicts.prototype.fetchConflicts = function (fs) {
            this.currentConflictsPagedItems(this.createPagedList(fs));
        };
        synchronizationConflicts.prototype.createPagedList = function (fs) {
            var fetcher = function (skip, take) { return new getFilesConflictsCommand(fs, skip, take).execute(); };
            return new pagedList(fetcher);
        };
        synchronizationConflicts.prototype.selectedConflicts = function () {
            return this.getDocumentsGrid().getSelectedItems().map(function (x) { return x.getId(); });
        };
        synchronizationConflicts.prototype.getDocumentsGrid = function () {
            var gridContents = $(synchronizationConflicts.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        synchronizationConflicts.prototype.resolveWithLocalVersion = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-conflict", "resolve", "local");
            var message;
            if (this.hasAllConflictsSelected()) {
                message = "Are you sure you want to resolve all conflicts by choosing the local version?";
            }
            else if (this.selectedConflicts().length === 1) {
                message = "Are you sure you want to resolve the conflict for file <b>" +
                    this.selectedConflicts()[0] +
                    "</b> by choosing the local version?";
            }
            else {
                message = "Are you sure you want to resolve the conflict for <b>" + this.selectedConflicts().length + "</b> selected files by choosing the local version?";
            }
            var resolveConflictViewModel = new resolveConflict(message, "Resolve conflict with local");
            resolveConflictViewModel
                .resolveTask
                .done(function () {
                var fs = _this.activeFilesystem();
                if (_this.hasAllConflictsSelected()) {
                    new resolveConflictsCommand(2, fs)
                        .execute()
                        .done(function () { return _this.fetchConflicts(_this.activeFilesystem()); });
                }
                else {
                    var selectedConflicts = _this.selectedConflicts();
                    for (var i = 0; i < selectedConflicts.length; i++) {
                        var conflict = selectedConflicts[i];
                        new resolveConflictCommand(conflict, 2, fs)
                            .execute();
                    }
                }
                _this.selectNone();
            });
            app.showBootstrapDialog(resolveConflictViewModel);
        };
        synchronizationConflicts.prototype.resolveWithRemoteVersion = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-conflict", "resolve", "remote");
            var message;
            if (this.hasAllConflictsSelected()) {
                message = 'Are you sure you want to resolve all conflicts by choosing the remote version?';
            }
            else if (this.selectedConflicts().length === 1) {
                message = "Are you sure you want to resolve the conflict for file <b>" +
                    this.selectedConflicts()[0] +
                    "</b> by choosing the remote version?";
            }
            else {
                message = "Are you sure you want to resolve the conflict for <b>" + this.selectedConflicts().length + "</b> selected files by choosing the remote version?";
            }
            var resolveConflictViewModel = new resolveConflict(message, "Resolve conflict with remote");
            resolveConflictViewModel
                .resolveTask
                .done(function () {
                var fs = _this.activeFilesystem();
                if (_this.hasAllConflictsSelected()) {
                    new resolveConflictsCommand(1, fs)
                        .execute()
                        .done(function () { return _this.fetchConflicts(_this.activeFilesystem()); });
                }
                else {
                    for (var i = 0; i < _this.selectedConflicts().length; i++) {
                        var conflict = _this.selectedConflicts()[i];
                        new resolveConflictCommand(conflict, 1, fs).execute()
                            .done(function () {
                            _this.fetchConflicts(_this.activeFilesystem());
                        });
                    }
                }
                _this.selectNone();
            });
            app.showBootstrapDialog(resolveConflictViewModel);
        };
        synchronizationConflicts.prototype.toggleSelectAll = function () {
            var conflictsGrid = this.getDocumentsGrid();
            if (!!conflictsGrid) {
                if (this.hasAnyConflictsSelected()) {
                    conflictsGrid.selectNone();
                }
                else {
                    conflictsGrid.selectSome();
                    this.isAnyConflictsAutoSelected(this.hasAllConflictsSelected() === false);
                }
            }
        };
        synchronizationConflicts.prototype.selectAll = function () {
            var conflictsGrid = this.getDocumentsGrid();
            if (!!conflictsGrid && !!this.currentConflictsPagedItems()) {
                var p = this.currentConflictsPagedItems();
                conflictsGrid.selectAll(p.totalResultCount());
            }
        };
        synchronizationConflicts.prototype.selectNone = function () {
            var conflictsGrid = this.getDocumentsGrid();
            if (!!conflictsGrid) {
                conflictsGrid.selectNone();
            }
        };
        return synchronizationConflicts;
    }(viewModelBase));
    synchronizationConflicts.gridSelector = "#synchronizationConflictsGrid";
    return synchronizationConflicts;
});

//# sourceMappingURL=synchronizationConflicts.js.map
