var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/appUrl", "models/filesystem/synchronizationReplicationSetup", "models/filesystem/synchronizationDestination", "viewmodels/viewModelBase", "commands/filesystem/getDestinationsCommand", "commands/filesystem/getFileSystemStatsCommand", "commands/filesystem/saveDestinationCommand", "common/eventsCollector"], function (require, exports, appUrl, synchronizationReplicationSetup, synchronizationDestination, viewModelBase, getDestinationsCommand, getFileSystemStatsCommand, saveDestinationCommand, eventsCollector) {
    "use strict";
    var synchronizationDestinations = (function (_super) {
        __extends(synchronizationDestinations, _super);
        function synchronizationDestinations() {
            var _this = _super.apply(this, arguments) || this;
            _this.dirtyFlag = new ko.DirtyFlag([]);
            _this.replicationsSetup = ko.observable(new synchronizationReplicationSetup({ Destinations: [], Source: null }));
            _this.saveIssued = false;
            return _this;
        }
        synchronizationDestinations.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var fs = this.activeFilesystem();
            if (fs) {
                this.fetchDestinations()
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forFilesystemFiles(_this.activeFilesystem()) }); });
            }
            return deferred;
        };
        synchronizationDestinations.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("KW8LAF");
            if (!this.subscription) {
            }
            this.dirtyFlag = new ko.DirtyFlag([this.replicationsSetup]);
            this.isSaveEnabled = ko.computed(function () {
                return _this.dirtyFlag().isDirty();
            });
        };
        synchronizationDestinations.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
        };
        synchronizationDestinations.prototype.saveChanges = function () {
            var _this = this;
            eventsCollector.default.reportEvent("fs-destinations", "save");
            if (this.replicationsSetup().source()) {
                this.saveReplicationSetup();
            }
            else {
                var fs = this.activeFilesystem();
                if (fs) {
                    new getFileSystemStatsCommand(fs)
                        .execute()
                        .done(function (result) { return _this.prepareAndSaveReplicationSetup(null /* TODO */); });
                }
            }
        };
        synchronizationDestinations.prototype.prepareAndSaveReplicationSetup = function (source) {
            this.replicationsSetup().source(source);
            this.saveReplicationSetup();
        };
        synchronizationDestinations.prototype.saveReplicationSetup = function () {
            var _this = this;
            var fs = this.activeFilesystem();
            if (fs) {
                var self = this;
                this.saveIssued = true;
                new saveDestinationCommand(this.replicationsSetup().toDto(), fs)
                    .execute()
                    .done(function () {
                    console.log("Reseted dirty flag");
                    _this.dirtyFlag().reset();
                }).always(function () { return _this.saveIssued = false; });
                ;
            }
        };
        synchronizationDestinations.prototype.createNewDestination = function () {
            eventsCollector.default.reportEvent("fs-destinations", "create");
            var fs = this.activeFilesystem();
            this.replicationsSetup().destinations.unshift(synchronizationDestination.empty(fs.name));
        };
        synchronizationDestinations.prototype.removeDestination = function (repl) {
            eventsCollector.default.reportEvent("fs-destinations", "remove");
            this.replicationsSetup().destinations.remove(repl);
        };
        synchronizationDestinations.prototype.fetchDestinations = function () {
            var _this = this;
            var deferred = $.Deferred();
            var fs = this.activeFilesystem();
            if (fs) {
                new getDestinationsCommand(fs)
                    .execute()
                    .done(function (data) { return _this.replicationsSetup(new synchronizationReplicationSetup({ Destinations: data.Destinations, Source: null })); })
                    .always(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        return synchronizationDestinations;
    }(viewModelBase));
    return synchronizationDestinations;
});

//# sourceMappingURL=synchronizationDestinations.js.map
