var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "viewmodels/viewModelBase", "common/adminLogsClient", "common/fileDownloader", "viewmodels/manage/adminLogsConfigureDialog", "models/database/debug/adminLogsConfig", "models/database/debug/adminLogsConfigEntry", "viewmodels/shell/accessHelper", "common/eventsCollector"], function (require, exports, app, viewModelBase, adminLogsClient, fileDownloader, adminLogsConfigureDialog, adminLogsConfig, adminLogsConfigEntry, accessHelper, eventsCollector) {
    "use strict";
    var adminLogs = (function (_super) {
        __extends(adminLogs, _super);
        function adminLogs() {
            var _this = _super.apply(this, arguments) || this;
            _this.adminLogsClient = ko.observable(null);
            _this.pendingLogs = [];
            _this.keepDown = ko.observable(false);
            _this.rawLogs = ko.observable([]);
            _this.connected = ko.observable(false);
            _this.entriesCount = ko.computed(function () { return _this.rawLogs().length; });
            _this.adminLogsConfig = ko.observable();
            _this.isForbidden = ko.observable();
            return _this;
        }
        adminLogs.prototype.canActivate = function (args) {
            this.isForbidden(accessHelper.isGlobalAdmin() === false);
            return true;
        };
        adminLogs.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('57BGF7');
        };
        adminLogs.prototype.redraw = function () {
            if (this.pendingLogs.length > 0) {
                var pendingCopy = this.pendingLogs;
                this.pendingLogs = [];
                var logsAsText = "";
                pendingCopy.forEach(function (log) {
                    var line = log.TimeStamp + ";" + log.Level.toUpperCase() + ";" + log.Database + ";" + log.LoggerName + ";" + log.Message + (log.Exception || "") + "\n";
                    if (log.StackTrace != null) {
                        line += log.StackTrace + "\n\n";
                    }
                    logsAsText += line;
                });
                // text: allows us to escape values
                $("<div/>").text(logsAsText).appendTo("#rawLogsContainer pre");
                (_a = this.rawLogs()).push.apply(_a, pendingCopy);
                this.rawLogs.valueHasMutated();
                if (this.keepDown()) {
                    var logsPre = document.getElementById('adminLogsPre');
                    logsPre.scrollTop = logsPre.scrollHeight;
                }
            }
            var _a;
        };
        adminLogs.prototype.clearLogs = function () {
            eventsCollector.default.reportEvent("admin-logs", "clear");
            this.pendingLogs = [];
            this.rawLogs([]);
            $("#rawLogsContainer pre").empty();
        };
        adminLogs.prototype.defaultLogsConfig = function () {
            var logConfig = new adminLogsConfig();
            logConfig.maxEntries(10000);
            logConfig.entries.push(new adminLogsConfigEntry("Raven.", "Debug", false));
            return logConfig;
        };
        adminLogs.prototype.configureConnection = function () {
            var _this = this;
            eventsCollector.default.reportEvent("admin-logs", "configure");
            this.intervalId = setInterval(function () { this.redraw(); }.bind(this), 1000);
            var currentConfig = this.adminLogsConfig() ? this.adminLogsConfig().clone() : this.defaultLogsConfig();
            var adminLogsConfigViewModel = new adminLogsConfigureDialog(currentConfig);
            app.showBootstrapDialog(adminLogsConfigViewModel);
            adminLogsConfigViewModel.configurationTask.done(function (x) {
                _this.adminLogsConfig(x);
                _this.reconnect();
            });
        };
        adminLogs.prototype.connect = function () {
            var _this = this;
            eventsCollector.default.reportEvent("admin-logs", "connect");
            if (!!this.adminLogsClient()) {
                this.reconnect();
                return;
            }
            if (!this.adminLogsConfig()) {
                this.configureConnection();
                return;
            }
            var tokenDeferred = $.Deferred();
            if (!this.adminLogsConfig().singleAuthToken()) {
            }
            else {
                tokenDeferred.resolve();
            }
            tokenDeferred.done(function () {
                _this.adminLogsClient(new adminLogsClient(_this.adminLogsConfig().singleAuthToken().Token));
                _this.adminLogsClient().connect();
                var categoriesConfig = _this.adminLogsConfig().entries().map(function (e) { return e.toDto(); });
                _this.adminLogsClient().configureCategories(categoriesConfig);
                _this.adminLogsClient().connectionOpeningTask.done(function () {
                    _this.connected(true);
                    _this.adminLogsClient().watchAdminLogs(function (event) {
                        _this.onLogMessage(event);
                    });
                });
                _this.adminLogsConfig().singleAuthToken(null);
            });
        };
        adminLogs.prototype.disconnect = function () {
            var _this = this;
            eventsCollector.default.reportEvent("admin-logs", "disconnect");
            if (!!this.adminLogsClient()) {
                this.adminLogsClient().dispose();
                return this.adminLogsClient().connectionClosingTask.then(function () {
                    _this.adminLogsClient(null);
                    _this.connected(false);
                });
            }
            else {
                app.showBootstrapMessage("Cannot disconnect, connection does not exist", "Disconnect");
                return $.Deferred().reject();
            }
        };
        adminLogs.prototype.reconnect = function () {
            var _this = this;
            eventsCollector.default.reportEvent("admin-logs", "reconnect");
            if (!this.adminLogsClient()) {
                this.connect();
            }
            else {
                this.disconnect().done(function () {
                    _this.connect();
                });
            }
        };
        adminLogs.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.logsContainer = document.getElementById("rawLogsContainer");
        };
        adminLogs.prototype.deactivate = function () {
            clearInterval(this.intervalId);
            if (this.adminLogsClient()) {
                this.adminLogsClient().dispose();
            }
        };
        adminLogs.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.disposeAdminLogsClient();
        };
        adminLogs.prototype.disposeAdminLogsClient = function () {
            var client = this.adminLogsClient();
            if (client) {
                client.dispose();
            }
        };
        adminLogs.prototype.onLogMessage = function (entry) {
            if (this.entriesCount() + this.pendingLogs.length < this.adminLogsConfig().maxEntries()) {
                this.pendingLogs.push(entry);
            }
            else {
                // stop logging
                var client = this.adminLogsClient();
                this.connected(false);
                client.dispose();
            }
        };
        adminLogs.prototype.exportLogs = function () {
            eventsCollector.default.reportEvent("admin-logs", "export");
            fileDownloader.downloadAsJson(this.rawLogs(), "logs.json");
        };
        adminLogs.prototype.toggleKeepDown = function () {
            this.keepDown.toggle();
            if (this.keepDown()) {
                var logsPre = document.getElementById('adminLogsPre');
                logsPre.scrollTop = logsPre.scrollHeight;
            }
        };
        return adminLogs;
    }(viewModelBase));
    return adminLogs;
});

//# sourceMappingURL=adminLogs.js.map
