var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "viewmodels/shell/accessHelper", "models/auth/license", "common/settingsAccessAuthorizer"], function (require, exports, durandalRouter, appUrl, viewModelBase, accessHelper, license, settingsAccessAuthorizer) {
    "use strict";
    var adminSettings = (function (_super) {
        __extends(adminSettings, _super);
        function adminSettings() {
            var _this = _super.call(this) || this;
            _this.router = null;
            _this.isSystemDatabaseForbidden = ko.observable();
            _this.settingsAccess = new settingsAccessAuthorizer();
            _this.router = durandalRouter.createChildRouter()
                .makeRelative({
                fromParent: true,
                moduleId: 'viewmodels/manage'
            });
            return _this;
        }
        adminSettings.prototype.canActivate = function (args) {
            return true;
        };
        adminSettings.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            if (!license.licenseStatus()) {
                // we want to make sure license information is loaded before building route info
                this.init(); //TODO: 
            }
            else {
                this.init();
            }
        };
        adminSettings.prototype.init = function () {
            var _this = this;
            this.isSystemDatabaseForbidden((accessHelper.isGlobalAdmin() || accessHelper.canReadWriteSettings() || accessHelper.canReadSettings()) === false);
            var canReadOrWrite = settingsAccessAuthorizer.canReadOrWrite();
            var isGlobalAdmin = accessHelper.isGlobalAdmin();
            var apiKeyRoute = { route: ['', 'apiKeys'], moduleId: 'viewmodels/manage/apiKeys', title: 'API Keys', nav: true, hash: appUrl.forApiKeys(), enabled: canReadOrWrite };
            var clusterRoute = { route: 'cluster', moduleId: "viewmodels/manage/cluster", title: "Cluster", nav: true, hash: appUrl.forCluster(), enabled: canReadOrWrite };
            var globalConfigRoute = { route: 'globalConfig*details', moduleId: 'viewmodels/manage/globalConfig/globalConfig', title: 'Global Configuration', nav: true, hash: appUrl.forGlobalConfig(), enabled: canReadOrWrite };
            var serverSmuggling = { route: "serverSmuggling", moduleId: "viewmodels/manage/serverSmuggling", title: "Server Smuggling", nav: true, hash: appUrl.forServerSmugging(), enabled: isGlobalAdmin };
            var backupRoute = { route: 'backup', moduleId: 'viewmodels/manage/backup', title: 'Backup', nav: true, hash: appUrl.forBackup(), enabled: isGlobalAdmin };
            var compactRoute = { route: 'compact', moduleId: 'viewmodels/manage/compact', title: 'Compact', nav: true, hash: appUrl.forCompact(), enabled: isGlobalAdmin };
            var restoreRoute = { route: 'restore', moduleId: 'viewmodels/manage/restore', title: 'Restore', nav: true, hash: appUrl.forRestore(), enabled: isGlobalAdmin };
            var adminLogsRoute = { route: 'adminLogs', moduleId: 'viewmodels/manage/adminLogs', title: 'Admin Logs', nav: true, hash: appUrl.forAdminLogs(), enabled: isGlobalAdmin };
            var topologyRoute = { route: 'topology', moduleId: 'viewmodels/manage/topology', title: 'Server Topology', nav: true, hash: appUrl.forServerTopology(), enabled: isGlobalAdmin };
            var trafficWatchRoute = { route: 'trafficWatch', moduleId: 'viewmodels/manage/trafficWatch', title: 'Traffic Watch', nav: true, hash: appUrl.forTrafficWatch(), enabled: isGlobalAdmin };
            var licenseInformation = { route: 'licenseInformation', moduleId: 'viewmodels/manage/licenseInformation', title: 'License Information', nav: true, hash: appUrl.forLicenseInformation(), enabled: canReadOrWrite };
            var debugInfoRoute = { route: 'debugInfo', moduleId: 'viewmodels/manage/infoPackage', title: 'Gather Debug Info', nav: true, hash: appUrl.forDebugInfo(), enabled: isGlobalAdmin };
            var ioTestRoute = { route: 'ioTest', moduleId: 'viewmodels/manage/ioTest', title: 'IO Test', nav: true, hash: appUrl.forIoTest(), enabled: isGlobalAdmin };
            var diskIoViewerRoute = { route: 'diskIoViewer', moduleId: 'viewmodels/manage/diskIoViewer', title: 'Disk IO Viewer', nav: true, hash: appUrl.forDiskIoViewer(), enabled: canReadOrWrite };
            var consoleRoute = { route: 'console', moduleId: "viewmodels/manage/console", title: "Administrator JS Console", nav: true, hash: appUrl.forAdminJsConsole(), enabled: isGlobalAdmin };
            var studioConfigRoute = { route: 'studioConfig', moduleId: 'viewmodels/manage/studioConfig', title: 'Studio Config', nav: true, hash: appUrl.forStudioConfig(), enabled: canReadOrWrite };
            var hotSpareRoute = { route: 'hotSpare', moduleId: 'viewmodels/manage/hotSpare', title: 'Hot Spare', nav: true, hash: appUrl.forHotSpare(), enabled: isGlobalAdmin };
            var routes = [
                apiKeyRoute,
                clusterRoute,
                globalConfigRoute,
                serverSmuggling,
                backupRoute,
                compactRoute,
                restoreRoute,
                adminLogsRoute,
                topologyRoute,
                trafficWatchRoute,
                licenseInformation,
                debugInfoRoute,
                ioTestRoute,
                diskIoViewerRoute,
                consoleRoute,
                studioConfigRoute
            ];
            /*
            if (license.licenseStatus().Attributes.hotSpare === "true")
                routes.push(hotSpareRoute);
            */
            this.router = this.router
                .reset()
                .map(routes)
                .buildNavigationModel();
            adminSettings.adminSettingsRouter = this.router;
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().find(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        };
        return adminSettings;
    }(viewModelBase));
    return adminSettings;
});

//# sourceMappingURL=adminSettings.js.map
