var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "commands/auth/getApiKeysCommand", "commands/auth/saveApiKeysCommand", "models/auth/apiKey", "viewmodels/viewModelBase", "common/settingsAccessAuthorizer", "common/eventsCollector"], function (require, exports, getApiKeysCommand, saveApiKeysCommand, apiKey, viewModelBase, settingsAccessAuthorizer, eventsCollector) {
    "use strict";
    var apiKeys = (function (_super) {
        __extends(apiKeys, _super);
        function apiKeys() {
            var _this = _super.call(this) || this;
            _this.apiKeys = ko.observableArray();
            _this.loadedApiKeys = ko.observableArray();
            _this.searchText = ko.observable();
            _this.noResults = ko.observable(false);
            _this.searchText.throttle(200).subscribe(function (value) { return _this.filterKeys(value); });
            return _this;
        }
        apiKeys.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            if (settingsAccessAuthorizer.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                this.fetchApiKeys().done(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        apiKeys.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('9CGJ4Y');
            this.dirtyFlag = new ko.DirtyFlag([this.apiKeys]);
            this.isSaveEnabled = ko.computed(function () { return !settingsAccessAuthorizer.isReadOnly() && _this.dirtyFlag().isDirty(); });
        };
        apiKeys.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            if (settingsAccessAuthorizer.isReadOnly()) {
                $('#manageApiKeys input:not(#apiKeysSearchInput)').attr('readonly', 'readonly');
                $('#manageApiKeys button').attr('disabled', 'true');
            }
            $("#manageApiKeys").on("keypress", 'input[name="databaseName"]', function (e) { return e.which !== 13; });
        };
        apiKeys.prototype.fetchApiKeys = function () {
            var _this = this;
            return new getApiKeysCommand()
                .execute()
                .done(function (results) {
                _this.apiKeys(results);
                _this.saveLoadedApiKeys(results);
                _this.apiKeys().forEach(function (key) {
                    _this.subscribeToObservableKeyName(key);
                });
            });
        };
        apiKeys.prototype.saveLoadedApiKeys = function (apiKeys) {
            // clones the apiKeys object
            //TODO: avoid using ko this.loadedApiKeys = ko.mapping.fromJS(ko.mapping.toJS(apiKeys));
        };
        apiKeys.prototype.subscribeToObservableKeyName = function (key) {
            var _this = this;
            key.name.subscribe(function (previousApiKeyName) {
                var existingApiKeysExceptCurrent = _this.apiKeys().filter(function (k) { return k !== key && k.name() === previousApiKeyName; });
                if (existingApiKeysExceptCurrent.length === 1) {
                    existingApiKeysExceptCurrent[0].nameCustomValidity('');
                }
            }, this, "beforeChange");
            key.name.subscribe(function (newApiKeyName) {
                var errorMessage = '';
                var isApiKeyNameValid = newApiKeyName.indexOf("\\") === -1;
                var existingApiKeys = _this.apiKeys().filter(function (k) { return k !== key && k.name() === newApiKeyName; });
                if (!isApiKeyNameValid) {
                    errorMessage = "API Key name cannot contain '\\'";
                }
                else if (existingApiKeys.length > 0) {
                    errorMessage = "API key name already exists!";
                }
                key.nameCustomValidity(errorMessage);
            });
        };
        apiKeys.prototype.createNewApiKey = function () {
            eventsCollector.default.reportEvent("api-keys", "create");
            var newApiKey = apiKey.empty();
            this.subscribeToObservableKeyName(newApiKey);
            newApiKey.generateSecret();
            this.apiKeys.unshift(newApiKey);
        };
        apiKeys.prototype.removeApiKey = function (key) {
            eventsCollector.default.reportEvent("api-keys", "remove");
            this.apiKeys.remove(key);
        };
        apiKeys.prototype.saveChanges = function () {
            var _this = this;
            eventsCollector.default.reportEvent("api-keys", "save");
            this.apiKeys().forEach(function (key) { return key.setIdFromName(); });
            var apiKeysNamesArray = this.apiKeys().map(function (key) { return key.name(); });
            var deletedApiKeys = this.loadedApiKeys().filter(function (key) { return !_.includes(apiKeysNamesArray, key.name()); });
            deletedApiKeys.forEach(function (key) { return key.setIdFromName(); });
            new saveApiKeysCommand(this.apiKeys(), deletedApiKeys)
                .execute()
                .done(function (result) {
                _this.updateKeys(result);
                _this.saveLoadedApiKeys(_this.apiKeys());
                _this.dirtyFlag().reset();
            });
        };
        apiKeys.prototype.updateKeys = function (serverKeys) {
            this.apiKeys().forEach(function (key) {
                var serverKey = serverKeys.find(function (k) { return k.Key === key.getId(); });
                if (serverKey) {
                    key.__metadata.etag(serverKey.Etag);
                    key.__metadata.lastModified = serverKey.Metadata["Last-Modified"]; //TODO last -modified is not longer used? use Raven-Last-Modified
                }
            });
        };
        apiKeys.prototype.filterKeys = function (filter) {
            var filterLower = filter.toLowerCase().trim();
            var matches = 0;
            this.apiKeys().forEach(function (k) {
                var isEmpty = filterLower.length === 0;
                var isMatch = k.name() != null && k.name().toLowerCase().indexOf(filterLower) !== -1;
                var visible = isEmpty || isMatch;
                k.visible(visible);
                if (visible) {
                    matches++;
                }
            });
            this.noResults(matches === 0);
        };
        return apiKeys;
    }(viewModelBase));
    return apiKeys;
});

//# sourceMappingURL=apiKeys.js.map
