var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/resources/database", "models/filesystem/filesystem", "models/counter/counterStorage", "commands/maintenance/backupDatabaseCommand", "commands/filesystem/backupFilesystemCommand", "commands/counter/backupCounterStorageCommand", "commands/resources/getResourceDrives", "viewmodels/shell/accessHelper", "common/shell/resourcesManager"], function (require, exports, viewModelBase, database, filesystem, counterStorage, backupDatabaseCommand, backupFilesystemCommand, backupCounterStorageCommand, getResourceDrives, accessHelper, resourcesManager) {
    "use strict";
    var resourceBackup = (function () {
        function resourceBackup(qualifier, resources) {
            var _this = this;
            this.qualifier = qualifier;
            this.resources = resources;
            this.resourcesManger = resourcesManager.default;
            this.incremental = ko.observable(false);
            this.resourceName = ko.observable('');
            this.backupLocation = ko.observable('');
            this.backupStatusMessages = ko.observableArray();
            this.isBusy = ko.observable();
            this.keepDown = ko.observable(false);
            this.drivesForCurrentResource = ko.observable([]);
            this.displaySameDriveWarning = ko.computed(function () {
                var resourceDrives = _this.drivesForCurrentResource();
                var location = _this.backupLocation().toLowerCase().substr(0, 3);
                return resourceDrives.indexOf(location) !== -1;
            });
            this.resourcesNames = ko.computed(function () { return resources().map(function (rs) { return rs.name; }); });
            this.fullTypeName = ko.computed(function () {
                var rs = resources();
                if (rs.length > 0) {
                    return rs[0].fullTypeName;
                }
                return null;
            });
            this.searchResults = ko.computed(function () {
                var newResourceName = _this.resourceName();
                return _this.resourcesNames().filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var newResourceName = _this.resourceName();
                var foundRs = _this.resources().find(function (rs) { return newResourceName === rs.name && rs.qualifier === _this.qualifier; });
                if (!foundRs && newResourceName.length > 0) {
                    errorMessage = _this.fullTypeName() + " name doesn't exist!";
                }
                return errorMessage;
            });
            this.resourceName.throttle(200).subscribe(function (resource) {
                var foundRs = _this.resources().find(function (rs) { return resource === rs.name && rs.qualifier === _this.qualifier; });
                if (foundRs) {
                    new getResourceDrives(foundRs.name, foundRs.type.toString()).execute()
                        .done(function (drives) {
                        _this.drivesForCurrentResource(drives);
                    });
                }
                else {
                    _this.drivesForCurrentResource([]);
                }
            });
        }
        resourceBackup.prototype.toggleKeepDown = function () {
            this.keepDown.toggle();
            this.forceKeepDown();
        };
        resourceBackup.prototype.forceKeepDown = function () {
            if (this.keepDown()) {
                var body = document.getElementsByTagName("body")[0];
                body.scrollTop = body.scrollHeight;
            }
        };
        resourceBackup.prototype.updateBackupStatus = function (newBackupStatus) {
            this.backupStatusMessages(newBackupStatus.Messages);
            this.isBusy(newBackupStatus.IsRunning);
            this.forceKeepDown();
        };
        return resourceBackup;
    }());
    var backupDatabase = (function (_super) {
        __extends(backupDatabase, _super);
        function backupDatabase() {
            var _this = _super.apply(this, arguments) || this;
            _this.resourcesManager = resourcesManager.default;
            _this.dbBackupOptions = new resourceBackup(database.qualifier, _this.resourcesManager.databases);
            _this.fsBackupOptions = new resourceBackup(filesystem.qualifier, _this.resourcesManager.fileSystems);
            _this.csBackupOptions = new resourceBackup(counterStorage.qualifier, _this.resourcesManager.counterStorages);
            _this.isForbidden = ko.observable();
            return _this;
        }
        backupDatabase.prototype.canActivate = function (args) {
            this.isForbidden(!accessHelper.isGlobalAdmin());
            return true;
        };
        backupDatabase.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("FT7RV6");
        };
        backupDatabase.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            $('form :input[name="databaseName"]').on("keypress", function (e) { return e.which !== 13; });
            $('form :input[name="filesystemName"]').on("keypress", function (e) { return e.which !== 13; });
        };
        backupDatabase.prototype.startDbBackup = function () {
            var backupOptions = this.dbBackupOptions;
            backupOptions.isBusy(true);
            var dbTobackup = this.resourcesManager.getDatabaseByName(backupOptions.resourceName());
            new backupDatabaseCommand(dbTobackup, backupOptions.backupLocation(), backupOptions.updateBackupStatus.bind(this.dbBackupOptions), backupOptions.incremental())
                .execute()
                .always(function () { return backupOptions.isBusy(false); });
        };
        backupDatabase.prototype.startFsBackup = function () {
            var backupOptions = this.fsBackupOptions;
            backupOptions.isBusy(true);
            var fsToBackup = this.resourcesManager.getFileSystemByName(backupOptions.resourceName());
            new backupFilesystemCommand(fsToBackup, backupOptions.backupLocation(), backupOptions.updateBackupStatus.bind(this.fsBackupOptions), backupOptions.incremental())
                .execute()
                .always(function () { return backupOptions.isBusy(false); });
        };
        backupDatabase.prototype.startCsBackup = function () {
            var backupOptions = this.csBackupOptions;
            backupOptions.isBusy(true);
            var csToBackup = this.resourcesManager.getCounterStorageByName(backupOptions.resourceName());
            new backupCounterStorageCommand(csToBackup, backupOptions.backupLocation(), backupOptions.updateBackupStatus.bind(this.csBackupOptions), backupOptions.incremental())
                .execute()
                .always(function () { return backupOptions.isBusy(false); });
        };
        return backupDatabase;
    }(viewModelBase));
    return backupDatabase;
});

//# sourceMappingURL=backup.js.map
