var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "commands/database/cluster/getClusterTopologyCommand", "common/messagePublisher", "models/database/cluster/nodeConnectionInfo", "viewmodels/manage/editNodeConnectionInfoDialog", "durandal/app", "commands/resources/getDatabaseStatsCommand", "commands/database/debug/getStatusDebugConfigCommand", "commands/database/cluster/extendRaftClusterCommand", "commands/database/cluster/initializeNewClusterCommand", "commands/database/cluster/leaveRaftClusterCommand", "commands/database/cluster/removeClusteringCommand", "commands/database/cluster/saveClusterConfigurationCommand", "commands/database/cluster/updateRaftClusterCommand", "commands/database/cluster/getClusterNodesStatusCommand", "viewmodels/shell", "common/bindingHelpers/autoRefreshBindingHandler", "common/settingsAccessAuthorizer", "commands/database/cluster/changeNodeVotingModeCommand", "common/eventsCollector"], function (require, exports, viewModelBase, appUrl, getClusterTopologyCommand, messagePublisher, nodeConnectionInfo, editNodeConnectionInfoDialog, app, getDatabaseStatsCommand, getStatusDebugConfigCommand, extendRaftClusterCommand, initializeNewClusterCommand, leaveRaftClusterCommand, removeClusteringCommand, saveClusterConfigurationCommand, updateRaftClusterCommand, getClusterNodesStatusCommand, shell, autoRefreshBindingHandler, settingsAccessAuthorizer, changeNodeVotingModeCommand, eventsCollector) {
    "use strict";
    var cluster = (function (_super) {
        __extends(cluster, _super);
        function cluster() {
            var _this = _super.call(this) || this;
            _this.topology = ko.observable();
            _this.systemDatabaseId = ko.observable();
            _this.serverUrl = ko.observable();
            autoRefreshBindingHandler.install();
            _this.clusterMode = ko.computed(function () {
                return _this.topology() && _this.topology().clusterMode();
            });
            return _this;
        }
        cluster.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            if (settingsAccessAuthorizer.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                $.when(this.fetchClusterTopology(null), this.fetchDatabaseId(null), this.fetchServerUrl(null))
                    .done(function () {
                    deferred.resolve({ can: true });
                    if (_this.clusterMode()) {
                        _this.fetchStatus(null);
                    }
                })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forAdminSettings() }); });
            }
            return deferred;
        };
        cluster.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("11HBHO");
        };
        cluster.prototype.refresh = function () {
            var _this = this;
            eventsCollector.default.reportEvent("cluster", "refresh");
            return this.fetchClusterTopology(null)
                .done(function () { return _this.fetchStatus(null); });
        };
        cluster.prototype.fetchClusterTopology = function (db) {
            var _this = this;
            return new getClusterTopologyCommand(db)
                .execute()
                .done(function (topo) {
                _this.topology(topo);
            })
                .fail(function () { return messagePublisher.reportError("Unable to fetch cluster topology"); });
        };
        cluster.prototype.fetchDatabaseId = function (db) {
            var _this = this;
            return new getDatabaseStatsCommand(db)
                .execute()
                .done(function (stats) {
                _this.systemDatabaseId(stats.DatabaseId); //TODO: make sure it works!
            });
        };
        cluster.prototype.fetchServerUrl = function (db) {
            var _this = this;
            return new getStatusDebugConfigCommand(db)
                .execute()
                .done(function (config) { return _this.serverUrl(config.ServerUrl); });
        };
        cluster.prototype.fetchStatus = function (db) {
            var _this = this;
            return new getClusterNodesStatusCommand(db)
                .execute()
                .done(function (status) { return _this.updateNodesStatus(status); });
        };
        cluster.prototype.updateNodesStatus = function (status) {
            var _this = this;
            status.forEach(function (nodeStatus) {
                var nci = _this.topology().allNodes().find(function (n) { return n.uri() === nodeStatus.Uri; });
                if (nci) {
                    nci.status(nodeStatus.Status);
                }
            });
        };
        cluster.prototype.addAnotherServerToCluster = function (forcedAdd) {
            var _this = this;
            eventsCollector.default.reportEvent("cluster", "add-server");
            var newNode = nodeConnectionInfo.empty();
            var dialog = new editNodeConnectionInfoDialog(newNode, false);
            dialog
                .onExit()
                .done(function (nci) {
                new extendRaftClusterCommand(null, nci.toDto(), false, forcedAdd)
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
            app.showBootstrapDialog(dialog);
        };
        cluster.prototype.removeClustering = function () {
            var _this = this;
            eventsCollector.default.reportEvent("cluster", "cleanup");
            this.confirmationMessage("Are you sure?", "You are about to clear cluster information on this server.")
                .done(function () {
                new removeClusteringCommand(null)
                    .execute()
                    .done(function () { return setTimeout(function () {
                    _this.refresh();
                    shell.clusterMode(false);
                }, 500); });
            });
        };
        cluster.prototype.createCluster = function () {
            var _this = this;
            eventsCollector.default.reportEvent("cluster", "create");
            var newNode = nodeConnectionInfo.empty();
            newNode.name(this.systemDatabaseId());
            newNode.uri(this.serverUrl());
            var dialog = new editNodeConnectionInfoDialog(newNode, false);
            dialog
                .onExit()
                .done(function (nci) {
                new extendRaftClusterCommand(null, nci.toDto(), true, false)
                    .execute()
                    .done(function () {
                    shell.clusterMode(true);
                    setTimeout(function () { return _this.refresh(); }, 500);
                    new saveClusterConfigurationCommand({ EnableReplication: true }, null)
                        .execute();
                });
            });
            app.showBootstrapDialog(dialog);
        };
        cluster.prototype.initializeNewCluster = function () {
            var _this = this;
            eventsCollector.default.reportEvent("cluster", "secede");
            this.confirmationMessage("Are you sure?", "You are about to initialize new cluster on this server.")
                .done(function () {
                new initializeNewClusterCommand(null)
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
        };
        cluster.prototype.editNode = function (node) {
            var _this = this;
            eventsCollector.default.reportEvent("cluster", "edit");
            var dialog = new editNodeConnectionInfoDialog(node, true);
            dialog.onExit()
                .done(function (nci) {
                new updateRaftClusterCommand(null, nci.toDto())
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
            app.showBootstrapDialog(dialog);
        };
        cluster.prototype.leaveCluster = function (node) {
            var _this = this;
            eventsCollector.default.reportEvent("cluster", "leave");
            this.confirmationMessage("Are you sure?", "You are removing node " + node.uri() + " from cluster.")
                .done(function () {
                new leaveRaftClusterCommand(null, node.toDto())
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
        };
        cluster.prototype.promoteNodeToVoter = function (node) {
            var _this = this;
            eventsCollector.default.reportEvent("cluster", "promote");
            var nodeAsDto = node.toDto();
            nodeAsDto.IsNoneVoter = false;
            this.confirmationMessage("Are you sure?", "You are promoting node " + node.uri() + " to voter.")
                .done(function () {
                new changeNodeVotingModeCommand(null, nodeAsDto)
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
        };
        return cluster;
    }(viewModelBase));
    return cluster;
});

//# sourceMappingURL=cluster.js.map
