var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/resources/database", "models/filesystem/filesystem", "commands/maintenance/startCompactCommand", "commands/filesystem/startCompactCommand", "viewmodels/shell/accessHelper", "common/shell/resourcesManager", "common/eventsCollector"], function (require, exports, viewModelBase, database, filesystem, startDbCompactCommand, startFsCompactCommand, accessHelper, resourcesManager, eventsCollector) {
    "use strict";
    var resourceCompact = (function () {
        function resourceCompact(parent, type, resources) {
            var _this = this;
            this.parent = parent;
            this.type = type;
            this.resources = resources;
            this.resourceName = ko.observable('');
            this.compactStatusMessages = ko.observableArray();
            this.compactStatusLastUpdate = ko.observable();
            this.keepDown = ko.observable(false);
            this.resourcesNames = ko.computed(function () { return resources().map(function (rs) { return rs.name; }); });
            this.searchResults = ko.computed(function () {
                var newResourceName = _this.resourceName();
                return _this.resourcesNames().filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = '';
                var newResourceName = _this.resourceName();
                var foundRs = _this.resources().find(function (rs) { return newResourceName === rs.name; });
                if (!foundRs && newResourceName.length > 0) {
                    errorMessage = (_this.type === database.type ? "Database" : "File system") + " name doesn't exist!";
                }
                return errorMessage;
            });
        }
        resourceCompact.prototype.toggleKeepDown = function () {
            eventsCollector.default.reportEvent("compact", "keep-down", this.type.toString());
            this.keepDown.toggle();
            this.forceKeepDown();
        };
        resourceCompact.prototype.forceKeepDown = function () {
            if (this.keepDown()) {
                var body = document.getElementsByTagName("body")[0];
                body.scrollTop = body.scrollHeight;
            }
        };
        resourceCompact.prototype.updateCompactStatus = function (newCompactStatus) {
            this.compactStatusMessages(newCompactStatus.Messages);
            this.compactStatusLastUpdate(newCompactStatus.LastProgressMessage);
            this.forceKeepDown();
            this.parent.isBusy(newCompactStatus.State === "Running");
        };
        return resourceCompact;
    }());
    var compact = (function (_super) {
        __extends(compact, _super);
        function compact() {
            var _this = _super.apply(this, arguments) || this;
            _this.resourcesManager = resourcesManager.default;
            _this.dbCompactOptions = new resourceCompact(_this, database.type, _this.resourcesManager.databases);
            _this.fsCompactOptions = new resourceCompact(_this, filesystem.type, _this.resourcesManager.fileSystems);
            _this.isBusy = ko.observable();
            _this.isForbidden = ko.observable();
            return _this;
        }
        compact.prototype.canActivate = function (args) {
            this.isForbidden(accessHelper.isGlobalAdmin() === false);
            return true;
        };
        compact.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('7HZGOE');
        };
        compact.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            $('form :input[name="databaseName"]').on("keypress", function (e) { return e.which !== 13; });
            $('form :input[name="filesystemName"]').on("keypress", function (e) { return e.which !== 13; });
        };
        compact.prototype.startDbCompact = function () {
            eventsCollector.default.reportEvent("database", "compact");
            this.isBusy(true);
            var self = this;
            new startDbCompactCommand(this.dbCompactOptions.resourceName(), self.dbCompactOptions.updateCompactStatus.bind(self.dbCompactOptions))
                .execute();
        };
        compact.prototype.startFsCompact = function () {
            eventsCollector.default.reportEvent("fs", "compact");
            this.isBusy(true);
            var self = this;
            new startFsCompactCommand(this.fsCompactOptions.resourceName(), self.fsCompactOptions.updateCompactStatus.bind(self.fsCompactOptions))
                .execute();
        };
        return compact;
    }(viewModelBase));
    return compact;
});

//# sourceMappingURL=compact.js.map
