var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "common/bindingHelpers/aceEditorBindingHandler", "commands/maintenance/adminJsScriptCommand", "common/settingsAccessAuthorizer", "common/eventsCollector"], function (require, exports, viewModelBase, aceEditorBindingHandler, adminJsScriptCommand, settingsAccessAuthorizer, eventsCollector) {
    "use strict";
    var consoleJs = (function (_super) {
        __extends(consoleJs, _super);
        function consoleJs() {
            var _this = _super.call(this) || this;
            _this.resourceName = ko.observable('');
            _this.isBusy = ko.observable();
            _this.responseText = ko.observable();
            _this.script = ko.observable();
            _this.results = ko.observable();
            _this.settingsAccess = new settingsAccessAuthorizer();
            _this.predefinedSamples = ko.observableArray([]);
            _this.predefinedSamples.push({ Name: "Get database stats", Code: "return database.Statistics;" });
            _this.predefinedSamples.push({
                Name: "Get configuration values", Code: "return {" +
                    "\n	RaiseBatchLimit : database.Configuration.AvailableMemoryForRaisingBatchSizeLimit," +
                    "\n	ReduceBatchLimit: database.Configuration.MaxNumberOfItemsToReduceInSingleBatch" +
                    "\n};"
            });
            _this.predefinedSamples.push({
                Name: "Change configuration on the fly", Code: "database.Configuration.DisableDocumentPreFetching = true;" +
                    "\ndatabase.Configuration.MaxNumberOfItemsToPreFetch = 1024;" +
                    "\ndatabase.Configuration.BulkImportBatchTimeout = System.TimeSpan.FromMinutes(13);"
            });
            _this.predefinedSamples.push({ Name: "Run idle operations", Code: "database.RunIdleOperations();" });
            _this.predefinedSamples.push({
                Name: "Put document",
                Code: "var doc = Raven.Json.Linq.RavenJObject.Parse('{ \"Name\" : \"Raven\" }');" +
                    "\nvar metadata = Raven.Json.Linq.RavenJObject.Parse('{ \"Raven-Entity-Name\" : \"Docs\" }');" +
                    "\n\ndatabase.Documents.Put('doc/1', null, doc, metadata, null);"
            });
            aceEditorBindingHandler.install();
            _this.resourcesNames = ko.computed(function () { return _this.resourcesManager.databases().map(function (rs) { return rs.name; }); });
            _this.searchResults = ko.computed(function () {
                var newResourceName = _this.resourceName();
                return _this.resourcesNames().filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            _this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = '';
                var newResourceName = _this.resourceName();
                var foundRs = _this.resourcesManager.getDatabaseByName(newResourceName);
                if (!foundRs && newResourceName.length > 0) {
                    errorMessage = "Database name doesn't exist!";
                }
                return errorMessage;
            });
            return _this;
        }
        consoleJs.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("6BJCAJ");
        };
        consoleJs.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            $('form :input[name="databaseName"]').on("keypress", function (e) { return e.which != 13; });
            $('form :input[name="filesystemName"]').on("keypress", function (e) { return e.which != 13; });
        };
        consoleJs.prototype.executeJs = function () {
            var _this = this;
            eventsCollector.default.reportEvent("console", "execute");
            this.isBusy(true);
            new adminJsScriptCommand(this.script(), this.resourceName())
                .execute()
                .done(function (result) {
                _this.results(JSON.stringify(result, null, 4));
            })
                .always(function () { _this.isBusy(false); });
        };
        consoleJs.prototype.fillWithSample = function (sample) {
            this.script(sample.Code);
        };
        return consoleJs;
    }(viewModelBase));
    return consoleJs;
});

//# sourceMappingURL=console.js.map
