var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "common/bindingHelpers/aceEditorBindingHandler", "viewmodels/viewModelBase", "commands/database/documents/getCustomFunctionsCommand", "commands/database/documents/deleteDocumentCommand", "commands/database/documents/saveCustomFunctionsCommand", "models/database/documents/customFunctions", "common/jsonUtil", "common/messagePublisher", "common/settingsAccessAuthorizer", "common/eventsCollector"], function (require, exports, aceEditorBindingHandler, viewModelBase, getCustomFunctionsCommand, deleteDocumentCommand, saveCustomFunctionsCommand, customFunctions, jsonUtil, messagePublisher, settingsAccessAuthorizer, eventsCollector) {
    "use strict";
    var globalConfigCustomFunctions = (function (_super) {
        __extends(globalConfigCustomFunctions, _super);
        function globalConfigCustomFunctions() {
            var _this = _super.call(this) || this;
            /* TODO
            developerLicense = globalConfig.developerLicense;
            canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations;*/
            _this.activated = ko.observable(false);
            aceEditorBindingHandler.install();
            _this.documentText = ko.observable("");
            _this.fetchCustomFunctions();
            _this.dirtyFlag = new ko.DirtyFlag([_this.documentText], false, jsonUtil.newLineNormalizingHashFunction);
            _this.isSaveEnabled = ko.computed(function () { return !settingsAccessAuthorizer.isReadOnly() && _this.dirtyFlag().isDirty(); });
            return _this;
        }
        globalConfigCustomFunctions.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#customFunctionsExample").popover({
                html: true,
                trigger: "hover",
                content: "Examples:<pre>exports.greet = <span class=\"code-keyword\">function</span>(name) {<br/>    <span class=\"code-keyword\">return</span> <span class=\"code-string\">\"Hello \" + name + \"!\"</span>;<br/>}</pre>"
            });
        };
        globalConfigCustomFunctions.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $(".custom-functions-form .editor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#customFunctionsEditor").on("DynamicHeightSet", function () { return _this.docEditor.resize(); });
            this.fetchCustomFunctions();
        };
        globalConfigCustomFunctions.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#customFunctionsEditor").off("DynamicHeightSet");
        };
        globalConfigCustomFunctions.prototype.fetchCustomFunctions = function () {
            var _this = this;
            var fetchTask = new getCustomFunctionsCommand(null, true).execute();
            fetchTask.done(function (cf) {
                _this.documentText(cf.functions);
                _this.activated(true);
                _this.dirtyFlag().reset();
            }).fail(function (xhr) {
                if (xhr.status === 404) {
                    _this.activated(false);
                }
            });
        };
        globalConfigCustomFunctions.prototype.saveChanges = function () {
            eventsCollector.default.reportEvent("global-config-custom-functions", "save");
            this.syncChanges(false);
        };
        globalConfigCustomFunctions.prototype.syncChanges = function (deleteConfig) {
            var _this = this;
            if (deleteConfig) {
                new deleteDocumentCommand("Raven/Global/Javascript/Functions", null)
                    .execute()
                    .done(function () { return messagePublisher.reportSuccess("Global Settings were successfully saved!"); })
                    .fail(function (response) { return messagePublisher.reportError("Failed to save global settings!", response.responseText, response.statusText); });
            }
            else {
                var annotations = this.docEditor.getSession().getAnnotations();
                var hasErrors = false;
                for (var i = 0; i < annotations.length; i++) {
                    if (annotations[i].type === "error") {
                        hasErrors = true;
                        break;
                    }
                }
                if (!hasErrors) {
                    var cf = new customFunctions({
                        Functions: this.documentText()
                    });
                    var saveTask = new saveCustomFunctionsCommand(null, cf, true).execute();
                    saveTask.done(function () { return _this.dirtyFlag().reset(); });
                }
                else {
                    messagePublisher.reportError("Errors in the functions file", "Please correct the errors in the file in order to save it.");
                }
            }
        };
        globalConfigCustomFunctions.prototype.activateConfig = function () {
            eventsCollector.default.reportEvent("global-config-custom-functions", "activate");
            this.activated(true);
            this.docEditor.resize();
        };
        globalConfigCustomFunctions.prototype.disactivateConfig = function () {
            var _this = this;
            eventsCollector.default.reportEvent("global-config-custom-functions", "disactivate");
            this.confirmationMessage("Delete global configuration for custom functions?", "Are you sure?")
                .done(function () {
                _this.documentText("");
                _this.activated(false);
                _this.syncChanges(true);
                _this.dirtyFlag().reset();
            });
        };
        return globalConfigCustomFunctions;
    }(viewModelBase));
    return globalConfigCustomFunctions;
});

//# sourceMappingURL=globalConfigCustomFunctions.js.map
