var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "common/settingsAccessAuthorizer", "commands/database/documents/getDocumentWithMetadataCommand", "models/database/cluster/databaseSetting", "commands/database/cluster/saveClusterConfigurationCommand", "viewmodels/shell", "common/eventsCollector"], function (require, exports, viewModelBase, appUrl, settingsAccessAuthorizer, getDocumentWithMetadataCommand, databaseSetting, saveClusterConfigurationCommand, shell, eventsCollector) {
    "use strict";
    var globalConfigDatabaseSettings = (function (_super) {
        __extends(globalConfigDatabaseSettings, _super);
        function globalConfigDatabaseSettings() {
            var _this = _super.apply(this, arguments) || this;
            _this.activated = ko.observable(false);
            /* TODO
            developerLicense = globalConfig.developerLicense;
            canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations;*/
            _this.databaseSettings = ko.observableArray([]);
            _this.settingsAccess = new settingsAccessAuthorizer();
            _this.loadedClusterConfigurationDto = ko.observable();
            _this.clusterMode = shell.clusterMode;
            return _this;
        }
        globalConfigDatabaseSettings.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            if (settingsAccessAuthorizer.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                this.fetchClusterConfiguration(null)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(null) }); });
            }
            return deferred;
        };
        globalConfigDatabaseSettings.prototype.saveChanges = function () {
            var _this = this;
            eventsCollector.default.reportEvent("global-config-database-settings", "save");
            var customSettings = {};
            var settings = this.databaseSettings();
            settings.forEach(function (x) {
                customSettings[x.key()] = x.value();
            });
            new saveClusterConfigurationCommand({
                EnableReplication: this.loadedClusterConfigurationDto().EnableReplication,
                DatabaseSettings: customSettings
            }, null)
                .execute()
                .done(function () { return _this.dirtyFlag().reset(); });
        };
        globalConfigDatabaseSettings.prototype.activateConfig = function () {
            eventsCollector.default.reportEvent("global-config-database-settings", "activate");
            this.activated(true);
            this.databaseSettings([]);
        };
        globalConfigDatabaseSettings.prototype.disactivateConfig = function () {
            var _this = this;
            eventsCollector.default.reportEvent("global-config-database-settings", "disactivate");
            this.confirmationMessage("Delete global configuration for cluster-wide database settings?", "Are you sure?")
                .done(function () {
                _this.activated(false);
                _this.databaseSettings([]);
                _this.saveChanges();
            });
        };
        globalConfigDatabaseSettings.prototype.navigateToCreateCluster = function () {
            this.navigate(this.appUrls.adminSettingsCluster());
        };
        globalConfigDatabaseSettings.prototype.addNewSetting = function () {
            eventsCollector.default.reportEvent("global-config-database-settings", "setting", "add");
            this.databaseSettings.push(new databaseSetting(null, null));
        };
        globalConfigDatabaseSettings.prototype.removeSetting = function (itemToRemove) {
            eventsCollector.default.reportEvent("global-config-database-settings", "setting", "remove");
            this.databaseSettings.remove(itemToRemove);
        };
        globalConfigDatabaseSettings.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("T3P7UA");
            this.dirtyFlag = new ko.DirtyFlag([this.databaseSettings]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        globalConfigDatabaseSettings.prototype.fetchClusterConfiguration = function (db) {
            var _this = this;
            var currentConfiguration = new getDocumentWithMetadataCommand("Raven/Cluster/Configuration", null, true)
                .execute()
                .done(function (result) {
                _this.loadedClusterConfigurationDto(result);
                if (result && result.DatabaseSettings) {
                    for (var key in result.DatabaseSettings) {
                        if (result.DatabaseSettings.hasOwnProperty(key)) {
                            _this.databaseSettings.push(new databaseSetting(key, result.DatabaseSettings[key]));
                        }
                    }
                    _this.activated(_this.databaseSettings().length > 0);
                    _this.dirtyFlag().reset();
                }
            });
            return currentConfiguration;
        };
        return globalConfigDatabaseSettings;
    }(viewModelBase));
    return globalConfigDatabaseSettings;
});

//# sourceMappingURL=globalConfigDatabaseSettings.js.map
