var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/database/globalConfig/getEffectiveSettingsCommand", "commands/database/globalConfig/saveGlobalSettingsCommand", "models/database/documents/document", "common/appUrl", "common/settingsAccessAuthorizer", "common/eventsCollector"], function (require, exports, viewModelBase, getEffectiveSettingsCommand, saveGlobalSettingsCommand, document, appUrl, settingsAccessAuthorizer, eventsCollector) {
    "use strict";
    var globalConfigQuotas = (function (_super) {
        __extends(globalConfigQuotas, _super);
        function globalConfigQuotas() {
            var _this = _super.apply(this, arguments) || this;
            /* TODO
            developerLicense = globalConfig.developerLicense;
            canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations; */
            _this.settingsDocument = ko.observable();
            _this.activated = ko.observable(false);
            _this.maximumSize = ko.observable();
            _this.warningLimitThreshold = ko.observable();
            _this.maxNumberOfDocs = ko.observable();
            _this.warningThresholdForDocs = ko.observable();
            return _this;
        }
        globalConfigQuotas.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            if (settingsAccessAuthorizer.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                // fetch current quotas from the database
                this.fetchQuotas(null)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
            }
            return deferred;
        };
        globalConfigQuotas.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.initializeDirtyFlag();
            this.isSaveEnabled = ko.computed(function () { return !settingsAccessAuthorizer.isReadOnly() && _this.dirtyFlag().isDirty(); });
        };
        globalConfigQuotas.prototype.fetchQuotas = function (db) {
            var _this = this;
            return new getEffectiveSettingsCommand(db)
                .execute()
                .done(function (doc) {
                _this.settingsDocument(doc);
                // we make decision based on first available property 
                var activated = !!doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"];
                _this.activated(activated);
                if (activated) {
                    _this.maximumSize(doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"] / 1024);
                    _this.warningLimitThreshold(doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"] / 1024);
                    _this.maxNumberOfDocs(doc["Settings"]["Raven/Quotas/Documents/HardLimit"]);
                    _this.warningThresholdForDocs(doc["Settings"]["Raven/Quotas/Documents/SoftLimit"]);
                }
            });
        };
        globalConfigQuotas.prototype.initializeDirtyFlag = function () {
            this.dirtyFlag = new ko.DirtyFlag([
                this.maximumSize,
                this.warningLimitThreshold,
                this.maxNumberOfDocs,
                this.warningThresholdForDocs
            ]);
        };
        globalConfigQuotas.prototype.saveChanges = function () {
            eventsCollector.default.reportEvent("global-config-quotas", "save");
            this.syncChanges(false);
        };
        globalConfigQuotas.prototype.syncChanges = function (deleteConfig) {
            var _this = this;
            var settingsDocument = this.settingsDocument();
            settingsDocument["@metadata"] = this.settingsDocument().__metadata;
            settingsDocument["@metadata"]["@etag"] = this.settingsDocument().__metadata["@etag"];
            var doc = new document(settingsDocument.toDto(true));
            if (deleteConfig) {
                delete doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"];
                delete doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"];
                delete doc["Settings"]["Raven/Quotas/Documents/HardLimit"];
                delete doc["Settings"]["Raven/Quotas/Documents/SoftLimit"];
                this.maximumSize(null);
                this.warningLimitThreshold(null);
                this.maxNumberOfDocs(null);
                this.warningThresholdForDocs(null);
            }
            else {
                doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"] = this.maximumSize() * 1024;
                doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"] = this.warningLimitThreshold() * 1024;
                doc["Settings"]["Raven/Quotas/Documents/HardLimit"] = this.maxNumberOfDocs();
                doc["Settings"]["Raven/Quotas/Documents/SoftLimit"] = this.warningThresholdForDocs();
            }
            var saveTask = new saveGlobalSettingsCommand(null, doc).execute();
            saveTask.done(function (saveResult) {
                _this.settingsDocument().__metadata["@etag"] = saveResult.ETag;
                _this.dirtyFlag().reset(); //Resync Changes
            });
        };
        globalConfigQuotas.prototype.activateConfig = function () {
            eventsCollector.default.reportEvent("global-config-quotas", "activate");
            this.activated(true);
        };
        globalConfigQuotas.prototype.disactivateConfig = function () {
            var _this = this;
            eventsCollector.default.reportEvent("global-config-quotas", "disactivate");
            this.confirmationMessage("Delete global configuration for quotas?", "Please note that databases with quotas bundle enabled and without local configuration may crash.")
                .done(function () {
                _this.activated(false);
                _this.syncChanges(true);
            });
        };
        return globalConfigQuotas;
    }(viewModelBase));
    return globalConfigQuotas;
});

//# sourceMappingURL=globalConfigQuotas.js.map
