var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "viewmodels/shell/accessHelper", "models/resources/database", "commands/database/documents/getDocumentWithMetadataCommand", "common/appUrl", "models/filesystem/filesystem", "commands/maintenance/monitorRestoreCommand", "commands/maintenance/startRestoreCommand", "commands/filesystem/startRestoreCommand", "common/shell/resourcesManager", "common/eventsCollector"], function (require, exports, viewModelBase, accessHelper, database, getDocumentWithMetadataCommand, appUrl, filesystem, monitorRestoreCommand, startDbRestoreCommand, startFsRestoreCommand, resourcesManager, eventsCollector) {
    "use strict";
    var resourceRestore = (function () {
        function resourceRestore(parent, type, resources) {
            var _this = this;
            this.parent = parent;
            this.type = type;
            this.resources = resources;
            this.defrag = ko.observable(false);
            this.backupLocation = ko.observable("");
            this.resourceLocation = ko.observable();
            this.indexesLocation = ko.observable();
            this.journalsLocation = ko.observable();
            this.resourceName = ko.observable();
            this.restoreStatusMessages = ko.observableArray();
            this.keepDown = ko.observable(false);
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var newResourceName = _this.resourceName();
                var foundDb = resources().find(function (rs) { return newResourceName === rs.name; });
                if (!!foundDb && newResourceName.length > 0) {
                    errorMessage = (_this.type === database.type ? "Database" : "File System") + " name already exist!";
                }
                return errorMessage;
            });
        }
        resourceRestore.prototype.toggleKeepDown = function () {
            eventsCollector.default.reportEvent("restore", "keep-down", this.type.toString());
            this.keepDown.toggle();
            this.forceKeepDown();
        };
        resourceRestore.prototype.forceKeepDown = function () {
            if (this.keepDown()) {
                var body = document.getElementsByTagName("body")[0];
                body.scrollTop = body.scrollHeight;
            }
        };
        resourceRestore.prototype.updateRestoreStatus = function (newRestoreStatus) {
            this.restoreStatusMessages(newRestoreStatus.Messages);
            this.forceKeepDown();
            this.parent.isBusy(newRestoreStatus.State === "Running");
        };
        return resourceRestore;
    }());
    var restore = (function (_super) {
        __extends(restore, _super);
        function restore() {
            var _this = _super.apply(this, arguments) || this;
            _this.resourceManager = resourcesManager.default;
            _this.dbRestoreOptions = new resourceRestore(_this, database.type, _this.resourceManager.databases);
            _this.fsRestoreOptions = new resourceRestore(_this, filesystem.type, _this.resourceManager.fileSystems);
            _this.disableReplicationDestinations = ko.observable(false);
            _this.generateNewDatabaseId = ko.observable(false);
            _this.isForbidden = ko.observable();
            _this.isBusy = ko.observable();
            _this.anotherRestoreInProgres = ko.observable(false);
            return _this;
        }
        restore.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            this.isForbidden(accessHelper.isGlobalAdmin() === false);
            if (this.isForbidden() === false) {
                this.isBusy(true);
                var self = this;
                new getDocumentWithMetadataCommand("Raven/Restore/InProgress", null, true).execute()
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forSettings(null) }); })
                    .done(function (result) {
                    if (result) {
                        // looks like another restore is in progress
                        _this.anotherRestoreInProgres(true);
                        new monitorRestoreCommand($.Deferred(), function (s) {
                            self.dbRestoreOptions.updateRestoreStatus.bind(self.dbRestoreOptions)(s);
                            self.fsRestoreOptions.updateRestoreStatus.bind(self.fsRestoreOptions)(s);
                        })
                            .execute()
                            .always(function () {
                            $("#databaseRawLogsContainer").resize();
                            $("#filesystemRawLogsContainer").resize();
                            _this.anotherRestoreInProgres(false);
                        });
                    }
                    else {
                        _this.isBusy(false);
                    }
                    deferred.resolve({ can: true });
                });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        restore.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("FT7RV6");
        };
        restore.prototype.startDbRestore = function () {
            eventsCollector.default.reportEvent("database", "restore");
            this.isBusy(true);
            var self = this;
            var restoreDatabaseDto = {
                BackupLocation: this.dbRestoreOptions.backupLocation(),
                DatabaseLocation: this.dbRestoreOptions.resourceLocation(),
                DatabaseName: this.dbRestoreOptions.resourceName(),
                DisableReplicationDestinations: this.disableReplicationDestinations(),
                GenerateNewDatabaseId: this.generateNewDatabaseId(),
                IndexesLocation: this.dbRestoreOptions.indexesLocation(),
                JournalsLocation: this.dbRestoreOptions.journalsLocation()
            };
            new startDbRestoreCommand(this.dbRestoreOptions.defrag(), restoreDatabaseDto, self.dbRestoreOptions.updateRestoreStatus.bind(self.dbRestoreOptions))
                .execute();
        };
        restore.prototype.startFsRestore = function () {
            eventsCollector.default.reportEvent("fs", "restore");
            this.isBusy(true);
            var self = this;
            var restoreFilesystemDto = {
                BackupLocation: this.fsRestoreOptions.backupLocation(),
                FilesystemLocation: this.fsRestoreOptions.resourceLocation(),
                FilesystemName: this.fsRestoreOptions.resourceName(),
                IndexesLocation: this.fsRestoreOptions.indexesLocation(),
                JournalsLocation: this.fsRestoreOptions.journalsLocation()
            };
            new startFsRestoreCommand(this.fsRestoreOptions.defrag(), restoreFilesystemDto, self.fsRestoreOptions.updateRestoreStatus.bind(self.fsRestoreOptions))
                .execute();
        };
        return restore;
    }(viewModelBase));
    return restore;
});

//# sourceMappingURL=restore.js.map
