var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "models/resources/serverSmugglingItem", "models/database/cluster/serverConnectionInfo", "commands/operations/performSmugglingCommand", "common/appUrl", "common/jsonUtil", "common/serverSmugglingLocalStorage", "common/settingsAccessAuthorizer", "common/shell/resourcesManager", "common/eventsCollector"], function (require, exports, viewModelBase, serverSmugglingItem, serverConnectionInfo, performSmugglingCommand, appUrl, jsonUtil, serverSmugglingLocalStorage, settingsAccessAuthorizer, resourcesManager, eventsCollector) {
    "use strict";
    var serverSmuggling = (function (_super) {
        __extends(serverSmuggling, _super);
        function serverSmuggling() {
            var _this = _super.call(this) || this;
            _this.resources = ko.observableArray([]);
            _this.selectedResources = ko.observableArray();
            _this.targetServer = ko.observable(new serverConnectionInfo());
            _this.settingsAccess = new settingsAccessAuthorizer();
            _this.showJsonRequest = ko.observable(false);
            _this.showCurlRequest = ko.observable(false);
            _this.inProgress = ko.observable(false);
            _this.resultsVisible = ko.observable(false);
            _this.messages = ko.observableArray([]);
            var smi = resourcesManager.default.databases().map(function (d) { return new serverSmugglingItem(d); });
            _this.resources(smi);
            _this.hasResources = ko.computed(function () { return _this.resources().length > 0; });
            _this.noIncremental = ko.computed(function () { return _this.selectedResources().length === 0; });
            _this.noStripReplication = ko.computed(function () {
                var resources = _this.selectedResources();
                var replicationCount = resources.filter(function (x) { return x.hasReplicationBundle(); }).length;
                return resources.length === 0 || replicationCount === 0;
            });
            _this.noDisableVersioning = ko.computed(function () {
                var resources = _this.selectedResources();
                var versioningCount = resources.filter(function (x) { return x.hasVersioningBundle(); }).length;
                return resources.length === 0 || versioningCount === 0;
            });
            _this.resourcesSelection = ko.computed(function () {
                var selectedResourcesCount = _this.selectedResources().length;
                if (selectedResourcesCount === _this.resources().length)
                    return 2 /* Checked */;
                if (selectedResourcesCount > 0)
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
            _this.incrementalSelection = ko.computed(function () {
                var resources = _this.selectedResources();
                if (resources.length === 0)
                    return 0 /* UnChecked */;
                var incrementalCount = resources.filter(function (x) { return x.incremental(); }).length;
                if (incrementalCount === resources.length)
                    return 2 /* Checked */;
                if (incrementalCount > 0)
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
            _this.stripReplicationSelection = ko.computed(function () {
                var resources = _this.selectedResources();
                var replicationCount = resources.filter(function (x) { return x.stripReplicationInformation(); }).length;
                if (resources.length === 0 || replicationCount === 0)
                    return 0 /* UnChecked */;
                var replicationBundleCount = resources.filter(function (x) { return x.hasReplicationBundle(); }).length;
                if (replicationBundleCount === replicationCount)
                    return 2 /* Checked */;
                if (replicationBundleCount > 0)
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
            _this.disableVersioningSelection = ko.computed(function () {
                var resources = _this.selectedResources();
                var versioningCount = resources.filter(function (x) { return x.shouldDisableVersioningBundle(); }).length;
                if (resources.length === 0 || versioningCount === 0)
                    return 0 /* UnChecked */;
                var versioningBundleCount = resources.filter(function (x) { return x.hasVersioningBundle(); }).length;
                if (versioningBundleCount === versioningCount)
                    return 2 /* Checked */;
                if (versioningBundleCount > 0)
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
            _this.submitEnabled = ko.computed(function () {
                var progress = _this.inProgress();
                var selection = _this.selectedResources().length > 0;
                var url = _this.targetServer().url();
                return !progress && selection && !!url;
            });
            _this.jsonRequest = ko.computed(function () {
                var request = _this.getJson();
                return jsonUtil.syntaxHighlight(request);
            });
            _this.curlRequest = ko.computed(function () {
                var json = _.replace(JSON.stringify(_this.getJson(), null, 0), /\"/g, "\\\"");
                return "curl -i -H \"Accept: application/json\" -H \"Content-Type: application/json\" -X POST --data \"" + json + "\" " + appUrl.forServer() + "/admin/serverSmuggling";
            });
            _this.jsonRequest.subscribe(function () { return _this.saveIntoLocalStorage(); });
            _this.restoreFromLocalStorage();
            return _this;
        }
        serverSmuggling.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("MUJQ7G");
        };
        serverSmuggling.prototype.restoreFromLocalStorage = function () {
            var savedValue = serverSmugglingLocalStorage.get();
            if (savedValue) {
                var self = this;
                var targetServer = this.targetServer();
                targetServer.url(savedValue.TargetServer.Url);
                targetServer.domain(savedValue.TargetServer.Domain);
                targetServer.username(savedValue.TargetServer.Username);
                targetServer.guessCredentialsType();
                // since resources might change over time we have to apply saved changes carefully. 
                savedValue.Config.forEach(function (savedConfig) {
                    var item = self.resources().find(function (r) { return r.resource.name === savedConfig.Name; });
                    if (item) {
                        self.selectedResources.push(item);
                        item.incremental(savedConfig.Incremental);
                        if (item.hasVersioningBundle()) {
                            item.shouldDisableVersioningBundle(savedConfig.ShouldDisableVersioningBundle);
                        }
                        if (item.hasReplicationBundle()) {
                            item.stripReplicationInformation(savedConfig.StripReplicationInformation);
                        }
                    }
                });
            }
        };
        serverSmuggling.prototype.toggleSelectAll = function () {
            if (this.selectedResources().length > 0) {
                this.selectedResources([]);
            }
            else {
                this.selectedResources(this.resources().slice(0));
            }
        };
        serverSmuggling.prototype.toggleSelectAllIncremental = function () {
            var resources = this.selectedResources();
            if (resources.length === 0)
                return;
            var hasSelected = resources.filter(function (x) { return x.incremental(); }).length > 0;
            for (var i = 0; i < resources.length; i++) {
                resources[i].incremental(!hasSelected);
            }
        };
        serverSmuggling.prototype.toggleSelectAllStripReplication = function () {
            var resources = this.selectedResources();
            var replicationBundleCount = resources.filter(function (x) { return x.hasReplicationBundle(); }).length;
            if (resources.length === 0 || replicationBundleCount === 0)
                return;
            var hasSelected = resources.filter(function (x) { return x.stripReplicationInformation(); }).length > 0;
            for (var i = 0; i < resources.length; i++) {
                if (resources[i].hasReplicationBundle()) {
                    resources[i].stripReplicationInformation(!hasSelected);
                }
            }
        };
        serverSmuggling.prototype.toggleSelectAllDisableVersioning = function () {
            var resources = this.selectedResources();
            var versioningBundleCount = resources.filter(function (x) { return x.hasVersioningBundle(); }).length;
            if (resources.length === 0 || versioningBundleCount === 0)
                return;
            var hasSelected = resources.filter(function (x) { return x.shouldDisableVersioningBundle(); }).length > 0;
            for (var i = 0; i < resources.length; i++) {
                if (resources[i].hasVersioningBundle()) {
                    resources[i].shouldDisableVersioningBundle(!hasSelected);
                }
            }
        };
        serverSmuggling.prototype.isSelected = function (item) {
            return this.selectedResources().indexOf(item) >= 0;
        };
        serverSmuggling.prototype.saveIntoLocalStorage = function () {
            var json = this.getJson();
            json.TargetServer.ApiKey = undefined;
            json.TargetServer.Password = undefined;
            serverSmugglingLocalStorage.setValue(json);
        };
        serverSmuggling.prototype.performMigration = function () {
            var _this = this;
            eventsCollector.default.reportEvent("server-smuggling", "execute");
            var request = this.getJson();
            this.messages([]);
            this.inProgress(true);
            this.resultsVisible(true);
            new performSmugglingCommand(request, null, function (status) { return _this.updateProgress(status); })
                .execute()
                .always(function () { return _this.inProgress(false); });
        };
        serverSmuggling.prototype.getJson = function () {
            var targetServer = this.targetServer().toDto();
            var config = this.selectedResources().map(function (r) { return r.toDto(); });
            return {
                TargetServer: targetServer,
                Config: config
            };
        };
        serverSmuggling.prototype.updateProgress = function (progress) {
            this.messages(progress.Messages);
        };
        serverSmuggling.prototype.toggleJson = function () {
            eventsCollector.default.reportEvent("server-smuggling", "toggle-json");
            this.showJsonRequest(!this.showJsonRequest());
        };
        serverSmuggling.prototype.toggleCurl = function () {
            eventsCollector.default.reportEvent("server-smuggling", "toggle-curl");
            this.showCurlRequest(!this.showCurlRequest());
        };
        return serverSmuggling;
    }(viewModelBase));
    return serverSmuggling;
});

//# sourceMappingURL=serverSmuggling.js.map
