var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/database/documents/getDocumentWithMetadataCommand", "models/database/documents/document", "common/serverBuildReminder", "models/resources/environmentColor", "viewmodels/shell", "viewmodels/shell/accessHelper", "common/numberFormattingStorage", "common/eventsCollector"], function (require, exports, viewModelBase, getDocumentWithMetadataCommand, documentClass, serverBuildReminder, environmentColor, shell, accessHelper, numberFormattingStorage, eventsCollector) {
    "use strict";
    var studioConfig = (function (_super) {
        __extends(studioConfig, _super);
        function studioConfig() {
            var _this = _super.call(this) || this;
            _this.configDocument = ko.observable();
            _this.timeUntilRemindToUpgrade = ko.observable();
            _this.isForbidden = ko.observable();
            _this.browserFormatExample = 5050.99.toLocaleString();
            _this.rawFormat = ko.observable();
            _this.sendUsageStats = ko.observable(true);
            _this.environmentColors = [
                new environmentColor("Default", "#f8f8f8"),
                new environmentColor("Development", "#80FF80"),
                new environmentColor("Staging", "#F5824D"),
                new environmentColor("Production", "#FF8585")
            ];
            _this.selectedColor = ko.observable();
            _this.documentId = shell.studioConfigDocumentId;
            _this.timeUntilRemindToUpgrade(serverBuildReminder.get());
            _this.mute = ko.computed(function () {
                var lastBuildCheck = _this.timeUntilRemindToUpgrade();
                var timestamp = Date.parse(lastBuildCheck);
                var isLegalDate = !isNaN(timestamp);
                return isLegalDate;
            });
            _this.timeUntilRemindToUpgradeMessage = ko.computed(function () {
                if (_this.mute()) {
                    var lastBuildCheck = _this.timeUntilRemindToUpgrade();
                    var lastBuildCheckMoment = moment(lastBuildCheck);
                    return "muted for another " + lastBuildCheckMoment.add("days", 7).fromNow(true);
                }
                return "mute for a week";
            });
            /*TODO: var color = this.environmentColors.filter((color) => color.name === shell.selectedEnvironmentColorStatic().name);
            var selectedColor = !!color[0] ? color[0] : this.environmentColors[0];
            this.selectedColor(selectedColor);*/
            var self = _this;
            _this.selectedColor.subscribe(function (newValue) { return self.setEnvironmentColor(newValue); });
            _this.isForbidden((accessHelper.isGlobalAdmin() || accessHelper.canReadWriteSettings() || accessHelper.canReadSettings()) === false);
            _this.isReadOnly = ko.computed(function () { return accessHelper.isGlobalAdmin() === false && accessHelper.canReadWriteSettings() === false && accessHelper.canReadSettings(); });
            _this.rawFormat(numberFormattingStorage.shouldUseRaw());
            return _this;
        }
        studioConfig.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            if (this.isForbidden() === false) {
                new getDocumentWithMetadataCommand(this.documentId, null)
                    .execute()
                    .done(function (doc) {
                    _this.configDocument(doc);
                    _this.sendUsageStats(doc["SendUsageStats"]);
                })
                    .fail(function () { return _this.configDocument(documentClass.empty()); })
                    .always(function () { return deferred.resolve({ can: true }); });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        studioConfig.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("4J5OUB");
        };
        studioConfig.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            var self = this;
            $(window).bind('storage', function (e) {
                if (e.originalEvent.key === serverBuildReminder.localStorageName) {
                    self.timeUntilRemindToUpgrade(serverBuildReminder.get());
                }
            });
            $("select").selectpicker();
            this.pickColor();
            $("#select-color li").each(function (index, element) {
                var color = _this.environmentColors[index];
                $(element).css("backgroundColor", color.backgroundColor);
            });
        };
        studioConfig.prototype.setEnvironmentColor = function (envColor) {
            eventsCollector.default.reportEvent("studio-config", "env-color");
            var newDocument = this.configDocument();
            newDocument["EnvironmentColor"] = envColor.toDto();
            var saveTask = this.saveStudioConfig(newDocument);
            /* TODOsaveTask.done(() => {
                //TODO:shell.selectedEnvironmentColorStatic(this.selectedColor());
                this.pickColor();
            }); */
        };
        studioConfig.prototype.setSendUsageStats = function (setting) {
            eventsCollector.default.reportEvent("studio-config", "usage-stats");
            if (this.sendUsageStats() !== setting) {
                var newDocument = this.configDocument();
                this.sendUsageStats(setting);
                newDocument["SendUsageStats"] = setting;
                var saveTask = this.saveStudioConfig(newDocument);
            }
        };
        studioConfig.prototype.pickColor = function () {
            $("#select-color button").css("backgroundColor", this.selectedColor().backgroundColor);
        };
        studioConfig.prototype.setUpgradeReminder = function (upgradeSetting) {
            eventsCollector.default.reportEvent("studio-config", "upgrade-reminder");
            serverBuildReminder.mute(upgradeSetting);
        };
        studioConfig.prototype.setNumberFormat = function (raw) {
            eventsCollector.default.reportEvent("studio-config", "number-format");
            this.rawFormat(raw);
            numberFormattingStorage.save(raw);
        };
        studioConfig.prototype.saveStudioConfig = function (newDocument) {
            /* TODO:return new saveDocumentCommand(this.documentId, newDocument, null)
                .execute()
                .done((saveResult: bulkDocumentDto[]) => {
                    this.configDocument(newDocument);
                    this.configDocument().__metadata['etag'] = saveResult[0].Etag;
                });*/
        };
        return studioConfig;
    }(viewModelBase));
    return studioConfig;
});

//# sourceMappingURL=studioConfig.js.map
