var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "viewmodels/viewModelBase", "viewmodels/manage/watchTrafficConfigDialog", "common/trafficWatchClient", "commands/auth/getSingleAuthTokenCommand", "moment", "common/fileDownloader", "commands/database/query/enableQueryTimings", "models/resources/database", "viewmodels/shell/accessHelper", "common/eventsCollector"], function (require, exports, app, viewModelBase, watchTrafficConfigDialog, trafficWatchClient, getSingleAuthTokenCommand, moment, fileDownloader, enableQueryTimings, database, accessHelper, eventsCollector) {
    "use strict";
    var trafficWatch = (function (_super) {
        __extends(trafficWatch, _super);
        function trafficWatch() {
            var _this = _super.call(this) || this;
            _this.logConfig = ko.observable();
            _this.isConnected = ko.observable(false);
            _this.recentEntries = ko.observableArray([]);
            _this.now = ko.observable();
            _this.updateNowTimeoutHandle = 0;
            _this.selectedLog = ko.observable();
            _this.keepDown = ko.observable(false);
            _this.watchedRequests = ko.observable(0);
            _this.averageRequestDuration = ko.observable();
            _this.summedRequestsDuration = 0;
            _this.minRequestDuration = ko.observable(1000000);
            _this.maxRequestDuration = ko.observable(0);
            _this.startTraceTime = ko.observable();
            _this.showLogDetails = ko.observable(false);
            _this.isForbidden = ko.observable();
            _this.filter = ko.observable();
            _this.filterDuration = ko.observable();
            _this.startTraceTimeHumanized = ko.computed(function () {
                var a = _this.now();
                if (!!_this.startTraceTime()) {
                    return _this.parseHumanReadableTimeString(_this.startTraceTime().toString(), true, false);
                }
                return "";
            });
            _this.isForbidden(accessHelper.isGlobalAdmin() === false);
            _this.filter.throttle(250).subscribe(function () { return _this.filterEntries(); });
            _this.filterDuration.throttle(250).subscribe(function () { return _this.filterEntries(); });
            return _this;
        }
        trafficWatch.prototype.filterEntries = function () {
            var _this = this;
            this.recentEntries().forEach(function (entry) {
                entry.Visible(_this.isVisible(entry));
            });
        };
        trafficWatch.prototype.canActivate = function (args) {
            return true;
        };
        trafficWatch.prototype.activate = function (args) {
            var widthUnit = 0.075;
            this.columnWidths = [
                ko.observable(100 * widthUnit),
                ko.observable(100 * widthUnit),
                ko.observable(100 * widthUnit),
                ko.observable(100 * widthUnit),
                ko.observable(100 * widthUnit * 7),
                ko.observable(100 * widthUnit)
            ];
            this.registerColumnResizing();
            this.updateHelpLink('EVEP6I');
        };
        trafficWatch.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.showLogDetails.subscribe(function (x) {
                $(".logRecords").toggleClass("logRecords-small");
            });
            this.updateCurrentNowTime();
            this.logRecordsElement = document.getElementById("logRecords");
        };
        trafficWatch.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(document).on("mousedown.logTableColumnResize", ".column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = _this.columnWidths[columnIndex]();
                startX = e.pageX;
                resizingColumn = true;
            });
            $(document).on("mouseup.logTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.logTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var logsRecordsContainerWidth = $("#logRecordsContainer").width();
                    var targetColumnSize = startingWidth + 100 * (e.pageX - startX) / logsRecordsContainerWidth;
                    _this.columnWidths[columnIndex](targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        trafficWatch.prototype.configureConnection = function () {
            var _this = this;
            eventsCollector.default.reportEvent("traffic-watch", "configure");
            var configDialog = new watchTrafficConfigDialog();
            app.showBootstrapDialog(configDialog);
            configDialog.configurationTask.done(function (x) {
                _this.logConfig(x);
                _this.enableTimingsTimer = setInterval(function () { return _this.enableQueryTiming(); }, 4.8 * 60 * 1000);
                _this.enableQueryTiming();
                _this.reconnect();
            });
        };
        trafficWatch.prototype.enableQueryTiming = function () {
            if (this.logConfig().Resource instanceof database) {
                new enableQueryTimings(this.logConfig().Resource).execute();
            }
        };
        trafficWatch.prototype.reconnect = function () {
            var _this = this;
            eventsCollector.default.reportEvent("traffic-watch", "reconnect");
            if (!this.watchClient) {
                if (!this.logConfig) {
                    app.showBootstrapMessage("Cannot reconnect, please configure connection properly", "Connection Error");
                    return;
                }
                this.connect();
            }
            else {
                this.disconnect().done(function () {
                    _this.connect();
                });
            }
        };
        trafficWatch.prototype.connect = function () {
            var _this = this;
            eventsCollector.default.reportEvent("traffic-watch", "connect");
            if (!!this.watchClient) {
                this.reconnect();
                return;
            }
            if (!this.logConfig()) {
                this.configureConnection();
                return;
            }
            var tokenDeferred = $.Deferred();
            if (!this.logConfig().SingleAuthToken) {
                new getSingleAuthTokenCommand(this.logConfig().Resource, this.logConfig().WatchedResourceMode === "AdminView")
                    .execute()
                    .done(function (tokenObject) {
                    _this.logConfig().SingleAuthToken = tokenObject;
                    tokenDeferred.resolve();
                })
                    .fail(function (e) {
                    app.showBootstrapMessage("You are not authorized to trace this resource", "Authorization error");
                });
            }
            else {
                tokenDeferred.resolve();
            }
            tokenDeferred.done(function () {
                _this.watchClient = new trafficWatchClient(_this.logConfig().ResourcePath, _this.logConfig().SingleAuthToken.Token);
                _this.watchClient.connect();
                _this.watchClient.connectionOpeningTask.done(function () {
                    _this.isConnected(true);
                    _this.watchClient.watchTraffic(function (event) {
                        _this.processHttpTraceMessage(event);
                    });
                    if (!_this.startTraceTime()) {
                        _this.startTraceTime(_this.now());
                    }
                });
                _this.logConfig().SingleAuthToken = null;
            });
        };
        trafficWatch.prototype.disconnect = function () {
            var _this = this;
            eventsCollector.default.reportEvent("traffic-watch", "disconnect");
            if (!!this.watchClient) {
                this.watchClient.disconnect();
                return this.watchClient.connectionClosingTask.done(function () {
                    _this.watchClient = null;
                    _this.isConnected(false);
                });
            }
            else {
                app.showBootstrapMessage("Cannot disconnect, connection does not exist", "Disconnect");
                return $.Deferred().reject();
            }
        };
        trafficWatch.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            if (this.enableTimingsTimer) {
                clearInterval(this.enableTimingsTimer);
                this.enableTimingsTimer = null;
            }
            if (this.isConnected())
                this.disconnect();
        };
        trafficWatch.prototype.isVisible = function (logEntry) {
            if (this.filterDuration() && logEntry.Duration < parseInt(this.filterDuration())) {
                return false;
            }
            if (this.filter() && logEntry.Url.indexOf(this.filter()) === -1) {
                return false;
            }
            return true;
        };
        trafficWatch.prototype.processHttpTraceMessage = function (e) {
            var logObject;
            var mapTimings = function (value) {
                var result = [];
                if (!value) {
                    return result;
                }
                for (var key in value) {
                    if (value.hasOwnProperty(key)) {
                        result.push({ key: key, value: value[key] });
                    }
                }
                return result;
            };
            logObject = {
                Time: this.createHumanReadableTime(e.TimeStamp, false, true),
                Duration: e.ElapsedMilliseconds,
                Resource: e.TenantName,
                Method: e.HttpMethod,
                Url: e.RequestUri,
                CustomInfo: e.CustomInfo,
                TimeStampText: this.createHumanReadableTime(e.TimeStamp, true, false),
                QueryTimings: mapTimings(e.QueryTimings),
                Visible: ko.observable()
            };
            if (logObject.CustomInfo) {
                var withBr = _.replace(decodeURIComponent(logObject.CustomInfo), /\n/g, "<Br />");
                logObject.CustomInfo = _.replace(withBr).replace(/Inner Request/g, "<strong>Inner Request</strong>");
            }
            if (e.InnerRequestsCount > 0) {
                logObject.Url = "(" + e.InnerRequestsCount + " requests) " + logObject.Url;
            }
            if (this.recentEntries().length == this.logConfig().MaxEntries) {
                this.recentEntries.shift();
            }
            logObject.Visible(this.isVisible(logObject));
            this.recentEntries.push(logObject);
            if (this.keepDown()) {
                this.logRecordsElement.scrollTop = this.logRecordsElement.scrollHeight * 1.1;
            }
            this.watchedRequests(this.watchedRequests() + 1);
            this.summedRequestsDuration += e.ElapsedMilliseconds;
            this.averageRequestDuration((this.summedRequestsDuration / this.watchedRequests()).toFixed(2));
            this.minRequestDuration(this.minRequestDuration() > e.ElapsedMilliseconds ? e.ElapsedMilliseconds : this.minRequestDuration());
            this.maxRequestDuration(this.maxRequestDuration() < e.ElapsedMilliseconds ? e.ElapsedMilliseconds : this.maxRequestDuration());
        };
        trafficWatch.prototype.selectLog = function (log) {
            this.selectedLog(log);
            this.showLogDetails(true);
            $(".logRecords").addClass("logRecords-small");
        };
        trafficWatch.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            if (this.updateNowTimeoutHandle != 0)
                clearTimeout(this.updateNowTimeoutHandle);
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 1000);
        };
        trafficWatch.prototype.createHumanReadableTime = function (time, chainHumanized, chainDateTime) {
            var _this = this;
            if (chainHumanized === void 0) { chainHumanized = true; }
            if (chainDateTime === void 0) { chainDateTime = true; }
            if (time) {
                return ko.computed(function () {
                    return _this.parseHumanReadableTimeString(time, chainHumanized, chainDateTime);
                });
            }
            return ko.computed(function () { return time; });
        };
        trafficWatch.prototype.parseHumanReadableTimeString = function (time, chainHumanized, chainDateTime) {
            if (chainHumanized === void 0) { chainHumanized = true; }
            if (chainDateTime === void 0) { chainDateTime = true; }
            var dateMoment = moment(time);
            var humanized = "", formattedDateTime = "";
            var agoInMs = dateMoment.diff(this.now());
            if (chainHumanized)
                humanized = moment.duration(agoInMs).humanize(true);
            if (chainDateTime)
                formattedDateTime = dateMoment.format(" (ddd MMM DD YYYY HH:mm:ss.SS[GMT]ZZ)");
            return humanized + formattedDateTime;
        };
        trafficWatch.prototype.formatLogRecord = function (logRecord) {
            return 'Request #' + _.padEnd(logRecord.RequestId.toString(), 4)
                + ' ' + _.padStart(logRecord.HttpMethod.toString(), 7)
                + ' - ' + _.padEnd(logRecord.ElapsedMilliseconds.toString(), 5) + ' ms - '
                + _.padStart(logRecord.TenantName, 10) + ' - '
                + logRecord.ResponseStatusCode + ' - ' + logRecord.RequestUri;
        };
        trafficWatch.prototype.resetStats = function () {
            eventsCollector.default.reportEvent("traffic-watch", "reset-stats");
            this.watchedRequests(0);
            this.averageRequestDuration("0");
            this.summedRequestsDuration = 0;
            this.minRequestDuration(1000000);
            this.maxRequestDuration(0);
            this.startTraceTime(null);
        };
        trafficWatch.prototype.exportTraffic = function () {
            eventsCollector.default.reportEvent("traffic-watch", "export");
            fileDownloader.downloadAsJson(this.recentEntries(), "traffic.json");
        };
        trafficWatch.prototype.clearLogs = function () {
            eventsCollector.default.reportEvent("traffic-watch", "clear");
            this.recentEntries.removeAll();
        };
        trafficWatch.prototype.toggleKeepDown = function () {
            eventsCollector.default.reportEvent("traffic-watch", "keep-down");
            this.keepDown.toggle();
            if (this.keepDown()) {
                this.logRecordsElement.scrollTop = this.logRecordsElement.scrollHeight * 1.1;
            }
        };
        return trafficWatch;
    }(viewModelBase));
    return trafficWatch;
});

//# sourceMappingURL=trafficWatch.js.map
