var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/auth/getSingleAuthTokenCommand", "common/appUrl", "common/shell/resourcesManager"], function (require, exports, app, dialog, dialogViewModelBase, getSingleAuthTokenCommand, appUrl, resourcesManager) {
    "use strict";
    var watchTrafficConfigDialog = (function (_super) {
        __extends(watchTrafficConfigDialog, _super);
        function watchTrafficConfigDialog() {
            var _this = _super.call(this) || this;
            _this.resourcesManager = resourcesManager.default;
            _this.configurationTask = $.Deferred();
            _this.watchedResourceMode = ko.observable("SingleResourceView");
            _this.resourceName = ko.observable("");
            _this.lastSearchedwatchedResourceName = ko.observable();
            _this.resourceAutocompletes = ko.observableArray([]);
            _this.maxEntries = ko.observable(1000);
            _this.allResourcesNames = _this.resourcesManager.resources().map(function (x) { return x.name; });
            _this.searchResults = ko.computed(function () {
                var newResourceName = _this.resourceName();
                return _this.allResourcesNames.filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            _this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var newResourceName = _this.resourceName();
                var isSingleResourceView = _this.watchedResourceMode() === "SingleResourceView";
                if (isSingleResourceView) {
                    if (!newResourceName) {
                        errorMessage = "Resource name is required";
                    }
                    else {
                        var foundResource = _this.allResourcesNames.find(function (name) { return name === newResourceName; });
                        if (!foundResource) {
                            errorMessage = "Resource name doesn't exist!";
                        }
                    }
                }
                return errorMessage;
            });
            return _this;
        }
        watchTrafficConfigDialog.prototype.bindingComplete = function () {
            document.getElementById("watchedResource").focus();
        };
        watchTrafficConfigDialog.prototype.enterKeyPressed = function () {
            return true;
        };
        watchTrafficConfigDialog.prototype.fetchResourcesAutocompletes = function (search) {
            if (this.resourceName() === search) {
                if (this.resourceAutocompletes.length === 1 && this.resourceName() === this.resourceAutocompletes()[0]) {
                    this.resourceAutocompletes.removeAll();
                    return;
                }
                this.resourceAutocompletes(this.allResourcesNames.filter(function (name) { return name.toLowerCase().indexOf(search.toLowerCase()) === 0; }));
            }
        };
        watchTrafficConfigDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        watchTrafficConfigDialog.prototype.deactivate = function () {
            this.configurationTask.reject();
        };
        watchTrafficConfigDialog.prototype.confirmConfig = function () {
            var _this = this;
            var tracedResource;
            if (this.watchedResourceMode() === "SingleResourceView")
                tracedResource = this.resourcesManager.resources().find(function (rs) { return rs.name === _this.resourceName(); });
            var resourcePath = appUrl.forResourceQuery(tracedResource);
            var getTokenTask = new getSingleAuthTokenCommand(tracedResource, this.watchedResourceMode() === "AdminView").execute();
            getTokenTask
                .done(function (tokenObject) {
                _this.configurationTask.resolve({
                    Resource: tracedResource,
                    ResourceName: tracedResource.name,
                    ResourcePath: resourcePath,
                    MaxEntries: _this.maxEntries(),
                    WatchedResourceMode: _this.watchedResourceMode(),
                    SingleAuthToken: tokenObject
                });
                dialog.close(_this);
            })
                .fail(function (e) {
                var response = JSON.parse(e.responseText);
                var msg = e.statusText;
                if ("Error" in response) {
                    msg += ": " + response.Error;
                }
                else if ("Reason" in response) {
                    msg += ": " + response.Reason;
                }
                app.showBootstrapMessage(msg, "Error");
            });
        };
        watchTrafficConfigDialog.prototype.generateBindingInputId = function (index) {
            return "binding-" + index;
        };
        return watchTrafficConfigDialog;
    }(dialogViewModelBase));
    return watchTrafficConfigDialog;
});

//# sourceMappingURL=watchTrafficConfigDialog.js.map
