var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/dialog", "models/resources/database", "viewmodels/resources/createResourceBase", "common/shell/resourcesManager", "commands/resources/createDatabaseCommand", "models/resources/creation/databaseCreationModel", "common/eventsCollector"], function (require, exports, dialog, database, createResourceBase, resourcesManager, createDatabaseCommand, databaseCreationModel, eventsCollector) {
    "use strict";
    var createDatabase = (function (_super) {
        __extends(createDatabase, _super);
        function createDatabase() {
            var _this = _super.apply(this, arguments) || this;
            _this.databaseBundles = [
                {
                    displayName: "Compression",
                    name: "Compression",
                    hasAdvancedConfiguration: false
                },
                {
                    displayName: "Encryption",
                    name: "Encryption",
                    hasAdvancedConfiguration: true
                }
            ];
            _this.bundlesEnabled = {
                encryption: _this.isBundleActiveComputed("Encryption")
            };
            _this.resourceModel = new databaseCreationModel();
            _this.advancedVisibility = {
                encryption: ko.pureComputed(function () { return _this.advancedBundleConfigurationVisible() === "Encryption"; })
            };
            return _this;
        }
        createDatabase.prototype.getResourceByName = function (name) {
            return resourcesManager.default.getDatabaseByName(name);
        };
        createDatabase.prototype.activate = function () {
            _super.prototype.activate.call(this);
            //TODO: if cluster mode preselect replication bundle
            //TODO: if !!this.licenseStatus() && this.licenseStatus().IsCommercial && this.licenseStatus().Attributes.periodicBackup !== "true" preselect periodic export
            //TODO: fetchClusterWideConfig
            //TODO: fetchCustomBundles
        };
        createDatabase.prototype.initObservables = function () {
            var _this = this;
            _super.prototype.initObservables.call(this);
            this.indexesPathPlaceholder = ko.pureComputed(function () {
                var name = _this.resourceModel.name();
                return "~/" + (name || "{Database Name}") + "/Indexes/";
            });
            this.databaseBundles.forEach(function (bundle) {
                if (!bundle.hasOwnProperty('validationGroup')) {
                    bundle.validationGroup = undefined;
                }
            });
            var encryptionConfig = this.databaseBundles.find(function (x) { return x.name === "Encryption"; });
            encryptionConfig.validationGroup = this.resourceModel.encryptionValidationGroup;
        };
        createDatabase.prototype.getAvailableBundles = function () {
            //TODO: concat with custom bundles 
            return this.databaseBundles;
        };
        createDatabase.prototype.createResource = function () {
            eventsCollector.default.reportEvent('resource', 'create');
            var globalValid = this.isValid(this.resourceModel.globalValidationGroup);
            var advancedValid = this.isValid(this.resourceModel.advancedValidationGroup);
            var encryptionValid = !this.bundlesEnabled.encryption() || this.isValid(this.resourceModel.encryptionValidationGroup);
            var allValid = globalValid && advancedValid && encryptionValid;
            if (allValid) {
                this.createResourceInternal();
            }
            else {
                if (!advancedValid) {
                    if (!this.advancedConfigurationVisible()) {
                        this.showAdvancedConfiguration();
                    }
                }
                else if (!encryptionValid) {
                    if (!this.advancedVisibility.encryption()) {
                        this.showAdvancedConfigurationFor("Encryption");
                    }
                }
            }
        };
        createDatabase.prototype.createResourceInternal = function () {
            var _this = this;
            var databaseDocument = this.resourceModel.toDto();
            resourcesManager.default.activateAfterCreation(database.qualifier, databaseDocument.Id);
            new createDatabaseCommand(databaseDocument)
                .execute()
                .always(function () {
                dialog.close(_this);
            });
            //TODO: issue requests for additional bundles configuration + show dialog about encryption configuration so user can save this 
        };
        createDatabase.prototype.isBundleActiveComputed = function (bundleName) {
            var _this = this;
            return ko.pureComputed(function () { return _.includes(_this.resourceModel.activeBundles(), bundleName); });
        };
        return createDatabase;
    }(createResourceBase));
    return createDatabase;
});

//# sourceMappingURL=createDatabase.js.map
