var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "common/appUrl", "viewmodels/viewModelBase", "viewmodels/shell/accessHelper", "viewmodels/resources/deleteResourceConfirm", "viewmodels/resources/createDatabase", "viewmodels/resources/disableResourceToggleConfirm", "commands/resources/disableResourceToggleCommand", "commands/database/index/toggleIndexingCommand", "commands/resources/deleteResourceCommand", "commands/resources/loadResourceCommand", "models/resources/info/resourcesInfo", "commands/resources/getResourcesCommand", "commands/resources/getResourceCommand", "models/resources/info/resourceInfo", "models/resources/info/databaseInfo", "models/resources/info/filesystemInfo"], function (require, exports, app, appUrl, viewModelBase, accessHelper, deleteResourceConfirm, createDatabase, disableResourceToggleConfirm, disableResourceToggleCommand, toggleIndexingCommand, deleteResourceCommand, loadResourceCommand, resourcesInfo, getResourcesCommand, getResourceCommand, resourceInfo, databaseInfo, filesystemInfo) {
    "use strict";
    var resources = (function (_super) {
        __extends(resources, _super);
        function resources() {
            var _this = _super.call(this) || this;
            _this.resources = ko.observable();
            _this.filters = {
                searchText: ko.observable(),
                includeDatabases: ko.observable(true),
                includeFileSystems: ko.observable(true)
            };
            _this.selectedResources = ko.observableArray([]);
            _this.spinners = {
                globalToggleDisable: ko.observable(false),
                itemTakedowns: ko.observableArray([]),
                disableIndexing: ko.observableArray([]),
                toggleRejectMode: ko.observableArray([]) //TODO: bind on UI
            };
            _this.compactView = resources.compactView;
            _this.isGlobalAdmin = accessHelper.isGlobalAdmin;
            _this.bindToCurrentInstance("toggleResource", "toggleDatabaseIndexing", "deleteResource", "activateResource");
            _this.initObservables();
            return _this;
        }
        resources.prototype.initObservables = function () {
            var _this = this;
            var filters = this.filters;
            filters.searchText.throttle(200).subscribe(function () { return _this.filterResources(); });
            filters.includeDatabases.subscribe(function () { return _this.filterResources(); });
            filters.includeFileSystems.subscribe(function () { return _this.filterResources(); });
            this.selectionState = ko.pureComputed(function () {
                var resources = _this.resources().sortedResources().filter(function (x) { return !x.filteredOut(); });
                var selectedCount = _this.selectedResources().length;
                if (resources.length && selectedCount === resources.length)
                    return 2 /* Checked */;
                if (selectedCount > 0)
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
            this.allCheckedResourcesDisabled = ko.pureComputed(function () {
                var selected = _this.getSelectedResources();
                return selected.length === selected.filter(function (x) { return x.disabled(); }).length;
            });
        };
        // Override canActivate: we can always load this page, regardless of any system db prompt.
        resources.prototype.canActivate = function (args) {
            return true;
        };
        resources.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            // we can't use createNotifications here, as it is called after *resource changes API* is connected, but user
            // can enter this view and never select resource
            this.addNotification(this.changesContext.globalChangesApi().watchItemsStartingWith("db/", function (e) { return _this.fetchResource(e); }));
            this.addNotification(this.changesContext.globalChangesApi().watchReconnect(function () { return _this.fetchResources(); }));
            // TODO: add notification for fs, cs, ts
            return this.fetchResources();
        };
        resources.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.updateHelpLink("Z8DC3Q");
            ko.postbox.publish("SetRawJSONUrl", appUrl.forDatabasesRawData());
        };
        resources.prototype.fetchResources = function () {
            var _this = this;
            return new getResourcesCommand()
                .execute()
                .done(function (info) { return _this.resources(new resourcesInfo(info)); });
        };
        resources.prototype.fetchResource = function (e) {
            var qualiferAndName = resourceInfo.extractQualifierAndNameFromNotification(e.Id);
            switch (e.Operation) {
                case "Loaded":
                case "Write":
                    this.updateResourceInfo(qualiferAndName.qualifier, qualiferAndName.name);
                    break;
                case "Delete":
                    var resource_1 = this.resources().sortedResources().find(function (rs) { return rs.qualifiedName === e.Id; });
                    if (resource_1) {
                        this.removeResource(resource_1);
                    }
                    break;
            }
        };
        resources.prototype.updateResourceInfo = function (qualifer, resourceName) {
            var _this = this;
            new getResourceCommand(qualifer, resourceName)
                .execute()
                .done(function (result) {
                _this.resources().updateResource(result, qualifer);
                _this.filterResources();
            });
        };
        resources.prototype.filterResources = function () {
            var _this = this;
            var filters = this.filters;
            var searchText = filters.searchText();
            var hasSearchText = !!searchText;
            if (hasSearchText) {
                searchText = searchText.toLowerCase();
            }
            var matchesFilters = function (rs) {
                if (rs instanceof databaseInfo && !filters.includeDatabases())
                    return false;
                if (rs instanceof filesystemInfo && !filters.includeFileSystems())
                    return false;
                return !hasSearchText || rs.name.toLowerCase().indexOf(searchText) >= 0;
            };
            var resources = this.resources();
            resources.sortedResources().forEach(function (resource) {
                var matches = matchesFilters(resource);
                resource.filteredOut(!matches);
                if (!matches) {
                    _this.selectedResources.remove(resource.qualifiedName);
                }
            });
        };
        resources.prototype.resourceUrl = function (rs) {
            if (rs instanceof databaseInfo) {
                var db = rs.asResource();
                return appUrl.forDocuments(null, db);
            }
            //TODO:fs, cs, ts
            return null;
        };
        resources.prototype.getSelectedResources = function () {
            var selected = this.selectedResources();
            return this.resources().sortedResources().filter(function (x) { return _.includes(selected, x.qualifiedName); });
        };
        resources.prototype.toggleSelectAll = function () {
            var selectedCount = this.selectedResources().length;
            if (selectedCount > 0) {
                this.selectedResources([]);
            }
            else {
                var namesToSelect_1 = [];
                this.resources().sortedResources().forEach(function (resource) {
                    if (!resource.filteredOut()) {
                        namesToSelect_1.push(resource.qualifiedName);
                    }
                });
                this.selectedResources(namesToSelect_1);
            }
        };
        resources.prototype.deleteResource = function (rs) {
            this.deleteResources([rs]);
        };
        resources.prototype.deleteSelectedResources = function () {
            this.deleteResources(this.getSelectedResources());
        };
        resources.prototype.deleteResources = function (toDelete) {
            var _this = this;
            var confirmDeleteViewModel = new deleteResourceConfirm(toDelete);
            confirmDeleteViewModel
                .result
                .done(function (confirmResult) {
                if (confirmResult.can) {
                    new deleteResourceCommand(toDelete.map(function (x) { return x.asResource(); }), !confirmResult.keepFiles)
                        .execute()
                        .done(function (deletedResources) {
                        deletedResources.forEach(function (rs) { return _this.onResourceDeleted(rs); });
                    });
                }
            });
            app.showBootstrapDialog(confirmDeleteViewModel);
        };
        resources.prototype.onResourceDeleted = function (deletedResource) {
            var matchedResource = this.resources()
                .sortedResources()
                .find(function (x) { return x.qualifiedName.toLowerCase() === deletedResource.qualifiedName.toLowerCase(); });
            if (matchedResource) {
                this.removeResource(matchedResource);
            }
        };
        resources.prototype.removeResource = function (rsInfo) {
            this.resources().sortedResources.remove(rsInfo);
            this.selectedResources.remove(rsInfo.qualifiedName);
        };
        resources.prototype.toggleSelectedResources = function () {
            var _this = this;
            var disableAll = !this.allCheckedResourcesDisabled();
            var selectedResources = this.getSelectedResources().map(function (x) { return x.asResource(); });
            if (selectedResources.length > 0) {
                var disableDatabaseToggleViewModel = new disableResourceToggleConfirm(selectedResources, disableAll);
                disableDatabaseToggleViewModel.result.done(function (result) {
                    if (result.can) {
                        _this.spinners.globalToggleDisable(true);
                        new disableResourceToggleCommand(selectedResources, disableAll)
                            .execute()
                            .done(function (disableResult) {
                            disableResult.forEach(function (x) { return _this.onResourceDisabled(x); });
                        })
                            .always(function () { return _this.spinners.globalToggleDisable(false); });
                    }
                });
                app.showBootstrapDialog(disableDatabaseToggleViewModel);
            }
        };
        resources.prototype.toggleResource = function (rsInfo) {
            var _this = this;
            var disable = !rsInfo.disabled();
            var rs = rsInfo.asResource();
            var disableDatabaseToggleViewModel = new disableResourceToggleConfirm([rs], disable);
            disableDatabaseToggleViewModel.result.done(function (result) {
                if (result.can) {
                    _this.spinners.itemTakedowns.push(rs.qualifiedName);
                    new disableResourceToggleCommand([rs], disable)
                        .execute()
                        .done(function (disableResult) {
                        disableResult.forEach(function (x) { return _this.onResourceDisabled(x); });
                    })
                        .always(function () { return _this.spinners.itemTakedowns.remove(rs.qualifiedName); });
                }
            });
            app.showBootstrapDialog(disableDatabaseToggleViewModel);
        };
        resources.prototype.onResourceDisabled = function (result) {
            var resources = this.resources().sortedResources();
            var matchedResource = resources.find(function (rs) { return rs.qualifiedName === result.QualifiedName; });
            if (matchedResource) {
                matchedResource.disabled(result.Disabled);
                // If Enabling a resource (that is selected from the top) than we want it to be Online(Loaded)
                if (matchedResource.isCurrentlyActiveResource() && !matchedResource.disabled()) {
                    new loadResourceCommand(matchedResource.asResource())
                        .execute();
                }
            }
        };
        resources.prototype.toggleDatabaseIndexing = function (db) {
            var _this = this;
            var enableIndexing = !db.indexingEnabled();
            var message = enableIndexing ? "Enable" : "Disable";
            this.confirmationMessage("Are you sure?", message + " indexing?")
                .done(function (result) {
                if (result.can) {
                    _this.spinners.disableIndexing.push(db.qualifiedName);
                    new toggleIndexingCommand(true, db.asResource())
                        .execute()
                        .done(function () { return db.indexingEnabled(enableIndexing); })
                        .always(function () { return _this.spinners.disableIndexing.remove(db.qualifiedName); });
                }
            });
        };
        resources.prototype.toggleRejectDatabaseClients = function (db) {
            var rejectClients = !db.rejectClients();
            var message = rejectClients ? "reject clients mode" : "accept clients mode";
            this.confirmationMessage("Are you sure?", "Switch to " + message)
                .done(function (result) {
                if (result.can) {
                }
            });
        };
        resources.prototype.newDatabase = function () {
            var createDbView = new createDatabase();
            app.showBootstrapDialog(createDbView);
        };
        resources.prototype.activateResource = function (rsInfo) {
            var resource = this.resourcesManager.getResourceByQualifiedName(rsInfo.qualifiedName);
            if (!resource || resource.disabled())
                return;
            resource.activate();
            this.updateResourceInfo(resource.qualifier, resource.name);
        };
        return resources;
    }(viewModelBase));
    resources.compactView = ko.observable(false);
    return resources;
});

//# sourceMappingURL=resources.js.map
