/// <reference path="../../typings/tsd.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "plugins/router", "durandal/app", "durandal/system", "durandal/viewLocator", "common/shell/menu", "common/shell/menu/generateMenuItems", "common/shell/activeResourceTracker", "common/shell/resourceSwitcher", "common/shell/searchBox", "models/database/documents/collection", "models/auth/license", "models/resources/environmentColor", "common/changesContext", "common/shell/routes", "viewmodels/shell/registration", "common/appUrl", "common/uploadQueueHelper", "common/bindingHelpers/dynamicHeightBindingHandler", "common/bindingHelpers/autoCompleteBindingHandler", "common/bindingHelpers/enableResizeBindingHandler", "common/bindingHelpers/helpBindingHandler", "common/oauthContext", "common/messagePublisher", "common/apiKeyLocalStorage", "common/extensions", "common/notifications/notificationCenter", "widgets/virtualGrid/virtualGrid", "commands/database/studio/getClientBuildVersionCommand", "commands/auth/getSupportCoverageCommand", "commands/database/studio/getServerConfigsCommand", "commands/database/cluster/getClusterTopologyCommand", "commands/resources/getServerBuildVersionCommand", "viewmodels/viewModelBase", "viewmodels/shell/accessHelper", "viewmodels/common/licensingStatus", "viewmodels/common/enterApiKey", "common/eventsCollector"], function (require, exports, router, app, sys, viewLocator, menu, generateMenuItems, activeResourceTracker, resourceSwitcher, searchBox, collection, license, environmentColor, changesContext, allRoutes, registration, appUrl, uploadQueueHelper, dynamicHeightBindingHandler, autoCompleteBindingHandler, enableResizeBindingHandler, helpBindingHandler, oauthContext, messagePublisher, apiKeyLocalStorage, extensions, notificationCenter, virtualGrid, getClientBuildVersionCommand, getSupportCoverageCommand, getServerConfigsCommand, getClusterTopologyCommand, getServerBuildVersionCommand, viewModelBase, accessHelper, licensingStatus, enterApiKey, eventsCollector) {
    "use strict";
    //TODO: extract cluster related logic to separate class
    //TODO: extract api key related logic to separate class 
    var shell = (function (_super) {
        __extends(shell, _super);
        function shell() {
            var _this = _super.call(this) || this;
            _this.router = router;
            _this.showContinueTestButton = ko.computed(function () { return viewModelBase.hasContinueTestOption(); }); //TODO:
            _this.notificationCenter = notificationCenter.instance;
            _this.isInCluster = ko.computed(function () { return shell.clusterMode(); }); //TODO: extract from shell
            _this.clientBuildVersion = ko.observable();
            _this.currentRawUrl = ko.observable("");
            _this.rawUrlIsVisible = ko.computed(function () { return _this.currentRawUrl().length > 0; });
            _this.showSplash = viewModelBase.showSplash;
            _this.licenseStatus = license.licenseCssClass;
            _this.supportStatus = license.supportCssClass;
            _this.mainMenu = new menu(generateMenuItems(activeResourceTracker.default.resource()));
            _this.searchBox = new searchBox();
            _this.resourceSwitcher = new resourceSwitcher();
            _this.displayUsageStatsInfo = ko.observable(false);
            _this.trackingTask = $.Deferred();
            _this.preLoadRecentErrorsView();
            extensions.install();
            _this.showLogOutButton = ko.computed(function () {
                var lsApiKey = apiKeyLocalStorage.get();
                var contextApiKey = oauthContext.apiKey();
                return lsApiKey || contextApiKey;
            });
            oauthContext.enterApiKeyTask = _this.setupApiKey();
            oauthContext.enterApiKeyTask.done(function () {
                changesContext.default
                    .connectGlobalChangesApi()
                    .done(function () {
                    _this.resourcesManager.createGlobalNotifications();
                });
            });
            ko.postbox.subscribe("SetRawJSONUrl", function (jsonUrl) { return _this.currentRawUrl(jsonUrl); });
            ko.postbox.subscribe("UploadFileStatusChanged", function (uploadStatus) { return _this.uploadStatusChanged(uploadStatus); });
            dynamicHeightBindingHandler.install();
            autoCompleteBindingHandler.install();
            enableResizeBindingHandler.install();
            helpBindingHandler.install();
            virtualGrid.install();
            _this.clientBuildVersion.subscribe(function (v) {
                return viewModelBase.clientVersion(v.Version);
            });
            shell.serverBuildVersion.subscribe(function (buildVersionDto) {
                _this.initAnalytics({ SendUsageStats: true }, [buildVersionDto]);
            });
            return _this;
        }
        // Override canActivate: we can always load this page, regardless of any system db prompt.
        shell.prototype.canActivate = function (args) {
            return true;
        };
        shell.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args, true);
            oauthContext.enterApiKeyTask.done(function () { return _this.connectToRavenServer(); });
            this.setupRouting();
            var self = this;
            $(window).bind("storage", function (e) {
                if (e.originalEvent.key === apiKeyLocalStorage.localStorageName) {
                    _this.onLogOut();
                }
            });
            $(window).resize(function () { return self.activeResource.valueHasMutated(); });
            this.fetchClientBuildVersion();
            this.fetchServerBuildVersion();
            return license.fetchLicenseStatus();
        };
        shell.prototype.setupRouting = function () {
            var routes = allRoutes.get(this.appUrls);
            routes.push.apply(routes, routes);
            router.map(routes).buildNavigationModel();
            //TODO: do we indicate this? router.isNavigating.subscribe(isNavigating => this.showNavigationProgress(isNavigating));
            //TODO: do we indicated this? router.on('router:navigation:cancelled', () => this.showNavigationProgress(false));
            appUrl.mapUnknownRoutes(router);
        };
        shell.prototype.attached = function () {
            _super.prototype.attached.call(this);
            sys.error = function (e) {
                console.error(e);
                messagePublisher.reportError("Failed to load routed module!", e);
            };
        };
        shell.prototype.initializeShellComponents = function () {
            var _this = this;
            this.mainMenu.initialize();
            var updateMenu = function (resource) {
                var items = generateMenuItems(resource);
                _this.mainMenu.update(items);
            };
            updateMenu(activeResourceTracker.default.resource());
            activeResourceTracker.default.resource.subscribe(updateMenu);
            this.resourceSwitcher.initialize();
            this.searchBox.initialize();
        };
        shell.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            $("#body").removeClass('loading-active');
            this.initializeShellComponents();
            registration.showRegistrationDialogIfNeeded(license.licenseStatus());
        };
        shell.prototype.preLoadRecentErrorsView = function () {
            // preload this view as in case of failure server can't serve it.
            viewLocator.locateView("views/common/recentErrors");
        };
        /*
        static fetchStudioConfig() {
            var hotSpareTask = new getHotSpareInformation().execute();
            var configTask = new getDocumentWithMetadataCommand(shell.studioConfigDocumentId, appUrl.getSystemDatabase(), true).execute();
    
            $.when(hotSpareTask, configTask).done((hotSpareResult, doc: documentClass) => {
                var hotSpare = <HotSpareDto>hotSpareResult[0];
    
                if (license.licenseStatus().Attributes.hotSpare === "true") {
                    // override environment colors with hot spare
                    this.activateHotSpareEnvironment(hotSpare);
                } else {
                    var envColor = doc && doc["EnvironmentColor"];
                    if (envColor != null) {
                        var color = new environmentColor(envColor.Name, envColor.BackgroundColor);
                        shell.selectedEnvironmentColorStatic(color);
                        shell.originalEnvironmentColor(color);
                    }
                }
            });
        }*/
        shell.prototype.setupApiKey = function () {
            var _this = this;
            // try to find api key as studio hash parameter
            var hash = window.location.hash;
            if (hash === "#has-api-key") {
                return this.showApiKeyDialog();
            }
            else if (hash.match(/#api-key/g)) {
                var match = /#api-key=(.*)/.exec(hash);
                if (match && match.length === 2) {
                    oauthContext.apiKey(match[1]);
                    apiKeyLocalStorage.setValue(match[1]);
                }
                var splittedHash = hash.split("&#api-key");
                var url = (splittedHash.length === 1) ? "#resources" : splittedHash[0];
                window.location.href = url;
            }
            else {
                var apiKeyFromStorage = apiKeyLocalStorage.get();
                if (apiKeyFromStorage) {
                    oauthContext.apiKey(apiKeyFromStorage);
                }
            }
            oauthContext.authHeader.subscribe(function (h) {
                if (_this.renewOAuthTokenTimeoutId) {
                    clearTimeout(_this.renewOAuthTokenTimeoutId);
                    _this.renewOAuthTokenTimeoutId = null;
                }
                if (h) {
                    _this.renewOAuthTokenTimeoutId = setTimeout(function () { return _this.renewOAuthToken(); }, 25 * 60 * 1000);
                }
            });
            return $.Deferred().resolve();
        };
        shell.prototype.renewOAuthToken = function () {
            /* TODO: oauthContext.authHeader(null);
             new getDatabaseStatsCommand(null).execute();*/
        };
        shell.prototype.getIndexingDisbaledValue = function (indexingDisabledString) {
            if (indexingDisabledString === undefined || indexingDisabledString == null)
                return false;
            if (indexingDisabledString.toLowerCase() === "true")
                return true;
            return false;
        };
        shell.prototype.launchDocEditor = function (docId, docsList) {
            var editDocUrl = appUrl.forEditDoc(docId, this.activeDatabase());
            this.navigate(editDocUrl);
        };
        shell.prototype.loadServerConfig = function () {
            var deferred = $.Deferred();
            new getServerConfigsCommand()
                .execute()
                .done(function (serverConfigs) {
                accessHelper.isGlobalAdmin(serverConfigs.IsGlobalAdmin);
                accessHelper.canReadWriteSettings(serverConfigs.CanReadWriteSettings);
                accessHelper.canReadSettings(serverConfigs.CanReadSettings);
                accessHelper.canExposeConfigOverTheWire(serverConfigs.CanExposeConfigOverTheWire);
            })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        shell.prototype.connectToRavenServer = function () {
            var serverConfigsLoadTask = this.loadServerConfig();
            var resourcesTask = this.resourcesManager.init();
            return $.when(serverConfigsLoadTask, resourcesTask);
        };
        shell.activateHotSpareEnvironment = function (hotSpare) {
            var color = new environmentColor(hotSpare.ActivationMode === "Activated" ? "Active Hot Spare" : "Hot Spare", "#FF8585");
            license.hotSpare(hotSpare);
        };
        shell.prototype.handleRavenConnectionFailure = function (result) {
            var _this = this;
            if (result.status === 401) {
                // Unauthorized might be caused by invalid credentials. 
                // Remove them from both local storage and oauth context.
                apiKeyLocalStorage.clean();
                oauthContext.clean();
            }
            sys.log("Unable to connect to Raven.", result);
            var tryAgain = "Try again";
            this.confirmationMessage(':-(', "Couldn't connect to Raven. Details in the browser console.", [tryAgain])
                .done(function () {
                _this.connectToRavenServer();
            });
        };
        shell.prototype.getDocCssClass = function (doc) {
            return collection.getCollectionCssClass(doc["@metadata"]["Raven-Entity-Name"], this.activeDatabase());
        };
        shell.prototype.fetchServerBuildVersion = function () {
            new getServerBuildVersionCommand()
                .execute()
                .done(function (serverBuildResult) {
                shell.serverBuildVersion(serverBuildResult);
                var currentBuildVersion = serverBuildResult.BuildVersion;
                if (currentBuildVersion !== DEV_BUILD_NUMBER) {
                    shell.serverMainVersion(Math.floor(currentBuildVersion / 10000));
                }
            });
        };
        shell.prototype.fetchClientBuildVersion = function () {
            var _this = this;
            new getClientBuildVersionCommand()
                .execute()
                .done(function (result) {
                _this.clientBuildVersion(result);
            });
        };
        shell.prototype.fetchClusterTopology = function () {
            new getClusterTopologyCommand(null)
                .execute()
                .done(function (topology) {
                shell.clusterMode(topology && topology.allNodes().length > 0);
            });
        };
        shell.prototype.fetchSupportCoverage = function () {
            new getSupportCoverageCommand()
                .execute()
                .done(function (result) {
                license.supportCoverage(result);
            });
        };
        shell.prototype.showApiKeyDialog = function () {
            var dialog = new enterApiKey();
            return app.showBootstrapDialog(dialog).then(function () { return window.location.href = "#resources"; });
        };
        shell.prototype.uploadStatusChanged = function (item) {
            var queue = uploadQueueHelper.parseUploadQueue(window.localStorage[uploadQueueHelper.localStorageUploadQueueKey + item.filesystem.name], item.filesystem);
            uploadQueueHelper.updateQueueStatus(item.id(), item.status(), queue);
            uploadQueueHelper.updateLocalStorage(queue, item.filesystem);
        };
        shell.prototype.showLicenseStatusDialog = function () {
            var dialog = new licensingStatus(license.licenseStatus(), license.supportCoverage(), license.hotSpare());
            app.showBootstrapDialog(dialog);
        };
        shell.prototype.logOut = function () {
            // this call dispatches storage event to all tabs in browser and calls onLogOut inside each.
            apiKeyLocalStorage.clean();
            apiKeyLocalStorage.notifyAboutLogOut();
        };
        shell.prototype.onLogOut = function () {
            window.location.hash = this.appUrls.hasApiKey();
            window.location.reload();
        };
        shell.prototype.navigateToClusterSettings = function () {
            this.navigate(this.appUrls.adminSettingsCluster());
        };
        shell.prototype.initAnalytics = function (config, buildVersionResult) {
            var _this = this;
            if (eventsCollector.gaDefined()) {
                if (config == null || !("SendUsageStats" in config)) {
                    // ask user about GA
                    this.displayUsageStatsInfo(true);
                    this.trackingTask.done(function (accepted) {
                        _this.displayUsageStatsInfo(false);
                        if (accepted) {
                            _this.configureAnalytics(true, buildVersionResult);
                        }
                    });
                }
                else {
                    this.configureAnalytics(config.SendUsageStats, buildVersionResult);
                }
            }
            else {
                // user has uBlock etc?
                this.configureAnalytics(false, buildVersionResult);
            }
        };
        shell.prototype.collectUsageData = function () {
            this.trackingTask.resolve(true);
        };
        shell.prototype.doNotCollectUsageData = function () {
            this.trackingTask.resolve(false);
        };
        shell.prototype.configureAnalytics = function (track, _a) {
            var buildVersionResult = _a[0];
            var currentBuildVersion = buildVersionResult.BuildVersion;
            var shouldTrack = track && currentBuildVersion !== DEV_BUILD_NUMBER;
            if (currentBuildVersion !== DEV_BUILD_NUMBER) {
                shell.serverMainVersion(Math.floor(currentBuildVersion / 10000));
            }
            var env = license.licenseStatus() && license.licenseStatus().LicenseType === "Commercial" ? "prod" : "dev";
            var version = buildVersionResult.FullVersion;
            eventsCollector.default.initialize(shell.serverMainVersion() + "." + shell.serverMinorVersion(), currentBuildVersion, env, version, shouldTrack);
        };
        return shell;
    }(viewModelBase));
    shell.studioConfigDocumentId = "Raven/StudioConfig";
    shell.clusterMode = ko.observable(false); //TODO: extract from shell
    shell.serverBuildVersion = ko.observable();
    shell.serverMainVersion = ko.observable(4);
    shell.serverMinorVersion = ko.observable(0);
    return shell;
});

//# sourceMappingURL=shell.js.map
