var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "viewmodels/dialogViewModelBase", "durandal/app", "plugins/dialog", "commands/licensing/licenseRegistrationCommand", "commands/licensing/licenseActivateCommand", "moment", "models/auth/license"], function (require, exports, dialogViewModelBase, app, dialog, licenseRegistrationCommand, licenseActivateCommand, moment, license) {
    "use strict";
    var registrationModel = (function () {
        function registrationModel() {
            this.name = ko.observable();
            this.email = ko.observable();
            this.company = ko.observable();
            this.setupValidation();
        }
        registrationModel.prototype.setupValidation = function () {
            this.name.extend({
                required: true
            });
            this.email.extend({
                required: true,
                email: true
            });
        };
        registrationModel.prototype.toDto = function () {
            return {
                Name: this.name(),
                Email: this.email(),
                Company: this.company()
            };
        };
        return registrationModel;
    }());
    var licenseKeyModel = (function () {
        function licenseKeyModel() {
            this.key = ko.observable();
            this.setupValidation();
        }
        licenseKeyModel.prototype.setupValidation = function () {
            var licenseValidator = function (license) {
                try {
                    var parsedLicense = JSON.parse(license);
                    var hasId = "Id" in parsedLicense;
                    var hasName = "Name" in parsedLicense;
                    var hasKeys = "Keys" in parsedLicense;
                    return hasId && hasName && hasKeys;
                }
                catch (e) {
                    return false;
                }
            };
            this.key.extend({
                required: true,
                validation: [{
                        validator: licenseValidator,
                        message: "Invalid license format"
                    }]
            });
        };
        return licenseKeyModel;
    }());
    var registrationDismissStorage = (function () {
        function registrationDismissStorage() {
        }
        registrationDismissStorage.getDismissedUntil = function () {
            var storedValue = localStorage.getObject(registrationDismissStorage.storageKey);
            if (storedValue) {
                return new Date(storedValue);
            }
            return null;
        };
        registrationDismissStorage.dismissFor = function (days) {
            localStorage.setObject(registrationDismissStorage.storageKey, moment().add(5, "days").toDate().getTime());
        };
        registrationDismissStorage.clearDismissStatus = function () {
            localStorage.removeItem(registrationDismissStorage.storageKey);
        };
        return registrationDismissStorage;
    }());
    registrationDismissStorage.storageKey = "registrationDismiss";
    var registration = (function (_super) {
        __extends(registration, _super);
        function registration(license, canBeDismissed) {
            var _this = _super.call(this) || this;
            _this.isBusy = ko.observable(false);
            _this.licenseKeySectionActive = ko.observable(false);
            _this.justRegistered = ko.observable(false);
            _this.dismissVisible = ko.observable(true);
            _this.registrationModel = ko.validatedObservable(new registrationModel());
            _this.licenseKeyModel = ko.validatedObservable(new licenseKeyModel());
            _this.hasInvalidLicense = ko.observable(false);
            _this.license = license;
            _this.bindToCurrentInstance("dismiss");
            _this.dismissVisible(canBeDismissed);
            return _this;
        }
        registration.showRegistrationDialogIfNeeded = function (license) {
            if (license.LicenseType === "Invalid") {
                registration.showRegistrationDialog(license, false);
                return;
            }
            if (license.LicenseType === "None") {
                var firstStart = moment(license.FirstServerStartDate);
                var weekAfterFirstStart = firstStart.add("1", "week");
                var treeWeeksAfterFirstStart = firstStart.add("3", "weeks");
                var now = moment();
                var shouldShow = false;
                var canDismiss = false;
                if (now.isBefore(weekAfterFirstStart)) {
                    shouldShow = false;
                }
                else {
                    if (now.isAfter(treeWeeksAfterFirstStart)) {
                        shouldShow = true;
                        canDismiss = false;
                    }
                    else {
                        // show if not dismissed
                        var dismissedUntil = registrationDismissStorage.getDismissedUntil();
                        canDismiss = true;
                        shouldShow = !dismissedUntil || dismissedUntil.getTime() < new Date().getTime();
                    }
                }
                if (shouldShow) {
                    registration.showRegistrationDialog(license, canDismiss);
                }
            }
        };
        registration.showRegistrationDialog = function (license, canBeDismissed) {
            var vm = new registration(license, canBeDismissed);
            app.showBootstrapDialog(vm);
        };
        registration.prototype.dismiss = function (days) {
            registrationDismissStorage.dismissFor(days);
            app.closeDialog(this);
        };
        registration.prototype.goToEnterLicense = function () {
            this.licenseKeySectionActive(true);
        };
        registration.prototype.goToRegistration = function () {
            this.licenseKeySectionActive(false);
        };
        registration.prototype.submit = function () {
            if (this.licenseKeySectionActive()) {
                this.submitLicenseKey();
            }
            else {
                this.submitRegistration();
            }
        };
        registration.prototype.submitRegistration = function () {
            var _this = this;
            if (!this.isValid(this.registrationModel)) {
                return;
            }
            this.isBusy(true);
            new licenseRegistrationCommand(this.registrationModel().toDto())
                .execute()
                .done(function () {
                _this.justRegistered(true);
                _this.licenseKeySectionActive(true);
            })
                .always(function () { return _this.isBusy(false); });
        };
        registration.prototype.submitLicenseKey = function () {
            var _this = this;
            if (!this.isValid(this.licenseKeyModel)) {
                return;
            }
            //TODO: parse pasted key into json and validate
            this.isBusy(true);
            var parsedLicense = JSON.parse(this.licenseKeyModel().key());
            new licenseActivateCommand(parsedLicense)
                .execute()
                .done(function () {
                license.fetchLicenseStatus();
                dialog.close(_this);
            })
                .always(function () { return _this.isBusy(false); });
        };
        return registration;
    }(dialogViewModelBase));
    return registration;
});

//# sourceMappingURL=registration.js.map
