var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/pagedList", "common/appUrl", "commands/timeSeries/putTypeCommand", "viewmodels/viewModelBase", "viewmodels/timeSeries/editTypeDialog", "models/timeSeries/timeSeriesType", "common/pagedResultSet", "commands/timeSeries/getTypesCommand", "models/timeSeries/typeChange"], function (require, exports, app, router, pagedList, appUrl, putTypeCommand, viewModelBase, editTypeDialog, timeSeriesType, pagedResultSet, getTypesCommand, typeChange) {
    "use strict";
    var types = (function (_super) {
        __extends(types, _super);
        function types() {
            var _this = _super.call(this) || this;
            _this.viewType = 2 /* TimeSeries */;
            _this.typesList = ko.observable();
            _this.selectedTypesIndices = ko.observableArray();
            _this.isAnyTypesAutoSelected = ko.observable(false);
            _this.isAllTypesAutoSelected = ko.observable(false);
            _this.showLoadingIndicator = ko.observable(false);
            _this.showLoadingIndicatorThrottled = _this.showLoadingIndicator.throttle(250);
            _this.isInitialized = types.isInitialized;
            _this.hasTypes = ko.computed(function () {
                var typesList = _this.typesList();
                return typesList && typesList.itemCount() > 0;
            });
            _this.hasAnyTypesSelected = ko.computed(function () { return _this.selectedTypesIndices().length > 0; });
            _this.canChangeType = ko.computed(function () {
                if (_this.selectedTypesIndices().length !== 1)
                    return false;
                var selectedItem = _this.getTypesGrid().getSelectedItems()[0];
                return selectedItem.keysCount() === 0;
            });
            return _this;
        }
        types.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.typesList(this.createTypesPagedList());
        };
        types.prototype.createTypesPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchTypes(skip, take); };
            var list = new pagedList(fetcher);
            return list;
        };
        types.prototype.fetchTypes = function (skip, take) {
            var deffered = $.Deferred();
            new getTypesCommand(this.activeTimeSeries()).execute()
                .done(function (types) {
                deffered.resolve(new pagedResultSet(types, 1024));
            });
            return deffered;
        };
        types.prototype.attached = function () {
            _super.prototype.attached.call(this);
            /*super.createKeyboardShortcut("F2", () => this.editSelectedTimeSeries(), timeSeries.gridSelector);*/
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            /*var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", () => this.getDocumentsGrid().deleteSelectedItems(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", () => this.copySelectedDocs(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", () => this.copySelectedDocIds(), docsPageSelector);*/
        };
        types.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            types.isInitialized(false);
        };
        types.prototype.createNotifications = function () {
            return [];
        };
        types.prototype.createPostboxSubscriptions = function () {
            return [];
        };
        types.prototype.newType = function () {
            var _this = this;
            var changeVm = new editTypeDialog(new typeChange(new timeSeriesType("", [""], 0, this.activeTimeSeries()), true), true);
            changeVm.updateTask.done(function (change) {
                new putTypeCommand(change.type(), change.fields(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showBootstrapDialog(changeVm);
        };
        types.prototype.changeType = function () {
            var _this = this;
            var grid = this.getTypesGrid();
            if (!grid)
                return;
            var selectedType = grid.getSelectedItems(1)[0];
            var change = new typeChange(selectedType);
            var typeChangeVM = new editTypeDialog(change, false);
            typeChangeVM.updateTask.done(function (change, isNew) {
                new putTypeCommand(change.type(), change.fields(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showBootstrapDialog(typeChangeVM);
        };
        types.prototype.refresh = function () {
            this.getTypesGrid().refreshCollectionData();
            this.typesList().invalidateCache();
        };
        types.prototype.deleteSelectedTypes = function () {
            var grid = this.getTypesGrid();
            if (grid) {
                grid.deleteSelectedItems();
            }
        };
        types.prototype.selectType = function (type, event) {
            if (!event || event.which !== 3) {
                type.activate();
                var timeSeriesTypeUrl = appUrl.forTimeSeriesType(type.name, this.activeTimeSeries());
                router.navigate(timeSeriesTypeUrl, false);
            }
        };
        types.prototype.getTypesGrid = function () {
            var gridContents = $(types.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        return types;
    }(viewModelBase));
    types.gridSelector = "#typesGrid";
    types.isInitialized = ko.observable(false);
    return types;
});

//# sourceMappingURL=types.js.map
