var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/pagedList", "common/appUrl", "commands/timeSeries/putPointCommand", "viewmodels/viewModelBase", "viewmodels/timeSeries/editPointDialog", "viewmodels/timeSeries/deleteKey", "models/timeSeries/pointChange", "models/timeSeries/timeSeriesPoint", "common/pagedResultSet", "commands/timeSeries/getPointsCommand"], function (require, exports, app, router, pagedList, appUrl, putPointCommand, viewModelBase, editPointDialog, deleteKey, pointChange, timeSeriesPoint, pagedResultSet, getPointsCommand) {
    "use strict";
    var timeSeriesPoints = (function (_super) {
        __extends(timeSeriesPoints, _super);
        function timeSeriesPoints() {
            var _this = _super.call(this) || this;
            _this.viewType = 2 /* TimeSeries */;
            _this.type = ko.observable();
            _this.fields = ko.observableArray();
            _this.key = ko.observable();
            _this.pointsCount = ko.observable();
            _this.minPoint = ko.observable();
            _this.maxPoint = ko.observable();
            _this.isFiltered = ko.observable(false);
            _this.startPointFilter = ko.observable();
            _this.endPointFilter = ko.observable();
            _this.isAggregated = ko.observable(false);
            _this.duration = ko.observable();
            _this.durationType = ko.observable();
            _this.pointsList = ko.observable();
            _this.selectedPointsIndices = ko.observableArray();
            _this.isAnyPointsAutoSelected = ko.observable(false);
            _this.isAllPointsAutoSelected = ko.observable(false);
            _this.showLoadingIndicator = ko.observable(false);
            _this.showLoadingIndicatorThrottled = _this.showLoadingIndicator.throttle(250);
            _this.isInitialized = timeSeriesPoints.isInitialized;
            _this.hasPoints = ko.computed(function () {
                return _this.pointsCount() > 0;
            });
            _this.hasAnyPointsSelected = ko.computed(function () { return _this.selectedPointsIndices().length > 0; });
            _this.hasAllPointsSelected = ko.computed(function () {
                var numOfSelectedPoints = _this.selectedPointsIndices().length;
                if (!!_this.key() && numOfSelectedPoints !== 0) {
                    return numOfSelectedPoints === _this.pointsCount();
                }
                return false;
            });
            _this.selectedPointsText = ko.computed(function () {
                if (!!_this.selectedPointsIndices()) {
                    var documentsText = "points";
                    return documentsText;
                }
                return "";
            });
            _this.pointsSelection = ko.computed(function () {
                if (_this.hasAllPointsSelected())
                    return 2 /* Checked */;
                if (_this.hasAnyPointsSelected())
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
            _this.isFiltered.subscribe(function () {
                _this.refresh();
            });
            _this.startPointFilter.subscribe(function () {
                _this.refresh();
            });
            _this.endPointFilter.subscribe(function () {
                _this.refresh();
            });
            _this.isAggregated.subscribe(function () {
                _this.refresh();
            });
            _this.duration.subscribe(function () {
                _this.refresh();
            });
            _this.durationType.subscribe(function () {
                _this.refresh();
            });
            return _this;
        }
        timeSeriesPoints.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            //TODO: update this in documentation
            //this.updateHelpLink('G8CDCP');
            // We can optionally pass in a key name to view's URL, e.g. #timeSeries/timeSeries?key=Foo&timeSeriestorage=test
            if (args && args.type && args.key) {
                this.fetchKey(args.type, args.key).done(function (result) {
                    _this.type(result.Type.Type);
                    _this.fields(result.Type.Fields);
                    _this.key(result.Key);
                    _this.pointsCount(result.PointsCount);
                    _this.minPoint(result.MinPoint);
                    _this.startPointFilter(result.MinPoint);
                    _this.maxPoint(result.MaxPoint);
                    _this.endPointFilter(result.MaxPoint);
                    if (!_this.pointsList()) {
                        _this.pointsList(_this.createPointsPagedList());
                    }
                    timeSeriesPoints.isInitialized(true);
                });
            }
        };
        timeSeriesPoints.prototype.createPointsPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchPoints(skip, take, _this.isFiltered() ? _this.startPointFilter() : null, _this.isFiltered() ? _this.endPointFilter() : null); };
            var list = new pagedList(fetcher);
            list.collectionName = this.key();
            return list;
        };
        timeSeriesPoints.prototype.fetchPoints = function (skip, take, start, end) {
            var _this = this;
            var doneTask = $.Deferred();
            new getPointsCommand(this.activeTimeSeries(), skip, take, this.type(), this.fields(), this.key(), start, end).execute()
                .done(function (points) { return doneTask.resolve(new pagedResultSet(points, _this.isFiltered() ? _this.pointsList().itemCount() + points.length : _this.pointsCount())); })
                .fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        timeSeriesPoints.prototype.attached = function () {
            _super.prototype.attached.call(this);
            /*super.createKeyboardShortcut("F2", () => this.editSelectedTimeSeries(), timeSeries.gridSelector);*/
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            /*var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", () => this.getDocumentsGrid().deleteSelectedItems(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", () => this.copySelectedDocs(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", () => this.copySelectedDocIds(), docsPageSelector);*/
        };
        timeSeriesPoints.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            timeSeriesPoints.isInitialized(false);
        };
        timeSeriesPoints.prototype.createNotifications = function () {
            return [];
        };
        timeSeriesPoints.prototype.createPostboxSubscriptions = function () {
            return [];
        };
        timeSeriesPoints.prototype.fetchKey = function (type, key) {
            var deferred = $.Deferred();
            //TODO: new getKeyCommand(this.activeTimeSeries(), type, key).execute().done((result: timeSeriesKeySummaryDto) => deferred.resolve(result));
            return deferred;
        };
        timeSeriesPoints.prototype.newPoint = function () {
            var _this = this;
            var changeVm = new editPointDialog(new pointChange(new timeSeriesPoint(this.type(), this.fields(), this.key(), moment().format(), this.fields().map(function (x) { return 0; })), true), true);
            changeVm.updateTask.done(function (change) {
                new putPointCommand(change.type(), change.key(), change.at(), change.values(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showBootstrapDialog(changeVm);
        };
        timeSeriesPoints.prototype.refresh = function () {
            var grid = this.getPointsGrid();
            if (grid) {
                grid.refreshCollectionData();
            }
            var pointsList = this.pointsList();
            if (pointsList) {
                pointsList.invalidateCache();
            }
            this.selectNone();
        };
        timeSeriesPoints.prototype.changePoint = function () {
            var _this = this;
            var grid = this.getPointsGrid();
            if (!grid)
                return;
            var selectedPoint = grid.getSelectedItems(1)[0];
            var change = new pointChange(selectedPoint);
            var pointChangeVM = new editPointDialog(change, false);
            pointChangeVM.updateTask.done(function (change, isNew) {
                new putPointCommand(change.type(), change.key(), change.at(), change.values(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showBootstrapDialog(pointChangeVM);
        };
        timeSeriesPoints.prototype.toggleSelectAll = function () {
            var pointsGrid = this.getPointsGrid();
            if (!!pointsGrid) {
                if (this.hasAnyPointsSelected()) {
                    pointsGrid.selectNone();
                }
                else {
                    pointsGrid.selectSome();
                    this.isAnyPointsAutoSelected(this.hasAllPointsSelected() === false);
                }
            }
        };
        timeSeriesPoints.prototype.selectAll = function () {
            var pointsGrid = this.getPointsGrid();
            if (!!pointsGrid && !!this.key()) {
                pointsGrid.selectAll(this.pointsCount());
            }
        };
        timeSeriesPoints.prototype.selectNone = function () {
            var pointsGrid = this.getPointsGrid();
            if (!!pointsGrid) {
                pointsGrid.selectNone();
            }
        };
        timeSeriesPoints.prototype.deleteSelectedPoints = function () {
            if (this.hasAllPointsSelected()) {
                this.deleteKey();
            }
            else {
                var grid = this.getPointsGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        timeSeriesPoints.prototype.deleteKey = function () {
            var _this = this;
            var deleteKeyVm = new deleteKey(this.type(), this.key(), this.activeTimeSeries());
            deleteKeyVm.deletionTask.done(function () {
                _this.navigate(appUrl.forTimeSeriesType(_this.type(), _this.activeTimeSeries()));
            });
            app.showBootstrapDialog(deleteKeyVm);
        };
        timeSeriesPoints.prototype.selectType = function (type, event) {
            if (!event || event.which !== 3) {
                type.activate();
                var timeSeriesTypeUrl = appUrl.forTimeSeriesType(type.name, this.activeTimeSeries());
                router.navigate(timeSeriesTypeUrl, false);
            }
        };
        timeSeriesPoints.prototype.getPointsGrid = function () {
            var gridContents = $(timeSeriesPoints.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        return timeSeriesPoints;
    }(viewModelBase));
    timeSeriesPoints.gridSelector = "#pointsGrid";
    timeSeriesPoints.isInitialized = ko.observable(false);
    return timeSeriesPoints;
});

//# sourceMappingURL=timeSeriesPoints.js.map
