var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "viewmodels/timeSeries/editPointDialog", "models/timeSeries/pointChange", "models/timeSeries/timeSeriesPoint", "commands/timeSeries/getTypesCommand", "commands/timeSeries/putPointCommand", "viewmodels/viewModelBase", "common/constants/events"], function (require, exports, app, router, appUrl, editPointDialog, pointChange, timeSeriesPoint, getTypesCommand, putPointCommand, viewModelBase, EVENTS) {
    "use strict";
    var timeSeriesTypes = (function (_super) {
        __extends(timeSeriesTypes, _super);
        function timeSeriesTypes() {
            var _this = _super.call(this) || this;
            _this.viewType = 2 /* TimeSeries */;
            _this.types = ko.observableArray();
            _this.selectedType = ko.observable().subscribeTo("ActivateType").distinctUntilChanged();
            _this.currentType = ko.observable();
            _this.currentTypePagedItems = ko.observable();
            _this.selectedKeysIndices = ko.observableArray();
            _this.isAnyKeysAutoSelected = ko.observable(false);
            _this.isAllKeysAutoSelected = ko.observable(false);
            _this.showLoadingIndicator = ko.observable(false);
            _this.showLoadingIndicatorThrottled = _this.showLoadingIndicator.throttle(250);
            _this.isInitialized = timeSeriesTypes.isInitialized;
            _this.selectedType.subscribe(function (t) { return _this.selectedTypeChanged(t); });
            _this.hasKeys = ko.computed(function () {
                var selectedType = _this.selectedType();
                if (!!selectedType) {
                    return _this.selectedType().keysCount() > 0;
                }
                return false;
            });
            _this.hasAnyKeysSelected = ko.computed(function () { return _this.selectedKeysIndices().length > 0; });
            _this.hasAllKeysSelected = ko.computed(function () {
                var numOfSelectedKeys = _this.selectedKeysIndices().length;
                if (!!_this.selectedType() && numOfSelectedKeys !== 0) {
                    return numOfSelectedKeys === _this.selectedType().keysCount();
                }
                return false;
            });
            _this.selectedKeysText = ko.computed(function () {
                if (!!_this.selectedKeysIndices()) {
                    var documentsText = "time series";
                    return documentsText;
                }
                return "";
            });
            _this.keysSelection = ko.computed(function () {
                if (_this.hasAllKeysSelected())
                    return 2 /* Checked */;
                if (_this.hasAnyKeysSelected())
                    return 1 /* SomeChecked */;
                return 0 /* UnChecked */;
            });
            return _this;
        }
        timeSeriesTypes.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            //TODO: update this in documentation
            //this.updateHelpLink('G8CDCP');
            // We can optionally pass in a key name to view's URL, e.g. #timeSeries/timeSeries?key=Foo&timeSeriestorage=test
            if (args && args.type && args.key) {
                this.typeToSelect = args.type;
            }
            var ts = this.activeTimeSeries();
            this.fetchTypes(ts).done(function (results) {
                _this.typesLoaded(results, ts);
                timeSeriesTypes.isInitialized(true);
            });
        };
        timeSeriesTypes.prototype.attached = function () {
            _super.prototype.attached.call(this);
            /*super.createKeyboardShortcut("F2", () => this.editSelectedTimeSeries(), timeSeries.gridSelector);*/
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            /*var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", () => this.getDocumentsGrid().deleteSelectedItems(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", () => this.copySelectedDocs(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", () => this.copySelectedDocIds(), docsPageSelector);*/
        };
        timeSeriesTypes.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            timeSeriesTypes.isInitialized(false);
        };
        timeSeriesTypes.prototype.createNotifications = function () {
            return [];
        };
        timeSeriesTypes.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                //    ko.postbox.subscribe("ChangePointValue", () => this.changePoint()),
                ko.postbox.subscribe(EVENTS.ChangesApi.Reconnected, function (ts) { return _this.reloadTimeSeriesData(ts); }),
                ko.postbox.subscribe("SortTypes", function () { return _this.sortTypes(); })
            ];
        };
        timeSeriesTypes.prototype.fetchTypes = function (ts) {
            var deferred = $.Deferred();
            new getTypesCommand(ts).execute().done(function (results) { return deferred.resolve(results); });
            return deferred;
        };
        timeSeriesTypes.prototype.typesLoaded = function (types, ts) {
            var _this = this;
            this.types(types);
            if (this.typeToSelect)
                this.types().find(function (g) { return g.name === _this.typeToSelect; }).activate();
            else {
                var allTypes = this.types();
                if (allTypes.length > 0) {
                    allTypes[0].activate();
                }
            }
        };
        timeSeriesTypes.prototype.newPoint = function () {
            var _this = this;
            var type = this.currentType();
            var changeVm = new editPointDialog(new pointChange(new timeSeriesPoint(type.name, type.fields, "", moment().format(), type.fields.map(function (x) { return 0; })), true), true);
            changeVm.updateTask.done(function (change) {
                new putPointCommand(change.type(), change.key(), change.at(), change.values(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showBootstrapDialog(changeVm);
        };
        timeSeriesTypes.prototype.refresh = function () {
            var selectedKeyName = this.selectedType().name;
            this.refreshGridAndKey(selectedKeyName);
        };
        timeSeriesTypes.prototype.refreshGridAndKey = function (changedKeyName) {
            var type = this.selectedType();
            if (type.name === changedKeyName) {
                this.getKeysGrid().refreshCollectionData();
            }
            type.getKeys().invalidateCache();
            this.selectNone();
        };
        timeSeriesTypes.prototype.selectedTypeChanged = function (selected) {
            if (!!selected) {
                var pagedList = selected.getKeys();
                this.currentTypePagedItems(pagedList);
                this.currentType(selected);
            }
        };
        timeSeriesTypes.prototype.toggleSelectAll = function () {
            var pointsGrid = this.getKeysGrid();
            if (!!pointsGrid) {
                if (this.hasAnyKeysSelected()) {
                    pointsGrid.selectNone();
                }
                else {
                    pointsGrid.selectSome();
                    this.isAnyKeysAutoSelected(this.hasAllKeysSelected() === false);
                }
            }
        };
        timeSeriesTypes.prototype.selectAll = function () {
            var typesGrid = this.getKeysGrid();
            var type = this.selectedType();
            if (!!typesGrid && !!type) {
                typesGrid.selectAll(type.keysCount());
            }
        };
        timeSeriesTypes.prototype.selectNone = function () {
            var pointsGrid = this.getKeysGrid();
            if (!!pointsGrid) {
                pointsGrid.selectNone();
            }
        };
        timeSeriesTypes.prototype.deleteSelectedKeys = function () {
            if (this.hasAllKeysSelected()) {
            }
            else {
                var grid = this.getKeysGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        timeSeriesTypes.prototype.updateTypes = function (receivedTypes) {
            var _this = this;
            var deletedTypes = [];
            this.types().forEach(function (t) {
                if (!receivedTypes.find(function (receivedGroup) { return t.name === receivedGroup.name; })) {
                    deletedTypes.push(t);
                }
            });
            this.types.removeAll(deletedTypes);
            receivedTypes.forEach(function (receivedGroup) {
                var foundType = _this.types().find(function (t) { return t.name === receivedGroup.name; });
                if (!foundType) {
                    _this.types.push(receivedGroup);
                }
                else {
                    foundType.keysCount(receivedGroup.keysCount());
                }
            });
        };
        timeSeriesTypes.prototype.refreshKeysData = function () {
            this.types().forEach(function (key) {
                var pagedList = key.getKeys();
                pagedList.invalidateCache();
            });
        };
        timeSeriesTypes.prototype.refreshKeys = function () {
            var _this = this;
            var deferred = $.Deferred();
            var ts = this.activeTimeSeries();
            this.fetchTypes(ts).done(function (results) {
                _this.updateTypes(results);
                _this.refreshKeysData();
                deferred.resolve();
            });
            return deferred;
        };
        timeSeriesTypes.prototype.reloadTimeSeriesData = function (ts) {
            var _this = this;
            if (ts.name === this.activeTimeSeries().name) {
                this.refreshKeys().done(function () { return _this.refreshKeysData(); });
            }
        };
        timeSeriesTypes.prototype.selectType = function (type, event) {
            if (!event || event.which !== 3) {
                type.activate();
                var timeSeriesTypeUrl = appUrl.forTimeSeriesType(type.name, this.activeTimeSeries());
                router.navigate(timeSeriesTypeUrl, false);
            }
        };
        timeSeriesTypes.prototype.getKeysGrid = function () {
            var gridContents = $(timeSeriesTypes.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        timeSeriesTypes.prototype.sortTypes = function () {
            this.types.sort(function (c1, c2) {
                return c1.name.toLowerCase() > c2.name.toLowerCase() ? 1 : -1;
            });
        };
        // Animation callbacks for the types list
        timeSeriesTypes.prototype.showTypeElement = function (element) {
            if (element.nodeType === 1 && timeSeriesTypes.isInitialized()) {
                $(element).hide().slideDown(500, function () {
                    ko.postbox.publish("SortTypes");
                    $(element).highlight();
                });
            }
        };
        timeSeriesTypes.prototype.hideTypeElement = function (element) {
            if (element.nodeType === 1) {
                $(element).slideUp(1000, function () { $(element).remove(); });
            }
        };
        return timeSeriesTypes;
    }(viewModelBase));
    timeSeriesTypes.gridSelector = "#keysGrid";
    timeSeriesTypes.isInitialized = ko.observable(false);
    return timeSeriesTypes;
});

//# sourceMappingURL=timeSeriesTypes.js.map
