define(["require", "exports"], function (require, exports) {
    "use strict";
    /**
     * Virtual grid column that renders hyperlinks.
     */
    var hyperlinkColumn = (function () {
        /**
         * Creates a new hyperlink column.
         * @param dataMemberName The name of the property containing the text to display in the hyperlink.
         * @param hrefMemberName The name of the property containing the link.
         * @param display The column header text.
         * @param width The width of the column, e.g. "20px" or "100%"
         */
        function hyperlinkColumn(dataMemberName, hrefMemberName, display, width) {
            this.dataMemberName = dataMemberName;
            this.hrefMemberName = hrefMemberName;
            this.display = display;
            this.width = width;
            this.cellClass = "hyperlink-cell";
        }
        hyperlinkColumn.prototype.renderCell = function (item, isSelected) {
            var cellValue = item[this.dataMemberName];
            var hyperlinkValue = item[this.hrefMemberName];
            return "<a href=\"" + (hyperlinkValue ? hyperlinkValue.toString() : "javascript:void(0)") + "\">" + (cellValue ? cellValue.toString() : "") + "</a>";
        };
        return hyperlinkColumn;
    }());
    return hyperlinkColumn;
});

//# sourceMappingURL=hyperlinkColumn.js.map
