define(["require", "exports", "plugins/router", "durandal/app", "common/appUrl", "models/database/documents/document", "models/counter/counterSummary", "viewmodels/common/deleteItems", "viewmodels/database/documents/copyDocuments", "widgets/virtualTable/row", "widgets/virtualTable/column", "models/database/documents/customColumnParams", "models/database/documents/customColumns", "models/database/documents/customFunctions", "models/timeSeries/timeSeriesKey", "common/shell/activeResourceTracker"], function (require, exports, router, app, appUrl, document, counterSummary, deleteItems, copyDocuments, row, column, customColumnParams, customColumns, customFunctions, timeSeriesKey, activeResourceTracker) {
    "use strict";
    //TODO: get rid of this class in flavor of new virtual pager
    var ctor = (function () {
        function ctor() {
            this.$window = $(window);
            this.recycleRows = ko.observableArray();
            this.rowHeight = 38;
            this.borderHeight = 2;
            this.viewportHeight = ko.observable(0);
            this.virtualRowCount = ko.observable(0);
            this.tableXScroll = ko.observable(0);
            this.columns = ko.observableArray();
            this.scrollThrottleTimeoutHandle = 0;
            this.firstVisibleRow = null;
            this.lastVisibleRow = null;
            this.itemsSourceSubscription = null;
            this.ensureColumnsAnimationFrameHandle = 0;
            this.headerVisible = ko.observable(false);
            this.shiftPressed = ko.observable(false);
        }
        ctor.prototype.activate = function (settings) {
            var _this = this;
            var defaults = {
                dynamicHeightTargetSelector: "footer",
                dynamicHeightBottomMargin: 0,
                selectedIndices: ko.observableArray(),
                showCheckboxes: true,
                showIds: true,
                useContextMenu: true,
                maxHeight: 'none',
                customColumnParams: {},
                isIndexMapReduce: ko.observable(true),
                isCopyAllowed: true,
                contextMenuOptions: ["CopyItems", "CopyIDs", "Delete", "ViewItem"],
                selectionEnabled: true,
                customColumns: ko.observable(customColumns.empty()),
                customFunctions: ko.observable(customFunctions.empty()),
                collections: ko.observableArray([]),
                rowsAreLoading: ko.observable(false),
                noResultsMessage: "No records found.",
                isAnyAutoSelected: ko.observable(false),
                isAllAutoSelected: ko.observable(false),
                viewType: 0 /* Documents */,
                isCounterAllGroupsGroup: ko.observable(false)
            };
            this.settings = $.extend(defaults, settings);
            this.bottomMargin = ko.computed(function () {
                // if header is visible we have to substruct it's height to avoid scroll
                var headerHeight = _this.headerVisible() ? 0 : 41;
                return headerHeight + (_this.settings.dynamicHeightBottomMargin || 0);
            });
            this.$window.resize(function () {
                _this.headerVisible($(".ko-grid-column-container", _this.grid).is(":visible"));
            });
            this.$window.on('keydown.virtualTable', function (e) { return _this.shiftPressed(e.shiftKey); });
            this.$window.on('keyup.virtualTable', function (e) { return _this.shiftPressed(e.shiftKey); });
            if (!!settings.isIndexMapReduce) {
                this.isIndexMapReduce = settings.isIndexMapReduce;
            }
            else {
                this.isIndexMapReduce = ko.observable(false);
            }
            this.getCollectionClassFromEntityNameMemoized = _.memoize(this.getCollectionClassFromEntityName);
            this.items = this.settings.itemsSource();
            this.focusableGridSelector = this.settings.gridSelector + " .ko-grid";
            this.virtualHeight = ko.computed(function () { return _this.rowHeight * _this.virtualRowCount(); });
            this.virtualWidth = ko.computed(function () { return _this.columns().map(function (x) { return x.width(); }).reduce(function (a, b) { return a + b; }, 0); });
            this.refreshIdAndCheckboxColumn();
            this.itemsSourceSubscription = this.settings.itemsSource.subscribe(function (list) {
                _this.recycleRows().forEach(function (r) {
                    r.resetCells();
                    _this.recycleRows.valueHasMutated();
                    r.isInUse(false);
                });
                _this.columns.valueHasMutated();
                _this.items = list;
                _this.settings.selectedIndices.removeAll();
                _this.columns.remove(function (c) { return (c.binding !== "Id" && c.binding !== "__IsChecked"); });
                if (_this.gridViewport) {
                    _this.gridViewport.scrollTop(0);
                }
                _this.onGridScrolled();
                _this.refreshIdAndCheckboxColumn();
            });
            this.noResults = ko.computed(function () {
                var numOfRowsInUse = _this.recycleRows().filter(function (r) { return r.isInUse(); }).length;
                return numOfRowsInUse === 0 && !_this.settings.rowsAreLoading();
            });
        };
        // Attached is called by Durandal when the view is attached to the DOM.
        // We use this to setup some UI-specific things like context menus, row creation, keyboard shortcuts, etc.
        ctor.prototype.attached = function () {
            var _this = this;
            this.grid = $(this.settings.gridSelector);
            if (this.grid.length !== 1) {
                // Don't throw an error here, because the user can cancel navigation, causing the element not to be found on the page. 
                //throw new Error("There should be 1 " + this.settings.gridSelector + " on the page, but found " + this.grid.length.toString());
                console.warn("There should be 1 " + this.settings.gridSelector + " on the page, but found " + this.grid.length.toString());
                return;
            }
            this.gridViewport = this.grid.find(".ko-grid-viewport-container");
            this.gridViewport.on('DynamicHeightSet', function () { return _this.onWindowHeightChanged(); });
            this.gridViewport.on('scroll', function () { return _this.tableXScroll(_this.gridViewport.scrollLeft()); });
            this.gridViewport.scroll(function () { return _this.onGridScrolled(); });
            this.setupKeyboardShortcuts();
            if (this.settings.useContextMenu) {
                this.setupContextMenu();
            }
        };
        ctor.prototype.compositionComplete = function () {
            this.registerColumnResizing();
        };
        ctor.prototype.detached = function () {
            $(this.settings.gridSelector).unbind('keydown.jwerty');
            this.gridViewport.off('DynamicHeightSet');
            this.gridViewport.off('scroll');
            this.$window.off('keydown.virtualTable');
            this.$window.off('keyup.virtualTable');
            if (this.itemsSourceSubscription) {
                this.itemsSourceSubscription.dispose();
            }
            this.unregisterColumnResizing();
        };
        ctor.prototype.calculateRecycleRowCount = function () {
            var requiredRowCount = Math.ceil(this.viewportHeight() / this.rowHeight);
            var rowCountWithPadding = requiredRowCount + 10;
            return rowCountWithPadding;
        };
        ctor.prototype.createRecycleRows = function (rowCount) {
            for (var i = this.recycleRows().length; i < rowCount; i++) {
                var newRow = new row(this.settings.showIds, this);
                newRow.createPlaceholderCells(this.columns().map(function (c) { return c.binding; }));
                newRow.rowIndex(i);
                var desiredTop = i * this.rowHeight;
                newRow.top(desiredTop);
                this.recycleRows.push(newRow);
            }
            for (var i = rowCount; i < this.recycleRows().length; i++) {
                var r = this.recycleRows()[i];
                r.isInUse(false);
            }
            app.trigger(this.settings.gridSelector + "RowsCreated", true);
        };
        ctor.prototype.onGridScrolled = function () {
            var _this = this;
            this.settings.rowsAreLoading(true);
            this.ensureRowsCoverViewport();
            this.scrollThrottleTimeoutHandle = this.requestAnimationFrame(function () { return _this.loadRowData(); }, this.scrollThrottleTimeoutHandle);
        };
        ctor.prototype.onWindowHeightChanged = function () {
            var _this = this;
            this.settings.rowsAreLoading(true);
            var newViewportHeight = this.gridViewport.height();
            this.viewportHeight(newViewportHeight);
            var desiredRowCount = this.calculateRecycleRowCount();
            this.createRecycleRows(desiredRowCount);
            this.ensureRowsCoverViewport();
            this.loadRowData();
            // Update row checked states.
            this.recycleRows().forEach(function (r) { return r.isChecked(_.includes(_this.settings.selectedIndices(), r.rowIndex())); });
        };
        ctor.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.setupKeyboardShortcut("DELETE", function () { return _this.deleteSelectedItems(); });
            this.setupKeyboardShortcut("Ctrl+C,D", function () { return _this.copySelectedDocs(); });
            this.setupKeyboardShortcut("Ctrl+C,I", function () { return _this.copySelectedDocIds(); });
        };
        ctor.prototype.setupKeyboardShortcut = function (keys, handler) {
            jwerty.key(keys, function (e) {
                e.preventDefault();
                handler();
            }, this, this.settings.gridSelector);
        };
        ctor.prototype.setupContextMenu = function () {
            var _this = this;
            var untypedGrid = this.grid;
            untypedGrid.contextmenu({
                target: '#gridContextMenu',
                before: function (e) {
                    var target = e.target;
                    var rowTag = (target.className.indexOf("ko-grid-row") > -1) ? $(target) : $(e.target).parents(".ko-grid-row");
                    var rightClickedElement = rowTag.length ? ko.dataFor(rowTag[0]) : null;
                    if (_this.settings.showCheckboxes && !_this.isIndexMapReduce()) {
                        // Select any right-clicked row.
                        if (rightClickedElement && rightClickedElement.isChecked != null && !rightClickedElement.isChecked()) {
                            _this.toggleRowChecked(rightClickedElement, false);
                        }
                    }
                    else {
                        if (rightClickedElement) {
                            _this.settings.selectedIndices([rightClickedElement.rowIndex()]);
                        }
                    }
                    return true;
                }
            });
        };
        ctor.prototype.refreshIdAndCheckboxColumn = function () {
            var containsId = this.columns().find(function (x) { return x.binding === "Id"; });
            if (!containsId && !this.isIndexMapReduce()) {
                var containsCheckbox = this.columns().find(function (x) { return x.binding === "__IsChecked"; });
                if (!containsCheckbox && this.settings.showCheckboxes) {
                    this.columns.push(new column("__IsChecked", ctor.selectColumnWidth));
                }
                if (this.settings.showIds !== false) {
                    this.columns.push(new column("Id", ctor.idColumnWidth));
                }
                this.columns.valueHasMutated();
            }
            else if (containsId && this.isIndexMapReduce()) {
                this.columns.remove(function (c) { return c.binding === "Id" || c.binding === "__IsChecked"; });
                this.columns.valueHasMutated();
            }
        };
        ctor.prototype.loadRowData = function () {
            var _this = this;
            if (this.items && this.firstVisibleRow) {
                this.settings.rowsAreLoading(true);
                // The scrolling has paused for a minute. See if we have all the data needed.
                var firstVisibleIndex = this.firstVisibleRow.rowIndex();
                var fetchTask = this.items.fetch(firstVisibleIndex, this.recycleRows().length);
                fetchTask.done(function (resultSet) {
                    var firstVisibleRowIndexHasChanged = firstVisibleIndex !== _this.firstVisibleRow.rowIndex();
                    if (!firstVisibleRowIndexHasChanged) {
                        _this.virtualRowCount(resultSet.totalResultCount);
                        resultSet.items.forEach(function (r, i) { return _this.fillRow(r, i + firstVisibleIndex); });
                        // when we have few rows and we delete once of them there might be old rows that must be removed
                        _this.recycleRows().filter(function (r, i) { return i >= resultSet.items.length + firstVisibleIndex && r.isInUse(); }).map(function (r) { return r.isInUse(false); });
                        // Because processing all columns can take time for many columns, we
                        // asynchronously load the column information in the next animation frame.
                        _this.ensureColumnsAnimationFrameHandle = _this.requestAnimationFrame(function () { return _this.ensureColumnsForRows(resultSet.items); }, _this.ensureColumnsAnimationFrameHandle);
                    }
                });
            }
        };
        ctor.prototype.requestAnimationFrame = function (action, existingHandleToCancel) {
            var result;
            if (window.requestAnimationFrame) {
                if (existingHandleToCancel) {
                    window.cancelAnimationFrame(existingHandleToCancel);
                }
                result = window.requestAnimationFrame(action);
            }
            else if (window.msRequestAnimationFrame) {
                if (window.msCancelRequestAnimationFrame) {
                    window.msCancelRequestAnimationFrame(existingHandleToCancel);
                }
                result = window.msRequestAnimationFrame(action);
            }
            else {
                if (existingHandleToCancel) {
                    window.clearTimeout(existingHandleToCancel);
                }
                result = setTimeout(action, 1);
            }
            this.settings.rowsAreLoading(false);
            return result;
        };
        ctor.prototype.fillRow = function (rowData, rowIndex) {
            var rowAtIndex = ko.utils.arrayFirst(this.recycleRows(), function (r) { return r.rowIndex() === rowIndex; });
            if (rowAtIndex) {
                rowAtIndex.fillCells(rowData);
                var entityName = this.getEntityName(rowData);
                rowAtIndex.collectionClass(this.getCollectionClassFromEntityNameMemoized(rowData, entityName));
                var editUrl = appUrl.forEditItem(!!rowData.getUrl() ? rowData.getUrl() : rowData["Id"], activeResourceTracker.default.resource(), rowIndex, entityName);
                rowAtIndex.editUrl(editUrl);
            }
        };
        ctor.prototype.editLastSelectedItem = function () {
            var selectedItem = this.getSelectedItems(1)[0];
            if (selectedItem) {
                var collectionName = this.items.collectionName;
                var itemIndex = this.settings.selectedIndices()[0];
                var editUrl = appUrl.forEditItem(selectedItem.getUrl(), activeResourceTracker.default.resource(), itemIndex, collectionName);
                router.navigate(editUrl);
            }
        };
        ctor.prototype.getCollectionClassFromEntityName = function (rowData, entityName) {
            /* TODO:
            if (rowData instanceof document) {
                return collection.getCollectionCssClass(entityName, appUrl.getDatabase());
            }
            if (rowData instanceof counterSummary) {
                return counterGroup.getGroupCssClass(entityName, appUrl.getCounterStorage());
            }
            if (rowData instanceof timeSeriesKey) {
                return timeSeriesType.getTypeCssClass(entityName, appUrl.getTimeSeries());
            }*/
            return "";
        };
        ctor.prototype.getEntityName = function (rowData) {
            var obj = rowData;
            if (obj && obj.getEntityName) {
                if (obj instanceof document) {
                    var documentObj = obj;
                    return documentObj.getEntityName();
                }
                if (obj instanceof counterSummary) {
                    var counterSummaryObj = obj;
                    return counterSummaryObj.getEntityName();
                }
                if (obj instanceof timeSeriesKey) {
                    var timeSeriesKeyObj = obj;
                    return timeSeriesKeyObj.getEntityName();
                }
            }
            return null;
        };
        /*isCounterView(): boolean {
            var item = this.items.getItem(0);
            return item instanceof counterSummary;
        }
    
        isTimeSeriesView(): boolean {
            var item = this.items.getItem(0);
            return item instanceof timeSeriesKey;
        }*/
        ctor.prototype.getColumnWidth = function (binding, defaultColumnWidth) {
            if (defaultColumnWidth === void 0) { defaultColumnWidth = 100; }
            var customColumns = this.settings.customColumns();
            var customConfig = customColumns.findConfigFor(binding);
            if (customConfig && customColumns.customMode()) {
                return customConfig.width();
            }
            if (binding === "Id" && defaultColumnWidth > ctor.idColumnWidth) {
                return ctor.idColumnWidth;
            }
            return defaultColumnWidth;
        };
        ctor.prototype.getColumnName = function (binding) {
            if (this.settings.customColumns().hasOverrides()) {
                var customConfig = this.settings.customColumns().findConfigFor(binding);
                if (customConfig) {
                    return customConfig.header();
                }
            }
            else {
                var columns = this.settings.customColumns().columns();
                for (var i = 0; i < columns.length; i++) {
                    if (columns[i].binding() === binding) {
                        return columns[i].header();
                    }
                }
            }
            return binding;
        };
        ctor.prototype.ensureColumnsForRows = function (rows) {
            // Hot path.
            // This is called when items finish loading and are ready for display.
            // Keep allocations to a minimum.
            var columnsNeeded = {};
            var hasOverrides = this.settings.customColumns().hasOverrides();
            if (hasOverrides) {
                var colParams = this.settings.customColumns().columns();
                for (var i = 0, x = colParams.length; i < x; i++) {
                    columnsNeeded[colParams[i].binding()] = null;
                }
            }
            else {
                for (var i = 0, x = rows.length; i < x; i++) {
                    var currentRow = rows[i];
                    var rowProperties = currentRow.getDocumentPropertyNames();
                    for (var j = 0, y = rowProperties.length; j < y; j++) {
                        columnsNeeded[rowProperties[j]] = null;
                    }
                }
            }
            var existingColumns = this.columns();
            var desiredColumns = existingColumns.concat([]);
            for (var i = 0; i < existingColumns.length; i++) {
                var colName = existingColumns[i].binding;
                delete columnsNeeded[colName];
            }
            var idColumn = this.columns().find(function (x) { return x.binding === "Id"; });
            var idColumnExists = idColumn ? 1 : 0;
            var unneededColumns = [];
            ko.utils.arrayForEach(existingColumns, function (col) {
                if (col.binding !== "Id" && col.binding !== "__IsChecked" && !hasOverrides && rows.every(function (row) { return !_.includes(row.getDocumentPropertyNames(), col.binding); })) {
                    unneededColumns.push(col.binding);
                }
            });
            desiredColumns = desiredColumns.filter(function (c) { return !_.includes(unneededColumns, c.binding); });
            this.settings.customColumns().columns.remove(function (c) { return _.includes(unneededColumns, c.binding()); });
            var columnsCurrentTotalWidth = 0;
            for (var i = 2; i < existingColumns.length; i++) {
                columnsCurrentTotalWidth += existingColumns[i].width();
            }
            var checkboxesWidth = this.settings.showCheckboxes ? ctor.selectColumnWidth : 0;
            var availiableWidth = this.grid.width() - checkboxesWidth - ctor.idColumnWidth * idColumnExists - columnsCurrentTotalWidth - ctor.optionalScrollSize;
            var freeWidth = availiableWidth;
            var fontSize = parseInt(this.grid.css("font-size"), 10);
            var columnCount = 0;
            for (var binding in columnsNeeded) {
                if (columnsNeeded.hasOwnProperty(binding)) {
                    var curColWidth = (binding.length + 2) * fontSize;
                    if (!hasOverrides && freeWidth < curColWidth) {
                        break;
                    }
                    freeWidth -= curColWidth;
                    columnCount++;
                }
            }
            var freeWidthPerColumn = hasOverrides ? 0 : Math.floor((freeWidth / (columnCount + 1)));
            var firstRow = this.recycleRows().length > 0 ? this.recycleRows()[0] : null;
            for (var binding in columnsNeeded) {
                var curColWidth = (binding.length + 2) * fontSize + freeWidthPerColumn;
                var columnWidth = this.getColumnWidth(binding, curColWidth);
                availiableWidth -= columnWidth;
                if (!hasOverrides && availiableWidth <= 0) {
                    break;
                }
                var columnName = this.getColumnName(binding);
                // Give priority to any Name column. Put it after the check column (0) and Id (1) columns.
                var newColumn = new column(binding, columnWidth, columnName);
                if ((binding === "Name") && (!this.settings.customColumns().customMode())) {
                    desiredColumns.splice(2, 0, newColumn);
                }
                else {
                    desiredColumns.push(newColumn);
                }
                var curColumnConfig = this.settings.customColumns().findConfigFor(binding);
                if (!curColumnConfig && !!firstRow) {
                    var curColumnTemplate = firstRow.getCellTemplate(binding);
                    var newCustomColumn = new customColumnParams({
                        Binding: binding,
                        Header: binding,
                        Template: curColumnTemplate,
                        DefaultWidth: availiableWidth > 0 ? Math.floor(columnWidth) : 0
                    });
                    if ((binding === "Name") && (!this.settings.customColumns().customMode())) {
                        this.settings.customColumns().columns.splice(0, 0, newCustomColumn);
                    }
                    else {
                        this.settings.customColumns().columns.push(newCustomColumn);
                    }
                }
            }
            // Update the columns only if we have to.
            var columnsHaveChanged = desiredColumns.length !== existingColumns.length || desiredColumns.some(function (newCol, index) { return newCol.binding !== existingColumns[index].binding; });
            if (columnsHaveChanged) {
                this.columns(desiredColumns);
            }
        };
        ctor.prototype.ensureRowsCoverViewport = function () {
            // This is hot path, called multiple times when scrolling. 
            // Keep allocations to a minimum.
            var viewportTop = this.gridViewport.scrollTop();
            var viewportBottom = viewportTop + this.viewportHeight();
            var positionCheck = viewportTop;
            this.firstVisibleRow = null;
            var rowAtPosition = null;
            while (positionCheck < viewportBottom) {
                rowAtPosition = this.findRowAtY(positionCheck);
                if (!rowAtPosition) {
                    // If there's no row at this position, recycle one.
                    rowAtPosition = this.getOffscreenRow(viewportTop, viewportBottom);
                    // Find out what the new top of the row should be.
                    var rowIndex = Math.floor(positionCheck / this.rowHeight);
                    var desiredNewRowY = rowIndex * this.rowHeight;
                    rowAtPosition.top(desiredNewRowY);
                    rowAtPosition.rowIndex(rowIndex);
                    rowAtPosition.resetCells();
                    rowAtPosition.isChecked(this.settings.selectedIndices.indexOf(rowIndex) !== -1);
                }
                if (!this.firstVisibleRow) {
                    this.firstVisibleRow = rowAtPosition;
                }
                positionCheck = rowAtPosition.top() + this.rowHeight;
            }
            this.lastVisibleRow = rowAtPosition;
        };
        ctor.prototype.getOffscreenRow = function (viewportTop, viewportBottom) {
            // This is hot path, called multiple times when scrolling.
            // Keep allocations to a minimum.
            var rows = this.recycleRows();
            for (var i = 0; i < rows.length; i++) {
                var row = rows[i];
                var rowTop = row.top();
                var rowBottom = rowTop + this.rowHeight;
                if (rowTop > viewportBottom || rowBottom < viewportTop) {
                    return row;
                }
            }
            throw new Error("Bug: couldn't find an offscreen row to recycle. viewportTop = " + viewportTop.toString() + ", viewportBottom = " + viewportBottom.toString() + ", recycle row count = " + rows.length.toString());
        };
        ctor.prototype.findRowAtY = function (y) {
            // This is hot path, called multiple times when scrolling. 
            // Keep allocations to a minimum.
            var rows = this.recycleRows();
            for (var i = 0; i < rows.length; i++) {
                var row = rows[i];
                var rowTop = row.top();
                var rowBottom = rowTop + this.rowHeight;
                if (rowTop <= y && rowBottom > y) {
                    return row;
                }
            }
            return null;
        };
        ctor.prototype.getTemplateFor = function (columnName) {
            if (this.settings.customColumns().hasOverrides()) {
                var customConfig = this.settings.customColumns().findConfigFor(columnName);
                if (customConfig) {
                    return customConfig.template();
                }
            }
            return undefined;
        };
        ctor.prototype.toggleRowChecked = function (row, eventFromCheckbox) {
            var _this = this;
            var isShiftSelect = this.shiftPressed();
            if (this.settings.isAllAutoSelected()) {
                var cachedIndeices = this.items.getCachedIndices(this.settings.selectedIndices());
                this.settings.selectedIndices(cachedIndeices);
                this.recycleRows().forEach(function (r) { return r.isChecked(_.includes(_this.settings.selectedIndices(), r.rowIndex())); });
                this.settings.isAllAutoSelected(false);
                this.settings.isAnyAutoSelected(true);
            }
            var rowIndex = row.rowIndex();
            var isChecked = row.isChecked();
            var firstIndex = _.first(this.settings.selectedIndices());
            var toggledIndices = isShiftSelect && this.settings.selectedIndices().length > 0 ? this.getRowIndicesRange(firstIndex, rowIndex) : [rowIndex];
            if (eventFromCheckbox) {
                // since checkbox generates event after check we have to invert condition
                isChecked = !isChecked;
            }
            if (isChecked) {
                // Going from checked to unchecked.
                this.settings.selectedIndices.removeAll(toggledIndices);
            }
            else {
                // Going from unchecked to checked.
                if (this.settings.selectedIndices.indexOf(rowIndex) === -1) {
                    toggledIndices
                        .filter(function (i) { return !_.includes(_this.settings.selectedIndices(), i); })
                        .reverse()
                        .forEach(function (i) { return _this.settings.selectedIndices.unshift(i); });
                }
            }
            this.recycleRows().forEach(function (r) { return r.isChecked(_.includes(_this.settings.selectedIndices(), r.rowIndex())); });
        };
        ctor.prototype.selectNone = function () {
            this.settings.selectedIndices([]);
            this.recycleRows().forEach(function (r) { return r.isChecked(false); });
            this.settings.isAnyAutoSelected(false);
            this.settings.isAllAutoSelected(false);
        };
        ctor.prototype.selectAll = function (documentCount) {
            var allIndices = [];
            /*this.settings.itemsSource().totalResultCount()*/
            for (var i = 0; i < documentCount; i++) {
                allIndices.push(i);
            }
            this.recycleRows().forEach(function (r) { return r.isChecked(true); });
            this.settings.selectedIndices(allIndices);
            this.settings.isAnyAutoSelected(false);
            this.settings.isAllAutoSelected(true);
        };
        ctor.prototype.selectSome = function () {
            var allIndices = [];
            var firstVisibleRowNumber = this.firstVisibleRow.rowIndex();
            var lastVisibleRowNumber = this.lastVisibleRow.rowIndex();
            var numOfRowsInUse = this.recycleRows().filter(function (r) { return r.isInUse(); }).length;
            var actualNumberOfVisibleRows = Math.min(lastVisibleRowNumber - firstVisibleRowNumber, numOfRowsInUse);
            for (var i = firstVisibleRowNumber; i < firstVisibleRowNumber + actualNumberOfVisibleRows; i++) {
                allIndices.push(i);
            }
            this.recycleRows().forEach(function (r) { return r.isChecked(_.includes(allIndices, r.rowIndex())); });
            this.settings.selectedIndices(allIndices);
            this.settings.isAllAutoSelected(false);
        };
        ctor.prototype.getRowIndicesRange = function (firstRowIndex, secondRowIndex) {
            var isCountingDown = firstRowIndex > secondRowIndex;
            var indices = [];
            if (isCountingDown) {
                for (var i = firstRowIndex; i >= secondRowIndex; i--)
                    indices.unshift(i);
            }
            else {
                for (var i = firstRowIndex; i <= secondRowIndex; i++)
                    indices.unshift(i);
            }
            return indices;
        };
        ctor.prototype.viewItem = function () {
            if (this.settings.selectedIndices().length > 0) {
                ko.postbox.publish("ViewItem", this.settings.selectedIndices()[0]);
            }
        };
        ctor.prototype.changeCounterValue = function () {
            if (this.settings.selectedIndices().length > 0) {
                ko.postbox.publish("ChangeCounterValue", this.settings.selectedIndices()[0]);
            }
        };
        ctor.prototype.resetCounter = function () {
            if (this.settings.selectedIndices().length > 0) {
                ko.postbox.publish("ResetCounter", this.settings.selectedIndices()[0]);
            }
        };
        ctor.prototype.copySelectedDocs = function () {
            this.showCopyDocDialog(false);
        };
        ctor.prototype.copySelectedDocIds = function () {
            this.showCopyDocDialog(true);
        };
        ctor.prototype.showCopyDocDialog = function (idsOnly) {
            var selectedDocs = this.getSelectedItems();
            var copyDocumentsVm = new copyDocuments(selectedDocs, this.focusableGridSelector);
            copyDocumentsVm.isCopyingDocs(idsOnly === false);
            app.showBootstrapDialog(copyDocumentsVm);
        };
        ctor.prototype.getSelectedItems = function (max) {
            if (!this.items || this.settings.selectedIndices().length === 0) {
                return [];
            }
            var sliced = max ? this.settings.selectedIndices.slice(0, max) : null;
            var maxSelectedIndices = sliced || this.settings.selectedIndices();
            return this.items.getCachedItemsAt(maxSelectedIndices);
        };
        ctor.prototype.refreshCollectionData = function () {
            //this.settings.itemsSource.valueHasMutated();
            this.items.invalidateCache(); // Causes the cache of items to be discarded.
            this.onGridScrolled(); // Forces a re-fetch of the rows in view.
            this.onWindowHeightChanged();
        };
        ctor.prototype.getNumberOfCachedItems = function () {
            var items = this.items;
            if (!!items) {
                return this.items.itemCount();
            }
            return 0;
        };
        ctor.prototype.deleteSelectedItems = function () {
            var _this = this;
            var items = this.getSelectedItems();
            var deleteDocsVm = new deleteItems(items, activeResourceTracker.default.resource(), this.focusableGridSelector);
            deleteDocsVm.deletionTask.done(function () {
                var deletedDocIndices = items.map(function (d) { return _this.items.indexOf(d); });
                deletedDocIndices.forEach(function (i) { return _this.settings.selectedIndices.remove(i); });
                _this.recycleRows().forEach(function (r) { return r.isChecked(_.includes(_this.settings.selectedIndices(), r.rowIndex())); }); // Update row checked states.
                _this.recycleRows().filter(function (r) { return deletedDocIndices.indexOf(r.rowIndex()) >= 0; }).forEach(function (r) { return r.isInUse(false); });
                _this.items.invalidateCache(); // Causes the cache of items to be discarded.
                _this.onGridScrolled(); // Forces a re-fetch of the rows in view.
                // Forces recalculation of recycled rows, in order to eliminate "duplicate" after delete
                // note: won't run on delete of last document(s) of a collection in order to prevent race condition 
                // with changes api. Now we don't use changes api to update the documents list, so this isn't a problem.
                _this.onWindowHeightChanged();
            });
            app.showBootstrapDialog(deleteDocsVm);
        };
        ctor.prototype.getDocumentHref = function (documentId) {
            if (typeof documentId == "string") {
                return appUrl.forEditItem(documentId, activeResourceTracker.default.database(), null, null);
            }
            else {
                return "#";
            }
        };
        ctor.prototype.getGroupHref = function (group) {
            if (typeof group == "string") {
                return appUrl.forCounterStorageCounters(group, activeResourceTracker.default.counterStorage());
            }
            else {
                return "#";
            }
        };
        ctor.prototype.selectGroup = function (groupName) {
            ko.postbox.publish("SelectGroup", groupName);
        };
        ctor.prototype.getColumnsNames = function () {
            var row = this.items.getAllCachedItems()[0];
            return row ? row.getDocumentPropertyNames() : [];
        };
        ctor.prototype.collectionExists = function (collectionName) {
            var result = this.settings.collections()
                .map(function (c) {
                return collectionName.toLowerCase().substr(0, c.name.length) === c.name.toLowerCase();
            })
                .reduce(function (p, c) { return c || p; }, false);
            return result;
        };
        ctor.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(this.settings.gridSelector).on("mousedown.virtualTableColumnResize", ".ko-grid-column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = parseInt(_this.columns()[columnIndex].width().toString());
                startX = e.pageX;
                resizingColumn = true;
            });
            $(this.settings.gridSelector).on("mouseup.virtualTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(this.settings.gridSelector).on("mousemove.virtualTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var targetColumnSize = startingWidth + e.pageX - startX;
                    _this.columns()[columnIndex].width(targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        ctor.prototype.unregisterColumnResizing = function () {
            $(this.settings.gridSelector).off("mousedown.virtualTableColumnResize");
            $(this.settings.gridSelector).off("mouseup.virtualTableColumnResize");
            $(this.settings.gridSelector).off("mousemove.virtualTableColumnResize");
        };
        return ctor;
    }());
    ctor.idColumnWidth = 200;
    ctor.selectColumnWidth = 38;
    ctor.optionalScrollSize = 20;
    return ctor;
});

//# sourceMappingURL=viewModel.js.map
